#region Using Directives

using System.Collections.Generic;

#endregion

namespace JadEngine.AI.NeuralNets
{
	public abstract class Net
	{
		#region Fields

		protected internal float learningRate;
		
		protected internal List<Layer> layers;

		#endregion

		#region Constructors

		protected Net()
		{
			layers = new List<Layer>();
		}
		
		protected Net(int numberHiddenLayers)
		{
			int i;

			layers = new List<Layer>();
			
			//Create input layer
			layers.Add(new Layer(LayerType.InputLayer));

			//Create hidden layers
			for (i = 0; i < numberHiddenLayers; i++)
				layers.Add(new Layer(LayerType.HiddenLayer));
			
			//Create output layer
			layers.Add(new Layer(LayerType.OutputLayer));
		}
		
		#endregion

		#region Properties

		public virtual float LearningRate
		{
			get { return learningRate; }
			set { learningRate = value; }
		}

		public virtual List<Layer> Layers
		{
			get { return layers; }
		}
		
		public virtual Layer InputLayer
		{
			get { return layers[0]; }
		}

		public virtual List<Layer> HiddenLayers
		{
			get
			{
				if (layers.Count < 3)
					return null;
				
				return layers.GetRange(1, layers.Count - 2);
			}
		}

		public virtual Layer OutputLayer
		{
			get { return layers[layers.Count - 1]; }
		}
		
		#endregion

		#region Methods

		public abstract void Train(List<Pattern> patterns);

		public abstract void Train(Pattern pattern);
		
		public abstract void Test(Pattern pattern);

		#endregion
	}
}
