#region Using directives

using System;
using System.IO;

using JadEngine.Core;

#endregion

namespace JadEngine.Core
{
    /// <summary>
    /// This class maintains the enum elements of the shader
    /// </summary>
    public class JFragmenEnum
	{
		#region Fields
    	
    	/// <summary>
    	/// TODO
    	/// </summary>
		public string Name;
    	
    	/// <summary>
    	/// TODO
    	/// </summary>
        public string[] Values;

		#endregion

		#region Constructors
    	
		/// <summary>
    	/// TODO
    	/// </summary>
    	/// <param name="name">TODO</param>
    	/// <param name="numElements">TODO</param>
		public JFragmenEnum(string name, int numElements)
        {
            Name = name;
            Values = new string[numElements];
		}

		#endregion

		#region Methods
    	
		/// <summary>
        /// Returns the index of a searched value
        /// </summary>
        /// <param name="value">Value to search</param>
        /// <returns>The index in the array or -1 if not found</returns>
        public int IndexOf(string value)
        {
            for (int c = 0; c < Values.Length; c++)
                if (value == Values[c])
                	return c;

            return -1;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
    public class JEffectFragment : IJListable
	{
		#region Fields

    	/// <summary>
    	/// TODO
    	/// </summary>
		static int IDGlobal = 0;

		/// <summary>
		/// TODO
		/// </summary>
		int id;

		/// <summary>
		/// TODO
		/// </summary>
		protected string name;

		/// <summary>
		/// TODO
		/// </summary>
		string Code;

		/// <summary>
		/// Maintains the vars list that are enumerated in AllInOne
		/// </summary>
		public JListString EnumValues;

    	/// <summary>
    	/// TODO
    	/// </summary>
		public string Vars;

		/// <summary>
		/// TODO
		/// </summary>
		public string VS;

		/// <summary>
		/// TODO
		/// </summary>
		public string PS;
    
		#endregion

		#region Constructors

    	/// <summary>
    	/// TODO
    	/// </summary>
    	/// <param name="filename">TODO</param>
		public JEffectFragment(string filename)
		{
    		// Assign the ID to be able to identify this fragment
			id = ++IDGlobal;

			name = filename;

			StreamReader st = new StreamReader(filename);
			Code = st.ReadToEnd();
			st.Close();

			name = Path.GetFileNameWithoutExtension(filename);

			VS = null;
			PS = null;

			Build();
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="filename">TODO</param>
		/// <param name="stream">TODO</param>
		public JEffectFragment(string filename, Stream stream)
		{
			// Assign the ID to be able to identify this fragment
			id = ++IDGlobal;

			StreamReader st = new StreamReader(stream);
			Code = st.ReadToEnd();
			st.Close();
    		
			name = Path.GetFileNameWithoutExtension(filename);

			VS = null;
			PS = null;

			Build();
		}

		#endregion

		#region Properties

    	/// <summary>
		/// Gets or sets the name value
		/// </summary>
		public string Name
		{
			get { return name; }
			set { name = value; }
		}

		/// <summary>
		/// Gets the ID value
		/// </summary>
		public int ID
		{
			get {  return id; }
		}

		#endregion
    	
		#region Methods

    	/// <summary>
    	/// TODO
    	/// </summary>
		void Build()
		{
			// Search the enum
			int psEnd = Code.IndexOf("#end Enum");

			if (psEnd != -1)
			{
				EnumValues = new JListString();

				JEffectFragments.ParseEnum(EnumValues, Code.Substring(0, psEnd));

				Code = Code.Substring(psEnd + 9);
			}

			// Search the VAR start
			int vsInit = Code.IndexOf("#region VAR");

			if (vsInit != -1)
			{
				vsInit += 12;

				int vsEnd = Code.IndexOf("#end VAR");

				if (vsEnd == -1)
					throw new Exception("Not found '#end VAR'");

				Vars = "// " + name + "\n" + Code.Substring(vsInit, (vsEnd) - (vsInit)) + "\n\n";
			}

			// Search the VS start
			vsInit = Code.IndexOf("#region VS");

			if (vsInit != -1)
			{
				vsInit += 11;

				int vsEnd = Code.IndexOf("#end VS");

				if (vsEnd == -1)
					throw new Exception("Not found '#end VS'");

				VS = "// " + name + "\n" + "{\n" + Code.Substring(vsInit, (vsEnd) - (vsInit)) + "\n}\n";
			}

			// Now the PS
			int psInit = Code.IndexOf("#region PS");

			if (psInit != -1)
			{
				psInit += 11;

				psEnd = Code.IndexOf("#end PS");

				if (psEnd == -1)
					throw new Exception("Not found '#end PS'");

				PS = "// " + name + "\n" + "{\n" + Code.Substring(psInit, (psEnd) - (psInit)) + "\n}\n";
			}
		}
    	
		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JEffectFragments : JList<JEffectFragment>
	{
		#region Fields

		/// <summary>
		/// Maintains the vars list that are enumerated in AllInOne
		/// </summary>
        public JListString EnumValues = new JListString();

		/// <summary>
		/// TODO
		/// </summary>
		static int sourcepos = 0;
		
		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="source">TODO</param>
		/// <returns>TODO</returns>
        static string ParseEnumGetLine(string source)
        {
            string r = string.Empty;

            for (; sourcepos < source.Length; sourcepos++)
            {
                if (source[sourcepos] == '\n')
                {
                	sourcepos++;
                	return r;
                }

                r += source[sourcepos];
            }
            
            return null;
         }

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="list">TODO</param>
		 /// <param name="source">TODO</param>
        public static void ParseEnum(JListString list, string source)
        {
			// Read the lines
            sourcepos=0;

            string line;

            while((line=ParseEnumGetLine(source))!=null)
            {
                int bracket1 = line.IndexOf('{');
                if (bracket1 == -1) continue;

                int bracket2 = line.IndexOf('}');
                if (bracket2 == -1) continue;

                string name = line.Substring(0, bracket1 - 1).Trim();

                string cleanline = line.Substring(bracket1 + 1, bracket2 - bracket1 - 1);

                int numElements;

                string[] elements = cleanline.Split(',');

                numElements = elements.Length;

                JFragmenEnum frag = new JFragmenEnum(name, numElements);

                for(int c=0;c<numElements;c++)
                {
                    string key = elements[c].Trim();
                    frag.Values[c] = key;
                }

                list.Add(name,frag);
            }
        }

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="list">TODO</param>
		/// <returns>TODO</returns>
        public static string EnumToDefines(JListString list)
        {
            string r = String.Empty;

            foreach(JFragmenEnum f in list.List)
            {
                r+="// Enum values "+f.Name +"\n";

                for (int c = 0; c < f.Values.Length; c++)
                {
					// BeRSeRKeR: Ive changed f.Values[c] also to uppercase
                    r += "#define " + f.Name.ToUpper() + "_" + f.Values[c].ToUpper() + " " + c.ToString() + "\n";
                }

                r+="\n";
            }

            return r;
        }

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="enumType">TODO</param>
		/// <param name="enumElement">TODO</param>
		/// <returns>TODO</returns>
		public static int EnumElementToInt(string enumType, string enumElement)
		{
			// Search the AllInOne.fx
			string define = EnumToDefines(Jad.EffectFragments.EnumValues);

			if (define.Contains(enumType))
			{
				string[] defineLines = define.Split('\n');

				foreach (string line in defineLines)
				{
					if (line.Contains(enumElement))
					{
						string[] defineTokens = line.Split(' ');
						return (int.Parse(defineTokens[defineTokens.Length - 1]));
					}
				}
			}
			
			else
			{
				// If its not in the AllInOne.fx, search in the fragment list of the shader
				foreach (JEffectFragment f in Jad.EffectFragments.List)
				{
					if (f.EnumValues != null)
					{
						define = EnumToDefines(f.EnumValues);

						if (define.Contains(enumType))
						{
							string[] defineLines = define.Split('\n');

							foreach (string line in defineLines)
							{
								if (line.Contains(enumElement))
								{
									string[] defineTokens = line.Split(' ');
									return (int.Parse(defineTokens[defineTokens.Length - 1]));
								}
							}
						}
					}
				}
			}

			// Couldnt resolve the value of the enum element
			return -1;
		}

        /// <summary>
        /// Loads all the shaders located at the directories InHouse and Base
        /// </summary>
        internal void LoadAllShaders()
		{
            LoadAllShaders(JPath.InHouseDirectory);
            LoadAllShaders(JPath.BaseDirectory);
        }

        /// <summary>
		/// Loads all the fragments located at the specified folder
        /// </summary>
        /// <param name="baseDir">The path</param>
        void LoadAllShaders(string baseDir)
        {
            // Lighting

        	// Search the files that are in that folder to include them
            string dirPath = baseDir + @"shaders\Lighting";

            DirectoryInfo dir = new DirectoryInfo(dirPath);

            if (!Directory.Exists(dirPath)) return;

            FileInfo[] files = dir.GetFiles("*.fx");

            for (int i = 0; i < files.Length; i++)
				Create(files[i].FullName); // Create the effect
            
            // Geometry

            dirPath = baseDir + @"shaders\geometry";

            dir = new DirectoryInfo(dirPath);

            if (!Directory.Exists(dirPath)) return;

            files = dir.GetFiles("*.fx");

            for (int i = 0; i < files.Length; i++)
                Create(files[i].FullName); // Create the effect
        }

		/// <summary>
		/// Inserts a fragment in the fragments list
		/// </summary>
		/// <param name="name">Fragment name</param>
		/// <returns>TODO</returns>
		public JEffectFragment Create(string name)
		{
			JEffectFragment fragment = new JEffectFragment(name);

			Add(fragment);

			return fragment;
		}

		/// <summary>
		/// Inserts a fragment in the fragments list using a resource
		/// </summary>
		/// <param name="name">Name of the object</param>
		/// <param name="stream">TODO</param>
		/// <returns>TODO</returns>
		public JEffectFragment Create(string name, Stream stream)
		{
			JEffectFragment fragment = new JEffectFragment(name, stream);

			Add(fragment);

			return fragment;
		}

		#endregion
	}
}
