//#define _DEBUG

#region Using directives

using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Particles;
using JadEngine.Tools.Parser;

#endregion

namespace JadEngine.Import
{
	internal class JadLoader
	{
		#region Internal classes

		/// <summary>
		/// TODO
		/// </summary>
		private class JHeader
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public int Version;
			
			/// <summary>
			/// TODO
			/// </summary>
			public int NumMaterials;
			
			/// <summary>
			/// TODO
			/// </summary>
			public int NumGeomObjects;
			
			/// <summary>
			/// TODO
			/// </summary>
			public int NumLights;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumParticleEffects;

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		private class JGlobalParams
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public List<float> AmbientColor;
			
			/// <summary>
			/// TODO
			/// </summary>
			public string FogType;
			
			/// <summary>
			/// TODO
			/// </summary>
			public List<float> FogColor;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float FogNear;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float FogFar;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float FogTop;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float FogBottom;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float FogDensity;

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		private class JTexMap
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Name;
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Type;
			
			/// <summary>
			/// TODO
			/// </summary>
			public int MapChannel;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float Amount;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float U_Tile;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float V_Tile;
			
			/// <summary>
			/// TODO
			/// </summary>
			public string FileName;

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		private class JMaterial
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Name;
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Script;
			
			/// <summary>
			/// TODO
			/// </summary>
			public List<float> DiffuseColor;
			
			/// <summary>
			/// TODO
			/// </summary>
			public List<float> SpecularColor;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float SpecularPower;
			
			/// <summary>
			/// TODO
			/// </summary>
			public List<float> EmissiveColor;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float IndexOfRefraction;
			
			/// <summary>
			/// TODO
			/// </summary>
			public int NumMaps;
			
			/// <summary>
			/// TODO
			/// </summary>
			public List<JTexMap> Maps;

			#endregion

			#region Methods
			
			/// <summary>
			/// TODO
			/// </summary>
			/// <param name="mapType">TODO</param>
			/// <returns>TODO</returns>
			public JTexMap GetMap(string mapType)
			{
				foreach (JTexMap texMap in Maps)
					if (texMap.Type == mapType)
						return texMap;

				return null;
			}

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		private class JGeomObject
		{
			#region Structs
			
			/// <summary>
			/// TODO
			/// </summary>
			public struct JTVertex
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public List<float> Coords;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JVertex
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public int Index;
				
				/// <summary>
				/// TODO
				/// </summary>
				public List<float> Coords;
				
				/// <summary>
				/// TODO
				/// </summary>
				public List<float> Normal;
				
				/// <summary>
				/// TODO
				/// </summary>
				public List<JTVertex> TCoords;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct Face
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public int Index;
				
				/// <summary>
				/// TODO
				/// </summary>
				public int A;
				
				/// <summary>
				/// TODO
				/// </summary>
				public int B;
				
				/// <summary>
				/// TODO
				/// </summary>
				public int C;
				
				/// <summary>
				/// TODO
				/// </summary>
				public int MatID;

				#endregion
			}

			#endregion

			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Name;

			/// <summary>
			/// TODO
			/// </summary>
			public string Type;

			/// <summary>
			/// TODO
			/// </summary>
			public bool receiveShadows;

			/// <summary>
			/// TODO
			/// </summary>
			public bool castShadows;

			/// <summary>
			/// TODO
			/// </summary>
			public float mass;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumMapChannels;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumUniqueMatIDs;

			/// <summary>
			/// TODO
			/// </summary>
			public List<int> UniqueMatIDs;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumVerts;

			/// <summary>
			/// TODO
			/// </summary>
			public List<JVertex> Vertices;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumFaces;

			/// <summary>
			/// TODO
			/// </summary>
			public List<Face> Faces;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Min;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Max;

			/// <summary>
			/// TODO
			/// </summary>
			public bool IsXRef;

			/// <summary>
			/// TODO
			/// </summary>
			public string XRefFileName;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Pivot;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Position;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Rotation;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Scale;

			/// <summary>
			/// TODO
			/// </summary>
			public string Parent;

			/// <summary>
			/// TODO
			/// </summary>
			public JMeshObject MeshObject;

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		private class JLight
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Name;

			/// <summary>
			/// TODO
			/// </summary>
			public string Type;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Position;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Rotation;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Color;

			/// <summary>
			/// TODO
			/// </summary>
			public float Multiplier;

			/// <summary>
			/// TODO
			/// </summary>
			public bool AffectDiffuse;

			/// <summary>
			/// TODO
			/// </summary>
			public bool AffectSpecular;

			/// <summary>
			/// TODO
			/// </summary>
			public bool CastShadows;

			/// <summary>
			/// TODO
			/// </summary>
			public string AttenuationDecay;

			/// <summary>
			/// TODO
			/// </summary>
			public float DecayRadius;

			/// <summary>
			/// TODO
			/// </summary>
			public float FarAttenStart;

			/// <summary>
			/// TODO
			/// </summary>
			public float FarAttenEnd;

			/// <summary>
			/// TODO
			/// </summary>
			public float FallOff;

			/// <summary>
			/// TODO
			/// </summary>
			public float HotSpot;

			/// <summary>
			/// TODO
			/// </summary>
			public bool UseProjector;

			/// <summary>
			/// TODO
			/// </summary>
			public string ProjectorFileName;

			/// <summary>
			/// TODO
			/// </summary>
			public bool UseShafts;

			/// <summary>
			/// TODO
			/// </summary>
			public string ShaftsFileName;

			/// <summary>
			/// TODO
			/// </summary>
			public int ShaftsSlicePlanes;

			/// <summary>
			/// TODO
			/// </summary>
			public bool UseVolume;

			/// <summary>
			/// TODO
			/// </summary>
			public float VolumePower;

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		private class JParticleFX
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public string Name;

			/// <summary>
			/// TODO
			/// </summary>
			public string Script;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Position;

			/// <summary>
			/// TODO
			/// </summary>
			public List<float> Rotation;

			#endregion
		}

		#endregion

		#region Delegates

		/// <summary>
		/// Delegate that specifies the reportback handler, when a file loads (.haddd only), this function will be called to report progress
		/// </summary>
		/// <param name="what">TODO</param>
		/// <param name="where">TODO</param>
		/// <param name="total">TODO</param>
		public delegate void ReportBackHandler(string what, int where, int total);

		#endregion

		#region Events

		/// <summary>
		/// OnReport event will be called when loading a .haddd file
		/// </summary>
		public event ReportBackHandler OnReport;

		#endregion

		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		private const int JAD_VERSION = 14;

		//TODO (VICENTE): CHANGE THIS
		/// <summary>
		/// TODO
		/// </summary>
		private readonly string[] extensions = {"", ".haddd"};

		/// <summary>
		/// TODO
		/// </summary>
		private static bool isXRef = false;

		/// <summary>
		/// TODO
		/// </summary>
		private JParser parser;

		/// <summary>
		/// TODO
		/// </summary>
		private JHeader header;

		/// <summary>
		/// TODO
		/// </summary>
		private JGlobalParams globalParams;

		/// <summary>
		/// TODO
		/// </summary>
		private List<JMaterial> materials;

		/// <summary>
		/// TODO
		/// </summary>
		private List<JGeomObject> geomObjects;

		/// <summary>
		/// TODO
		/// </summary>
		private List<JLight> lights;

		/// <summary>
		/// TODO
		/// </summary>
		private List<JParticleFX> particles;

		/// <summary>
		/// TODO
		/// </summary>
		private StreamWriter dbgFile;

		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 scale;

		/// <summary>
		/// Mesh objects loaded on the file. Needed for the XRef
		/// </summary>
		internal List<JMeshObject> MeshObjects;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JadLoader()
		{
            MeshObjects = new List<JMeshObject>();
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fileName">TODO</param>
		/// <param name="inScale">TODO</param>
		/// <returns>TODO</returns>
        public bool Load(string fileName, Vector3 inScale)
		{
            MeshObjects.Clear();

            JToken token = new JToken();

			// Assign the scale
			scale = inScale;

			// Create the parser
			parser = new JParser();

            string pathFileName = fileName;

			// Load the file in the parser
			if (!parser.LoadFromFile(pathFileName))
				parser.Error("Couldn't load file: {0}", pathFileName);

			// Create the header
			header = new JHeader();

			//TODO (VICENTE): CHANGE THIS
			// File type (Haddd or PreHaddd)
			parser.ReadToken(token);

			// Need to convert from PreHaddd to Haddd?
			if (token == "PreHadddVersion") //Yes
			{
				JadConverter converter = new JadConverter();
				converter.Convert(pathFileName);
				PrivateLoad(pathFileName);
			}

			else if (token == "HadddVersion") //No
			{
				if (!PrivateLoad(pathFileName))
					return false;
			}

			else
				parser.Error("expected 'HadddVersion' or 'PreHadddVersion' but found '{0}'", token.Text);

			DeleteXRefObjects();

			// Add the loaded objects. We didnt do it sooner because of the XRef objects
            Jad.Import.MeshObjects.AddRange(MeshObjects);

			return true;
		}

		/// <summary>
		/// Delete the objects marked as XRef, because they must not be used in the
		/// scene as they have been cloned
		/// </summary>
		void DeleteXRefObjects()
		{
			bool oneDeleted = true;

			while (oneDeleted)
			{
				oneDeleted = false;
				foreach (JMeshObject o in Jad.Scene.MeshObjects.List)
				{
					if (o.IsXRef)
					{
						Jad.Scene.MeshObjects.Delete(o);
						oneDeleted = true;

						break;
					}
				}
			}
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="index">TODO</param>
		/// <returns>TODO</returns>
		public JMeshObject GetMeshObject(int index)
		{
			return (index >= 0 && index < geomObjects.Count) ? geomObjects[index].MeshObject : null;
		}

		#endregion

		#region Load start

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fileName">TODO</param>
		/// <returns>TODO</returns>
		private bool PrivateLoad(string fileName)
		{
#if(_DEBUG)
			dbgFile = new StreamWriter(fileName + ".debug.txt");
#endif
			// Create the parser
			parser = new JParser();

			// Load the file in the parser
			if (!parser.LoadFromFile(fileName))
				parser.Error("Couldn't load file: {0}", fileName);
#if(_DEBUG)
			dbgFile.WriteLine("File \"{0}\" loaded\n", fileName);
#endif
			// Parse the header
			ParseHeader();

			// Parse the global parameters
			ParseGlobalParams();

			// Parse the materials
			for (int i = 0; i < header.NumMaterials; i++)
			{
				if (OnReport != null)
					OnReport("Load Materials", i, header.NumMaterials);

				ParseMaterials();
			}

			// Parse the GeomObjects
			for (int i = 0; i < header.NumGeomObjects; i++)
			{
				if (OnReport != null)
					OnReport("Load Objects", i, header.NumGeomObjects);

				ParseGeomObjects();
			}

			// Parse the lights
			for (int i = 0; i < header.NumLights; i++)
			{
				if (OnReport != null)
					OnReport("Load Lights", i, header.NumLights);

				ParseLights();
			}

			// Parse the particle effects
			for (int i = 0; i < header.NumParticleEffects; i++)
			{
				if (OnReport != null)
					OnReport("Load Particles", i, header.NumParticleEffects);

				ParseParticleEffects();
			}

			// Create the materials
			CreateMaterials();

			// Create the geometry
			if (!CreateGeomObjects())
				return false;

			// Create the lights
			CreateLights();

			// Create the particle effect
			CreateParticleEffects();
#if(_DEBUG)
			dbgFile.Close();
#endif
			return true;
		}

		#endregion

		#region Methods to load the header

		/// <summary>
		/// TODO
		/// </summary>
		private void ParseHeader()
		{
			JToken token = new JToken();

			// Create the header
			header = new JHeader();

			//TODO (VICENTE): CHANGE THIS
			// Type of file (Haddd o PreHaddd)
			parser.ReadToken(token);

			if (token == "PreHadddVersion")
			{
			}
			
			else if (token == "HadddVersion")
			{
			}
			
			else
			{
				parser.Error("expected 'HadddVersion' or 'PreHadddVersion' but found '{0}'", token.Text);
			}

			// Version
			header.Version = parser.ParseInt();

			if (header.Version != JAD_VERSION && header.Version != 13)
				parser.Error("Invalid version {0}. Should be version {1}", header.Version, JAD_VERSION);

			// Number of materials
			parser.ExpectTokenString("numMaterials");
			header.NumMaterials = parser.ParseInt();

			// Create the materials list
			materials = new List<JMaterial>(header.NumMaterials);

			// Number of GeomObjects
			parser.ExpectTokenString("numGeomObjects");
			header.NumGeomObjects = parser.ParseInt();

			// Create the list of GeomObjects
			geomObjects = new List<JGeomObject>(header.NumGeomObjects);

			// Number of lights
			parser.ExpectTokenString("numLights");
			header.NumLights = parser.ParseInt();

			// Create the lights
			lights = new List<JLight>(header.NumLights);

			// Number of particle effects
			parser.ExpectTokenString("numParticleEffects");
			header.NumParticleEffects = parser.ParseInt();

			// Create the list of particle effects
			particles = new List<JParticleFX>(header.NumParticleEffects);
#if(_DEBUG)
			dbgFile.WriteLine("version: {0}\n", header.Version);
			dbgFile.WriteLine("numMaterials: {0}", header.NumMaterials);
			dbgFile.WriteLine("numGeomObjects: {0}", header.NumGeomObjects);
			dbgFile.WriteLine("numLights: {0}", header.NumLights);
			dbgFile.WriteLine("numParticleEffects: {0}\n", header.NumParticleEffects);
#endif
		}

		#endregion

		#region Methods to load the global parameters

		/// <summary>
		/// TODO
		/// </summary>
		private void ParseGlobalParams()
		{
			globalParams = new JGlobalParams();

			parser.ExpectTokenString("global");
			parser.ExpectTokenString("{");

			// Ambient color
			parser.ExpectTokenString("ambientColor");
			globalParams.AmbientColor = new List<float>(3);
			parser.Parse1DMatrix(3, globalParams.AmbientColor);

			// Fog
			parser.ExpectTokenString("fogType");
			globalParams.FogType = parser.ParseString();

			if (globalParams.FogType != "none")
			{
				parser.ExpectTokenString("fogColor");
				globalParams.FogColor = new List<float>(3);
				parser.Parse1DMatrix(3, globalParams.FogColor);

				if (globalParams.FogType == "linear" || globalParams.FogType == "exponential")
				{
					parser.ExpectTokenString("fogNear");
					globalParams.FogNear = parser.ParseFloat();
					parser.ExpectTokenString("fogFar");
					globalParams.FogFar = parser.ParseFloat();
				}
				
				else if (globalParams.FogType == "layered")
				{
					parser.ExpectTokenString("fogTop");
					globalParams.FogTop = parser.ParseFloat();
					parser.ExpectTokenString("fogBottom");
					globalParams.FogBottom = parser.ParseFloat();
					parser.ExpectTokenString("fogDensity");
					globalParams.FogDensity = parser.ParseFloat();
				}
			}

			parser.ExpectTokenString("}");

#if(_DEBUG)
			dbgFile.WriteLine("globalParams");
			dbgFile.WriteLine("\tambientColor ( {0} {1} {2} )", globalParams.AmbientColor[0], globalParams.AmbientColor[1], globalParams.AmbientColor[2]);
			dbgFile.WriteLine("\tfogType \"{0}\"", globalParams.FogType);
			
			if (globalParams.FogType != "none")
			{
				dbgFile.WriteLine("\tfogColor ( {0} {1} {2} )", globalParams.FogColor[0], globalParams.FogColor[1], globalParams.FogColor[2]);

				if (globalParams.FogType == "linear" || globalParams.FogType == "exponential")
				{
					dbgFile.WriteLine("\tfogNear {0}", globalParams.FogNear);
					dbgFile.WriteLine("\tfogFar {0}", globalParams.FogFar);
				}

				else if (globalParams.FogType == "layered")
				{
					dbgFile.WriteLine("\tfogTop {0}", globalParams.FogTop);
					dbgFile.WriteLine("\tfogBottom {0}", globalParams.FogBottom);
					dbgFile.WriteLine("\tfogDensity {0}", globalParams.FogDensity);
				}
			}
#endif

			// If loading a mesh object as reference, do not load the global parameters
			if (!isXRef)
			{
				// Set the obtained values

				// Ambient color
				Jad.Scene.Lights.Ambient = new JColor(globalParams.AmbientColor[0], globalParams.AmbientColor[1], globalParams.AmbientColor[2]);

				// Fog
				if (globalParams.FogType != "none")
				{
					if (globalParams.FogType == "linear")
						Jad.Scene.Fog.Type = JFogType.Linear;
					
					else if (globalParams.FogType == "exponential")
						Jad.Scene.Fog.Type = JFogType.ExponentialSquared;
					
					else if (globalParams.FogType == "layered")
						Jad.Scene.Fog.Type = JFogType.Layered;

					Jad.Scene.Fog.Color = new JColor(globalParams.FogColor[0], globalParams.FogColor[1], globalParams.FogColor[2]);

					if (Jad.Scene.Fog.Type == JFogType.Linear || Jad.Scene.Fog.Type == JFogType.ExponentialSquared)
					{
						Jad.Scene.Fog.Near = globalParams.FogNear;
						Jad.Scene.Fog.Far = globalParams.FogFar;
					}
					
					else if (Jad.Scene.Fog.Type == JFogType.Layered)
					{
						Jad.Scene.Fog.Top = globalParams.FogTop;
						Jad.Scene.Fog.Bottom = globalParams.FogBottom;
						Jad.Scene.Fog.Density = globalParams.FogDensity;
					}
				}
			}
		}

		#endregion

		#region Methods for the materials

		/// <summary>
		/// TODO
		/// </summary>
		private void ParseMaterials()
		{
			JToken token = new JToken();
			JMaterial auxMtl = new JMaterial();

			parser.ExpectTokenString("material");

			// Name
			auxMtl.Name = parser.ParseString();

#if(_DEBUG)
			dbgFile.WriteLine("material {0}", auxMtl.Name);
#endif

			parser.ExpectTokenString("{");

			// Two things can happen:
			// 1. Find the "script" keyworkd (JadMaterial)
			// 2. Find the "numMaps" keyword (StandardMaterial)

			parser.ReadToken(token);

			// JadMaterial
			auxMtl.Script = string.Empty;
			if (token == "script")
			{
				// Material script
				auxMtl.Script = parser.ParseString();
#if(_DEBUG)
			dbgFile.WriteLine("\tscript {0}", auxMtl.Script);
#endif
			}
			
			// StandardMaterial
			else if (token == "numMaps")
			{
				// Number of texture maps
				auxMtl.NumMaps = parser.ParseInt();

				// Create the texture maps list
				auxMtl.Maps = new List<JTexMap>(auxMtl.NumMaps);

				// Diffuse color
				auxMtl.DiffuseColor = new List<float>(3);
				parser.ExpectTokenString("diffuseColor");
				parser.Parse1DMatrix(3, auxMtl.DiffuseColor);

				// Specular color
				auxMtl.SpecularColor = new List<float>(3);
				parser.ExpectTokenString("specularColor");
				parser.Parse1DMatrix(3, auxMtl.SpecularColor);

				// Specular power
				parser.ExpectTokenString("specularPower");
				auxMtl.SpecularPower = parser.ParseFloat();

				// Emissive color
				auxMtl.EmissiveColor = new List<float>(3);
				parser.ExpectTokenString("emissiveColor");
				parser.Parse1DMatrix(3, auxMtl.EmissiveColor);

				// Index of refraction
				parser.ExpectTokenString("indexOfRefraction");
				auxMtl.IndexOfRefraction = parser.ParseFloat();

#if(_DEBUG)
			dbgFile.WriteLine("\tnumMaps {0}", auxMtl.NumMaps);
			dbgFile.WriteLine("\tdiffuseColor ( {0} {1} {2} )", auxMtl.DiffuseColor[0], auxMtl.DiffuseColor[1], auxMtl.DiffuseColor[2]);
			dbgFile.WriteLine("\tspecularColor ( {0} {1} {2} )", auxMtl.SpecularColor[0], auxMtl.SpecularColor[1], auxMtl.SpecularColor[2]);
			dbgFile.WriteLine("\tspecularPower {0}", auxMtl.SpecularPower);
			dbgFile.WriteLine("\temissiveColor ( {0} {1} {2} )", auxMtl.EmissiveColor[0], auxMtl.EmissiveColor[1], auxMtl.EmissiveColor[2]);
			dbgFile.WriteLine("\tindexOfRefraction {0}", auxMtl.IndexOfRefraction);
#endif

				// Parse the texture maps
				for (int i = 0; i < auxMtl.NumMaps; i++)
				{
					JTexMap texMap;

					ParseTextureMap(out texMap);
					auxMtl.Maps.Add(texMap);
				}
			}

			// End of the material
			parser.ExpectTokenString("}");

			// Add the material
			materials.Add(auxMtl);
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="texMap">TODO</param>
		private void ParseTextureMap(out JTexMap texMap)
		{
			JToken token = new JToken();

			texMap = new JTexMap();

			parser.ExpectTokenString("map");

			// Name
			texMap.Name = parser.ParseString();

			parser.ExpectTokenString("{");

			// Type
			parser.ExpectTokenString("type");
			parser.ReadToken(token);
			texMap.Type = token.Text;

			// Map channel
			parser.ExpectTokenString("mapChannel");
			texMap.MapChannel = parser.ParseInt();

			// Amount
			parser.ExpectTokenString("amount");
			texMap.Amount = parser.ParseFloat();

			// U Tile
			parser.ExpectTokenString("u_tile");
			texMap.U_Tile = parser.ParseFloat();

			// V Tile
			parser.ExpectTokenString("v_tile");
			texMap.V_Tile = parser.ParseFloat();

			// Bitmap file
		    parser.ExpectTokenString("fileName");
		    texMap.FileName = parser.ParseString();

		    texMap.FileName = Path.GetFileNameWithoutExtension(texMap.FileName);

			// End of the texture map
			parser.ExpectTokenString("}");

#if(_DEBUG)
			dbgFile.WriteLine("\tmap {0}", texMap.Name);
			dbgFile.WriteLine("\t\ttype {0}", texMap.Type);
			dbgFile.WriteLine("\t\tmapChannel {0}", texMap.MapChannel);
			dbgFile.WriteLine("\t\tamount {0}", texMap.Amount);
			dbgFile.WriteLine("\t\tu_tile {0}", texMap.U_Tile);
			dbgFile.WriteLine("\t\tv_tile {0}", texMap.V_Tile);
			dbgFile.WriteLine("\t\tfileName \"{0}\"", texMap.FileName);
#endif
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <remarks>Empty method</remarks>
		private void CreateMaterials()
		{
		}

		#endregion

		#region Methods for the geometry

		/// <summary>
		/// TODO
		/// </summary>
		private void ParseGeomObjects()
		{
			JToken token = new JToken();

			JGeomObject auxGeom = new JGeomObject();

			parser.ReadToken(token);
			if (token != "geomObject" && token != "XRefGeomObject")
			{
				parser.Error("expected 'geomObject' or 'XRefGeomObject' but found '{0}'", token.Text);
				return;
			}

			auxGeom.IsXRef = (token == "XRefGeomObject");

			// Name
			auxGeom.Name = parser.ParseString();

			parser.ExpectTokenString("{");

			// Filename
			if (auxGeom.IsXRef)
			{
				parser.ExpectTokenString("fileName");
				auxGeom.XRefFileName = parser.ParseString();
			}

			// Type
			parser.ExpectTokenString("type");
			auxGeom.Type = parser.ParseString();

			// Receive shadows
			parser.ExpectTokenString("receiveShadows");
			auxGeom.receiveShadows = parser.ParseBool();

			// Cast shadows
			parser.ExpectTokenString("castShadows");
			auxGeom.castShadows = parser.ParseBool();

			// Mass
			parser.ExpectTokenString("mass");
			auxGeom.mass = parser.ParseFloat();

			// Parent & Transformation

			if (header.Version != 13)
			{
				// Parent
				parser.ExpectTokenString("parent");
				auxGeom.Parent = parser.ParseString();
				
				// Pivot
				auxGeom.Pivot = new List<float>(3);
				parser.ExpectTokenString("pivot");
				parser.Parse1DMatrix(3, auxGeom.Pivot);
				
				// Position
				auxGeom.Position = new List<float>(3);
				parser.ExpectTokenString("position");
				parser.Parse1DMatrix(3, auxGeom.Position = new List<float>(3));
				
				// Rotation
				auxGeom.Rotation = new List<float>(3);
				parser.ExpectTokenString("rotation");
				parser.Parse1DMatrix(3, auxGeom.Rotation);
				
				// Scale
				auxGeom.Scale = new List<float>(3);
				parser.ExpectTokenString("scale");
				parser.Parse1DMatrix(3, auxGeom.Scale);
			}

			if (!auxGeom.IsXRef)
			{
				// Nmero de canales de mapeado
				parser.ExpectTokenString("numMapChannels");
				auxGeom.NumMapChannels = parser.ParseInt();

				// Nmero de identificadores nicos de materiales que utiliza este objeto
				parser.ExpectTokenString("numUniqueMatIDs");
				auxGeom.NumUniqueMatIDs = parser.ParseInt();

				// Creamos la lista de identificadores nicos de materiales
				auxGeom.UniqueMatIDs = new List<int>(auxGeom.NumUniqueMatIDs);

				// Analizamos los identificadores
				for (int i = 0; i < auxGeom.NumUniqueMatIDs; i++)
					auxGeom.UniqueMatIDs.Add(parser.ParseInt());


				// Bounds
				parser.ExpectTokenString("bounds");
				auxGeom.Min = new List<float>(3);
				
				parser.Parse1DMatrix(3, auxGeom.Min);
				auxGeom.Max = new List<float>(3);
				
				parser.Parse1DMatrix(3, auxGeom.Max);
			}

#if(_DEBUG)
			if (auxGeom.IsXRef)
				dbgFile.WriteLine("XRefGeomObject {0}", auxGeom.Name);

			else
				dbgFile.WriteLine("geomObject {0}", auxGeom.Name);

			dbgFile.WriteLine("\ttype \"{0}\"", auxGeom.Type);
			dbgFile.WriteLine("\treceiveShadows {0}", auxGeom.receiveShadows);
			dbgFile.WriteLine("\tcastShadows {0}", auxGeom.castShadows);
			dbgFile.WriteLine("\tmass {0}", auxGeom.mass);

			dbgFile.WriteLine("\tparent \"{0}\"", auxGeom.Parent);
			dbgFile.WriteLine("\tpivot ( {0} {1} {2} )", auxGeom.Pivot[0], auxGeom.Pivot[1], auxGeom.Pivot[2]);
			dbgFile.WriteLine("\tposition ( {0} {1} {2} )", auxGeom.Position[0], auxGeom.Position[1], auxGeom.Position[2]);
			dbgFile.WriteLine("\trotation ( {0} {1} {2} )", auxGeom.Rotation[0], auxGeom.Rotation[1], auxGeom.Rotation[2]);
			dbgFile.WriteLine("\tscale ( {0} {1} {2} )", auxGeom.Scale[0], auxGeom.Scale[1], auxGeom.Scale[2]);

			if (!auxGeom.IsXRef)
			{
				dbgFile.WriteLine("\tnumMapChannels {0}", auxGeom.NumMapChannels);
				dbgFile.WriteLine("\tnumUniqueMatIDs {0}", auxGeom.NumUniqueMatIDs);
				dbgFile.Write("\t\t ");
				foreach (int id in auxGeom.UniqueMatIDs)
					dbgFile.Write(" {0}", id);
			}

			dbgFile.Write("\n");
#endif

			if (!auxGeom.IsXRef)
			{
				// Number of vertex
				parser.ExpectTokenString("numVerts");
				auxGeom.NumVerts = parser.ParseInt();

				// Create the vertex list
				auxGeom.Vertices = new List<JGeomObject.JVertex>(auxGeom.NumVerts);

#if(_DEBUG)
				dbgFile.WriteLine("\tbounds ( {0} {1} {2} ) ( {3} {4} {5} )",	auxGeom.Min[0], auxGeom.Min[1], auxGeom.Min[2],
																				auxGeom.Max[0], auxGeom.Max[1], auxGeom.Max[2]);
				dbgFile.WriteLine("\tnumVerts {0}", auxGeom.NumVerts);
#endif

				// Parse the vertex
				for (int i = 0; i < auxGeom.NumVerts; i++)
				{
					JGeomObject.JVertex auxVertex;

					parser.ExpectTokenString("vert");
					auxVertex.Index = parser.ParseInt();

					if (auxVertex.Index != i)
						parser.Error("Expected vertex number {0}, found {1}", i, auxVertex.Index);

					// Vertex coordinates
					auxVertex.Coords = new List<float>(3);
					parser.Parse1DMatrix(3, auxVertex.Coords);

					// Normal
					auxVertex.Normal = new List<float>(3);
					parser.Parse1DMatrix(3, auxVertex.Normal);

					// Create as many texture coordinate pairs as map channels
					auxVertex.TCoords = new List<JGeomObject.JTVertex>(auxGeom.NumMapChannels);

					// Parse the texture coordinates (one pair for each map channel)
					for (int j = 0; j < auxGeom.NumMapChannels; j++)
					{
						JGeomObject.JTVertex auxTVertex;

						auxTVertex.Coords = new List<float>(2);
						parser.Parse1DMatrix(2, auxTVertex.Coords);

						auxVertex.TCoords.Add(auxTVertex);
					}

					// Add the vertex to the list
					auxGeom.Vertices.Add(auxVertex);

#if(_DEBUG)
					dbgFile.Write("\t\tvert {0} ( {1} {2} {3} )", auxVertex.Index, auxVertex.Coords[0], auxVertex.Coords[1], auxVertex.Coords[2]);
					
					for (int j = 0; j < auxGeom.NumMapChannels; j++)
						dbgFile.Write(" ( {0} {1} )", auxVertex.TCoords[j].Coords[0], auxVertex.TCoords[j].Coords[1]);
					
					dbgFile.Write("\n");
#endif
				}

				// Number of faces
				parser.ExpectTokenString("numFaces");
				auxGeom.NumFaces = parser.ParseInt();

				// Create the faces list
				auxGeom.Faces = new List<JGeomObject.Face>(auxGeom.NumFaces);

#if(_DEBUG)
				dbgFile.WriteLine("\tnumFaces {0}", auxGeom.NumFaces);
#endif

				// Parse the faces
				for (int i = 0; i < auxGeom.NumFaces; i++)
				{
					JGeomObject.Face auxFace;

					parser.ExpectTokenString("face");
					auxFace.Index = parser.ParseInt();

					if (auxFace.Index != i)
					{
						parser.Error("Expected face number {0}, found {1}", i, auxFace.Index);
					}

					// First index
					auxFace.A = parser.ParseInt();

					// Second index
					auxFace.B = parser.ParseInt();

					// Third index
					auxFace.C = parser.ParseInt();

					// Material index
					auxFace.MatID = parser.ParseInt();

					// Add the face to the list
					auxGeom.Faces.Add(auxFace);

#if(_DEBUG)
					dbgFile.WriteLine("\t\tface {0} {1} {2} {3} {4}", auxFace.Index, auxFace.A, auxFace.B, auxFace.C, auxFace.MatID);
#endif
				}
			}

			// End of the GeomObject
			parser.ExpectTokenString("}");

			// Add the object to the geometry list
			geomObjects.Add(auxGeom);
		}
				
		/// <summary>
		/// TODO
		/// </summary>
		private bool CreateGeomObjects()
		{
			foreach (JGeomObject geom in geomObjects)
			{
				isXRef = geom.IsXRef;

				if (geom.IsXRef)
				{
					JMeshObject refObject = Jad.Scene.MeshObjects.Exists(geom.XRefFileName);

					if (refObject != null)
						geom.MeshObject = refObject.Clone(geom.Name);
					
					else
					{
						JadLoader loader = new JadLoader();

						if (!loader.Load(geom.XRefFileName, scale))
							return false;

						geom.MeshObject = loader.MeshObjects[0].Clone(geom.Name);
						loader.MeshObjects[0].IsXRef = true;
					}
				}
				
				else
				{
					bool use32BitIndices;

					// Use 32 or 16 bit index?
					use32BitIndices = (geom.NumVerts > ushort.MaxValue) ? true : false;

					List<int>[] facesPerMaterial = new List<int>[geom.NumUniqueMatIDs];

					for (int i = 0; i < geom.NumUniqueMatIDs; i++)
						facesPerMaterial[i] = new List<int>();

					geom.MeshObject = Jad.Scene.MeshObjects.Create(geom.Name);

					JVertexBuffer vb=geom.MeshObject.Mesh.VertexBuffer = new JVertexBuffer(geom.NumVerts);

					JIndexBuffer ib = geom.MeshObject.Mesh.IndexBuffer = new JIndexBuffer(use32BitIndices, geom.NumFaces * 3);

					vb.Position = new Vector3[geom.NumVerts];
					vb.Normal = new Vector3[geom.NumVerts];

					int numTexCoord=geom.NumMapChannels;

					// At least one texture coordinate
					if (numTexCoord == 0) numTexCoord = 1;

					vb.TexCoord = new Vector2[numTexCoord, geom.NumVerts];

					// Fill the vertex array
					for (int i = 0; i < geom.NumVerts; i++)
					{
						// Vertex coordiantes
						vb.Position[i].X = geom.Vertices[i].Coords[0] * scale.X;
						vb.Position[i].Y = geom.Vertices[i].Coords[1] * scale.Y;
						vb.Position[i].Z = geom.Vertices[i].Coords[2] * scale.Z;

						// Normal
						vb.Normal[i].X = geom.Vertices[i].Normal[0];
						vb.Normal[i].Y = geom.Vertices[i].Normal[1];
						vb.Normal[i].Z = geom.Vertices[i].Normal[2];
						
						// Texture coordinates

						if (geom.NumMapChannels == 0)
						{
							vb.TexCoord[0,i].X = 0;
							vb.TexCoord[0,i].Y = 0;
						}

						for (int p = 0; p < geom.NumMapChannels; p++)
						{
							vb.TexCoord[p,i].X = geom.Vertices[i].TCoords[p].Coords[0];
							vb.TexCoord[p,i].Y = geom.Vertices[i].TCoords[p].Coords[1];
						}
					}

					// Fill the faces array, putting them according to their faceID
					for (int i = 0; i < geom.NumFaces; i++)
					{
						int faceID=geom.Faces[i].MatID;

						facesPerMaterial[faceID].Add(geom.Faces[i].A);
						facesPerMaterial[faceID].Add(geom.Faces[i].B);
						facesPerMaterial[faceID].Add(geom.Faces[i].C);
					}

					// Now that the faces are ordered by face, put them in the array

					geom.MeshObject.Mesh.AttributeRange = new AttributeRange[geom.NumUniqueMatIDs];

					for (int i = 0,n=0; i < geom.NumUniqueMatIDs; i++)
					{
						geom.MeshObject.Mesh.AttributeRange[i].AttributeId = i;
						geom.MeshObject.Mesh.AttributeRange[i].FaceStart = n/3;
						geom.MeshObject.Mesh.AttributeRange[i].FaceCount = facesPerMaterial[i].Count/3;
						geom.MeshObject.Mesh.AttributeRange[i].VertexCount = 0;
						geom.MeshObject.Mesh.AttributeRange[i].VertexStart = 0;

						for (int a = 0; a < facesPerMaterial[i].Count; a++)
							ib.Indices[n++] = facesPerMaterial[i][a];
					}

					geom.MeshObject.Mesh.CenterAtOrigin();

					// Tell the engine that the normals are computed

                    geom.MeshObject.Mesh.NormalsComputed = true;

					geom.MeshObject.Position = geom.MeshObject.Mesh.TranslationToLocal;

					// Apparience

					Scene.JMaterial[] meshMaterials = new Scene.JMaterial[geom.NumUniqueMatIDs];

					for (int i = 0; i < geom.NumUniqueMatIDs; i++)
					{
						int matID = geom.UniqueMatIDs[i];

						JMaterial auxMtl = materials[matID];

						// The material is defined in a script
						if (auxMtl.Script.Length > 0)
						{
							meshMaterials[i] = Jad.Scene.Materials.Load(auxMtl.Script);
							if (meshMaterials[i] == null)
								return false;

							foreach (JMaterialLayer layer in meshMaterials[i].Layers)
								if (layer.DetailMap.Enabled)
									layer.DetailMap.UseDifferentTexCoords = (geom.NumMapChannels > 1);
						}
						
						else // Create the material manually
						{
							meshMaterials[i] = Jad.Scene.Materials.Create();

							JMaterialLayer layer = new JMaterialLayer();

							meshMaterials[i].AddLayer(layer);

							// These are the material properties that must be loaded even if there is no map
							layer.DiffuseMap.Color = new JColor(auxMtl.DiffuseColor[0], auxMtl.DiffuseColor[1], auxMtl.DiffuseColor[2]);
							layer.SpecularMap.Color = new JColor(auxMtl.SpecularColor[0], auxMtl.SpecularColor[1], auxMtl.SpecularColor[2]);
							layer.SpecularMap.Power = auxMtl.SpecularPower;
							layer.SpecularMap.Enabled = (layer.SpecularMap.Power > 0.0f && layer.SpecularMap.Color != JColor.Black);
							layer.EmissiveMap.Color = new JColor(auxMtl.EmissiveColor[0], auxMtl.EmissiveColor[1], auxMtl.EmissiveColor[2]);

							if (auxMtl.NumMaps > 0)
							{
								// Diffuse Map
								JTexMap diffuseMap = auxMtl.GetMap("Diffuse");

								if (diffuseMap != null)
								{
                                    layer.DiffuseMap.Texture = Jad.Video.Textures.Create2D(diffuseMap.FileName, true);
                                    layer.DiffuseMap.UTile = diffuseMap.U_Tile;
                                    layer.DiffuseMap.VTile = diffuseMap.V_Tile;
                                    layer.DiffuseMap.TextureEnabled = true;
								}

								// Bump Map
								JTexMap bumpMap = auxMtl.GetMap("Bump");

								if (bumpMap != null)
								{
									layer.NormalMap.Texture = Jad.Video.Textures.Create2D(bumpMap.FileName, false);

									// Haddd: tell that the format is unknown so the compresion load politics
									// arent used

									layer.NormalMap.Texture.LoadingParams.Format = Format.Unknown;

									layer.NormalMap.Texture.Load();

									layer.NormalMap.Bumpiness = bumpMap.Amount;
									layer.NormalMap.Enabled = true;
									layer.NormalType = JNormalType.Mapping;
								}

								// Displacement Map
								JTexMap dispMap = auxMtl.GetMap("Displace");

								if (dispMap != null)
								{
									layer.HeightMap.Enabled = true;
									layer.HeightMap.Texture = Jad.Video.Textures.Create2D(dispMap.FileName, true);
									layer.HeightMap.ParallaxAmount = dispMap.Amount * 0.1f;
								}

								// Opacity Map: its supposed that the alpha channel of the diffuse map is the one that tells the opacity,
								// and also we only use the alpha test as the opacity mode
								JTexMap opacityMap = auxMtl.GetMap("Opacity");

								if (opacityMap != null)
								{
									if (diffuseMap != null && diffuseMap.FileName == opacityMap.FileName)
									{
										// Tell the material that it has alphatest
										meshMaterials[i].HasAlphaTest = true;
										layer.RenderStates.AlphaTestEnable= true;
										layer.RenderStates.ReferenceAlpha= 0;
										layer.RenderStates.AlphaFunction=Compare.Greater;
									}
								}

								// Specular Map

								// First try with the map "Specular"...
								JTexMap specularMap = auxMtl.GetMap("Specular");

								// If theres no "Specular", try with the "Shine"
								if (specularMap == null)
									specularMap = auxMtl.GetMap("Shine");

								if (specularMap != null)
								{
									layer.SpecularMap.Texture = Jad.Video.Textures.Create2D(specularMap.FileName, true);
									layer.SpecularMap.Enabled = true;
								}

								else
									layer.SpecularMap.Enabled = (layer.SpecularMap.Power > 0.0f && layer.SpecularMap.Color != JColor.Black);


								// Emissive Map
								JTexMap emissiveMap = auxMtl.GetMap("SelfIllum");

								if (emissiveMap != null)
								{
									layer.EmissiveMap.Texture = Jad.Video.Textures.Create2D(emissiveMap.FileName, true);
									layer.EmissiveMap.Mode = JEmissiveMode.Texture;
									layer.EmissiveMap.Intensity = emissiveMap.Amount;
									layer.EmissiveMap.Enabled = true;
								}
								
								else
								{
									layer.EmissiveMap.Enabled = (layer.EmissiveMap.Color != JColor.Black);
									if (layer.EmissiveMap.Enabled)
										layer.EmissiveMap.Mode = JEmissiveMode.Color;
								}

								// Reflection Map
								JTexMap reflMap = auxMtl.GetMap("Reflect");

								if (reflMap != null)
								{
									layer.ReflectionMap.Texture = Jad.Video.Textures.CreateCube(reflMap.FileName, true);
									layer.ReflectionMap.Reflectivity = reflMap.Amount;
									layer.ReflectionMap.Enabled = true;
								}

								// Refraction Map
								JTexMap refrMap = auxMtl.GetMap("Refract");

								if (refrMap != null)
								{
									layer.RefractionMap.Texture = Jad.Video.Textures.CreateCube(refrMap.FileName, true);
									layer.RefractionMap.Transmittance = refrMap.Amount;
									layer.RefractionMap.IndexOfRefraction = auxMtl.IndexOfRefraction;
									layer.RefractionMap.Enabled = true;
								}

								// Light Map
								JTexMap lightMap = auxMtl.GetMap("Lightmap");

								if (lightMap != null)
								{
									layer.LightMap.Texture = Jad.Video.Textures.Create2D(lightMap.FileName, true);
									layer.LightMap.Enabled = true;
								}
							}
						}

						meshMaterials[i].MatUniqueID = matID;
					}

					// If the object type is "decal"
					if (geom.Type == "Decal")
					{
						// Tell the material it has alphatest
						meshMaterials[0].HasAlphaTest = true;
						meshMaterials[0].Priority = JMaterialPriority.Decal;
						meshMaterials[0].RenderStates.DepthBias=1;
						meshMaterials[0].Layers[0].RenderStates.AlphaTestEnable=false;
						meshMaterials[0].Layers[0].RenderStates.AlphaBlendEnable=true;
						meshMaterials[0].Layers[0].RenderStates.SourceBlend=Blend.SourceAlpha;
						meshMaterials[0].Layers[0].RenderStates.DestinationBlend=Blend.InvSourceAlpha;
					}

					geom.MeshObject.Material = meshMaterials;
				}

				geom.MeshObject.CastShadows = geom.castShadows;
				geom.MeshObject.ReceiveShadows = geom.receiveShadows;

				if (!geom.IsXRef) // Add it to the informative list mesh objects created 
					MeshObjects.Add(geom.MeshObject);
			}

            return true;
		}

		#endregion

		#region Methods for the lights

		/// <summary>
		/// TODO
		/// </summary>
		private void ParseLights()
		{
			JLight auxLight = new JLight();

			JToken token = new JToken();

			parser.ExpectTokenString("light");

			// Name
			auxLight.Name = parser.ParseString();

			parser.ExpectTokenString("{");

			// Type
			parser.ExpectTokenString("type");
			parser.ReadToken(token);
			auxLight.Type = token.Text;

			// Position
			auxLight.Position = new List<float>(3);
			parser.ExpectTokenString("position");
			parser.Parse1DMatrix(3, auxLight.Position);

#if(_DEBUG)
			dbgFile.WriteLine("light {0}", auxLight.Name);
			dbgFile.WriteLine("\ttype {0}", auxLight.Type);
			dbgFile.WriteLine("\tposition ( {0} {1} {2} )", auxLight.Position[0], auxLight.Position[1], auxLight.Position[2]);
#endif

			// If the light is "spot" or "direct", read the rotation
			if (auxLight.Type == "targetSpot" || auxLight.Type == "freeSpot" || auxLight.Type == "targetDirect" || auxLight.Type == "freeDirect")
			{
				// Rotation
				auxLight.Rotation = new List<float>(4);
				parser.ExpectTokenString("rotation");
				parser.Parse1DMatrix(4, auxLight.Rotation);

#if(_DEBUG)
				dbgFile.WriteLine("\trotation ( {0} {1} {2} {3} )", auxLight.Rotation[0], auxLight.Rotation[1], auxLight.Rotation[2], auxLight.Rotation[3]);
#endif
			}

			// Casts shadows?
			parser.ExpectTokenString("castShadows");
			auxLight.CastShadows = parser.ParseBool();

			// Multiplier
			parser.ExpectTokenString("multiplier");
			auxLight.Multiplier = parser.ParseFloat();

			// Color
			auxLight.Color = new List<float>(3);
			parser.ExpectTokenString("color");
			parser.Parse1DMatrix(3, auxLight.Color);

			// Attenuation type
			parser.ExpectTokenString("attenuationDecay");
			parser.ReadToken(token);
			auxLight.AttenuationDecay = token.Text;

			if (auxLight.AttenuationDecay == "Inverse" || auxLight.AttenuationDecay == "InverseSquare")
			{
				// Attenuation radius
				parser.ExpectTokenString("decayRadius");
				auxLight.DecayRadius = parser.ParseFloat();
			}
			else if (auxLight.AttenuationDecay == "DualRadius")
			{
				// Internal radius of the double attenuation
				parser.ExpectTokenString("farAttenStart");
				auxLight.FarAttenStart = parser.ParseFloat();

				// External radius of the double attenuation
				parser.ExpectTokenString("farAttenEnd");
				auxLight.FarAttenEnd = parser.ParseFloat();
			}

			if (auxLight.Type == "targetSpot" || auxLight.Type == "freeSpot" || auxLight.Type == "targetDirect" || auxLight.Type == "freeDirect")
			{
				// FallOff
				parser.ExpectTokenString("fallOff");
				auxLight.FallOff = parser.ParseFloat();

				// HotSpot
				parser.ExpectTokenString("hotSpot");
				auxLight.HotSpot = parser.ParseFloat();
			}

			// Affects the diffuse component?
			parser.ExpectTokenString("affectDiffuse");
			auxLight.AffectDiffuse = parser.ParseBool();

			// Affects the specular component?
			parser.ExpectTokenString("affectSpecular");
			auxLight.AffectSpecular = parser.ParseBool();

			// Is the projector active?
			parser.ExpectTokenString("useProjector");
			auxLight.UseProjector = parser.ParseBool();

#if(_DEBUG)
			dbgFile.WriteLine("\tmultiplier {0}", auxLight.Multiplier);
			dbgFile.WriteLine("\tcolor ( {0} {1} {2} )", auxLight.Color[0], auxLight.Color[1], auxLight.Color[2]);
			dbgFile.WriteLine("\tcastShadows {0}", auxLight.CastShadows);
			dbgFile.WriteLine("\tattenuationDecay {0}", auxLight.AttenuationDecay);
			if (auxLight.AttenuationDecay == "Inverse" || auxLight.AttenuationDecay == "InverseSquare")
				dbgFile.WriteLine("\tdecayRadius {0}", auxLight.DecayRadius);
			else if (auxLight.AttenuationDecay == "DualRadius")
			{
				dbgFile.WriteLine("\tfarAttenStart {0}", auxLight.FarAttenStart);
				dbgFile.WriteLine("\tfarAttenEnd {0}", auxLight.FarAttenEnd);
			}
			if (auxLight.Type == "targetSpot" || auxLight.Type == "freeSpot")
			{
				dbgFile.WriteLine("\tfallOff {0}", auxLight.FallOff);
				dbgFile.WriteLine("\thotSpot {0}", auxLight.HotSpot);
			}
			dbgFile.WriteLine("\taffectDiffuse {0}", auxLight.AffectDiffuse);
			dbgFile.WriteLine("\taffectSpecular {0}", auxLight.AffectSpecular);
			dbgFile.WriteLine("\tuseProjector {0}", auxLight.UseProjector);
#endif

			// If using a projector, parse the file of the texture map of the projector
			if (auxLight.UseProjector)
			{
				parser.ExpectTokenString("projectorMap");
				auxLight.ProjectorFileName = parser.ParseString();

                auxLight.ProjectorFileName = Path.GetFileNameWithoutExtension(auxLight.ProjectorFileName);
#if(_DEBUG)
				dbgFile.WriteLine("\tprojectorMap {0}", auxLight.ProjectorFileName);
#endif
			}

			// Are the light shafts active?
			parser.ExpectTokenString("useShafts");
			auxLight.UseShafts = parser.ParseBool();

			if (auxLight.UseShafts)
			{
				// File of the map that contains the cookie
				parser.ExpectTokenString("shaftsMap");
				auxLight.ShaftsFileName = parser.ParseString();

                auxLight.ShaftsFileName = Path.GetFileNameWithoutExtension(auxLight.ShaftsFileName);

				// Slice planes that form the shafts
				parser.ExpectTokenString("slicePlanes");
				auxLight.ShaftsSlicePlanes = parser.ParseInt();
			}

			// Is the light volume active?
			parser.ExpectTokenString("useVolume");
			auxLight.UseVolume = parser.ParseBool();

			// Power of the volume
			if (auxLight.UseVolume)
			{
				parser.ExpectTokenString("volumePower");
				auxLight.VolumePower = parser.ParseFloat();
			}

#if(_DEBUG)
			dbgFile.WriteLine("\tuseShafts {0}", auxLight.UseShafts);
			
			if (auxLight.UseShafts)
			{
				dbgFile.WriteLine("\tshaftsMap {0}", auxLight.ShaftsFileName);
				dbgFile.WriteLine("\tslicePlanes {0}", auxLight.ShaftsSlicePlanes);
			}
			
			dbgFile.WriteLine("\tuseVolume {0}", auxLight.UseVolume);
			
			if (auxLight.UseVolume)
				dbgFile.WriteLine("\tvolumePower {0}", auxLight.VolumePower);
#endif

			// End of the light
			parser.ExpectTokenString("}");

			// Add the light to the list of lights
			lights.Add(auxLight);
		}

		/// <summary>
		/// TODO
		/// </summary>
		private void CreateLights()
		{
			foreach (JLight light in lights)
			{
				JLightType type = JLightType.Unknown;

				// Type
				if (light.Type == "omni")
					type = JLightType.Omni;
				
				else if (light.Type == "freeSpot" || light.Type == "targetSpot")
					type = JLightType.Spot;
				
				else if (light.Type == "freeDirect" || light.Type == "targetDirect")
					type = JLightType.Directional;

				// Create the light
				JBaseLight newLight = Jad.Scene.Lights.Create(light.Name, type);

				// Set the values

				newLight.Position = new Vector3(light.Position[0] * scale.X, light.Position[1] * scale.Y, light.Position[2] * scale.Z);

				if (newLight.Type == JLightType.Spot || newLight.Type == JLightType.Directional)
					newLight.Rotation = new Quaternion(light.Rotation[0], light.Rotation[1], light.Rotation[2], light.Rotation[3]);

				newLight.CastShadows = light.CastShadows;

				newLight.Multiplier = light.Multiplier;

				newLight.Color = new JColor(light.Color[0], light.Color[1], light.Color[2]);

				// Attenuation type
				if (light.AttenuationDecay == "None")
					newLight.AttenuationType = JLightAttenuationType.None;
				
				else if (light.AttenuationDecay == "Inverse")
				{
					newLight.AttenuationType = JLightAttenuationType.Inverse;
					newLight.Radius = light.DecayRadius;
				}
				
				else if (light.AttenuationDecay == "InverseSquare")
				{
					newLight.AttenuationType = JLightAttenuationType.InverseSquare;
					newLight.Radius = light.DecayRadius;
				}
				
				else if (light.AttenuationDecay == "DualRadius")
				{
					newLight.AttenuationType = JLightAttenuationType.DualRadius;
					newLight.FarAttenuationStart = light.FarAttenStart * scale.X;
					newLight.FarAttenuationEnd = light.FarAttenEnd * scale.X;
				}

				if (newLight.Type == JLightType.Spot || newLight.Type == JLightType.Directional)
				{
					JSpotLight spot = newLight as JSpotLight;
					spot.FallOff = light.FallOff;
					spot.HotSpot = light.HotSpot;
				}

				newLight.AffectDiffuse = light.AffectDiffuse;
				newLight.AffectSpecular = light.AffectSpecular;

				newLight.Projector.Enabled = light.UseProjector;

				// Depending on the light type the projector will be a 2D texture of a cubemap
				if (newLight.Projector.Enabled)
				{
					if (newLight.Type == JLightType.Omni)
						newLight.Projector.Map = Jad.Video.Textures.CreateCube(light.ProjectorFileName, true);
					
					else if (newLight.Type == JLightType.Spot || newLight.Type == JLightType.Directional)
						newLight.Projector.Map = Jad.Video.Textures.Create2D(light.ProjectorFileName, true);
				}

				if (newLight.Type == JLightType.Spot || newLight.Type == JLightType.Directional)
				{
					((JSpotLight)newLight).Shafts.Enabled = light.UseShafts;
					if (light.UseShafts)
					{
						((JSpotLight)newLight).Shafts.CookieTexture = Jad.Video.Textures.Create2D(light.ShaftsFileName, true);
						((JSpotLight)newLight).Shafts.NumPlanes = light.ShaftsSlicePlanes;
					}
				}

				newLight.Volumetric.Enabled = light.UseVolume;
				if (light.UseVolume)
					newLight.Volumetric.PowValue = light.VolumePower;
			}
		}

		#endregion

		#region Methods for the particle effects

		/// <summary>
		/// TODO
		/// </summary>
		private void ParseParticleEffects()
		{
			JParticleFX auxParticleFX = new JParticleFX();

			parser.ExpectTokenString("particleEffect");

			// Name
			auxParticleFX.Name = parser.ParseString();

			parser.ExpectTokenString("{");

			// Script
			parser.ExpectTokenString("script");
			auxParticleFX.Script = parser.ParseString();

			// Position
			auxParticleFX.Position = new List<float>(3);
			parser.ExpectTokenString("position");
			parser.Parse1DMatrix(3, auxParticleFX.Position);

			// Rotation
			auxParticleFX.Rotation = new List<float>(4);
			parser.ExpectTokenString("rotation");
			parser.Parse1DMatrix(4, auxParticleFX.Rotation);

#if(_DEBUG)
			dbgFile.WriteLine("particleEffect {0}", auxParticleFX.Name);
			dbgFile.WriteLine("\tscript \"{0}\"", auxParticleFX.Script);
			dbgFile.WriteLine("\tposition ( {0} {1} {2} )", auxParticleFX.Position[0], auxParticleFX.Position[1], auxParticleFX.Position[2]);
			dbgFile.WriteLine("\trotation ( {0} {1} {2} {3} )", auxParticleFX.Rotation[0], auxParticleFX.Rotation[1], auxParticleFX.Rotation[2], auxParticleFX.Rotation[3]);
#endif

			// End of the particle effect
			parser.ExpectTokenString("}");

			// Add the particle effect to the list of particle effects
			particles.Add(auxParticleFX);
		}

		/// <summary>
		/// TODO
		/// </summary>
		private void CreateParticleEffects()
		{
			foreach (JParticleFX pFX in particles)
			{
				JParticleEffect auxParticleFX = Jad.Scene.Particles.Create(pFX.Name, pFX.Script);
				
				auxParticleFX.Position = new Vector3(pFX.Position[0], pFX.Position[1], pFX.Position[2]);
				auxParticleFX.Rotation = new Quaternion(pFX.Rotation[0], pFX.Rotation[1], pFX.Rotation[2], pFX.Rotation[3]);
			}
		}

		#endregion
	}
}
