#region Using Directives

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

#endregion

namespace JadEngine.Input
{
	/// <summary>
	/// Enumeration for button mask
	/// </summary>
	public enum EButtonMask : ushort
	{
		Up = 0x00000001,
		Down = 0x00000002,
		Left = 0x00000004,
		Right = 0x00000008,
		Start = 0x00000010,
		Back = 0x00000020,
		ThumbLeft = 0x00000040,
		ThumbRight = 0x00000080,
		ShoulderLeft = 0x0100,
		ShoulderRight = 0x0200,
		A = 0x1000,
		B = 0x2000,
		X = 0x4000,
		Y = 0x8000,
		None = 0
	}

	/// <summary>
	/// TODO
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	public struct JGamePad
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public EButtonMask Buttons;

		/// <summary>
		/// TODO
		/// </summary>
		public byte LeftTrigger;

		/// <summary>
		/// TODO
		/// </summary>
		public byte RightTrigger;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbLeftX;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbLeftY;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbRightX;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbRightY;

		#endregion
	}

	/// <summary>
	/// Information about the vibration motors
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	internal struct Vibration
	{
		#region Fields

		/// <summary>
		/// Left motor speed
		/// </summary>
		public ushort LeftMotorSpeed;

		/// <summary>
		/// Right motor speed
		/// </summary>
		public ushort RightMotorSpeed;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="left">Left motor speed</param>
		/// <param name="right">Right motor speed</param>
		public Vibration(ushort left, ushort right)
		{
			LeftMotorSpeed = left;
			RightMotorSpeed = right;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	public struct State
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public uint PacketNumber;

		/// <summary>
		/// TODO
		/// </summary>
		public JGamePad Gamepad;

		#endregion
	}

	/// <summary>
	/// The collection of <see cref="JX360Controller"/>
	/// </summary>
	public class JControllerCollection
	{
		#region Fields

		/// <summary>
		/// List of controllers
		/// </summary>
		private List<JX360Controller> controllers = new List<JX360Controller>();

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JControllerCollection()
		{
			for (int i = 0; i < 4; i++)
				controllers.Add(new JX360Controller(i));
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets a controller
		/// </summary>
		/// <param name="index">Index of the controller</param>
		/// <returns>The selected controller</returns>
		/// <remarks>Correct values range from 0 to 3</remarks>
		public JX360Controller this[int index]
		{
			get { return this.controllers[index]; }
		}

		#endregion
	}

	/// <summary>
	/// XInput wrapper
	/// </summary>
	public class JXInput
	{
		#region Constants

		/// <summary>
		/// TODO
		/// </summary>
		public const int ERROR_SUCCESS = 0;

		#endregion

		#region Fields

		/// <summary>
		/// Collection of controllers
		/// </summary>
		private static JControllerCollection controllerList = new JControllerCollection();

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		private JXInput()
		{
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets the collection of controllers
		/// </summary>
		public static JControllerCollection Controllers
		{
			get { return controllerList; }
		}

		#endregion

		#region Methods

		[System.Security.SuppressUnmanagedCodeSecurity] // We won't use this maliciously
		[DllImport("xinput9_1_0.dll", EntryPoint = "XInputSetState", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi)]
		internal static extern int SetState(int dwUserIndex, ref Vibration pVibration);

		[System.Security.SuppressUnmanagedCodeSecurity] // We won't use this maliciously
		[DllImport("xinput9_1_0.dll", EntryPoint = "XInputGetState", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi)]
		internal static extern int GetState(int dwUserIndex, ref State pState);

		#endregion
	}
}
