// Author: Mihai Cozma

#region Using Directives

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using JadEngine.Core;

#endregion

namespace JadEngine
{
	/// <summary>
	/// Base class to create Jad applications
	/// </summary>
	public class JStateApplication
	{
		#region Fields

		/// <summary>
		/// Indicates if the application must quit
		/// </summary>
		private static bool quit = false;

		/// <summary>
		/// Application's state machine
		/// </summary>
		private JStateMachine machine;

		#endregion

		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
		public struct Message
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public IntPtr hWnd;

			/// <summary>
			/// TODO
			/// </summary>
			public uint msg;

			/// <summary>
			/// TODO
			/// </summary>
			public IntPtr wParam;

			/// <summary>
			/// TODO
			/// </summary>
			public IntPtr lParam;

			/// <summary>
			/// TODO
			/// </summary>
			public uint time;

			/// <summary>
			/// TODO
			/// </summary>
			public System.Drawing.Point p;

			#endregion
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JStateApplication()
		{
			machine = new JStateMachine();
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets if there are messages waiting to be processed or not
		/// </summary>
		private bool AppStillIdle
		{
			get
			{
				Message msg;
				return !PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
			}
		}

		/// <summary>
		/// Set the quit value to true for quiting the application
		/// </summary>
		public static bool Quit
		{
			set { quit = value; }
		}

		/// <summary>
		/// Gets the application's state machine
		/// </summary>
		public JStateMachine States
		{
			get { return machine; }
		}

		#endregion

		#region Methods

		// We won't use this maliciously
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("User32.dll", CharSet = CharSet.Auto)]
		public static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="sender">TODO</param>
		/// <param name="e">TODO</param>
		protected void OnApplicationIdle(object sender, EventArgs e)
		{
			// Render a frame during idle time (no messages are waiting)
			while (AppStillIdle)
			{
				// Run the current state 
				machine.OnRun();

				if (quit)
				{
					//Runs the ending code of the current state
					machine.OnEnd();

					// Free resources and shutdown the engine
					Jad.End();

					// Close the main form
					Jad.Form.Close();
				}
			}
		}

		/// <summary>
		/// Adds the Idle event handler when needed and runs the application
		/// </summary>
		/// <returns></returns>
		public void AddOnIdle()
		{
			Application.Idle += new EventHandler(OnApplicationIdle);
			Application.Run(Jad.Form);
		}

		#endregion
	}
}
