#region Using Directives

using System;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;
using Microsoft.DirectX.DirectSound;

using JadEngine.Core;
using JadEngine.Import;
using JadEngine.Input;
using JadEngine.Physics;
using JadEngine.Scene;
using JadEngine.Sound;
using JadEngine.Tools;
using JadEngine.Video;

#endregion

namespace JadEngine
{
	/// <summary>
	/// Core class that contains all the engine elements
	/// </summary>
    public sealed class Jad
	{
		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		public struct JVideoRecorder
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public string PrefixScreenShot;
			
			/// <summary>
			/// TODO
			/// </summary>
			public ImageFileFormat FileFormat;
			
			/// <summary>
			/// TODO
			/// </summary>
			public string Path;
			
			/// <summary>
			/// TODO
			/// </summary>
			public float FPS;
			
			/// <summary>
			/// TODO
			/// </summary>
			private bool enabled;

			#endregion

			#region Properties

			/// <summary>Gets or sets if the engine its in video mode, so its using fixedTime and the frames should be recorded</summary>
			public bool Enabled
			{
				get { return enabled; }
				set
				{
					enabled = value;

					if (enabled)
					{
						// Set the FPS speed, this must be done before setting FixedMode to true
						Timer.FixedModeFPS = FPS;

						// Set that the timer works in fixed mode
						Timer.FixedMode = true;

						// Waiting is not needed
						Timer.FixedModeWait = false;
					}
				}
			}

			#endregion
			
			#region Methods

			/// <summary>
			/// TODO
			/// </summary>
			public void DeleteScreenShots()
			{
				string[] files = Directory.GetFiles(Path, PrefixScreenShot + "*.*");
				
				for (int i = 0; i < files.Length; i++)
					File.Delete(files[i]);
			}

			/// <summary>
			/// TODO
			/// </summary>
			public void Init()
			{
				Init("f", ImageFileFormat.Png, 1.0f / 25.0f, true);
			}

			/// <summary>
			/// TODO
			/// </summary>
			/// <param name="prefix">TODO</param>
			/// <param name="fileFormat">TODO</param>
			/// <param name="fps">TODO</param>
			/// <param name="deleteDirectoryScreenShots">TODO</param>
			public void Init(string prefix, ImageFileFormat fileFormat, float fps, bool deleteDirectoryScreenShots)
			{
				PrefixScreenShot = prefix;

				FileFormat = fileFormat;

				FPS = fps;

				if (deleteDirectoryScreenShots)
					DeleteScreenShots();

				Enabled = true;
			}

			#endregion
		}

		#endregion

        #region Fields

		/// <summary>
		/// TODO
		/// </summary>
		private static bool active = false;
		
		/// <summary>
		/// TODO
		/// </summary>
		private static JConfiguration configuration = new JConfiguration();

        /// <summary>
        /// Importer to import the data (lights, meshes,...) from a file
        /// </summary>
		private static JImport import = new JImport();

		/// <summary>
		/// Scene management:lights, nodes, materials, cameras,...
		/// </summary>
        public static JScene Scene = null;

        /// <summary>
        /// Default timer used in the engine
        /// </summary>
        static JTimer timer = null;

        /// <summary>
        /// Keyboard and mouse management
        /// </summary>
        static JInput input = null;

		/// <summary>
		/// TODO
		/// </summary>
        public static JGizmos Gizmos;
        
        /// <summary>
        /// Sound management
        /// </summary>
        static JSound sound = null;

		/// <summary>
		/// TODO
		/// </summary>
        static public JEffectBuilder EffectBuilder;

		/// <summary>
		/// TODO
		/// </summary>
		public static JEffectFragments EffectFragments;
		
		/// <summary>
		/// Newton physics
		/// </summary>
		public static JPhysics Physics;

		/// <summary>
		/// Video card and video objects management
		/// </summary>
		public static JVideo Video;

		/// <summary>
		/// TODO
		/// </summary>
		public static JVideoRecorder VideoRecorder;

		/// <summary>
		/// Form associated to the main window
		/// </summary>
		public static Form Form;
		
		/// <summary>
		/// Handler of the form or control where the render is done
		/// </summary>
		public static IntPtr Handle;
		
		/// <summary>
		/// Engine assembly
		/// </summary>
		public static Assembly AssemblyEngine;
		
		/// <summary>
		/// Max number of layers per material supported by the engine
		/// </summary>
		public static readonly int MaxLayers = 8;
		
		/// <summary>
		/// Max number of materials per mesh supported by the engine
		/// </summary>
		public static readonly int MaxMaterialsPerMesh = 16;

		/// <summary>
		/// Engine version
		/// </summary>
		public static readonly string Version = "Jad v1.0.0.0";

		/// <summary>
		/// Numeric format with "." as decimal symbol for the imports
		/// </summary>
		public static NumberFormatInfo DecimalFormatInfo;

		/// <summary>
		/// This field tells if pressing F12 creates and saves a screenshot
		/// </summary>
		public static bool UseF12ToSnapshot = true;

		/// <summary>
		/// TODO
		/// </summary>
		static int numPhoto = 0;
		
        #endregion
		
        #region Constructors

		/// <summary>
		/// TODO
		/// </summary>
        static Jad()
        {
			// Get access to the assembly for the resources
			AssemblyEngine = Assembly.GetExecutingAssembly();
			
			// To import the numbers that are saved using "." as the decimal separator symbol, 
			// it must be said explicitly to the NumberFormatInfo
            DecimalFormatInfo = new NumberFormatInfo();
            DecimalFormatInfo.NumberDecimalSeparator = ".";

			JLogWriter.Create("log");

            Video = new JVideo();

            Physics = new JPhysics();

			VideoRecorder.FileFormat = ImageFileFormat.Png;
			VideoRecorder.Path = @"ScreenShots\";
			VideoRecorder.PrefixScreenShot = "f";
			VideoRecorder.FPS = 1.0f / 25.0f;
		}

        #endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public static bool Active
		{
			get { return active; }
			set { active = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public static JConfiguration Configuration
		{
			get { return configuration; }
		}

		/// <summary>
		/// Gets the importer (to import the meshes, lights... from a file)
		/// </summary>
		public static JImport Import
		{
			get { return import; }
		}

		/// <summary>
		/// Gets or sets the keyboard and mouse management
		/// </summary>
		public static JInput Input
		{
			get { return input; }
			set { input = value; }
		}

		/// <summary>
		/// Gets the sound management
		/// </summary>
		public static JSound Sound
		{
			get { return sound; }
		}

		/// <summary>
		/// Gets or sets the default timer used by the engine
		/// </summary>
		public static JTimer Timer
		{
			get { return timer; }
			set { timer = value; }
		}

		#endregion
		
        #region Methods

		/// <summary>
		/// Creates a standard form. The client area will be filled by the engine
		/// </summary>
		public static void CreateStandardForm()
		{
			Form = new JBaseForm();
			Handle = Form.Handle;
		}

        /// <summary>
        /// Create the instances after the device has been created
        /// </summary>
        public static void AfterCreateDevice()
        {
        	// Get the directory path where the engine was executed
			JPath.GameDirectory = Directory.GetCurrentDirectory()+@"\";

            EffectFragments = new JEffectFragments();
            EffectBuilder = new JEffectBuilder();

        	// Dont use lighting, so disable it
            Video.Device.RenderState.Lighting = false;

        	// Activate the z-buffer
            Video.Device.RenderState.ZBufferEnable = true;

            Video.AfterCreate();

        	// Create and start the timer
			timer = new JTimer();
			timer.Start();

			// Activate HDR
            if (Configuration.Video.HDR)
				Video.RenderTargets.RTFormat = Format.A16B16G16R16F;
        }

        /// <summary>
        /// Creates and load the default object and textures
        /// </summary>
        public static void LoadCreateInHouse()
        {
            // Load default values
			Video.Effects.LoadInHouse();
			Video.Textures.LoadInHouse();
			Video.RenderTargets.CreateDefault();

        	// Lastly create the scene as it needs some default values created by this method
            Scene = new JScene();
			Scene.MeshObjects.LoadInHouse();

        	// Load ALL available shaders
			EffectFragments.LoadAllShaders();

			// Create the default material
			Scene.Materials.CreateDefaultMaterial();

			// Creamos los gizmos
			Gizmos = new JGizmos();
            
        	// TODO: ideally we would want to do this in the "Scene" constructor, where the
        	// root is created, but sadly "CreateByDefault" is called after the "AfterCreateDevice",
        	// and in that method is where the root is created, so the gizmos wouldnt be created yet
        	
        	// Gizmo for the root node of the scene
			Scene.Root.Gizmo = JGizmos.DummyGizmo;

            // Creamos la cmara dirigida por el usuario
        	
        	// Create the user directed camera
            JCameraFirstPerson user_camera = Scene.Cameras.CreateFirstPerson("haddd_camera");
            
        	user_camera.FOV = Geometry.DegreeToRadian(60.0f);
            user_camera.FarPlane = 20.0f;  // 20 m
            user_camera.NearPlane = 0.10f;   // 10 cm
            user_camera.Sensitivity = 20.0f;
            user_camera.Speed = 5.0f;
            user_camera.Position = new Vector3(0.0f, 0.0f, -5.0f);

        	// Assigning this field is very important as the engine needs to know
        	// always which is the active camera to use it in the shaders
            Scene.Camera = user_camera;
        }

        /// <summary>
        /// The application needs to be finalized in a certain order. If not, it could
        /// try to free first the object that contains the device before the objects that
        /// use it
        /// </summary>
        public static void End()
        {
			Scene.Dispose();

			Physics.Destroy();

            if(sound != null)
            	sound.Dispose();

			Video.Delete();

			JLogWriter.Close();

            timer.Stop();

            if(input != null)
            	input.Dispose();

            active = false;
        }
 
        /// <summary>
        /// After the user has created the objects, some things and values must be
        /// created and calculated
        /// </summary>
        public static void Begin()
        {
            Scene.CreatePostEffects();
			
        	// If HDR is activated, create the needed things for the scene
            if (Configuration.Video.HDR)
				Scene.CreateHDR();

			Video.Meshes.CenterAtOrigin();
			Video.Meshes.ComputeTangent();

			timer.Reset();

            active = true;
        }

        /// <summary>
        /// Creates the Input manager needed for the keyboard ad mouse
        /// </summary>
        public static void CreateInput()
        {
            Input = new JInput(Handle);
        }

        /// <summary>
        /// Creates the sound class
        /// </summary>
        /// <param name="cooperativeLevel">TODO</param>
		/// <param name="quality">TODO</param>
        public static void CreateSound(CooperativeLevel cooperativeLevel, JSoundQuality quality)
        {
            sound = new JSound(cooperativeLevel, quality);
        }
		
        /// <summary>
		/// Updates some internal fields of the engine. It must be called at the start of each frame
        /// </summary>
        public static void BeginUpdate()
        {
            // Update the input
            if (Input != null)
            {
                Input.Update();

				// F11 to commute the wireframe modes
                if (Input.Keyboard.KeyTouch(Key.F11))
                {
                    int n = (int)Scene.WireFrameMode;

                    n++;

                    if (n > 2)
                    	n = 0;
                    
                    Scene.WireFrameMode=(JWireframeMode) n;
                }

                // F12 to save a screenshot
                if (Input.Keyboard.KeyTouch(Key.F12))
                {
                	// In the first screenshot, delete all older files
					if (numPhoto == 0)
					{
						string[] files = Directory.GetFiles("ScreenShots", "snapshot*.*");
						for (int i = 0; i < files.Length; i++)
							File.Delete(files[i]);
					}

					numPhoto++;

                    ImageFileFormat format = ImageFileFormat.Jpg;

                    if(Input.Keyboard.KeyPressed(Key.LeftShift) || Input.Keyboard.KeyPressed(Key.RightShift))
                    	format=ImageFileFormat.Png;

                    Video.SaveSnapshot(@"ScreenShots\snapshot" + numPhoto.ToString("D3"), format);
                }
            }

        	// Update the state timers
			JTimerKey.Update();

			if(Timer.FixedMode && Timer.FixedModeWait)
				Timer.WaitForFixedTime();
		}

        /// <summary>
        /// Updates some internal fields of the engine. It must be called at the end of each frame
        /// </summary>
        public static void EndUpdate()
        {
			// Update the physics
            Physics.Update();

        	// Update the timer
            Timer.Frame();

			Scene.Update();

            // Update the sound
            if (Sound != null)
            	Sound.Update();
        }

        /// <summary>
		/// This method returns the TimeBased value that happens in 1 second
        /// </summary>
        /// <param name="value">TODO</param>
		/// <returns>TODO</returns>
        public static float At1Second(float value)
        {
            return value * Timer.DeltaTime;
        }

        /// <summary>
		/// This method returns the TimeBased value that occurs in X seconds
        /// </summary>
        /// <param name="seconds">TODO</param>
		/// <param name="value">TODO</param>
		/// <returns>TODO</returns>
        public static float AtXSecond(float seconds, float value)
        {
            return (value / seconds) * Timer.DeltaTime;
        }

		#endregion
	}
}
