#region Using directives

using System;

#endregion

namespace JadEngine.Physics
{
    /// <summary>
    /// Handles the Newton materials
    /// </summary>
    public class JPhysicsMaterial
    {
        #region Fields
        
        /// <summary>
        /// The Newton material pointer
        /// </summary>
        int material;

        #endregion

        #region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JPhysicsMaterial()
		{
			material = JNewtonWrapper.MaterialCreateGroupID(Jad.Physics.World);
		}
    	
        /// <summary>
        /// Builds the material, giving it a fixed value for the material pointer
        /// </summary>
        /// <param name="id">Value for material</param>
        public JPhysicsMaterial(int id)
        {
            material = id;
        }

		#endregion

		#region Propierties

		/// <summary>
		/// Gets the Newton material pointer
		/// </summary>
		public int Material
		{
			get { return material; }
		}

		#endregion
    	
        #region Methods

		/// <summary>
		/// Create a collision callback system when two materials collide
		/// </summary>
		/// <param name="mat1">First material</param>
		/// <param name="mat2">Second material</param>
		/// <param name="userData">User data</param>
		/// <param name="begin">Delegate when the contact begins</param>
		/// <param name="process">Delegate when the contact happens</param>
		/// <param name="end">Delete when the contact has been processed</param>
		public static void SetCollisionDelegate(JPhysicsMaterial mat1, JPhysicsMaterial mat2, Object userData, MaterialSetCollisionBegin begin, MaterialSetCollisionProcess process, MaterialSetCollisionEnd end)
		{
			JNewtonWrapper.MaterialSetCollisionCallback(Jad.Physics.World, mat1.Material, mat2.Material, userData, begin, process, end);
		}
    	
        /// <summary>
        /// Sets the contact elasticity
        /// </summary>
		/// <param name="elasticity">The new elasticity value</param>
		public void SetContactElasticity(float elasticity)
        {
			JNewtonWrapper.MaterialSetContactElasticity(material, elasticity);
        }

        /// <summary>
        /// Sets the default elasticity between the material and another material
        /// </summary>
        /// <param name="id1">The second material</param>
        /// <param name="elasticity">The new value</param>
        public void SetDefaultElasticity(int id1, float elasticity)
        {
            JNewtonWrapper.MaterialSetDefaultElasticity(Jad.Physics.World, material, id1, elasticity);
        }

        /// <summary>
        /// Sets the default elasticity
        /// </summary>
		/// <param name="elasticity">The new elasticity value</param>
        public void SetDefaultElasticity(float elasticity)
        {
            SetDefaultElasticity(material, elasticity);
        }

        /// <summary>
        /// Sets the default friction between the material and another material
        /// </summary>
        /// <param name="id1">The second material</param>
        /// <param name="staticFriction">Static value</param>
        /// <param name="kineticFriction">Kinetic value</param>
		public void SetDefaultFriction(int id1, float staticFriction, float kineticFriction)
		{
			JNewtonWrapper.MaterialSetDefaultFriction(Jad.Physics.World, material, id1, staticFriction, kineticFriction);
		}

        /// <summary>
        /// Sets the default friction for this material
        /// </summary>
        /// <param name="staticFriction">Static value</param>
        /// <param name="kineticFriction">Kinetic value</param>
        public void SetDefaultFriction(float staticFriction, float kineticFriction)
		{
			SetDefaultFriction(material, staticFriction, kineticFriction);
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="id1">TODO</param>
		/// <param name="mode">TODO</param>
        public void SetContinuousCollisionMode(int id1, JContinuousCollisionMode mode)
        {
            JNewtonWrapper.MaterialSetContinuousCollisionMode(Jad.Physics.World, material, id1, (uint)mode);
        }

        #endregion
    }
}
