#region Using directives

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Physics
{
    /// <summary>
    /// Class to control the hinge joints by Newton
    /// </summary>
    public class JHingeJoint : JBaseJoint
    {
        #region Fields

        /// <summary>
        /// The pivot point in World Space
        /// </summary>
        Vector3 pivotPoint;

        /// <summary>
        /// The rotation vector
        /// </summary>
        Vector3 pinDir;

		/// <summary>
		/// Max angle limit in radians
		/// </summary>
		public float MaxAngleLimit = Geometry.DegreeToRadian(180f);

		/// <summary>
		/// Min angle limit in radians
		/// </summary>
		public float MinAngleLimit = Geometry.DegreeToRadian(45f);

		/// <summary>
		/// Enable / Disable the angle limit cones
		/// </summary>
		public bool UseAngleLimit = false;

		/// <summary>
		/// The current joint angle
		/// </summary>
		public float Angle = 0f;
    	
        #endregion

        #region Constructors

        /// <summary>
        /// Builds a Hinge joint
        /// </summary>
        /// <param name="pivotPoint">Pivot point in World space</param>
        /// <param name="pinDir">Rotation vector</param>
        /// <param name="child">Child <see cref="JRigidBody"/></param>
        /// <param name="parent">Parent <see cref="JRigidBody"/> or null if its the first one</param>
        public JHingeJoint(Vector3 pivotPoint, Vector3 pinDir, JRigidBody child, JRigidBody parent)
            : base(child, parent)
		{
			int bparent=0;

			if (parent != null) bparent = parent.Body;

            this.pivotPoint = pivotPoint;
            this.pinDir = pinDir;

            joint = JNewtonWrapper.ConstraintCreateHinge(Jad.Physics.World, ref pivotPoint,ref pinDir, child.Body, bparent);

			JNewtonWrapper.HingeSetUserCallback(joint, Jad.Physics.hingeJointCallBack);

            OnCreate();

        }

        #endregion

		#region Propierties

		/// <summary>
		/// Gets the pivot point in World Space
		/// </summary>
		public Vector3 PivotPoint
		{
			get { return pivotPoint; }
		}

		/// <summary>
		/// Gets the rotation vector
		/// </summary>
		public Vector3 PinDir
		{
			get { return pinDir; }
		}

		#endregion
    }
}
