#region Using directives

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Physics
{
    /// <summary>
    /// Class that controls a Slider joint from Newton
    /// </summary>
    public class JSliderJoint : JBaseJoint
    {
        #region Fields

        /// <summary>
        /// The pivot point in World Space
        /// </summary>
        Vector3 pivotPoint;

        /// <summary>
        /// The rotation vector
        /// </summary>
        Vector3 pinDir;

		/// <summary>
		/// Max angle limit in radians
		/// </summary>
		public float AngleLimit = Geometry.DegreeToRadian(180f);

		/// <summary>
		/// Enable / Disable the angle limit
		/// </summary>
		public bool UseAngleLimit = false;

		/// <summary>
		/// Minimum distance limit, handled by the callback
		/// </summary>
		public float LimitsMin;

		/// <summary>
		/// Maximum distance limits, handled by the callback
		/// </summary>
		public float LimitsMax;

        #endregion

		#region Constructors

		/// <summary>
		/// Builds a slider joint. This kind of joint its like a bar atached to an object
		/// </summary>
		/// <param name="pivotPoint">Pivot point in World space</param>
		/// <param name="pinDir">Rotation vector</param>
		/// <param name="child">Child <see cref="JRigidBody"/></param>
		/// <param name="parent">Parent <see cref="JRigidBody"/>. Can not be null</param>
		public JSliderJoint(Vector3 pivotPoint, Vector3 pinDir, JRigidBody child, JRigidBody parent)
			: base(child, parent)
		{
			this.pivotPoint = pivotPoint;
			this.pinDir = pinDir;

			joint = JNewtonWrapper.ConstraintCreateSlider(Jad.Physics.World, ref pivotPoint, ref pinDir, child.Body, parent.Body);

			JNewtonWrapper.SliderSetUserCallback(joint, Jad.Physics.sliderJointCallBack);

			OnCreate();
		}

		#endregion
    	
        #region Propiedades

        /// <summary>
        /// Gets the pivot point in World Space
        /// </summary>
        public Vector3 PivotPoint
        {
			get { return pivotPoint; }
        }

        /// <summary>
        /// Gets the rotation vector
        /// </summary>
        public Vector3 PinDir
        {
            get { return pinDir; }
        }

        #endregion
    }
}
