#region Using directives

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Physics
{
    /// <summary>
    /// An up vector joint is a constraint that allows a body to translate freely in 3d space, 
    /// but it only allows the body to rotate around the pin direction vector. 
    /// This could be use by the application to control a character with physics and collision. 
    /// </summary>
    public class JUpVectorJoint : JBaseJoint
    {
        #region Fields

        /// <summary>
        /// The rotation vector
        /// </summary>
        Vector3 pinDir;

		/// <summary>
		/// Angle limit in radians
		/// </summary>
		public float AngleLimit = Geometry.DegreeToRadian(180f);

		/// <summary>
		/// Enable / Disable the angle limit cones
		/// </summary>
		public bool UseAngleLimit = false;

		/// <summary>
		/// Minimum distance limit, handled by the callback
		/// </summary>
		public float LimitsMin;

    	/// <summary>
    	/// Maximum distance limits, handled by the callback
    	/// </summary>
    	public float LimitsMax;

    	#endregion

        #region Constructors

        /// <summary>
        /// Builds an UpVector joint. You can move freely, but only 
        /// rotate on the rotation axis indicated
        /// </summary>
        /// <param name="pinDir">The rotation vector</param>
        /// <param name="body">The body linked to the joint</param>
        public JUpVectorJoint(Vector3 pinDir, JRigidBody body) : base(body,null)
        {
            joint = JNewtonWrapper.ConstraintCreateUpVector(Jad.Physics.World, ref pinDir, body.Body);

            this.pinDir = pinDir;

            OnCreate();
        }

        #endregion

		#region Propierties

		/// <summary>
		/// Gets or sets the rotation vector</summary>
		public Vector3 PinDir
		{
			get { return pinDir; }
			set
			{
				pinDir = value; 
				JNewtonWrapper.UpVectorSetPin(joint, ref pinDir);
			}
		}

		#endregion
    }
}
