#region Using directives

using System;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Particles;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// TODO
	/// </summary>
	public abstract class JGizmo
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected const float avatarWidth = 0.15f;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const float avatarHeight = 0.15f;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const float maxFrustumDistance = 0.0f;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const string meshAvatarFile = "mesh";
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const string particleEffectAvatarFile = "particleEffects";
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const string omniAvatarFile = "omni";
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const string spotAvatarFile = "spot";
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const string cameraAvatarFile = "camera";
		
		/// <summary>
		/// TODO
		/// </summary>
		protected const string dummyAvatarFile = "dummy";

		/// <summary>
		/// Avatar that represents the node
		/// </summary>
		protected JBillboard avatar;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JEffect effect;
		
		/// <summary>
		/// TODO
		/// </summary>
        protected JColor color;
        
		/// <summary>
		/// TODO
		/// </summary>
		protected int numIndices;
		
		/// <summary>
		/// Buffer with the indices of the gizmo
		/// </summary>
		protected short[] indices;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected int numVertices;
		
		/// <summary>
		/// Buffer with the vertex of the gizmos
		/// </summary>
		protected CustomVertex.PositionOnly[] vertices;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected Matrix world;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		protected JGizmo()
		{
			// Default color for the gizmo lines
			color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);

			// Default world matrix
			world = Matrix.Identity;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// Gets the avatar that represents the node
		/// </summary>
		public JBillboard Avatar
		{
			get { return avatar; }
		}

		/// <summary>
		/// TODO
		/// </summary>
        public JColor Color
        {
			get { return color; }
			set { color = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		public virtual void Render(JNode node)
		{
			// Set the avatar position
			avatar.Position = node.Position;

			// Render the avatar
			avatar.Render();

			// Only paint the gizmo lines if there are vertices that must be paint!
			if (numVertices > 0 && numIndices > 0)
			{
				// Configure the effect
				effect.DXResource.Technique = effect.handlers.renderScene;
				effect.Begin();

				effect.DXResource.SetValue(effect.handlers.worldViewProjection, world * Jad.Scene.Camera.ViewProjection);
				effect.DXResource.SetValue(Jad.Video.Effects.InHouse.PosColor.handlers.materialHandler.Diffuse, color.ToArgb());

				Jad.Video.Device.VertexFormat = CustomVertex.PositionOnly.Format;

				effect.DXResource.BeginPass(0);
				effect.DXResource.CommitChanges();

				// Render the gizmo lines
				Jad.Video.Device.DrawIndexedUserPrimitives(PrimitiveType.LineList, 0, numIndices, numIndices / 2, indices, true, vertices);

				effect.DXResource.EndPass();

				effect.End();
			}
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JMeshGizmo : JGizmo
    {
        #region Fields

        /// <summary>
        /// Indicates if we want to render the HierarchicalBoundings too
        /// </summary>
        bool showHierarchicalBoundings;

        #endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JMeshGizmo()
			: base()
		{
			// Create the avatar
			avatar = new JBillboard("MeshGizmoAvatar");
			avatar.Width = avatarWidth;
			avatar.Height = avatarHeight;
			avatar.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
			avatar.Texture = Jad.Video.Textures.Create2D(meshAvatarFile, true);
			avatar.Texture.InHouse = true;
			avatar.Effect = Jad.Video.Effects.InHouse.PosColorTex1;

			// Gizmo lines color
			color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
		}

		#endregion
		
        #region Propierties

        /// <summary>
        /// Gets or sets if we want to render the HierarchicalBoundings too
        /// </summary>
        public bool ShowHierarchicalBoundings
        {
            get { return showHierarchicalBoundings; }
            set { showHierarchicalBoundings = value; }
        }

        #endregion

		#region Methods

		/// <summary>
        /// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		/// <remarks>It renders the Bounding Box + HierarchicalBoundingBox (Yellow)</remarks>
		public override void Render(JNode node)
		{
            // Avatar position
            avatar.Position = node.Position;

            // Render the avatar
            avatar.Render();

            Vector3[] vertices = node.BoundingBox.ToRenderLines();

            Jad.Video.Effects.InHouse.PosColor.RenderLines(color, vertices.Length, vertices);

            if (showHierarchicalBoundings)
            {
                vertices = node.HierarchicalBoundingBox.ToRenderLines();
                Jad.Video.Effects.InHouse.PosColor.RenderLines(JColor.Yellow, vertices.Length, vertices);
            }
        }

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JParticleEffectGizmo : JGizmo
	{
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JParticleEffectGizmo() : base()
		{
			// Create the avatar
			avatar = new JBillboard("ParticleEffectGizmoAvatar");
			avatar.Width = avatarWidth;
			avatar.Height = avatarHeight;
            avatar.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
            avatar.Texture = Jad.Video.Textures.Create2D(particleEffectAvatarFile, true);
			avatar.Texture.InHouse = true;
			avatar.Effect = Jad.Video.Effects.InHouse.PosColorTex1;

			// Gizmo lines color
            color = new JColor(0.0f, 0.0f, 1.0f, 1.0f);

            // Effect
			effect = Jad.Video.Effects.InHouse.PosColor;

			// Allocate memory for the vertices
			numVertices = 8;
			vertices = new CustomVertex.PositionOnly[numVertices];

			// Allocate memory for the indices
			numIndices = 24;
			indices = new short[numIndices];

			// Define the indices
			indices[0] = 0;	// Top
			indices[1] = 1;
			indices[2] = 1;
			indices[3] = 2;
			indices[4] = 2;
			indices[5] = 3;
			indices[6] = 3;
			indices[7] = 0;
			indices[8] = 4;	// Bottom
			indices[9] = 5;
			indices[10] = 5;
			indices[11] = 6;
			indices[12] = 6;
			indices[13] = 7;
			indices[14] = 7;
			indices[15] = 4;
			indices[16] = 0;	// Right-Far
			indices[17] = 6;
			indices[18] = 1;	// Right-Near
			indices[19] = 7;
			indices[20] = 3;	// Left-Far
			indices[21] = 5;
			indices[22] = 2;	// Left-Near
			indices[23] = 4;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		public override void Render(JNode node)
		{
			Vector3 min;
			Vector3 max;

			JParticleEffect fx = node as JParticleEffect;

			min = fx.LocalBoundingBox.Min;
			max = fx.LocalBoundingBox.Max;

			// Define the vertices
			vertices[0] = new CustomVertex.PositionOnly(max.X, max.Y, max.Z);
			vertices[1] = new CustomVertex.PositionOnly(max.X, max.Y, min.Z);
			vertices[2] = new CustomVertex.PositionOnly(min.X, max.Y, min.Z);
			vertices[3] = new CustomVertex.PositionOnly(min.X, max.Y, max.Z);

			vertices[4] = new CustomVertex.PositionOnly(min.X, min.Y, min.Z);
			vertices[5] = new CustomVertex.PositionOnly(min.X, min.Y, max.Z);
			vertices[6] = new CustomVertex.PositionOnly(max.X, min.Y, max.Z);
			vertices[7] = new CustomVertex.PositionOnly(max.X, min.Y, min.Z);

			// Render
			base.Render(node);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JOmniLightGizmo : JGizmo
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected const int numSubdivisions = 30;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected int numVerticesPerCircle;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected int numIndicesPerCircle;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JOmniLightGizmo() : base()
		{
			// Create the avatar
			avatar = new JBillboard("OmniLightGizmoAvatar");
			avatar.Width = avatarWidth;
			avatar.Height = avatarHeight;
            avatar.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
            avatar.Texture = Jad.Video.Textures.Create2D(omniAvatarFile,true);
			avatar.Texture.InHouse = true;
			avatar.Effect = Jad.Video.Effects.InHouse.PosColorTex1;

			// Gizmo lines color
            color = new JColor(1.0f, 1.0f, 0.0f, 1.0f);

            // Effect
			effect = Jad.Video.Effects.InHouse.PosColor;

			// Allocate memory for the vertices
			numVerticesPerCircle = numSubdivisions;
			numVertices = numVerticesPerCircle * 3;
			vertices = new CustomVertex.PositionOnly[numVertices];

			// Allocate memory for the indices
			numIndicesPerCircle = numVerticesPerCircle * 2;
			numIndices = numIndicesPerCircle * 3;
			indices = new short[numIndices];

			// Define the indices
			for (int i = 0; i < numVerticesPerCircle; i++)
			{
				int idx1, idx2;

				idx1 = i;

				if (i < numVerticesPerCircle - 1)
					idx2 = i + 1;
				
				else
					idx2 = 0;

				int c1_offs = 0;
				int c2_offs = numVerticesPerCircle;
				int c3_offs = numVerticesPerCircle * 2;

				indices[c1_offs * 2 + i * 2 + 0] = (short)(idx1 + c1_offs);
				indices[c1_offs * 2 + i * 2 + 1] = (short)(idx2 + c1_offs);

				indices[c2_offs * 2 + i * 2 + 0] = (short)(idx1 + c2_offs);
				indices[c2_offs * 2 + i * 2 + 1] = (short)((idx2 > 0 ? idx2 : 0) + c2_offs);

				indices[c3_offs * 2 + i * 2 + 0] = (short)(idx1 + c3_offs);
				indices[c3_offs * 2 + i * 2 + 1] = (short)((idx2 > 0 ? idx2 : 0) + c3_offs);
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		public override void Render(JNode node)
		{
			JOmniLight omni = node as JOmniLight;

			// Light color
			//avatar.Color = omni.Color;

			// Define the vertices
			float delta = (float)(2.0 * Math.PI) / numVerticesPerCircle;

			int c1_offs = 0;
			int c2_offs = numVerticesPerCircle;
			int c3_offs = numVerticesPerCircle * 2;

			for (int i = 0; i < numVerticesPerCircle; i++)
			{
				float radians = delta * i;
				float cos = (float)Math.Cos(radians) * omni.Radius;
				float sin = (float)Math.Sin(radians) * omni.Radius;

				vertices[c1_offs + i] = new CustomVertex.PositionOnly(cos + omni.Position.X, sin + omni.Position.Y, omni.Position.Z);
				vertices[c2_offs + i] = new CustomVertex.PositionOnly(cos + omni.Position.X, omni.Position.Y, sin + omni.Position.Z);
				vertices[c3_offs + i] = new CustomVertex.PositionOnly(omni.Position.X, cos + omni.Position.Y, sin + omni.Position.Z);
			}

			// Render
			base.Render(node);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JSpotLightGizmo : JGizmo
	{
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JSpotLightGizmo() : base()
		{
			// Create the avatar
			avatar = new JBillboard("SpotLightGizmoAvatar");
			avatar.Width = avatarWidth;
			avatar.Height = avatarHeight;
            avatar.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
            avatar.Texture = Jad.Video.Textures.Create2D(spotAvatarFile,true);
			avatar.Texture.InHouse = true;
			avatar.Effect = Jad.Video.Effects.InHouse.PosColorTex1;

			// Gizmo lines color
            color = new JColor(1.0f, 1.0f, 0.0f, 1.0f);

            // Effect
			effect = Jad.Video.Effects.InHouse.PosColor;

			// Allocate memory for the vertices
			numVertices = 8;
			vertices = new CustomVertex.PositionOnly[numVertices];

			// Define the vertices
			vertices[0] = new CustomVertex.PositionOnly( 1.0f, 1.0f,  1.0f);
			vertices[1] = new CustomVertex.PositionOnly( 1.0f, 1.0f, -1.0f);
			vertices[2] = new CustomVertex.PositionOnly(-1.0f, 1.0f, -1.0f);
			vertices[3] = new CustomVertex.PositionOnly(-1.0f, 1.0f,  1.0f);

			vertices[4] = new CustomVertex.PositionOnly(-1.0f, -1.0f, -1.0f);
			vertices[5] = new CustomVertex.PositionOnly(-1.0f, -1.0f,  1.0f);
			vertices[6] = new CustomVertex.PositionOnly( 1.0f, -1.0f,  1.0f);
			vertices[7] = new CustomVertex.PositionOnly( 1.0f, -1.0f, -1.0f);

			// Allocate memory for the indices
			numIndices = 24;
			indices = new short[numIndices];

			// Define the indices
			indices[0]  = 0;	// Top
			indices[1]  = 1;
			indices[2]  = 1;
			indices[3]  = 2;
			indices[4]  = 2;
			indices[5]  = 3;
			indices[6]  = 3;
			indices[7]  = 0;
			indices[8]  = 4;	// Bottom
			indices[9]  = 5;
			indices[10] = 5;
			indices[11] = 6;
			indices[12] = 6;
			indices[13] = 7;
			indices[14] = 7;
			indices[15] = 4;
			indices[16] = 0;	// Right-Far
			indices[17] = 6;
			indices[18] = 1;	// Right-Near
			indices[19] = 7;
			indices[20] = 3;	// Left-Far
			indices[21] = 5;
			indices[22] = 2;	// Left-Near
			indices[23] = 4;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		public override void Render(JNode node)
		{
			JSpotLight spot = node as JSpotLight;

			// Avatar color
			//avatar.Color = spot.Color;

			float scale = 1.0f;

			if(maxFrustumDistance > 0.0f)
				scale = maxFrustumDistance / spot.Radius;

			// Set the matrix to transform from clip-space to world-space
            world = Matrix.Invert(spot.projectorPerspective) * Matrix.Scaling(scale, scale, scale) * spot.WorldTM;

			// Render
			base.Render(node);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JCameraGizmo : JGizmo
	{
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JCameraGizmo() : base()
		{
			// Create the avatar
			avatar = new JBillboard("CameraGizmoAvatar");
			avatar.Width = avatarWidth;
			avatar.Height = avatarHeight;
            avatar.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
            avatar.Texture = Jad.Video.Textures.Create2D(cameraAvatarFile,true);
			avatar.Texture.InHouse = true; 
			avatar.Effect = Jad.Video.Effects.InHouse.PosColorTex1;

			// Gizmo lines color
            color = new JColor(1.0f, 0.0f, 0.0f, 1.0f);

            // Effect
			effect = Jad.Video.Effects.InHouse.PosColor;

			// Allocate memory for the vertices
			numVertices = 8;
			vertices = new CustomVertex.PositionOnly[numVertices];

			// Define the vertices
			vertices[0] = new CustomVertex.PositionOnly(1.0f, 1.0f, 1.0f);
			vertices[1] = new CustomVertex.PositionOnly(1.0f, 1.0f, -1.0f);
			vertices[2] = new CustomVertex.PositionOnly(-1.0f, 1.0f, -1.0f);
			vertices[3] = new CustomVertex.PositionOnly(-1.0f, 1.0f, 1.0f);

			vertices[4] = new CustomVertex.PositionOnly(-1.0f, -1.0f, -1.0f);
			vertices[5] = new CustomVertex.PositionOnly(-1.0f, -1.0f, 1.0f);
			vertices[6] = new CustomVertex.PositionOnly(1.0f, -1.0f, 1.0f);
			vertices[7] = new CustomVertex.PositionOnly(1.0f, -1.0f, -1.0f);

			// Allocate memory for the indices
			numIndices = 24;
			indices = new short[numIndices];

			// Define the indices
			indices[0] = 0;	// Top
			indices[1] = 1;
			indices[2] = 1;
			indices[3] = 2;
			indices[4] = 2;
			indices[5] = 3;
			indices[6] = 3;
			indices[7] = 0;
			indices[8] = 4;	// Bottom
			indices[9] = 5;
			indices[10] = 5;
			indices[11] = 6;
			indices[12] = 6;
			indices[13] = 7;
			indices[14] = 7;
			indices[15] = 4;
			indices[16] = 0;	// Right-Far
			indices[17] = 6;
			indices[18] = 1;	// Right-Near
			indices[19] = 7;
			indices[20] = 3;	// Left-Far
			indices[21] = 5;
			indices[22] = 2;	// Left-Near
			indices[23] = 4;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		public override void Render(JNode node)
		{
			JCamera camera = node as JCamera;

			float scale = 1.0f;

			if (maxFrustumDistance > 0.0f)
				scale = maxFrustumDistance / camera.FarPlane;

			// Set the matrix to transform from clip-space to world-space
			world = Matrix.Invert(camera.Projection) * Matrix.Scaling(scale, scale, scale) * Matrix.Invert(camera.View);

			// Render
			base.Render(node);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JDummyGizmo : JGizmo
	{
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JDummyGizmo() : base()
		{
			// Create the avatar
			avatar = new JBillboard("DummyGizmoAvatar");
			avatar.Width = avatarWidth;
			avatar.Height = avatarHeight;
            avatar.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
            avatar.Texture = Jad.Video.Textures.Create2D(dummyAvatarFile, true);
			avatar.Texture.InHouse = true;
            avatar.Effect = Jad.Video.Effects.InHouse.PosColorTex1;

			// Vertices and indices arent needed
			numVertices = numIndices = 0;
		}

		#endregion Constructores

		#region Methods

		/// <summary>
		/// Renders the gizmo
		/// </summary>
		/// <param name="node">Node associated with this gizmo</param>
		public override void Render(JNode node)
		{
			// Render
			base.Render(node);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JGizmos
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		private static JMeshGizmo meshGizmo;
		
		/// <summary>
		/// TODO
		/// </summary>
		private static JParticleEffectGizmo particleEffectGizmo;
		
		/// <summary>
		/// TODO
		/// </summary>
		private static JOmniLightGizmo omniGizmo;
		
		/// <summary>
		/// TODO
		/// </summary>
		private static JSpotLightGizmo spotGizmo;
		
		/// <summary>
		/// TODO
		/// </summary>
		private static JCameraGizmo cameraGizmo;
		
		/// <summary>
		/// TODO
		/// </summary>
		private static JDummyGizmo dummyGizmo;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JGizmos()
		{
			// Gizmo for the nodes of type "Mesh"
			meshGizmo = new JMeshGizmo();

			// Gizmo for the nodes of type "ParticleEffect"
			particleEffectGizmo = new JParticleEffectGizmo();

			// Gizmo for the nodes of type "OmniLight"
			omniGizmo = new JOmniLightGizmo();

			// Gizmo for the nodes of type "SpotLight"
			spotGizmo = new JSpotLightGizmo();

			// Gizmo for the nodes of type "Camera"
			cameraGizmo = new JCameraGizmo();

			// Gizmo for the nodes of type "Dummy"
			dummyGizmo = new JDummyGizmo();
		}

		#endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public static JMeshGizmo MeshGizmo
		{
			get { return meshGizmo; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public static JParticleEffectGizmo ParticleEffectGizmo
		{
			get { return particleEffectGizmo; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public static JOmniLightGizmo OmniGizmo
		{
			get { return omniGizmo; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public static JSpotLightGizmo SpotLightGizmo
		{
			get { return spotGizmo; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public static JCameraGizmo CameraGizmo
		{
			get { return cameraGizmo; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public static JDummyGizmo DummyGizmo
		{
			get { return dummyGizmo; }
		}

		#endregion
	}
}
