#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;
using JadEngine.Tools;

#endregion

namespace JadEngine.Scene
{
    /// <summary>
    /// The various light types that Jad suports
    /// </summary>
	public enum JLightType
	{
    	/// <summary>
    	/// TODO
    	/// </summary>
		Omni=0,

		/// <summary>
		/// TODO
		/// </summary>
		Spot = 1,

		/// <summary>
		/// TODO
		/// </summary>
		Directional = 2,

		/// <summary>
		/// TODO
		/// </summary>
		Unknown = 3
	}

    /// <summary>
    /// The type of attenuation
    /// </summary>
	public enum JLightAttenuationType
	{
		/// <summary>
		/// TODO
		/// </summary>
		None = 0,

		/// <summary>
		/// TODO
		/// </summary>
		Inverse = 1,

		/// <summary>
		/// TODO
		/// </summary>
		InverseSquare = 2,

		/// <summary>
		/// TODO
		/// </summary>
		DualRadius = 3
	}

    /// <summary>
    /// The type of projectors asociated to the lights
    /// </summary>
	public enum JProjectorType
	{
		/// <summary>
		/// TODO
		/// </summary>
		Sampler2D,

		/// <summary>
		/// TODO
		/// </summary>
		SamplerCUBE
	}

    /// <summary>
    /// This is the base light class, inherited by the other light types
    /// </summary>
	public abstract class JBaseLight : JNode
	{
		#region Structs

    	/// <summary>
    	/// TODO
    	/// </summary>
        public struct JCorona
		{
			#region Fields
    		
			/// <summary>
			/// TODO
			/// </summary>
			public bool Enabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal Query MaxPixelsQuery;

			/// <summary>
			/// TODO
			/// </summary>
			internal Query CurPixelsQuery;

			/// <summary>
			/// TODO
			/// </summary>
			public int maxPixels;

    		/// <summary>
    		/// TODO
    		/// </summary>
    		public int curPixels;

    		/// <summary>
			/// TODO
			/// </summary>
			public JTexture Texture;

			#endregion

			#region Methods
    		
			/// <summary>
    		/// TODO
    		/// </summary>
			public void Init()
            {
                Texture = Jad.Video.Textures.Create2D("corona2", true);
                Reset();
            }

			/// <summary>
			/// TODO
			/// </summary>
			public void Destroy()
            {
                if (MaxPixelsQuery != null)
                {
                    MaxPixelsQuery.Dispose();
                    CurPixelsQuery.Dispose();
                }
            }

			/// <summary>
			/// TODO
			/// </summary>
			internal void Reset()
            {
                MaxPixelsQuery = new Query(Jad.Video.Device, QueryType.Occlusion);
                CurPixelsQuery = new Query(Jad.Video.Device, QueryType.Occlusion);
            }

			/// <summary>
			/// TODO
			/// </summary>
			internal void OnLost()
            {
                Destroy();
			}

			#endregion
		}

        /// <summary>
        /// Defines the possible parameters for the volumetric lights
        /// </summary>
		public struct JVolumetricLight
		{
			#region Fields
        	
			/// <summary>
			/// TODO
			/// </summary>
			public bool Enabled;
			
        	/// <summary>
        	/// TODO
        	/// </summary>
        	public float PowValue;

			#endregion
		}

        /// <summary>
        /// Defines the possible parameters for the projectors
        /// </summary>
        public struct JProjectorLight
		{
			#region Fields
        	
			/// <summary>
            /// Enable/Disable the projector
            /// </summary>
            public bool Enabled;
            
        	/// <summary>
        	/// The matrix for the projector
        	/// </summary>
            public Matrix Matrix;
            
        	/// <summary>
        	/// The texture for the projector. Can be <see cref="JTexture"/> for spot lights or <see cref="JTextureCube"/> for omni lights
        	/// </summary>
            public JTextureBase Map;

			#endregion
		}

        #endregion

		#region Fields

		/// <summary>
		/// The type of the light
		/// </summary>
		protected JLightType type;
		
    	/// <summary>
    	/// The color
    	/// </summary>
		protected JColor color;
		
    	/// <summary>
    	/// The intensity
    	/// </summary>
		protected float multiplier;
		
    	/// <summary>
    	/// Indicates if the light affects the diffuse material of the objects
    	/// </summary>
		protected bool affectDiffuse;
        
    	/// <summary>
    	/// Indicates if the light affects the specular material of the objects
    	/// </summary>
        protected bool affectSpecular;
		
    	/// <summary>
    	/// The type of attenuation
    	/// </summary>
		protected JLightAttenuationType attenType;
		
    	/// <summary>
    	/// The attenuation start point, relative to the light
    	/// </summary>
		protected float farAttenStart;
        
    	/// <summary>
    	/// The attenuation end point, relative to the light
    	/// </summary>
        protected float farAttenEnd;
		
    	/// <summary>
    	/// Indicates if this light cast shadows on the scene
    	/// </summary>
		protected bool castShadows;
		
    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <remarks>TEMPORAL!</remarks>
		protected JMeshObject mesh;

		/// <summary>
		/// The values for volumetric lights
		/// </summary>
		public JVolumetricLight Volumetric;

		/// <summary>
		/// The values for the projector
		/// </summary>
		public JProjectorLight Projector;

		/// <summary>
		/// The corona effect and halo surrounding the light
		/// </summary>
		public JCorona Corona;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="name">The name of the light</param>
		public JBaseLight(string name)
			: base(name)
		{
			// Light type
			type = JLightType.Unknown;

			// Default color
			color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);

			// Normal intensity
			multiplier = 1.0f;

			// This is the attenuation that happens in the real world
			attenType = JLightAttenuationType.InverseSquare;

			// Default radius
			farAttenEnd = 200.0f;
			farAttenStart = 50.0f;

			// It affects the diffuse and specular components
			affectDiffuse = true;
			affectSpecular = true;

			Volumetric.PowValue = 1.0f;
		}

		#endregion
    	
		#region Properties
    	
        /// <summary>
        /// Gets the light type
        /// </summary>
        public JLightType Type
		{
			get { return type; }
		}

        /// <summary>
        /// Gets or sets the color
        /// </summary>
        public JColor Color
		{
			get { return color; }
			set { color = value; }
		}

		/// <summary>
		/// Gets color * (multiplier + Jad.Scene.Lights.Intensity). Used for shaders
		/// </summary>
		/// <value></value>
		internal JColor ShaderColor
		{
			get { return color * (multiplier+Jad.Scene.Lights.Intensity); }
		}

        /// <summary>
        /// Gets or sets the intensity of the light
        /// </summary>
        public float Multiplier
		{
			get { return multiplier; }
			set { multiplier = value; }
		}

        /// <summary>
        /// Gets or sets if the light affects the diffuse material of the objects
        /// </summary>
        public bool AffectDiffuse
		{
			get { return affectDiffuse; }
			set { affectDiffuse = value; }
		}

        /// <summary>
        /// Gets or sets if the light affects the specular material of the objects
        /// </summary>
        public bool AffectSpecular
		{
			get { return affectSpecular; }
			set { affectSpecular = value; }
		}

        /// <summary>
        /// Gets or sets the attenuation type
        /// </summary>
        public JLightAttenuationType AttenuationType
		{
			get { return attenType; }
			set { attenType = value; }
		}

        /// <summary>
        /// Gets or sets the attenuation start point, relative to the light
        /// </summary>
        public float FarAttenuationStart
		{
			get { return farAttenStart; }
			set { farAttenStart = value; }
		}

        /// <summary>
        /// Gets or sets the attenuation end point, relative to the light. This is must be value, used to compute the visibility
        /// </summary>
        public float FarAttenuationEnd
		{
			get { return farAttenEnd; }
			set { farAttenEnd = value; }
		}

        /// <summary>
        /// Gets the name used in the <see cref="JEffectBuilder"/>
        /// </summary>
		internal abstract string NameInShaders
		{
			get;
		}

        /// <summary>
        /// Gets or sets if this light cast shadows on the scene
        /// </summary>
        public bool CastShadows
		{
			get { return castShadows; }
			set { castShadows = value; }
		}

		/// <summary>
        /// Gets or sets the <see cref="FarAttenuationEnd"/>, that is the attenuation radius of the light
		/// </summary>
		/// <value></value>
		public float Radius
		{
			get { return farAttenEnd; }
			set { farAttenEnd = value; }
		}

		#endregion

        #region Override

        /// <summary>
        /// Compute the world and hierarchical bounds
        /// </summary>
        internal override void ComputeBounds()
        {
            // Use the sphere radius
            boundingSphere.Radius = Radius;
            boundingSphere.Center = Position;

            hierarchicalBoundingSphere = boundingSphere;

            // The bounding box is created using the bounding sphere

            boundingBox.FromHBoundingSphere(boundingSphere);

            hierarchicalBoundingBox.FromHBoundingSphere(hierarchicalBoundingSphere);
        }

        /// <summary>
        /// Computes the visibility of the light
        /// </summary>
        /// <param name="hasChilds">TODO</param>
        /// <returns>True if the child nodes must be checked</returns>
        internal override bool ComputeVisibility(bool hasChilds)
        {
			if (!CheckVisibility)
				return !hide;

            bool childVisible;

            // If it doesnt have children and its hidden, dont check the visibility
        	if (!hasChilds && hide)
        		visible = childVisible = false;
        	
        	else
        	{
        		// Test if the light sphere is visible
        		// If its not visible, theres no need to check the ones that are below in the hierarchy
        		
        		visible = Jad.Scene.Camera.Frustum.BSphereInside(Position, Radius);

        		childVisible = visible;

        		// If the user hides the light, it wont be visible, but it could be possible
        		// for it children to be visible
        		if (hide)
        			visible = false;
        	}

        	return childVisible;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Called from the EffectBuilder to set the value for the hash code
        /// </summary>
        /// <param name="u">Referenced JUniqueEffectID values that will be updated</param>
		internal virtual void BuildHashCode(ref JUniqueEffectID u)
		{
			// HalfVector: ???
			u.LightType = (byte)type;
            u.Attenuation = (byte)AttenuationType;

			if (Volumetric.Enabled)
				u.Global |= (int)JUniqueEffectIDGlobal.VolumetricLight;
			
			if (Projector.Enabled) u.Global |= (int)JUniqueEffectIDGlobal.ProjectorEnabled;
		}

		/// <summary>
		/// Apply the light on the effect
		/// </summary>
		public void ApplyEffect(JEffect effect)
		{
			JColor color;

			// If the position light has not being initialized in the shader, return
			if (effect.handlers.lightHandler.Position == null)
				return;

			effect.SetVector4(effect.handlers.lightHandler.Position, Position.X, Position.Y, Position.Z, Volumetric.PowValue);
			effect.SetVector4(effect.handlers.lightHandler.Direction, Look);

			if (effect.handlers.lightHandler.Color != null)
			{
				color = ShaderColor;

				effect.DXResource.SetValue(effect.handlers.lightHandler.Color, color.ToVector4());
			}

			if (effect.handlers.textureProjector != null)
			{
				// Set the projector texture
                if (Projector.Map != null)
                {
                    effect.DXResource.SetValue(effect.handlers.textureProjector, Projector.Map.DXResource);

                    JStateManager sm = Jad.Video.StateManager;

                    int stage = (int)JVideo.JSamplerStages.Projector;

                    sm.SetSamplerState(stage, SamplerStageStates.AddressU, (int)TextureAddress.Wrap);
                    sm.SetSamplerState(stage, SamplerStageStates.AddressV, (int)TextureAddress.Wrap);
                    sm.SetSamplerState(stage, SamplerStageStates.BorderColor, System.Drawing.Color.Black.ToArgb());
                    sm.SetSamplerState(stage, SamplerStageStates.MagFilter, (int)TextureFilter.Linear);
                    sm.SetSamplerState(stage, SamplerStageStates.MinFilter, (int)TextureFilter.Linear);
                    sm.SetSamplerState(stage, SamplerStageStates.MipFilter, (int)TextureFilter.Linear);
                }
			}

			if (effect.handlers.textureShadow != null)
				effect.DXResource.SetValue(effect.handlers.textureShadow, Jad.Video.RenderTargets.ShadowMask.Texture.DXResource);

			float hotSpot = 0f, fallOff = 0f;

			if (Type == JLightType.Spot || Type == JLightType.Directional)
			{
				JSpotLight l = (JSpotLight)this;

				hotSpot = (float)Math.Cos(Geometry.DegreeToRadian(l.HotSpot));
				fallOff = (float)Math.Cos(Geometry.DegreeToRadian(l.FallOff));
			}

			if (effect.handlers.lightHandler.FarAttenStart_End_hotSpot_FallOff != null)
				effect.SetVector4(effect.handlers.lightHandler.FarAttenStart_End_hotSpot_FallOff, FarAttenuationStart, FarAttenuationEnd, hotSpot, fallOff);

			if (effect.handlers.lightHandler.ProjectorTransformation != null)
				effect.DXResource.SetValue(effect.handlers.lightHandler.ProjectorTransformation, Projector.Matrix);

			if (effect.handlers.lightHandler.ShadowMapTransformation != null)
			{
				if (Type == JLightType.Spot)
				{
					JSpotLight spot = (JSpotLight)this;
					effect.DXResource.SetValue(effect.handlers.lightHandler.ShadowMapTransformation, spot.shadowMapTransformation);
				}
			}

			// NEW HalfVector: Set light shaft effect constants

			if (Type == JLightType.Spot)
			{
				JSpotLight spot = (JSpotLight)this;
				if (spot != null && spot.Shafts.Enabled)
					effect.SetLightShaftsConstants(spot.Shafts);
			}
		}

        /// <summary>
        /// Updates the light. Called each frame by the engine
        /// </summary>
        protected override void Update()
        {
            base.Update();
        }

		#endregion
	}

    /// <summary>
    /// This is a light that emits from the point in all directions.
    /// Its like a sphere, where the radius is the farAttenuationEnd value
    /// </summary>
	public class JOmniLight : JBaseLight
	{
		#region Constructors

		/// <summary>
		/// The default constructor
		/// </summary>
		/// <param name="name">The name of the light</param>
		public JOmniLight(string name)
			: base(name)
		{
			// Light type
			type = JLightType.Omni;

			// Gizmos for this node
			Gizmo = JGizmos.OmniGizmo;
		}

		#endregion
    	
		#region Properties

		/// <summary>
		/// Gets the name used on the <see cref="JEffectBuilder"/>
		/// </summary>
		internal override string NameInShaders
		{
			get { return "LIGHT_OMNI"; }
		}

		#endregion

		#region Methods

        /// <summary>
        /// Updates the light. Called each frame by the engine
        /// </summary>
        protected override void Update()
        {
        	if (Projector.Enabled)
        		Projector.Matrix = Matrix.Invert(WorldTM);

        	base.Update();
        }

		#endregion
	}

    /// <summary>
    /// A spot light is a light targeting a point, with an inner and an outer cone
    /// </summary>
	public class JSpotLight : JBaseLight
	{
		#region Structs

        /// <summary>
        /// The struct that encapsulates the shafts effects
        /// The shafts use the FarAttenuationEnd property of the light to create
        /// the effect, so the NumPlanes are fitted between the position and the
        /// FarAttenuationEnd
        /// </summary>
		/// <remarks>Be careful with this value</remarks>
		public struct JShafts
		{
			#region Fields

			/// <summary>
			/// Enable / Disable shafts
			/// </summary>
            public bool Enabled;

            /// <summary>
            /// Number of planes used
            /// </summary>
            public int NumPlanes;

			/// <summary>
			/// TODO
			/// </summary>
			public float Power;
			
        	/// <summary>
        	/// TODO
        	/// </summary>
			public float Intensity;

            /// <summary>
            /// Texture used for shafts effect
            /// </summary>
            private JTexture cookieTexture;
			
        	/// <summary>
        	/// TODO
        	/// </summary>
			public bool UseCookie;

			/// <summary>
			/// TODO
			/// </summary>
			public bool ProjectShadow;

			/// <summary>Software or hardware clipping</summary>
			public bool SoftwareClipping;

			#endregion

			#region Properties

			/// <summary>
			/// Gets or sets the texture used for the shafts effect
			/// </summary>
			public JTexture CookieTexture
			{
				get { return cookieTexture; }
				set
				{
					cookieTexture = value;
					UseCookie = (value != null);
				}
			}

			#endregion
		}

		#endregion

		#region Fields

    	/// <summary>
    	/// TODO
    	/// </summary>
		protected float hotSpot;
		
    	/// <summary>
    	/// TODO
    	/// </summary>
		protected float fallOff;
		
    	/// <summary>
		/// TODO
		/// </summary>
		internal Matrix projectorPerspective;
		
    	/// <summary>
		/// TODO
		/// </summary>
        internal Matrix shadowMapTransformation;
		
    	/// <summary>
		/// TODO
		/// </summary>
		protected float aspect;
		
    	/// <summary>
		/// TODO
		/// </summary>
		protected JFrustum frustum;

		/// <summary>
		/// TODO
		/// </summary>
    	public JShafts Shafts;
    	
		#endregion

		#region Constructors

		/// <summary>
		/// The default constructor
		/// </summary>
		/// <param name="name">The name of the light</param>
		public JSpotLight(string name)
			: base(name)
		{
			// Light type
			type = JLightType.Spot;

			// Gizmos for this node
			Gizmo = JGizmos.SpotLightGizmo;

			// Default HotSpot
			hotSpot = 43.0f;
			// Default FallOff
			fallOff = 45.0f;

			// Default aspect ratio
			aspect = 1.0f;

			// Frustum
			frustum = new JFrustum(this);
			BuildProjectorMatrix();

			// Shafts arent rendered by default
			Shafts.Enabled = false;
			Shafts.NumPlanes = 30;
			Shafts.Power = 0.8f;
			Shafts.Intensity = 0.4f;
			Shafts.SoftwareClipping = false;
			Shafts.ProjectShadow = false;
			Shafts.UseCookie = false;
		}

		#endregion
    	
		#region Properties

        /// <summary>
        /// Gets the name used on the <see cref="JEffectBuilder"/>
        /// </summary>
		internal override string NameInShaders
		{
			get { return "LIGHT_SPOT"; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public float HotSpot
		{
			get { return hotSpot; }
			set { hotSpot = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public float FallOff
		{
			get { return fallOff; }
			set { fallOff = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		protected float Aspect
		{
			get { return aspect; }
			set { aspect = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public JFrustum Frustum
		{
			get { return frustum; }
			set { frustum = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
        protected override void Update()
        {
			BuildProjectorMatrix();

            if(Jad.Scene.Shadows.ShadowMode==JShadowMode.ShadowMap)
			    BuildShadowMapMatrix();

            base.Update();
        }

		/// <summary>
		/// TODO
		/// </summary>
		public virtual void BuildProjectorMatrix()
		{
			float w = 1.0f, h = 1.0f;

			if (Projector.Map != null)
			{
                w = Projector.Map.SurfaceDes.Width;
                h = Projector.Map.SurfaceDes.Height;
			}

			// The Falloff of the light is half the emission cone angle, so the real FOV is the double
			projectorPerspective = Matrix.PerspectiveFovLH(Geometry.DegreeToRadian(fallOff * 2.0f), w / h, 0.25f, farAttenEnd);

			Matrix offset = Matrix.Identity;

			offset.M11 = 0.5f;
			offset.M22 = -0.5f;
			offset.M41 = 0.5f;
			offset.M42 = 0.5f;

			Projector.Matrix = Matrix.Invert(WorldTM) * projectorPerspective * offset;

			// Update the frustum
			frustum.UpdateFromSpotLight(this);
		}

		/// <summary>
		/// TODO
		/// </summary>
		public virtual void BuildShadowMapMatrix()
		{
			float w = Jad.Video.Width;
			float h = Jad.Video.Height;

			// The Falloff of the light is half the emission cone angle, so the real FOV is the double
			Matrix perspective = Matrix.PerspectiveFovLH(Geometry.DegreeToRadian(fallOff * 2.0f), w / h, Jad.Scene.Camera.NearPlane, farAttenEnd);

			Matrix offset = Matrix.Identity;

			offset.M11 = 0.5f;
			offset.M22 = -0.5f;
			offset.M41 = 0.5f;
			offset.M42 = 0.5f;

			shadowMapTransformation = Matrix.Invert(WorldTM) * perspective * offset;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="min">TODO</param>
		/// <param name="max">TODO</param>
		public void CalculateBounds(out Vector3 min, out Vector3 max)
		{
			min = new Vector3(float.MaxValue, float.MaxValue, float.MaxValue);
			max = new Vector3(float.MinValue, float.MinValue, float.MinValue);

			Matrix toWorldSpace = Matrix.Invert(projectorPerspective) * WorldTM;

			Vector3[] wsVerts = new Vector3[8];

			for (int i = 0; i < 8; i++)
			{
				wsVerts[i] = Vector3.TransformCoordinate(Jad.Scene.Lights.FrustumVerts[i], toWorldSpace);

				if (wsVerts[i].X < min.X)
					min.X = wsVerts[i].X;
				if (wsVerts[i].Y < min.Y)
					min.Y = wsVerts[i].Y;
				if (wsVerts[i].Z < min.Z)
					min.Z = wsVerts[i].Z;

				if (wsVerts[i].X > max.X)
					max.X = wsVerts[i].X;
				if (wsVerts[i].Y > max.Y)
					max.Y = wsVerts[i].Y;
				if (wsVerts[i].Z > max.Z)
					max.Z = wsVerts[i].Z;
			}
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="min">TODO</param>
		/// <param name="max">TODO</param>
		public void CalculateBoundsInViewSpace(out Vector3 min, out Vector3 max)
		{
			min = new Vector3(float.MaxValue, float.MaxValue, float.MaxValue);
			max = new Vector3(float.MinValue, float.MinValue, float.MinValue);

			Matrix toViewSpace = Matrix.Invert(projectorPerspective) * WorldTM * Jad.Scene.Camera.View;

			Vector3[] vsVerts = new Vector3[8];

			for (int i = 0; i < 8; i++)
			{
				vsVerts[i] = Vector3.TransformCoordinate(Jad.Scene.Lights.FrustumVerts[i], toViewSpace);

				if (vsVerts[i].X < min.X)
					min.X = vsVerts[i].X;
				if (vsVerts[i].Y < min.Y)
					min.Y = vsVerts[i].Y;
				if (vsVerts[i].Z < min.Z)
					min.Z = vsVerts[i].Z;

				if (vsVerts[i].X > max.X)
					max.X = vsVerts[i].X;
				if (vsVerts[i].Y > max.Y)
					max.Y = vsVerts[i].Y;
				if (vsVerts[i].Z > max.Z)
					max.Z = vsVerts[i].Z;
			}
		}
    	
		/// <summary>
		/// TODO
		/// </summary>
		public void RenderShafts()
		{
			if (!Shafts.Enabled)
				return;

			List<short> indices = new List<short>();
			List<CustomVertex.PositionOnly> vertices = new List<CustomVertex.PositionOnly>();

			JEffect fx = Jad.Video.Effects.InHouse.Shafts;

			// Reference to the active camera
			JCamera camera = Jad.Scene.Camera;

			// Calculate light frustum bounding box in view-space
			Vector3 min, max;
			CalculateBoundsInViewSpace(out min, out max);

			// Bounding box extensions
			Vector3 dims = max - min;

			// Bounding box mids extensions
			Vector3 halfDims = dims * 0.5f;

			// Point where the sampling planes will start
			Vector3 start = new Vector3(min.X + halfDims.X, min.Y + halfDims.Y, min.Z);
			
			// Transform to world space
			start.TransformCoordinate(Matrix.Invert(camera.View));

			// Up and right vectors that will be used to create the sampling planes
			Vector3 up = camera.Up * halfDims.Y;
			Vector3 right = camera.Right * halfDims.X;

			// The vector along the sampling planes will be created
			Vector3 look = camera.Look;

			// Z spacing between sampling planes
			float dZ = dims.Z / Shafts.NumPlanes;

			for (int i = 0; i < Shafts.NumPlanes; i++)
			{
				Vector3 pos = start + (dZ * i) * look;

				Vector3 v0 = -right - up + pos;
				Vector3 v1 = -right + up + pos;
				Vector3 v2 = right + up + pos;
				Vector3 v3 = right - up + pos;

				Vector3[] quad = new Vector3[] { v0, v1, v2, v3 };

				if (Shafts.SoftwareClipping)
					quad = frustum.ClipToFrustum(quad);

				if (quad != null)
				{
					// Vertices definition
					short numVerts = (short)vertices.Count;
					
					foreach (Vector3 v in quad)
						vertices.Add(new CustomVertex.PositionOnly(v));

					// Indices definition
					short numIndices = (short)((quad.Length - 2) * 3);

					short i0 = 0, i1 = 1, i2 = 2;
					for (short j = 0; j < numIndices / 3; j++)
					{
						if (j > 0)
							i1 = i2++;

						indices.Add((short)(i0 + numVerts));
						indices.Add((short)(i1 + numVerts));
						indices.Add((short)(i2 + numVerts));
					}
				}
			}

			if (vertices.Count == 0)
				return;

			ApplyEffect(fx);

			fx.DXResource.SetValue(fx.handlers.worldViewProjection, Jad.Scene.Camera.ViewProjection);
			fx.DXResource.SetValue(fx.handlers.world, Matrix.Identity);

			if (Shafts.UseCookie)
				fx.DXResource.SetValue(fx.handlers.textureProjector, Shafts.CookieTexture.DXResource);

			fx.DXResource.SetValue(fx.handlers.textureEmissive, Jad.Video.Textures.InHouse.Noise.DXResource);

			// Configure the FX
			if (Jad.Scene.Shadows.ShadowMode != JShadowMode.ShadowMap)
			{
				if (!Shafts.UseCookie)
					fx.DXResource.Technique = fx.handlers.renderScene;
				
				else
					fx.DXResource.Technique = fx.DXResource.GetTechnique("RenderScene_CookieAndNoShadow");
			}
			
			else
			{
				if (Shafts.UseCookie && Shafts.ProjectShadow)
				{
					EffectHandle h = fx.DXResource.GetParameter(null, "texturaShadowMap");
					fx.DXResource.SetValue(h, Jad.Video.RenderTargets.ShadowMap.Texture.DXResource);
					fx.DXResource.Technique = fx.DXResource.GetTechnique("RenderScene_CookieAndShadow");
				}
				
				else if (Shafts.UseCookie && !Shafts.ProjectShadow)
				{
					fx.DXResource.Technique = fx.DXResource.GetTechnique("RenderScene_CookieAndNoShadow");
				}
				
				else if (!Shafts.UseCookie && !Shafts.ProjectShadow)
				{
					fx.DXResource.Technique = fx.DXResource.GetTechnique("RenderScene_NoCookieAndNoShadow");
				}
			}

			fx.Begin();

			fx.BeginPass(0);
			fx.Commit();

			if (!Shafts.SoftwareClipping)
				frustum.EnableDeviceClipPlanes();

			//Jad.Video.Device.SetRenderState(RenderStates.FillMode, (int)FillMode.WireFrame);

			Jad.Video.Device.VertexFormat = CustomVertex.PositionOnly.Format;
			Jad.Video.Device.DrawIndexedUserPrimitives(PrimitiveType.TriangleList, 0, vertices.Count, indices.Count / 3, indices.ToArray(), true, vertices.ToArray());

			//Jad.Video.Device.SetRenderState(RenderStates.FillMode, (int)FillMode.Solid);

			if (!Shafts.SoftwareClipping)
				frustum.DisableDeviceClipPlanes();

			fx.EndPass();
			fx.End();
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="cameraName">TODO</param>
		/// <returns>TODO</returns>
		public JCamera CreateCamera(string cameraName)
		{
			JCamera camera = new JCamera(cameraName, Geometry.DegreeToRadian(fallOff * 2.0f), Jad.Scene.Camera.NearPlane, farAttenEnd);

            CopyTransformValues(camera);

			return camera;
		}

		#endregion
	}

    /// <summary>
    /// A direct light is a light with no position, only direction. Used for lights that are far away, like the sun
    /// </summary>
	public class JDirectLight : JSpotLight
	{
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="name">The name of the light</param>
		public JDirectLight(string name)
			: base(name)
		{
			// Light type
			type = JLightType.Directional;

			// Gizmos for this node
			Gizmo = JGizmos.SpotLightGizmo;

			// Default HotSpot
			//hotSpot = 85.0f;
			
			// Default FallOff
			//fallOff = 90.0f;
		}

		#endregion
    	
		#region Properties

        /// <summary>
        /// Gets the name used on the <see cref="JEffectBuilder"/>
        /// </summary>
        internal override string NameInShaders
		{
			get { return "LIGHT_DIRECT"; }
		}

		#endregion

		#region Overrides Node

        /// <summary>
        /// Computes the visibility of the light
        /// </summary>
        /// <returns>true if we must check the child nodes</returns>
        internal override bool ComputeVisibility(bool hasChilds)
        {
			visible = CheckVisibility && !hide;

            return true;
        }

        #endregion
	}

    /// <summary>
    /// Class that sorts lights by name
    /// </summary>
    public class JSortLightsByName : IComparer<JBaseLight>
    {
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="a">TODO</param>
		/// <param name="b">TODO</param>
		/// <returns>TODO</returns>
        public int Compare(JBaseLight a, JBaseLight b)
        {
            if (a.Name.CompareTo(b.Name) < 0)
                return -1;
			
            if (a.Name == b.Name)
                return 0;

            return 1;
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="x">TODO</param>
		/// <param name="y">TODO</param>
		/// <returns>TODO</returns>
        public bool Equals(JBaseLight x, JBaseLight y)
        {
            throw new NotImplementedException();
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="obj">TODO</param>
		/// <returns>TODO</returns>
        public int GetHashCode(JBaseLight obj)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// This class mantains all the lights on the scene
    /// </summary>
    public class JLights : JListUnique<JBaseLight>
    {
        #region Structs

    	/// <summary>
    	/// TODO
    	/// </summary>
        public struct JShafts
		{
			#region Fields
    		
			/// <summary>
            /// Process a blur after the shafts
            /// </summary>
            public bool ApplyBlur;

            /// <summary>
            /// Marks the stencil so only 50% pixels are drawn
            /// </summary>
            public bool UseStencil;

            /// <summary>
            /// Postproduction filter for soft shadows
            /// </summary>
            public JPostProduction PostProduction;

    		/// <summary>
    		/// TODO
    		/// </summary>
            public JPostFilterGaussianBlur BlurH;

    		/// <summary>
    		/// TODO
    		/// </summary>
    		public JPostFilterGaussianBlur BlurV;

			#endregion

			#region Methods
    		
			/// <summary>
    		/// Sets the default options
    		/// </summary>
            public void Initialize()
            {
                // If its inizialized, exit
                if (PostProduction != null) return;

                // Create the filters to make the blur in the soft shadows
                PostProduction = new JPostProduction();

                // Create the gaussian blur filter
                JGaussianProperties[] props = new JGaussianProperties[1];
				
                // This values are very dependant on the number of samples
                // If the amp is very low, its possible that the total weight is < 1
                // and thenn all the scene will be darker

                int diameter;
				
                // Values for 4
                diameter = 4;
                props[0].RadiusScale = 0.11f;
                props[0].Amp = 0.27f;

                // Values for 16
                //diameter = 16;
                //props[0].radius_scale = 0.11f;
                //props[0].amp = 0.1f;

                JPostProductionEffect post = new JPostProductionEffect("LightShafts");

                // -----------------------------------------------------------------
                // We want to "protect" the active render target, because it will be
                // used at the end of the effect
                // -----------------------------------------------------------------

                post.Add(new JPostFilterProtectPrimary(null, false));

                JGaussianBlur gaussianBlur = new JGaussianBlur(diameter, props, false);
                gaussianBlur.GenerateKernel();

                JPostProductionRT postRT = new JPostProductionRT(0, 1f);

                postRT.FixedRenderTarget = Jad.Video.RenderTargets.ShadowMask2;

                // Set an horizontal blur filter
                // Set a different RT that will act as texture

                BlurH = new JPostFilterGaussianBlur(postRT, gaussianBlur, JBlurType.Horizontal);

                post.Add(BlurH);

				// Set a vertical blur filter
                postRT = new JPostProductionRT(1, 1f);

                // The result will be saved in this RT
                postRT.FixedRenderTarget = Jad.Video.RenderTargets.ShadowMask;

                BlurV = new JPostFilterGaussianBlur(postRT, gaussianBlur, JBlurType.Vertical);

                post.Add(BlurV);

                // -----------------------------------------------------------------
                // Now, we want to blend the original RT, (locked by the JPostFilterProtectPrimary)
                // with the blurred
                // -----------------------------------------------------------------

                postRT = new JPostProductionRT(2, 1f);
                post.Add(new JPostFilterBlend(postRT));

                PostProduction.Add(post);

                PostProduction.MakeARenderTargetEnd = false;

                // Set the value to 1 so there is no attenuation
                BlurH.GlowAttenuation = BlurV.GlowAttenuation = 1f;
			}

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
    	struct CoronaVertex
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 vtxCoord;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector2 texCoord0;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector2 axes;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector4 color;

			#endregion
		}
    	
        #endregion

		#region Constants

    	/// <summary>
    	/// TODO
    	/// </summary>
		const float coronaProbeSize = 0.3f;

		/// <summary>
		/// Glow max size
		/// </summary>
		const float maxSize = 1.0f;
    	
    	/// <summary>
    	/// Glow min size
    	/// </summary>
		const float minSize = 0.25f;

		/// <summary>
		/// TODO
		/// </summary>
		const float minDistSq = 0.7f * 0.7f;
		
    	/// <summary>
    	/// TODO
    	/// </summary>
    	const float maxDistSq = 4.0f * 4.0f;
    	
		#endregion
    	
		#region Fields

		/// <summary>
		/// Encapsulates the properties of the light shafts
		/// </summary>
		public JShafts Shafts;

		/// <summary>
		/// Intensity that will be added to all the lights
		/// </summary>
		public float Intensity;

		/// <summary>
		/// Ambient light used for the ambient pass
		/// </summary>
		public JColor Ambient;

		/// <summary>
		/// Enable / Disable the lights
		/// </summary>
		public bool Enabled = true;

		/// <summary>
		/// Default values for the shafts constructor
		/// </summary>
		internal Vector3[] FrustumVerts;

    	/// <summary>
    	/// TODO
    	/// </summary>
		short[] coronaIndices;

		/// <summary>
		/// TODO
		/// </summary>
		CoronaVertex[] coronaVertices;

		/// <summary>
		/// TODO
		/// </summary>
		VertexDeclaration coronaVertexDecl;

		/// <summary>
		/// TODO
		/// </summary>
		JEffect coronaEffect;

		/// <summary>
		/// TODO
		/// </summary>
		CoronaVertex[] coronaProbeVertices;
    	
		#endregion
    	
		#region Constructors

		/// <summary>
		/// Default Constructor
		/// </summary>
		public JLights()
		{
			// Set the ambient light
			Ambient = new JColor(0.22f, 0.22f, 0.22f, 1f);

			// Set this values to use them in the shafts
			FrustumVerts = new Vector3[8];

			FrustumVerts[0] = new Vector3(1.0f, 1.0f, 1.0f);
			FrustumVerts[1] = new Vector3(1.0f, 1.0f, -1.0f);
			FrustumVerts[2] = new Vector3(-1.0f, 1.0f, -1.0f);
			FrustumVerts[3] = new Vector3(-1.0f, 1.0f, 1.0f);

			FrustumVerts[4] = new Vector3(-1.0f, -1.0f, -1.0f);
			FrustumVerts[5] = new Vector3(-1.0f, -1.0f, 1.0f);
			FrustumVerts[6] = new Vector3(1.0f, -1.0f, 1.0f);
			FrustumVerts[7] = new Vector3(1.0f, -1.0f, -1.0f);

			Shafts.ApplyBlur = true;
		}

		#endregion
    	
		#region Methods

        /// <summary>
        /// Create a new light and add it to the scene
        /// </summary>
        /// <param name="name">Name of the light</param>
        /// <param name="type">Type of light</param>
        /// <returns>The light created</returns>
		public JBaseLight Create(string name, JLightType type)
		{
			JBaseLight light;

			switch (type)
			{
				case JLightType.Omni:
				{
					// Create a omni light
					light = new JOmniLight(name);
					break;
				}

				case JLightType.Spot:
				{
					// Create a spot light
					light = new JSpotLight(name);
					break;
				}

				case JLightType.Directional:
				{
					// Create a directional light
					light = new JDirectLight(name);
					break;
				}

				default:
					light = null;
					break;
			}

			// Initializethe data for the corona of the light
			light.Corona.Init();

			Add(light);

			// Add the light to the scene graph
			Jad.Scene.Root.AddChild(light);

			return light;
		}

        /// <summary>
        /// Render the shafts effect if enabled
        /// </summary>
		internal void RenderShafts()
		{
			bool processAny = false;

			foreach (JBaseLight light in listData)
			{
				if (light.Visible && !light.Hide && light.Type == JLightType.Spot)
				{
					JSpotLight spot = (JSpotLight)light;

					if (spot.Shafts.Enabled)
					{
						processAny = true;
						break;
					}
				}
			}

			if (processAny)
			{
				Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);
				Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, true);
				Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.SourceAlpha);
				Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.One);

				JRenderTarget lastRT = Jad.Video.RenderTargets.ActiveRT;
				JRenderTarget activeRT = null;

				Shafts.Initialize();

				if (Shafts.ApplyBlur)
				{
					// Search a free RT

					if (lastRT == Jad.Video.RenderTargets.Primary)
						activeRT = Jad.Video.RenderTargets.Secondary;

					else
						activeRT = Jad.Video.RenderTargets.Primary;

					activeRT.Begin();

					Jad.Video.Render.Clear(ClearFlags.Target, JColor.Black, 1, 0);
				}

				if (Shafts.UseStencil)
					Jad.Video.Render.MakeStencil50Percent();

				foreach (JBaseLight light in listData)
				{
					if (light.Visible && !light.Hide && light.Type == JLightType.Spot)
					{
						JSpotLight spot = (JSpotLight)light;
						spot.RenderShafts();
					}
				}
                
                Jad.Video.StateManager.SetRenderState(RenderStates.StencilEnable, false);
				Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.One);
				Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.Zero);
				Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, true);

				if (Shafts.ApplyBlur)
				{
					Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);

					activeRT.End();

					// Activate again the RT with the scene render

					// Process the RT

					Shafts.PostProduction.Process(activeRT, false);

					Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
				}
			}
		}

        /// <summary>
        /// Check if the ray intersects the light, asuming the light has a sphere of radius 0.25
        /// </summary>
        /// <param name="start">The start of the ray segment</param>
        /// <param name="end">The end of the ray segment</param>
        /// <returns>The light if it intersects or null if not</returns>
		/// <remarks>Used for picking</remarks>
		public JBaseLight Intersect(Vector3 start, Vector3 end)
		{
			JBaseLight collision = null;
			float distance = float.MaxValue;

			// Create an object to test collisions
			JMesh mesh = JMesh.CreateSphere(0.25f, 4, 4);

			for (int j = 0; j < Count; j++)
			{
				JBaseLight m = this[j];

				// Transform the vectors to object space
				Vector3 startOS = start, endOS = end;

				Matrix toObjectSpace = Matrix.Invert(m.WorldTM);

				startOS.TransformCoordinate(toObjectSpace);
				endOS.TransformCoordinate(toObjectSpace);

				float dist;

				Vector3 rayDir = Vector3.Normalize(endOS - startOS);

				if (mesh.Intersect(startOS, rayDir, false, out dist))
				{
					// Get the closest one
					if (dist < distance)
					{
						collision = m;
						distance = dist;
					}
				}
			}

			Jad.Video.Meshes.Remove(mesh);

			return collision;
		}

        /// <summary>
        /// Sorts the lights by name
        /// </summary>
        public void SortByName()
        {
            listData.Sort(new JSortLightsByName());
        }

        #endregion

		#region Coronas

    	/// <summary>
    	/// TODO
    	/// </summary>
		public void InitCoronas()
		{
			VertexElement[] vElements = new VertexElement[5];
			vElements[0] = new VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0);
			vElements[1] = new VertexElement(0, 12, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0);
			vElements[2] = new VertexElement(0, 20, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 1);
			vElements[3] = new VertexElement(0, 28, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.Color, 0);
			vElements[4] = VertexElement.VertexDeclarationEnd;

			coronaVertexDecl = new VertexDeclaration(Jad.Video.Device, vElements);

			coronaProbeVertices = new CoronaVertex[4];
			coronaProbeVertices[0] = new CoronaVertex();
			coronaProbeVertices[0].vtxCoord = Vector3.Empty;
			coronaProbeVertices[0].texCoord0 = new Vector2(0.0f, 0.0f);
			coronaProbeVertices[0].axes = new Vector2(-1.0f, 1.0f) * coronaProbeSize;
			coronaProbeVertices[0].color = Vector4.Empty;

			coronaProbeVertices[1] = new CoronaVertex();
			coronaProbeVertices[1].vtxCoord = Vector3.Empty;
			coronaProbeVertices[1].texCoord0 = new Vector2(1.0f, 0.0f);
			coronaProbeVertices[1].axes = new Vector2(1.0f, 1.0f) * coronaProbeSize;
			coronaProbeVertices[1].color = Vector4.Empty;

			coronaProbeVertices[2] = new CoronaVertex();
			coronaProbeVertices[2].vtxCoord = Vector3.Empty;
			coronaProbeVertices[2].texCoord0 = new Vector2(0.0f, 1.0f);
			coronaProbeVertices[2].axes = new Vector2(-1.0f, -1.0f) * coronaProbeSize;
			coronaProbeVertices[2].color = Vector4.Empty;

			coronaProbeVertices[3] = new CoronaVertex();
			coronaProbeVertices[3].vtxCoord = Vector3.Empty;
			coronaProbeVertices[3].texCoord0 = new Vector2(1.0f, 1.0f);
			coronaProbeVertices[3].axes = new Vector2(1.0f, -1.0f) * coronaProbeSize;
			coronaProbeVertices[3].color = Vector4.Empty;

			coronaIndices = new short[6];
			coronaVertices = new CoronaVertex[4];

			coronaVertices[0] = new CoronaVertex();
			coronaVertices[0].vtxCoord = Vector3.Empty;
			coronaVertices[0].texCoord0 = new Vector2(0.0f, 0.0f);
			coronaVertices[0].axes = new Vector2(-1.0f, 1.0f);
			coronaVertices[0].color = Vector4.Empty;

			coronaVertices[1] = new CoronaVertex();
			coronaVertices[1].vtxCoord = Vector3.Empty;
			coronaVertices[1].texCoord0 = new Vector2(1.0f, 0.0f);
			coronaVertices[1].axes = new Vector2(1.0f, 1.0f);
			coronaVertices[1].color = Vector4.Empty;

			coronaVertices[2] = new CoronaVertex();
			coronaVertices[2].vtxCoord = Vector3.Empty;
			coronaVertices[2].texCoord0 = new Vector2(1.0f, 1.0f);
			coronaVertices[2].axes = new Vector2(1.0f, -1.0f);
			coronaVertices[2].color = Vector4.Empty;

			coronaVertices[3] = new CoronaVertex();
			coronaVertices[3].vtxCoord = Vector3.Empty;
			coronaVertices[3].texCoord0 = new Vector2(0.0f, 1.0f);
			coronaVertices[3].axes = new Vector2(-1.0f, -1.0f);
			coronaVertices[3].color = Vector4.Empty;

			coronaIndices[0] = 0;
			coronaIndices[1] = 1;
			coronaIndices[2] = 3;
			coronaIndices[3] = 1;
			coronaIndices[4] = 2;
			coronaIndices[5] = 3;

			coronaEffect = Jad.Video.Effects.Create("corona", true);
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		public void DestroyCoronas()
		{
			coronaVertexDecl.Dispose();

    		for (int i = 0; i < Count; i++)
    			listData[i].Corona.Destroy();
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		public void RenderCoronas()
		{
			//Jad.Video.Device.SetRenderState(RenderStates.SourceBlend, (int)Blend.SourceAlpha);
			Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.One);
			Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.One);
			Jad.Video.StateManager.SetRenderState(RenderStates.CullMode, (int)Cull.None);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);

			// Configurate the effect
			coronaEffect.DXResource.Technique = coronaEffect.handlers.renderScene;
			coronaEffect.Begin();

			Matrix viewProj = Jad.Scene.Camera.ViewProjection;
			Vector4 right = new Vector4(Jad.Scene.Camera.Right.X, Jad.Scene.Camera.Right.Y, Jad.Scene.Camera.Right.Z, 0.0f);
			Vector4 up = new Vector4(Jad.Scene.Camera.Up.X, Jad.Scene.Camera.Up.Y, Jad.Scene.Camera.Up.Z, 0.0f);

			coronaEffect.DXResource.SetValue("cam_right", right);
			coronaEffect.DXResource.SetValue("cam_up", up);
			coronaEffect.DXResource.SetValue(coronaEffect.handlers.worldViewProjection, viewProj);

			Jad.Video.Device.VertexDeclaration = coronaVertexDecl;

			coronaEffect.DXResource.BeginPass(0);

			for (int i = 0; i < Count; i++)
			{
				JBaseLight light = listData[i];

				if (!light.Corona.Enabled) continue;

				Vector4 lightPos = new Vector4(light.Position.X, light.Position.Y, light.Position.Z, 0.0f);
				coronaEffect.DXResource.SetValue("lightPos", lightPos);

				coronaEffect.DXResource.CommitChanges();

				Vector3 cPos = Jad.Scene.Camera.Position;

				float distSq = (light.Position - cPos).LengthSq();

				float weight = (minDistSq - distSq) / (minDistSq - maxDistSq);
				float coronaSize = (1.0f - weight) * minSize + weight * maxSize;
				coronaSize = Math.Min(maxSize, Math.Max(coronaSize, minSize));

                light.Corona.curPixels = light.Corona.maxPixels = 0;

				// Get the number of pixels from the last frame
                while ((light.Corona.maxPixels = (int)light.Corona.MaxPixelsQuery.GetData(typeof(int), true)) == null)
				{
				}
				
                while ((light.Corona.curPixels = (int)light.Corona.CurPixelsQuery.GetData(typeof(int), true)) == null)
				{
				}

				// Real Pixels

                Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
                Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
                Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.LessEqual);
                Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, 0);

                light.Corona.CurPixelsQuery.Issue(IssueFlags.Begin);
				Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, coronaProbeVertices);
                light.Corona.CurPixelsQuery.Issue(IssueFlags.End);

				// Pixels maximum

                Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.Always);

                light.Corona.MaxPixelsQuery.Issue(IssueFlags.Begin);
				Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, coronaProbeVertices);
                light.Corona.MaxPixelsQuery.Issue(IssueFlags.End);

				// Render

                float alpha = (light.Corona.maxPixels > 0 ? ((float)light.Corona.curPixels / (float)light.Corona.maxPixels) : 0.0f) * light.Multiplier * 2.0f;

				if (alpha > 0.0f)
				{
					Vector4 color = light.Color.ToVector4();

					color.W = alpha < 0.0f ? 0.0f : (alpha > 1.0f ? 1.0f : alpha);

					coronaEffect.DXResource.SetValue("color", color);
					coronaEffect.DXResource.SetValue("size", coronaSize);
                    coronaEffect.DXResource.SetValue(coronaEffect.handlers.textureDiffuse, light.Corona.Texture.DXResource);

					coronaEffect.DXResource.CommitChanges();

					Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.Always);
					Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, true);
					Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, (int)(ColorWriteEnable.RedGreenBlueAlpha));

					Jad.Video.Device.DrawIndexedUserPrimitives(PrimitiveType.TriangleList, 0, 4, 2, coronaIndices, true, coronaVertices);
				}
			}

			coronaEffect.DXResource.EndPass();
			coronaEffect.End();

			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.LessEqual);
			Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
			Jad.Video.StateManager.SetRenderState(RenderStates.CullMode, (int)Cull.CounterClockwise);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, true);
			Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, (int)(ColorWriteEnable.RedGreenBlueAlpha));
		}

        /// <summary>
        /// Called OnReset device
        /// </summary>
        public void Reset()
        {
            JLogWriter.Write("Lights.Reset", true);

            for (int c = 0; c < Count; c++)
            {
                JBaseLight light = this[c];

            	if (light.Corona.MaxPixelsQuery != null)
            		light.Corona.Reset();
            }
        }

        /// <summary>
        /// Called OnLostDevice
        /// </summary>
        internal void OnLost()
        {
            JLogWriter.Write("Lights.OnLost", true);

            for (int c = 0; c < Count; c++)
            {
                JBaseLight light = this[c];

            	if (light.Corona.MaxPixelsQuery != null)
            		light.Corona.OnLost();
            }
        }

        #endregion
	}
}