#region Using directives

using System.Collections.Generic;

#endregion

namespace JadEngine.Tools
{
	/// <summary>
	/// Class added each TimerKey
	/// </summary>
	public class JTimerKeyInternal
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		public float NumberOfSeconds;

		/// <summary>
		/// TODO
		/// </summary>
		public float CurrentTime;

		/// <summary>
		/// TODO
		/// </summary>
		public string Name;

		/// <summary>
		/// TODO
		/// </summary>
		public bool State;

		#endregion
	}

	/// <summary>
	/// This class maintains a list of timerKeys that are timers used to indicate state changes in
	/// a given moment. For example, change state every 5 seconds. Its created and then forgotten
	/// and the only needed thing is to test the State var
	/// </summary>
	public static class JTimerKey
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		static List<JTimerKeyInternal> timerKeys= new List<JTimerKeyInternal>();

		#endregion

		#region Methods
		
		/// <summary>
		/// Add a timer with time
		/// </summary>
		/// <param name="name">Name of the timer, to later search for it</param>
		/// <param name="numberOfSeconds">TODO</param>
		/// <returns></returns>
		public static JTimerKeyInternal Create(string name, float numberOfSeconds)
		{
			JTimerKeyInternal t = new JTimerKeyInternal();
			
			t.NumberOfSeconds = numberOfSeconds;
			t.Name = name;
			t.CurrentTime = 0;
			t.State = false;

			timerKeys.Add(t);

			return t;
		}

		/// <summary>
		/// Returns the state of a timer
		/// </summary>
		/// <param name="name">Name of the timer</param>
		/// <returns>The state of the timer</returns>
		public static bool State(string name)
		{
			return Find(name).State;
		}

		/// <summary>
		/// Searches a timer by its name and returns it
		/// </summary>
		/// <param name="name">Name of the timer to search</param>
		/// <returns>The timer if found, null if not</returns>
		/// <remarks>Lineal search</remarks>
		public static JTimerKeyInternal Find(string name)
		{
			for (int i = 0; i < timerKeys.Count; i++)
				if (timerKeys[i].Name == name)
					return timerKeys[i];

			return null;
		}

		/// <summary>
		/// Updates all the timers, changing the state if needed
		/// </summary>
		public static void Update()
		{
			float i = Jad.At1Second(1f);

			for (int p = 0; p < timerKeys.Count; p++)
			{
				JTimerKeyInternal t = timerKeys[p];
				t.CurrentTime += i;
				if (t.CurrentTime >= t.NumberOfSeconds)
				{
					t.State = !t.State;
					t.CurrentTime = t.CurrentTime % t.NumberOfSeconds;
				}
			}
		}

		#endregion
	}
}
