#region Using directives

using System;
using System.IO;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// This class mantains the effects on the engine. Based on the DX effect resource. Loads
	/// an effect and sets the common handles used for the engine. Has methods to render lines 
    /// and triangles, used commonly to render simple quads
    /// </summary>
    public class JEffect : IJListable, IDisposable
    {
        #region Readonly Fields

        /// <summary>
        /// The extensions list to load an effect
        /// </summary>
        private readonly string[] extensions ={ "", ".fx" };

		#endregion

		#region Structs
    	
		/// <summary>
        /// Struct with the differente parameters used to load an effect
        /// </summary>
        public struct JLoadingParams
		{
			#region Fields
			
			/// <summary>
            /// Errors that appear when loading the effect
            /// </summary>
            public string Errors;
            
			/// <summary>
			/// Flags used to create the effect
			/// </summary>
            public ShaderFlags Flags;
            
			/// <summary>
			/// The effect pool on loading
			/// </summary>
            public EffectPool Pool;
            
			/// <summary>
			/// Defines on loading
			/// </summary>
            public Macro[] Defines;
            
			/// <summary>
			/// Includes on loading
			/// </summary>
            public Include Includes;

			#endregion
		}

		/// <summary>
		/// Common handles used for the engine
		/// </summary>
		public struct JHandles
		{
			#region Structs

			/// <summary>
			/// TODO
			/// </summary>
			public struct JLightHandler
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle FarAttenStart_End_hotSpot_FallOff;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Position;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Color;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Direction;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle ProjectorTransformation;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle ShadowMapTransformation;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JCameraHandler
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Position;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JMaterialHandler
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Specular;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle SpecularPower_Bumpiness_Reflectivity_Transmittance;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Diffuse;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Emissive;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle IndexOfRefraction;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle ParallaxAmount;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JEnvironmentHandler
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Ambient_TimeElapsed;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Screen_Shadows;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JFogHandler
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle FogValues;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle FogColor;

				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JMapHandler
			{
				#region Fields
				
				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Tile;
				
				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			public struct JDOFHandler
			{
				#region Fields

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Planes;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle MaxBlur;
				
				#endregion
			}

			/// <summary>
			/// TODO
			/// </summary>
			/// <remarks>NEW HalfVector</remarks>
			public struct JLightShaftHandler
			{
				#region Fields

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Power;

				/// <summary>
				/// TODO
				/// </summary>
				public EffectHandle Intensity;

				#endregion
			}
			
			#endregion

			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle renderScene;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle world;

			/// <summary>
			/// TODO
			/// </summary>
			/// <remarks>World inverse</remarks>
			public EffectHandle iWorld;
			
			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle worldView;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle worldViewProjection;
			
			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle View;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureDiffuse;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureSpecular;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureNormal;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureProjector;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureShadow;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureReflection;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureRefraction;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureEmissive;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureLightMap;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureBackBuffer;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureParallaxValues;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle textureDetailMask;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle weightKeyFrame;

			/// <summary>
			/// TODO
			/// </summary>
			public EffectHandle animationEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			public JLightHandler lightHandler;

			/// <summary>
			/// TODO
			/// </summary>
			public JCameraHandler cameraHandler;

			/// <summary>
			/// TODO
			/// </summary>
			public JMaterialHandler materialHandler;

			/// <summary>
			/// TODO
			/// </summary>
			public JEnvironmentHandler env;

			/// <summary>
			/// TODO
			/// </summary>
			public JFogHandler fogHandler;

			/// <summary>
			/// TODO
			/// </summary>
			public JMapHandler mapDiffuseHandler;

			/// <summary>
			/// TODO
			/// </summary>
			public JDOFHandler DOFHandler;

			/// <summary>
			/// TODO
			/// </summary>
			public JLightShaftHandler shaftHandler;

			#endregion
		}
    	
        #endregion
    	
        #region Fields

		/// <summary>
		/// Id asigned internaly by the engine to the effect
		/// </summary>
		int id;

		/// <summary>
		/// Name of the effect
		/// </summary>
		string name;
    	
		/// <summary>
		/// DX Resource
		/// </summary>
        public Effect dxResource;

        /// <summary>
        /// Fullpath used for loading the effect
        /// </summary>
        string fullPath;

		/// <summary>
		/// Common handlers used for the engine
		/// </summary>
		public JHandles handlers;
    	
    	/// <summary>
    	/// Helper var to pass parameters
    	/// </summary>    	
        static Vector4 vector4;

		/// <summary>
		/// Helper var to pass parameters
		/// </summary>    	
		static float[] array2 = new float[2];

		/// <summary>
		/// Helper var to pass parameters
		/// </summary>    	
		static float[] array3 = new float[3];

		/// <summary>
		/// Loadign parms used on loading the effect
		/// </summary>
		public JLoadingParams LoadingParams;

		/// <summary>
		/// Track whether Dispose has been called.
		/// </summary>
		private bool disposed = false;
    	
        #endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="name">Name of the effect</param>
		public JEffect(string name)
		{
			/*
			#if(DEBUG_VS)
			shaderFlags |= ShaderFlags.ForceVertexShaderSoftwareNoOptimizations;
			#endif
			#if(DEBUG_PS)
			shaderFlags |= ShaderFlags.ForcePixelShaderSoftwareNoOptimizations;
			#endif
			*/
			
			id = ++JEffects.IDGlobal;
			this.name = name;

			LoadingParams.Flags = ShaderFlags.None;
			LoadingParams.Pool = null;
			LoadingParams.Defines = null;
			LoadingParams.Includes = null;

			// Search in the available paths the name of the effect with the possible extensions
			if (!JPath.FindFullPath(name, JPath.BaseDirectory, "effects", extensions, out fullPath))
				if (!JPath.FindFullPath(name, JPath.InHouseDirectory, "effects", extensions, out fullPath))
					JPath.FindFullPath(name, null, "effects", extensions, out fullPath);
		}

		#endregion
    	
        #region Properties

        /// <summary>
        /// Gets or sets the DX resource
        /// </summary>
        public Effect DXResource
        {
            get { return dxResource; }
            set { dxResource = value; }
        }

        /// <summary>
        /// Gets or sets the fffect name
        /// </summary>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets the fffect ID
        /// </summary>
		public int ID
		{
			get { return id; }
		}

        /// <summary>
        /// Gets the fullpath used to load the effect
        /// </summary>
        public string FullPath
        {
            get { return fullPath; }
        }

        #endregion

        #region Dispose

		/// <summary>
		/// Implement IDisposable
		/// </summary>
		/// <remarks>Do not make this method virtual. A derived class should not be able to override this method</remarks>
        public void Dispose()
        {
            Dispose(true);
            // This object will be cleaned up by the Dispose method.
            // Therefore, you should call GC.SupressFinalize to
            // take this object off the finalization queue 
            // and prevent finalization code for this object
            // from executing a second time.
            GC.SuppressFinalize(this);
        }

		/// <summary>TODO</summary>
		/// <param name="disposing">TODO</param>
		/// <remarks>
		/// Dispose(bool disposing) executes in two distinct scenarios.
		/// If disposing equals true, the method has been called directly
		/// or indirectly by a user's code. Managed and unmanaged resources
		/// can be disposed.
		/// If disposing equals false, the method has been called by the 
		/// runtime from inside the finalizer and you should not reference 
		/// other objects. Only unmanaged resources can be disposed.
		/// </remarks>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    dxResource.StateManager = null;
                    // Dispose managed resources.
                    dxResource.Dispose();
                }

                // Call the appropriate methods to clean up 
                // unmanaged resources here.
                // If disposing is false, 
                // only the following code is executed.
            }
            disposed = true;
        }

        #endregion
    	
        #region Methods

		/// <summary>
        /// Set a float4 constant with the yzw=1, and x parameter
		/// </summary>
		/// <param name="handle">TODO</param>
		/// <param name="x">The x value</param>
		public void SetVector4(EffectHandle handle, float x)
		{
			vector4.X = x;
			vector4.Y = 1f;
			vector4.Z = 1f;
			vector4.W = 1f;
			DXResource.SetValue(handle, vector4);
		}

		/// <summary>
        /// Set a float4 constant with the zw=1, and the x, y parameters
        /// </summary>
        /// <param name="handle">TODO</param>
        /// <param name="x">The x value</param>
		/// <param name="y">The y value</param>
        public void SetVector4(EffectHandle handle, float x, float y)
        {
            vector4.X = x;
            vector4.Y = y;
            vector4.Z = 1f;
            vector4.W = 1f;
            DXResource.SetValue(handle, vector4);
        }

        /// <summary>
        /// Set a float4 constant with the w=1, and the x, y, z parameters
        /// </summary>
        /// <param name="handle">TODO</param>
		/// <param name="x">The x value</param>
		/// <param name="y">The y value</param>
        /// <param name="z">The z value</param>
        public void SetVector4(EffectHandle handle, float x, float y, float z)
        {
            vector4.X = x;
            vector4.Y = y;
            vector4.Z = z;
            vector4.W = 1f;
            DXResource.SetValue(handle, vector4);
        }

        /// <summary>
        /// Set a float4 constant
        /// </summary>
		/// <param name="handle">TODO</param>
		/// <param name="x">The x value</param>
		/// <param name="y">The y value</param>
		/// <param name="z">The z value</param>
		/// <param name="w">The w value</param>
        public void SetVector4(EffectHandle handle, float x, float y, float z, float w)
        {
            vector4.X = x;
            vector4.Y = y;
            vector4.Z = z;
            vector4.W = w;
            DXResource.SetValue(handle, vector4);
        }

        /// <summary>
        /// Set a float4 constant with the w=1, and the x, y, z parameters
        /// </summary>
		/// <param name="handle">TODO</param>
		/// <param name="v">Vector that holds the x, y, z values</param>
		public void SetVector4(EffectHandle handle, Vector3 v)
        {
            vector4.X = v.X;
            vector4.Y = v.Y;
            vector4.Z = v.Z;
            vector4.W = 1f;

            DXResource.SetValue(handle, vector4);
        }

		/// <summary>
		/// Loads an effect and adds the AllInOne if needed
		/// </summary>
		/// <returns>True if all was ok, false if otherwise</returns>
		public bool Load()
		{
			StreamReader stream = new StreamReader(fullPath);

			if (stream == null)
				throw new Exception("Effect not found:" + fullPath);

			string source = stream.ReadToEnd();

			stream.Close();

			LoadingParams.Errors = String.Empty;

			// If it starts with AllInOne, then add the AllInOne shader

			if (source.StartsWith("AllInOne", StringComparison.OrdinalIgnoreCase))
				source = Jad.EffectBuilder.allInOneFXForEffect + "//" + source;

			Effect effect = Effect.FromString(Jad.Video.Device, source, null, null, LoadingParams.Flags, LoadingParams.Pool, out LoadingParams.Errors);

			if (effect == null)
				throw new Exception("Error loading effect:" + fullPath + @"\n" + LoadingParams.Errors);

			dxResource = effect;

			GetHandles();

			return true;
		}

        /// <summary>
        /// Gets the common constants used by the engine to access it quickly.
        /// It also sets the StateManager
        /// </summary>
        public void GetHandles()
        {
            // Techniques

			handlers.renderScene = DXResource.GetTechnique("RenderScene");

            // Transformation

            handlers.worldViewProjection = DXResource.GetParameter(null, "WorldViewProjection");
            handlers.worldView = DXResource.GetParameter(null, "WorldView");
            handlers.world = DXResource.GetParameter(null, "World");
            handlers.iWorld = DXResource.GetParameter(null, "IWorld");
            handlers.View = DXResource.GetParameter(null, "View");

            // Light

			handlers.lightHandler.FarAttenStart_End_hotSpot_FallOff = DXResource.GetParameter(null, "Light_FarAttenStart_End_hotSpot_FallOff");
			handlers.lightHandler.Position = DXResource.GetParameter(null, "Light_Position");
			handlers.lightHandler.Color = DXResource.GetParameter(null, "Light_Color");
			handlers.lightHandler.Direction = DXResource.GetParameter(null, "Light_Direction");
			handlers.lightHandler.ProjectorTransformation = DXResource.GetParameter(null, "Light_ProjectorTransformation");
			handlers.lightHandler.ShadowMapTransformation = DXResource.GetParameter(null, "Light_ShadowMapTransformation");

			// Material

            handlers.materialHandler.Specular = DXResource.GetParameter(null, "Material_Specular");
            handlers.materialHandler.SpecularPower_Bumpiness_Reflectivity_Transmittance = DXResource.GetParameter(null, "Material_SpecularPower_Bumpiness_Reflectivity_Transmittance");
            handlers.materialHandler.Diffuse = DXResource.GetParameter(null, "Material_Diffuse");
            handlers.materialHandler.Emissive = DXResource.GetParameter(null, "Material_Emissive");
			handlers.materialHandler.IndexOfRefraction = DXResource.GetParameter(null, "Material_IOR");
			handlers.materialHandler.ParallaxAmount = DXResource.GetParameter(null, "Material_ParallaxAmount");

			// Environment

			handlers.env.Ambient_TimeElapsed = DXResource.GetParameter(null, "Env_Ambient_TimeElapsed");
			handlers.env.Screen_Shadows = DXResource.GetParameter(null, "Env_Screen_Shadows");

			// Fog

			handlers.fogHandler.FogValues = DXResource.GetParameter(null, "FogValues");
			handlers.fogHandler.FogColor = DXResource.GetParameter(null, "FogColor");

			// Camera

            handlers.cameraHandler.Position = DXResource.GetParameter("camera", "position");

            // Maps

            handlers.mapDiffuseHandler.Tile = DXResource.GetParameter("mapDiffuse", "tile");

            // JTextures

            handlers.textureDiffuse = DXResource.GetParameter(null, "textureDiffuse");
			handlers.textureSpecular = DXResource.GetParameter(null, "textureSpecular");
			handlers.textureNormal = DXResource.GetParameter(null, "textureNormal");
			handlers.textureProjector = DXResource.GetParameter(null, "textureProjector");
            handlers.textureShadow = DXResource.GetParameter(null, "textureShadow");
			handlers.textureReflection = DXResource.GetParameter(null, "textureReflection");
			handlers.textureRefraction = DXResource.GetParameter(null, "textureRefraction");
			handlers.textureEmissive = DXResource.GetParameter(null, "textureEmissive");
            handlers.textureLightMap = DXResource.GetParameter(null, "textureLightMap");
            handlers.textureBackBuffer = DXResource.GetParameter(null, "textureBackBuffer");
			handlers.textureParallaxValues = DXResource.GetParameter(null, "textureParallaxValues");
            handlers.textureDetailMask = DXResource.GetParameter(null, "textureDetailMask");

            // Otros

            handlers.weightKeyFrame = DXResource.GetParameter(null, "pesoKeyFrame");
            handlers.animationEnabled = DXResource.GetParameter(null, "animationEnabled");
            
            // Depth of Field

			handlers.DOFHandler.Planes = DXResource.GetParameter(null, "DOF_planes");
			handlers.DOFHandler.MaxBlur = DXResource.GetParameter(null, "DOF_maxBlur");

			// NEW HalfVector: Light Shafts

			handlers.shaftHandler.Power = DXResource.GetParameter(null, "power");
			handlers.shaftHandler.Intensity = DXResource.GetParameter(null, "intensity");

			// StateManager
            dxResource.StateManager = Jad.Video.Effects.EffectStateManager;
		}

        /// <summary>
        /// Begin the effect, without setting the save state
        /// </summary>
        /// <returns>Number of passes</returns>
		public int Begin()
		{
			// Start the effect without saving the states

			int r = DXResource.Begin(FX.DoNotSaveState);

			SetCommonConstants();

			return r;
		}

        /// <summary>
        /// Ends  the effect
        /// </summary>
        public void End()
        {
            dxResource.End();
        }

        /// <summary>
        /// Begins the pass
        /// </summary>
        /// <param name="pass">Number of pass</param>
        public void BeginPass(int pass)
        {
            dxResource.BeginPass(pass);
        }

        /// <summary>
        /// Process all the changes (<see cref="Effect.CommitChanges"/>)
        /// </summary>
        public void Commit()
        {
            dxResource.CommitChanges();
        }

        /// <summary>
        /// Ends the pass
        /// </summary>
        public void EndPass()
        {
            dxResource.EndPass();
        }

		/// <summary> 
		/// Sets a technique, if not found sets the RenderScene technique
		/// </summary> 
        /// <param name="techniqueName">Name of the technique</param> 
        public void SetTechnique(string techniqueName)
        {
            EffectHandle t = dxResource.GetTechnique(techniqueName);

			if (t != null)
				dxResource.Technique = t;
		
			else
				dxResource.Technique = handlers.renderScene;
        }

		/// <summary>
        /// Sets the common constants for all the shaders (ambient, time, camera...)
        /// </summary>
        public virtual void SetCommonConstants()
        {
            if (handlers.env.Ambient_TimeElapsed != null)
			{
				JColor color=Jad.Scene.Lights.Ambient;
				SetVector4(handlers.env.Ambient_TimeElapsed,color.Red,color.Green,color.Blue,Jad.Timer.Time);
			}

			// Camera
			if (handlers.cameraHandler.Position != null)
				SetVector4(handlers.cameraHandler.Position, Jad.Scene.Camera.Position);

			if (handlers.View != null)
				dxResource.SetValue(handlers.View, Jad.Scene.Camera.View);

			if (handlers.env.Screen_Shadows != null && Jad.Video.RenderTargets.ActiveRT != null)
			{
				if(Jad.Video.RenderTargets.ShadowMap!=null)
					SetVector4(handlers.env.Screen_Shadows, Jad.Video.RenderTargets.ActiveRT.Width, Jad.Video.RenderTargets.ActiveRT.Height, Jad.Video.RenderTargets.ShadowMap.Width, Jad.Video.RenderTargets.ShadowMap.Height);
				
				else
					SetVector4(handlers.env.Screen_Shadows, Jad.Video.RenderTargets.ActiveRT.Width, Jad.Video.RenderTargets.ActiveRT.Height);
			}
			
            if (handlers.textureBackBuffer != null && Jad.Scene.BackBuffer != null)
                dxResource.SetValue(handlers.textureBackBuffer, Jad.Scene.BackBuffer.Texture.DXResource);

			SetDOFConstants();
		}

        /// <summary>
        /// Sets the Depth of field constants
        /// </summary>
        void SetDOFConstants()
        {
			if (handlers.DOFHandler.Planes != null)
			{
				float nearFocalPlaneDepth = Jad.Scene.DepthOfField.NearFocalPlaneDepth;
				float farFocalPlaneDepth = Jad.Scene.DepthOfField.FarFocalPlaneDepth;

				float nearBlurDepth = Jad.Scene.DepthOfField.NearBlurDepth;
				float farBlurDepth = Jad.Scene.DepthOfField.FarBlurDepth;

				nearFocalPlaneDepth = nearFocalPlaneDepth < nearBlurDepth ? nearBlurDepth : nearFocalPlaneDepth;
				farFocalPlaneDepth = farFocalPlaneDepth > farBlurDepth ? farBlurDepth : farFocalPlaneDepth;

				float nearBlurDepthRange = nearFocalPlaneDepth - nearBlurDepth;
				float farBlurDepthRange = farBlurDepth - farFocalPlaneDepth;

				Vector4 dof_planes = new Vector4(nearBlurDepthRange, nearFocalPlaneDepth, farFocalPlaneDepth, farBlurDepthRange);

				dxResource.SetValue(handlers.DOFHandler.Planes, dof_planes);
				dxResource.SetValue(handlers.DOFHandler.MaxBlur, Jad.Scene.DepthOfField.MaxBlur);
			}
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="shaft">TODO</param>
		/// <remarks>NEW HalfVector</remarks>
		public void SetLightShaftsConstants(JSpotLight.JShafts shaft)
		{
			if (handlers.shaftHandler.Power != null)
			{
				dxResource.SetValue(handlers.shaftHandler.Power, shaft.Power);
				dxResource.SetValue(handlers.shaftHandler.Intensity, shaft.Intensity);
			}
		}

        #endregion

        #region Rendering

        /// <summary>
        /// Renders a transformed line list with a color
        /// </summary>
        /// <param name="color">Color of the lines</param>
        /// <param name="vertexCount">Number of vertices</param>
        /// <param name="vertices">Array of vertices</param>
        public void RenderLinesTransformed(JColor color, int vertexCount, Vector3[] vertices)
        {
            // Configure the effect
            DXResource.Technique = handlers.renderScene;
            Begin();

            DXResource.SetValue(handlers.worldViewProjection, Matrix.Identity);
            DXResource.SetValue(handlers.materialHandler.Diffuse, color.ToVector4());

            Jad.Video.Device.VertexFormat = CustomVertex.Transformed.Format;

            DXResource.BeginPass(0);
            DXResource.CommitChanges();

            Jad.Video.Device.DrawUserPrimitives(PrimitiveType.LineList, vertexCount / 2, vertices);

            DXResource.EndPass();

            End();
        }

        /// <summary>
        /// Renders a non transformed line list with a color, using the Identity matrix for world transformation
        /// </summary>
        /// <param name="color">Color of the lines</param>
        /// <param name="vertexCount">Number of vertices</param>
        /// <param name="vertices">Array of vertices</param>
		public void RenderLines(JColor color, int vertexCount, Vector3[] vertices)
		{
			Matrix world = Matrix.Identity;
        	
			RenderLines(ref world, color, vertexCount, vertices);
		}

        /// <summary>
        /// Renders a non transformed line list with a color, setting the world matrix transformation
        /// </summary>
        /// <param name="world">World matrix</param>
        /// <param name="color">Color of the lines</param>
        /// <param name="vertexCount">Number of vertices</param>
        /// <param name="vertices">Array of vertices</param>
        public void RenderLines(ref Matrix world, JColor color,int vertexCount, Vector3[] vertices)
        {
			// Configure the effect
            DXResource.Technique = handlers.renderScene;
            Begin();

            DXResource.SetValue(handlers.worldViewProjection, world * Jad.Scene.Camera.ViewProjection);
            DXResource.SetValue(handlers.materialHandler.Diffuse, color.ToVector4());

            Jad.Video.Device.VertexFormat = CustomVertex.PositionOnly.Format;

            DXResource.BeginPass(0);
            DXResource.CommitChanges();

            Jad.Video.Device.DrawUserPrimitives(PrimitiveType.LineList, vertexCount/2,vertices);

            DXResource.EndPass();

            End();
        }

        /// <summary>
        /// Renders a non transformed indexed line list with a color, setting the world matrix transformation
        /// </summary>
        /// <param name="world">World matrix</param>
        /// <param name="color">Color of the lines</param>
        /// <param name="vertexCount">Number of vertices</param>
        /// <param name="vertices">Array of vertices</param>
        /// <param name="numIndices">Number of indices</param>
        /// <param name="indices">Array of indices</param>
        public void RenderLinesIndexed(ref Matrix world, JColor color, int vertexCount, Vector3[] vertices, int numIndices, object indices)
        {
			// Configure the effect
            DXResource.Technique = handlers.renderScene;
            Begin();

            DXResource.SetValue(handlers.worldViewProjection, world * Jad.Scene.Camera.ViewProjection);
            DXResource.SetValue(handlers.materialHandler.Diffuse, color.ToVector4());

            Jad.Video.Device.VertexFormat = CustomVertex.PositionOnly.Format;

            DXResource.BeginPass(0);
            DXResource.CommitChanges();

            Jad.Video.Device.DrawIndexedUserPrimitives(PrimitiveType.LineList, 0, numIndices, numIndices / 2, indices, true, vertices);

            DXResource.EndPass();

            End();
        }

        /// <summary>
        /// Renders a indexed triangle list with a color, setting the world matrix transformation
        /// </summary>
        /// <param name="world">World matrix</param>
        /// <param name="color">Color of the lines</param>
        /// <param name="vertexCount">Number of vertices</param>
        /// <param name="vertices">Array of vertices</param>
        /// <param name="numIndices">Number of indices</param>
        /// <param name="indices">Array of indices</param>
        public void RenderIndexed(ref Matrix world, JColor color, int vertexCount, Vector3[] vertices, int numIndices, object indices)
        {
			// Configure the effect
            DXResource.Technique = handlers.renderScene;
            Begin();

            DXResource.SetValue(handlers.worldViewProjection, world * Jad.Scene.Camera.ViewProjection);
            DXResource.SetValue(handlers.materialHandler.Diffuse, color.ToVector4());

            Jad.Video.Device.VertexFormat = CustomVertex.PositionOnly.Format;

            DXResource.BeginPass(0);
            DXResource.CommitChanges();

			Jad.Video.Device.DrawIndexedUserPrimitives(PrimitiveType.TriangleList, 0, numIndices, numIndices / 2, indices, true, vertices);

            DXResource.EndPass();

            End();
        }

        #endregion

        #region Reset / Lost

        /// <summary>
        /// Called after a Reset device
        /// </summary>
        internal void Reset()
        {
            dxResource.OnResetDevice();
        }

        /// <summary>
        /// Called on OnLostDevice
        /// </summary>
        internal void OnLost()
        {
            dxResource.OnLostDevice();
        }

        #endregion
    }

    /// <summary>
    /// Mantains the list of effects, and contains the default effects created by the engine
    /// </summary>
    public class JEffects : JList<JEffect>, IDisposable
    {
        #region Structs

        /// <summary>
        /// Default effects loaded by the engine
        /// </summary>
        public struct JInHouse
		{
			#region Structs

			/// <summary>
			/// TODO
			/// </summary>
			public struct JPostProcess
			{
				#region Fields

				/// <summary>
				/// TODO
				/// </summary>
				public JEffect radialBlur;

				/// <summary>
				/// TODO
				/// </summary>
				public JEffect depthOfField;

				/// <summary>
				/// TODO
				/// </summary>
				public JEffect GaussianBlur;

				/// <summary>
				/// Edge detection
				/// </summary>
				public JEffect Sobel;

				/// <summary>
				/// HDR filter
				/// </summary>
				public JEffect HDR;

				/// <summary>
				/// Used in HDR
				/// </summary>
				public JEffect ToneMapping;

				/// <summary>
				/// TODO
				/// </summary>
				public JEffect Blur;

				/// <summary>
				/// TODO
				/// </summary>
				public JEffect HighPass;

				/// <summary>
				/// TODO
				/// </summary>
				public JEffect MotionBlur;

				#endregion
			}

			#endregion
        	
			#region Fields
        	
			/// <summary>
        	/// TODO
        	/// </summary>
            public bool Loaded;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect SkyBox;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect PostProcessEffect;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect PosColor;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect PosColorTex1;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect Shafts;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect ShowZ;

			/// <summary>
			/// TODO
			/// </summary>
			public JEffect ShowFloat;

			/// <summary>
			/// TODO
			/// </summary>
			public JPostProcess PostProcess;

			#endregion
		}
    	
        #endregion

		#region Static Fields

		/// <summary>
		/// TODO
		/// </summary>
		internal static int IDGlobal = 0;

		/// <summary>
		/// TODO
		/// </summary>
		public static bool CatchErrors = true;

		#endregion

        #region Fields

		/// <summary>
		/// The effect state manager to handle the states changes from effects
		/// </summary>
		internal JEffectStateManager EffectStateManager;

		/// <summary>
		/// Default effects created by the Jad team
		/// </summary>
		public JInHouse InHouse;

		/// <summary>
		/// Track whether Dispose has been called
		/// </summary>
		private bool disposed = false;

        #endregion

        #region Constructors

        /// <summary>
        /// Default Constructor
        /// </summary>
        public JEffects()
        {
            InHouse.Loaded = false;
            EffectStateManager = new JEffectStateManager();
        }

        #endregion

        #region Dispose

		/// <summary>
		/// Implement IDisposable
		/// </summary>
		/// <remarks>
		/// Do not make this method virtual.
		/// A derived class should not be able to override this method
		/// </remarks>
        public void Dispose()
        {
            Dispose(true);
            // This object will be cleaned up by the Dispose method.
            // Therefore, you should call GC.SupressFinalize to
            // take this object off the finalization queue 
            // and prevent finalization code for this object
            // from executing a second time.
            GC.SuppressFinalize(this);
        }

		/// <summary>
		/// TODO
		/// </summary>
		/// <remarks>
		/// Dispose(bool disposing) executes in two distinct scenarios.
		/// If disposing equals true, the method has been called directly
		/// or indirectly by a user's code. Managed and unmanaged resources
		/// can be disposed.
		/// If disposing equals false, the method has been called by the 
		/// runtime from inside the finalizer and you should not reference 
		/// other objects. Only unmanaged resources can be disposed
		/// </remarks>
		/// <param name="disposing">TODO</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    // Dispose managed resources.
                	for (int c = 0; c < Count; c++)
                		this[c].Dispose();
                }

                // Call the appropriate methods to clean up 
                // unmanaged resources here.
                // If disposing is false, 
                // only the following code is executed.
            }
			
            disposed = true;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Inserts an effect. The name must not include any extension or path.
        /// If the effect exists, returns the reference to the existing effect
        /// </summary>
        /// <param name="name">TODO</param>
        /// <param name="load">TODO</param>
        /// <returns>valid JEffect reference</returns>
        public JEffect Create(string name, bool load)
        {
            JEffect effect;

        	// If there isnt any effect with that name, return null
			effect = Exists(name);
            if (effect == null)
            {
                effect = new JEffect(name);

                if (load) effect.Load();

                // Add it to the list

                Add(effect);
            }

            return effect;
        }

        /// <summary>
        /// Loads the default effects
        /// </summary>
        internal void LoadInHouse()
        {
            if(InHouse.Loaded) return;

            InHouse.Loaded = true;

            InHouse.PosColor = Create("poscolor", true);
            InHouse.PosColorTex1 = Create("posColorTex1", true);
            InHouse.SkyBox = Create("skybox", true);
			InHouse.Shafts = Create("lightShafts", true);
			InHouse.ShowZ= Create("showDOF", true);
            InHouse.ShowFloat = Create("showFloat", true);

			InHouse.PostProcessEffect = Create("postproceso", true);

			InHouse.PostProcess.radialBlur = Create("radialBlur", true);
            InHouse.PostProcess.depthOfField = Create("depthoffield", true);
            InHouse.PostProcess.GaussianBlur = Create("GaussianBlur", true);
            InHouse.PostProcess.Sobel = Create("sobel", true);
            InHouse.PostProcess.HDR = Create("hdr", true);
            InHouse.PostProcess.ToneMapping = Create("ToneMapping", true);
            InHouse.PostProcess.Blur = Create("blur", true);
            InHouse.PostProcess.HighPass = Create("High_Pass", true);
            InHouse.PostProcess.MotionBlur = Create("MotionBlur", true);
        }

		#endregion 
    	
		#region Reset / Lost
    	
		/// <summary>
        /// Called OnReset device
        /// </summary>
        public void Reset()
        {
            JLogWriter.Write("Effects.Reset", true);

            for (int c = 0; c < Count; c++)
            {
                JEffect effect = this[c];

                effect.Reset();
            }
        }

        /// <summary>
        /// Called OnLostDevice
        /// </summary>
        internal void OnLost()
        {
            JLogWriter.Write("Effects.OnLost", true);

            for (int c = 0; c < Count; c++)
            {
                JEffect effect=this[c];

                effect.OnLost();
            }
        }

        #endregion
    }	
}
