#region Using directives

using System.Drawing;

using Microsoft.DirectX.Direct3D;

using JadEngine.Core;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JRender
	{
		#region Structs
		
		/// <summary>
		/// TODO
		/// </summary>
		public struct JStats
		{
			#region Fields
        	
			/// <summary>
            /// Number of FPS text
            /// </summary>
            public string Fps;

			#endregion
        	
			#region Methods

			/// <summary>
            /// Updates the render values
            /// </summary>
            public void Update()
            {
				Fps = string.Format(Jad.Timer.Fps.ToString("f2", System.Globalization.CultureInfo.CurrentUICulture));
			}

			#endregion
		};

		#endregion

		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public JStats Stats;

		/// <summary>
		/// Clear flags
		/// </summary>
		public ClearFlags ClearFlags = ClearFlags.Target | ClearFlags.ZBuffer | ClearFlags.Stencil;

		/// <summary>
		/// Color after a Clear Clear
		/// </summary>
		public JColor Color;

		/// <summary>
		/// Z value after a Clear
		/// </summary>
		float ZDepth = 1;

		/// <summary>
		/// Stencil value after a Clear
		/// </summary>
		int Stencil = 0;

		/// <summary>
		/// TODO
		/// </summary>
		Rectangle[] rect;
		
		/// <summary>
		/// TODO
		/// </summary>
		JRenderTarget grid;
		
		#endregion
		
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JRender()
		{
			Color = new JColor(0, 0, 0);
		}

		#endregion

		#region Methods

		/// <summary>
		/// Clears the viewport
        /// </summary>
        public void Clear()
        {
            Jad.Video.Device.Clear(ClearFlags, Color.ToArgb(), ZDepth, Stencil);
        }

		/// <summary>
		/// Clears Z and Stencil if Clear is set to that
		/// </summary>
		public void ClearZ()
		{
			Jad.Video.Device.Clear(ClearFlags.ZBuffer, Color.ToArgb(), ZDepth, Stencil);
		}

		/// <summary>
		/// Clears the viewport
        /// </summary>
        public void Clear(ClearFlags flags,JColor color,float zDepth,int stencil)
        {
            Jad.Video.Device.Clear(flags, color.ToArgb(), zDepth, stencil);
        }

        /// <summary>
        /// Starts the BeginScene for the device
        /// </summary>
        public void Begin()
        {
            Jad.Video.Device.BeginScene();
        }

        /// <summary>
        /// Makes an EndScene
        /// </summary>
        public void End()
        {
            Jad.Video.Device.EndScene();
        }

        /// <summary>
        /// Presents the backbuffer and updates the stats
        /// </summary>
        public void Present()
        {
            int result;

            if (Jad.Video.Device.CheckCooperativeLevel(out result))
            {
				//Okay to render
                try
                {
                        Jad.Video.Device.Present();
                        Stats.Update();
                }
            	
                catch(DeviceLostException)
                {
                    Jad.Video.Device.CheckCooperativeLevel(out result);
                }
            	
                catch(DeviceNotResetException)
                {
                    Jad.Video.Device.CheckCooperativeLevel(out result);
                }
            }
        	
            if (result == (int)ResultCode.DeviceLost)
            {
                //Thread.Sleep(500); //Can't Reset yet, wait for a bit
            }
        	
            else if (result == (int)ResultCode.DeviceNotReset)
            {
                Jad.Video.Adapter.Reset();
            }
        }

        /// <summary>
        /// Renders a full screen rectangle
        /// </summary>
        public void RenderScreenAlignedQuad()
        {
            RenderScreenAlignedQuad(1f,1f,1f,1f);
        }

        /// <summary>
        /// Renders a full screen rectangle
        /// </summary>
        /// <param name="divisorx">With divisor</param>
        /// <param name="divisory">Height divisor</param>
        /// <param name="tu">Max texture u value</param>
        /// <param name="tv">Min texture u value</param>
        public void RenderScreenAlignedQuad(float divisorx,float divisory,float tu,float tv)
        {
            CustomVertex.TransformedTextured[] v = new CustomVertex.TransformedTextured[4];

            float w, h;

            if (Jad.Video.RenderTargets.ActiveRT == null)
            {
                w = Jad.Video.Width;
                h = Jad.Video.Height;
            }
        	
            else
            {
                w = (float)Jad.Video.RenderTargets.ActiveRT.Width;
                h = (float)Jad.Video.RenderTargets.ActiveRT.Height;
            }

        	if (divisorx != 1f)
        		w *= divisorx;

        	if (divisory != 1f)
        		h *= divisory;

        	v[0] = new CustomVertex.TransformedTextured(-0.5f, -0.5f, 0.5f, 1.0f, 0.0f, 0.0f);
            v[1] = new CustomVertex.TransformedTextured(w - 0.5f, -0.5f, 0.5f, 1.0f, tu, 0.0f);
            v[2] = new CustomVertex.TransformedTextured(-0.5f, h - 0.5f, 0.5f, 1.0f, 0.0f, tv);
            v[3] = new CustomVertex.TransformedTextured(w - 0.5f, h - 0.5f, 0.5f, 1.0f, tu, tv);

            Jad.Video.Device.VertexFormat = CustomVertex.TransformedTextured.Format;
            Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, v);
        }

        /// <summary>
        /// Renders a full screen rectangle, adding an offset to the texture
        /// </summary>
        /// <param name="offset">Texture offset</param>
        public void RenderScreenAlignedQuad(float offset)
        {
            CustomVertex.TransformedTextured[] v = new CustomVertex.TransformedTextured[4];

            float w, h;

            if (Jad.Video.RenderTargets.ActiveRT == null)
            {
                w = Jad.Video.Width;
                h = Jad.Video.Height;
            }
        	
            else
            {
                w = (float)Jad.Video.RenderTargets.ActiveRT.Width;
                h = (float)Jad.Video.RenderTargets.ActiveRT.Height;
            }

            v[0] = new CustomVertex.TransformedTextured(-0.5f, -0.5f, 0.5f, 1.0f, offset, offset);
            v[1] = new CustomVertex.TransformedTextured(w - 0.5f, -0.5f, 0.5f, 1.0f, 1.0f - offset, offset);
            v[2] = new CustomVertex.TransformedTextured(-0.5f, h - 0.5f, 0.5f, 1.0f, offset, 1.0f - offset);
            v[3] = new CustomVertex.TransformedTextured(w - 0.5f, h - 0.5f, 0.5f, 1.0f, 1.0f - offset, 1.0f - offset);

            Jad.Video.Device.VertexFormat = CustomVertex.TransformedTextured.Format;
            Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, v);
        }

        /// <summary>
        /// Renders a screen rectangle
        /// </summary>
        /// <param name="x">Screen coords x(left)</param>
        /// <param name="y">Screen coords y(top)</param>
        /// <param name="x2">Screen coords x(right)</param>
        /// <param name="y2">Screen coords y(botom)</param>
        public void RenderScreenAlignedQuadByCoords(float x,float y,float x2,float y2)
        {
            CustomVertex.TransformedTextured[] v = new CustomVertex.TransformedTextured[4];
            float tu = 0, tv = 0;

            v[0] = new CustomVertex.TransformedTextured(x-0.5f, y-0.5f, 0.5f, 1.0f, 0.0f, 0.0f);
            v[1] = new CustomVertex.TransformedTextured(x2 - 0.5f, y-0.5f, 0.5f, 1.0f, tu, 0.0f);
            v[2] = new CustomVertex.TransformedTextured(x-0.5f, y2 - 0.5f, 0.5f, 1.0f, 0.0f, tv);
            v[3] = new CustomVertex.TransformedTextured(x2 - 0.5f, y2 - 0.5f, 0.5f, 1.0f, tu, tv);

            Jad.Video.Device.VertexFormat = CustomVertex.TransformedTextured.Format;
            Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, v);
		}

		/// <summary>
		/// TODO
		/// </summary>
        public void MakeStencil50Percent()
        {
            int w=Jad.Video.Width;
            int h=Jad.Video.Height;

            if (rect == null)
            {
                rect = new Rectangle[(w / 2) + (h / 2)];

                int n = 0;

                for (int y = 0; y < h; y += 2)
                {

                    rect[n].X = 0;
                    rect[n].Y = y;
                    rect[n].Width = w;
                    rect[n].Height = 1;
                    n++;
                }

                for (int x = 0; x < w; x += 2)
                {

                    rect[n].X = x;
                    rect[n].Y = 0;
                    rect[n].Width = 1;
                    rect[n].Height = h;
                    n++;
                }

                grid = Jad.Video.RenderTargets.CreateRelative("grid", 1, Format.A8R8G8B8, false, true);

                grid.Begin();
                Jad.Video.Device.Clear(ClearFlags.Target, new JColor(0, 0, 0, 0).ToArgb(), 0, 0);
                Jad.Video.Device.Clear(ClearFlags.Target, new JColor(1, 1, 1, 1).ToArgb(), 0, 0, rect);
                grid.End();
                grid.Texture.Save("grid", ImageFileFormat.Bmp);
            }

            Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, true);

			// Set up the Zbuffer with 1 0
            //Jad.Video.Device.Clear(ClearFlags.ZBuffer, new Color(0, 0, 0, 0).ToArgb(), 0, 0, rect);
            Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);

            // Set up the stencil with 1 in the even lines

            Jad.Video.StateManager.SetRenderState(RenderStates.StencilEnable, true);
            Jad.Video.Device.Clear(ClearFlags.Stencil, new JColor(0, 0, 0, 0).ToArgb(), 0, 0);
            Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
            Jad.Video.StateManager.SetRenderState(RenderStates.AlphaTestEnable, true);
            Jad.Video.StateManager.SetRenderState(RenderStates.AlphaFunction, (int)Compare.NotEqual);
            Jad.Video.StateManager.SetRenderState(RenderStates.ReferenceAlpha, 0);

            Jad.Video.StateManager.SetRenderState(RenderStates.StencilFunction, (int)Compare.Always);
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilFail, (int)StencilOperation.Keep);
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilZBufferFail, (int)StencilOperation.Keep);
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilPass, (int)StencilOperation.Increment);
            Jad.Video.StateManager.SetRenderState(RenderStates.ReferenceStencil, 0x1);
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilMask, unchecked((int)0xffffffff));
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilWriteMask, unchecked((int)0xffffffff));
            grid.Texture.Activate(0);
            RenderScreenAlignedQuad();

            Jad.Video.StateManager.SetRenderState(RenderStates.AlphaTestEnable, false);
            Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
            //Jad.Video.Device.Clear(ClearFlags.Stencil, new Color(0, 0, 0, 0).ToArgb(), 1, 1, rect);

            // Set up stencil compare fuction, reference value, and masks
            // Stencil test passes if ((ref & mask) cmpfn (stencil & mask)) is true.
            // Note: since we set up the stencil-test to always pass, the STENCILFAIL 
            // renderstate is really not needed.
			Jad.Video.StateManager.SetRenderState(RenderStates.StencilFunction, (int)Compare.Equal);
			Jad.Video.StateManager.SetRenderState(RenderStates.StencilFail, (int)StencilOperation.Keep);
			Jad.Video.StateManager.SetRenderState(RenderStates.StencilZBufferFail, (int)StencilOperation.Keep);
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilPass, (int)StencilOperation.Keep);
            Jad.Video.StateManager.SetRenderState(RenderStates.ReferenceStencil, 0x1);
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilMask, unchecked((int)0xffffffff));
            Jad.Video.StateManager.SetRenderState(RenderStates.StencilWriteMask, unchecked((int)0xffffffff));
        }
		
		#endregion
	}
}
