#region Using directives

using System;

using System.Drawing;

using Microsoft.DirectX.Direct3D;

using JadEngine.Core;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JTextureBase : IJListable
	{
		#region Interop Services

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="hdcDest">Handle to destination DC</param>
		/// <param name="nXDest">x-coord of destination upper-left corner</param>
		/// <param name="nYDest">y-coord of destination upper-left corner</param>
		/// <param name="nWidth">Width of destination rectangle</param>
		/// <param name="nHeight">Height of destination rectangle</param>
		/// <param name="hdcSrc">Handle to source DC</param>
		/// <param name="nXSrc">x-coordinate of source upper-left corner</param>
		/// <param name="nYSrc">y-coordinate of source upper-left corner</param>
		/// <param name="dwRop">Raster operation code</param>
		/// <returns>TODO</returns>
		[System.Runtime.InteropServices.DllImportAttribute("gdi32.dll")]
		public static extern int BitBlt(IntPtr hdcDest, int nXDest, int nYDest, int nWidth, int nHeight, IntPtr hdcSrc, int nXSrc, int nYSrc, int dwRop);

		#endregion

		#region Structs

		/// <summary>
		/// Struct with the necessary parameters to load a texture
		/// </summary>
		public struct HLoadingParams
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public int Width;

			/// <summary>
			/// TODO
			/// </summary>
			public int Height;

			/// <summary>
			/// TODO
			/// </summary>
			public int Mip;

			/// <summary>
			/// TODO
			/// </summary>
			public Usage Usage;

			/// <summary>
			/// TODO
			/// </summary>
			public Format Format;

			/// <summary>
			/// TODO
			/// </summary>
			public Pool Pool;

			/// <summary>
			/// TODO
			/// </summary>
			public Filter Filter;

			/// <summary>
			/// TODO
			/// </summary>
			public Filter MipFilter;

			/// <summary>
			/// TODO
			/// </summary>
			public int ColorKey;

			#endregion
		}

		#endregion

		#region Constants

		/// <summary>
		/// Possible texture extensions
		/// </summary>
		/// <remarks>
		/// If the "" extension is added, the developer can insert a texture with extension, forcing
		/// the system to load that extension
		/// </remarks>
		protected readonly string[] extensions ={ "", ".png", ".jpg", ".tga", ".bmp", ".dds", ".dib", ".hdr", ".pfm", ".ppm" };

		#endregion

		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		static int IDGlobal = 0;

		/// <summary>
		/// TODO
		/// </summary>
		int id;

		/// <summary>
		/// Name of the texture, without path or extension
		/// </summary>
		protected string name;

		/// <summary>
		/// Complete path of the texture
		/// </summary>
		protected string fullPath;

		/// <summary>
		/// The DX object
		/// </summary>
		protected BaseTexture dxResource;

		/// <summary>
		/// Information about the image
		/// </summary>
		protected ImageInformation imageInfo;

		/// <summary>
		/// Level 0 surface description
		/// </summary>
		protected SurfaceDescription surfaceDes;

		/// <summary>
		/// Texture saved in a GDI+ image
		/// </summary>
		protected Image image;

		/// <summary>
		/// Tells if a texture is loaded by default in the engine, so when freeing the scene it must not be freed
		/// </summary>
		public bool InHouse = false;

		/// <summary>
		/// Tells if the texture acts as a RenderTarget
		/// </summary>
		public bool RenderTarget = false;

		/// <summary>
		/// Tells how the object is going to be loaded
		/// </summary>
		public HLoadingParams LoadingParams;

		#endregion

		#region Constructors

		/// <summary>
		/// Creates a TextureBase object and assigns it a name. It also
		/// initalizes the LoadingParams
		/// </summary>
		/// <param name="name">Name of the texture</param>
		public JTextureBase(string name)
		{
			id = ++IDGlobal;

			// Set the name
			this.name = name;

			// Create the default parameters to load the texture
			LoadingParams.Width = D3DX.Default;
			LoadingParams.Height = D3DX.Default;
			LoadingParams.Mip = D3DX.Default;
			LoadingParams.Usage = Usage.None;

			if (Jad.Video.Textures.CompressImagesOnLoad)
				LoadingParams.Format = Format.Dxt1;

			else
				LoadingParams.Format = Jad.Video.Textures.LoadingFormat;

			// When setting the Pool to DEFAULT, we are the ones who
			// must restore de device

			LoadingParams.Pool = Pool.Managed;
			LoadingParams.Filter = (Filter) D3DX.Default;
			LoadingParams.MipFilter = (Filter) D3DX.Default;
			LoadingParams.ColorKey = 0;

			DXResource = null;
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets the level 0 surface description
		/// </summary>
		public SurfaceDescription SurfaceDes
		{
			get { return surfaceDes; }
		}

		/// <summary>
		/// Gets the information about the image
		/// </summary>
		public ImageInformation ImageInfo
		{
			get { return imageInfo; }
		}

		/// <summary>
		/// Gets or sets the DX object
		/// </summary>
		public BaseTexture DXResource
		{
			get { return dxResource; }
			set { dxResource = value; }
		}

		/// <summary>
		/// Gets the complete path of the texture
		/// </summary>
		public string FullPath
		{
			get { return fullPath; }
		}

		/// <summary>
		/// Gets or sets the name of the texture, without path or extension
		/// </summary>
		public string Name
		{
			get { return name; }
			set { name = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public int ID
		{
			get { return id; }
		}

		/// <summary>
		/// Gets the texture saved as a GDI+ image
		/// </summary>
		public Image Image
		{
			get { return image; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// Searches in the extensions the texture file to load
		/// </summary>
		protected void SearchFullPath()
		{
			// Always search in the textures directory
			if (!JPath.FindFullPath(name, JPath.BaseDirectory, JPath.Directories.Textures, extensions, out fullPath))
				JPath.FindFullPath(name, JPath.InHouseDirectory, JPath.Directories.Textures, extensions, out fullPath);
		}

		/// <summary>
		/// Activates the texture
		/// </summary>
		/// <param name="channel">The Stage where the texture will be applied</param>
		public void Activate(int channel)
		{
			if (dxResource == null)
				throw new InvalidOperationException("The texture " + name + " doesnt exists.");

			Jad.Video.Device.SetTexture(channel, dxResource);
		}

		/// <summary>
		/// Saves the texture in the disk
		/// </summary>
		/// <param name="fileName">Name of the file</param>
		/// <param name="fileFormat">Saving format</param>
		public virtual void Save(string fileName, ImageFileFormat fileFormat)
		{
		}

		/// <summary>
		/// TODO
		/// </summary>
		public virtual void Reset()
		{
		}

		/// <summary>
		/// TODO
		/// </summary>
		public virtual void OnLost()
		{
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="srcGraphics">TODO</param>
		/// <param name="destGraphics">TODO</param>
		/// <param name="w">TODO</param>
		/// <param name="h">TODO</param>
		protected void CopyGraphicsContent(Graphics srcGraphics, Graphics destGraphics, int w, int h)
		{
			IntPtr srcHdc = srcGraphics.GetHdc();
			IntPtr destHdc = destGraphics.GetHdc();

			BitBlt(destHdc, 0, 0, w, h, srcHdc, 0, 0, 0x00CC0020);

			srcGraphics.ReleaseHdc(srcHdc);
			destGraphics.ReleaseHdc(destHdc);
		}

		#endregion
	}
}
