#region Using directives

using System;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Implements the texture volume functionality
    /// </summary>
    public class JTextureVolume : JTextureBase
    {
        #region Constructors

    	/// <summary>
    	/// Default constructor
    	/// </summary>
    	/// <param name="name">Name of the texture</param>
		public JTextureVolume(string name)
    		: base(name)
		{ }

        #endregion

		#region Properties

		/// <summary>
		/// Gets or sets the DX resource
		/// </summary>
		/// TODO (VICENTE): SHADOWING, IS THIS A GOOD IDEA???
		public new VolumeTexture DXResource
		{
			get { return (VolumeTexture) dxResource; }
			set { dxResource = value; }
		}

		#endregion
    	
        #region Methods

		/// <summary>
		/// Called after a Reset device
		/// </summary>
		public override void Reset()
		{
		}

		/// <summary>
		/// Called on OnLostDevice
		/// </summary>
		public override void OnLost()
		{
		}
    	
        /// <summary>
        /// Loads a texture from a file using the loading parameters
        /// </summary>
        /// <returns>True if everything was right, false if not</returns>
        public bool Load()
        {
        	// Only load the texture if it hasnt being loaded yet
            if (dxResource == null)
            {
                SearchFullPath();

            	if (fullPath == null)
            		throw new InvalidOperationException("Couldnt find the texture " + Name);

            	try
                {
                    dxResource = TextureLoader.FromVolumeFile(Jad.Video.Device,
                        fullPath,
                        0,
                        0,
						0,	// Depth
						0,	// MipLevels
                        LoadingParams.Usage,
                        LoadingParams.Format,
                        LoadingParams.Pool,
                        LoadingParams.Filter,
                        LoadingParams.MipFilter,
                        LoadingParams.ColorKey);
                }
            	
                catch (DirectXException e)
                {
                    throw new InvalidOperationException("When loading the texture " + Name + " in " + fullPath + ". Error:" + e.ErrorString);
                }

                if (dxResource == null) return false;

				VolumeTexture texture = (VolumeTexture)dxResource;
				VolumeDescription surface = texture.GetLevelDescription(0);

                imageInfo = TextureLoader.ImageInformationFromFile(fullPath);

                JLogWriter.Write("Texture Volume:" + name, true);
                JLogWriter.Write("\tFullPath:" + fullPath, true);
				JLogWriter.Write("\tSize:" + surface.Width.ToString() + " x " + surface.Height.ToString() + " x " + surface.Depth.ToString(), true);
				JLogWriter.Write("\tFormat:" + surface.Format.ToString(), true);

				JLogWriter.Write(imageInfo.ToString(), true);

				JLogWriter.WriteSeparator();
            }

            return true;
        }

        #endregion
    }
}
