#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX;

using JadEngine.Particles;

#endregion

namespace Viewer
{
	public partial class ParticleSystemEmitterLinePanel: UserControl
	{
		#region Events

		public event EventHandler EmitterChanged;

		#endregion

		#region Fields

		public JEmitterLine Emitter;

		#endregion

		#region Constructors

		public ParticleSystemEmitterLinePanel()
		{
			InitializeComponent();
		}

		#endregion

		#region Methods

		public void UpdateEmitter(JEmitterLine emitter)
		{
			Emitter = emitter;

			numEmitterLineP1X.Value = (decimal)emitter.StartPoint.X;
			numEmitterLineP1Y.Value = (decimal)emitter.StartPoint.Y;
			numEmitterLineP1Z.Value = (decimal)emitter.StartPoint.Z;

			numEmitterLineP2X.Value = (decimal)emitter.EndPoint.X;
			numEmitterLineP2Y.Value = (decimal)emitter.EndPoint.Y;
			numEmitterLineP2Z.Value = (decimal)emitter.EndPoint.Z;
		}

		protected virtual void OnEmitterChange(EventArgs e)
		{
			if (EmitterChanged != null)
				EmitterChanged(this, e);
		}

		#endregion

		#region Event Handlers

		private void numEmitterLineP1X_ValueChanged(object sender, EventArgs e)
		{
			Emitter.StartPoint = new Vector3((float)numEmitterLineP1X.Value, Emitter.StartPoint.Y, Emitter.StartPoint.Z);
			OnEmitterChange(new EventArgs());
		}

		private void numEmitterLineP1Y_ValueChanged(object sender, EventArgs e)
		{
			Emitter.StartPoint = new Vector3(Emitter.StartPoint.X, (float)numEmitterLineP1Y.Value, Emitter.StartPoint.Z);
			OnEmitterChange(new EventArgs());
		}

		private void numEmitterLineP1Z_ValueChanged(object sender, EventArgs e)
		{
			Emitter.StartPoint = new Vector3(Emitter.StartPoint.X, Emitter.StartPoint.Y, (float)numEmitterLineP1Z.Value);
			OnEmitterChange(new EventArgs());
		}

		private void numEmitterLineP2X_ValueChanged(object sender, EventArgs e)
		{
			Emitter.EndPoint = new Vector3((float)numEmitterLineP2X.Value, Emitter.EndPoint.Y, Emitter.EndPoint.Z);
			OnEmitterChange(new EventArgs());
		}

		private void numEmitterLineP2Y_ValueChanged(object sender, EventArgs e)
		{
			Emitter.EndPoint = new Vector3(Emitter.EndPoint.X, (float)numEmitterLineP2Y.Value, Emitter.EndPoint.Z);
			OnEmitterChange(new EventArgs());
		}

		private void numEmitterLineP2Z_ValueChanged(object sender, EventArgs e)
		{
			Emitter.EndPoint = new Vector3(Emitter.EndPoint.X, Emitter.EndPoint.Y, (float)numEmitterLineP2Z.Value);
			OnEmitterChange(new EventArgs());
		}

		#endregion
	}
}
