#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX;

using JadEngine.Particles;

#endregion

namespace Viewer
{
	public partial class ParticleSystemParamsPanel: UserControl
	{
		#region Events

		public event EventHandler ProfileChanged;

		#endregion

		#region Fields

		public bool Reset;
		public JParticleSystemProfile Profile;

		private ParticleSystemEmitterLinePanel emitterLinePanel;
		private ParticleSystemEmitterSpherePanel emitterSpherePanel;
		private ParticleSystemEmitterBoxPanel emitterBoxPanel;

		bool justLoaded = false;

		#endregion

		#region Constructors

		public ParticleSystemParamsPanel()
		{
			InitializeComponent();

			emitterLinePanel = new ParticleSystemEmitterLinePanel();
			emitterSpherePanel = new ParticleSystemEmitterSpherePanel();
			emitterBoxPanel = new ParticleSystemEmitterBoxPanel();

			emitterLinePanel.EmitterChanged += new EventHandler(OnEmiterChanged);

			justLoaded = true;
		}

		#endregion

		#region Methods

		public void UpdateProfile(JParticleSystemProfile profile)
		{
			Profile = profile;

			numPFxCount.Value = (decimal)profile.numParticles;
			numPFxLifeTime.Value = (decimal)profile.lifeTime;
			numPFxBirthRate.Value = (decimal)profile.birthRate;
			chkPFxLoop.Checked = profile.loop;
			chkPFxSortParticles.Checked = profile.sort;
			chkPFxConstBirthRate.Checked = profile.constBirthRate;
			numPFxSizeStart.Value = (decimal)profile.sizeStart;
			numPFxSizeEnd.Value = (decimal)profile.sizeEnd;
			numPFxAspectStart.Value = (decimal)profile.aspectStart;
			numPFxAspectEnd.Value = (decimal)profile.aspectEnd;
			numPFxSpeedStart.Value = (decimal)profile.speedStart;
			numPFxSpeedEnd.Value = (decimal)profile.speedEnd;
			numPFxRotationStart.Value = (decimal)profile.angleStart;
			numPFxRotationEnd.Value = (decimal)profile.angleEnd;
			lblPFxColorStart.BackColor = profile.colorStart.ToSystemColor();
			lblPFxColorEnd.BackColor = profile.colorEnd.ToSystemColor();
			numPFxAlphaStart.Value = (decimal)profile.colorStart.Alpha * 255;
			numPFxAlphaEnd.Value = (decimal)profile.colorEnd.Alpha * 255;
			txtPFxMtlFileName.Text = Profile.material.Name;
			numPFxPosX.Value = (decimal)profile.position.X;
			numPFxPosY.Value = (decimal)profile.position.Y;
			numPFxPosZ.Value = (decimal)profile.position.Z;
			numPFxDirX.Value = (decimal)profile.direction.X;
			numPFxDirY.Value = (decimal)profile.direction.Y;
			numPFxDirZ.Value = (decimal)profile.direction.Z;
			numPFxTheta.Value = (decimal)profile.theta;
			numPFxGravX.Value = (decimal)profile.gravity.X;
			numPFxGravY.Value = (decimal)profile.gravity.Y;
			numPFxGravZ.Value = (decimal)profile.gravity.Z;
			if (profile.orientationType == JParticleOrientation.View)
			{
				rdoPFxOrientationView.Checked = true;
				numPFxOrientationTrailSegments.Enabled = false;
				numPFxOrientationTrailTime.Enabled = false;
				numPFxRotationStart.Enabled = true;
				numPFxRotationEnd.Enabled = true;
				numPFxAspectStart.Enabled = true;
				numPFxAspectEnd.Enabled = true;
			}
			else
			{
				rdoPFxOrientationAimed.Checked = true;
				numPFxOrientationTrailSegments.Enabled = true;
				numPFxOrientationTrailTime.Enabled = true;
				numPFxRotationStart.Enabled = false;
				numPFxRotationEnd.Enabled = false;
				numPFxAspectStart.Enabled = false;
				numPFxAspectEnd.Enabled = false;

				numPFxOrientationTrailSegments.Value = (decimal)profile.trailSegments;
				numPFxOrientationTrailTime.Value = (decimal)profile.trailTime;
			}
			if (profile.emitter is JEmitterPoint)
			{
				cbPFxEmitterType.SelectedIndex = 0;
				pnlEmitterParams.Controls.Clear();
			}
			else if (profile.emitter is JEmitterLine)
			{
				cbPFxEmitterType.SelectedIndex = 1;
				pnlEmitterParams.Controls.Clear();
				pnlEmitterParams.Controls.Add(emitterLinePanel);
				emitterLinePanel.UpdateEmitter(profile.emitter as JEmitterLine);
			}
			else if (profile.emitter is JEmitterSphere)
			{
				cbPFxEmitterType.SelectedIndex = 2;
				pnlEmitterParams.Controls.Clear();
				pnlEmitterParams.Controls.Add(emitterSpherePanel);
				emitterSpherePanel.UpdateEmitter(profile.emitter as JEmitterSphere);
			}
			else if (profile.emitter is JEmitterBox)
			{
				cbPFxEmitterType.SelectedIndex = 3;
				pnlEmitterParams.Controls.Clear();
				pnlEmitterParams.Controls.Add(emitterBoxPanel);
				emitterBoxPanel.UpdateEmitter(profile.emitter as JEmitterBox);
			}			
		}

		protected virtual void OnProfileChanged(EventArgs e)
		{
			if (ProfileChanged != null)
				ProfileChanged(this, e);
		}

		public void OnEmiterChanged(object sender, EventArgs e)
		{
		}

		#endregion

		#region Event Handlers

		private void numPFxCount_ValueChanged(object sender, EventArgs e)
		{
			Reset = true;
			Profile.numParticles = (uint)numPFxCount.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxLifeTime_ValueChanged(object sender, EventArgs e)
		{
			Reset = true;
			Profile.lifeTime = (float)numPFxLifeTime.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxBirthRate_ValueChanged(object sender, EventArgs e)
		{
			Reset = true;
			Profile.birthRate = (float)numPFxBirthRate.Value;
			OnProfileChanged(new EventArgs());
		}

		private void checkPFxLoop_CheckedChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.loop = chkPFxLoop.Checked;
			OnProfileChanged(new EventArgs());
		}

		private void chkPFxSortParticles_CheckedChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.sort = chkPFxSortParticles.Checked;
			OnProfileChanged(new EventArgs());
		}

		private void chkPFxConstBirthrate_CheckedChanged(object sender, EventArgs e)
		{
			Reset = true;
			Profile.constBirthRate = chkPFxConstBirthRate.Checked;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxSizeStart_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.sizeStart = (float)numPFxSizeStart.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxSizeEnd_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.sizeEnd = (float)numPFxSizeEnd.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxAspectStart_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.aspectStart = (float)numPFxAspectStart.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxAspectEnd_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.aspectEnd = (float)numPFxAspectEnd.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxSpeedStart_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.speedStart = (float)numPFxSpeedStart.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxSpeedEnd_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.speedEnd = (float)numPFxSpeedEnd.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxRotationStart_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.angleStart = (float)numPFxRotationStart.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxRotationEnd_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.angleEnd = (float)numPFxRotationEnd.Value;
			OnProfileChanged(new EventArgs());
		}

		private void lblPFxColorStart_Click(object sender, EventArgs e)
		{
			ColorDialog dlg = new ColorDialog();
			dlg.FullOpen = true;
			dlg.Color = lblPFxColorStart.BackColor;

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Reset = false;
				lblPFxColorStart.BackColor = dlg.Color;
				Profile.colorStart = new JadEngine.Core.JColor(dlg.Color);
				Profile.colorStart.Alpha = (float)numPFxAlphaStart.Value / 255.0f;
				OnProfileChanged(new EventArgs());
			}
		}

		private void numPFxAlphaStart_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.colorStart.Alpha = (float)numPFxAlphaStart.Value / 255.0f;
			OnProfileChanged(new EventArgs());
		}

		private void lblPFxColorEnd_Click(object sender, EventArgs e)
		{
			ColorDialog dlg = new ColorDialog();
			dlg.FullOpen = true;
			dlg.Color = lblPFxColorEnd.BackColor;

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Reset = false;
				lblPFxColorEnd.BackColor = dlg.Color;
				Profile.colorEnd = new JadEngine.Core.JColor(dlg.Color);
				Profile.colorEnd.Alpha = (float)numPFxAlphaEnd.Value / 255.0f;
				OnProfileChanged(new EventArgs());
			}
		}

		private void numPFxAlphaEnd_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.colorEnd.Alpha = (float)numPFxAlphaEnd.Value / 255.0f;
			OnProfileChanged(new EventArgs());
		}

		private void btnPFxMtlBrowse_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.RestoreDirectory = true;
			dlg.Filter = "Material files (*.material)|*.material|All files (*.*)|*.*";
			dlg.FilterIndex = 0;

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Reset = false;
				Profile.material = JadEngine.Jad.Scene.Materials.Load(dlg.FileName);
				txtPFxMtlFileName.Text = Profile.material.Name;
				OnProfileChanged(new EventArgs());
			}
		}

		private void numPFxPosX_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.position.X = (float)numPFxPosX.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxPosY_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.position.Y = (float)numPFxPosY.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxPosZ_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.position.Z = (float)numPFxPosZ.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxDirX_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.direction.X = (float)numPFxDirX.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxDirY_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.direction.Y = (float)numPFxDirY.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxDirZ_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.direction.Z = (float)numPFxDirZ.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxTheta_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.theta = (float)numPFxTheta.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxGravX_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.gravity.X = (float)numPFxGravX.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxGravY_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.gravity.Y = (float)numPFxGravY.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxGravZ_ValueChanged(object sender, EventArgs e)
		{
			Reset = false;
			Profile.gravity.Z = (float)numPFxGravZ.Value;
			OnProfileChanged(new EventArgs());
		}

		private void rdoPFxOrientationView_CheckedChanged(object sender, EventArgs e)
		{
			Reset = false;
			numPFxOrientationTrailSegments.Enabled = false;
			numPFxOrientationTrailTime.Enabled = false;
			numPFxRotationStart.Enabled = true;
			numPFxRotationEnd.Enabled = true;
			numPFxAspectStart.Enabled = true;
			numPFxAspectEnd.Enabled = true;

			Profile.orientationType = JParticleOrientation.View;

			OnProfileChanged(new EventArgs());
		}

		private void rdoPFxOrientationAimed_CheckedChanged(object sender, EventArgs e)
		{
			Reset = true;

			numPFxOrientationTrailSegments.Enabled = true;
			numPFxOrientationTrailTime.Enabled = true;
			numPFxRotationStart.Enabled = false;
			numPFxRotationEnd.Enabled = false;
			numPFxAspectStart.Enabled = false;
			numPFxAspectEnd.Enabled = false;

			Profile.orientationType = JParticleOrientation.Aimed;
			Profile.trailSegments = (uint)numPFxOrientationTrailSegments.Value;
			Profile.trailTime = (float)numPFxOrientationTrailTime.Value;

			OnProfileChanged(new EventArgs());
		}

		private void numPFxOrientationTrailSegments_ValueChanged(object sender, EventArgs e)
		{
			Reset = true;
			Profile.trailSegments = (uint)numPFxOrientationTrailSegments.Value;
			OnProfileChanged(new EventArgs());
		}

		private void numPFxOrientationTrailTime_ValueChanged(object sender, EventArgs e)
		{
			Reset = true;
			Profile.trailTime = (float)numPFxOrientationTrailTime.Value;
			OnProfileChanged(new EventArgs());
		}

		private void cbPFxEmitterType_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (justLoaded)
			{
				justLoaded = false;
				return;
			}

			Reset = false;

			switch (cbPFxEmitterType.SelectedIndex)
			{
				case 0:
					Profile.emitter = new JEmitterPoint(Profile.position);
					pnlEmitterParams.Controls.Clear();
					pnlEmitterParams.Controls.Add(lblNone);
					break;
				case 1:
					JEmitterLine emitterLine = new JEmitterLine(new Vector3(-0.5f, 0.0f, 0.0f), new Vector3(0.5f, 0.0f, 0.0f));
					Profile.emitter = emitterLine;
					pnlEmitterParams.Controls.Clear();
					pnlEmitterParams.Controls.Add(emitterLinePanel);
					emitterLinePanel.UpdateEmitter(emitterLine);
					break;
				case 2:
					JEmitterSphere emitterSphere = new JEmitterSphere(Profile.position, 1.0f, false);
					Profile.emitter = emitterSphere;
					pnlEmitterParams.Controls.Clear();
					pnlEmitterParams.Controls.Add(emitterSpherePanel);
					emitterSpherePanel.UpdateEmitter(emitterSphere);
					break;
				case 3:
					JEmitterBox emitterBox = new JEmitterBox(new Vector3(-0.5f, -0.5f, -0.5f), new Vector3(0.5f, 0.5f, 0.5f));
					Profile.emitter = emitterBox;
					pnlEmitterParams.Controls.Clear();
					pnlEmitterParams.Controls.Add(emitterBoxPanel);
					emitterBoxPanel.UpdateEmitter(emitterBox);
					break;
				default:
					break;
			}

			OnProfileChanged(new EventArgs());
		}

		#endregion
	}
}
