#region Using directives

using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;

using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;

using JadEngine;
using JadEngine.Input;
using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Scene;
using JadEngine.Scene.MD5;
using JadEngine.Physics;
using JadEngine.Particles;
using JadEngine.Sound;
using JadEngine.Tools;
using JadEngine.Import;

#endregion

namespace Viewer
{
	public partial class ViewerFrm : Form
	{
		#region Fields

		/// <summary>The name of the scene loaded that will appear at the form caption</summary>
		string sceneName;
		private static string caption = "Jad Viewer";

		public JViewer jViewer;

		// Used to move the light
		public JNode NodeSelected;
		public TreeNode TreeNodeSelected;

		bool mouseDown = false;
		Point mouseDownPoint = Point.Empty;
		bool mouseControlsCamera = false;

		public bool ShowFPS = false;
		bool ShowGizmos = false;
		public bool ShowGrid = false;

		JSpotLight flashLight;

		JCameraPathAnimator curPathAnim;

		public JParticleEffect pFX;

		public JMeshObject test_model;

		public bool showCameraPos = false;

		bool paused;

		bool pickMode;
		bool pickLightsMode;
		bool pickPFxMode;

		bool justLoaded = false;
		bool changedFromHere = false;

		#endregion

		#region Constructors

		public ViewerFrm()
		{
			Application.EnableVisualStyles();

			InitializeComponent();

			this.renderCtrl.MouseWheel += new System.Windows.Forms.MouseEventHandler(this.renderCtrl_MouseWheel);

			EnablePFxUI(false);
		}

		#endregion

		#region Initialize Controls

		void InitializeScene()
		{
			Jad.Scene.ShowAxes = showAxesToolStripMenuItem.Checked;

			// Scene Pag

			numericBumpiness.Value = (decimal) Jad.Scene.MeshObjects.Bumpiness;

			numericFogNear.Value = (decimal) Jad.Scene.Fog.Near;

			numericFogFar.Value = (decimal) Jad.Scene.Fog.Far;

			numericFogDensity.Value = (decimal) Jad.Scene.Fog.Density;

			numericFogTop.Value = (decimal) Jad.Scene.Fog.Top;

			numericFogBottom.Value = (decimal) Jad.Scene.Fog.Bottom;

			comboFogMode.SelectedIndex = (int) Jad.Scene.Fog.Type;

			comboFogMode_SelectedIndexChanged(null, null);

			labelFogColor.BackColor = Jad.Scene.Fog.Color.ToSystemColor();

			labelAmbient.BackColor = Jad.Scene.Lights.Ambient.ToSystemColor();

			numExtents.Value = jViewer.Grid.GridExtents;
			numSpacing.Value = (decimal) jViewer.Grid.GridSpacing;
			checkBoxGridShow.Checked = ShowGrid;

			comboShadowsMode.SelectedIndex = 0;
			checkBoxShadowSoft.Enabled = (comboShadowsMode.SelectedIndex > 0);

			checkBoxShadowDebug.Checked = Jad.Scene.Shadows.Stencil.Debug;

			checkBoxMotionBlur.Checked = Jad.Scene.MotionBlur.Enabled;
			numericUpDownMotionBlur.Value = (decimal) Jad.Scene.MotionBlur.Filter.PixelBlurConst;
			numericUpDownNumSamples.Value = Jad.Scene.MotionBlur.Filter.NumSamples;

			checkBoxBloom.Checked = Jad.Scene.BloomEffect.Enabled;
			numericUpDownBloomHighPass.Value = (decimal) Jad.Scene.BloomEffect.HighPass.HighPassValue;
			numericBloomMultiply.Value = (decimal) Jad.Scene.BloomEffect.HighPass.Multiplier;

			// NOTE! THIS IS DONE EVERYTIME A SCENE LOADS, PROBABLY IT'S NOT RIGHT!
			particleSystemPanel.ProfileChanged += new EventHandler(OnProfileChanged);

			numTimer.Value = timerMotor.Interval;
		}

		#endregion

		#region MainForm Events

		private void ViewerFrm_Load(object sender, EventArgs e)
		{
			InitializeScene();

			ChangeWindowCaption();

			SetCameraDefaults();
		}

		private void ViewerFrm_Resize(object sender, EventArgs e)
		{
			if (WindowState == FormWindowState.Minimized)
			{
				// The engine is not active
				Jad.Active = false;

				return;
			}
			else if (Jad.Active == false)
				Jad.Active = true;

			ResizeRender();

			Jad.Video.Adapter.Reset();
		}

		private void ViewerFrm_FormClosing(object sender, FormClosingEventArgs e)
		{
			Jad.End();
		}

		private void ViewerFrm_Deactivate(object sender, EventArgs e)
		{
			Jad.Active = false;
		}

		private void ViewerFrm_Activated(object sender, EventArgs e)
		{
			Jad.Active = true;
		}

		#endregion

		#region File Menu

		private void loadToolStripMenuItem_Click(object sender, EventArgs e)
		{
			OpenFileDialog f;

			f = new OpenFileDialog();
			f.RestoreDirectory = true;
			f.Filter = "Jad files (*.haddd)|*.haddd|MD5 files (*.md5def)|*.md5def|ParticleFX files (*.pfx)|*.pfx|All Jad files (*.haddd;*.md5def;*.pfx)|*.haddd;*.md5def;*.pfx|All files (*.*)|*.*";
			//f.Filter = "Jad files (*.haddd)|*.haddd|X files (*.x)|*.x|ParticleFX files (*.pfx)|*.pfx|All Jad files (*.haddd;*.x;*.pfx)|*.haddd;*.x;*.pfx|All files (*.*)|*.*";
			f.FilterIndex = 4;
			if (f.ShowDialog() == DialogResult.Cancel)
				return;

			// Release the scene objects
			// Do not delete AllInOne objects

			Jad.Scene.Destroy();
			Jad.Video.Meshes.Destroy();
			Jad.Video.Textures.Destroy();

			// Recreate the camera
			JCameraFirstPerson user_camera = Jad.Scene.Cameras.CreateFirstPerson("user_camera");
			user_camera.FOV = Geometry.DegreeToRadian(60.0f);
			user_camera.FarPlane = 50.0f;  // 50 m
			user_camera.NearPlane = 0.01f;   // 1 cm
			user_camera.Sensitivity = 20.0f;
			user_camera.Speed = 5.0f;
			user_camera.Position = new Vector3(0.0f, 1.0f, -5.0f);
			Jad.Scene.Camera = user_camera;

			SetCameraDefaults();

			sceneName = System.IO.Path.GetFileName(f.FileName);

			string ext = System.IO.Path.GetExtension(f.FileName);

			switch (ext.ToLower())
			{
				case ".haddd":
				case ".x":
					if (!Jad.Import.Load(f.FileName))
						return;

					Jad.Video.Meshes.CenterAtOrigin();
					Jad.Scene.MeshObjects.ComputeTangent();

					break;
				case ".pfx":
					if (!LoadPFx(f.FileName))
						return;
					break;
				default:
					MessageBox.Show("Unknown file format", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
					break;
			}

			if (Jad.Scene.Lights.Count == 0 && ext.ToLower().CompareTo(".pfx") != 0)
			{
				// Frontal light
				JDirectLight light = (JDirectLight) Jad.Scene.Lights.Create("direct01", JLightType.Directional);
				light.Position = new Vector3(0.0f, 5.0f, -5.0f);
				light.Angles = new Vector3((float) (Math.PI / 4.0), 0.0f, 0.0f);
				//light.Angles = new Vector3(0.0f, 0.0f, 0.0f);
				light.Color = JColor.White;
				light.Multiplier = 1.0f;
				light.AffectSpecular = true;
				light.FarAttenuationStart = 10;
				light.FarAttenuationEnd = 100;

				// Rear light
				light = (JDirectLight) Jad.Scene.Lights.Create("direct02", JLightType.Directional);
				light.Position = new Vector3(0.0f, 5.0f, 5.0f);
				light.Angles = new Vector3((float) (Math.PI / 4.0), (float) (Math.PI), 0.0f);
				//light.Angles = new Vector3(0.0f, (float)(Math.PI), 0.0f);
				light.Color = JColor.White;
				light.Multiplier = 1.0f;
				light.AffectSpecular = false;
				light.FarAttenuationStart = 10;
				light.FarAttenuationEnd = 100;

				// Right light
				light = (JDirectLight) Jad.Scene.Lights.Create("direct03", JLightType.Directional);
				light.Position = new Vector3(-5.0f, 5.0f, 0.0f);
				//light.Angles = new Vector3((float)(Math.PI / 4.0), -(float)(Math.PI/2.0), 0.0f);
				light.Angles = new Vector3(0.0f, -(float) (Math.PI / 2.0), 0.0f);
				light.Color = JColor.White;
				light.Multiplier = 1.0f;
				light.AffectSpecular = false;
				light.FarAttenuationStart = 10;
				light.FarAttenuationEnd = 100;

				// Left light
				light = (JDirectLight) Jad.Scene.Lights.Create("direct04", JLightType.Directional);
				light.Position = new Vector3(5.0f, 5.0f, 0.0f);
				//light.Angles = new Vector3((float)(Math.PI / 4.0), (float)(Math.PI / 2.0), 0.0f);
				light.Angles = new Vector3(0.0f, (float) (Math.PI / 2.0), 0.0f);
				light.Color = JColor.White;
				light.Multiplier = 1.0f;
				light.AffectSpecular = false;
				light.FarAttenuationStart = 10;
				light.FarAttenuationEnd = 100;
			}

			Jad.Scene.MeshObjects.ComputeBoundingBox();

			if (ext.ToLower().CompareTo(".pfx") != 0)
				Jad.Scene.Camera.AdjustToBBox(Jad.Scene.MeshObjects.BoundingBox);

			Jad.Scene.MeshObjects.Sort(JSortMeshObjectBy.Name);

			UpdateObjectsTree();
			UpdateLightsTree();

			cbCameraPaths.Items.Clear();

			ChangeWindowCaption();

			InitializeScene();
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Close();
		}

		#endregion

		#region View Menu

		private void showCameraPosToolStripMenuItem_Click(object sender, EventArgs e)
		{
			showCameraPos = !showCameraPos;
		}

		private void showFPSToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			ShowFPS = !ShowFPS;
		}

		private void showNamesToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Jad.Scene.ShowNames = showNamesToolStripMenuItem.Checked;
		}

		private void showGizmosToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ShowGizmos = !ShowGizmos;

			Jad.Scene.ShowGizmos = ShowGizmos;
		}

		private void showAxesToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Jad.Scene.ShowAxes = showAxesToolStripMenuItem.Checked;
		}

		private void showGridToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ShowGrid = !ShowGrid;
		}

		private void pausedToolStripMenuItem_Click(object sender, EventArgs e)
		{
			paused = !paused;
		}

		#endregion

		#region Environment Menu

		private void attachlighttocameraToolStripMenuItem_Click(object sender, EventArgs e)
		{
			AttachLight();
		}

		private void pickModeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			pickMode = !pickMode;
			pickLightsMode = pickPFxMode = pickLightsToolStripMenuItem.Checked = pickParticleEffectsToolStripMenuItem.Checked = false;
			pickLightsToolStripMenuItem.Checked = false;
			
			if (pickMode)
				renderCtrl.Cursor = Cursors.Hand;
			else
				renderCtrl.Cursor = Cursors.Cross;
		}

		private void pickLightsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			pickLightsMode = !pickLightsMode;
			pickMode = pickPFxMode = pickModeToolStripMenuItem.Checked = pickParticleEffectsToolStripMenuItem.Checked = false;

			if (pickLightsMode)
				renderCtrl.Cursor = Cursors.Hand;
			else
				renderCtrl.Cursor = Cursors.Cross;
		}

		private void pickParticleEffectsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			pickPFxMode = !pickPFxMode;
			pickMode = pickLightsMode = pickModeToolStripMenuItem.Checked = pickLightsToolStripMenuItem.Checked = false;

			if (pickPFxMode)
				renderCtrl.Cursor = Cursors.Hand;
			else
				renderCtrl.Cursor = Cursors.Cross;
		}

		private void createEnvMapFromCameraToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog dlg = new SaveFileDialog();
			dlg.RestoreDirectory = true;
			dlg.Filter = "PNG format (*.png)|*.png";

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Jad.Scene.Begin();
				Jad.Scene.RenderCameraToCubeMap(Jad.Video.RenderTargets.Cube);
				Jad.Video.RenderTargets.Cube.Texture.Save(Path.GetDirectoryName(dlg.FileName) + "\\" + Path.GetFileNameWithoutExtension(dlg.FileName), ImageFileFormat.Png);
				Jad.Scene.End();
			}

			dlg.Dispose();
		}

		#endregion

		#region Help Menu

		private void aboutViewerv01ToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ViewerAbout dlg = new ViewerAbout();
			dlg.ShowDialog();
		}

		private void keysToolStripMenuItem_Click(object sender, EventArgs e)
		{
			MessageBox.Show("Click on the render area to enable 'camera mode'.\n" +
							"Press W to move forward.\n" +
							"Press S to move backward.\n" +
							"Press A to step left.\n" +
							"Press D to move step right.\n" +
							"Press right mouse button to move up.\n" +
							"Press left mouse button to move down.\n" +
							"Use mouse wheel to zoom in/out.\n" +
							"\nPress F12 to save snapshot\n" +
							"Press NumPad + to add bumpiness\n" +
							"Press NumPad - to substract bumpiness\n" +
							"\nPress ESC to go back to 'normal mode'.",
							"Keys", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}

		#endregion

		#region RenderControl Events

		private void renderCtrl_MouseWheel(object sender, MouseEventArgs e)
		{
			float v = 1f;

			// If Shift is pressed, go faster
			if ((Control.ModifierKeys & Keys.Shift) != 0)
				v *= 2f;

			if (e.Delta < 0)
				v = -v;

			DisplaceZ(v);

		}

		private void renderCtrl_MouseDown(object sender, MouseEventArgs e)
		{
			float x = (float) e.X;
			float y = (float) e.Y;

			mouseDownPoint.X = e.X;
			mouseDownPoint.Y = e.Y;

			mouseDown = true;

			mouseControlsCamera = true;
		}

		private void renderCtrl_MouseUp(object sender, MouseEventArgs e)
		{
			mouseDown = false;

			if (pickMode || pickLightsMode || pickPFxMode)
			{
				Vector3 p0 = Vector3.Empty, p1 = Vector3.Empty;
				JMath.ScreenToWorld(e.X, e.Y, ref p0, ref p1);

				if (pickMode)
				{
					JMeshObject m = Jad.Scene.MeshObjects.Intersect(p0, p1, true);
					if (m != null)
					{
						TreeNode[] nodes = treeObjects.Nodes.Find(m.ID.ToString(), false);
						treeObjects.Focus();
						treeObjects.SelectedNode = nodes[0];
					}
				}

				else if (pickLightsMode)
				{
					JBaseLight l = Jad.Scene.Lights.Intersect(p0, p1);
					if (l != null)
					{
						TreeNode[] nodes = treeLights.Nodes.Find(l.ID.ToString(), false);
						treeLights.Focus();
						treeLights.SelectedNode = nodes[0];
					}
				}

				else
				{
					JParticleEffect p = Jad.Scene.Particles.Intersect(p0, p1);
					if (p != null)
					{
						LoadPFx(p);
					}
				}
			}
		}

		private void renderCtrl_MouseMove(object sender, MouseEventArgs e)
		{
			if (!mouseDown)
				return;

			return;
		}

		private void renderCtrl_MouseClick(object sender, EventArgs e)
		{
			if (!paused)
			{
				renderCtrl.Focus();

				if (!(pickMode || pickLightsMode || pickPFxMode)) Jad.CreateInput();
			}
		}

		private void renderCtrl_SizeChanged(object sender, EventArgs e)
		{
			ChangeWindowCaption();
		}

		void ChangeWindowCaption()
		{
			string title = caption;

			if (sceneName != null) title += " [" + sceneName + "]";

			Text = title + string.Format(" ({0}x{1})", renderCtrl.Width, renderCtrl.Height);
		}

		#endregion

		#region Timer

		private void timerMotor_Tick(object sender, EventArgs e)
		{
			//return;
			if (Created)
			{
				if (Jad.Active)
				{
					Jad.BeginUpdate();

					// If false we want to exit the app
					bool continuar = jViewer.Action();

					Jad.EndUpdate();

					if (Jad.Active)
					{
						Jad.Scene.MotionBlur.Enabled = this.checkBoxMotionBlur.Checked;
						Jad.Scene.MotionBlur.Filter.PixelBlurConst = (float) this.numericUpDownMotionBlur.Value;
						Jad.Scene.MotionBlur.Filter.NumSamples = (int) this.numericUpDownNumSamples.Value;

						Jad.Scene.BloomEffect.Enabled = this.checkBoxBloom.Checked;
						Jad.Scene.BloomEffect.HighPass.HighPassValue = (float) this.numericUpDownBloomHighPass.Value;
						Jad.Scene.BloomEffect.HighPass.Multiplier = (float) this.numericBloomMultiply.Value;

						if (paused)
							jViewer.RenderLastFrame();
						else
							jViewer.Render();


						numCameraPosX.Value = new Decimal(Jad.Scene.Camera.Position.X);
						numCameraPosY.Value = new Decimal(Jad.Scene.Camera.Position.Y);
						numCameraPosZ.Value = new Decimal(Jad.Scene.Camera.Position.Z);

						numCameraRotX.Value = new Decimal(Geometry.RadianToDegree(Jad.Scene.Camera.Angles.X));
						numCameraRotY.Value = new Decimal(Geometry.RadianToDegree(Jad.Scene.Camera.Angles.Y));
						numCameraRotZ.Value = new Decimal(Geometry.RadianToDegree(Jad.Scene.Camera.Angles.Z));

						if (curPathAnim != null && !curPathAnim.Paused)
						{
							JPRFAnimationKey key = curPathAnim.AtTime(Jad.Timer.Time) as JPRFAnimationKey;

							Jad.Scene.Camera.Position = key.Position;
							Jad.Scene.Camera.Rotation = key.Rotation;
							Jad.Scene.Camera.FOV = key.FOV;

							numCameraPathFrame.Value = curPathAnim.CurrentKey;
						}
					}
				}

				else
				{
					//Application.DoEvents();
				}
			}
		}

		#endregion

		#region Channel/Animations

		private void chkRemoveOriginOffset_CheckedChanged(object sender, EventArgs e)
		{
		}

		private void chkChannel_CheckedChanged(object sender, EventArgs e)
		{
		}

		private void cb_SelectedValueChanged(object sender, EventArgs e)
		{
		}

		#endregion

		#region Objects Tree

		private void numericParallaxAmount_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.MeshObjects.ParallaxAmount = (float) numericParallaxAmount.Value;
		}

		private void numericBumpiness_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.MeshObjects.Bumpiness = (float) numericBumpiness.Value;
		}

		private void treeObjects_DoubleClick(object sender, EventArgs e)
		{
			JMeshObject mo = null;
			JMaterialLayer layer = null;
			ColorDialog dlg = null;

			TreeNode node = treeObjects.SelectedNode;

			if (node == null) return;

			if (node.Tag != null)
			{
				if (node.Tag is JMeshObject)
					mo = (JMeshObject) node.Tag;

				if (mo != null)
				{
					if (node.Name == "Hide")
					{
						mo.Hide = !mo.Hide;

						node.Text = "Hide:" + mo.Hide.ToString();
					}
					if (node.Name == "SBB")
					{
						mo.DebugRendering.ShowBox = !mo.DebugRendering.ShowBox;

						node.Text = "Show Bounding Box:" + mo.DebugRendering.ShowBox.ToString();
					}
					if (node.Name == "SBS")
					{
						mo.DebugRendering.ShowSphere = !mo.DebugRendering.ShowSphere;

						node.Text = "Show Bounding Sphere:" + mo.DebugRendering.ShowSphere.ToString();
					}
					if (node.Name == "SBBH")
					{
						mo.DebugRendering.ShowHierarchicalBox = !mo.DebugRendering.ShowHierarchicalBox;

						node.Text = "Show Hier. Bounding Box:" + mo.DebugRendering.ShowHierarchicalBox.ToString();
					}
					if (node.Name == "SBSH")
					{
						mo.DebugRendering.ShowHierarchicalSphere = !mo.DebugRendering.ShowHierarchicalSphere;

						node.Text = "Show Hier. Bounding Sphere:" + mo.DebugRendering.ShowHierarchicalSphere.ToString();
					}
					if (node.Name == "SHN")
					{
						mo.DebugRendering.ShowNormals = !mo.DebugRendering.ShowNormals;

						node.Text = "Show Normals:" + mo.DebugRendering.ShowNormals.ToString();
					}
					if (node.Name == "SHT")
					{
						mo.DebugRendering.ShowTangents = !mo.DebugRendering.ShowTangents;

						node.Text = "Show Tangents:" + mo.DebugRendering.ShowTangents.ToString();
					}

					if (node.Name == "SHA")
					{
						mo.DebugRendering.ShowAxes = !mo.DebugRendering.ShowAxes;

						node.Text = "Show Axes:" + mo.DebugRendering.ShowAxes.ToString();
					}
					if (node.Name == "CAST_SHADOWS")
					{
						mo.CastShadows = !mo.CastShadows;

						node.Text = "Cast Shadows:" + mo.CastShadows.ToString();
					}
					if (node.Name == "REC_SHADOWS")
					{
						mo.ReceiveShadows = !mo.ReceiveShadows;

						node.Text = "Receive Shadows:" + mo.ReceiveShadows.ToString();
					}
					if (node.Name == "AffectLight")
					{
						mo.AffectLight = !mo.AffectLight;

						node.Text = "AffectLight:" + mo.AffectLight.ToString();
					}
				}
				else
				{
					JMaterial mat = null;

					if (node.Tag is JMaterial)
						mat = (JMaterial) node.Tag;

					if (mat != null)
					{
						if (node.Name == "WSN")
						{
							mat.WorldSpaceNormals = !mat.WorldSpaceNormals;

							node.Text = "WorldSpaceNormals:" + mat.WorldSpaceNormals.ToString();
						}
					}

				}
				return;
			}

			// For the maps, instead of saving the tag in each node, it's saved
			// two levels higher, in the layer
			if (node.Parent != null)	// Diffuse example
			{
				if (node.Parent.Parent != null)	// Layer
				{
					if (node.Parent.Parent.Tag != null)
					{
						if (node.Parent.Parent.Tag is JMaterialLayer)
							layer = (JMaterialLayer) node.Parent.Parent.Tag;

						if (layer != null)
						{
							switch (node.Name)
							{
								case "Dif_E":

									layer.DiffuseMap.TextureEnabled = !layer.DiffuseMap.TextureEnabled;

									node.Text = "Map Enabled:" + layer.DiffuseMap.TextureEnabled.ToString();

									break;

								case "Dif_C":

									dlg = new ColorDialog();

									dlg.FullOpen = true;

									dlg.Color = layer.DiffuseMap.Color.ToSystemColor();

									if (dlg.ShowDialog() == DialogResult.OK)
									{
										layer.DiffuseMap.Color = new JColor(dlg.Color);


										node.Text = "Color:" + layer.DiffuseMap.Color.ToString();
									}

									break;

								case "Spe_E":

									layer.SpecularMap.Enabled = !layer.SpecularMap.Enabled;

									node.Text = "Enabled:" + layer.SpecularMap.Enabled.ToString();

									break;

								case "Nor_E":

									layer.NormalMap.Enabled = !layer.NormalMap.Enabled;

									node.Text = "Enabled:" + layer.NormalMap.Enabled.ToString();

									break;

								case "Emi_E":

									layer.EmissiveMap.Enabled = !layer.EmissiveMap.Enabled;

									node.Text = "Enabled:" + layer.EmissiveMap.Enabled.ToString();

									break;

								case "Emi_M":

									switch (layer.EmissiveMap.Mode)
									{
										case JEmissiveMode.Diffuse:
											layer.EmissiveMap.Mode = JEmissiveMode.Color;
											break;
										case JEmissiveMode.Color:
											layer.EmissiveMap.Mode = JEmissiveMode.Texture;
											break;
										case JEmissiveMode.Texture:
											layer.EmissiveMap.Mode = JEmissiveMode.Diffuse;
											break;
									}

									node.Text = "Mode:" + EmissiveModeToString(layer.EmissiveMap.Mode);

									break;

								case "Emi_C":

									dlg = new ColorDialog();

									dlg.FullOpen = true;

									dlg.Color = layer.EmissiveMap.Color.ToSystemColor();

									if (dlg.ShowDialog() == DialogResult.OK)
									{
										layer.EmissiveMap.Color = new JColor(dlg.Color);


										node.Text = "Color:" + layer.EmissiveMap.Color.ToString();
									}

									break;

								case "Height_E":

									layer.HeightMap.Enabled = !layer.HeightMap.Enabled;

									node.Text = "Enabled:" + layer.HeightMap.Enabled.ToString();

									break;
							}
						}
					}
				}
			}
		}

		void InsertMaterialStates(TreeNode parent, JMaterialStates states)
		{
			TreeNode detNode;
			TreeNode node = parent.Nodes.Add("States");
			detNode = node.Nodes.Add("CullMode:" + states.CullMode.ToString());
			detNode = node.Nodes.Add("DepthBias:" + states.DepthBias.ToString());
			detNode = node.Nodes.Add("FillMode:" + states.FillMode.ToString());
			detNode = node.Nodes.Add("ShadeMode:" + states.ShadeMode.ToString());
		}

		void InsertMaterialLayerStates(TreeNode parent, JMaterialLayerStates states)
		{
			TreeNode detNode;
			TreeNode node = parent.Nodes.Add("States");
			detNode = node.Nodes.Add("AlphaBlendEnable:" + states.AlphaBlendEnable.ToString());
			detNode = node.Nodes.Add("AlphaFunction:" + states.AlphaFunction.ToString());
			detNode = node.Nodes.Add("AlphaTestEnable:" + states.AlphaTestEnable.ToString());
			detNode = node.Nodes.Add("BlendOperation:" + states.BlendOperation.ToString());
			detNode = node.Nodes.Add("DestinationBlend:" + states.DestinationBlend.ToString());
			detNode = node.Nodes.Add("ReferenceAlpha:" + states.ReferenceAlpha.ToString());
			detNode = node.Nodes.Add("SourceBlend:" + states.SourceBlend.ToString());
		}

		void InsertMaterialLayerSamplerStates(string mapName, TreeNode parent, JSamplerVideoStates states)
		{
			TreeNode detNode;
			TreeNode node = parent.Nodes.Add("States");

			detNode = node.Nodes.Add(mapName + "_AddressU", "AddressU:" + states.AddressU.ToString());
			detNode = node.Nodes.Add(mapName + "_AddressV", "AddressV:" + states.AddressV.ToString());
			detNode = node.Nodes.Add(mapName + "_AddressW", "AddressW:" + states.AddressW.ToString());
			detNode = node.Nodes.Add(mapName + "_BorderColor", "BorderColor:" + states.BorderColor.ToString());
			detNode = node.Nodes.Add(mapName + "_DMapOffset", "DMapOffset:" + states.DMapOffset.ToString());
			detNode = node.Nodes.Add(mapName + "_ElementIndex", "ElementIndex:" + states.ElementIndex.ToString());
			detNode = node.Nodes.Add(mapName + "_MinFilter", "MinFilter:" + states.MinFilter.ToString());
			detNode = node.Nodes.Add(mapName + "_MagFilter", "MagFilter:" + states.MagFilter.ToString());
			detNode = node.Nodes.Add(mapName + "_MipFilter", "MipFilter:" + states.MipFilter.ToString());
			detNode = node.Nodes.Add(mapName + "_MaxAnisotropy", "MaxAnisotropy:" + states.MaxAnisotropy.ToString());
			detNode = node.Nodes.Add(mapName + "_MaxMipLevel", "MaxMipLevel:" + states.MaxMipLevel.ToString());
			detNode = node.Nodes.Add(mapName + "_MipMapLevelOfDetailBias", "MipMapLevelOfDetailBias:" + states.MipMapLevelOfDetailBias.ToString("N2"));
			detNode = node.Nodes.Add(mapName + "_SrgbTexture", "SrgbTexture:" + states.SrgbTexture.ToString());
		}

		void UpdateObjectsTree()
		{
			// Fill the list of the tree

			treeObjects.Nodes.Clear();

			int totalFaces = 0;

			foreach (JMeshObject mo in Jad.Scene.MeshObjects.List)
			{
				// 0 indicates it's an object

				int numFaces = mo.Mesh.IndexBuffer.NumberOfIndices / 3;

				totalFaces += numFaces;

				TreeNode node = treeObjects.Nodes.Add(mo.ID.ToString(), mo.Name + " (" + numFaces.ToString("N0") + " faces)");

				node.Tag = mo;

				TreeNode objectNode = node.Nodes.Add("ID", "ID:" + mo.ID.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("Hide", "Hide:" + mo.Hide.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SBB", "Show Bounding Box:" + mo.DebugRendering.ShowBox.ToString());

				objectNode = node.Nodes.Add("Bounding Box:" + JMath.Vector3ToString(mo.BoundingBox.Min) + " - " + JMath.Vector3ToString(mo.BoundingBox.Max));

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SBS", "Show Bounding Sphere:" + mo.DebugRendering.ShowSphere.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SBBH", "Show Hier. Bounding Box:" + mo.DebugRendering.ShowHierarchicalBox.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SBSH", "Show Hier. Bounding Sphere:" + mo.DebugRendering.ShowHierarchicalSphere.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SHN", "Show Normals:" + mo.DebugRendering.ShowNormals.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SHT", "Show Tangents:" + mo.DebugRendering.ShowTangents.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("SHA", "Show Axes:" + mo.DebugRendering.ShowAxes.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("CAST_SHADOWS", "Cast Shadows:" + mo.CastShadows.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("REC_SHADOWS", "Receive Shadows:" + mo.ReceiveShadows.ToString());

				objectNode.Tag = mo;

				objectNode = node.Nodes.Add("AffectLight", "AffectLight:" + mo.AffectLight.ToString());

				objectNode.Tag = mo;

				for (int i = 0; i < mo.Material.Length; i++)
				{
					JMaterial mat = mo.Material[i];

					string key = "M" + mo.ID.ToString() + "-" + i.ToString();

					// It seems mat.Name is null...
					// M indicates it's a material
					TreeNode matNode = node.Nodes.Add(key, "Mat #" + i.ToString() + mat.MapsActiveToString());

					matNode.Tag = mat;

					TreeNode matNode2 = matNode.Nodes.Add("Priority:" + mat.Priority.ToString());

					matNode2.Tag = mat;

					matNode2 = matNode.Nodes.Add("WSN", "WorldSpaceNormals:" + mat.WorldSpaceNormals.ToString());

					matNode2.Tag = mat;

					for (int l = 0; l < mat.Layers.Count; l++)
					{
						JMaterialLayer layer = mat.Layers[l];

						key += ";Layer:" + l.ToString();

						TreeNode layerNode = matNode.Nodes.Add(key, "Layer #" + l.ToString() + layer.MapsActiveToString());

						layerNode.Tag = layer;

						TreeNode mapNode, mapDetNode;

						// Material data

						mapNode = layerNode.Nodes.Add("Normal Type:" + NormalTypeToString(layer.NormalType));

						// Diffuse

						mapNode = layerNode.Nodes.Add("Diffuse");

						if (!layer.DiffuseMap.TextureEnabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Dif_E", "Map Enabled:" + layer.DiffuseMap.TextureEnabled.ToString());

						if (layer.DiffuseMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.DiffuseMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("Dif_C", "Color:" + layer.DiffuseMap.Color.ToString());
						//mapDetNode = mapNode.Nodes.Add("U,V tiling:" + layer.DiffuseMap.UTile.ToString("N2", Jad.DecimalFormatInfo) + "," + layer.DiffuseMap.VTile.ToString("N2"));

						InsertMaterialLayerSamplerStates("Diffuse", mapNode, layer.DiffuseMap.SamplerStates);

						// Specular

						mapNode = layerNode.Nodes.Add("Specular");

						if (!layer.SpecularMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Spe_E", "Enabled:" + layer.SpecularMap.Enabled.ToString());

						if (layer.SpecularMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.SpecularMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("Color:" + layer.SpecularMap.Color.ToString());

						mapDetNode = mapNode.Nodes.Add("SPE_Power", "Power:" + layer.SpecularMap.Power.ToString("N2", Jad.DecimalFormatInfo));

						mapDetNode.ForeColor = Color.Blue;

						InsertMaterialLayerSamplerStates("Specular", mapNode, layer.SpecularMap.SamplerStates);

						// NormalMap

						mapNode = layerNode.Nodes.Add("NormalMap");

						if (!layer.NormalMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Nor_E", "Enabled:" + layer.NormalMap.Enabled.ToString());

						if (layer.NormalMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.NormalMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("Nor_Bumpy", "Bumpiness:" + layer.NormalMap.Bumpiness.ToString("N2", Jad.DecimalFormatInfo));

						mapDetNode.ForeColor = Color.Blue;

						InsertMaterialLayerSamplerStates("NormalMap", mapNode, layer.NormalMap.SamplerStates);

						// Emissive

						mapNode = layerNode.Nodes.Add("Emissive");

						if (!layer.EmissiveMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Emi_E", "Enabled:" + layer.EmissiveMap.Enabled.ToString());

						if (layer.EmissiveMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.EmissiveMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("Emi_M", "Mode:" + EmissiveModeToString(layer.EmissiveMap.Mode));

						mapDetNode = mapNode.Nodes.Add("Emi_C", "Color:" + layer.EmissiveMap.Color.ToString());
						mapDetNode = mapNode.Nodes.Add("Emi_I", "Intensity:" + layer.EmissiveMap.Intensity.ToString("N2", Jad.DecimalFormatInfo));

						mapDetNode.ForeColor = Color.Blue;

						InsertMaterialLayerSamplerStates("Emissive", mapNode, layer.EmissiveMap.SamplerStates);

						// ReflectionMap

						mapNode = layerNode.Nodes.Add("ReflectionMap");

						if (!layer.ReflectionMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Refl_E", "Enabled:" + layer.ReflectionMap.Enabled.ToString());

						if (layer.ReflectionMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.ReflectionMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("Reflectivity:" + layer.ReflectionMap.Reflectivity.ToString("N2", Jad.DecimalFormatInfo));

						InsertMaterialLayerSamplerStates("ReflectionMap", mapNode, layer.ReflectionMap.SamplerStates);

						// RefractionMap

						mapNode = layerNode.Nodes.Add("Refr_E", "RefractionMap");

						if (!layer.RefractionMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Enabled:" + layer.RefractionMap.Enabled.ToString());

						if (layer.RefractionMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.RefractionMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("IndexOfRefraction:" + layer.RefractionMap.IndexOfRefraction.ToString("N2", Jad.DecimalFormatInfo));

						mapDetNode = mapNode.Nodes.Add("Transmittance:" + layer.RefractionMap.Transmittance.ToString("N2", Jad.DecimalFormatInfo));

						InsertMaterialLayerSamplerStates("RefractionMap", mapNode, layer.RefractionMap.SamplerStates);

						// HeightMap

						mapNode = layerNode.Nodes.Add("HeightMap");

						if (!layer.HeightMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Height_E", "Enabled:" + layer.HeightMap.Enabled.ToString());

						if (layer.HeightMap.Texture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.HeightMap.Texture.Name);

						mapDetNode = mapNode.Nodes.Add("Height_A", "Amount:" + layer.HeightMap.ParallaxAmount.ToString("N2", Jad.DecimalFormatInfo));

						mapDetNode.ForeColor = Color.Blue;

						InsertMaterialLayerSamplerStates("HeightMap", mapNode, layer.HeightMap.SamplerStates);

						// DetailMap

						mapNode = layerNode.Nodes.Add("DetailMap");

						if (!layer.DetailMap.Enabled)
							mapNode.ForeColor = Color.Gray;

						mapDetNode = mapNode.Nodes.Add("Detail_E", "Enabled:" + layer.DetailMap.Enabled.ToString());

						if (layer.DetailMap.MaskTexture != null)
							mapDetNode = mapNode.Nodes.Add("Texture:" + layer.DetailMap.MaskTexture.Name);

						mapDetNode = mapNode.Nodes.Add("Detail_A", "Amount:" + layer.DetailMap.Amount.ToString("N2", Jad.DecimalFormatInfo));

						mapDetNode.ForeColor = Color.Blue;

						InsertMaterialLayerSamplerStates("DetailMap", mapNode, layer.DetailMap.SamplerStates);

						InsertMaterialLayerStates(layerNode, layer.RenderStates);
					}

					InsertMaterialStates(matNode, mat.RenderStates);

				}

			}

			if (totalFaces > 0)
				tabObjects.Text = "Objects (" + totalFaces.ToString("N0") + " faces)";
			else
				tabObjects.Text = "Objects";
		}

		private void treeObjects_KeyDown(object sender, KeyEventArgs e)
		{
			float inc = 0.0f;

			if (e.KeyCode == Keys.Add)
				inc = 0.1f;
			if (e.KeyCode == Keys.Subtract)
				inc = -0.1f;

			if (inc == 0) return;

			// If Shift is pressed move faster
			if ((e.Modifiers & Keys.Shift) != 0) inc *= 4;

			// If Alt is pressed move slower
			if ((e.Modifiers & Keys.Alt) != 0) inc *= 0.5f;

			JMaterialLayer layer = null;

			TreeNode node = treeObjects.SelectedNode;

			if (node.Parent != null)	// Diffuse
			{
				if (node.Parent.Parent != null)	// Layer
				{
					if (node.Parent.Parent.Tag != null)
					{
						if (node.Parent.Parent.Tag is JMaterialLayer)
							layer = (JMaterialLayer) node.Parent.Parent.Tag;

						if (layer != null)
						{
							switch (node.Name)
							{

								case "SPE_Power":

									layer.SpecularMap.Power += inc * 10;

									node.Text = "Power:" + layer.SpecularMap.Power.ToString("N2", Jad.DecimalFormatInfo);

									break;

								case "Nor_Bumpy":

									layer.NormalMap.Bumpiness += inc;

									node.Text = "Bumpiness:" + layer.NormalMap.Bumpiness.ToString("N2", Jad.DecimalFormatInfo);

									break;

								case "Emi_I":

									layer.EmissiveMap.Intensity += inc;
									node.Text = "Intensity:" + layer.EmissiveMap.Intensity.ToString("N2", Jad.DecimalFormatInfo);

									break;

								case "Height_A":

									layer.HeightMap.ParallaxAmount += inc * 0.1f;
									node.Text = "Amount:" + layer.HeightMap.ParallaxAmount.ToString("N2", Jad.DecimalFormatInfo);

									break;

								case "Detail_A":

									layer.DetailMap.Amount += inc;
									node.Text = "Amount:" + layer.DetailMap.Amount.ToString("N2", Jad.DecimalFormatInfo);

									break;
							}
						}
					}
				}
			}
		}

		#endregion

		#region Lights Tree

		private void btnCreateLight_Click(object sender, EventArgs e)
		{
			ViewerLight frm = new ViewerLight();

			if (frm.ShowDialog() == DialogResult.OK)
			{
				this.UpdateLightsTree();
			}

			frm.Dispose();
		}

		private void btnRemoveLight_Click(object sender, EventArgs e)
		{
			TreeNode node = treeLights.SelectedNode;

			if (node == null)
			{
				MessageBox.Show("You must select a light first", "Remove light", MessageBoxButtons.OK, MessageBoxIcon.Information);
				return;
			}

			JBaseLight light = (JBaseLight) node.Tag;

			if (light == null)
			{
				MessageBox.Show("No light selected", "Remove light", MessageBoxButtons.OK, MessageBoxIcon.Information);
				return;
			}

			if (MessageBox.Show("Are you sure you want to delete light \"" + light.Name + "\"?", "Remove light", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation) != DialogResult.Yes)
				return;

			Jad.Scene.Root.RemoveChild(light, true);
			Jad.Scene.Lights.Delete(light);

			UpdateLightsTree();
		}

		private void btnEdit_Click(object sender, EventArgs e)
		{
			TreeNode node = treeLights.SelectedNode;

			if (node == null)
			{
				MessageBox.Show("You must select a light first", "Edit light", MessageBoxButtons.OK, MessageBoxIcon.Information);
				return;
			}

			JBaseLight light = (JBaseLight) node.Tag;

			if (light == null)
			{
				MessageBox.Show("No light selected", "Edit light", MessageBoxButtons.OK, MessageBoxIcon.Information);
				return;
			}

			ViewerLight frm = new ViewerLight();

			frm.EditMode = true;
			frm.LightToEdit = light;

			if (frm.ShowDialog() == DialogResult.OK)
			{
				this.UpdateLightsTree();
			}

			frm.Dispose();
		}

		private void treeLights_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Delete)
			{
				this.btnRemoveLight_Click(null, null);
				return;
			}

			// Test if keys + or - are pressed
			float inc = 0;

			if (e.KeyCode == Keys.Add) inc = 1;
			if (e.KeyCode == Keys.Subtract) inc = -1;

			if (inc == 0) return;

			// Shift moves faster
			if ((e.Modifiers & Keys.Shift) != 0) inc *= 4;

			// Alt moves slower
			if ((e.Modifiers & Keys.Alt) != 0) inc *= 0.5f;

			TreeNode node = treeLights.SelectedNode;

			if (node == null)
				return;

			TreeNode parent = node.Parent;

			if (parent == null)
				return;

			JBaseLight light = (JBaseLight) parent.Tag;
			JSpotLight spot = null;

			if (light.Type == JLightType.Spot || light.Type == JLightType.Directional)
				spot = (JSpotLight) light;

			if (node.Name == "I")
			{
				light.Multiplier += 0.1f * inc;
				node.Text = "Intensity:" + light.Multiplier.ToString("N2", Jad.DecimalFormatInfo);
			}
			if (node.Name == "FS")
			{
				light.FarAttenuationStart += 0.1f * inc;
				node.Text = "FarAttenuationStart:" + light.FarAttenuationStart.ToString("N2", Jad.DecimalFormatInfo);
			}
			if (node.Name == "FE")
			{
				light.FarAttenuationEnd += 0.1f * inc;
				node.Text = "FarAttenuationEnd:" + light.FarAttenuationEnd.ToString("N2", Jad.DecimalFormatInfo);
			}
			if (node.Name == "VOP")
			{
				light.Volumetric.PowValue += 0.1f * inc;
				node.Text = "Volumetric Power:" + light.Volumetric.PowValue.ToString("N2", Jad.DecimalFormatInfo);
			}
			if (node.Name == "HS" && spot != null)
			{
				spot.HotSpot += 0.1f * inc;
				node.Text = "HotSpot:" + spot.HotSpot.ToString("N2", Jad.DecimalFormatInfo);
			}
			if (node.Name == "FA" && spot != null)
			{
				spot.FallOff += 0.1f * inc;
				node.Text = "FallOff:" + spot.FallOff.ToString("N2", Jad.DecimalFormatInfo);
			}
		}

		private void treeLights_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			TreeNode node = treeLights.SelectedNode;

			if (node == null)
				return;

			TreeNode parent = node.Parent;

			if (parent == null)
				return;

			JBaseLight light = (JBaseLight) parent.Tag;
			JSpotLight spot = null;

			if (light.Type == JLightType.Spot || light.Type == JLightType.Directional)
				spot = (JSpotLight) light;

			if (node.Name == "C")
			{
				ColorDialog dlg = new ColorDialog();

				dlg.FullOpen = true;

				dlg.Color = light.Color.ToSystemColor();

				if (dlg.ShowDialog() == DialogResult.OK)
				{
					light.Color = new JColor(dlg.Color);

					node.Text = "Color:" + light.Color.ToString();

				}
			}

			if (node.Name == "E")
			{
				light.Hide = !light.Hide;

				node.Text = "Hide:" + light.Hide.ToString();
			}
			if (node.Name == "SH")
			{
				light.CastShadows = !light.CastShadows;

				node.Text = "Cast Shadows:" + light.CastShadows.ToString();
			}
			if (node.Name == "VO")
			{
				light.Volumetric.Enabled = !light.Volumetric.Enabled;

				node.Text = "Volumetric:" + light.Volumetric.Enabled.ToString();
			}

			if (node.Name == "D")
			{
				light.AffectDiffuse = !light.AffectDiffuse;

				node.Text = "Affect Diffuse:" + light.AffectDiffuse.ToString();
			}
			if (node.Name == "S")
			{
				light.AffectSpecular = !light.AffectSpecular;

				node.Text = "Affect Specular:" + light.AffectSpecular.ToString();
			}
			if (node.Name == "ATT")
			{
				if (light.AttenuationType == JLightAttenuationType.DualRadius)
					light.AttenuationType = JLightAttenuationType.None;
				else
					light.AttenuationType = (JLightAttenuationType) ((int) light.AttenuationType + 1);

				node.Text = "Attenuation:" + AttenuationToString(light.AttenuationType);
			}

			if (node.Name == "P")	// Position
			{
				renderCtrl.Focus();

				Jad.CreateInput();

				JCameraFirstPerson camera = (JCameraFirstPerson) Jad.Scene.Camera;

				// Only move the light, not the camera
				camera.UseInput = false;

				NodeSelected = light;

				TreeNodeSelected = node;
			}


			// Spot

			if (node.Name == "PME")
			{
				spot.Projector.Enabled = !spot.Projector.Enabled;

				node.Text = "Projector Enabled:" + spot.Projector.Enabled.ToString();
			}

			if (node.Name == "SHAFTE")
			{
				spot.Shafts.Enabled = !spot.Shafts.Enabled;

				node.Text = "Shafts Enabled:" + spot.Shafts.Enabled.ToString();
			}
		}

		void UpdateLightsTree()
		{
			// Sort the lights

			Jad.Scene.Lights.SortByName();

			// Fill the tree list

			TreeView tree = treeLights;

			tree.Nodes.Clear();

			foreach (JBaseLight light in Jad.Scene.Lights.List)
			{
				TreeNode node = tree.Nodes.Add(light.ID.ToString(), light.Name);

				node.Tag = light;

				TreeNode matNode = node.Nodes.Add("ID:" + light.ID.ToString());

				matNode = node.Nodes.Add("Type:" + LightTypeToString(light.Type));

				if (!light.Hide)
					matNode.ForeColor = Color.Gray;

				matNode = node.Nodes.Add("E", "Hide:" + light.Hide.ToString());

				matNode = node.Nodes.Add("SH", "Cast Shadows:" + light.CastShadows.ToString());

				// Directional lights don't have position
				if (light.Type != JLightType.Directional)
					matNode = node.Nodes.Add("P", "Pos:" + JMath.Vector3ToString(light.Position));

				matNode = node.Nodes.Add("A", "Angles:" + JMath.Vector3ToString(light.Angles));

				matNode = node.Nodes.Add("C", "Color:" + light.Color.ToString());

				matNode = node.Nodes.Add("I", "Intensity:" + light.Multiplier.ToString("N2", Jad.DecimalFormatInfo));

				matNode.ForeColor = Color.Blue;

				matNode = node.Nodes.Add("D", "Affect Diffuse:" + light.AffectDiffuse.ToString());

				matNode = node.Nodes.Add("S", "Affect Specular:" + light.AffectSpecular.ToString());

				matNode = node.Nodes.Add("ATT", "Attenuation:" + AttenuationToString(light.AttenuationType));

				matNode = node.Nodes.Add("FS", "FarAttenuationStart:" + light.FarAttenuationStart.ToString("N2", Jad.DecimalFormatInfo));

				matNode.ForeColor = Color.Blue;

				matNode = node.Nodes.Add("FE", "FarAttenuationEnd:" + light.FarAttenuationEnd.ToString("N2", Jad.DecimalFormatInfo));

				matNode.ForeColor = Color.Blue;

				matNode = node.Nodes.Add("VO", "Volumetric:" + light.Volumetric.Enabled.ToString());

				matNode = node.Nodes.Add("VOP", "Volumetric Power:" + light.Volumetric.PowValue.ToString("N2", Jad.DecimalFormatInfo));

				matNode.ForeColor = Color.Blue;

				if (light.Type == JLightType.Spot || light.Type == JLightType.Directional)
				{
					JSpotLight spot = light as JSpotLight;

					matNode = node.Nodes.Add("HS", "HotSpot:" + spot.HotSpot.ToString("N2", Jad.DecimalFormatInfo));
					matNode = node.Nodes.Add("FA", "FallOff:" + spot.FallOff.ToString("N2", Jad.DecimalFormatInfo));

					matNode = node.Nodes.Add("PME", "Projector Enabled:" + spot.Projector.Enabled.ToString());
					matNode = node.Nodes.Add("PM", "ProjectorMap:" + (spot.Projector.Map == null ? "" : spot.Projector.Map.Name));

					matNode = node.Nodes.Add("SHAFTE", "Shafts Enabled:" + spot.Shafts.Enabled.ToString());
					matNode = node.Nodes.Add("SHAFTM", "ShaftsMap:" + (spot.Shafts.CookieTexture == null ? "" : spot.Shafts.CookieTexture.Name));
					matNode = node.Nodes.Add("SHAFTP", "Shafts Planes:" + spot.Shafts.NumPlanes.ToString());
				}
			}

			tabLights.Text = "Lights (" + Jad.Scene.Lights.Count.ToString("N0") + ")";
		}
		
		#endregion

		#region Playback

		private void btnPlay_Click(object sender, EventArgs e)
		{
		}

		private void btnStop_Click(object sender, EventArgs e)
		{
		}

		private void btnStart_Click(object sender, EventArgs e)
		{
		}

		private void btnEnd_Click(object sender, EventArgs e)
		{
		}

		private void numFrame_ValueChanged(object sender, EventArgs e)
		{
		}

		private void tbFrame_ValueChanged(object sender, EventArgs e)
		{
		}

		private void numFrameRate_ValueChanged(object sender, EventArgs e)
		{
		}

		#endregion

		#region Split Container

		private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
		{
			ResizeRender();

			Jad.Video.Adapter.Reset();
		}

		#endregion

		#region Methods

		string NormalTypeToString(JNormalType normalType)
		{
			switch (normalType)
			{
				case JNormalType.None:
					return "None";

				case JNormalType.Mapping:
					return "Mapping";

				case JNormalType.Vertex:
					return "Vertex";
			}

			return "??";
		}

		string LightTypeToString(JLightType lightType)
		{
			switch (lightType)
			{
				case JLightType.Directional:
					return "Directional";

				case JLightType.Omni:
					return "Omni";

				case JLightType.Spot:
					return "Spot";

				case JLightType.Unknown:
					return "Unknown";
			}
			return "??";
		}

		string AttenuationToString(JLightAttenuationType attType)
		{
			switch (attType)
			{
				case JLightAttenuationType.None:
					return "None";

				case JLightAttenuationType.DualRadius:
					return "DualRadius";

				case JLightAttenuationType.Inverse:
					return "Inverse";

				case JLightAttenuationType.InverseSquare:
					return "InverseSquare";
			}
			return "??";
		}

		string EmissiveModeToString(JEmissiveMode mode)
		{
			switch (mode)
			{
				case JEmissiveMode.Color:
					return "Color";

				case JEmissiveMode.Diffuse:
					return "Diffuse";

				case JEmissiveMode.Texture:
					return "Texture";
			}
			return "??";
		}

		void AttachLight()
		{
			// Flashlight
			flashLight = (JSpotLight) Jad.Scene.Lights.Create("flashlight", JLightType.Spot);
			flashLight.Position = new Vector3(1.0f, 0.0f, 0.0f);
			flashLight.Angles = new Vector3(Geometry.DegreeToRadian(2.0f), Geometry.DegreeToRadian(-2.0f), 0.0f);
			flashLight.Pivot = new Vector3(0.0f, 0.0f, 0.0f);
			flashLight.Multiplier = 1.0f;
			flashLight.Color = new JColor(1.0f, 1.0f, 1.0f, 1.0f);
			flashLight.AffectDiffuse = true;
			flashLight.AffectSpecular = true;
			flashLight.AttenuationType = JLightAttenuationType.DualRadius;
			flashLight.FarAttenuationStart = 10.0f;
			flashLight.FarAttenuationEnd = 100.0f;
			flashLight.HotSpot = 25.0f;
			flashLight.FallOff = 30.0f;
			//flashLight.ProjectorMap = Jad.Video.Textures.Create("flashlight2", HTextureType.Texture2D, true);
			flashLight.Attach(Jad.Scene.Camera);

			UpdateLightsTree();
		}

		/// <summary>
		/// Sets the render size fitting the screen
		/// </summary>
		public void ResizeRender()
		{
			Size s = renderCtrl.Size;

			// Modify the params to indicate the size change
			Jad.Video.Adapter.Initialization.PresentParams.BackBufferWidth = s.Width;
			Jad.Video.Adapter.Initialization.PresentParams.BackBufferHeight = s.Height;
		}

		private void DisplaceZ(float inc)
		{
			Vector3 look = Jad.Scene.Camera.Look;

			look *= inc;

			Jad.Scene.Camera.Position += look;
		}

		#endregion

		#region Camera

		private void SetCameraDefaults()
		{
			numCameraSpeed.Value = new Decimal(((JCameraFirstPerson) Jad.Scene.Camera).Speed);
			numCameraSensitivity.Value = new Decimal(((JCameraFirstPerson) Jad.Scene.Camera).Sensitivity);
			numCameraFov.Value = new Decimal(Geometry.RadianToDegree(Jad.Scene.Camera.FOV));

			numCameraDOFNearBlurDepth.Value = new Decimal(Jad.Scene.DepthOfField.NearBlurDepth);
			numCameraDOFFarBlurDepth.Value = new Decimal(Jad.Scene.DepthOfField.FarBlurDepth);
			numCameraDOFNearFocalDepthPlane.Value = new Decimal(Jad.Scene.DepthOfField.NearFocalPlaneDepth);
			numCameraDOFFarFocalDepthPlane.Value = new Decimal(Jad.Scene.DepthOfField.FarFocalPlaneDepth);
			numCameraDOFMaxBlur.Value = new Decimal(Jad.Scene.DepthOfField.MaxBlur);
		}

		private void SetCameraKeyAtFrame(int frame)
		{
			if (curPathAnim == null)
				return;

			float time = (float) frame / (float) curPathAnim.FrameRate;
			JPRFAnimationKey key = curPathAnim.AtTime(time) as JPRFAnimationKey;

			Jad.Scene.Camera.Position = key.Position;
			Jad.Scene.Camera.Rotation = key.Rotation;
			Jad.Scene.Camera.FOV = key.FOV;
		}

		private void numCameraPosX_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.TranslateX((float) numCameraPosX.Value);
		}

		private void numCameraPosY_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.TranslateY((float) numCameraPosY.Value);
		}

		private void numCameraPosZ_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.TranslateZ((float) numCameraPosZ.Value);
		}

		private void numCameraRotX_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.RotateX(Geometry.DegreeToRadian((float) numCameraRotX.Value));
		}

		private void numCameraRotY_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.RotateY(Geometry.DegreeToRadian((float) numCameraRotY.Value));
		}

		private void numCameraRotZ_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.RotateZ(Geometry.DegreeToRadian((float) numCameraRotZ.Value));
		}

		private void numSpeed_ValueChanged(object sender, EventArgs e)
		{
			((JCameraFirstPerson) Jad.Scene.Camera).Speed = (float) numCameraSpeed.Value;
		}

		private void numSensitivity_ValueChanged(object sender, EventArgs e)
		{
			((JCameraFirstPerson) Jad.Scene.Camera).Sensitivity = (float) numCameraSensitivity.Value;
		}

		private void numFov_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Camera.FOV = Geometry.DegreeToRadian((float) numCameraFov.Value);
		}

		private void chkDOFEnabled_CheckedChanged(object sender, EventArgs e)
		{
			Jad.Scene.DepthOfField.Enabled = chkCameraDOFEnabled.Checked;
		}

		private void numNearBlurDepth_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.DepthOfField.NearBlurDepth = (float) numCameraDOFNearBlurDepth.Value;
		}

		private void numFarBlurDepth_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.DepthOfField.FarBlurDepth = (float) numCameraDOFFarBlurDepth.Value;
		}

		private void numNearFocalDepthPlane_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.DepthOfField.NearFocalPlaneDepth = (float) numCameraDOFNearFocalDepthPlane.Value;
		}

		private void numFarFocalDepthPlane_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.DepthOfField.FarFocalPlaneDepth = (float) numCameraDOFFarFocalDepthPlane.Value;
		}

		private void numMaxBlur_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.DepthOfField.MaxBlur = (float) numCameraDOFMaxBlur.Value;
		}

		private void cbCameraPaths_SelectedIndexChanged(object sender, EventArgs e)
		{
			curPathAnim = cbCameraPaths.SelectedItem as JCameraPathAnimator;

			numCameraPathFrame.Value = 0;
			numCameraPathFrame.Maximum = curPathAnim.NumberOfKeys - 1;
			numCameraPathFrameRate.Value = curPathAnim.FrameRate;
			tbCameraPathFrame.Value = 0;
			tbCameraPathFrame.Maximum = curPathAnim.NumberOfKeys - 1;

			curPathAnim.Pause = true;
		}

		private void btnCameraAddPath_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();

			dlg = new OpenFileDialog();
			dlg.RestoreDirectory = true;
			dlg.Filter = "Camera files (*.camera)|*.camera|All files (*.*)|*.*";
			dlg.FilterIndex = 0;

			if (dlg.ShowDialog() == DialogResult.Cancel)
				return;

			string fileName = System.IO.Path.GetFileNameWithoutExtension(dlg.FileName);

			// The path exists?
			if (cbCameraPaths.FindString(fileName) >= 0)
				return;

			cbCameraPaths.Items.Add(new JCameraPathAnimator(dlg.FileName, true));

			cbCameraPaths.SelectedIndex = cbCameraPaths.Items.Count - 1;
		}

		private void btnCameraDelPath_Click(object sender, EventArgs e)
		{
			if (cbCameraPaths.SelectedIndex >= 0)
			{
				cbCameraPaths.Items.RemoveAt(cbCameraPaths.SelectedIndex);

				curPathAnim = null;

				numCameraPathFrame.Value = 0;
				numCameraPathFrame.Maximum = 5;
				tbCameraPathFrame.Value = 0;
				tbCameraPathFrame.Maximum = 5;
			}
			if (cbCameraPaths.Items.Count > 0)
				cbCameraPaths.SelectedIndex = 0;
		}

		private void numCameraPathFrame_ValueChanged(object sender, EventArgs e)
		{
			tbCameraPathFrame.Value = (int) numCameraPathFrame.Value;
		}

		private void numCameraPathFrameRate_ValueChanged(object sender, EventArgs e)
		{
			if (curPathAnim != null)
				curPathAnim.FrameRate = (int) numCameraPathFrameRate.Value;
		}

		private void tbCameraPathFrame_ValueChanged(object sender, EventArgs e)
		{
			if (curPathAnim == null)
				return;

			if (curPathAnim.Pause)
			{
				SetCameraKeyAtFrame(tbCameraPathFrame.Value);

				numCameraPathFrame.Value = tbCameraPathFrame.Value;
			}
		}

		private void btnCameraPathStart_Click(object sender, EventArgs e)
		{
			if (curPathAnim == null)
				return;

			curPathAnim.Pause = true;

			SetCameraKeyAtFrame(0);

			numCameraPathFrame.Value = 0;
		}

		private void btnCameraPathPlay_Click(object sender, EventArgs e)
		{
			if (curPathAnim == null)
				return;

			curPathAnim.Pause = false;
		}

		private void btnCameraPathStop_Click(object sender, EventArgs e)
		{
			if (curPathAnim == null)
				return;

			curPathAnim.Pause = true;
		}

		private void btnCameraPathEnd_Click(object sender, EventArgs e)
		{
			if (curPathAnim == null)
				return;

			curPathAnim.Pause = true;

			SetCameraKeyAtFrame(curPathAnim.NumberOfKeys - 1);

			numCameraPathFrame.Value = curPathAnim.NumberOfKeys - 1;
		}

		#endregion

		#region Scene Tab

		private void numericFogNear_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Fog.Near = (float) numericFogNear.Value;
		}

		private void numericFogFar_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Fog.Far = (float) numericFogFar.Value;
		}

		private void numericFogTop_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Fog.Top = (float) numericFogTop.Value;
		}

		private void comboFogMode_SelectedIndexChanged(object sender, EventArgs e)
		{
			Jad.Scene.Fog.Type = (JFogType) comboFogMode.SelectedIndex;

			switch (Jad.Scene.Fog.Type)
			{
				case JFogType.None:

					numericFogNear.Enabled = false;
					numericFogFar.Enabled = false;
					numericFogDensity.Enabled = false;
					numericFogTop.Enabled = false;
					numericFogBottom.Enabled = false;

					break;

				case JFogType.Linear:

					numericFogNear.Enabled = true;
					numericFogFar.Enabled = true;
					numericFogDensity.Enabled = false;
					numericFogTop.Enabled = false;
					numericFogBottom.Enabled = false;

					break;

				case JFogType.Exponential:

					numericFogNear.Enabled = false;
					numericFogFar.Enabled = true;
					numericFogDensity.Enabled = true;
					numericFogTop.Enabled = false;
					numericFogBottom.Enabled = false;

					break;

				case JFogType.ExponentialSquared:

					numericFogNear.Enabled = false;
					numericFogFar.Enabled = true;
					numericFogDensity.Enabled = true;
					numericFogTop.Enabled = false;
					numericFogBottom.Enabled = false;

					break;

				case JFogType.Layered:

					numericFogNear.Enabled = false;
					numericFogFar.Enabled = true;
					numericFogDensity.Enabled = true;
					numericFogTop.Enabled = true;
					numericFogBottom.Enabled = true;

					break;
			}
		}

		private void numericFogDensity_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Fog.Density = (float) numericFogDensity.Value;
		}

		private void numericFogBottom_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Fog.Bottom = (float) numericFogBottom.Value;
		}

		private void numericLightIntensity_ValueChanged(object sender, EventArgs e)
		{
			Jad.Scene.Lights.Intensity = (float) numericLightIntensity.Value;
		}

		private void labelFogColor_Click(object sender, EventArgs e)
		{

			ColorDialog dlg = new ColorDialog();

			dlg.FullOpen = true;

			dlg.Color = Jad.Scene.Fog.Color.ToSystemColor();

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Jad.Scene.Fog.Color = new JColor(dlg.Color);

				labelFogColor.BackColor = dlg.Color;
			}
		}

		private void numExtents_ValueChanged(object sender, EventArgs e)
		{
			jViewer.Grid.GridExtents = (int) numExtents.Value;
		}

		private void numSpacing_ValueChanged(object sender, EventArgs e)
		{
			jViewer.Grid.GridSpacing = (float) numSpacing.Value;
		}

		private void checkBoxGridShow_CheckedChanged(object sender, EventArgs e)
		{
			ShowGrid = checkBoxGridShow.Checked;
		}

		private void labelAmbient_Click(object sender, EventArgs e)
		{
			ColorDialog dlg = new ColorDialog();

			dlg.FullOpen = true;

			dlg.Color = Jad.Scene.Lights.Ambient.ToSystemColor();

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Jad.Scene.Lights.Ambient = new JColor(dlg.Color);

				labelAmbient.BackColor = dlg.Color;
			}
		}

		private void comboShadowsMode_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch (comboShadowsMode.SelectedIndex)
			{
				case 0:
					Jad.Scene.Shadows.ShadowMode = JShadowMode.None;
					break;

				case 1:
					Jad.Scene.Shadows.ShadowMode = JShadowMode.StencilVolume;
					Jad.Scene.Shadows.Stencil.DepthPass = true;

					break;

				case 2:
					Jad.Scene.Shadows.ShadowMode = JShadowMode.StencilVolume;
					Jad.Scene.Shadows.Stencil.DepthPass = false;

					break;

				case 3:
					Jad.Scene.Shadows.ShadowMode = JShadowMode.ShadowMap;
					break;
			}

			checkBoxShadowSoft.Enabled = (comboShadowsMode.SelectedIndex > 0);
		}

		private void checkBoxShadowSoft_CheckedChanged(object sender, EventArgs e)
		{
			Jad.Scene.Shadows.SoftShadows = checkBoxShadowSoft.Checked;
		}

		private void checkBoxShadowDebug_CheckedChanged(object sender, EventArgs e)
		{
			Jad.Scene.Shadows.Stencil.Debug = checkBoxShadowDebug.Checked;
		}

		private void comboWireFrame_SelectedIndexChanged(object sender, EventArgs e)
		{
			Jad.Scene.WireFrameMode = (JWireframeMode) comboWireFrame.SelectedIndex;
		}

		private void numTimer_ValueChanged(object sender, EventArgs e)
		{
			timerMotor.Stop();
			timerMotor.Interval = (int) numTimer.Value;
			timerMotor.Start();
		}

		#endregion

		#region Particle Effects

		private bool LoadPFx(string fileName)
		{
			if (pFX != null)
			{
				DialogResult dlgRes = MessageBox.Show("Your current particle effect will be lost. Continue?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
				if (dlgRes == DialogResult.No)
					return true;
			}

			if ((pFX = Jad.Scene.Particles.Create("user_pfx", fileName)) == null)
				return false;

			justLoaded = true;

			pFX.Angles = new Vector3(Geometry.DegreeToRadian(-90.0f), 0.0f, 0.0f);

			txtPFxName.Text = pFX.Name;

			cbPFxParticleSystems.Items.Clear();

			int nParticleSystem = 0;
			foreach (JParticleSystem ps in pFX.ParticleSystems)
			{
				string num = (nParticleSystem > 10) ? nParticleSystem.ToString() : "0" + nParticleSystem.ToString();
				cbPFxParticleSystems.Items.Add("ps" + num);
				nParticleSystem++;
			}

			cbPFxParticleSystems.SelectedIndex = 0;

			particleSystemPanel.UpdateProfile(((JParticleSystem) pFX.ParticleSystems[0]).Profile);

			EnablePFxUI(true);

			return true;
		}

		private bool LoadPFx(JParticleEffect p)
		{
			if (pFX != null)
			{
				DialogResult dlgRes = MessageBox.Show("Your current particle effect will be lost. Continue?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
				if (dlgRes == DialogResult.No)
					return true;
			}

			if (p == null)
				return false;

			pFX = p;

			justLoaded = true;

			txtPFxName.Text = pFX.Name;

			cbPFxParticleSystems.Items.Clear();

			int nParticleSystem = 0;
			foreach (JParticleSystem ps in pFX.ParticleSystems)
			{
				string num = (nParticleSystem > 10) ? nParticleSystem.ToString() : "0" + nParticleSystem.ToString();
				cbPFxParticleSystems.Items.Add("ps" + num);
				nParticleSystem++;
			}

			cbPFxParticleSystems.SelectedIndex = 0;

			particleSystemPanel.UpdateProfile(((JParticleSystem) pFX.ParticleSystems[0]).Profile);

			EnablePFxUI(true);

			return true;
		}

		private void EnablePFxUI(bool enable)
		{
			btnPFxSave.Enabled = enable;
			cbPFxParticleSystems.Enabled = enable;
			btnPFxAdd.Enabled = enable;
			btnPFxRemove.Enabled = enable;
			particleSystemPanel.Enabled = enable;
		}

		private void btnPFxNew_Click(object sender, EventArgs e)
		{
			if (pFX != null)
			{
				DialogResult dlgRes = MessageBox.Show("Your current particle effect will be lost. Continue?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
				if (dlgRes == DialogResult.No)
					return;
			}

			Jad.Scene.Particles.Clear();
			JParticleSystemProfile defProfile = JParticleSystemProfile.CreateDefaultProfile();
			pFX = Jad.Scene.Particles.Create("user_pfx", null);
			pFX.Position = Jad.Scene.Camera.Position;
			pFX.Angles = Jad.Scene.Camera.Angles;
			//pFX.Angles = new Vector3(Geometry.DegreeToRadian(-90.0f), 0.0f, 0.0f);
			JParticleSystem ps = new JParticleSystem(pFX, defProfile);
			pFX.ParticleSystems.Add(ps);

			cbPFxParticleSystems.Items.Clear();
			cbPFxParticleSystems.Items.Add("ps00");
			cbPFxParticleSystems.SelectedIndex = 0;

			EnablePFxUI(true);
		}

		private void btnPFxSave_Click(object sender, EventArgs e)
		{
			SaveFileDialog dlg = new SaveFileDialog();
			dlg.Filter = "ParticleFX files (*.pfx)|*.pfx|All files (*.*)|*.*";
			dlg.FilterIndex = 0;
			dlg.RestoreDirectory = true;

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				pFX.Save(dlg.FileName);
			}
		}

		private void cbPFxParticleSystems_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (pFX == null || justLoaded)
			{
				justLoaded = false;
				return;
			}

			changedFromHere = true;

			JParticleSystem ps = pFX.ParticleSystems[cbPFxParticleSystems.SelectedIndex] as JParticleSystem;
			particleSystemPanel.UpdateProfile(ps.Profile);
		}

		private void btnPFxAdd_Click(object sender, EventArgs e)
		{
			JParticleSystem ps = new JParticleSystem(pFX, JParticleSystemProfile.CreateDefaultProfile());
			pFX.ParticleSystems.Add(ps);

			string num = (cbPFxParticleSystems.Items.Count > 10) ? cbPFxParticleSystems.Items.Count.ToString() : "0" + cbPFxParticleSystems.Items.Count.ToString();
			cbPFxParticleSystems.Items.Add("ps" + num);
			cbPFxParticleSystems.SelectedIndex = cbPFxParticleSystems.Items.Count - 1;

			particleSystemPanel.UpdateProfile(ps.Profile);
		}

		private void btnPFxRemove_Click(object sender, EventArgs e)
		{
			if (cbPFxParticleSystems.Items.Count > 0)
			{
				DialogResult dlgResult = MessageBox.Show("Are you sure you want to remove the current particle system?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

				if (dlgResult == DialogResult.Yes)
				{
					pFX.ParticleSystems.RemoveAt(cbPFxParticleSystems.SelectedIndex);
					cbPFxParticleSystems.Items.RemoveAt(cbPFxParticleSystems.SelectedIndex);
					
					if (cbPFxParticleSystems.Items.Count > 0)
						cbPFxParticleSystems.SelectedIndex = 0;
					
					else
					{
						particleSystemPanel.UpdateProfile(JParticleSystemProfile.CreateDefaultProfile());
						EnablePFxUI(false);
						pFX = null;
					}
				}
			}
		}

		public void OnProfileChanged(object sender, EventArgs e)
		{
			if (pFX == null || changedFromHere)
			{
				changedFromHere = false;
				return;
			}

			if (pFX.ParticleSystems.Count <= 0)
				return;

			JParticleSystem ps = pFX.ParticleSystems[cbPFxParticleSystems.SelectedIndex] as JParticleSystem;

			if (particleSystemPanel.Reset)
				pFX.ParticleSystems[cbPFxParticleSystems.SelectedIndex] = new JParticleSystem(pFX, particleSystemPanel.Profile);
			else
				ps.Profile = particleSystemPanel.Profile;
		}

		#endregion
	}

	public class Grid
	{
		#region Fields

		private int gridExtents;
		private float gridSpacing;

		private Vector3[] lines;

		public int GridExtents
		{
			get
			{
				return gridExtents;
			}
			set
			{
				if (gridExtents != value)
				{
					gridExtents = value;
					Build();
				}
			}
		}
		public float GridSpacing
		{
			get
			{
				return gridSpacing;
			}
			set
			{
				if (gridSpacing != value)
				{
					gridSpacing = value;
					Build();
				}
			}
		}

		#endregion

		#region Constructors

		public Grid()
		{
			this.gridExtents = 10;
			this.gridSpacing = 0.25f;

			Build();
		}

		public Grid(int gridExtents, float gridSpacing)
		{
			this.gridExtents = gridExtents;
			this.gridSpacing = gridSpacing;

			Build();
		}

		#endregion

		#region Methods

		private void Build()
		{
			lines = new Vector3[(gridExtents * gridExtents + 2) * 2];

			// Horizontal lines
			for (int nLine = 0; nLine < gridExtents + 1; nLine++)
			{
				float x1 = -(gridExtents * gridSpacing * 0.5f);
				float x2 = gridExtents * gridSpacing * 0.5f;
				float y = 0.0f;
				float z = nLine * gridSpacing - gridExtents * gridSpacing * 0.5f;

				lines[nLine * 2 + 0] = new Vector3(x1, y, z);
				lines[nLine * 2 + 1] = new Vector3(x2, y, z);
			}

			// Vertical lines
			for (int nLine = 0; nLine < gridExtents + 1; nLine++)
			{
				float x = nLine * gridSpacing - gridExtents * gridSpacing * 0.5f;
				float y = 0.0f;
				float z1 = -(gridExtents * gridSpacing * 0.5f);
				float z2 = gridExtents * gridSpacing * 0.5f;

				lines[(gridExtents + 1) * 2 + nLine * 2 + 0] = new Vector3(x, y, z1);
				lines[(gridExtents + 1) * 2 + nLine * 2 + 1] = new Vector3(x, y, z2);
			}
		}

		public void Render()
		{
			Matrix world = Matrix.Identity;

			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
			Jad.Video.Effects.InHouse.PosColor.RenderLines(ref world, JColor.White, (gridExtents * gridExtents + 2) * 2, lines);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, false);
		}

		#endregion
	}

	public class JViewer
	{
		#region Fields

		private Grid grid;

		ViewerFrm frm;

		JSprite sp;

		#endregion

		#region Constructors

		public JViewer()
		{
		}

		#endregion

		#region Properties

		public Grid Grid
		{
			get { return grid; }
		}

		#endregion

		#region Methods

		public void MainLoop()
		{
			Jad.Configuration.GetConfiguration();

			Jad.Video.Adapter.Initialization.ReadFromConfig();

			using (frm = new ViewerFrm())
			{
				// Adjust the render size
				frm.ResizeRender();

				Jad.Form = frm;

				Jad.Handle = frm.Handle;

				Jad.Video.Adapter.CreateDevice(frm.renderCtrl);

				if (!InitGame(frm))
				{
					MessageBox.Show("It wasnt possible to start D3D.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
					return;
				}

				frm.jViewer = this;

				frm.timerMotor.Enabled = true;

				frm.ShowDialog();
			}
		}

		/// <summary>
		/// The engine calls this method after creating the device
		/// </summary>
		/// <param name="form">Form used as window</param>
		/// <returns>True if everything went right</returns>
		public bool InitGame(Form form)
		{
			// Create the fonts and texture objects
			Jad.AfterCreateDevice();

			// Load default textures, scenes, shaders,...
			Jad.LoadCreateInHouse();

			// Create the RTs for these effects
			Jad.Video.RenderTargets.CreateShadowMap();
			Jad.Video.RenderTargets.CreateMotionBlur();

			Jad.Scene.Lights.Ambient = new JColor(0.1568f);

			grid = new Grid();

			sp = new JSprite();

			// Create one RT to save the EnvMap
			Jad.Video.RenderTargets.Cube = Jad.Video.RenderTargets.CreateAbsolute("CubeEM", 256, 256, Jad.Video.RenderTargets.RTFormat, true, true);
			Jad.Video.RenderTargets.Cube.Texture.InHouse = true;

			// Copy the last frame to allow pauses
			Jad.Scene.CopyLastFrame = true;

			// Start the engine
			Jad.Begin();

			return true;
		}

		/// <summary>
		/// This method is called every frame and its where the input
		/// events and the game action must be placed
		/// </summary>
		/// <returns>True to continue, false to exit the application</returns>
		public bool Action()
		{
			if (Jad.Input != null)
			{
				if (Jad.Input.Keyboard.KeyPressed(Key.NumPadPlus))
					Jad.Scene.MeshObjects.Bumpiness += 0.1f;

				if (Jad.Input.Keyboard.KeyPressed(Key.NumPadMinus))
					Jad.Scene.MeshObjects.Bumpiness -= 0.1f;

				JCameraFirstPerson camera = (JCameraFirstPerson) Jad.Scene.Camera;

				// Move the light
				if (!camera.UseInput)
				{
					JNode node = frm.NodeSelected;

					Vector3 pos = node.Position;

					float speed = 0.1f;

					if (Jad.Input.Keyboard.KeyPressed(Key.A))
						pos -= camera.Right * speed;

					if (Jad.Input.Keyboard.KeyPressed(Key.D))
						pos += camera.Right * speed;

					if (Jad.Input.Keyboard.KeyPressed(Key.W))
						pos += camera.Look * speed;

					if (Jad.Input.Keyboard.KeyPressed(Key.S))
						pos -= camera.Look * speed;

					if (Jad.Input.Mouse.MouseButtonPressed(JMouseButton.LeftButton))
						pos -= camera.Up * speed;

					if (Jad.Input.Mouse.MouseButtonPressed(JMouseButton.RightButton))
						pos += camera.Up * speed;

					node.Position = pos;

					frm.TreeNodeSelected.Text = "Pos:" + JMath.Vector3ToString(pos);
				}

				if (Jad.Input.Keyboard.KeyPressed(Key.Escape))
				{
					Jad.Input.Dispose();
					Jad.Input = null;

					camera.UseInput = true;
				}
			}

			return true;
		}

		/// <summary>
		/// Render method
		/// </summary>
		public void Render()
		{
			if (Jad.Video.Device == null)
				return;

			// Prepare the scene to start the render
			Jad.Scene.Begin();

			Jad.Scene.Render();

			// Do the post-production
			Jad.Scene.ProcessPostProduction();

			if (frm.ShowGrid)
				grid.Render();

			// Gizmos, Axis...
			Jad.Scene.RenderMiscellaneous();

			int i = 0; int x, y, w, h;

			if (frm.glowToolStripMenuItem.Checked) i++;
			if (frm.depthofFieldToolStripMenuItem.Checked) i++;
			if (frm.shadowMapToolStripMenuItem.Checked) i++;
			if (frm.shadowMaskToolStripMenuItem.Checked) i++;

			if (i < 4) i = 4;

			x = 0;
			y = 0;
			w = frm.renderCtrl.ClientSize.Width / i;

			h = frm.renderCtrl.ClientSize.Height / i;

			if (frm.glowToolStripMenuItem.Checked)
			{

				sp.SetPositionSize(x, y, w, h);
				sp.Effect = Jad.Video.Effects.InHouse.PosColorTex1;
				sp.Texture = Jad.Video.RenderTargets.Glow.Texture;
				sp.Render();

				x += w;
			}

			if (frm.depthofFieldToolStripMenuItem.Checked)
			{

				sp.SetPositionSize(x, y, w, h);
				sp.Effect = Jad.Video.Effects.InHouse.ShowZ;
				sp.Texture = Jad.Video.RenderTargets.Glow.Texture;
				sp.Render();

				x += w;
			}

			if (frm.shadowMapToolStripMenuItem.Checked)
			{

				sp.SetPositionSize(x, y, w, h);
				sp.Effect = Jad.Video.Effects.InHouse.ShowFloat;
				sp.Texture = Jad.Video.RenderTargets.ShadowMap.Texture;
				sp.Render();

				x += w;
			}

			if (frm.shadowMaskToolStripMenuItem.Checked)
			{

				sp.SetPositionSize(x, y, w, h);
				sp.Effect = Jad.Video.Effects.InHouse.PosColorTex1;
				sp.Texture = Jad.Video.RenderTargets.ShadowMask.Texture;
				sp.Render();

				x += w;
			}

			// Get access to the 1st font. The engine creates a font by default
			JFont font = Jad.Video.Fonts[0];

			// Start fonts rendering
			Jad.Video.Fonts.Begin();

			if (frm.ShowFPS)
				font.RenderLine(Jad.Video.Render.Stats.Fps, 0, JColor.White);

			if (frm.showCameraPos)
				font.RenderLine(JMath.Vector3ToString(Jad.Scene.Camera.Position), 0, JColor.White);

			// End fonts rendering
			Jad.Video.Fonts.End();

			Jad.Scene.End();
		}

		/// <summary>
		/// Render the last frame. This is done because if the viewer is paused, its
		/// only needed to refresh the control, not render the scene again
		/// </summary>
		public void RenderLastFrame()
		{
			// Update the timer
			Jad.Timer.Frame();

			// Start the render
			Jad.Video.Render.Begin();

			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, false);
			JRenderTarget rt = Jad.Video.RenderTargets.LastFrame;

			// Set the effect

			JEffect effect = Jad.Video.Effects.InHouse.PostProcessEffect;

			// Set the technique
			effect.DXResource.Technique = effect.handlers.renderScene;

			// Start the effect
			int steps = effect.Begin();

			// Loop through all the steps of the effect
			for (int step = 0; step < steps; step++)
			{
				effect.DXResource.BeginPass(step);

				effect.DXResource.SetValue(effect.handlers.textureDiffuse, rt.Texture.DXResource);

				// Commit the changes
				effect.DXResource.CommitChanges();

				// Draw a rectangle that fills all the screen
				Jad.Video.Render.RenderScreenAlignedQuad(1f, 1f, 1f, 1f);

				effect.DXResource.EndPass();
			}

			// End the effect
			effect.End();

			Jad.Scene.End();
		}

		#endregion
	}

	/// <summary>
	/// The segment is represented as P+t*D, where P is the segment origin,
	/// D is a unit-length direction vector and |t| is minor or equal than e. The value e is
	/// referred to as the extent of the segment.  The end points of the
	/// segment are P-e*D and P+e*D.  The user must ensure that the direction
	/// vector is unit-length.  The representation for a segment is analogous
	/// to that for an oriented bounding box.  P is the center, D is the
	/// axis direction, and e is the extent.
	/// </summary>
	class Segment
	{
		#region Fields

		public float Extent;
		public Vector3 Origin, Direction;

		#endregion

		#region Constructors

		public Segment()
		{
		}

		public Segment(Vector3 origin, Vector3 direction, float extent)
		{
			Extent = extent;
			Origin = origin;
			Direction = direction;
		}

		#endregion
	}

	/// <summary>
	/// Cylinder line segment has end points C-(H/2)*D and C+(H/2)*D where
	/// D is a unit-length vector.  H is infinity for infinite cylinder.
	/// </summary>
	class Cylinder
	{
		#region Fields

		public Segment Segment;
		public float Height, Radius;
		
		#endregion

		#region Constructors

		public Cylinder()
		{
		}

		public Cylinder(Segment segment, float height, float radius)
		{
			Segment = segment;
			Height = height;
			Radius = radius;
		}

		#endregion

		#region Methods

		public bool LineIntersect(Vector3 origin, Vector3 dir, out Vector3[] intrPoints)
		{
			float[] aT = new float[2];
			intrPoints = new Vector3[2];
			int iQuantity = LineIntersect(origin, dir, out aT);
			for (int i = 0; i < iQuantity; i++)
			{
				intrPoints[i] = Vector3.Empty;
				intrPoints[i] = origin + aT[i] * dir;
			}

			int intersectionType = 0;

			if (iQuantity == 2)
				intersectionType = 1;
			else if (iQuantity == 1)
				intersectionType = 2;
			else
				intersectionType = 0;

			return intersectionType != 0;
		}

		private void GenerateOrthonormalBasis(out Vector3 u, out Vector3 v, Vector3 w, bool unitLengthW)
		{
			u = v = Vector3.Empty;

			if (!unitLengthW)
				w.Normalize();

			float invLength;

			if (Math.Abs(w.X) >= Math.Abs(w.Y))
			{
				// W.x or W.z is the largest magnitude component, swap them
				invLength = (float) (1 / Math.Sqrt(w.X * w.X + w.Z * w.Z));
				u.X = -w.Z * invLength;
				u.Y = 0.0f;
				u.Z = w.X * invLength;
				v.X = w.Y * u.Z;
				v.Y = w.Z * u.X - w.X * u.Z;
				v.Z = -w.Y * u.X;
			}

			else
			{
				// W.y or W.z is the largest magnitude component, swap them
				invLength = (float) (Math.Sqrt(w.Y * w.Y + w.Z * w.Z));
				u.X = 0.0f;
				u.Y = w.Z * invLength;
				u.Z = -w.Y * invLength;
				v.X = w.Y * u.Z - w.Z * u.Y;
				v.Y = -w.X * u.Z;
				v.Z = w.X * u.Y;
			}
		}

		private int LineIntersect(Vector3 origin, Vector3 dir, out float[] aT)
		{
			// Create a coordinate system for the cylinder.  In this system, the
			// cylinder segment center C is the origin and the cylinder axis direction
			// W is the z-axis.  U and V are the other coordinate axis directions.
			// If P = x*U+y*V+z*W, the cylinder is x^2 + y^2 = r^2, where r is the
			// cylinder radius.  The end caps are |z| = h/2, where h is the cylinder
			// height.

			Vector3 u, v, w = Segment.Direction;
			GenerateOrthonormalBasis(out u, out v, w, true);
			float halfHeight = 0.5f * Height;
			float rSqr = Radius * Radius;

			// convert incoming line origin to cylinder coordinates
			Vector3 diff = origin - Segment.Origin;
			Vector3 p = new Vector3(Vector3.Dot(u, diff), Vector3.Dot(v, diff), Vector3.Dot(w, diff));

			// Get the z-value, in cylinder coordinates, of the incoming line's
			// unit-length direction.
			float dZ = Vector3.Dot(w, dir);

			aT = new float[2];

			if (Math.Abs(dZ) >= 1.0f - 0.00001f)
			{
				// The line is parallel to the cylinder axis.  Determine if the line
				// intersects the cylinder end disks.
				float radialSqrDist = rSqr - p.X * p.X - p.Y * p.Y;
				if (radialSqrDist < 0.0f)
				{
					// line outside the cylinder, no intersection
					return 0;
				}

				// line intersects the cylinder end disks
				if (dZ > 0.0f)
				{
					aT[0] = -p.Z - halfHeight;
					aT[1] = -p.Z + halfHeight;
				}

				else
				{
					aT[0] = p.Z - halfHeight;
					aT[1] = p.Z + halfHeight;
				}

				return 2;
			}

			// convert incoming line unit-length direction to cylinder coordinates
			Vector3 d = new Vector3(Vector3.Dot(u, dir), Vector3.Dot(v, dir), dZ);

			float a0, a1, fA2, discr, root, inv, t;

			if (Math.Abs(d.Z) <= 0.00001f)
			{
				// The line is perpendicular to the cylinder axis.
				if (Math.Abs(p.Z) > halfHeight)
				{
					// line is outside the planes of the cylinder end disks
					return 0;
				}

				// Test intersection of line P+t*D with infinite cylinder
				// x^2+y^2 = r^2.  This reduces to computing the roots of a
				// quadratic equation.  If P = (px,py,pz) and D = (dx,dy,dz),
				// then the quadratic equation is
				// (dx^2+dy^2)*t^2 + 2*(px*dx+py*dy)*t + (px^2+py^2-r^2) = 0

				a0 = p.X * p.X + p.Y * p.Y - rSqr;
				a1 = p.X * d.X + p.Y * d.Y;
				fA2 = d.X * d.X + d.Y * d.Y;
				discr = a1 * a1 - a0 * fA2;
				if (discr < 0.0f)
				{
					// line does not intersect cylinder
					return 0;
				}
				else if (discr > 0.00001f)
				{
					// line intersects cylinder in two places
					root = (float) Math.Sqrt(discr);
					inv = 1.0f / fA2;
					aT[0] = (-a1 - root) * inv;
					aT[1] = (-a1 + root) * inv;
					return 2;
				}
				else
				{
					// line is tangent to the cylinder
					aT[0] = -a1 / fA2;
					return 1;
				}
			}

			// test plane intersections first
			int quantity = 0;
			inv = 1.0f / d.Z;

			float t0 = (-halfHeight - p.Z) * inv;
			float xTmp = p.X + t0 * d.X;
			float yTmp = p.Y + t0 * d.Y;
			if (xTmp * xTmp + yTmp * yTmp <= rSqr)
			{
				// planar intersection inside the top cylinder end disk
				aT[quantity++] = t0;
			}

			float t1 = (+halfHeight - p.Z) * inv;
			xTmp = p.X + t1 * d.X;
			yTmp = p.Y + t1 * d.Y;
			if (xTmp * xTmp + yTmp * yTmp <= rSqr)
			{
				// planar intersection inside the bottom cylinder end disk
				aT[quantity++] = t1;
			}

			if (quantity == 2)
			{
				// line intersects both top and bottom cylinder end disks
				if (aT[0] > aT[1])
				{
					float save = aT[0];
					aT[0] = aT[1];
					aT[1] = save;
				}
				return 2;
			}

			// If iQuantity == 1, then the line must intersect cylinder wall in a
			// single point somewhere between the end disks.  This case is detected
			// in the following code that tests for intersection between line and
			// cylinder wall.
			a0 = p.X * p.X + p.Y * p.Y - rSqr;
			a1 = p.X * d.X + p.Y * d.Y;
			fA2 = d.X * d.X + d.Y * d.Y;
			discr = a1 * a1 - a0 * fA2;
			if (discr < 0.0f)
			{
				// line does not intersect cylinder wall
				//assert(iQuantity == 0);
				return 0;
			}
			else if (discr > 0.00001)
			{
				root = (float) Math.Sqrt(discr);
				inv = 1.0f / fA2;
				t = (-a1 - root) * inv;
				if (t0 <= t1)
				{
					if (t0 <= t && t <= t1)
						aT[quantity++] = t;
				}
				else
				{
					if (t1 <= t && t <= t0)
						aT[quantity++] = t;
				}

				if (quantity == 2)
				{
					// Line intersects one of the cylinder end disks and once on the
					// cylinder wall.
					if (aT[0] > aT[1])
					{
						float save = aT[0];
						aT[0] = aT[1];
						aT[1] = save;
					}
					return 2;
				}

				t = (-a1 + root) * inv;
				if (t0 <= t1)
				{
					if (t0 <= t && t <= t1)
						aT[quantity++] = t;
				}
				else
				{
					if (t1 <= t && t <= t0)
						aT[quantity++] = t;
				}
			}
			else
			{
				t = -a1 / fA2;
				if (t0 <= t1)
				{
					if (t0 <= t && t <= t1)
						aT[quantity++] = t;
				}
				else
				{
					if (t1 <= t && t <= t0)
						aT[quantity++] = t;
				}
			}

			if (quantity == 2)
			{
				if (aT[0] > aT[1])
				{
					float save = aT[0];
					aT[0] = aT[1];
					aT[1] = save;
				}
			}

			return quantity;
		}

		#endregion
	}
}