#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using JadEngine.Scene;
using JadEngine.Collections;

#endregion


namespace JadEngine.Animation
{

    #region Enumerations

    /// <summary>
    /// Enumeration for animation controller types
    /// </summary>
    public enum AnimationControllerType
    {
        /// <summary>
        /// Used for standard animations(transformations)
        /// </summary>
        Standard,

        /// <summary>
        /// Used for mesh morphing animation
        /// </summary>
        Morph,

        /// <summary>
        /// Used for Skeletal/Skinning animation
        /// </summary>
        Skin
    }

    #endregion

    /// <summary>
    /// Base for animation controllers. There are simple controllers, morphing controllers and skinning controllers
    /// </summary>
    public class JAnimationControllerBase 
    {
        #region Fields

        /// <summary>
        /// Name
        /// </summary>
        protected string name;

		/// <summary>
		/// Controller's type
		/// </summary>
		protected AnimationControllerType controllerType;

        /// <summary>
        /// Key frames
        /// </summary>
        protected List<JAnimationKey> keyFrames;

        /// <summary>
        /// Animation channels
        /// </summary>
        protected JListUnique<JAnimationChannel> channels;

		/// <summary>
		/// Parent animation
		/// </summary>
		protected IJAnimation _parent;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        public JAnimationControllerBase()
        {
            keyFrames = new List<JAnimationKey>();
            channels = new JListUnique<JAnimationChannel>();
			_parent = null;
        }

        #endregion

        #region Properties

        /// <summary>
        /// TODO
        /// </summary>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// TODO
        /// </summary>
        public AnimationControllerType ControllerType
        {
            get { return controllerType; }
            set { controllerType = value; }
        }

        /// <summary>
        /// TODO
        /// </summary>
        public List<JAnimationKey> KeyFrames
        {
            get { return keyFrames; }
            set { keyFrames = value; }
        }

        /// <summary>
        /// TODO
        /// </summary>
        public JListUnique<JAnimationChannel> Channels
        {
            get { return channels; }
            set { channels = value; }
        }

		/// <summary>
		/// TODO
		/// </summary>
		public IJAnimation ParentAnimation
		{
			get { return _parent; }
			set { _parent = value; }
		}

        #endregion

		#region Methods

		/// <summary>
		/// Updates the current key frame (the one which is displayed) with the information from
		/// active channels within an animation clip
		/// </summary>
		/// <param name="channels"></param>
		public virtual void Update(List<JAnimationChannel> channels) { }

		#endregion

	}
 
}
