#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;

using JadEngine.Core;
using JadEngine.Scene;

#endregion

namespace JadEngine.Import
{
	/// <summary>
	/// The different ways of importing a mesh on haddd
	/// </summary>
	public class JImport
	{
		#region Delegates

		/// <summary>
		/// Delegate that specifies the reportback handler, when a file loads (.haddd only), this function will be called to report progress
		/// </summary>
		/// <param name="what">TODO</param>
		/// <param name="where">TODO</param>
		/// <param name="total">TODO</param>
		public delegate void ReportBackHandler(string what, int where, int total);

		#endregion

		#region Events
		/// <summary>
		/// OnReport event will be called when loading a .haddd file
		/// </summary>
		public event ReportBackHandler OnReport;

		#endregion

		#region Fields

		/// <summary>
		/// The Jad loader
		/// </summary>
		JadLoader jad = new JadLoader();

		/// <summary>
		/// The .x loader
		/// </summary>
		JXLoader x = new JXLoader();

		/// <summary>
		/// Mesh objects loaded on the file
		/// </summary>
		public List<JMeshObject> MeshObjects;

		/// <summary>
		/// Full path
		/// </summary>
		string FullPath;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JImport()
		{
			MeshObjects = new List<JMeshObject>();
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fileName">TODO</param>
		/// <param name="scale">TODO</param>
		/// <returns>TODO</returns>
		public bool Load(string fileName, float scale)
		{
			return Load(fileName, new Vector3(scale, scale, scale));
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fileName">TODO</param>
		/// <returns>TODO</returns>
		public bool Load(string fileName)
		{
			return Load(fileName, new Vector3(1f, 1f, 1f));
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fileName">TODO</param>
		/// <param name="inScale">TODO</param>
		/// <returns>TODO</returns>
		public bool Load(string fileName, Vector3 inScale)
		{
			bool ok = false;

			// Clear the objects list created in this scene
			MeshObjects.Clear();

			string ext = Path.GetExtension(fileName).ToLower();

			if (ext == ".x")
				ok = x.Load(fileName, inScale);

			if (ext == ".haddd")
			{
				//Add the report delegate
				jad.OnReport += new JadLoader.ReportBackHandler(jad_OnReport);
				ok = jad.Load(fileName, inScale);
			}

			if (ok)
			{
				Jad.Scene.Root.SceneGraph.Update();

				// Set that its needed to recalculate the nodes transformation
				Jad.Scene.Root.ResetTransform();

				// Calculate the nodes transformation
				Jad.Scene.Root.TransformDown();

				// Calculate the Bounding Boxes and Spheres
				Jad.Scene.Root.CalculateGlobalBounds();
			}

			return ok;
		}

		/// <summary>
		/// This method raises the OnReport event
		/// </summary>
		/// <param name="what">TODO</param>
		/// <param name="where">TODO</param>
		/// <param name="total">TODO</param>
		void jad_OnReport(string what, int where, int total)
		{
			if (OnReport != null)
				OnReport(what, where, total);
		}

		#endregion
	}
}
