#region Using directives

using System.Collections.Generic;
using System.Drawing;

using Microsoft.DirectX.Direct3D;

using JadEngine.Video;
using JadEngine.Tools;

#endregion

// http://www.blackpawn.com/texts/lightmaps/default.html

namespace JadEngine.Import
{
	/// <summary>
	/// TODO
	/// </summary>
    public class JPackNode
	{
		#region Fields
    	
		/// <summary>
		/// TODO
		/// </summary>
		public JLightMapPacket packet;

		/// <summary>
		/// TODO
		/// </summary>
		public JPackNode child0;

		/// <summary>
		/// TODO
		/// </summary>
		public JPackNode child1;

		/// <summary>
		/// TODO
		/// </summary>
		public Rectangle rc;

		/// <summary>
		/// TODO
		/// </summary>
		public JTexture source;

		#endregion

		#region Constructors
    	
		/// <summary>
		/// Default constructor
		/// </summary>
		public JPackNode()
		{
			child0 = child1 = null;
			source = null;
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// Tells if the node is a leaf of the tree.
        /// </summary>
        /// <remarks>The root has to return that is a leaf also</remarks>
        /// <returns>TODO</returns>
        bool IsLeaf()
        {
            return (child0 == null && child1 == null);
        }

        /// <summary>
        /// Tests if the image fits in this node
        /// </summary>
        /// <param name="img">TODO</param>
		/// <returns>TODO</returns>
        bool Fits(JTexture img)
        {
        	// Get the size available in the destination
            Size destSize = rc.Size;

			// Size of the image to insert
            Size srcSize = new Size(img.SurfaceDes.Width, img.SurfaceDes.Height);

        	if (srcSize.Width > destSize.Width || srcSize.Height > destSize.Height)
        		return false;

        	else
        		return true;
        }

        /// <summary>
        /// Tests if the image fits perfectly in this node
        /// </summary>
        /// <param name="img">TODO</param>
		/// <returns>TODO</returns>
        bool FitsPerfect(JTexture img)
        {
			// Get the size available in the destination
            Size destSize = rc.Size;

			// Size of the image to insert
            Size srcSize = new Size(img.SurfaceDes.Width, img.SurfaceDes.Height);

        	if (srcSize.Width != destSize.Width || srcSize.Height != destSize.Height)
        		return false;

        	else
        		return true;
        }

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="img">TODO</param>
		/// <param name="father">TODO</param>
		/// <returns>TODO</returns>
        public JPackNode Insert(JTexture img, JLightMapPacket father)
        {
            JPackNode newNode;

			// If not in a leaf
            if (!IsLeaf())
            {
                // Test with the first
                newNode = child0.Insert(img,father);
                if (newNode != null)
                	return newNode;
            	
                // There wasnt enough space, test the second
                newNode = child1.Insert(img,father);
                if (newNode != null)
                	return newNode;

                // It doesnt fit!
                return null;
            }
			
            else // In a leaf
            {
            	//If theres a lightmap in this leaf, continue searching
                if (source != null)
                	return null;

				// If it doesnt fit, continue searching
                if (!Fits(img))
                	return null;

                // If it fits perfectly, this is the node!
                if (FitsPerfect(img))
                {
                    source = img;
                    packet = father;
                	
                    return this;
                }

                // Create the children

                child0 = new JPackNode();
                child1 = new JPackNode();

                int sw = img.SurfaceDes.Width;
                int sh = img.SurfaceDes.Height;

            	// See where to do the split
                int dw = rc.Width - sw;
                int dh = rc.Height - sh;

                if (dw > dh)
                {
                    child0.rc = new Rectangle(rc.Left, rc.Top, sw , rc.Height);
                    child1.rc = new Rectangle(rc.Left + sw, rc.Top, rc.Width-sw, rc.Height);
                }
            	
                else
                {
                    child0.rc = new Rectangle(rc.Left, rc.Top, rc.Width, sh );
                    child1.rc = new Rectangle(rc.Left, rc.Top + sh, rc.Width, rc.Height-sh);
                }

                return child0.Insert(img,father);
            }
        }

        /// <summary>
        /// Searchs the original texture and returns the name of the created lightmap
        /// </summary>
		/// <param name="sourceText">TODO</param>
        /// <returns>TODO</returns>
        public bool FindSourceTexture(JTexture sourceText)
        {
            if (source == sourceText)
            	return true;

        	if (child0 != null)
        		if (child0.FindSourceTexture(sourceText)) return true;

        	if (child1 != null)
        		if (child1.FindSourceTexture(sourceText)) return true;

        	return false;
		}

		/// <summary>
		/// TODO
		/// </summary>
		public void Debug()
		{
			Jad.LogWriter.WriteLine(rc.ToString());

			if (child0 != null)
				child0.Debug();

			else
				Jad.LogWriter.WriteLine("0-");

			if (child1 != null)
				child1.Debug();

			else
				Jad.LogWriter.WriteLine("1-");
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
    public class JLightMapPacker
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
        List<JLightMapPacket> list;
		
		/// <summary>
		/// TODO
		/// </summary>
        JLightMapPacket actual;
		
		/// <summary>
		/// TODO
		/// </summary>
        int width, height;

		#endregion

		#region Constructors
		
		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="pwidth">TODO</param>
		/// <param name="pheight">TODO</param>
		public JLightMapPacker(int pwidth, int pheight)
        {
            list = new List<JLightMapPacket>();
            
            width = pwidth;
            height = pheight;

            actual = new JLightMapPacket("LightMap0", width, height);

            list.Add(actual);
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// Insert a texture in the actual package
		/// </summary>
		/// <remarks>A package is a texture that acts as a group of textures</remarks>
		/// <param name="src">TODO</param>
		/// <returns>TODO</returns>
		public JPackNode Insert(JTexture src)
		{
			Jad.LogWriter.WriteLine("Trying to pack:" + src.Name + " (" + src.SurfaceDes.Width.ToString() + "," + src.SurfaceDes.Height.ToString());

			JPackNode node = actual.Insert(src);

			// Doesnt fit, create another package
			if (node == null)
			{
				actual = new JLightMapPacket("LightMap"+list.Count.ToString(), width, height);
				node = actual.Insert(src);
				list.Add(actual);
			}

			return node;
		}

		/// <summary>
		/// Tests if the size of the surfaces is too big and can be reduced
		/// </summary>
		public void Optimize()
		{
			foreach (JLightMapPacket p in list)
				p.Optimize();
		}
		
		/// <summary>
		/// Saves the textures in C:\
		/// </summary>
		/// <param name="path">TODO</param>
		public void SaveTexture(string path)
		{
			foreach (JLightMapPacket p in list)
				p.SaveTexture(path);
		}

		/// <summary>
		/// Searchs the original texture and returns the name of the created lightmap
		/// </summary>
		/// <param name="source">TODO</param>
		/// <returns>TODO</returns>
		public string FindSourceTexture(JTexture source)
		{
			foreach (JLightMapPacket p in list)
			{
				string n = p.FindSourceTexture(source);
				if (n != null) return n;
			}
			
			return null;
		}

		/// <summary>
		/// TODO
		/// </summary>
		public void Debug()
		{
			Jad.LogWriter.WriteLine("LightMapPacker");
			actual.Debug();
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
    public class JLightMapPacket
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
        public string name;

		/// <summary>
		/// TODO
		/// </summary>
		public JPackNode root;

		/// <summary>
		/// TODO
		/// </summary>
		public JTexture destTexture;

		/// <summary>
		/// TODO
		/// </summary>
		public int width;

		/// <summary>
		/// TODO
		/// </summary>
		public int height;

		#endregion

		#region Constructors
		
		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="name">TODO</param>
		/// <param name="width">TODO</param>
		/// <param name="height">TODO</param>
		public JLightMapPacket(string name, int width, int height)
        {
            this.name = name;

			// Create the main node
            root = new JPackNode();
            
			// Give it the rectangle dimensions
            root.rc = new Rectangle(0, 0, width , height);

			// Create the destination texture
            destTexture = Jad.Video.Textures.Create2D(name, false);
            destTexture.Create(width, height, Format.A8R8G8B8);
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="src">TODO</param>
		/// <returns>TODO</returns>
		public JPackNode Insert(JTexture src)
        {
            JPackNode node=root.Insert(src,this);

            if (node == null)
            	return null;

			// Save the maximum values, so that if theres wasted space
			// in the texture, we can remove it
            if (node.rc.Right > width) width = node.rc.Right;
            if (node.rc.Bottom > height) height = node.rc.Bottom;

			// Insert the original texture in the correct place
            Texture srcT = src.DXResource;
            Texture destT = destTexture.DXResource;
            Surface srcSurface = srcT.GetSurfaceLevel(0);
            Surface destSurface = destT.GetSurfaceLevel(0);

            SurfaceLoader.FromSurface(destSurface, node.rc, srcSurface, Filter.None, 0);

            srcSurface.Dispose();
            destSurface.Dispose();

            return node;
        }

        /// <summary>
		/// Tests if the size of the surfaces is too big and can be reduced
        /// </summary>
        public void Optimize()
        {
            if (destTexture.SurfaceDes.Width > width || destTexture.SurfaceDes.Height > height)
            {
                JTextureBase old = destTexture;

            	// Delete the old texture from the list
                Jad.Video.Textures.Delete(old);
            	
            	// Its needed to change the name or the TextureFactory will return a reference
            	// to the original texture
            	
            	// Create the new texture with the correct size
                JTexture newTexture = Jad.Video.Textures.Create2D(name, false);

                newTexture.Create(width, height, Format.A8R8G8B8);

                // Copy

                Texture srcT = destTexture.DXResource;
                Texture destT = newTexture.DXResource;
                Surface srcSurface = srcT.GetSurfaceLevel(0);
                Surface destSurface = destT.GetSurfaceLevel(0);

                SurfaceLoader.FromSurface(destSurface, new Rectangle(0,0,width,height), srcSurface, Filter.None, 0);

                srcSurface.Dispose();
                destSurface.Dispose();

                // Dispose the old texture
                destTexture.DXResource.Dispose();

                // Associate the new one
                destTexture = newTexture;
            }
        }

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="path">TODO</param>
        public void SaveTexture(string path)
        {
            destTexture.Save(path+name, ImageFileFormat.Bmp);
        }

        /// <summary>
        /// Searchs the original texture and returns the name of the created lightmap
        /// Busca la textura origen y nos devuelve el nombre del lightmap creado
        /// </summary>
        /// <param name="source">TODO</param>
		/// <returns>TODO</returns>
        public string FindSourceTexture(JTexture source)
        {
            if (root.FindSourceTexture(source))
            	return name;

            return null;
		}

		/// <summary>
		/// TODO
		/// </summary>
		public void Debug()
		{
			Jad.LogWriter.WriteLine("HPackNode");
			root.Debug();
		}

		#endregion
	}
}

