#region Using Directives

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

#endregion

namespace JadEngine.Input.XInput
{
	/// <summary>
	/// Enumeration for button mask
	/// </summary>
	internal enum EButtonMask : ushort
	{
        /// <summary>Up</summary>
		Up = 0x0001,
        /// <summary>Down</summary>
        Down = 0x0002,
        /// <summary>Left</summary>
        Left = 0x0004,
        /// <summary>Right</summary>
        Right = 0x0008,
        /// <summary>Start</summary>
        Start = 0x0010,
        /// <summary>Back</summary>
        Back = 0x0020,
        /// <summary>Thumb Left</summary>
        ThumbLeft = 0x0040,
        /// <summary>Thumb Right</summary>
        ThumbRight = 0x0080,
        /// <summary>Shoulder Left</summary>
        ShoulderLeft = 0x0100,
        /// <summary>Shoulder Right</summary>
        ShoulderRight = 0x0200,
        /// <summary>A button</summary>
        A = 0x1000,
        /// <summary>B button</summary>
        B = 0x2000,
        /// <summary>X button</summary>
        X = 0x4000,
        /// <summary>Y button</summary>
        Y = 0x8000,
        /// <summary>None</summary>
        None = 0
	}

	/// <summary>
	/// TODO
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	internal struct XInputGamePad
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public EButtonMask Buttons;

		/// <summary>
		/// TODO
		/// </summary>
		public byte LeftTrigger;

		/// <summary>
		/// TODO
		/// </summary>
		public byte RightTrigger;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbLeftX;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbLeftY;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbRightX;

		/// <summary>
		/// TODO
		/// </summary>
		public short ThumbRightY;

		#endregion
	}

	/// <summary>
	/// Information about the vibration motors
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	internal struct XInputVibration
	{
		#region Fields

		/// <summary>
		/// Left motor speed
		/// </summary>
		public ushort LeftMotorSpeed;

		/// <summary>
		/// Right motor speed
		/// </summary>
		public ushort RightMotorSpeed;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="left">Left motor speed</param>
		/// <param name="right">Right motor speed</param>
		public XInputVibration(ushort left, ushort right)
		{
			LeftMotorSpeed = left;
			RightMotorSpeed = right;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	internal struct XInputState
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public uint PacketNumber;

		/// <summary>
		/// TODO
		/// </summary>
		public XInputGamePad Gamepad;

		#endregion
	}

	/// <summary>
	/// XInput wrapper
	/// </summary>
	public static class JXInput
	{
		#region Constants

		/// <summary>
		/// TODO
		/// </summary>
		public const int ERROR_SUCCESS = 0;

		#endregion

		#region Methods

		[System.Security.SuppressUnmanagedCodeSecurity] // We won't use this maliciously
		[DllImport("xinput9_1_0.dll", EntryPoint = "XInputSetState", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi)]
		internal static extern int SetState(int playerIndex, ref XInputVibration vibration);

		[System.Security.SuppressUnmanagedCodeSecurity] // We won't use this maliciously
		[DllImport("xinput9_1_0.dll", EntryPoint = "XInputGetState", CharSet = CharSet.Auto, CallingConvention = CallingConvention.Winapi)]
		internal static extern int GetState(int playerIndex, ref XInputState state);

		#endregion
	}
}
