#region Using Directives

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

#endregion

namespace JadEngine
{
	/// <summary>
	/// Base class to create Jad applications
	/// </summary>
	public abstract class JApplication
	{
		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
		public struct Message
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public IntPtr hWnd;

			/// <summary>
			/// TODO
			/// </summary>
			public uint msg;

			/// <summary>
			/// TODO
			/// </summary>
			public IntPtr wParam;

			/// <summary>
			/// TODO
			/// </summary>
			public IntPtr lParam;

			/// <summary>
			/// TODO
			/// </summary>
			public uint time;

			/// <summary>
			/// TODO
			/// </summary>
			public System.Drawing.Point p;

			#endregion
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets if there are messages waiting to be processed or not
		/// </summary>
		private bool AppStillIdle
		{
			get
			{
				Message msg;
				return !PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// PeekMessage pInvoke: We will not use this maliciously
		/// </summary>
        /// <param name="msg">Pointer to an MSG structure that receives message information.</param>
        /// <param name="hWnd">Handle to the window whose messages are to be examined. The window must belong to the current thread.</param>
		/// <param name="messageFilterMin">Specifies the value of the first message in the range of messages to be examined. Use WM_KEYFIRST to specify the first keyboard message or WM_MOUSEFIRST to specify the first mouse message.  If wMsgFilterMin and wMsgFilterMax are both zero, PeekMessage returns all available messages (that is, no range filtering is performed).</param>
		/// <param name="messageFilterMax">Specifies the value of the last message in the range of messages to be examined. Use WM_KEYLAST to specify the last keyboard message or WM_MOUSELAST to specify the last mouse message.  If wMsgFilterMin and wMsgFilterMax are both zero, PeekMessage returns all available messages (that is, no range filtering is performed).</param>
		/// <param name="flags">Specifies how messages are handled. This parameter can be one of the following values: PM_NOREMOVE or PM_REMOVE</param>
        /// <returns>If a message is available, the return value is nonzero.</returns>
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("User32.dll", CharSet = CharSet.Auto)]
		public static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="sender">TODO</param>
		/// <param name="e">TODO</param>
		private void OnApplicationIdle(object sender, EventArgs e)
		{
			// Render a frame during idle time (no messages are waiting)
			while (AppStillIdle)
			{
				// Begin application update
				Jad.BeginUpdate();

				// Update the application. If Action() returns false means we want to quit the application
				bool bContinue = Update();

				// End application update
				Jad.EndUpdate();

				if (bContinue)
				{
					// Render scene
					Render();
				}
				else
				{
					// End application
					End();

					// Free resources and shutdown the engine
					Jad.End();

					// Close the main form
					Jad.Form.Close();
				}
			}
		}

		/// <summary>
		/// Starts the application
		/// </summary>
		public virtual void Start()
		{
			// Init adapter parameters with values read from the configuration file
			Jad.Video.Adapter.Initialization.ReadFromConfig();

			// Create the main form
			Jad.CreateStandardForm();

			// Create the device
			Jad.Video.Adapter.CreateDevice(Jad.Form);

			// Initialize game
			if (InitGame(Jad.Form))
			{

				// More information on handling messages like this, here:
				// http://blogs.msdn.com/tmiller/archive/category/2068.aspx
				Application.Idle += new EventHandler(OnApplicationIdle);
				Application.Run(Jad.Form);
			}

			else
			{
				// There were problems
				MessageBox.Show("Can't init the game.");

				// Close the main form
				Jad.Form.Close();
			}
		}

		/// <summary>
		/// Override this method and put your game initialization code into it
		/// </summary>
		/// <param name="form">The main form</param>
		/// <returns>Returns false if something went wrong, otherwise returns true</returns>
		public virtual bool InitGame(Form form)
		{
			return true;
		}

		/// <summary>
		/// Override this method and put your game update code into
		/// </summary>
		/// <returns>Returns false if you want exit from the application, otherwise returns true</returns>
		public virtual bool Update()
		{
			return true;
		}

		/// <summary>
		/// Override this method and put your game render code into
		/// </summary>
		public virtual void Render()
		{
		}

		/// <summary>
		/// Override this method and put your shutdown code into
		/// </summary>
		public virtual void End()
		{
		}

		#endregion
	}
}
