#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Mathematics.Geometry;
using JadEngine.Input;
using JadEngine.Video;

#endregion

namespace JadEngine.Scene
{

	/// <summary>
	/// TODO
	/// </summary>
	public class JCameraFirstPerson : JCamera
	{
		#region Delegates

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="before">TODO</param>
		/// <param name="after">TODO</param>
		/// <returns>TODO</returns>
		public delegate Vector3 CheckCollision(Vector3 before, Vector3 after);

		#endregion

		#region Fields

		/// <summary>
		/// Mouse sensitivity
		/// </summary>
		protected float sensitivity;

		/// <summary>
		/// Traslation speed
		/// </summary>
		protected float speed;

		/// <summary>
		/// Indicates if the camera must collide with the Newton world
		/// </summary>
		protected bool useWorldColision;

		/// <summary>
		/// TODO
		/// </summary>
		public CheckCollision CheckCollisionDelegate;

		/// <summary>
		/// Distance form the camera to check if the line segment collide with the world
		/// </summary>
		public float WorldCollisionDistance = 0.5f;

		// Input keys
		Key[] _keysMoveX;
		Key[] _keysMoveMinusX;
		Key[] _keysMoveY;
		Key[] _keysMoveMinusY;
		Key[] _keysMoveZ;
		Key[] _keysMoveMinusZ;

		DirectionalPad[] _dpMoveX;
		DirectionalPad[] _dpMoveMinusX;
		DirectionalPad[] _dpMoveY;
		DirectionalPad[] _dpMoveMinusY;
		DirectionalPad[] _dpMoveZ;
		DirectionalPad[] _dpMoveMinusZ;

		#endregion

		#region Constructors

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="name">Name of the camera</param>
		/// <param name="sensitivity">Mouse sensitivity</param>
		/// <param name="speed">Traslation speed of the camera</param>
		/// <param name="cameras">The parent object</param>
		public JCameraFirstPerson(string name, float sensitivity, float speed, JCameras cameras)
			: base(name, cameras)
		{
			this.sensitivity = sensitivity;
			this.speed = speed;
			LoadConfiguration();
		}

		/// <summary>
		/// Defaul constructor
		/// </summary>
		/// <param name="name">Name of the camera</param>
        /// <param name="cameras">The parent object</param>
        public JCameraFirstPerson(string name,JCameras cameras)
			: this(name,1.0f,0.8f,cameras)
		{}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="name">Name of the camera</param>
		/// <param name="fov">Camera FOV</param>
		/// <param name="nPlane">Distance of the near plane of the camera</param>
		/// <param name="fPlane">Distance of the far plane of the camera</param>
		/// <param name="sensitivity">Mouse sensitivity</param>
		/// <param name="speed">Traslation speed of the camera</param>
        /// <param name="cameras">The parent object</param>
        public JCameraFirstPerson(string name, float fov, float nPlane, float fPlane, float sensitivity, float speed, JCameras cameras)
			: base(name, fov, fov, nPlane, fPlane,cameras)
		{
			this.sensitivity = sensitivity;
			this.speed = speed;
		}

        /// <summary>Load from a binary importer</summary>
        /// <param name="import">A valid object</param>
        /// <param name="nodeName">The node name</param>
		public JCameraFirstPerson(JBinaryImport import, string nodeName)
			: this(nodeName, null)
		{
			this.cameras = import.Scene.Cameras;

			cameras.Add(this);

			Load(import);
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets if the camera must collide with the Newton world
		/// </summary>
		public bool UseWorldColision
		{
			get { return useWorldColision; }
			set { useWorldColision = value; }
		}

		/// <summary>
		/// Gets or sets the mouse sensitivity
		/// </summary>
		public float Sensitivity
		{
			get { return sensitivity; }
			set { sensitivity = value; }
		}

		/// <summary>
		/// Gets or sets the traslation speed
		/// </summary>
		public float Speed
		{
			get { return speed; }
			set { speed = value; }
		}

		#endregion

		#region Methods

		#region Private Methods

		/// <summary>
		/// Load all of our info from the config file
		/// </summary>
		private void LoadConfiguration()
		{
			UseInput = Jad.Configuration.ReadValue<bool>("Cameras/JCameraFirstPerson/Input", "UseInput", UseInput);

			_keysMoveX = GetKeysFromConfig("Cameras/JCameraFirstPerson/Input/MoveX", "D");
			_keysMoveY = GetKeysFromConfig("Cameras/JCameraFirstPerson/Input/MoveY", "None");
			_keysMoveZ = GetKeysFromConfig("Cameras/JCameraFirstPerson/Input/MoveZ", "W");

			_keysMoveMinusX = GetKeysFromConfig("Cameras/JCameraFirstPerson/Input/Move-X", "A");
			_keysMoveMinusY = GetKeysFromConfig("Cameras/JCameraFirstPerson/Input/Move-Y", "None");
			_keysMoveMinusZ = GetKeysFromConfig("Cameras/JCameraFirstPerson/Input/Move-Z", "S");

			_dpMoveMinusX = GetDPadFromConfig("Cameras/JCameraFirstPerson/Input/Move-X",new DirectionalPad[] {DirectionalPad.Left,DirectionalPad.DownLeft,DirectionalPad.UpLeft});
			_dpMoveX = GetDPadFromConfig("Cameras/JCameraFirstPerson/Input/MoveX", new DirectionalPad[] { DirectionalPad.Right, DirectionalPad.DownRight, DirectionalPad.UpRight });
			_dpMoveMinusY = GetDPadFromConfig("Cameras/JCameraFirstPerson/Input/Move-Y", new DirectionalPad[] { DirectionalPad.Down, DirectionalPad.DownRight, DirectionalPad.DownLeft });
			_dpMoveY = GetDPadFromConfig("Cameras/JCameraFirstPerson/Input/MoveY", new DirectionalPad[] { DirectionalPad.Up, DirectionalPad.UpLeft, DirectionalPad.UpRight });
			_dpMoveMinusZ = GetDPadFromConfig("Cameras/JCameraFirstPerson/Input/Move-Z", new DirectionalPad[] { });
			_dpMoveZ = GetDPadFromConfig("Cameras/JCameraFirstPerson/Input/MoveZ", new DirectionalPad[] { });
		}

		/// <summary>
		/// Get the key values from the configuration file
		/// </summary>
		private Key[] GetKeysFromConfig(string path, string defaultKey)
		{
			Key[] keys;
			string[] sKeys = Jad.Configuration.ReadValueArray<string>(path, "Key");
			if (sKeys.Length > 0)
			{
				keys = new Key[sKeys.Length];
				for (int i = 0; i < keys.Length; ++i)
					keys[i] = JKeyConverter.ConvertFrom(sKeys[i]);
			}
			else
			{
				keys = new Key[1];
				keys[0] = JKeyConverter.ConvertFrom(defaultKey);
			}
			return keys;
		}

		/// <summary>
		/// Get directional pad values from the configuration file
		/// </summary>
		private DirectionalPad[] GetDPadFromConfig(string path, DirectionalPad[] defaults)
		{
			DirectionalPad[] directions;
			string[] sDirs = Jad.Configuration.ReadValueArray<string>(path, "DPad");
			if (sDirs.Length > 0)
			{
				directions = new DirectionalPad[sDirs.Length];
				for (int i = 0; i < sDirs.Length; ++i)
				{
					try
					{
						directions[i] = (DirectionalPad) Enum.Parse(typeof(DirectionalPad), sDirs[i]);
					}
					catch
					{ }
				}
			}
			else
				directions = defaults;
			return directions;
		}

		#endregion

		/// <summary>
		/// Updates the camera
		/// </summary>
		public override void UpdateAction()
		{
			float time = Jad.Timer.DeltaTime;

			if (UseInput)
			{
				// Camera position
				JInputBuffer inputBuffer = this.InputView.Input;

				Vector3 prevPos = Transform.Position, PrevPos2 = Transform.Position;
				Vector3 prevLocalPos = Transform.LocalPosition;

				MoveX(-inputBuffer.Joystick.Stick[0] * 5, time);
				MoveY(inputBuffer.Joystick.Stick[1] * 5, time);
				MoveZ(inputBuffer.Joystick.Stick[2] * 5, time);

				foreach (Key key in _keysMoveX ) if (inputBuffer.Keyboard[key].Down)  MoveX(speed, time);
				foreach (Key key in _keysMoveY) if (inputBuffer.Keyboard[key].Down) MoveY(speed, time);
				foreach (Key key in _keysMoveZ) if (inputBuffer.Keyboard[key].Down) MoveZ(speed, time);
				foreach (Key key in _keysMoveMinusX) if (inputBuffer.Keyboard[key].Down) MoveX(-speed, time);
				foreach (Key key in _keysMoveMinusY) if (inputBuffer.Keyboard[key].Down) MoveY(-speed, time);
				foreach (Key key in _keysMoveMinusZ) if (inputBuffer.Keyboard[key].Down) MoveZ(-speed, time);

				foreach (DirectionalPad dpad in _dpMoveMinusX) if (inputBuffer.Joystick.DirectionalPad == dpad) MoveX(1f, time);
				foreach (DirectionalPad dpad in _dpMoveX) if (inputBuffer.Joystick.DirectionalPad == dpad) MoveX(-1f, time);
				foreach (DirectionalPad dpad in _dpMoveMinusY) if (inputBuffer.Joystick.DirectionalPad == dpad) MoveY(1f, time);
				foreach (DirectionalPad dpad in _dpMoveY) if (inputBuffer.Joystick.DirectionalPad == dpad) MoveY(-1f, time);
				foreach (DirectionalPad dpad in _dpMoveMinusZ) if (inputBuffer.Joystick.DirectionalPad == dpad) MoveZ(1f, time);
				foreach (DirectionalPad dpad in _dpMoveZ) if (inputBuffer.Joystick.DirectionalPad == dpad) MoveZ(-1f, time);

				if (inputBuffer.Mouse.RightButton)
					MoveY(speed, time);

				if (inputBuffer.Mouse.LeftButton)
					MoveY(-speed, time);
				
				// Calculate the wheel movement

				float zDesp = inputBuffer.Mouse.Wheel;

				if (zDesp != 0)
				{
					zDesp *= 0.1f;
					MoveZ(speed * zDesp, time);
				}

				// Simulate the final position
				Vector3 newPosition = Transform.Position + (Transform.LocalPosition - prevLocalPos);

				// If there was a collision, get the original position
				if (CheckWorldCollision(newPosition, ref prevPos))
				{
					Transform.Position = prevPos;
					Transform.LocalPosition = prevLocalPos + (Transform.Position - PrevPos2);
				}

				// Calculate the camera position

				CalculateMouseRotation(inputBuffer.Mouse.X, inputBuffer.Mouse.Y);
			}

		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="relativeX">TODO</param>
		/// <param name="relativeY">TODO</param>
		public void CalculateMouseRotation(int relativeX, int relativeY)
		{
			Vector2 pos = new Vector2();
			pos.X = relativeX;
			pos.Y = relativeY;

			float mouseX = pos.X * sensitivity;
			float mouseY = pos.Y * sensitivity;

			float pitch = Transform.Angles.X + Geometry.DegreeToRadian(mouseY * 0.01f);
			float yaw = Transform.Angles.Y + Geometry.DegreeToRadian(mouseX * 0.01f);

			// Pitch limits
			if (pitch > Geometry.DegreeToRadian(90.0f))
				pitch = Geometry.DegreeToRadian(90.0f);

			else if (pitch < Geometry.DegreeToRadian(-90.0f))
				pitch = Geometry.DegreeToRadian(-90.0f);

			// Yaw limits
			if (yaw >= Geometry.DegreeToRadian(360.0f))
				yaw = 0.0f;

			else if (yaw <= Geometry.DegreeToRadian(-360.0f))
				yaw = 0.0f;

			// Update the angles
			Transform.Angles = new Vector3(pitch, yaw, 0.0f);
		}

		/// <summary>
		/// TODO
		/// </summary>
		protected override void Update()
		{
			base.Update();
		}

		/// <summary>
		/// Checks collisions with the world
		/// </summary>
		/// <param name="newPosition">TODO</param>
		/// <param name="pos">TODO</param>
		/// <returns>True if there was a collision, false if not</returns>
		private bool CheckWorldCollision(Vector3 newPosition, ref Vector3 pos)
		{
			float p;

			if (useWorldColision)
			{
				// If theres a delegate, call it
				if (CheckCollisionDelegate != null)
				{
					pos = CheckCollisionDelegate(newPosition, pos);

					return true;
				}

				else
				{
					// Test the position where the camera was and the direction
					// it has * WorldCollisionDistance meters
					Vector3 dir = newPosition - pos;
					dir.Normalize();
					Vector3 p0 = pos;
					Vector3 p1 = pos + dir * WorldCollisionDistance;

					return Jad.Physics.WorldRayCast(ref p0, ref p1, out p);
				}
			}

			return false;
		}

		/// <summary>
		/// Moves the camera in the X axis
		/// </summary>
		/// <param name="speed">Displacement speed</param>
		public void MoveX(float speed)
		{
			MoveX(speed, 1f);
		}

		/// <summary>
		/// Moves the camera in the Y axis
		/// </summary>
		/// <param name="speed">Displacement speed</param>
		public void MoveY(float speed)
		{
			MoveY(speed, 1f);
		}

		/// <summary>
		/// Moves the camera in the Z axis
		/// </summary>
		/// <param name="speed">Displacement speed</param>
		public void MoveZ(float speed)
		{
			MoveZ(speed, 1f);
		}

		/// <summary>
		/// Moves the camera in the X axis
		/// </summary>
		/// <param name="speed">Displacement speed</param>
		/// <param name="time">Time increment since last update</param>
		private void MoveX(float speed, float time)
		{
			Transform.LocalPosition += speed * time * Right;
		}

		/// <summary>
		/// Moves the camera in the Y axis
		/// </summary>
		/// <param name="speed">Displacement speed</param>
		/// <param name="time">Time increment since last update</param>
		private void MoveY(float speed, float time)
		{
			Transform.LocalPosition += speed * time * Up;
		}

		/// <summary>
		/// Moves the camera in the Z axis
		/// </summary>
		/// <param name="speed">Displacement speed</param>
		/// <param name="time">Time increment since last update</param>
		private void MoveZ(float speed, float time)
		{
			Transform.LocalPosition += speed * time * Look;
		}

		#endregion

		#region Serializable

		/// <summary>
		/// Current FileVersion
		/// </summary>
		int fileVersion = 1;

		/// <summary>
		/// Loads the camera values from the binary format
		/// </summary>
		/// <param name="import">The binary importer</param>
		public bool Load(JBinaryImport import)
		{

			int fileVersionLoader = import.LoadInt32();

			base.Load(import);

			sensitivity = import.LoadSingle();
			speed = import.LoadSingle();
			useWorldColision = import.LoadBoolean();
			WorldCollisionDistance = import.LoadSingle();
			UseInput = import.LoadBoolean();

			return true;
		}

		/// <summary>
		/// Save the camera information.
		/// </summary>
		/// <param name="path">The path to save the files</param>
		/// <returns>The filename used to find this file</returns>
		public override string Save(string path)
		{
			string fileName = @"\Cameras\" + name;

			if (!Directory.Exists(path + @"\Cameras\"))
				Directory.CreateDirectory(path + @"\Cameras\");

			JBinaryExport export = new JBinaryExport(path + fileName);

			// Save the class name
			export.Save(this.GetType().FullName);

			export.Save(fileVersion);

			// Save the camera information
			base.Save(export);

			export.Save(sensitivity);
			export.Save(speed);
			export.Save(useWorldColision);
			export.Save(WorldCollisionDistance);
			export.Save(UseInput);

			export.Dispose();

			return fileName;
		}

		#endregion
	}
}
