#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Mathematics.Geometry;
using JadEngine.Input;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JCameraSelfDriven : JCamera
	{
		#region Fields

		/// <summary>
		/// Path the camera will follow
		/// </summary>
		private JCameraPathAnimator path;

		/// <summary>
		/// Tells if the camera values must be updated or not
		/// </summary>
		public bool UpdateCamera = true;

		#endregion

		#region Constructors

		/// <summary>
		/// Defaul constructor
		/// </summary>
		/// <param name="name">Name of the camera</param>
		/// <param name="loop">TODO</param>
        /// <param name="cameras">The parent object</param>
		public JCameraSelfDriven(string name, bool loop, JCameras cameras)
			: base(name, cameras)
		{
			try
			{
				path = new JCameraPathAnimator(name, loop);
			}

			catch (Exception e)
			{
				throw e;
			}
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="name">Name of the camera</param>
		/// <param name="fileName">TODO</param>
		/// <param name="fov">Camera FOV</param>
		/// <param name="nPlane">Distance of the near plane of the camera</param>
		/// <param name="fPlane">Distance of the far plane of the camera</param>
        /// <param name="cameras">The parent object</param>
        public JCameraSelfDriven(string name, string fileName, float fov, float nPlane, float fPlane, JCameras cameras)
			: base(name, fov, fov, nPlane, fPlane,cameras)
		{
			try
			{
				path = new JCameraPathAnimator(fileName, true);
			}

			catch (Exception e)
			{
				throw e;
			}
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the camera path
		/// </summary>
		public JCameraPathAnimator Path
		{
			get { return path; }
			set
			{
				path = value;
				if (path != null) path.Reset();
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Updates the camera
		/// </summary>
		protected override void Update()
		{
			if (!UpdateCamera)
				return;

			if (path == null)
				return;

			float time = (float) Jad.Timer.Time;

			JPRFAnimationKey key = (JPRFAnimationKey) path.AtTime(time);

			xfov = key.FOV;
			Transform.Position = key.Position;
			Transform.Rotation = key.Rotation;

			base.Update();
		}

		#endregion
	}
}
