#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Mathematics.Geometry;
using JadEngine.Particles;
using JadEngine.Physics;
using JadEngine.Video;
using JadEngine.Tools;

#endregion

namespace JadEngine.Scene
{
	#region MeshObjects

	/// <summary>
	/// TODO
	/// </summary>
	public enum JSortMeshObjectBy
	{
		/// <summary>
		/// TODO
		/// </summary>
		Priority,

		/// <summary>
		/// TODO
		/// </summary>
		Name
	}

	/// <summary>
	/// Class that sorts JMeshObjects by priority
	/// </summary>
	public class JSortJMeshObjectsPriority : IComparer<JMeshObject>
	{
		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="a">TODO</param>
		/// <param name="b">TODO</param>
		/// <returns>TODO</returns>
		public int Compare(JMeshObject a, JMeshObject b)
		{
			if (a.Priority < b.Priority)
				return -1;

			if (a.Priority == b.Priority)
				return 0;

			return 1;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="x">TODO</param>
		/// <param name="y">TODO</param>
		/// <returns>TODO</returns>
		/// <remarks>Not implemented</remarks>
		public bool Equals(JMeshObject x, JMeshObject y)
		{
			throw new NotImplementedException();
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="obj">TODO</param>
		/// <returns>TODO</returns>
		/// <remarks>Not implemented</remarks>
		public int GetHashCode(JMeshObject obj)
		{
			throw new NotImplementedException();
		}

		#endregion
	}

	/// <summary>
	///Class that sorts JMeshObjects by name
	/// </summary>
	public class JSortJMeshObjectsName : IComparer<JMeshObject>
	{
		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="a">TODO</param>
		/// <param name="b">TODO</param>
		/// <returns>TODO</returns>
		public int Compare(JMeshObject a, JMeshObject b)
		{
			if (a.Name.CompareTo(b.Name) < 0)
				return -1;

			if (a.Name == b.Name)
				return 0;

			return 1;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="x">TODO</param>
		/// <param name="y">TODO</param>
		/// <returns>TODO</returns>
		/// <remarks>Not implemented</remarks>
		public bool Equals(JMeshObject x, JMeshObject y)
		{
			throw new NotImplementedException();
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="obj">TODO</param>
		/// <returns>TODO</returns>
		/// <remarks>Not implemented</remarks>
		public int GetHashCode(JMeshObject obj)
		{
			throw new NotImplementedException();
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JMeshObjects
	{
		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		public struct JInHouse
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public JMeshObject Axes;

			#endregion
		}

		#endregion

		#region Fields

        /// <summary>The parent scene</summary>
        JScene scene;

		/// <summary>The MeshObject List</summary>
		public JListUnique<JMeshObject> Items;

		/// <summary>
		/// BB of the scene
		/// </summary>
		/// <remarks>Only updated when the method is called</remarks>
		public JBoundingBox3 BoundingBox;

		/// <summary>
		/// Default objects created with the engine
		/// </summary>
		public JInHouse InHouse;

		/// <summary>
		/// Value added to all Bumpiness when the mesh are rendered
		/// </summary>
		public float Bumpiness = 0.0f;

		/// <summary>
		/// Value added to all ParallaxAmount when the mesh are rendered
		/// </summary>
		public float ParallaxAmount = 0.0f;

		#endregion

        #region Properties

        /// <summary>Return the scene linked to this object</summary>
        public JScene Scene
        {
            get
            {
                return scene;
            }
        }

        #endregion

        #region List Properties

        /// <summary>
		/// Look for the list by name. The name MUST be at the list
		/// </summary>
		/// <value>The element found</value>
		public JMeshObject this[string name]
		{
			get { return Items[name]; }
		}

		/// <summary>
		/// Look for the list by index
		/// </summary>
		/// <value>The element found</value>
		public JMeshObject this[int index]
		{
			get { return Items[index]; }
		}

		/// <summary>
		/// Gets the number of elements in the collection.
		/// </summary>
		/// <value>The count of elements</value>
		public int Count
		{
			get { return Items.Count; }
		}

		/// <summary>
		/// Gets the listData elements, compatible with IEnumerator
		/// </summary>
		public List<JMeshObject> List
		{
			get { return Items.List; }
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JMeshObjects(JScene scene)
		{
            this.scene = scene;

			BoundingBox = new JBoundingBox3();

			Items = new JListUnique<JMeshObject>();
		}

		#endregion

		#region Methods

		/// <summary>
		/// Loads some default MeshObjects
		/// </summary>
		public void LoadInHouse()
		{
		}

		/// <summary>
		/// Sorts the mesh list
		/// </summary>
		/// <param name="sort">Sort criteria</param>
		public void Sort(JSortMeshObjectBy sort)
		{
			if (sort == JSortMeshObjectBy.Priority)
				List.Sort(new JSortJMeshObjectsPriority());

			else
				List.Sort(new JSortJMeshObjectsName());
		}

		/// <summary>
		/// Inserts a MeshObject on the list, creating the <see cref="JMesh"/> object
		/// </summary>
		/// <param name="name">Unique objects name</param>
		/// <returns>A valid JMeshObject</returns>
		public JMeshObject Create(string name)
		{
			return Create(name, true);
		}

		/// <summary>
		/// Inserts a MeshObject on the list, creating the <see cref="JMesh"/> object
		/// and indicating the type of class to be created
		/// </summary>
		/// <param name="name">Unique objects name</param>
		/// <param name="type">Type of the mesh</param>
		/// <returns>A valid JMeshObject</returns>
		public JMeshObject Create(string name, Type type)
		{
			JMeshObject mesh = Items.Exists(name);

			if (mesh != null) return mesh;

			mesh = (JMeshObject)Activator.CreateInstance(type, name, true,this);

			Add(mesh);

			return mesh;
		}

		/// <summary>
		/// Inserts a MeshObject on the list
		/// </summary>
		/// <param name="name">Unique objects name</param>
		/// <param name="createMesh">true to create the <see cref="JMesh"/> associated with the JMeshObject</param>
		/// <returns>A valid JMeshObject</returns>
		public JMeshObject Create(string name, bool createMesh)
		{
			JMeshObject mesh = Items.Exists(name);

			if (mesh != null) return mesh;

			mesh = new JMeshObject(name, createMesh,this);

			Add(mesh);

			return mesh;
		}

		/// <summary>
		/// Adds a created JMeshObject to the list and scene graph
		/// </summary>
		/// <param name="meshObject">A valid JMeshObject</param>
		public void Add(JMeshObject meshObject)
		{
			Items.Add(meshObject);

			// Add the mesh to the scene graph
			scene.Root.SceneGraph.AddChild(meshObject);
		}

		/// <summary>
		/// Deletes all the elements, but NOT remove them from the scene graph
		/// </summary>
		public void Destroy()
		{
			Items.Clear();
		}

		/// <summary>
		/// Deletes an item from the list and remove it from the scene graph
		/// </summary>
		/// <param name="item">A valid object</param>
		public void Delete(JMeshObject item)
		{
			Items.Delete(item);
			item.SceneGraph.Delete();
		}

		/// <summary>
		/// Calculates the tangent and the normal of all meshes
		/// </summary>
		public void ComputeTangent()
		{
			// Calculate the tangent-space
			for (int j = 0; j < Count; j++)
				this[j].Mesh.ComputeTangent();
		}

		/// <summary>
		/// Calculates the bounding box of all meshes
		/// </summary>
		public void ComputeBoundingBox()
		{
			BoundingBox.Reset();

			for (int j = 0; j < Count; j++)
				BoundingBox += this[j].Bounding.BoundingBox;
		}

		/// <summary>
		/// Searches the object that intersects a line
		/// </summary>
		/// <param name="start">The start ray point</param>
		/// <param name="end">The end ray point</param>
		/// <param name="backFaceCulling">Set to False to test back faces</param>
		/// <returns>The nearest MeshObject</returns>
		public JMeshObject Intersect(Vector3 start, Vector3 end, bool backFaceCulling)
		{
			JMeshObject collision = null;
			float distance = float.MaxValue;

			for (int j = 0; j < Count; j++)
			{
				JMeshObject m = this[j];

				float dist;

				if (m.Intersect(start, end, backFaceCulling, out dist))
				{
					// Get the closest one
					if (dist < distance)
					{
						collision = m;

						distance = dist;
					}
				}
			}

			return collision;
		}

		#endregion

	}

	#endregion
}