#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{

		#region AmbientOnlyPass

		/// <summary>
		/// Renders the ambient pass
		/// </summary>
		void RenderAmbientPass()
		{
			RenderAmbientPassPerLayer(0);
			ProcessLayers(1, null);
		}

		/// <summary>
		/// Renders the layers list of meshes for the ambient pass
		/// </summary>
		/// <param name="numLayer">Number of layer</param>
		void RenderAmbientPassPerLayer(int numLayer)
		{
			if (lights.Ambient.IsBlack()) return;

			// Set the rendering status
			status = JSceneStatus.Rendering;

			// Now to be able to render the layers correctly
			// Render all layers appart and then join them

			// The first layer is rendered normally

			if (numLayer == 0) Jad.Video.StateManager.SetRenderState(RenderStates.SeparateAlphaBlendEnable, false);

			JColor ambient = lights.Ambient;

			List<LayersForLighting> list = listLayers[numLayer];

			for (int i = 0; i < list.Count; i++)
			{
				LayersForLighting lfl = list[i];

				JMaterial mat = lfl.layer.Parent;

				mat.Begin();

				JMaterialLayer layer = lfl.layer;

				layer.Begin();

				// Create the effect for this pass

				JEffect effect = Jad.EffectBuilder.SetLightPass(null, layer, false);

				int passes = effect.Begin();

				// Set the material values

				layer.ApplyEffect(effect);

				for (int p = 0; p < mat.meshObjectPerMaterial.Count; p++)
				{
					JMeshObjectPerMaterial mp = mat.meshObjectPerMaterial[p];
					JMeshObject mesh = mp.meshObject;

					if (!mp.rendered[numLayer])
					{
						bool shadows = scene.RenderWithShadows;

						// If it doesnt recieve shadows, tell the builder that we dont want them
						if (!mesh.ReceiveShadows && scene.Shadows.ShadowMode == JShadowMode.ShadowMap) scene.RenderWithShadows = false;

						// The backbuffer is needed in the glow, so set the ambient to one
						if (layer.EmissiveMap.Enabled)
							lights.Ambient.ToWhite();

						// If the object is not affected by light, its because it is a self-ilum object
						// so, add the emissive color to the ambient color
						if (!mesh.AffectLight)
							lights.Ambient += layer.EmissiveMap.Color;

						if (mat.Priority != JMaterialPriority.Transparency)
						{
							Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, true);
							Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.One);
							Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.One);
						}

						effect.SetVector4(effect.handlers.env.Ambient_TimeElapsed, lights.Ambient.Red, lights.Ambient.Green, lights.Ambient.Blue, Jad.Timer.Time);

						mesh.RenderBase(mp.numMat, passes, effect);

						// Restore the color
						lights.Ambient = ambient;

						// Restore the initial state
						scene.RenderWithShadows = shadows;
					}
				}

				effect.End();
				layer.End();
				mat.End();
			}
		}

		#endregion

	}
}