#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{
		#region structs

		/// <summary>
		/// TODO
		/// </summary>
		internal struct LayersForLighting
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			internal int numMat;

			/// <summary>
			/// TODO
			/// </summary>
			internal JMaterialLayer layer;

			#endregion
		}

		/// <summary>
		/// Class used to sort <see cref="LayersForLighting"/>
		/// </summary>
		internal class JSortLayersForLighting : IComparer<LayersForLighting>
		{
			#region Methods

			/// <summary>
			/// TODO
			/// </summary>
			/// <param name="a">TODO</param>
			/// <param name="b">TODO</param>
			/// <returns>TODO</returns>
			public int Compare(LayersForLighting a, LayersForLighting b)
			{
				if ((int)a.layer.RenderStates.SourceBlend < (int)b.layer.RenderStates.SourceBlend)
					return -1;

				if ((int)a.layer.RenderStates.SourceBlend == (int)b.layer.RenderStates.SourceBlend)
					return 0;

				return 1;
			}

			/// <summary>
			/// TODO
			/// </summary>
			/// <param name="x">TODO</param>
			/// <param name="y">TODO</param>
			/// <returns>TODO</returns>
			/// <remarks>Not implemented</remarks>
			public bool Equals(LayersForLighting x, LayersForLighting y)
			{
				throw new NotImplementedException();
			}

			/// <summary>
			/// TODO
			/// </summary>
			/// <param name="obj">TODO</param>
			/// <returns>TODO</returns>
			/// <remarks>Not implemented</remarks>
			public int GetHashCode(LayersForLighting obj)
			{
				throw new NotImplementedException();
			}

			#endregion
		}

		#endregion

		///<summary>
		/// Array of layers to render separately. Used on light and ambient passs
		/// </summary>
		internal List<LayersForLighting>[] listLayers;

        /// <summary>
        /// Create a JSceneRenderer2_0 for a scene
        /// </summary>
        /// <param name="scene">The scene</param>
		public JSceneRenderer2_0(JScene scene)
			: base(scene)
		{
			// Create the lists used for the multilayer render
			listLayers = new List<LayersForLighting>[Jad.MaxLayers - 1];
			for (int i = 0; i < Jad.MaxLayers - 1; i++)
				listLayers[i] = new List<LayersForLighting>();

		}

        /// <summary>
        /// Called after scene.Begin
        /// </summary>
		public override void Begin()
		{
			base.Begin();
		}

        /// <summary>
        /// The render method
        /// </summary>
		public override void Render()
		{
			base.Render();

			CreateSortedMaterialList(MeshObjectsToRender);

			if (scene.WireFrameMode == JWireframeMode.OnlyWireFrame)
			{
				RenderZPass();

				RenderWireFramePass();
			}

			else
			{
				RenderZPass();

				// The ZBuffer writting its disabled here

				RenderParallaxPass();

                //view.RenderTargets.LastFrame.CopyFrom(view.BackBuffer);
                //view.RenderTargets.LastFrame.Texture.Save("Parallax", ImageFileFormat.Jpg);

                if (scene.RenderSkyBoxFirst && scene.skyBox != null) scene.skyBox.Render();

				scene.status = JSceneStatus.Rendering;

				// Loop the lights

				if (lights.Enabled)
					for (int i = 0; i < lights.Count; i++)
						RenderLightPass(lights[i]);

				RenderAmbientPass();

				RenderMotionBlurPass();

				if (scene.WireFrameMode == JWireframeMode.Render_Plus_Wireframe)
					RenderWireFramePass();

				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, true);

				//Jad.Video.Device.SetRenderState(RenderStates.FillMode, (int)FillMode.WireFrame);

				Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);

				// Render the effect of the lights
				lights.RenderShafts();

				lights.RenderCoronas();

				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);

				RenderGlowDOFPass();

				// To render some objects we give them access to the backbuffer.
				// Copy the backbuffer to do this

				if (scene.Fog.Type != JFogType.None) RenderFogPass();
			}

			Jad.Video.StateManager.SetRenderState(RenderStates.AlphaTestEnable, false);

			if (scene.RenderPostObjects)
			{
				//JRenderTarget rt;

				Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);

                CopyBackBufferToBackBufferTexture();
                /*
                // As we render to the backbuffer,
                // we must copy the primary surface to be used for the post objects
                rt = view.RenderTargets.Primary;

				scene.BackBuffer = rt;

				JTexture texture = (JTexture)scene.BackBuffer.Texture;

                texture.CopyFromSurface(view.BackBuffer);
                */
				scene.status = JSceneStatus.PostPass;

				for (int i = 0; i < MeshObjectsToRenderPostProcess.Count; i++)
				{
					JMeshObject mesh = MeshObjectsToRenderPostProcess[i];

					if (mesh.Visible)
					{
						mesh.UpdateShaders();

						mesh.RenderPostPass();
					}
				}

			}

			Jad.Video.StateManager.SetRenderState(RenderStates.CullMode, (int) Cull.CounterClockwise);
			Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);
			scene.status = JSceneStatus.Stopped;

			if (!scene.RenderSkyBoxFirst && scene.skyBox != null) scene.skyBox.Render();
		}

        /// <summary>
        /// Called on scene.End
        /// </summary>
		public override void End()
		{
			base.End();
		}

		/// <summary>
		/// Creates a sortered list from the materials list
		/// </summary>
		/// <param name="list">TODO</param>
		void CreateSortedMaterialList(List<JMeshObject> list)
		{
			// Delete the lists used for the multilayer render
			for (int i = 0; i < Jad.MaxLayers - 1; i++)
				listLayers[i].Clear();

			for (int i = 0; i < materials.Count; i++)
			{
				JMaterial mat = materials[i];

				if (mat.meshObjectPerMaterial.Count == 0) continue;

				// Generate the list of layers to sort it by blending
				for (int numLayer = 0; numLayer < mat.Layers.Count; numLayer++)
				{
					LayersForLighting l = new LayersForLighting();

					l.layer = mat.Layers[numLayer];

					listLayers[numLayer].Add(l);
				}
			}
		}

	}
}