#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{
		#region MotionBlurPass

		/// <summary>
		/// Renders the motion blur pass
		/// </summary>
		void RenderMotionBlurPass()
		{
			if (!scene.MotionBlur.Enabled) return;

			// Calculate the new camera position to calculate the velocity
			// This is needed because the values after the near plane return bad values
			// that cause jaggies in the screen

			Matrix viewProj;

			if (scene.MotionBlur.DistanceToCamera == 0)
				viewProj = camera.ViewProjection;

			else
			{
				Matrix view = camera.View;
				Vector3 newPos = camera.Transform.Position - camera.Look * scene.MotionBlur.DistanceToCamera;
				view.M41 = -newPos.X;
				view.M42 = -newPos.Y;
				view.M43 = -newPos.Z;
				viewProj = view * camera.Projection;
			}

			Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);

			status = JSceneStatus.MotionBlurPass;

			Jad.Video.Views.currentView.RenderTargets.MotionBlur.Begin();

			Jad.Video.Views.currentView.Clear(ClearFlags.Target, new JColor(0, 0, 0, 0), 0, 0);

			for (int i = 0; i < materials.Count; i++)
			{
				JMaterial m = materials[i];

				// If there arent objects to render continue
				if (m.meshObjectPerMaterial.Count == 0) continue;

				JMaterialLayer layer = m.Layers[0];

				m.Begin();

				layer.Begin();

				// Create the effect for this pass

				JEffect effect = Jad.EffectBuilder.SetMotionBlurPass(layer);

				int npasos = effect.Begin();

				// Set the material values

				layer.ApplyEffectMotionBlurPass(effect);

				for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
				{
					JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
					JMeshObject mesh = mp.meshObject;

					Matrix wbpCurrent = mesh.shaders.World * viewProj;

					// Set the current and last matrix
					effect.DXResource.SetValue("WorldViewProjectionCurrent", wbpCurrent);
					effect.DXResource.SetValue("WorldViewProjectionLast", mesh.shaders.LastWorldViewProjection);

					mesh.RenderBase(mp.numMat, npasos, effect);

					// Save this value to make the pixel motion blur
					mesh.shaders.LastWorldViewProjection = wbpCurrent;
				}

				effect.End();
				layer.End();
			}

			Jad.Video.Views.currentView.RenderTargets.MotionBlur.End();
		}

		#endregion

	}
}