#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{

		#region ParallaxPass

		/// <summary>
		/// Renders the ParallaxPass, to set the right values to the Parallax Render Target
		/// </summary>
		void RenderParallaxPass()
		{
            bool OcclusionMapping = false;

			if (!needsParallaxPass) return;
            
			status = JSceneStatus.ParallaxPass;
            // This solves the problem, but it's not the solution
            //Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, false);
            view.RenderTargets.Parallax.Begin();
            
			for (int i = 0; i < materials.Count; i++)
			{
				JMaterial m = materials[i];

				// If there arent objects to render, continue
				if (m.meshObjectPerMaterial.Count == 0) continue;

				if (m.Priority != JMaterialPriority.Transparency)
				{
					m.Begin();

					// Only affects the first layer
					for (int l = 0; l < 1; l++)
					{
						JMaterialLayer layer = m.Layers[l];

						layer.Begin();

						// Create the effect for this pass

						JEffect effect = Jad.EffectBuilder.SetZPass(layer, false);

						int passes = effect.Begin();

						// Set the material values

						layer.ApplyEffectZPass(effect);

						JBaseLight light;

                        if (layer.HeightMap.OcclusionMapping)
                        {
                            OcclusionMapping = true;

                            if (JEffectBuilder.PixelShaderVersionInteger != 20)
                            {
                                light = lights[0];
                                light.ApplyEffect(effect);
                            }
                        }

						for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
						{
							JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
							JMeshObject mesh = mp.meshObject;

							mesh.RenderBase(mp.numMat, passes, effect);

						}

						effect.End();

						layer.End();
					}

					m.End();
				}
			}

            // OcclusionMapping for shader 2.0
            if (OcclusionMapping && JEffectBuilder.PixelShaderVersionInteger == 20)
			{
				// For shaders 2.0, we need to make 2 passes 
				// The shadow value is saved on the Z value of the RT

				status = JSceneStatus.ParallaxShadowPass;

				for (int i = 0; i < materials.Count; i++)
				{
					JMaterial m = materials[i];

					// If there arent objects to render, continue
					if (m.meshObjectPerMaterial.Count == 0) continue;

					if (m.Priority != JMaterialPriority.Transparency)
					{
						m.Begin();

						// Only affects the first layer
						for (int l = 0; l < 1; l++)
						{
							JMaterialLayer layer = m.Layers[l];

							layer.Begin();

							// Create the effect for this pass

							JEffect effect = Jad.EffectBuilder.SetZPass(layer, false);

							int npasos = effect.Begin();

							// Set the material values

							layer.ApplyEffectZPass(effect);

							if (lights[0] != null) lights[0].ApplyEffect(effect);

							// Set the parallax texture

							if (effect.handlers.textureParallaxValues != null)
								effect.DXResource.SetValue(effect.handlers.textureParallaxValues, Jad.Video.Views.currentView.RenderTargets.Parallax.Texture.DXResource);

							for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
							{
								JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
								JMeshObject mesh = mp.meshObject;

								mesh.RenderBase(mp.numMat, npasos, effect);
							}

							effect.End();

							//Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, (int)ColorWriteEnable.RedGreenBlueAlpha);

							layer.End();
						}

						m.End();
					}
				}
			}
            
			view.RenderTargets.Parallax.End();

            //Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
        }

		#endregion

	}
}