#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{

		#region ZPass

		/// <summary>
		/// Renders the ZPass to mark the Z values to the right values.
		/// </summary>
		void RenderZPass()
		{
			needsParallaxPass = false;

			status = JSceneStatus.ZPass;

			// Disable the color writing
			Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, 0);

			for (int i = 0; i < materials.Count; i++)
			{
				JMaterial m = materials[i];

				// If there are no objects to render, continue
				if (m.meshObjectPerMaterial.Count == 0) continue;

				if (m.Priority == JMaterialPriority.Transparency)
				{
					// Only the first layer is used for the Z
					for (int l = 0; l < 1; l++)
					{
						for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
						{
							JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
							JMeshObject mesh = mp.meshObject;

							// Update the shaders
							// Only needed in this pass
							mesh.UpdateShaders();
						}
					}
				}

				else
				{
					m.Begin();

					// Only the first layer is used for the Z
					for (int l = 0; l < 1; l++)
					{
						JMaterialLayer layer = m.Layers[l];

						layer.Begin();

						if (layer.HeightMap.Enabled)
						{
							if (Jad.Video.Adapter.Information.supportsParallax)
								needsParallaxPass = true;

							else // If parallax is not supported, disable it
								layer.HeightMap.Enabled = false;
						}

						// Create the effect for this pass

						JEffect effect = Jad.EffectBuilder.SetZPass(layer, false);

						int passes = effect.Begin();

						// Set the material values

						layer.ApplyEffectZPass(effect);

						for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
						{
							JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
							JMeshObject mesh = mp.meshObject;

							// Update the shaders
							// Only needed in this pass
							mesh.UpdateShaders();

							mesh.RenderBase(mp.numMat, passes, effect);
						}

						effect.End();
						layer.End();
					}

					m.End();
				}
			}

			// No more need to write in the Z
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);

			// Enable the color write
			Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, (int)ColorWriteEnable.RedGreenBlueAlpha);
		}

		#endregion

	}
}