#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

#endregion

namespace JadEngine.Scripting
{
	/// <summary>
	/// A class containing error information for a single compile error.
	/// </summary>
	public class CompileError
	{
		#region Fields
		
		private int _line;
		private string _errorText;
		private string _errorNumber;

		#endregion

		#region Properties

		/// <summary>
		/// Gets the line number where the error occurred.
		/// </summary>
		public int Line 
		{ 
			get { return _line; } 
		}

		/// <summary>
		/// Gets the error's description.
		/// </summary>
		public string ErrorText 
		{ 
			get { return _errorText; } 
		}
		
		/// <summary>
		/// Gets the error's number (BC00032, etc).
		/// </summary>
		public string ErrorNumber 
		{ 
			get { return _errorNumber; } 
		}

		#endregion

		#region Constructor
		
		/// <summary>
		/// The CompileError constructor.
		/// </summary>
		/// <param name="number">The error's number.</param>
		/// <param name="line">Where the error took place.</param>
		/// <param name="text">A description of the error.</param>
		public CompileError(string number, int line, string text)
		{
			_errorNumber = number;
			_line = line;
			_errorText = text;
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// Outputs the error as a readable string.
		/// </summary>
		/// <returns>A string representing the error.</returns>
		public override string ToString()
		{
			return string.Format("[Line {0}] {1}: {2}", _line.ToString("0000"), _errorNumber, _errorText);
		}

		#endregion
	}

}
