using System;
using System.Collections.Generic;
using System.Text;

namespace JadEngine.Scripting
{
	/// <summary>
	/// Exception thrown when languages are not supported by the engine
	/// </summary>
	public class LanguageNotSupportedException : ApplicationException 
	{
		/// <summary>
		/// Default contructor
		/// </summary>
		public LanguageNotSupportedException(string message)
			: base(message)
		{
		}
		/// <summary>
		/// Default contructor
		/// </summary>
		public LanguageNotSupportedException(string message, Exception innerException)
			: base(message, innerException)
		{
		}
	}

	/// <summary>
	/// RuntimeExceptions are thrown whenever there is an error generated after the compiler is finished compiling.
	/// </summary>
	public class RuntimeException : ApplicationException
	{
		/// <summary>
		/// Creates a RuntimeException.
		/// </summary>
		/// <param name="message"></param>
		public RuntimeException(string message) 
			: base(message) 
		{ 
		}

		/// <summary>
		/// Creates a RuntimeException.
		/// </summary>
		public RuntimeException(string message, Exception inner) 
			: base(message, inner) 
		{ 
		}
	}

	/// <summary>
	/// CompileExceptions are thrown whenever there is an error generated from the compiler.
	/// </summary>
	public class CompileException : ApplicationException
	{
		private CompileError[] _errors;
		
		/// <summary>
		/// Creates a CompileException.
		/// </summary>
		/// <param name="message">The message of the exception.</param>
		public CompileException(string message) 
			: base(message) 
		{ 
		}

		/// <summary>
		/// Creates a CompileException.
		/// </summary>
		/// <param name="message">The message of the exception.</param>
		/// <param name="inner">The inner exception of which the final error came from.</param>
		public CompileException(string message, Exception inner) 
			: base(message, inner) 
		{ 
		}
		
		/// <summary>
		/// Creates a CompileException.
		/// </summary>
		/// <param name="message">The message of the exception.</param>
		/// <param name="inner">The inner exception of which the final error came from.</param>
		/// <param name="errors">An array of compile errors that occurred.</param>
		public CompileException(string message, Exception inner, CompileError[] errors) 
			: base(message, inner) 
		{ 
			_errors = errors; 
		}

		/// <summary>
		/// Gets the array of compiler errors that occurred.
		/// </summary>
		public CompileError[] CompileErrors
		{
			get
			{
				return _errors;
			}
		}
	}
}
