#region Using Directives

using System;
using System.CodeDom.Compiler;

#endregion

namespace JadEngine.Scripting
{
	/// <summary>
	/// Base class for all scripting languages.  Derivitives of this implement support
	/// for a separate scripting language in the engine.
	/// </summary>
	public abstract class JScriptLanguage
	{
		#region Properties

		/// <summary>
		/// When implemented, returns the CodeDomProvider of the language.
		/// </summary>
		public abstract CodeDomProvider CodeDomProvider
		{
			get;
		}

		/// <summary>
		/// Return any compiler options this language should use
		/// </summary>
		public virtual string CompilerOptions
		{
			get { return ""; }
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// When implemented, returns a new string for the using/import statement of this language.
		/// </summary>
		/// <remarks>This should include a newline if a language requires it.</remarks>
		/// <param name="namespaceName">The namespace to import</param>
		/// <example>
		/// C#:      FormatUsingStatement("System") == "using System;" + NewLine
		/// VB.Net:  FormatUsingStatement("System") == "Import System" + NewLine
		///</example>
		public abstract string FormatUsingStatement(string namespaceName);

		/// <summary>
		/// Take code and wrap it in a class definition
		/// </summary>
		/// <param name="className">The name to use for the class</param>
		/// <param name="code">The code to wrap</param>
		/// <returns>The wrapped code</returns>
		public abstract string WrapInClass(string className, string code);
		#endregion
	}
}
