#region Using Directives

using System;
using System.Reflection;
using System.CodeDom.Compiler;
using Microsoft.CSharp;
using System.Security.Policy;
using System.Security.Permissions;
using System.Security;

#endregion

namespace JadEngine.Scripting.Languages
{
	/// <summary>
	/// CSharp provider for the scripting engine
	/// </summary>
	public class CSharp: JScriptLanguage
	{
		#region Fields
		private static CodeDomProvider _codeDomProvider = new CSharpCodeProvider();
		#endregion

		#region Properties
		/// <summary>
		/// Return the CSharp code provider
		/// </summary>
		public override System.CodeDom.Compiler.CodeDomProvider CodeDomProvider
		{
			get 
			{
				return _codeDomProvider; 
			}
		}

		/// <summary>
		/// Return Compiler options saying to optimize us for a library
		/// </summary>
		public override string CompilerOptions
		{
			get
			{
				return "/target:library"; // /optimize";
			}
		}
		#endregion
		
		#region Methods
		/// <summary>
		/// Format the using statement
		/// </summary>
		public override string FormatUsingStatement(string namespaceName)
		{
			return "using " + namespaceName + ";" + Environment.NewLine;
		}

		/// <summary>
		/// Take code and wrap it in a class definition
		/// </summary>
		/// <param name="className">The name to use for the class</param>
		/// <param name="code">The code to wrap</param>
		/// <returns>The wrapped code</returns>
		public override string WrapInClass(string className, string code)
		{
			return Environment.NewLine + "public class " + className + Environment.NewLine + "{" + Environment.NewLine + code + Environment.NewLine + "}" + Environment.NewLine;
		}

		#endregion
	}
}
