#region Using Directives

using System.Collections;
using System.IO;

using JadEngine.Tools;
using JadEngine.VFS;

#endregion

namespace JadEngine.Tools
{
	/// <summary>
	/// Reads the Configuration.xml file and saves the information on the class
	/// </summary>
	public class JConfiguration : JXmlHandler
	{
		#region Structs

		/// <summary>
		/// Encapsulates the video properties
		/// </summary>
		public struct JVideoProperties
		{
			#region Structs

			/// <summary>
			/// Encapsulates the resolution properties
			/// </summary>
			public struct JResolutionProperties
			{
				#region Fields

				/// <summary>
				/// TODO
				/// </summary>
				public int Width;

				/// <summary>
				/// TODO
				/// </summary>
				public int Height;

				#endregion
			}

			#endregion

			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public bool Windowed;

			/// <summary>
			/// TODO
			/// </summary>
			public bool HDR;

			/// <summary>
			/// TODO
			/// </summary>
			public JResolutionProperties Resolution;

			#endregion
		}

		#endregion

		#region Fields

		private JVideoProperties _video;
		private static string _filename = "Configuration.xml";

		#endregion

		#region Properties

		/// <summary>
		/// Values related to video
		/// </summary>
		public JVideoProperties Video
		{
			get { return _video; }
			set { _video = value; }
		}

		/// <summary>
		/// Gets or sets the configuration filename (normally configuration.xml) we will
		/// use during setup.
		/// </summary>
		public static string ConfigurationFilename
		{
			get { return _filename; }
			set { _filename = value; }
		}

		#endregion

		#region Constructor

		/// <summary>
		/// Reads the Configuration.xml file
		/// </summary>
		public JConfiguration()
			: base(_filename)
		{ }

		/// <summary>
		/// Reads the Configuration.xml file
		/// </summary>
		/// <param name="qualifiedFilename">File to load</param>
		public JConfiguration(string qualifiedFilename)
			: base(qualifiedFilename)
		{
			_filename = qualifiedFilename;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Reload our XML file from disk
		/// </summary>
		public void Reload()
		{
			base.Reload(_filename);
		}

		internal void LoadVideoSettings()
		{
			_video.Resolution.Width = ReadValue<int>("Video", "Width", 800);
			_video.Resolution.Height = ReadValue<int>("Video", "Height", 600);
			_video.Windowed = !ReadValue<bool>("Video", "FullScreen", false);
			_video.HDR = ReadValue<bool>("Video", "HDR", false);
		}

		internal void LoadVFSSettings()
		{
			string relativePath;
			JFilesSource source = null;

			// HACK: VFS[1] is a cheat on the JXmlHandler class to avoid the dictionary search and 
			// get the LoadFromFile value from the VFS tag of the first configuration file.
			// If the VFS it's located in another file, then modify the Main VFS RootDirectory
			// (or it won't find anything later)
			relativePath = ReadValue<string>("VFS[1]", "LoadFromFile", "");
			if (relativePath != string.Empty)
				Jad.VFS.RootDirectory = Path.Combine(Jad.VFS.RootDirectory, Path.GetDirectoryName(relativePath));
			
			string[] keys = this.GetPaths("VFS/FilesSource", "Name");
			foreach (string key in keys)
			{
				string type = this.ReadValue<string>(key, "Type", "");
				string name = this.ReadValue<string>(key, "Name", "");
				string path = this.ReadValue<string>(key, "Path", "");

				source = Jad.VFS.CreateSource(type, name, path);

				if (source != null)
				{
					string[] paths = this.GetPaths(key + "/DefinedPath", "Name");

					foreach (string definedPath in paths)
					{
						name = this.ReadValue<string>(definedPath, "Name", "");
						path = this.ReadValue<string>(definedPath, "Path", "");

						source.AddDefinedPath(name, path);
					}

					Jad.VFS.Sources.Add(source);
				}
			}
		}

		#endregion
	}
}
