#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

#endregion

namespace JadEngine.VFS.Storage.Filters
{
	/// <summary>
	/// User control that configures encryption filters
	/// </summary>
	public partial class JEncryptionConfigurationUI : UserControl, IJFilterConfigurationUI
	{
		#region Fields

		/// <summary>
		/// Filter to configure
		/// </summary>
		JEncryptionFilter _filter;

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the filter configured by this interface
		/// </summary>
		[System.ComponentModel.Browsable(false)]
		public IJFilter Filter
		{
			get { return _filter; }
			set { _filter = value as JEncryptionFilter; }
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JEncryptionConfigurationUI()
		{
			InitializeComponent();
		}

		#endregion

		#region Methods

		/// <summary>
		/// Configures the filter
		/// </summary>
		public void Configure()
		{
			_filter.SetKey(txtKey.Text);
			_filter.SetIV(txtIV.Text);
		}

		#endregion

		#region Event Handlers

		/// <summary>
		/// Validation event (text can't be empty)
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void txtKey_Validating(object sender, CancelEventArgs e)
		{
			if (txtKey.Text == String.Empty)
			{
				e.Cancel = true;
				errorProvider.SetError(txtKey, "The key can't be an empty.");
			}
		}

		/// <summary>
		/// Validated event
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void txtKey_Validated(object sender, EventArgs e)
		{
			errorProvider.SetError(txtKey, "");
		}

		/// <summary>
		/// Enter event
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void txtKey_Enter(object sender, EventArgs e)
		{
			txtKey.SelectAll();
		}

		/// <summary>
		/// Validation event (text can't be empty)
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void txtIV_Validating(object sender, CancelEventArgs e)
		{
			if (txtIV.Text == String.Empty)
			{
				e.Cancel = true;
				errorProvider.SetError(txtIV, "The initializing vector can't be an empty.");
			}
		}

		/// <summary>
		/// Validated event
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void txtIV_Validated(object sender, EventArgs e)
		{
			errorProvider.SetError(txtIV, "");
		}

		/// <summary>
		/// Enter event
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void txtIV_Enter(object sender, EventArgs e)
		{
			txtIV.SelectAll();
		}

		#endregion
	}
}
