#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics.Geometry;
using JadEngine.Physics;
using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;
using JadEngine.Collections;

#endregion

namespace JadEngine.Video
{
	#region JDepthStencil

	/// <summary>
	/// A depth stencil surface
    /// </summary>
    public class JDepthStencil : IJListable,IDisposable
    {
        #region Fields

		Surface surface;

        internal SurfaceDescription description;

        #endregion

        #region IJListable

        /// <summary>unique identifier</summary>
        int id;

        /// <summary>name</summary>
        protected string name;

        /// <summary>
        /// Gets the name
        /// </summary>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets the internal ID
        /// </summary>
        public int ID
        {
            get { return id; }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get/Set the DX surface
        /// </summary>
		public Surface Surface
		{
		  get { return surface; }
		  set 
          { 
              surface = value;
              if(surface!=null) description = surface.Description;
          }
		}

        #endregion

        #region Dispose

        /// <summary>
        /// Disposes the resource AND set surface=null
        /// </summary>
        public void Dispose()
        {
			if (surface != null)
			{
				surface.Dispose();

				surface = null;
			}

        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor 
        /// </summary>
		/// <param name="name">DepthStencil name</param>
        public JDepthStencil(string name)
        {
            this.name = name;

        }

        #endregion

        #region Methods

        /// <summary>
        /// Set the device depth stencil surface
        /// </summary>
        public void Set()
        {
            Jad.Video.DepthStencils.Set(this);
        }

        #endregion

        #region Reset / Lost

        /// <summary>
        /// Called OnReset device.
        /// </summary>
        public void Reset()
        {
			Jad.LogWriter.WriteLine("JDepthStencil.Reset:" + name + " Width:" + description.Width + " ; Height:" + description.Height);

            DepthFormat format = (DepthFormat)description.Format;

            // We must recreate the depth stencil

            surface = Jad.Video.Device.CreateDepthStencilSurface(description.Width, description.Height, format, description.MultiSampleType, description.MultiSampleQuality, false);
        }

        /// <summary>
        /// Called OnLostDevice
        /// </summary>
        internal void OnLost()
        {
			Jad.LogWriter.WriteLine("JDepthStencil.OnLost:" + name);

            Dispose();

        }

        #endregion
    }

    #endregion

}
