#region Using directives

using System;
using System.Drawing;
using System.Windows.Forms;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// Class that creates the default form associated with the engine
	/// </summary>
	public class JBaseForm : Form
	{
		#region Constructors
		
		/// <summary>
		/// Default constructor
		/// </summary>
		public JBaseForm()
		{
			// Set the initial size of our form
			ClientSize = new Size(Jad.Video.Adapter.Initialization.PresentParams.BackBufferWidth,
													  Jad.Video.Adapter.Initialization.PresentParams.BackBufferHeight);
			// And its caption
			Text = "Jad 3D Engine";

			StartPosition = FormStartPosition.CenterScreen;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Override OnResize to handle form resizing
		/// </summary>
		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			// No resize in full screen
			if (!Jad.Configuration.Video.Windowed)
				return;

			if (!Jad.Active) return;

			// Modify the parameters to indicate the size change
			JView view = Jad.Video.Views[0];

			view.SetResetValues(Size.Width, Size.Height);

			// Reset the device. This will call the Lost and Reset events of the device
			Jad.Video.Adapter.Reset();
		}

		#endregion
	}
}
