#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics.Geometry;
using JadEngine.Physics;
using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;
using JadEngine.Collections;

#endregion

namespace JadEngine.Video
{

	#region Enum

	/// <summary>
	/// The plane you want the rectangle be created
	/// </summary>
	public enum JCreateRectangle
	{
		/// <summary>
		/// XZ plane
		/// </summary>
		XZ,

		/// <summary>
		/// XY plane
		/// </summary>
		XY,

		/// <summary>
		/// YZ plane
		/// </summary>
		YZ
	};

	#endregion

	/// <summary>
	/// A mesh is a group of vertex maintained by DX that 
	/// are rendered directly to the graphic card
	/// </summary>
	public class JMesh : IJListable, IDisposable, ICloneable
	{
		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		public struct JForEachFaceParams
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public Object User;

			/// <summary>
			/// TODO
			/// </summary>
			public JMesh Mesh;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumFace;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 v0;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 v1;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 v2;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 Normal;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3[] Vertices;

			/// <summary>
			/// TODO
			/// </summary>
			public int Vertex0Index;

			/// <summary>
			/// TODO
			/// </summary>
			public int Vertex1Index;

			/// <summary>
			/// TODO
			/// </summary>
			public int Vertex2Index;

			/// <summary>
			/// TODO
			/// </summary>
			public Matrix World;

			/// <summary>
			/// TODO
			/// </summary>
			public JMeshObject MeshObject;

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
		struct JVertexFormat
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public float X;

			/// <summary>
			/// TODO
			/// </summary>
			public float Y;

			/// <summary>
			/// TODO
			/// </summary>
			public float Z;

			/// <summary>
			/// TODO
			/// </summary>
			public float Tu;

			/// <summary>
			/// TODO
			/// </summary>
			public float Tv;

			#endregion
		}

		#endregion

		#region Internal Classes

		/// <summary>
		/// TODO
		/// </summary>
		class JIntersectParams
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 Start;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 Dir;

			/// <summary>
			/// TODO
			/// </summary>
			public float Distance;

			/// <summary>
			/// TODO
			/// </summary>
			public int NumFaceCollision;

			/// <summary>
			/// TODO
			/// </summary>
			public bool BackFaceCulling;

			#endregion

			#region Constructors

			/// <summary>
			/// Default constructor
			/// </summary>
			/// <param name="start">TODO</param>
			/// <param name="dir">TODO</param>
			/// <param name="backFaceCulling">TODO</param>
			public JIntersectParams(Vector3 start, Vector3 dir, bool backFaceCulling)
			{
				Start = start;
				Dir = dir;
				BackFaceCulling = backFaceCulling;
				Distance = float.MaxValue;
				NumFaceCollision = -1;
			}

			#endregion
		}

		#endregion

		#region Delegates Definitions and Methods

		/// <summary>
		/// Delegate used to see if more faces should be tested
		/// </summary>
		/// <param name="face">TODO</param>
		public delegate bool ForEachFaceCB(JForEachFaceParams face);

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="face">TODO</param>
		/// <returns>TODO</returns>
		public static bool IntersectCB(JForEachFaceParams face)
		{
			JIntersectParams v = (JIntersectParams) face.User;

			IntersectInformation i;

			// Test BackFaceCulling
			if (v.BackFaceCulling)
			{
				if (Vector3.Dot(v.Dir, face.Normal) < 0) return true;
			}

			if (Geometry.IntersectTri(face.v0, face.v1, face.v2, v.Start, v.Dir, out i))
			{
				if (i.Dist < v.Distance)
				{
					v.Distance = i.Dist;
					v.NumFaceCollision = face.NumFace;
				}
			}

			return true;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="face">TODO</param>
		/// <returns>TODO</returns>
		public static bool FaceCB(JForEachFaceParams face)
		{
			float[] v = (float[]) face.User;

			v[0] = face.v0.X;
			v[1] = face.v0.Y;
			v[2] = face.v0.Z;
			v[3] = face.v1.X;
			v[4] = face.v1.Y;
			v[5] = face.v1.Z;
			v[6] = face.v2.X;
			v[7] = face.v2.Y;
			v[8] = face.v2.Z;

			return true;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="face">TODO</param>
		/// <returns>TODO</returns>
		public static bool GenerateTreeCollisionCB(JForEachFaceParams face)
		{
			//Add this face to the collision tree

			float[] v = new float[9];

			v[0] = face.v0.X;
			v[1] = face.v0.Y;
			v[2] = face.v0.Z;
			v[3] = face.v1.X;
			v[4] = face.v1.Y;
			v[5] = face.v1.Z;
			v[6] = face.v2.X;
			v[7] = face.v2.Y;
			v[8] = face.v2.Z;

			JCollision collision = (JCollision) face.User;

			// Get the subset n of the face

			int numAttID = 0;

			if (face.Mesh.AttributeRange != null)
			{
				for (int i = 0; i < face.Mesh.AttributeRange.Length; i++)
				{
					if (face.NumFace >= face.Mesh.AttributeRange[i].FaceStart)
					{
						numAttID = i;
						break;
					}
				}
			}

			int materialID = face.MeshObject.Material[numAttID].PhysicsPhysicsMaterial.Material;

			collision.TreeCollisionAddFace(3, v, 12, materialID);

			return true;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="face">TODO</param>
		/// <returns>TODO</returns>
		public static bool GenerateFaceNormalsCB(JForEachFaceParams face)
		{
			Vector3 cross1 = (face.v2 - face.v1);
			Vector3 cross2 = (face.v1 - face.v0);

			face.Mesh.FacesNormal[face.NumFace] = Vector3.Cross(cross1, cross2);

			return true;
		}

		#endregion

		#region Static Fields

		/// <summary>
		/// Vertices for the method CreateCube
		/// </summary>
		static float[] cubeVertices = new float[] {
			-0.5f, 0, -0.5f,1,1,	// 0. Bottom
			0.5f,0,0.5f,0,0,		// 1
			-0.5f,0,0.5f,1,0,		// 2
			0.5f,0,-0.5f,0,1,		// 3
			-0.5f,1,-0.5f,0,1,		// 4. Top
			0.5f,1,0.5f,1,0,		// 5
			0.5f,1,-0.5f,1,1,		// 6
			-0.5f,1,0.5f,0,0,		// 7
			-0.5f,0,-0.5f,0,1,		// 8. Front
			0.5f,1,-0.5f,1,0,		// 9
			0.5f,0,-0.5f,1,1,		// 10
			-0.5f,1,-0.5f,0,0,		// 11
			0.5f,0,-0.5f,0,1,		// 12. Right
			0.5f,1,0.5f,1,0,		// 13
			0.5f,0,0.5f,1,1,		// 14
			0.5f,1,-0.5f,0,0,		// 15
			0.5f,0,0.5f,0,1,		// 16. Back
			-0.5f,1,0.5f,1,0,		// 17
			-0.5f,0,0.5f,1,1,		// 18
			0.5f,1,0.5f,0,0,		// 19
			-0.5f,0,0.5f,0,1,		// 20. Left
			-0.5f,1,-0.5f,1,0,		// 21
			-0.5f,0,-0.5f,1,1,		// 22
			-0.5f,1,0.5f,0,0 };		// 23

		/// <summary>
		/// Indices definition for the method CreateCube24Vertices
		/// </summary>
		static int[] cubeFaces = new int[] {
			0,1,2,
			1,0,3,
			4,5,6,
			5,4,7,
			8,9,10,
			9,8,11,
			12,13,14,
			13,12,15,
			16,17,18,
			17,16,19,
			20,21,22,
			21,20,23 };

		/// <summary>
		/// TODO
		/// </summary>
		static ForEachFaceCB GenerateTreeCollisionDelegate = new ForEachFaceCB(GenerateTreeCollisionCB);

		/// <summary>
		/// TODO
		/// </summary>
		static ForEachFaceCB GenerateFaceNormalsDelegate = new ForEachFaceCB(GenerateFaceNormalsCB);

		/// <summary>
		/// TODO
		/// </summary>
		static ForEachFaceCB FaceDelegate = new ForEachFaceCB(FaceCB);

		/// <summary>
		/// TODO
		/// </summary>
		static ForEachFaceCB IntersectDelegate = new ForEachFaceCB(IntersectCB);

		#endregion

		#region IJListable

		/// <summary>Mesh unique identifier</summary>
		int id;

		/// <summary>Mesh name</summary>
		protected string name;

		/// <summary>
		/// Gets the mesh name
		/// </summary>
		public string Name
		{
			get { return name; }
			set { name = value; }
		}

		/// <summary>
		/// Gets the mesh internal ID
		/// </summary>
		public int ID
		{
			get { return id; }
		}

		#endregion

		#region Fields

		/// <summary>
		/// Displacement computed to move the object to 0, 0, 0
		/// </summary>
		protected Vector3 translationToLocal;

		/// <summary>
		/// The ShadowVolume used for the stencil shadows
		/// </summary>
		JShadowVolume shadowVolume;

		/// <summary>
		/// Used for the ComputeTangentSpace. If this field is false, the method will compute the normals too
		/// </summary>
		public bool NormalsComputed;

		/// <summary>
		/// True if the object is centered at 0, 0, 0
		/// </summary>
		public bool CenteredAtOrigin;

		/// <summary>
		/// Number of attributes (or subsets) that the mesh contains
		/// </summary>
		public int NumAttributes = 1;

		/// <summary>
		/// Tells the engine if it needs to compute tangents
		/// </summary>
		public bool TangentComputed;

		/// <summary>
		/// The attribute range, used for submaterials into the mesh. If NumAttributes=1 this value can be null
		/// </summary>
		public AttributeRange[] AttributeRange;

		/// <summary>
		/// The VertexBuffer
		/// </summary>
		public JVertexBuffer VertexBuffer;

		/// <summary>
		/// The IndexBuffer
		/// </summary>
		public JIndexBuffer IndexBuffer;

		/// <summary>
		/// Smooth groups for faces. null if they are not defined
		/// </summary>
		public int[] SmoothGroups;

		/// <summary>
		/// True indicates that this mesh has not to be destroyed when you destroy an scene
		/// </summary>
		internal bool InHouse;

		/// <summary>
		/// Array of face normals. Use <see cref="GenerateFaceNormals"/>
		/// </summary>
		public Vector3[] FacesNormal;

		/// <summary>
		/// Track whether Dispose has been called.
		/// </summary>
		private bool disposed = false;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <remarks>Creates a shadow volume for this mesh</remarks>
		public JMesh(string name)
		{
			// Set the ID
			this.id = Jad.GetUniqueID();

			if (name == null) this.name = id.ToString();
			else this.name = name;

			shadowVolume = new JShadowVolume(this);

		}

		/// <summary>
		/// Default constructor. The mesh name will be the internal id value assigned
		/// </summary>
		public JMesh()
			: this(null)
		{
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets the local bounding box
		/// </summary>
		public JBoundingBox3 BoundingBox
		{
			get { return VertexBuffer.BoundingBox; }
		}

		/// <summary>
		/// Gets the local bounding sphere
		/// </summary>
		public JBoundingSphere BoundingSphere
		{
			get { return VertexBuffer.BoundingSphere; }
		}

		/// <summary>
		/// Gets the ShadowVolume used for the stencil shadows
		/// </summary>
		public JShadowVolume ShadowVolume
		{
			get { return shadowVolume; }
		}

		/// <summary>
		/// Gets the displacement computed to move the object at 0, 0, 0
		/// </summary>
		public Vector3 TranslationToLocal
		{
			get { return translationToLocal; }
		}

		#endregion

		#region Dispose

		/// <summary>
		/// Implement IDisposable
		/// </summary>
		public void Dispose()
		{
			// Dispose managed resources.
			if (VertexBuffer != null)
			{
				Jad.Video.VertexIndexBuffers.Delete(VertexBuffer);
			}
			if (IndexBuffer != null)
			{
				Jad.Video.VertexIndexBuffers.Delete(IndexBuffer);
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Renders the mesh indicating the attributeID
		/// </summary>
		/// <param name="ID">AttributeID, always must be 0 based and continous (0, 1, 2,...)</param>
		public void Render(int ID)
		{
			if (AttributeRange == null)
			{
				Jad.Video.Device.SetStreamSource(0, VertexBuffer.DXResource, 0, VertexBuffer.sizeInBytes);

				Jad.Video.Device.VertexDeclaration = VertexBuffer.Declaration;

				Jad.Video.Device.Indices = IndexBuffer.DXResource;

				Jad.Video.Device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, VertexBuffer.NumberOfVertices, 0, IndexBuffer.NumberOfIndices / 3);
			}

			else
			{
				Jad.Video.Device.SetStreamSource(0, VertexBuffer.DXResource, 0, VertexBuffer.sizeInBytes);

				Jad.Video.Device.VertexDeclaration = VertexBuffer.Declaration;

				Jad.Video.Device.Indices = IndexBuffer.DXResource;

				int minVertexIndex = 0, vertexCount = AttributeRange[ID].VertexCount;

				// If vertexCount is 0, it means that some system was used to create the
				// attributes that doesnt re-place the vertices. So, they are all sent
				// to the shader, even if the graphic card calculates the shader for each
				// vertex instead of all together

				if (vertexCount == 0)
					vertexCount = VertexBuffer.NumberOfVertices;

				else
					for (int i = 0; i < ID; i++)
						minVertexIndex += AttributeRange[i].VertexCount;

				Jad.Video.Device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, minVertexIndex, vertexCount, AttributeRange[ID].FaceStart * 3, AttributeRange[ID].FaceCount);

			}
		}

		/// <summary>
		/// Translate the mesh so the center of the mesh will be (0, 0, 0)
		/// </summary>
		public void CenterAtOrigin()
		{
			// If the center is at (0, 0, 0) no need to move it
			if (CenteredAtOrigin)
				return;

			if (BoundingSphere.Center != Vector3.Empty)
			{
				Vector3 center = BoundingSphere.Center;

				ProcessMatrix(Matrix.Translation(-center));

				// Save the displacement to set it later to the mesh. This is used
				// when loading .jad files

				translationToLocal = center;
			}

			CenteredAtOrigin = true;
		}

		/// <summary>
		/// Apply a matrix to the vertex position
		/// </summary>
		/// <param name="matrix">The matrix to multiply</param>
		public void ProcessMatrix(Matrix matrix)
		{
			for (int c = 0; c < VertexBuffer.NumberOfVertices; c++)
				VertexBuffer.Position[c].TransformCoordinate(matrix);

			VertexBuffer.NotifyPositionChange();

			VertexBuffer.BoundingNeedsUpdate = true;
		}

		/// <summary>
		/// Adds some noise to the vertex position
		/// </summary>
		/// <param name="noiseVector">Noise multiplicator</param>
		public void Noise(Vector3 noiseVector)
		{
			Random r = new Random();

			for (int c = 0; c < VertexBuffer.NumberOfVertices; c++)
			{
				Vector3 noise = new Vector3((float) r.NextDouble(), (float) r.NextDouble(), (float) r.NextDouble());

				noise.X *= noiseVector.X;
				noise.Y *= noiseVector.Y;
				noise.Z *= noiseVector.Z;

				VertexBuffer.Position[c] += noise;
			}

			VertexBuffer.NotifyPositionChange();
		}

		/// <summary>
		/// Compute the face normals
		/// </summary>
		public void GenerateFaceNormals()
		{
			FacesNormal = new Vector3[IndexBuffer.NumberOfIndices / 3];

			ForEachFace(Matrix.Identity, GenerateFaceNormalsDelegate, null, -1, null);
		}

		/// <summary>
		/// Computes the ray intersection and returns some interesting values
		/// </summary>
		/// <param name="start">Ray start point</param>
		/// <param name="dir">Ray direction</param>
		/// <param name="backFaceCulling">True if it should check also the backface</param>
		/// <param name="distance">Out parameter that returns the distance from the start</param>
		/// <returns>True if there is colision, false if not</returns>
		public bool Intersect(Vector3 start, Vector3 dir, bool backFaceCulling, out float distance)
		{
			bool colide;

			JIntersectParams p = new JIntersectParams(start, dir, backFaceCulling);

			ForEachFace(Matrix.Identity, IntersectDelegate, p, -1, null);

			distance = p.Distance;

			colide = (p.NumFaceCollision >= 0);

			return colide;
		}

		/// <summary>
		/// Build the Newton collision tree for this mesh
		/// </summary>
		/// <param name="collision">collision to store the tree</param>
		/// <param name="malla">MeshObject that contains the World Matrix for this mesh</param>
		/// <remarks>The BeginBuild method should be called before calling this one</remarks>
		public void GenerateTreeCollision(JCollision collision, JMeshObject malla)
		{
			// Debemos transformar el mesh en coords locales pq luego lo har
			// el motor en Jad.Begin y entonces no coincide los vrtices
			// que le paso a Newton con lo que se dibuja

			// Tranform the mesh to local coordinate because later on the engine will
			// do it in Jad.Begin and then the vertex that are drawed and the vertex
			// passed to Newton dont match

			CenterAtOrigin();

			ForEachFace(malla.Transform.WorldTM, GenerateTreeCollisionDelegate, collision, -1, malla);
		}

		/// <summary>
		/// For each face, calls a callback
		/// </summary>
		/// <param name="matrix">Matrix to multiply each vertex</param>
		/// <param name="cb">CallBack that will be called for every face</param>
		/// <param name="user">Parameter to be passed to the callback</param>
		/// <param name="numFace">-1 to process all the faces. If you give another number, only this face will be processed</param>
		/// <param name="malla">JMeshObject to be passed to the callback</param>
		public void ForEachFace(Matrix matrix, ForEachFaceCB cb, Object user, int numFace, JMeshObject malla)
		{
			JForEachFaceParams faceParam = new JForEachFaceParams();
			faceParam.Mesh = this;
			faceParam.User = user;
			faceParam.World = matrix;
			faceParam.MeshObject = malla;

			int numFaces = IndexBuffer.NumberOfIndices / 3;

			Vector3 v0, v1, v2;
			int i = 0, i2 = numFaces;

			// If a face number is set, modify the loop so only that face is examined
			if (numFace != -1)
			{
				i = numFace;
				i2 = i + 1;
			}

			for (; i < i2; i++)
			{
				int Index0 = IndexBuffer.Indices[3 * i + 0];
				int Index1 = IndexBuffer.Indices[3 * i + 1];
				int Index2 = IndexBuffer.Indices[3 * i + 2];

				v0 = VertexBuffer.Position[Index0];
				v1 = VertexBuffer.Position[Index1];
				v2 = VertexBuffer.Position[Index2];

				v0.TransformCoordinate(matrix);
				v1.TransformCoordinate(matrix);
				v2.TransformCoordinate(matrix);

				faceParam.NumFace = i;
				faceParam.v0 = v0;
				faceParam.v1 = v1;
				faceParam.v2 = v2;
				faceParam.Normal = Vector3.Cross(v0, v1);
				faceParam.Vertices = VertexBuffer.Position;
				faceParam.Vertex0Index = Index0;
				faceParam.Vertex1Index = Index1;
				faceParam.Vertex2Index = Index2;

				// If false exit the loop
				if (!cb(faceParam)) break;
			}
		}

		/// <summary>
		/// Computes the tangent space vectors and normal vectors (if needed)
		/// </summary>
		public void ComputeTangent()
		{
			if (TangentComputed) return;

			// If there arent texture coordinates its impossible to calculate the tangent
			if (VertexBuffer.NumTextCoord == 0) return;

			int numVertices = VertexBuffer.NumberOfVertices;

			if (VertexBuffer.Normal == null) VertexBuffer.Normal = new Vector3[numVertices];
			if (VertexBuffer.Tangent == null) VertexBuffer.Tangent = new Vector3[numVertices];

			Vector3 normal = Vector3.Empty;

			int numFaces = IndexBuffer.NumberOfIndices / 3;

			bool[] used = new bool[numVertices];

			// Calculate the tangent space for each face
			for (int i = 0; i < numFaces; i++)
			{
				Vector3 v0, v1, v2;
				Vector2 uv0, uv1, uv2;
				float area, f, sign;

				// Indice of the three vertex that make the face

				int i0, i1, i2;

				i0 = IndexBuffer.Indices[i * 3 + 0];
				i1 = IndexBuffer.Indices[i * 3 + 1];
				i2 = IndexBuffer.Indices[i * 3 + 2];

				// Vertices of the face

				v0 = VertexBuffer.Position[i0];
				v1 = VertexBuffer.Position[i1];
				v2 = VertexBuffer.Position[i2];

				// Coordenadas de textura de la cara

				uv0 = VertexBuffer.TexCoord[0, i0];
				uv1 = VertexBuffer.TexCoord[0, i1];
				uv2 = VertexBuffer.TexCoord[0, i2];

				// XYZ & UV derivatives

				float[] d0 = new float[5];
				float[] d1 = new float[5];

				d0[0] = v0.X - v1.X;
				d0[1] = v0.Y - v1.Y;
				d0[2] = v0.Z - v1.Z;
				d0[3] = uv0.X - uv1.X;
				d0[4] = uv0.Y - uv1.Y;

				d1[0] = v0.X - v2.X;
				d1[1] = v0.Y - v2.Y;
				d1[2] = v0.Z - v2.Z;
				d1[3] = uv0.X - uv2.X;
				d1[4] = uv0.Y - uv2.Y;

				// Normal

				if (!NormalsComputed)
				{

					normal.X = d0[1] * d1[2] - d0[2] * d1[1];
					normal.Y = d0[2] * d1[0] - d0[0] * d1[2];
					normal.Z = d0[0] * d1[1] - d0[1] * d1[0];

					f = (float) (1.0 / Math.Sqrt(normal.X * normal.X + normal.Y * normal.Y + normal.Z * normal.Z));

					normal.X *= f;
					normal.Y *= f;
					normal.Z *= f;
				}

				// Area sign

				area = d0[3] * d1[4] - d0[4] * d1[3];
				sign = Math.Sign(area);

				// First tangent

				Vector3 tangent;

				tangent.X = d0[0] * d1[4] - d0[4] * d1[0];
				tangent.Y = d0[1] * d1[4] - d0[4] * d1[1];
				tangent.Z = d0[2] * d1[4] - d0[4] * d1[2];

				f = (float) (1.0 / Math.Sqrt(tangent.X * tangent.X + tangent.Y * tangent.Y + tangent.Z * tangent.Z));
				f *= sign;

				tangent.X *= f;
				tangent.Y *= f;
				tangent.Z *= f;

				if (used[i0])
				{
					if (!NormalsComputed) VertexBuffer.Normal[i0] += normal;

					VertexBuffer.Tangent[i0] += tangent;
				}

				else
				{
					if (!NormalsComputed) VertexBuffer.Normal[i0] = normal;

					VertexBuffer.Tangent[i0] = tangent;

					used[i0] = true;
				}

				if (used[i1])
				{
					if (!NormalsComputed) VertexBuffer.Normal[i1] += normal;

					VertexBuffer.Tangent[i1] += tangent;
				}

				else
				{
					if (!NormalsComputed) VertexBuffer.Normal[i1] = normal;

					VertexBuffer.Tangent[i1] = tangent;

					used[i1] = true;
				}

				if (used[i2])
				{
					if (!NormalsComputed) VertexBuffer.Normal[i2] += normal;

					VertexBuffer.Tangent[i2] += tangent;
				}

				else
				{
					if (!NormalsComputed) VertexBuffer.Normal[i2] = normal;

					VertexBuffer.Tangent[i2] = tangent;

					used[i2] = true;
				}
			}

			// Renormalize
			for (int i = 0; i < numVertices; i++)
			{
				// Normal

				if (!NormalsComputed) VertexBuffer.Normal[i].Normalize();

				// Tangent

				VertexBuffer.Tangent[i].Normalize();
			}

			VertexBuffer.NotifyTangentChange();

			if (!NormalsComputed) VertexBuffer.NotifyNormalChange();

			NormalsComputed = true;

			TangentComputed = true;
		}

		#endregion

		#region ICloneable

		/// <summary>
		/// Creates an exact copy of the mesh
		/// </summary>
		/// <returns>Returns the copy of the mesh object</returns>
		public object Clone()
		{
			JMesh newMesh = Jad.Video.Meshes.Create();

			newMesh.VertexBuffer = VertexBuffer;
			newMesh.IndexBuffer = IndexBuffer;

			newMesh.CenteredAtOrigin = CenteredAtOrigin;
			newMesh.NumAttributes = NumAttributes;

			newMesh.translationToLocal = translationToLocal;

			newMesh.TangentComputed = TangentComputed;
			newMesh.NormalsComputed = NormalsComputed;

			newMesh.AttributeRange = AttributeRange;

			return newMesh;
		}

		#endregion

		#region Serializable

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads a binary format mesh
		/// </summary>
		/// <param name="fullPath">The full path</param>
		/// <returns>True if evertything is ok</returns>
		public bool Load(string fullPath)
		{
			JBinaryImport import = new JBinaryImport(fullPath);

			int fileVersionLoader = import.LoadInt32();

			VertexBuffer = new JVertexBuffer(import);

			IndexBuffer = new JIndexBuffer(import);

			import.Dispose();

			return true;
		}

		/// <summary>
		/// Save the VB and IB information.
		/// </summary>
        /// <param name="filename">Base filename where we save</param>
		/// <returns>True on successful save</returns>
		public bool Save(string filename)
		{
			bool result = true;

			// Each mesh has is own file
			// And we need to set the ID becuase the ID is what
			// we have on the scene file
			JBinaryExport export = new JBinaryExport(filename + "\\" + name);

			export.Save(fileVersion);

			if (!VertexBuffer.Save(export)) result = false;
			else
			{
				result = IndexBuffer.Save(export);
			}

			export.Dispose();

			return result;
		}

		#endregion

		#region Static Member (Mesh<->VB)

		/// <summary>
		/// Builds the AttributeRange from an array of attributes
		/// </summary>
		/// <param name="mesh">A valid Mesh object</param>
		/// <param name="attributes">Array of attributes per face</param>
		/// <param name="numAttributes">Number of attributes</param>
		/// <returns>The AttributeRange or null if there is only 1 attribute</returns>
		public static AttributeRange[] CreateAttributeRange(Mesh mesh, int[] attributes, int numAttributes)
		{
			if (numAttributes == 1) return null;

			int[] attributeI = mesh.LockAttributeBufferArray(LockFlags.None);

			Array.Copy(attributes, attributeI, attributes.Length);

			mesh.UnlockAttributeBuffer(attributeI);

			return Optimize(mesh);
		}

		/// <summary>
		/// Optimize a mesh using the OptimizeAttributeSort flag
		/// </summary>
		/// <param name="mesh">TODO</param>
		public static AttributeRange[] Optimize(Mesh mesh)
		{
			int[] adjacency = new int[mesh.NumberFaces * 3];

			mesh.GenerateAdjacency(0.0f, adjacency);

			mesh.OptimizeInPlace(MeshFlags.OptimizeAttributeSort /* | MeshFlags.OptimizeCompact | MeshFlags.OptimizeVertexCache*/, adjacency);

			return mesh.GetAttributeTable();
		}

		///<summary>
		///Creates a DX Mesh object from a Vertex and Index Buffer
		///</summary>
		///<param name="vb">A valid VertexBuffer</param>
		///<param name="ib">A valid IndexBuffer</param>
		///<param name="attributeRange">The attribute range or null if does not exist</param>
		///<returns>A valid Mesh object</returns>
		public static Mesh MeshFromBuffers(JVertexBuffer vb, JIndexBuffer ib, AttributeRange[] attributeRange)
		{
			VertexFormats format = VertexFormats.Position;

			if (vb.Normal != null) format |= VertexFormats.Normal;

			format |= (VertexFormats) (vb.NumTextCoord << (int) VertexFormats.TextureCountShift);

			Mesh mesh = new Mesh(ib.NumberOfIndices / 3, vb.NumberOfVertices, MeshFlags.Managed | MeshFlags.Use32Bit, format, Jad.Video.Device);

			int numOfFloats = 3 + 3 + 2;

			float[] vertices = new float[vb.NumberOfVertices * numOfFloats];

			for (int i = 0, offset = 0; i < vb.NumberOfVertices; i++, offset += numOfFloats)
			{
				int n = offset;

				vertices[n++] = vb.Position[i].X;
				vertices[n++] = vb.Position[i].Y;
				vertices[n++] = vb.Position[i].Z;

				if (vb.Normal != null)
				{
					vertices[n++] = vb.Normal[i].X;
					vertices[n++] = vb.Normal[i].Y;
					vertices[n++] = vb.Normal[i].Z;
				}

				for (int t = 0; t < vb.NumTextCoord; t++)
				{
					vertices[n++] = vb.TexCoord[t, i].X;
					vertices[n++] = vb.TexCoord[t, i].Y;
				}
			}

			mesh.SetVertexBufferData(vertices, LockFlags.None);
			mesh.SetIndexBufferData(ib.Indices, LockFlags.None);

			if (attributeRange != null) mesh.SetAttributeTable(attributeRange);

			return mesh;
		}

		/// <summary>
		/// Create the buffers from a valid DX Mesh
		/// </summary>
		/// <param name="mesh">A valid DX Mesh</param>
		/// <param name="vbDest">The JVertexBuffer that will be created by the method</param>
		/// <param name="ibDest">The JIndexBuffer that will be created by the method</param>
		public static AttributeRange[] BuffersFromMesh(Mesh mesh, out JVertexBuffer vbDest, out JIndexBuffer ibDest)
		{
			bool hasNormals = false;

			using (VertexBuffer vb = mesh.VertexBuffer)
			{
				int vertexSize = vb.Description.Size / 4;
				int numVertices = mesh.NumberVertices;

				// Vertices array (each float is a vertex element)
				float[] vertices = (float[]) vb.Lock(0, typeof(float), LockFlags.None, vertexSize);

				// Calculate the displacement needed to change from one vertex to another
				int stride = vertexSize / numVertices;

				vbDest = new JVertexBuffer(numVertices);

				vbDest.Position = new Vector3[numVertices];

				if ((mesh.VertexFormat & VertexFormats.Normal) != 0)
				{
					vbDest.Normal = new Vector3[numVertices];

					hasNormals = true;
				}

				uint f = (uint) mesh.VertexFormat & (uint) VertexFormats.TextureCountMask;

				f >>= 8;

				int numCoordText = (int) (f & 255);

				if (numCoordText > 0)
				{
					vbDest.TexCoord = new Vector2[numCoordText, numVertices];
				}

				for (int i = 0, p = 0; i < numVertices; i++, p += stride)
				{
					int pos = p;

					vbDest.Position[i].X = vertices[pos];
					vbDest.Position[i].Y = vertices[pos + 1];
					vbDest.Position[i].Z = vertices[pos + 2];

					pos += 3;

					if (hasNormals)
					{
						vbDest.Normal[i].X = vertices[pos];
						vbDest.Normal[i].Y = vertices[pos + 1];
						vbDest.Normal[i].Z = vertices[pos + 2];

						pos += 3;
					}

					for (int a = 0; a < numCoordText; a++)
					{
						vbDest.TexCoord[a, i].X = vertices[pos];
						vbDest.TexCoord[a, i].Y = vertices[pos + 1];
						pos += 2;
					}
				}

				vb.Unlock();
			}

			using (IndexBuffer ib = mesh.IndexBuffer)
			{
				int indexSize;

				ushort[] indices;
				int[] indices32;

				if (ib.Description.Is16BitIndices)
				{

					indexSize = ib.Description.Size / 2;

					ibDest = new JIndexBuffer(false, indexSize);

					// Indices array
					indices = (ushort[]) ib.Lock(0, typeof(ushort), LockFlags.ReadOnly, indexSize);

					JArray.CopyInt16ToInt32(indices, ibDest.Indices, indexSize);
				}
				else
				{

					indexSize = ib.Description.Size / 4;

					ibDest = new JIndexBuffer(true, indexSize);

					// Indices array
					indices32 = (int[]) ib.Lock(0, typeof(int), LockFlags.ReadOnly, indexSize);

					indices32.CopyTo(ibDest.Indices, 0);
				}

				ib.Unlock();
			}

			return mesh.GetAttributeTable();
		}

		#endregion

		#region Static Member (Creation)

		/// <summary>
		/// Creates a textured Rectangle
		/// </summary>
		/// <param name="width">Width</param>
		/// <param name="height">Height</param>
		/// <param name="numVertexWidth">Number of vertices per width</param>
		/// <param name="numVertexHeight">Number of vertices per height</param>
		/// <param name="creationType">Creation Plane</param>
		/// <param name="invert_faces">True to invert the faces</param>
		/// <param name="scale_u">Scale for the u texcoord</param>
		/// <param name="scale_v">Scale for the u texcoord</param>
		/// <returns>a valid JMesh object</returns>
		public static JMesh CreateRectangle(float width, float height, int numVertexWidth, int numVertexHeight, JCreateRectangle creationType, bool invert_faces, float scale_u, float scale_v)
		{
			int n, a;
			int x, y;
			float fx, fy, w, h, tu, tv;

			if (numVertexWidth < 2) numVertexWidth = 2;
			if (numVertexHeight < 2) numVertexHeight = 2;

			n = numVertexWidth * numVertexHeight;

			JMesh mesh = Jad.Video.Meshes.Create();

			mesh.VertexBuffer = new JVertexBuffer(n);

			mesh.VertexBuffer.Position = new Vector3[n];

			mesh.VertexBuffer.TexCoord = new Vector2[1, n];

			w = (float) numVertexWidth - 1;
			h = (float) numVertexHeight - 1;

			for (y = a = 0; y < numVertexHeight; y++)
			{
				tv = ((float) y / h);
				fy = tv * height;
				for (x = 0; x < numVertexWidth; x++)
				{
					tu = ((float) x / w);
					fx = tu * width;
					if (creationType == JCreateRectangle.XZ)
					{
						mesh.VertexBuffer.Position[a].X = fx;
						mesh.VertexBuffer.Position[a].Y = 0;
						mesh.VertexBuffer.Position[a].Z = fy;
						// Centramos el rectngulo
						mesh.VertexBuffer.Position[a].X -= width / 2;
						mesh.VertexBuffer.Position[a].Z -= height / 2;
					}
					else if (creationType == JCreateRectangle.XY)
					{
						mesh.VertexBuffer.Position[a].X = fx;
						mesh.VertexBuffer.Position[a].Y = fy;
						mesh.VertexBuffer.Position[a].Z = 0;

						// Center the rectangle
						mesh.VertexBuffer.Position[a].X -= width / 2;
						mesh.VertexBuffer.Position[a].Y -= height / 2;

					}
					else
					{
						mesh.VertexBuffer.Position[a].X = 0;
						mesh.VertexBuffer.Position[a].Y = fy;
						mesh.VertexBuffer.Position[a].Z = fx;

						// Center the rectangle
						mesh.VertexBuffer.Position[a].Z -= width / 2;
						mesh.VertexBuffer.Position[a].Y -= height / 2;
					}

					mesh.VertexBuffer.TexCoord[0, a].X = tu * scale_u;
					mesh.VertexBuffer.TexCoord[0, a].Y = (1 - tv) * scale_v;

					a++;
				}
			}

			// Now the indices
			int[] indices, i;
			int numberOfIndices = 0, indexToCurrentVertex;

			// Suppose the worst scenario: 6 indices per vertex
			indices = i = new int[n * 6];

			int yv = 0, yJump = numVertexWidth;

			for (y = 0; y < numVertexHeight - 1; y++)
			{
				indexToCurrentVertex = yv;

				yv += numVertexWidth;

				for (x = 0; x < numVertexWidth - 1; x++)
				{

					if (invert_faces)
					{
						i[numberOfIndices + 2] = (indexToCurrentVertex);
						i[numberOfIndices + 1] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 0] = (indexToCurrentVertex + 1);

						i[numberOfIndices + 5] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 4] = (indexToCurrentVertex + yJump + 1);
						i[numberOfIndices + 3] = (indexToCurrentVertex + 1);
					}

					else
					{
						i[numberOfIndices + 0] = indexToCurrentVertex;
						i[numberOfIndices + 1] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 2] = (indexToCurrentVertex + 1);

						i[numberOfIndices + 3] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 4] = (indexToCurrentVertex + yJump + 1);
						i[numberOfIndices + 5] = (indexToCurrentVertex + 1);
					}

					numberOfIndices += 6;

					indexToCurrentVertex++;
				}
			}

			mesh.IndexBuffer = new JIndexBuffer(false, numberOfIndices);

			Array.Copy(indices, mesh.IndexBuffer.Indices, numberOfIndices);

			return mesh;
		}

		/// <summary>
		/// Builds a Text 3DMesh(without text coords) 
		/// </summary>
		/// <param name="fuente">System font</param>
		/// <param name="text">text to build</param>
		/// <param name="extrude">Z extrusion</param>
		/// <returns>A valid JMesh object</returns>
		public static JMesh CreateText3D(System.Drawing.Font fuente, string text, float extrude)
		{
			Mesh temp;
			temp = Mesh.TextFromFont(Jad.Video.Device, fuente, text, 0.001f, extrude);

			// Must be sure that there are texture coordiantes
			// Must do this because the engine uses by default the PosTex1 appeal, and
			// when using shaders < 2.0 if you use a channel that its not defined,
			// it generates an error

			JMesh mesh = Jad.Video.Meshes.Create();

			BuffersFromMesh(temp, out mesh.VertexBuffer, out mesh.IndexBuffer);

			mesh.VertexBuffer.TexCoord = new Vector2[1, mesh.VertexBuffer.NumberOfVertices];

			temp.Dispose();

			return mesh;

		}

		/// <summary>
		/// Creates a textured cylinder
		/// </summary>
		/// <param name="radio1">Inner radio</param>
		/// <param name="radio2">Outer radio</param>
		/// <param name="length">Cylinder's lenght</param>
		/// <param name="numHorizontalSlices">Number of horizontal slices</param>
		/// <param name="numVerticalSlices">Number of vertical slices</param>
		/// <returns>A valid JMesh object</returns>
		public static JMesh CreateCylinder(float radio1, float radio2, float length, int numHorizontalSlices, int numVerticalSlices)
		{
			Mesh tmp = Mesh.Cylinder(Jad.Video.Device, radio1, radio2, length, numHorizontalSlices, numVerticalSlices);

			return CalculateTextureCoords(tmp);
		}

		/// <summary>
		/// Creates a sphere with spherical texture coords
		/// </summary>
		/// <param name="radius">Radius</param>
		/// <param name="numHorizontalSlices">Number of horizontal slices</param>
		/// <param name="numVerticalSlices">Number of vertical slices</param>
		/// <returns>A valid JMesh object</returns>
		public static JMesh CreateSphere(float radius, int numHorizontalSlices, int numVerticalSlices)
		{
			Mesh tmp = Mesh.Sphere(Jad.Video.Device, radius, numHorizontalSlices, numVerticalSlices);

			return CalculateTextureCoords(tmp);
		}

		/// <summary>
		/// Creates a textured cube with 24 vertices. The faces are distributed this way:
		/// 0,1: Bottom
		/// 2,3: Top
		/// 4,5: Front
		/// 6,7: Right
		/// 8,9: Back
		/// 10,11: Left
		/// </summary>
		/// <param name="size">Width, height, depth</param>
		/// <returns>A valid JMesh object</returns>
		public static JMesh CreateCube(Vector3 size)
		{
			return CreateCube(size.X, size.Y, size.Z);
		}

		/// <summary>
		/// Creates a textured cube with 24 vertices
		/// </summary>
		/// <returns>A valid JMesh object</returns>
		public static JMesh CreateCube(float width, float height, float depth)
		{
			JMesh mesh = Jad.Video.Meshes.Create();

			mesh.VertexBuffer = new JVertexBuffer(24);
			mesh.VertexBuffer.Position = new Vector3[24];
			mesh.VertexBuffer.TexCoord = new Vector2[1, 24];

			for (int i = 0, n = 0; i < cubeVertices.Length; i += 5, n++)
			{
				mesh.VertexBuffer.Position[n].X = cubeVertices[i] * width;
				mesh.VertexBuffer.Position[n].Y = cubeVertices[i + 1] * height;
				mesh.VertexBuffer.Position[n].Z = cubeVertices[i + 2] * depth;
				mesh.VertexBuffer.TexCoord[0, n].X = cubeVertices[i + 3];
				mesh.VertexBuffer.TexCoord[0, n].Y = cubeVertices[i + 4];
			}

			mesh.IndexBuffer = new JIndexBuffer(false, cubeFaces.Length);

			cubeFaces.CopyTo(mesh.IndexBuffer.Indices, 0);

			return mesh;
		}

		/// <summary>
		/// Compute the texture coord for the cylinder and sphere objects
		/// </summary>
		/// <param name="tmp">A valid Mesh object</param>
		private static JMesh CalculateTextureCoords(Mesh tmp)
		{
			Mesh pTempMesh;

			pTempMesh = tmp.Clone(tmp.Options.Value, VertexFormats.Position | VertexFormats.Normal | VertexFormats.Texture1, Jad.Video.Device);

			pTempMesh.ComputeNormals();

			VertexBuffer v = pTempMesh.VertexBuffer;
			int n = pTempMesh.NumberVertices;

			CustomVertex.PositionNormalTextured[] verts = (CustomVertex.PositionNormalTextured[]) v.Lock(0, typeof(CustomVertex.PositionNormalTextured), LockFlags.None, n);

			// For Positional Spherical Mapping
			// Calculate center
			Vector3 vCent = new Vector3();

			using (VertexBuffer vb = tmp.VertexBuffer)
			{
				GraphicsStream vertexData = vb.Lock(0, 0, LockFlags.None);
				Geometry.ComputeBoundingSphere(vertexData,
					tmp.NumberVertices,
					tmp.VertexFormat,
					out vCent);
				vb.Unlock();
			}

			for (int i = 0; i < n; i++)
			{

				// Both Spherical Mapping techniques from
				// http://www.mvps.org/directx/articles/spheremap.htm

				/*
					// Spherical Mapping with Normals - start
					verts[i].Tu = (float)(verts[i].Nx / Math.PI + 0.5f);
					verts[i].Tv = (float)(verts[i].Ny / Math.PI + 0.5f);
					// Spherical Mapping with Normals - end
				*/

				//*
				// Positional Spherical Mapping
				// Calculate normalized offset from center
				Vector3 vert;
				vert = verts[i].Position - vCent;
				vert = Vector3.Normalize(vert);

				// calculate texture coordinates
				verts[i].Tu = (float) (Math.Asin(vert.X) / Math.PI + 0.5f);
				verts[i].Tv = (float) (Math.Asin(vert.Y) / Math.PI + 0.5f);
			}

			v.Unlock();
			v.Dispose();

			tmp.Dispose();

			JMesh mesh = Jad.Video.Meshes.Create();

			// Transform the DX mesh to Vertex & Index

			BuffersFromMesh(pTempMesh, out mesh.VertexBuffer, out mesh.IndexBuffer);

			pTempMesh.Dispose();

			return mesh;
		}

		/// <summary>
		/// Creates a polygon with any number of sides
		/// </summary>
		/// <param name="verts">Vertices that form the polygon. Must form a convex hull</param>
		/// <param name="reverse">TODO</param>
		/// <param name="u_tile">TODO</param>
		/// <param name="v_tile">TODO</param>
		public static JMesh CreatePolygon(List<Vector3> verts, bool reverse, float u_tile, float v_tile)
		{
			short numIndices = (short) ((verts.Count - 2) * 3);
			short[] indices = new short[numIndices];

			JVertexFormat[] vertexFormat = new JVertexFormat[verts.Count];

			// Fill the vertex coordinates
			for (int v = 0; v < verts.Count; v++)
			{
				int index = reverse ? verts.Count - v - 1 : v;

				vertexFormat[v].X = verts[index].X;
				vertexFormat[v].Y = verts[index].Y;
				vertexFormat[v].Z = verts[index].Z;
			}

			// Calculate the texture coordinates

			// Calculate the maximum and minimim of the polygon
			Vector3 min = new Vector3(float.MaxValue, float.MaxValue, float.MaxValue);
			Vector3 max = new Vector3(float.MinValue, float.MinValue, float.MinValue);
			foreach (Vector3 v in verts)
			{
				// Min
				if (v.X < min.X)
					min.X = v.X;
				if (v.Y < min.Y)
					min.Y = v.Y;
				if (v.Z < min.Z)
					min.Z = v.Z;

				// Max
				if (v.X > max.X)
					max.X = v.X;
				if (v.Y > max.Y)
					max.Y = v.Y;
				if (v.Z > max.Z)
					max.Z = v.Z;
			}

			// Calculate the plane where the poligon is
			Plane plane = Plane.FromPoints(verts[0], verts[1], verts[2]);

			// Depending on the plane, calculate the texture coordinates
			// This is the same process as the one used to calculate the texture coordinates on a lightmap
			for (int v = 0; v < vertexFormat.Length; v++)
			{
				// ZY plane
				if (Math.Abs(plane.A) > Math.Abs(plane.B) && Math.Abs(plane.A) > Math.Abs(plane.C))
				{
					float width = Math.Abs(max.Z - min.Z);
					float height = Math.Abs(max.Y - min.Y);

					vertexFormat[v].Tu = (vertexFormat[v].Z - min.Z) / width;
					vertexFormat[v].Tv = 1.0f - ((vertexFormat[v].Y - min.Y) / height);
				}

				// XZ plane
				else if (Math.Abs(plane.B) > Math.Abs(plane.A) && Math.Abs(plane.B) > Math.Abs(plane.C))
				{
					float width = Math.Abs(max.X - min.X);
					float height = Math.Abs(max.Z - min.Z);

					vertexFormat[v].Tu = (vertexFormat[v].X - min.X) / width;
					vertexFormat[v].Tv = 1.0f - ((vertexFormat[v].Z - min.Z) / height);
				}

				// XY plane
				else if (Math.Abs(plane.C) > Math.Abs(plane.A) && Math.Abs(plane.C) > Math.Abs(plane.B))
				{
					float width = Math.Abs(max.X - min.X);
					float height = Math.Abs(max.Y - min.Y);

					vertexFormat[v].Tu = (vertexFormat[v].X - min.X) / width;
					vertexFormat[v].Tv = 1.0f - ((vertexFormat[v].Y - min.Y) / height);
				}

				vertexFormat[v].Tu *= u_tile;
				vertexFormat[v].Tv *= v_tile;
			}

			// Calculate the indices
			short v0 = 0, v1 = 1, v2 = 2;
			for (short i = 0; i < numIndices / 3; i++)
			{
				if (i > 0)
					v1 = v2++;

				indices[i * 3 + 0] = v0;
				indices[i * 3 + 1] = v1;
				indices[i * 3 + 2] = v2;
			}

			// Create the mesh and fill it
			JMesh mesh = Jad.Video.Meshes.Create();

			Mesh temp = new Mesh(numIndices / 3, verts.Count, MeshFlags.Managed, CustomVertex.PositionTextured.Format, Jad.Video.Device);
			temp.SetVertexBufferData(vertexFormat, LockFlags.None);
			temp.SetIndexBufferData(indices, LockFlags.None);

			BuffersFromMesh(temp, out mesh.VertexBuffer, out mesh.IndexBuffer);

			temp.Dispose();

			return mesh;
		}

		#endregion
	}

	/// <summary>
	/// The list of JMesh in the engine
	/// </summary>
	public class JMeshes:IDisposable
	{
		#region Fields

		/// <summary>
		/// The list of meshes
		/// </summary>
		public JListUnique<JMesh> Items;

		#endregion

		#region List Properties

		/// <summary>
		/// Look for the list by name. The name MUST be at the list
		/// </summary>
		/// <value>The element found</value>
		public JMesh this[string name]
		{
			get { return Items[name]; }
		}

		/// <summary>
		/// Look for the list by index
		/// </summary>
		/// <value>The element found</value>
		public JMesh this[int index]
		{
			get { return Items[index]; }
		}

		/// <summary>
		/// Gets the number of elements in the collection.
		/// </summary>
		/// <value>The count of elements</value>
		public int Count
		{
			get { return Items.Count; }
		}

		/// <summary>
		/// Gets the listData elements, compatible with IEnumerator
		/// </summary>
		public List<JMesh> List
		{
			get { return Items.List; }
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JMeshes()
		{
			Items = new JListUnique<JMesh>();
		}

		#endregion

		#region Dispose

		/// <summary>
		/// Implement IDisposable
		/// </summary>
		public void Dispose()
		{
			// Dispose managed resources.
			for (int c = 0; c < List.Count; c++)
			{
				List[c].Dispose();
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Deletes all the meshes except those marked as InHouse
		/// </summary>
		public void Destroy()
		{
			bool exit = false;

			while (!exit)
			{
				exit = true;

				foreach(JMesh mesh in List)
				{
					if (!mesh.InHouse)
					{
						Delete(mesh);

						// Al eliminar un elemento de la lista, volvemos a empezar
						exit = false;

						break;
					}
				}
			}
		}

		/// <summary>
		/// Creates a new Mesh and adds it to the list. The name will be the ID
		/// generated for this mesh
		/// </summary>
		/// <returns>A valid JMesh object</returns>
		public JMesh Create()
		{
			return Create(null);
		}

		/// <summary>
		/// Creates a new Mesh and adds it to the list
		/// </summary>
		/// <param name="name">The mesh name</param>
		/// <returns>A valid JMesh object</returns>
		public JMesh Create(string name)
		{
			JMesh mesh = new JMesh(name);

			// Add the mesh to the list
			Add(mesh);

			return mesh;
		}

		/// <summary>
		/// Adds the mesh to the list
		/// </summary>
		/// <param name="mesh">A valid object</param>
		public void Add(JMesh mesh)
		{
			Items.Add(mesh);
		}

		/// <summary>
		/// Remove a mesh from the list and deletes its resources
		/// </summary>
		/// <param name="mesh">A valid JMesh to delete</param>
		public void Delete(JMesh mesh)
		{
			mesh.Dispose();

			Items.Delete(mesh);
		}

		/// <summary>
		/// Center at origin all the meshes
		/// </summary>
		public void CenterAtOrigin()
		{
			foreach (JMesh s in List)
				s.CenterAtOrigin();
		}

		/// <summary>
		/// Compute Tangent and Normal if needed
		/// </summary>
		public void ComputeTangent()
		{
			foreach (JMesh s in List)
				s.ComputeTangent();
		}

		#endregion

		#region Serializable

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;
		/// <summary>Directory binary import/export</summary>
		const string directory = @"\Meshes\";

		/// <summary>
		/// Load the meshes from the binary format
		/// </summary>
		/// <param name="import">A valid object</param>
		/// <returns>True if everything is ok</returns>
		public bool Load(JBinaryImport import)
		{
			int fileVersionaLoaded = import.LoadInt32();

			int num = import.LoadInt32();

			bool result = true;

			for (int i = 0; i < num; i++)
			{
				// We get the name
				string name = import.LoadString();

				string path = import.basePath + directory + name;

				JMesh mesh = Create(name);

				if (!mesh.Load(path)) { result = false; break; }
			}

			return result;
		}

		/// <summary>
		/// Save the Meshes to the binary format
		/// </summary>
		/// <param name="export">A valid object</param>
		/// <returns>True if everything is ok</returns>
		public bool Save(JBinaryExport export)
		{
			bool result = true;

			export.Save(fileVersion);

			export.Save(List.Count);

			string path = export.basePath + directory;

			Directory.CreateDirectory(path);

			foreach (JMesh s in List)
			{
				// Save the name at the list mesh
				export.Save(s.Name);

				// Save the mesh at the right path
				if (!s.Save(path)) { result = false; break; }
			}

			return result;
		}

		#endregion
	}
}
