#region Using directives

using System;

using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// A sampler is a texture and its sampler states setting
	/// </summary>
	public struct JSampler
	{
		#region Fields

		/// <summary>The texture</summary>
		public JTextureBase Texture;

		/// <summary>The sampler states to be applied</summary>
		public JSamplerVideoStates States;

		#endregion

		#region Methods

		/// <summary>
		/// Initializes the states to the default values and sets the texture=null
		/// </summary>
		public void Initialize()
		{
			States.Initialize();
			Texture = null;
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		internal bool Load(JBinaryImport import)
		{
			int fileVersionLoaded = import.LoadInt32();

			Texture = import.LoadTexture();

			States.Load(import);

			return true;
		}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		internal bool Save(JBinaryExport export)
		{
			export.Save(fileVersion);

			export.Save(Texture);

			States.Save(export);

			return true;
		}

		#endregion
	}
}
