#region Using directives

using System;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Implements a sprite in screen coordinates
    /// </summary>
    /// <remarks>
    /// TODO: Control from Video the sprites that are added to be able
    /// to do a Dispose if necessary
    /// </remarks>
    public class JSprite : IDisposable
    {
        #region Fields
        
    	/// <summary>
    	/// TODO
    	/// </summary>
        VertexBuffer vertexBuffer=null;

		/// <summary>
		/// TODO
		/// </summary>
		JTextureBase texture = null;

		/// <summary>
		/// TODO
		/// </summary>
		JEffect effect;

		/// <summary>
		/// The rotation angles
		/// </summary>
		Vector3 angles;

		/// <summary>
		/// The center of the sprite
		/// </summary>
		Vector3 center;

        /// <summary>
        /// The rotation matrix
        /// </summary>
        Matrix rotation;

		/// <summary>
		/// The sprite width and height
		/// </summary>
		Vector2 size;

    	/// <summary>
    	/// TODO
    	/// </summary>
		public JColor Color = JColor.White;

		/// <summary>
		/// Track whether Dispose has been called.
		/// </summary>
		private bool disposed = false;
    	
        #endregion

		#region Constructors

    	/// <summary>
    	/// Default constructor
    	/// </summary>
		public JSprite()
		{
    		// Create the vertexbuffer
            vertexBuffer = new VertexBuffer(typeof(CustomVertex.TransformedTextured), 4, Jad.Video.Device, 0, CustomVertex.TransformedTextured.Format, Pool.Managed);

			effect = Jad.Video.Effects.InHouse.PosColorTex1;

			rotation = Matrix.Identity;
		}

		#endregion
    	
        #region Properties

        /// <summary>
        /// Gets or sets the sprite center
        /// </summary>
        public Vector3 Center
        {
            get { return center; }
            set
            {
                center = value;
                Transform();
            }
        }

        /// <summary>
        /// Gets or sets the sprite width and height
        /// </summary>
        public Vector2 Size
        {
            get { return size; }
            set
            {
                size = value;
                Transform();
            }
        }

        /// <summary>
        /// Gets or sets the rotation angles
        /// </summary>
        public Vector3 Angles
        {
            get { return angles; }
            set
            {
                angles = value;
                rotation = Matrix.RotationYawPitchRoll(angles.Y, angles.X, angles.Z);
                Transform();
            }
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
        public JTextureBase Texture
        {
            get { return texture; }
			set { texture = value; }
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
		public JEffect Effect
		{
			get { return effect; }
			set { effect = value; }
		}

        #endregion

        #region Dispose

		/// <summary>
		/// Implement IDisposable
		/// </summary>
		/// <remarks>Do not make this method virtual. A derived class should not be able to override this method</remarks>
        public void Dispose()
        {
            Dispose(true);
            // This object will be cleaned up by the Dispose method.
            // Therefore, you should call GC.SupressFinalize to
            // take this object off the finalization queue 
            // and prevent finalization code for this object
            // from executing a second time.
            GC.SuppressFinalize(this);
        }

		/// <summary>TODO</summary>
		/// <param name="disposing">TODO</param>
		/// <remarks>
		/// Dispose(bool disposing) executes in two distinct scenarios.
		/// If disposing equals true, the method has been called directly
		/// or indirectly by a user's code. Managed and unmanaged resources
		/// can be disposed.
		/// If disposing equals false, the method has been called by the 
		/// runtime from inside the finalizer and you should not reference 
		/// other objects. Only unmanaged resources can be disposed.
		/// </remarks>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    vertexBuffer.Dispose();
                }

                // Call the appropriate methods to clean up 
                // unmanaged resources here.
                // If disposing is false, 
                // only the following code is executed.
            }
			
            disposed = true;
        }

        #endregion

        #region Methods

		/// <summary>
		/// Defines the position, height and width
		/// Define la posicin y el ancho y el alto
		/// </summary>
		/// <param name="x">x position</param>
		/// <param name="y">y position</param>
		/// <param name="w">Width</param>
		/// <param name="h">Height</param>
		public void SetPositionSize(int x, int y, int w, int h)
		{
			SetTopBottom(x, y, x+w - 1, y+h - 1);
		}

        /// <summary>
        /// Defines the rectangle positions
        /// </summary>
        /// <param name="x1">Left of the rectangle</param>
        /// <param name="y1">Top of the rectangle</param>
        /// <param name="x2">Right of the rectangle</param>
        /// <param name="y2">Bottom of the rectangle</param>
        public void SetTopBottom(int x1,int y1,int x2,int y2)
        {
            float yBottom = (float)y2, yTop = (float)y1;
            float xLeft = (float)x1, xRight = (float)x2;

            float width = xRight - xLeft + 1;
            float height = yBottom - yTop + 1;

            size = new Vector2(width, height);

            width /= 2; height /= 2;

            center = new Vector3(xLeft + width, yTop + height, 0);

            Transform();
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
        void Transform()
        {
            GraphicsStream stm = vertexBuffer.Lock(0, 0, 0);
            CustomVertex.TransformedTextured[] verts = new CustomVertex.TransformedTextured[4];

            float width = size.X;
            float height = size.Y;

            Matrix world = rotation * Matrix.Translation(center);

            width /= 2; height /= 2;

            Vector3 p = new Vector3(-width, height, 0);
            p.TransformCoordinate(world);

            verts[0].X = p.X; verts[0].Y = p.Y; verts[0].Z = 0f; verts[0].Rhw = 1;

            p = new Vector3(-width, -height, 0);
            p.TransformCoordinate(world);

            verts[1].X = p.X; verts[1].Y = p.Y; verts[1].Z = 0f; verts[1].Rhw = 1;

            p = new Vector3(width, height, 0);
            p.TransformCoordinate(world);

            verts[2].X = p.X; verts[2].Y = p.Y; verts[2].Z = 0f; verts[2].Rhw = 1;

            p = new Vector3(width, -height, 0);
            p.TransformCoordinate(world);

            verts[3].X = p.X; verts[3].Y = p.Y; verts[3].Z = 0f; verts[3].Rhw = 1;

            verts[0].Tu = 0; verts[0].Tv = 1;
            verts[1].Tu = 0; verts[1].Tv = 0;
            verts[2].Tu = 1; verts[2].Tv = 1;
            verts[3].Tu = 1; verts[3].Tv = 0;

            stm.Write(verts);

            vertexBuffer.Unlock();
        }

        /// <summary>
        /// Renders the sprite
        /// </summary>
		/// <remarks>
		/// This method uses TransformedTextured VertexFormat, so the vertex shader will not be called
		/// </remarks>
        public void Render()
        {
            //World = rotation * Matrix.Translation(Center);

            //effect.DXResource.SetValue(effect.handlers.worldViewProjection, World);

			effect.DXResource.SetValue(effect.handlers.textureDiffuse, texture.DXResource);
			
			if(effect.handlers.materialHandler.Diffuse!=null)
				effect.DXResource.SetValue(effect.handlers.materialHandler.Diffuse, Color.ToVector4());

			effect.SetTechnique("RenderScene");

			effect.Begin();

			effect.BeginPass(0);
			effect.Commit();

			Jad.Video.Device.SetStreamSource(0, vertexBuffer, 0);

			Jad.Video.Device.VertexFormat = CustomVertex.TransformedTextured.Format;
            Jad.Video.Device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 2);

			effect.EndPass();
			effect.End();
		}

        #endregion
    }
}
