#region Using directives

using System;
using System.IO;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// Implements the texture volume functionality
	/// </summary>
	public class JTextureVolume : JTextureBase
	{
		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="name">Name of the texture</param>
		public JTextureVolume(string name)
			: base(name)
		{ }

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the DX resource
		/// </summary>
		/// TODO (VICENTE): SHADOWING, IS THIS A GOOD IDEA???
		public new VolumeTexture DXResource
		{
			get { return (VolumeTexture) dxResource; }
			set { dxResource = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// Called after a Reset device
		/// </summary>
		public override void Reset()
		{
		}

		/// <summary>
		/// Called on OnLostDevice
		/// </summary>
		public override void OnLost()
		{
		}

		/// <summary>
		/// Loads a texture from a file using the loading parameters
		/// </summary>
		/// <returns>True if everything was right, false if not</returns>
		public bool Load()
		{
			Stream stream = null;

			// Only load the texture if it hasnt being loaded yet
			if (dxResource == null)
			{
				try
				{
					foreach (String extension in extensions)
					{
						try
						{
							stream = Jad.VFS.GetFileFromDefinedPath("Textures", name + extension, true);

							if (stream != null)
								break;
						}

						catch { }
					}

					if (stream == null)
						throw new IOException("Texture not found: " + Name);

					dxResource = TextureLoader.FromVolumeStream(Jad.Video.Device,
						stream,
						(int) stream.Length,
						0,
						0,
						0,	// Depth
						0,	// MipLevels
						LoadingParams.Usage,
						LoadingParams.Format,
						LoadingParams.Pool,
						LoadingParams.Filter,
						LoadingParams.MipFilter,
						LoadingParams.ColorKey);

					if (dxResource == null)
						return false;

					VolumeTexture texture = (VolumeTexture) dxResource;
					VolumeDescription surface = texture.GetLevelDescription(0);

					stream.Seek(0, SeekOrigin.Begin);
					imageInfo = TextureLoader.ImageInformationFromStream(stream, (int) stream.Length);

					WriteLog(surface);
				}

				catch (DirectXException e)
				{
					throw new InvalidOperationException("When loading the texture " + Name + ". Error:" + e.ErrorString);
				}

				finally
				{
					if (stream != null)
						stream.Close();
				}
			}

			return true;
		}

		/// <summary>
		/// Writes information about the texture loaded in the log
		/// </summary>
		private void WriteLog(VolumeDescription surface)
		{
			Jad.LogWriter.WriteLine("JVFS: Texture Volume:" + name);
			Jad.LogWriter.WriteLine("\tSize:" + surface.Width.ToString() + " x " + surface.Height.ToString() + " x " + surface.Depth.ToString());
			Jad.LogWriter.WriteLine("\tFormat:" + surface.Format.ToString());
			Jad.LogWriter.WriteLine(imageInfo.ToString());
			Jad.LogWriter.WriteSeparator();
		}

		#endregion
	}
}
