#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

using JadEngine;
using JadEngine.Video;
using JadEngine.Scripting;
using JadEngine.Input;
using JadEngine.Core;

#endregion

namespace JadScript
{
	public static class JadScriptOptions
	{
		static bool _reinitialize;
		static bool _shutdown;
		internal static Dictionary<string, JScriptObject> _scriptObjects;
		internal static Dictionary<string, JScriptObject> _scriptGeneratedObjects = new Dictionary<string, JScriptObject>();
		/// <summary>
		/// Shutdown the engine during the next update cycle
		/// </summary>
		public static bool Shutdown
		{
			get { return JadScriptOptions._shutdown; }
			set { JadScriptOptions._shutdown = value; }
		}

		/// <summary>
		/// Should we reinitialize our scripts on the next update?
		/// </summary>
		public static bool Reinitialize
		{
			get { return _reinitialize; }
			set { _reinitialize = value; }
		}

		private static JScriptObject GetObjectByName(string name, bool create)
		{
			JScriptObject obj = null;
			if ( (_scriptObjects != null) && (_scriptObjects.TryGetValue(name, out obj)))
				return obj;

			if (!_scriptGeneratedObjects.TryGetValue(name, out obj))
			{
				if (create)
				{
					obj = new JScriptObject(null);
					_scriptGeneratedObjects.Add(name, obj);
				}
			}
			
			return obj;
		}
		public static void SetObjectValue(string name, object value)
		{
			JScriptObject obj = GetObjectByName(name, true);
			obj.Object = value;
		}

		public static object GetObjectValue(string name)
		{
			JScriptObject obj = GetObjectByName(name, false);
			if (obj == null)
				return null;
			return obj.Object;
		}

		public static T GetObject<T>(string name)
		{
			T retVal;
			try
			{
				retVal = (T) GetObjectValue(name);
			}
			catch
			{
				retVal = default(T);
			}
			return retVal;
		}

		public static void SetObject<T>(string name, T value)
		{
			SetObjectValue(name, value);
		}
	}
}
