#region Using Directives

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;

#endregion

namespace JadScript
{
	static class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string [] args)
		{
			// If we are passed a configuration file, use it
			if (args.Length == 0)
			{
					OpenFileDialog openFileDialog1 = new OpenFileDialog();
					DirectoryInfo dInfo = new DirectoryInfo(Application.StartupPath);
					while (dInfo.GetDirectories("Scripts").Length == 0)
					{
						if (dInfo.FullName.Length < 4)
							break;
						dInfo = dInfo.Parent;
					}

					if (dInfo.GetDirectories("Scripts").Length == 0)
						openFileDialog1.InitialDirectory = dInfo.FullName;
					else
						openFileDialog1.InitialDirectory = dInfo.FullName + "\\Scripts";

					openFileDialog1.Filter = "XML files (*.XML)|*.XML";
					openFileDialog1.RestoreDirectory = true;
					openFileDialog1.Title = "Choose Configuration XML File:";
					if(openFileDialog1.ShowDialog() != DialogResult.OK)
						return;
					JadEngine.Tools.JConfiguration.ConfigurationFilename = openFileDialog1.FileName;
			}
			else
				JadEngine.Tools.JConfiguration.ConfigurationFilename = Path.Combine(Directory.GetCurrentDirectory(),args[0]);

			Directory.SetCurrentDirectory(Path.GetDirectoryName(JadEngine.Tools.JConfiguration.ConfigurationFilename));
			

			// Create a new file where well write console output
			StreamWriter sw = new StreamWriter("out.txt");

			// Redirect console output to this file
			Console.SetOut(sw);
			
			// Create the application object
			JadScriptApp app = new JadScriptApp();

			// Start the application
			try
			{
				app.Start();
			}
			catch (Exception e)
			{
				// On an exception, we need to tell the engine to shutdown gracefully,
				// or it causes another exception to throw
				JadEngine.Jad.End();

				// Put in a custom exception display handler for our application
				ExceptionDisplayForm form = new ExceptionDisplayForm(e);
				form.ShowDialog();
			}

			// Close console output file
			sw.Close();
		}
	}
}