//----------------------------------------------------------------------------------

#include "Definitions.h"

MATRIX WorldView;							// World * View
MATRIX World;
Camara camara;

texture texturaBackBuffer;

struct VS_INPUT
{
	float4 position : POSITION;
};

struct VS_OUTPUT
{
	float4 position : POSITION;
	float distance	: TEXCOORD0;
};

VS_OUTPUT VShader(VS_INPUT data_in)
{
	VS_OUTPUT data_out;
   
	data_out.position = mul(data_in.position, WorldViewProjection);

	// Distancia de la cmara al vrtice en view-space
	float4 wvPosition = mul(data_in.position, WorldView);
	data_out.distance = wvPosition.z;

	return data_out;
}

float4 PShader(VS_OUTPUT data_in) : COLOR0
{
	return data_in.distance;
}

technique RenderScene
{
	pass p0
	{
		VertexShader = compile vs_2_0 VShader();
		PixelShader = compile ps_2_0 PShader();
	}
}

/*
	QUAD
*/
struct VS_FOG_INPUT
{
	float4 position : POSITION;
};

struct VS_FOG_OUTPUT
{
	float4	position	: POSITION;
	float4	screenCoord	: TEXCOORD0;
	float	distance	: TEXCOORD1;
};

sampler map_front = 
sampler_state
{
    Texture = <texturaDiffuse>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler map_back = 
sampler_state
{
    Texture = <texturaBackBuffer>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

VS_FOG_OUTPUT VShaderFog( VS_FOG_INPUT data_in )
{
	VS_FOG_OUTPUT data_out;
   
	// Pasamos la posicin a clip-space
	data_out.position = mul(data_in.position, WorldViewProjection);

	// Output the screen-space texture coordinates
	data_out.screenCoord.x = ( data_out.position.x * 0.5f + data_out.position.w * 0.5f );
	data_out.screenCoord.y = ( data_out.position.w * 0.5f - data_out.position.y * 0.5f );
	data_out.screenCoord.z = data_out.position.w;
	data_out.screenCoord.w = data_out.position.w;

	// Hacemos la correcin de textura que necesita DX
	data_out.screenCoord.x += (0.5f / ENV_ScreenWidth) * data_out.position.w;
	data_out.screenCoord.y += (0.5f / ENV_ScreenHeight) * data_out.position.w;

	// Distancia de la cmara al vrtice en view-space
	float4 wvPosition = mul(data_in.position, WorldView);
	data_out.distance = wvPosition.z;

   return data_out;
}

float4 PShaderFog(VS_FOG_OUTPUT data_in) : COLOR0
{
	float P = data_in.distance;
	float fogStart = tex2Dproj(map_front, data_in.screenCoord).x;
	float fogEnd = tex2Dproj(map_back, data_in.screenCoord).x;

	float maxFogDist = 7.0f;

	if(fogEnd > 0)
		if(fogEnd < maxFogDist)
			fogEnd = maxFogDist;

	float fogIntensity = 1.0f - saturate( (fogEnd - P) / (fogEnd - fogStart) );

	if(fogStart == 0.0f && fogEnd == 0.0f)
		return 0;
	else
		return fogIntensity;
		//return float4(fogIntensity * float3(1, 0.3, 0.3), 1.0f);
}

technique RenderScene_Quad
{
	pass p0
	{
		VertexShader = compile vs_2_0 VShaderFog();
		PixelShader = compile ps_2_0 PShaderFog();
	}
}




struct VS_QUAD_INPUT
{
	float4 position : POSITION;
};

struct VS_QUAD_OUTPUT
{
	float4	position	: POSITION;
	float4	screenCoord	: TEXCOORD0;
};

sampler fog_map = 
sampler_state
{
    Texture = <texturaDiffuse>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

VS_QUAD_OUTPUT VShaderQuad( VS_QUAD_INPUT data_in )
{
	VS_QUAD_OUTPUT data_out;
   
	// Pasamos la posicin a clip-space
	data_out.position = mul(data_in.position, WorldViewProjection);

	// Output the screen-space texture coordinates
	data_out.screenCoord.x = ( data_out.position.x * 0.5f + data_out.position.w * 0.5f );
	data_out.screenCoord.y = ( data_out.position.w * 0.5f - data_out.position.y * 0.5f );
	data_out.screenCoord.z = data_out.position.w;
	data_out.screenCoord.w = data_out.position.w;

	// Hacemos la correcin de textura que necesita DX
	data_out.screenCoord.x += (0.5f / ENV_ScreenWidth) * data_out.position.w;
	data_out.screenCoord.y += (0.5f / ENV_ScreenHeight) * data_out.position.w;

   return data_out;
}

float4 PShaderQuad(VS_QUAD_OUTPUT data_in) : COLOR0
{
	return tex2D(fog_map, data_in.screenCoord);
}

technique RenderScene_FogQuad
{
	pass p0
	{
		VertexShader = compile vs_2_0 VShaderQuad();
		PixelShader = compile ps_2_0 PShaderQuad();
	}
}

struct VS_VOLUMETRIC_INPUT
{
	float4 position : POSITION;
};

struct VS_VOLUMETRIC_OUTPUT
{
	float4	position	: POSITION;
	float4	screenCoord	: TEXCOORD0;
	float3	wPosition	: TEXCOORD1;
};

VS_VOLUMETRIC_OUTPUT VShaderVolumetric( VS_VOLUMETRIC_INPUT data_in )
{
	VS_VOLUMETRIC_OUTPUT data_out;
   
	// Pasamos la posicin a clip-space
	data_out.position = mul(data_in.position, WorldViewProjection);

	// Output the screen-space texture coordinates
	data_out.screenCoord.x = ( data_out.position.x * 0.5f + data_out.position.w * 0.5f );
	data_out.screenCoord.y = ( data_out.position.w * 0.5f - data_out.position.y * 0.5f );
	data_out.screenCoord.z = data_out.position.w;
	data_out.screenCoord.w = data_out.position.w;

	// Hacemos la correcin de textura que necesita DX
	data_out.screenCoord.x += (0.5f / ENV_ScreenWidth) * data_out.position.w;
	data_out.screenCoord.y += (0.5f / ENV_ScreenHeight) * data_out.position.w;

	// Calculamos la posicin del vrtice en world-space
	data_out.wPosition = mul(data_in.position, World);
	
  return data_out;
}

float4 DistancePointLine( float3 Point, float3 LineStart, float3 LineEnd)
{
    float LineMag;
    float U;
    float3 Intersection;
    float3 LineVector=LineEnd-LineStart;
 
    LineMag = dot( LineVector,LineVector);
    
	// Debemos saturar porque si fuera > 1 significara que est detras del vrtice y por tanto no queremos verlo
	
    U=saturate(dot(Point-LineStart,LineVector)/(LineMag));
         
	Intersection= LineStart+U*(LineVector);
	
    return float4(Intersection,distance(Intersection,Point));
 
}
float powVolumetric;

float4 PShaderVolumetric(VS_VOLUMETRIC_OUTPUT data_in) : COLOR0
{
	float3 fragmentColor=0;
	
	// Cogemos el color del backbufer
	
	//fragmentColor = tex2Dproj(map_back, data_in.screenCoord);

	// Calculamos la distancia de la luz a la lnea que va del vrtice en WS a la cmara
	float4 intersection=DistancePointLine(LIGHT_Position,camara.posicion,data_in.wPosition);
	
	float distance=intersection.w;
	
	// A mayor distancia menos influencia
	
	//if(LIGHT_FarAttenEnd-distance>=0)
	{
		distance=max(0,1-(distance*powVolumetric));
		
		intersection.w=1;
					
		/*
		    // Calculamos el efecto spot. Pero por desgracia no hay suficientes slots libres
		    float3 wLight=LIGHT_Position-(float3)intersection;
  			
	        if(Uniform_lightType == LIGHT_SPOT)
	        {
		        // Mezclamos la atenuacin de la distancia de la luz con la atenuacin del efecto spot
		        distance *= spot(wLight);
	        }
	    */
	    
		fragmentColor+=(LIGHT_Color*distance);
		
	}
	
	return float4(fragmentColor,1.0f);
}

technique RenderScene_Volumetric
{
	pass p0
	{
		VertexShader = compile vs_2_0 VShaderVolumetric();
		PixelShader = compile ps_2_0 PShaderVolumetric();
	}
}
