AllInOne

texture textureDiffuse;
texture textureEmissive;
texture textureProjector;	// Mapa de textura del proyector
texture textureShadowMap;	// shadowMap

float power = 0.8f, intensity = 0.4f;

struct VS_DATAIN
{
	float4 position		: POSITION0;
	float2 texCoord0	: TEXCOORD0;
};

struct VS_DATAOUT
{
	float4 position			: POSITION;
	float2 texCoord0		: TEXCOORD0;
	float4 cookieTexCoord	: TEXCOORD1;
	float4 noise1TexCoord	: TEXCOORD2;
	float4 noise2TexCoord	: TEXCOORD3;
	float3 wLightVector		: TEXCOORD4;
	float4 shadowTexCoord	: TEXCOORD5;
};

VS_DATAOUT VShader( VS_DATAIN data_in )
{
	VS_DATAOUT data_out = (VS_DATAOUT)0;

	float time = (sin(ENV_Time * 0.5f) * 0.5f + 0.5f) + 1.0f;

	data_out.position = mul(data_in.position, WorldViewProjection);

	data_out.texCoord0 = data_in.texCoord0;

	float4 projectedTexCoord = mul(data_in.position, Light_ProjectorTransformation);

	data_out.cookieTexCoord = projectedTexCoord;

	float4 scroll1 = float4(0.2f, -0.1f, 0.0f, 0.0f);
	float4 scroll2 = float4(-0.1f, 0.2f, 0.0f, 0.0f);

	float s, c;
	sincos(10.0f * ENV_Time * 3.141592f / 180.0f, s, c);

	float x = projectedTexCoord.x * c - projectedTexCoord.y * s;
	float y = projectedTexCoord.x * s + projectedTexCoord.y * c;

	data_out.noise1TexCoord = projectedTexCoord;
	data_out.noise1TexCoord.xy *= 0.2f;
	data_out.noise1TexCoord += float4(scroll1.x * time, scroll1.y * time, scroll1.z * time, scroll1.w * time);
	data_out.noise1TexCoord.xy += float2(x, y);

	data_out.noise2TexCoord = projectedTexCoord;
	data_out.noise2TexCoord.xy *= 0.2f;
	data_out.noise2TexCoord += float4(scroll2.x * time, scroll2.y * time, scroll2.z * time, scroll2.w * time);
	data_out.noise2TexCoord.xy += float2(-x, -y);

	data_out.wLightVector = LIGHT_Position - data_in.position;
	
	data_out.shadowTexCoord = mul(data_in.position, Light_ShadowMapTransformation);

	return data_out;
}

sampler base_map =
sampler_state
{
	Texture		= <textureDiffuse>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
    AddressU	= Wrap;
    AddressV	= Wrap;
};

sampler noise_map =
sampler_state
{
	Texture		= <textureEmissive>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
    AddressU	= Wrap;
    AddressV	= Wrap;
};

sampler projector_map =
sampler_state
{
	Texture		= <textureProjector>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
    AddressU	= Border;
    AddressV	= Border;
};

sampler shadow_map =
sampler_state
{
	Texture		= <textureShadowMap>;
	MipFilter	= POINT;
	MinFilter	= POINT;
	MagFilter	= POINT;
	AddressU	= Clamp;
	AddressV	= Clamp;
};

struct PS_DATAIN
{
	float2 texCoord0		: TEXCOORD0;
	float4 cookieTexCoord	: TEXCOORD1;
	float4 noise1TexCoord	: TEXCOORD2;
	float4 noise2TexCoord	: TEXCOORD3;
	float3 wLightVector		: TEXCOORD4;
	float4 shadowTexCoord	: TEXCOORD5;
};

// Funcin que calcula la atenuacin
float distanceAttenuation(float3 lightVector)
{
	float lDist = length(lightVector);
	float atten = 1 - saturate(lDist / LIGHT_FarAttenEnd);
	return pow(atten * atten, power);
}

// Funcin que calcula la atenuacin para el cono de las luces de tipo spot
float coneAttenuation(float3 nLightVector)
{
	float3 L = -nLightVector;
	return smoothstep(LIGHT_FallOff, LIGHT_HotSpot, dot(L, LIGHT_Direction));
}

float4 PShader( PS_DATAIN data_in, uniform bool useCookie, uniform bool useShadows) : COLOR0
{
	float4 fragmentColor = float4(1.0f, 1.0f, 1.0f, 1.0f);

	// Cookie contribution
	float4 cookieColor = 1;
	if (useCookie)
		cookieColor = tex2Dproj(projector_map, data_in.cookieTexCoord);
	
	// Si if this fragment is in shadow
	float shadow = 1.0f;
	if (useShadows)
	{
		// Distance to the fragment in light clip-space
		float dist = 1 - (data_in.shadowTexCoord.z / data_in.shadowTexCoord.w);
		// If distance to the shadow fragment is less than distance to the fragment, the fragment is in shadow
		shadow = (tex2Dproj(shadow_map, data_in.shadowTexCoord).x > dist) ? 0.0f : 1.0f;
	}
	
	// Calculate attenuation
	float attenuation = distanceAttenuation(data_in.wLightVector);
	attenuation *= coneAttenuation(normalize(data_in.wLightVector));
	
	// Calculate noise contribution
	float noiseFactor = 1.0f;
	//float noise1 = tex2Dproj(noise_map, data_in.noise1TexCoord).r;
	//float noise2 = tex2Dproj(noise_map, data_in.noise2TexCoord).g;
	//noiseFactor = lerp(noise1, noise2, 0.5f).r * 0.6;
	
	// Final color
	fragmentColor.rgb = cookieColor * noiseFactor * attenuation * LIGHT_Color * shadow * intensity;
	//fragmentColor.rgb = projectorColor * attenuation * LIGHT_Color * shadow;
	fragmentColor.a = saturate(dot(fragmentColor.rgb, float3(1.0f, 1.0f, 1.0f)));

	return fragmentColor;
	//return fragmentColor.ggga;
}

technique RenderScene
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader(false, false);
    }
}

technique RenderScene_CookieAndShadow
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader(true, true);
    }
}

technique RenderScene_CookieAndNoShadow
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader(true, false);
    }
}

technique RenderScene_NoCookieAndShadow
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader(false, true);
    }
}