AllInOne

struct VS_INPUT
{
	float4 position:     POSITION;
	float2 texCoord0:    TEXCOORD0;
	float3 Normal    : NORMAL;  
	float3 Tangent   : TANGENT; 
};

struct VS_OUTPUT
{
	float4 position:     POSITION;
    float3 Eye        : TEXCOORD0;
    float4 Wave0      : TEXCOORD1;
    float2 Wave1      : TEXCOORD2;
    float2 Wave2      : TEXCOORD3;
    float2 Wave3      : TEXCOORD4;        
    float4 ScreenPos  : TEXCOORD5;            
    float3 Light   	  : TEXCOORD6;
};
float3 lightDirection;

VS_OUTPUT VShader( VS_INPUT vin )
{
	VS_OUTPUT data_out;
   
	//float3 wPos = mul(vin.position, World);
	data_out.position = mul(vin.position, WorldViewProjection);
	data_out.ScreenPos = ToScreenSpace(data_out.position);

    float2 fTranslation=float2(fmod(ENV_Time, 100.0f)*0.01f, 0);
    //float2 vTexCoords=wPos.xy*0.001f;
    float2 vTexCoords=vin.texCoord0;
	
    // Output bump layers texture coordinates    
    //float fSinTranslation=sin(fTranslation.x*100.0f)*0.005f;
    //float2 vTranslation0=fTranslation+fSinTranslation;
    //float2 vTranslation1=fTranslation-fSinTranslation;
    //float2 vTranslation2=fTranslation;
    
    // Scale texture coordinates to get mix of low/high frequency details
    data_out.Wave0.xy = vTexCoords.xy+fTranslation*2.0f;
    data_out.Wave1.xy = vTexCoords.xy*2.0f+fTranslation*4.0f;
    data_out.Wave2.xy = vTexCoords.xy*4.0f+fTranslation*2.0f;
    data_out.Wave3.xy = vTexCoords.xy*8.0f+fTranslation;                
                  
    // perspective corrected projection      
    data_out.Wave0.zw=data_out.position.w;
	
    // get tangent space basis    
    float3x3 objToTangentSpace=GetTangentSpace(vin.Tangent, vin.Normal);
            
    float3 EyeVec=camera.position-mul(vin.position,World); 
    data_out.Eye = mul(objToTangentSpace, EyeVec);
	
	//float3 L = normalize( float3(20.0f, 10.0f, 26.0f) );
	float3 L=normalize(lightDirection);
	data_out.Light=mul(objToTangentSpace,L);	
	
	return data_out;
}

struct PS_INPUT
{
    float3 Eye        : TEXCOORD0;
    float4 Wave0      : TEXCOORD1;
    float2 Wave1      : TEXCOORD2;
    float2 Wave2      : TEXCOORD3;
    float2 Wave3      : TEXCOORD4;        
    float4 ScreenPos  : TEXCOORD5;            
    float3 Light      : TEXCOORD6;
};

texture _currentRender;
texture _normalMap;
texture textureSpecular;

sampler currentRender : register(s0) = 
sampler_state
{
    Texture = <_currentRender>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler normalMap : register(s1) = 
sampler_state
{
    Texture = <_normalMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler specular_map: register(s2) =
sampler_state
{
	Texture		= <textureSpecular>;
};

// This assumes NdotL comes clamped
float Fresnel(float NdotL, float fresnelBias, float fresnelPow)
{
  float facing = (1.0 - NdotL);
  return  max(fresnelBias + (1.0-fresnelBias)*pow(facing, fresnelPow), 0.0);
}

float4 PShader(PS_INPUT data_in):COLOR0
{
	float3 vEye=normalize(data_in.Eye);
	
  // Get bump layers
  float3 vBumpTexA = tex2D(normalMap, data_in.Wave0.xy).xyz;
  float3 vBumpTexB = tex2D(normalMap, data_in.Wave1.xy).xyz;
  float3 vBumpTexC = tex2D(normalMap, data_in.Wave2.xy).xyz;
  float3 vBumpTexD = tex2D(normalMap, data_in.Wave3.xy).xyz;
  
  // Average bump layers
  float3 vBumpTex=normalize(2.0 * (vBumpTexA.xyz + vBumpTexB.xyz + vBumpTexC.xyz + vBumpTexD.xyz)-4.0);
  
  // Apply individual bump scale for refraction and reflection
  float3 vRefrBump = vBumpTex.xyz * float3(0.02f, 0.02, 1.0f);
  float3 vReflBump = vBumpTex.xyz * float3(0.1f, 0.1f, 1.0f);  
  // Compute projected coordinates
  float2 vProj = (data_in.ScreenPos.xy/data_in.ScreenPos.w);
  float4 vReflection = tex2D(specular_map, vProj.xy+ vReflBump.xy);
  float4 vRefrA = tex2D(currentRender, vProj.xy + vRefrBump.xy);
	// Mask occluders from refraction map

	//float4 vRefrB = tex2D(currentRender, vProj.xy);

	//float4 vRefraction = vRefrB * vRefrA.w + vRefrA * (1 - vRefrA.w);
	float4 vRefraction = vRefrA	;

  // Compute Fresnel term
  float NdotL = max(dot(vEye, vReflBump), 0);
  float facing = (1.0 - NdotL);
  float fresnel = Fresnel(NdotL, 0.2f, 5.0f);
  
  // Use distance to lerp between refraction and deep water color
  float fDistScale = saturate(0.8f/data_in.Wave0.w);
  float3 WaterDeepColor = (vRefraction.xyz * fDistScale + (1.0f - fDistScale) * float3(0, 0.1f, 0.125f));  
  
  // Lerp between water color and deep water color
  float3 WaterColor = float3(0, 0.1f, 0.15f);
  float3 waterColor = (WaterColor * facing + WaterDeepColor * (1.0f - facing));
  float3 cReflect = fresnel * vReflection;

  float3 L = data_in.Light;
  float3 H = normalize(L + vEye);
  float3 specColor = (specular(vBumpTex, H, MATERIAL_SpecularPower) *fresnel)* MATERIAL_Specular.rgb;
  
  cReflect+=specColor;

  // final water = reflection_color * fresnel + water_color
  return float4(cReflect + waterColor, 1);  
  
}

technique RenderScene
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader();
    }
}

