//----------------------------------------------------------------------------------
#region Enum

// Tipos de onda
Wave { Sin, Triangle, Square, SawTooth, InvSawTooth, WTurb }

#end Enum

//*********** Las siguientes variables DEBEN tener EXACTAMENTE este nombre ***********

// Definimos el tipo MATRIX como una matriz de 4x4 floats

#define MATRIX matrix<float, 4, 4>

#define PI 3.1415926535897932384626433832795

MATRIX World;					// World
MATRIX WorldView;				// World * View
MATRIX WorldViewProjection;		// World * View * Projection
MATRIX IWorld;					// Inversa de World
MATRIX View;					// View

#define ZPASS_NONE 0
#define ZPASS_WITHOUTALPHATEST 1
#define ZPASS_WITHALPHATEST 2

#define FOG_NONE 0
#define FOG_LINEAR 1
#define FOG_EXPONENTIAL 2
#define FOG_EXPONENTIAL_SQUARED 3
#define FOG_LAYERED 4

#define EMISSIVEMODEL_TEXTURE 1
#define EMISSIVEMODEL_COLOR 2
#define EMISSIVEMODEL_DIFFUSE 3


// Tipos de luz
#define LIGHT_OMNI		0
#define LIGHT_SPOT		1
#define LIGHT_DIRECT	2
#define LIGHT_UNKNOWN	3

// Normal
#define NORMAL_NONE 0
#define NORMAL_VERTEX 1
#define NORMAL_MAPPING 2

// Estructura que representa una funcin de	onda
struct Wave	
{
	int		type;		// Tipo	de onda	que	se define
	float	base;		// Base	de la onda
	float	amplitude;	// Amplitud	de la onda
	float	phase;		// Fase	de la onda
	float	frequency;	// Frecuencia de la	onda en	Hz
};

// Funcin que evalua una funcin de onda
float evalWave(Wave	wave, float	time)
{
	float x, y;	

	x =	(time +	wave.phase)	* wave.frequency;
	x -= floor(x);

	if(wave.type == WAVE_SIN)
	{
		y =	sin(x *	2.0f * PI);	
	}
	else if(wave.type == WAVE_SQUARE)
	{
		if (x <	0.5f)
			y =	1.0f;
		else
			y =	-1.0f;
	}
	else if(wave.type == WAVE_SAWTOOTH)
	{
		y =	x;
	}
	else if(wave.type == WAVE_INVSAWTOOTH)
	{
		y =	1.0f - x;
	}
	else if(wave.type == WAVE_TRIANGLE)
	{
		if (x <	0.5f)
			y =	4.0f * x - 1.0f;
		else
			y =	-4.0f *	x +	3.0f;
	}

	return (y *	wave.amplitude)	+ wave.base;
}

//--------------------------------------------------------------------------------------------------
// Light struct. We must do this way to minimize the shader's constant table
//--------------------------------------------------------------------------------------------------

#define LIGHT_FarAttenStart Light_FarAttenStart_End_hotSpot_FallOff.x
#define LIGHT_FarAttenEnd Light_FarAttenStart_End_hotSpot_FallOff.y
#define LIGHT_HotSpot Light_FarAttenStart_End_hotSpot_FallOff.z
#define LIGHT_FallOff Light_FarAttenStart_End_hotSpot_FallOff.w

#define LIGHT_Atten_Constant Light_AttenOpenGL.x
#define LIGHT_Atten_Linear Light_AttenOpenGL.y
#define LIGHT_Atten_Quadratic Light_AttenOpenGL.z

#define LIGHT_Position Light_Position.xyz
#define LIGHT_Direction Light_Direction.xyz
#define LIGHT_Color Light_Color
#define LIGHT_PowVolumetric Light_Position.w

float4 Light_FarAttenStart_End_hotSpot_FallOff;
float4 Light_AttenOpenGL;

float4 Light_Position;
float4 Light_Color;
float4 Light_Direction;			// w free
MATRIX	Light_ProjectorTransformation;
MATRIX	Light_ShadowMapTransformation;

//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
// Material struct. We must do this way to minimize the shader's constant table
//--------------------------------------------------------------------------------------------------
#define MATERIAL_SpecularPower Material_SpecularPower_Bumpiness_Reflectivity_Transmittance.x
#define MATERIAL_Bumpiness Material_SpecularPower_Bumpiness_Reflectivity_Transmittance.y
#define MATERIAL_Reflectivity Material_SpecularPower_Bumpiness_Reflectivity_Transmittance.z
#define MATERIAL_Transmittance Material_SpecularPower_Bumpiness_Reflectivity_Transmittance.w
#define MATERIAL_Specular Material_Specular
#define MATERIAL_Diffuse Material_Diffuse
#define MATERIAL_Emissive Material_Emissive.xyz
#define MATERIAL_IntensityEmissive Material_Emissive.w
#define MATERIAL_IOR Material_IOR.x
#define DETAIL_Amount Material_IOR.y
#define MATERIAL_ENVMAP_DISTANCE Material_IOR.z
#define MATERIAL_GHOST_POWER Material_IOR.w
#define MATERIAL_GHOST_ADD Material_ParallaxAmount.y
#define MATERIAL_PARALLAXAMOUNT Material_ParallaxAmount.x
#define MATERIAL_PARALLAX_PERSPECTIVEBIAS  Material_ParallaxAmount.z
#define MATERIAL_PARALLAX_SHADOWOPACITY Material_ParallaxAmount.w

float4	Material_Specular;
float4	Material_SpecularPower_Bumpiness_Reflectivity_Transmittance;
float4	Material_Diffuse;
float4	Material_Emissive;
float4	Material_IOR;	
float4	Material_ParallaxAmount;

//--------------------------------------------------------------------------------------------------

struct Camera {
	float4	position;
};

#define ENV_Ambient Env_Ambient_TimeElapsed.xyz
#define ENV_Time Env_Ambient_TimeElapsed.w

#define ENV_ScreenWidth Env_Screen_Shadows.x
#define ENV_ScreenHeight Env_Screen_Shadows.y
#define ENV_ShadowWidth Env_Screen_Shadows.z
#define ENV_ShadowHeight Env_Screen_Shadows.w

#define DOF_nearBlurDepthRange		DOF_planes.x
#define DOF_nearFocalPlaneDepth		DOF_planes.y
#define DOF_farFocalPlaneDepth		DOF_planes.z
#define DOF_farBlurDepthRange		DOF_planes.w

#define FOG_Near FogValues.x
#define FOG_Far FogValues.y
#define FOG_Density FogValues.z
#define FOG_Bottom FogValues.w
#define FOG_Top FogValues.x

#define FOG_Color FogColor.xyz

float DOF_maxBlur;
float4 DOF_planes;	// DOF_nearBlurDepthRange / DOF_nearFocalPlaneDepth / DOF_farFocalPlaneDepth / DOF_farBlurDepthRange

float4	Env_Ambient_TimeElapsed;
float4	Env_Screen_Shadows;	// los 2 primeros informan del ancho/alto del RT. Los 2 siguientes del ancho/alto de las sombras

float4 FogValues;   // Near / Far / Density / Bottom
float4 FogColor;    // Color / PenumbraWidth
//------------------------------------------------------------------------------------
// Mapas de textura. Por desgracia no se puede aadir la textura dentro de la estructura
//------------------------------------------------------------------------------------

struct map {
	float2	tile;		// u,v del tiling de la textura
};


Camera camera;

#define SHADOW_EPSILON -0.0007f

float shadow_bias = 0.0f;

struct VS_IN
{
	float4 position0	: POSITION0;
	float4 position1	: POSITION1;
	float3 normal0		: NORMAL0;
	float3 normal1		: NORMAL1;
	float3 tangent0		: TANGENT0;
	float3 tangent1		: TANGENT1;
	float2 texCoord0	: TEXCOORD0;
	float2 texCoord1	: TEXCOORD1;
	float2 texCoord2	: TEXCOORD2;
	float4 diffuse		: COLOR0;
	float4 specular		: COLOR1;
};

float4 ToScreenSpace(float4 position)
{
    float4 screenCoord;
    
	screenCoord.x = ( position.x * 0.5f + position.w * 0.5f );
	screenCoord.y = ( position.w * 0.5f - position.y * 0.5f );
	screenCoord.z = position.w;
	screenCoord.w = position.w;

	// Hacemos la correcin de textura que necesita DX
	screenCoord.x += (0.5f / ENV_ScreenWidth) * position.w;
	screenCoord.y += (0.5f / ENV_ScreenHeight) * position.w;
	
	return screenCoord;
}

// Funcin que pasa un vector del rango [0,1] al [-1,1]
float3 expandI(float3 v)
{
	return (v * 2.0f - 1.0f);
}

float3 GetNormal(float4 normal)
{
	float3 N ;
	
	if(Uniform_NORMALRTFLOATINGPOINT)
	{
		N = normal.xyz;
	}
	else
	{
		// Esto es en el caso de que se utilizara un RT de 32 bits
		N = expandI(normal.xyz);
	}

	//N = normalize(N);
	return N;
}

float3 StoreNormal(float3 normal)
{
	float3 N ;
	
	if(Uniform_NORMALRTFLOATINGPOINT)
	{
		N = normal;
	}
	else
	{
		// Esto es en el caso de que se utilizara un RT de 32 bits
		N = normal * 0.5f + 0.5f;
	}

	return N;
}