//--------------------------------------------------------------------------------------------------
// ALLINONE:PixelShader
//--------------------------------------------------------------------------------------------------
struct PS_IN
{
	float4 texCoord0	: TEXCOORD0;
	float4 wPosition		: TEXCOORD1;
	float4 screenCoord	: TEXCOORD2;
	float4 diffuse		: COLOR0;
	float4 specular		: COLOR1;
};

float4 PShader(PS_IN data_in) : COLOR0
{
	float4 colorFirstPass;

  	if(Uniform_Fog==FOG_LAYERED)
  	{      
 	    // Get fog parameter
	    float fFogTop	= FOG_Top;
	    float fFogEnd	= FOG_Far;
	    float fFogRange	= FOG_Top;
	    float fFogDensity = FOG_Density;
        float fFogBottom = FOG_Bottom;
        
	    // Calculate the world position
	    float4 worldPos		= data_in.wPosition;
	    
	    float4 vCamera=camera.position;
    	
	    // Project both points into the x-z plane
	    float4	vCameraProj, vWorldProj;
	    vCameraProj		= vCamera;
	    vCameraProj.y	= 0;
	    vWorldProj		= worldPos;
	    vWorldProj.y	= 0;
    	
	    // Scaled distance calculation in x-z plane
	    float fDeltaD = distance(vCameraProj, vWorldProj)/fFogEnd*2.0f;

	    // Height based calculations
	    float fDeltaY, fDensityIntegral;
	    
	    // Comprobamos si la cmara est por encima de la niebla
	    if(vCamera.y > fFogTop || vCamera.y < fFogBottom)
	    {
	        // si el vrtice est por debajo de la niebla
	        
		    if (worldPos.y >= fFogBottom && worldPos.y < fFogTop)
		    {
			    fDeltaY = (fFogTop - worldPos.y) / fFogRange * 2.0f;
			    fDensityIntegral = (fDeltaY * fDeltaY *fFogDensity);
		    }
		    else
		    {
			    fDeltaY = 0.0f;
			    fDensityIntegral = 0.0f;
		    }
	    }
	    else    // La cmara est por debajo, es decir, en la niebla
	    {
	        // si el vrtice est por debajo de la niebla
	        
		    if (worldPos.y >= fFogBottom && worldPos.y < fFogTop)
		    {
			    float fDeltaA = (fFogTop - vCamera.y) / fFogRange * 2;
			    float fDeltaB = (fFogTop - worldPos.y) / fFogRange * 2;
			    fDeltaY =abs(fDeltaA -fDeltaB);
			    fDensityIntegral = abs((fDeltaA * fDeltaA * fFogDensity) - (fDeltaB * fDeltaB * fFogDensity));

		    }
		    else
		    {
			    fDeltaY = abs(fFogTop - vCamera.y) / fFogRange * 2;
			    fDensityIntegral = abs(fDeltaY * fDeltaY *fFogDensity);
		    }
	    }

	    float fDensity;
	    if (fDeltaY != 0.0f)
	    {
		    fDensity = (sqrt(1.0f + ((fDeltaD / fDeltaY) * (fDeltaD / fDeltaY)))) * fDensityIntegral;
	    }
	    else
		    fDensity = 0.0f;

	    float f= exp(-fDensity);

  	    colorFirstPass= tex2Dproj(BackBuffer_map, data_in.screenCoord);
	    colorFirstPass=lerp(FogColor,colorFirstPass,f);
	}
	else
  	{
  	    colorFirstPass = tex2Dproj(BackBuffer_map, data_in.screenCoord);
	    colorFirstPass = lerp(FogColor, colorFirstPass, data_in.wPosition.w);
	}
	
	return colorFirstPass;

}


