struct VS_OUT
{
	float4 position		: POSITION;
	float4 texCoord0	: TEXCOORD0;	// Esto incluye la 0 y la 1
	float4 t1			: TEXCOORD1;
	float4 t2			: TEXCOORD2;
	float4 t3			: TEXCOORD3;
	float4 t4			: TEXCOORD4;
	float4 t5			: TEXCOORD5;
	float4 t6			: TEXCOORD6;
	float4 t7			: TEXCOORD7;
	float4 diffuse		: COLOR0;
	float4 specular		: COLOR1;
};

VS_OUT VShader( VS_IN data_in)
{
	VS_OUT data_out = (VS_OUT)0;

	float4 finalPosition	= data_in.position0;
	float3 finalNormal		= data_in.normal0;
	float3 finalTangent		= data_in.tangent0;
	
	data_out.diffuse = float4(1, 1, 1, 1);
	data_out.specular = float4(1, 1, 1, 1);
	
    float3 tViewVector;					// Tangent space View Vector
    float3 wNormal,wTangent,wBinormal;	// Normal to World Space
    float4 screenCoord,projTexCoord;
    float3 wLightVector,wViewVector,wPosition;
    
    tViewVector=wNormal=wTangent=wBinormal=float3(0,0,0);
    wLightVector=wViewVector=wPosition=float3(0,0,0);
    screenCoord=projTexCoord=float4(0,0,0,0);
    
	// ALLINONE:Geometry

	// Pasamos la posicin a clip-space
	data_out.position = mul(finalPosition, WorldViewProjection);

	if(Uniform_useTexCoordForDetailMap)
	{
    	data_out.texCoord0.xy = data_in.texCoord1 * mapDiffuse.tile;
    	data_out.texCoord0.zw = data_in.texCoord2 * mapDiffuse.tile;
    }
	else 
	{
	    data_out.texCoord0.xy = data_in.texCoord0 * mapDiffuse.tile;
	}
		
    // Coordenadas en Screen Space
    screenCoord=ToScreenSpace(data_out.position);
	
	// Calculamos la posicin del vrtice en world-space
	wPosition = mul(finalPosition, World);
		
	// ALLINONE:LIGHTING

	// Light vector en world-space
	if(Uniform_lightType == LIGHT_DIRECT) 
		wLightVector= -LIGHT_Direction;
	else
		wLightVector = LIGHT_Position - wPosition;
	
	// View vector en world-space
	wViewVector	= camera.position - wPosition;

	// Si el proyector est activado, pasamos la coordenada de textura transformada
	if(Uniform_projectorEnabled)
	{
		if(Uniform_lightType == LIGHT_OMNI)
		{
			projTexCoord = mul(wLightVector, Light_ProjectorTransformation);
		}
		else
		{
			projTexCoord = mul(float4(wPosition,1.0f), Light_ProjectorTransformation);
		}
	}
    
	// Normal en world-space
	// Como utilizamos la inversa traspuesta del world, hacemos la multiplicacin modificando
	// el orden de los parmetros
	/*
	wNormal = mul(finalNormal, (float3x3)World);
	wTangent = mul(finalTangent, (float3x3)World);
	*/
	wNormal = mul((float3x3)IWorld,finalNormal);
	wTangent = mul((float3x3)IWorld,finalTangent);
	
	// Preparamos la matriz para pasar a tangentspace
	float3x3 toTangentSpace=GetTangentSpace(wTangent, wNormal);
	wBinormal=toTangentSpace[1];
	
	// Pasamos el vector view a tangent-space
	tViewVector	= mul(toTangentSpace,wViewVector );
	
	// Si queremos la normal en tangent space, debemos pasar al PS la informacin
	if(!Uniform_WorldSpaceNormals)
	{
	
		// Pasamos el vector de la luz a tangent-space
		wBinormal = mul(toTangentSpace, wLightVector);
		// Pasamos el vector de la normal a tangent-space
		wNormal = mul(toTangentSpace, wNormal);
	
	}

	
    data_out.t1.xyz=tViewVector;
    data_out.t1.w=wNormal.x;
    data_out.t2.xy=wNormal.yz;
    data_out.t2.zw=wTangent.xy;
    data_out.t3.x=wTangent.z;
    data_out.t3.yzw=wBinormal;
    data_out.t4.xyz=screenCoord.xyw;
    data_out.t4.w=wPosition.z;    
    data_out.t5=projTexCoord;
    data_out.t6.xyz=wLightVector;
    data_out.t6.w=wViewVector.x;
    data_out.t7.xy=wViewVector.yz;
    data_out.t7.zw=wPosition.xy;
    
	return data_out;
}
