// Funcin que calcula la contribucin de la componente diffuse
float diffuse(float3 N, float3 L)
{
	return max(0.0f, dot(N, L));
}

// Funcin que calcula la contrubucin de la componente specular
float specular(float3 R, float3 V, float specularPower)
{
	return pow(max(0.0f, dot(R, V)), specularPower);
}

// Funcin que calcula la atenuacin para el cono de las luces de tipo spot
float spot(float3 nlightVector)
{
	float3 L = -nlightVector;
	return smoothstep(LIGHT_FallOff, LIGHT_HotSpot, dot(L, LIGHT_Direction));
}

// Funcin que calcula la contribucin del proyector de la luz
float4 projector(sampler projectorMap, float4 projTexCoord)
{
	float4 projectorColor = 1.0f;

	if (Uniform_lightType == LIGHT_OMNI) {
		projectorColor = texCUBEproj(projectorMap, -projTexCoord);
	} 
	else {
		projectorColor = projTexCoord.w < 0.0f ? 0.0f : tex2Dproj(projectorMap, projTexCoord);
	}
	
	return projectorColor;
}

// Funcin que calcula la contribucin de un environment map
float4 environment(sampler envMap, float3 texCoords)
{
	return texCUBE(envMap, texCoords);
}

// Funcin que pasa un vector del rango [0,1] al [-1,1]
float3 expand(float3 v)
{
	return (v * 2.0f - 1.0f);
}

// ALLINONE:LIGHT SHADER

////////////////////////////////////////////////////////////////////////////
// Compute the [0,1] clamped dot3 for vec0 and vec1 
////////////////////////////////////////////////////////////////////////////
float Dot3Clamp (float3 vec0, float3 vec1)
{
    return saturate (dot (vec0, vec1));
}

/////////////////////////////////////////////////////////////////////////////
// Computes an approximated fresnel term == [((1.0f - N.V)^5) * 0.95f] + 0.05f
/////////////////////////////////////////////////////////////////////////////
float ComputeFresnelApprox (float3 normalVec, float3 viewVec)
{
    // (1-N.V)^4
    float NdotV5 = pow (1.0f - Dot3Clamp (normalVec, viewVec), 4.0f);

    // scale and bias to fit to real fresnel curve
    return (NdotV5 * 0.95f) + 0.05f;
}

////////////////////////////////////////////////////////////////////////////
// Compute transmission direction T from incident direction I, normal N,
// going from medium with refractive index n1 to medium with refractive
// index n2, with refraction governed by Snell's law:
//               n1*sin(theta1) = n2*sin(theta2).
// If there is total internal reflection, return 0, else set T and
// return 1. All vectors unit.
// From: http://research.microsoft.com/~hollasch/cgindex/render/refraction.txt
////////////////////////////////////////////////////////////////////////////
float3 TransmissionDirection (float fromIR, float toIR, 
                                float3 incoming, float3 normal)
{
	float eta = fromIR/toIR; // relative index of refraction
    float c1 = -dot (incoming, normal); // cos(theta1)
    float cs2 = 1.-eta*eta*(1.-c1*c1); // cos^2(theta2)
    float3 v = (eta*incoming + (eta*c1-sqrt(cs2))*normal);
    if (cs2 < 0.) v = 0; // total internal reflection
    return v;
}

float2 CalculateParallax(float height, float2 tex_coords, float3 view_vector, float parallax_amount)
{
	return (height * parallax_amount - parallax_amount * 0.5f) * view_vector + tex_coords;
}
/*
float2 CalculateParallaxOcclusion2(sampler normal_map,float height, float2 inTexCoord, float3 vView, float parallax_amount)
{
   // Compute initial parallax displacement direction:
   float2 vParallaxDirection= vView.xy;
   
   // Compute the length of parallax displacement vector
   // i.e. the amount of parallax displacement.
   float fParallaxLength = -sqrt( 1 - vView.z * vView.z ) / vView.z; 
   
   // Compute parallax bias parameter which allows
   // the artists control over the parallax perspective
   // bias via an artist editable parameter 'fPerspectiveBias'
   // Use view vector length in tangent space as an offset 
   // limiting technique as well:
   float fPerspectiveBias=0.5f;
   float fBumpHeightScale=0.94f;fBumpHeightScale=parallax_amount;
   
   float fParallaxBias = fPerspectiveBias + (1 - fPerspectiveBias) * ( 2 * vView.z - 1 );
   
   //fParallaxBias=1.0f;
   
   // Compute the actual reverse parallax displacement vector:
   float2 inParallax = -vParallaxDirection * fParallaxLength * fParallaxBias * fBumpHeightScale; 
   
   // Compute height map profile for 8 samples:
   // Note that the height map is contained in the alpha channel
   // of the input normal map:
   float h0 = tex2D( normal_map, inTexCoord - inParallax * 1.000 ).w;
   float h1 = tex2D( normal_map, inTexCoord - inParallax * 0.875 ).w;
   float h2 = tex2D( normal_map, inTexCoord - inParallax * 0.750 ).w;
   float h3 = tex2D( normal_map, inTexCoord - inParallax * 0.625 ).w;
   float h4 = tex2D( normal_map, inTexCoord - inParallax * 0.500 ).w;
   float h5 = tex2D( normal_map, inTexCoord - inParallax * 0.375 ).w;
   float h6 = tex2D( normal_map, inTexCoord - inParallax * 0.250 ).w;
   float h7 = tex2D( normal_map, inTexCoord - inParallax * 0.125 ).w;
   
   // Determine the section in the height profile that contains
   // a sample that is higher than the view vector. This determines
   // the correct displacement point:    
   float x;
   float y;
   float xh;
   float yh;
   
   if      (h7 > 0.875) { x = 0.937; y = 0.938; xh = h7; yh = h7; }
   else if (h6 > 0.750) { x = 0.750; y = 0.875; xh = h6; yh = h7; }
   else if (h5 > 0.625) { x = 0.625; y = 0.750; xh = h5; yh = h6; }
   else if (h4 > 0.500) { x = 0.500; y = 0.625; xh = h4; yh = h5; }
   else if (h3 > 0.375) { x = 0.375; y = 0.500; xh = h3; yh = h4; }
   else if (h2 > 0.250) { x = 0.250; y = 0.375; xh = h2; yh = h3; }
   else if (h1 > 0.125) { x = 0.125; y = 0.250; xh = h1; yh = h2; }
   else                 { x = 0.000; y = 0.125; xh = h0; yh = h1; }
   
   // Compute the intersection between the view vector and the highest
   // sample point in the height profile:
   float fParallaxEffect = (x * (y - yh) - y * (x - xh))/((y - yh) - (x - xh));
   
   float2 parallaxDisplacedCoord=inTexCoord-float2( inParallax * (1 - fParallaxEffect ));
   
   return parallaxDisplacedCoord;
}
*/
float ComputeParallaxOcclusionShadowing(sampler normal_map,float2 inTexCoord,float2 inXY,float2 inFade)
{
   float fShadowSoftening = 0.3f;
   
   // Compute the blurry shadows (note that this computation takes into 
   // account self-occlusion for shadow computation):
   float h0 =  tex2D( normal_map, inTexCoord).w;
   float hA = (tex2D( normal_map, inTexCoord + inXY * 0.88 ).w - h0 - 0.88 ) *  1 * fShadowSoftening;
   float h9 = (tex2D( normal_map, inTexCoord + inXY * 0.77 ).w - h0 - 0.77 ) *  2 * fShadowSoftening;
   float h8 = (tex2D( normal_map, inTexCoord + inXY * 0.66 ).w - h0 - 0.66 ) *  4 * fShadowSoftening;
   float h7 = (tex2D( normal_map, inTexCoord + inXY * 0.55 ).w - h0 - 0.55 ) *  6 * fShadowSoftening;
   float h6 = (tex2D( normal_map, inTexCoord + inXY * 0.44 ).w - h0 - 0.44 ) *  8 * fShadowSoftening;
   float h5 = (tex2D( normal_map, inTexCoord + inXY * 0.33 ).w - h0 - 0.33 ) * 10 * fShadowSoftening;
   float h4 = (tex2D( normal_map, inTexCoord + inXY * 0.22 ).w - h0 - 0.22 ) * 12 * fShadowSoftening;
   
   // Compute the actual shadow strength:
   float fShadow = 1 - max(max(max(max(max(max(hA, h9), h8), h7), h6), h5), h4);
   
   return saturate(fShadow * inFade.x + inFade.y);
}

float3 CalculateParallaxOcclusion(sampler normal_map,float2 inTexCoord, float2 inParallax,float2 lightVector)
{
   
   float x;
   float y;
   float xh;
   float yh;
   float shadow=1.0f;
   
   // Compute height map profile for 8 samples:
   // Note that the height map is contained in the alpha channel
   // of the input normal map:
   if(Uniform_PixelShaderVersion==20)
   {
	   float h0 = tex2D( normal_map, inTexCoord - inParallax * 1.000 ).w;
	   float h2 = tex2D( normal_map, inTexCoord - inParallax * 0.750 ).w;
	   float h3 = tex2D( normal_map, inTexCoord - inParallax * 0.625 ).w;
	   float h4 = tex2D( normal_map, inTexCoord - inParallax * 0.500 ).w;
	   float h5 = tex2D( normal_map, inTexCoord - inParallax * 0.375 ).w;
	   float h6 = tex2D( normal_map, inTexCoord - inParallax * 0.250 ).w;
	   float h7 = tex2D( normal_map, inTexCoord - inParallax * 0.125 ).w;
	   
	   // Determine the section in the height profile that contains
	   // a sample that is higher than the view vector. This determines
	   // the correct displacement point:    
	   
	   if      (h7 > 0.875) { x = 0.937; y = 0.938; xh = h7; yh = h7; }
	   else if (h6 > 0.750) { x = 0.750; y = 0.875; xh = h6; yh = h7; }
	   else if (h5 > 0.625) { x = 0.625; y = 0.750; xh = h5; yh = h6; }
	   else if (h4 > 0.500) { x = 0.500; y = 0.625; xh = h4; yh = h5; }
	   else if (h3 > 0.375) { x = 0.375; y = 0.500; xh = h3; yh = h4; }
	   else if (h2 > 0.250) { x = 0.250; y = 0.375; xh = h2; yh = h3; }
	   else                 { x = 0.000; y = 0.125; xh = h0; yh = h2; }
	   
   }
   else
   {
	   float h0 = tex2D( normal_map, inTexCoord - inParallax * 1.000 ).w;
	   float h1 = tex2D( normal_map, inTexCoord - inParallax * 0.875 ).w;
	   float h2 = tex2D( normal_map, inTexCoord - inParallax * 0.750 ).w;
	   float h3 = tex2D( normal_map, inTexCoord - inParallax * 0.625 ).w;
	   float h4 = tex2D( normal_map, inTexCoord - inParallax * 0.500 ).w;
	   float h5 = tex2D( normal_map, inTexCoord - inParallax * 0.375 ).w;
	   float h6 = tex2D( normal_map, inTexCoord - inParallax * 0.250 ).w;
	   float h7 = tex2D( normal_map, inTexCoord - inParallax * 0.125 ).w;
	   
	   // Determine the section in the height profile that contains
	   // a sample that is higher than the view vector. This determines
	   // the correct displacement point:    
	   
	   if      (h7 > 0.875) { x = 0.937; y = 0.938; xh = h7; yh = h7; }
	   else if (h6 > 0.750) { x = 0.750; y = 0.875; xh = h6; yh = h7; }
	   else if (h5 > 0.625) { x = 0.625; y = 0.750; xh = h5; yh = h6; }
	   else if (h4 > 0.500) { x = 0.500; y = 0.625; xh = h4; yh = h5; }
	   else if (h3 > 0.375) { x = 0.375; y = 0.500; xh = h3; yh = h4; }
	   else if (h2 > 0.250) { x = 0.250; y = 0.375; xh = h2; yh = h3; }
	   else if (h1 > 0.125) { x = 0.125; y = 0.250; xh = h1; yh = h2; }
	   else                 { x = 0.000; y = 0.125; xh = h0; yh = h2; }
	   
   }
   
   // Compute the intersection between the view vector and the highest
   // sample point in the height profile:
   float fParallaxEffect = (x * (y - yh) - y * (x - xh))/((y - yh) - (x - xh));
   
   float2 parallaxDisplacedCoord = inTexCoord - float2( inParallax * (1 - fParallaxEffect ));
   
   if(Uniform_PixelShaderVersion != 20)
		shadow=ComputeParallaxOcclusionShadowing(normal_map, parallaxDisplacedCoord, lightVector, float2(1,0));


	return float3(parallaxDisplacedCoord, shadow);
}

