﻿import System
import JadEngine
import JadEngine.Core
import JadEngine.Scene
import JadEngine.Video
import JadEngine.Input
import JadEngine.Particles
import Microsoft.DirectX
import Microsoft.DirectX.Direct3D
import JadScript


// This example will teach you some advanced particle effects.
public class AdvancedParticleEffects:

	#region Fields
	// The particle effect
	private _pFX as JParticleEffect
	// The hebe model	
	private _hebeModel as JMeshObject
	
	private _state as int = (-1)
	
	#endregion
	
	#region Methods
	// Our Configuration tells the engine to call this method 
	// after the engine initialization.  We can optionally return
	// false to exit out of the application at this point.
	// We will use this as a the point in which we setup
	// our scene.
	public def InitGame() as bool:
		// Set us to be fixed at 30 FPS
		Jad.Timer.FixedMode = true
		Jad.Timer.FixedModeFPS = (1.0F / 30.0F)
		
		
		// Create the objects in the scene
		CreateObjects()
		
		// Create our particle effect
		_pFX = Jad.Scene.Particles.Create('smoke_prt', 'smoke_prt.pfx')
		_pFX.Transform.Angles = Vector3(cast(single, ((-Math.PI) / 2)), 0.0F, 0.0F)
		_pFX.Transform.Position = Vector3(0.0F, 0.05000000075F, 0.0F)
		_pFX.ParticleSystems[0].MeshObject.Material[0].Priority = JMaterialPriority.Transparency
		
		// Create an omni light
		light = cast(JOmniLight, Jad.Scene.Lights.Create('light', JLightType.Omni))
		// Set its position
		light.Transform.Position = Vector3(0, 0, -0.0F)
		// Intensity
		light.Multiplier = 1.0F
		// Attenuation
		light.Attenuation.Type = JLightAttenuationType.DualRadius
		// Distance to start attenuating the light
		light.Attenuation.Start = 0.20000000298F
		// Distance where the light is completely attenuated
		light.Attenuation.End = 0.5F
		// Color
		light.Color = JColor.Yellow
		// The light affects the diffuse component of the material
		light.AffectDiffuse = true
		// The light affects the specular component of the material
		light.AffectSpecular = true
		// Don´t generate shadows
		light.CastShadows = false
		
		// Create a spot light that will be associated with the lamp,
		// so when the lamp moves, the light will move with it
		spotLight = cast(JSpotLight, Jad.Scene.Lights.Create('spot', JLightType.Spot))
		spotLight.Hide = true
		// Set its position, in this example as the light is associated with the lamp,
		// the vector is a relative position to the lamp
		spotLight.Transform.Position = Vector3(0, 1.0F, -0.40000000596F)
		// By default, it is created looking at the Z axis. We want that light
		// goes down, so we rotate it in the X axis
		spotLight.Transform.Angles = Vector3(Geometry.DegreeToRadian(45.0F), 0, 0)
		// Intensity
		spotLight.Multiplier = 1.0F
		// Attenuation
		spotLight.Attenuation.Type = JLightAttenuationType.DualRadius
		// Distance to start attenuating the light
		spotLight.Attenuation.Start = 5.0F
		// Distance where the light is completely attenuated
		spotLight.Attenuation.End = 9.0F
		// Color
		spotLight.Color = JColor.White
		// The light affect the diffuse component of the material
		spotLight.AffectDiffuse = true
		// The light affect the specular component of the material
		spotLight.AffectSpecular = false
		// Don´t generate shadows
		spotLight.CastShadows = false
		// Cone aperture of the spot light
		spotLight.HotSpot = 25.0F
		// Maximum cone aperture
		spotLight.FallOff = 30.0F
		spotLight.Projector.Map = Jad.Video.Textures.Create2D('projector', true)
		spotLight.Projector.Enabled = true
		
		Jad.Scene.Lights.Ambient = JColor(0.0F)
		
		camera = (Jad.Scene.Camera as JCameraFirstPerson)
		camera.Transform.Position = Vector3(0.0F, 0.0F, -2.0F)
		camera.Speed = 2.0F
		
		return true

	// Creates the objects of the scene
	private def CreateObjects():
		// Load hebe
		Jad.Import.Load('hebe.haddd')
		
		// Get the first loaded object (the only one)
		model as JMeshObject = Jad.Import.MeshObjects[0]
		model.Transform.Position = Vector3(-0.25F, -0.37000000477F, 0.20000000298F)
		_hebeModel = model

	// Change the mode
	public def ChangeMode():
		if _pFX.ParticleSystems[0].MeshObject.Material[0].Priority == JMaterialPriority.Transparency:
			_pFX.ParticleSystems[0].MeshObject.Material[0].Priority = JMaterialPriority.PostProcess
		else:
			_pFX.ParticleSystems[0].MeshObject.Material[0].Priority = JMaterialPriority.Transparency

	// Toggle on or off Spherical Mapping on the particle
	public def ToggleSphericalMapping():
		_pFX.ParticleSystems[0].MeshObject.Material[0].SphericalMapping = (not _pFX.ParticleSystems[0].MeshObject.Material[0].SphericalMapping)

	// Set whether or not the particles are active
	public ParticlesActive as bool:
		set:
			if value:
				_pFX.Play()
			else:
				_pFX.Pause()

	// This method is called every frame. 
	// It´s where the input events and the game action should 
	// be placed.
	// We can optionally return a bool which will allow us to
	// exit the application by returning false.
	// Return True to continue, false to exit the application
	public def Update() as bool:
		inc as single = Jad.At1Second(1.0F)
		pos as Vector3 = _hebeModel.Transform.Position
		
		if Jad.Input.Keyboard[Key.Left].Down:
			pos.X -= inc
		if Jad.Input.Keyboard[Key.Right].Down:
			pos.X += inc
		if Jad.Input.Keyboard[Key.Up].Down:
			pos.Y += inc
		if Jad.Input.Keyboard[Key.Down].Down:
			pos.Y -= inc
		if Jad.Input.Keyboard[Key.Home].Down:
			pos.Z += inc
		if Jad.Input.Keyboard[Key.End].Down:
			pos.Z -= inc
		
		_hebeModel.Transform.Position = pos
		
		// Start recording?
		if Jad.Input.Keyboard[Key.F5].Pressed:
			enabled = (not Jad.Video.Views[0].VideoRecorder.Enabled)
			Jad.Video.Views[0].VideoRecorder.Enabled = enabled
			
			if enabled:
				Jad.Video.Views[0].VideoRecorder.Init()
		
		if Jad.Video.Views[0].VideoRecorder.Enabled:
			if _state == (-1):
				_state = 0
				_pFX.ParticleSystems[0].MeshObject.Material[0].Priority = JMaterialPriority.PostProcess
			if (_state == 0) and (Jad.Timer.Time > 5):
				_state = 1
				
				_pFX.ParticleSystems[0].MeshObject.Material[0].Priority = JMaterialPriority.Transparency
			if (_state == 1) and (Jad.Timer.Time > 10):
				_state = 2
				
				_pFX.ParticleSystems[0].MeshObject.Material[0].Priority = JMaterialPriority.PostProcess
				_pFX.ParticleSystems[0].MeshObject.Material[0].SphericalMapping = true
			if (_state == 2) and (Jad.Timer.Time > 15):
				return false
		// Everything was right, continue
		return true

	// This method gets called during the rendering.  It allows us
	// to do render post-processing, such as displaying text
	// to the screen.
	// All of the details of rendering are handled for us, so
	// we only need this if we want to do extra stuff during rendering.
	public def Render():
		showHelp = true
		keyEvents = (JadScriptOptions.GetObjectValue('keyHandler') as StandardKeyEventHandler)
		if keyEvents is not null:
			showHelp = keyEvents.ShowHelp
		
		if showHelp:
			text as string
			// Get access to the 1st font. The engine creates a font by default
			font as JFont = Jad.Video.Fonts[0]
			
			font.RenderLine((Jad.Version + '. Press ESC to exit'), 0, JColor.Yellow)
			font.RenderLine((Jad.Video.Views.CurrentView.Stats.Fps + ' FPS'), 0, JColor.White)
			font.RenderLine(('Mode (M):' + _pFX.ParticleSystems[0].MeshObject.Material[0].Priority.ToString()), 0, JColor.White)
			font.RenderLine(('Volumetric (V):' + _pFX.ParticleSystems[0].MeshObject.Material[0].SphericalMapping.ToString()), 0, JColor.White)
			if _pFX.ParticleSystems[0].MeshObject.Material[0].Priority == JMaterialPriority.Transparency:
				text = 'Particles are now affected by the light'
			else:
				text = 'Standard Particles'
			if _pFX.ParticleSystems[0].MeshObject.Material[0].SphericalMapping:
				text = 'Volumetric particles ENABLED'
			font.RenderCentered(text, (Jad.Video.Height - 25), JColor.White)
	
	#endregion

