﻿Imports System
Imports JadEngine
Imports JadEngine.Core
Imports JadEngine.Scene
Imports JadEngine.Video
Imports JadEngine.Input
Imports JadEngine.Particles
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D
Imports JadScript


''' <summary>
''' This example will teach you some advanced particle effects.
''' </summary>
Public Class AdvancedParticleEffects
	#Region "Fields"

	''' <summary>
	''' The particle effect
	''' </summary>
	Private _pFX As JParticleEffect

	''' <summary>
	''' The hebe model
	''' </summary>
	Private _hebeModel As JMeshObject

	Private _state As Integer = -1

	#End Region

	#Region "Methods"

	''' <summary>
	''' Our Configuration tells the engine to call this method 
	''' after the engine initialization.  We can optionally return
	''' false to exit out of the application at this point.
	''' We will use this as a the point in which we setup
	''' our scene.
	''' </summary>
	''' <returns>True if everything went right</returns>
	Public Function InitGame() As Boolean
		' Set us to be fixed at 30 FPS
		Jad.Timer.FixedMode = True
		Jad.Timer.FixedModeFPS = 1F / 30F


		' Create the objects in the scene
		CreateObjects()

		' Create our particle effect
        _pFX = Jad.Scene.Particles.Create("smoke_prt", "smoke_prt.pfx")
		_pFX.Transform.Angles = New Vector3(CSng((-Math.PI / 2)), 0F, 0F)
		_pFX.Transform.Position = New Vector3(0F, 0.05F, 0F)
		_pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.Transparency

		' Create an omni light
		Dim light As JOmniLight = DirectCast(Jad.Scene.Lights.Create("light", JLightType.Omni), JOmniLight)
		' Set its position
		light.Transform.Position = New Vector3(0, 0, -0F)
		' Intensity
		light.Multiplier = 1F
		' Attenuation
		light.Attenuation.Type = JLightAttenuationType.DualRadius
		' Distance to start attenuating the light
		light.Attenuation.Start = 0.2F
		' Distance where the light is completely attenuated
		light.Attenuation.[End] = 0.5F
		' Color
		light.Color = JColor.Yellow
		' The light affects the diffuse component of the material
		light.AffectDiffuse = True
		' The light affects the specular component of the material
		light.AffectSpecular = True
		' Don´t generate shadows
		light.CastShadows = False

		' Create a spot light that will be associated with the lamp,
		' so when the lamp moves, the light will move with it
		Dim spotLight As JSpotLight = DirectCast(Jad.Scene.Lights.Create("spot", JLightType.Spot), JSpotLight)
		spotLight.Hide = True
		' Set its position, in this example as the light is associated with the lamp,
		' the vector is a relative position to the lamp
		spotLight.Transform.Position = New Vector3(0, 1F, -0.4F)
		' By default, it is created looking at the Z axis. We want that light
		' goes down, so we rotate it in the X axis
		spotLight.Transform.Angles = New Vector3(Geometry.DegreeToRadian(45F), 0, 0)
		' Intensity
		spotLight.Multiplier = 1F
		' Attenuation
		spotLight.Attenuation.Type = JLightAttenuationType.DualRadius
		' Distance to start attenuating the light
		spotLight.Attenuation.Start = 5F
		' Distance where the light is completely attenuated
		spotLight.Attenuation.[End] = 9F
		' Color
		spotLight.Color = JColor.White
		' The light affect the diffuse component of the material
		spotLight.AffectDiffuse = True
		' The light affect the specular component of the material
		spotLight.AffectSpecular = False
		' Don´t generate shadows
		spotLight.CastShadows = False
		' Cone aperture of the spot light
		spotLight.HotSpot = 25F
		' Maximum cone aperture
		spotLight.FallOff = 30F
		spotLight.Projector.Map = Jad.Video.Textures.Create2D("projector", True)
		spotLight.Projector.Enabled = True

		Jad.Scene.Lights.Ambient = New JColor(0F)

		Dim camera As JCameraFirstPerson = TryCast(Jad.Scene.Camera, JCameraFirstPerson)
		camera.Transform.Position = New Vector3(0F, 0F, -2F)
		camera.Speed = 2F

		Return True
	End Function

	''' <summary>
	''' Creates the objects of the scene
	''' </summary>
	Private Sub CreateObjects()
		' Load hebe
        Jad.Import.Load("hebe.haddd")

		' Get the first loaded object (the only one)
		Dim model As JMeshObject = Jad.Import.MeshObjects(0)
		model.Transform.Position = New Vector3(-0.25F, -0.37F, 0.2F)
		_hebeModel = model
	End Sub

	''' <summary>
	''' Change the mode
	''' </summary>
	Public Sub ChangeMode()
		If _pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.Transparency Then
			_pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.PostProcess
		Else
			_pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.Transparency
		End If
	End Sub

	''' <summary>
	''' Toggle on or off Spherical Mapping on the particle
	''' </summary>
	''' <returns></returns>
	Public Sub ToggleSphericalMapping()
		_pFX.ParticleSystems(0).MeshObject.Material(0).SphericalMapping = Not _pFX.ParticleSystems(0).MeshObject.Material(0).SphericalMapping
	End Sub

	''' <summary>
	''' Set whether or not the particles are active
	''' </summary>
	Public WriteOnly Property ParticlesActive() As Boolean
		Set
			If value Then
				_pFX.Play()
			Else
				_pFX.Pause()
			End If
		End Set
	End Property

	''' <summary>
	''' This method is called every frame. 
	''' It´s where the input events and the game action should 
	''' be placed.
	''' We can optionally return a bool which will allow us to
	''' exit the application by returning false.
	''' </summary>
	''' <returns>True to continue, false to exit the application</returns>
	Public Function Update() As Boolean
		Dim inc As Single = Jad.At1Second(1F)
		Dim pos As Vector3 = _hebeModel.Transform.Position

		If Jad.Input.Keyboard(Key.Left).Down Then
			pos.X -= inc
		End If
		If Jad.Input.Keyboard(Key.Right).Down Then
			pos.X += inc
		End If
		If Jad.Input.Keyboard(Key.Up).Down Then
			pos.Y += inc
		End If
		If Jad.Input.Keyboard(Key.Down).Down Then
			pos.Y -= inc
		End If
		If Jad.Input.Keyboard(Key.Home).Down Then
			pos.Z += inc
		End If
		If Jad.Input.Keyboard(Key.[End]).Down Then
			pos.Z -= inc
		End If

		_hebeModel.Transform.Position = pos

		' Start recording?
		If Jad.Input.Keyboard(Key.F5).Pressed Then
			Jad.Video.Views(0).VideoRecorder.Enabled = Not Jad.Video.Views(0).VideoRecorder.Enabled

			If Jad.Video.Views(0).VideoRecorder.Enabled Then
				Jad.Video.Views(0).VideoRecorder.Init()
			End If
		End If

		If Jad.Video.Views(0).VideoRecorder.Enabled Then
			If _state = -1 Then
				_state = 0
				_pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.PostProcess
			End If
			If _state = 0 AndAlso Jad.Timer.Time > 5 Then
				_state = 1

				_pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.Transparency
			End If
			If _state = 1 AndAlso Jad.Timer.Time > 10 Then
				_state = 2

				_pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.PostProcess
				_pFX.ParticleSystems(0).MeshObject.Material(0).SphericalMapping = True
			End If
			If _state = 2 AndAlso Jad.Timer.Time > 15 Then
				Return False
			End If
		End If
		' Everything was right, continue
		Return True
	End Function

	''' <summary>
	''' This method gets called during the rendering.  It allows us
	''' to do render post-processing, such as displaying text
	''' to the screen.
	''' All of the details of rendering are handled for us, so
	''' we only need this if we want to do extra stuff during rendering.
	''' </summary>
	Public Sub Render()
		If JadScriptOptions.GetObjectValue("ShowHelp") Then
			Dim text As String
			' Get access to the 1st font. The engine creates a font by default
			Dim font As JFont = Jad.Video.Fonts(0)

			font.RenderLine(Jad.Version + ". Press ESC to exit", 0, JColor.Yellow)
			font.RenderLine(Jad.Video.Views.CurrentView.Stats.Fps + " FPS", 0, JColor.White)
			font.RenderLine("Mode (M):" + _pFX.ParticleSystems(0).MeshObject.Material(0).Priority.ToString(), 0, JColor.White)
			font.RenderLine("Volumetric (V):" + _pFX.ParticleSystems(0).MeshObject.Material(0).SphericalMapping.ToString(), 0, JColor.White)
			If _pFX.ParticleSystems(0).MeshObject.Material(0).Priority = JMaterialPriority.Transparency Then
				text = "Particles are now affected by the light"
			Else
				text = "Standard Particles"
			End If
			If _pFX.ParticleSystems(0).MeshObject.Material(0).SphericalMapping Then
				text = "Volumetric particles ENABLED"
			End If
			font.RenderCentered(text, Jad.Video.Height - 25, JColor.White)
		End If
	End Sub

	#End Region
End Class
