﻿#Region "Using directives"

Imports System
Imports Microsoft.DirectX

Imports JadScript
Imports JadEngine
Imports JadEngine.Video
Imports JadEngine.Core
Imports JadEngine.Mathematics
Imports JadEngine.Scene
Imports JadEngine.Input

#End Region

''' <summary>
''' This example will teach you to use the emissive mapping material 
''' to make a glow effect and the self-ilum possibilites of the engine.
''' You can see the different fog options of the engine, and 
''' change a lot of parameters.
''' </summary>
Public Class GlowAndFog
	#Region "Fields"
	''' <summary>
	''' The emissive map layer
	''' </summary>
	Private _layer As JMaterialLayer

	''' <summary>
	''' TODO
	''' </summary>
	Private _hebeModel As JMeshObject

	''' <summary>
	''' Indicates if the glow is animated or not
	''' </summary>
	Private _glowAnimated As Boolean

	#End Region

	#Region "Methods"

	''' <summary>
	''' The engine calls this method after creating the device
	''' </summary>
	''' <param name="form">Form used as window</param>
	''' <returns>True if everything went right</returns>
	Public Function InitGame() As Boolean
		CreateRoom()

		CreateGlowWall()

		CreateObjects()

		Jad.Scene.Fog.Type = JFogType.Linear

		' Set the bottom value for the layered fog
		Jad.Scene.Fog.Bottom = -1.5F
		' Set the top value for the layered fog
		Jad.Scene.Fog.Top = -0.8F


		' Create an omni light
		Dim light As JOmniLight = DirectCast(Jad.Scene.Lights.Create("light", JLightType.Omni), JOmniLight)

		' Set its position
		light.Transform.Position = New Vector3(0, 0, -1F)

		' Intensity
		light.Multiplier = 1F

		' Attenuation
		light.Attenuation.Type = JLightAttenuationType.DualRadius

		' Distance to start attenuating the light
		light.Attenuation.Start = 1F

		' Distance where the light is completely attenuated
		light.Attenuation.[End] = 2F

		' Color
		light.Color = JColor.Yellow

		' The light affects the diffuse component of the material
		light.AffectDiffuse = True

		' The light affects the specular component of the material
		light.AffectSpecular = True

		' Don´t generate shadows
		light.CastShadows = False

		' Put ambient light
		Jad.Scene.Lights.Ambient = New JColor(0.4F, 0.4F, 0.4F)

		Return True
	End Function

	''' <summary>
	''' Creates the room of the scene
	''' </summary>
	Private Sub CreateRoom()
		Dim cubeMesh As JMesh = JMesh.CreateCube(4, 3, 3)

		cubeMesh.IndexBuffer.InvertFaces()

		Dim cube As JMeshObject = Jad.Scene.MeshObjects.Create("room", False)

		cube.Mesh = cubeMesh

		' Material

		Dim material As JMaterial() = New JMaterial(0) {}

		material(0) = Jad.Scene.Materials.Create()

		' Create the layer
		Dim layer As New JMaterialLayer()

		' Add it to the material
		material(0).AddLayer(layer)

		layer.DiffuseMap.Texture = Jad.Video.Textures.Create2D("brick_d", True)

		' Set the material we have just created to our JMeshObject
		cube.Material = material
	End Sub

	''' <summary>
	''' Creates the glowing wall
	''' </summary>
	Public Sub CreateGlowWall()
		Dim cubeMesh As JMesh = JMesh.CreateRectangle(2, 2, 2, 2, JCreateRectangle.XY, False, _
			1F, 1F)

		Dim cube As JMeshObject = Jad.Scene.MeshObjects.Create("wall", False)

		cube.Mesh = cubeMesh

		cube.Transform.Position = New Vector3(0, 0, 1.49F)

		' Material
		Dim material As JMaterial = Jad.Scene.Materials.Create()

		' Create the layer
		_layer = New JMaterialLayer()

		' Add it to the material
		material.AddLayer(_layer)

		_layer.DiffuseMap.Texture = Jad.Video.Textures.Create2D("projector", True)

		' Enable de Emissive map
		_layer.EmissiveMap.Enabled = True

		' Set a color, when Mode=JEmissiveMode.Color
		_layer.EmissiveMap.Color = JColor.Yellow

		' Set the intensity of the emissivemap.
		_layer.EmissiveMap.Intensity = 0.1F

		' Set the texture for the Mode=JEmissiveMode.Texture
		_layer.EmissiveMap.Texture = Jad.Video.Textures.Create2D("projector", True)

		' Set the mode. The diffuse mode will use the DiffuseMap.Texture
		_layer.EmissiveMap.Mode = JEmissiveMode.Diffuse

		' Set the material we have just created to our JMeshObject
		cube.Material = New JMaterial() {material}
	End Sub

	''' <summary>
	''' Creates the object of the scene
	''' </summary>
	Public Sub CreateObjects()
		' Load hebe

        Jad.Import.Load("hebe.haddd")

		' We get the first loaded object ( the only one )

		Dim model As JMeshObject = Jad.Import.MeshObjects(0)

		model.Transform.Position = New Vector3(-1.5F, -1.5F + model.Mesh.BoundingBox.Height / 2F, 0)

		' The self-ilum objects add this color to the ambient component
		model.Material(0).Layers(0).EmissiveMap.Color = New JColor(0.3F)

		' We tell the engine to render this mesh without lights, only ambient+EmissiveMap.Color
		model.AffectLight = False

		_hebeModel = model
	End Sub

	''' <summary>
	''' Toggle whether the glow is animated
	''' </summary>
	Public Sub ToggleGlowAnimated()
		_glowAnimated = Not _glowAnimated
	End Sub

	''' <summary>
	''' Toggle whether the model is affected by light
	''' </summary>
	Public Sub ToggleAffectLight()
		_hebeModel.AffectLight = Not _hebeModel.AffectLight
	End Sub

	''' <summary>
	''' Switch between fog modes
	''' </summary>
	Public Sub ChangeFogMode()
		' There are 4 fog modes, so we are going to alternate them
		Dim m As Integer = CInt(Jad.Scene.Fog.Type)
		m += 1
		If m > 4 Then
			m = 0
		End If
		Jad.Scene.Fog.Type = DirectCast(m, JFogType)
	End Sub

	''' <summary>
	''' Switch between emmisive modes
	''' </summary>
	Public Sub ChangeEmmisiveMode()
		' There are 3 emissive modes, so we are going to alternate them
		Dim m As Integer = CInt(_layer.EmissiveMap.Mode)
		m += 1
		If m > 3 Then
			m = 1
		End If
		_layer.EmissiveMap.Mode = DirectCast(m, JEmissiveMode)
	End Sub

	''' <summary>
	''' This method is called every frame and it´s where the input
	''' events and the game action must be placed
	''' </summary>
	''' <returns>True to continue, false to exit the application</returns>
	Public Function Update() As Boolean
		If Jad.Input.Keyboard(Key.NumPad9).Down Then
			Jad.Scene.Fog.Top += 0.1F
		End If
		If Jad.Input.Keyboard(Key.NumPad3).Down Then
			Jad.Scene.Fog.Top -= 0.1F
		End If

		If Jad.Input.Keyboard(Key.NumPadPlus).Down Then
			Jad.Scene.Fog.Near += 0.1F
		End If
		If Jad.Input.Keyboard(Key.NumPadMinus).Down Then
			Jad.Scene.Fog.Near -= 0.1F
		End If

		If Jad.Input.Keyboard(Key.NumPad8).Down Then
			Jad.Scene.Fog.Density += 0.1F
		End If
		If Jad.Input.Keyboard(Key.NumPad2).Down Then
			Jad.Scene.Fog.Density -= 0.1F
		End If

		' Modify the parameters of the postproduction effect asociated with the glow
		If _glowAnimated Then
			Jad.Scene.GlowEffect.HorizontalBlur.GlowAttenuation = 1F + CSng(Math.Abs(Math.Sin(Jad.Timer.Time)))
		Else
			Jad.Scene.GlowEffect.HorizontalBlur.GlowAttenuation = 1.2F

		End If

		Jad.Scene.GlowEffect.VerticalBlur.GlowAttenuation = Jad.Scene.GlowEffect.HorizontalBlur.GlowAttenuation

		' Everything was right, continue
		Return True
	End Function

	''' <summary>
	''' Render method
	''' </summary>
	Public Sub Render()
		If JadScriptOptions.GetObjectValue("ShowHelp") Then
			' Get access to the 1st font. The engine creates a font by default
			Dim font As JFont = Jad.Video.Fonts(0)

			font.RenderLine(Jad.Version + ". Press ESC to exit. Press G to Show/Hide Gizmos. F1 show/hide text", 0, JColor.Yellow)
			font.RenderLine(Jad.Video.Views.CurrentView.Stats.Fps + " FPS", 0, JColor.White)
			font.RenderLine("Wall Emissive mode(E):" + _layer.EmissiveMap.Mode.ToString(), 0, JColor.White)
			font.RenderLine("Wall Glow Animated(T):" + _glowAnimated.ToString(), 0, JColor.White)
			font.RenderLine("Model AffectLight(I):" + _hebeModel.AffectLight.ToString(), 0, JColor.White)

			Dim fogText As String = "Fog mode(F):" + Jad.Scene.Fog.Type.ToString()
			If Jad.Scene.Fog.Type <> JFogType.None Then
				fogText += " Near (+,-):" + Jad.Scene.Fog.Near.ToString("N2")

				If Jad.Scene.Fog.Type = JFogType.Layered Then
					fogText += " Top (9,3):" + Jad.Scene.Fog.Top.ToString("N2")
				End If
				If Jad.Scene.Fog.Type <> JFogType.Linear Then
					fogText += " Density (8,2):" + Jad.Scene.Fog.Density.ToString("N2")
				End If
			End If

			font.RenderLine(fogText, 0, JColor.White)
		End If
	End Sub

	''' <summary>
	''' The Reinitialize method is called to re-initialize us after we reload
	''' our script.  Without this, reloading would cause an exception
	''' the first time we tried to access our private data, since it would
	''' be null.
	''' Returning false will cause the engine to shutdown.
	''' </summary>
	Public Function Reinitialize() As Boolean
		_layer = Jad.Scene.MeshObjects("wall").Material(0).Layers(0)
		_hebeModel = Jad.Import.MeshObjects(0)
		If (_layer Is Nothing) OrElse (_hebeModel Is Nothing) Then
			Return False
		End If

		Return True
	End Function

	#End Region
End Class
