#Region "Using Directives"

Imports System
Imports JadEngine
Imports JadEngine.Core
Imports JadEngine.Scene
Imports JadEngine.Video
Imports JadEngine.Input
Imports JadScript

#End Region

''' <summary>
''' Class to handle our standard key events from nearly every application.
''' </summary>
Public Class StandardKeyEventHandler
	#Region "Properties"
	''' <summary>
	''' True to display help on the screen
	''' </summary>
	Public Property ShowHelp() As Boolean
		Get
			Return JadScriptOptions.GetObject(Of Boolean)("ShowHelp")
		End Get
		Set
			JadScriptOptions.SetObject(Of Boolean)("ShowHelp", value)
		End Set
	End Property
	#End Region

	#Region "Constructor"
	Public Sub New()
		ShowHelp = True
	End Sub
	#End Region

	#Region "Methods"

	''' <summary>
	''' Shutdown the application.  
	''' This is triggered by a KeyEvent:
	''' Pressing the Escape key
	''' </summary>
	Public Sub Shutdown()
		JadScriptOptions.Shutdown = True
	End Sub

	''' <summary>
	''' Reload all the scripts for this application.
	''' This is triggered by a KeyEvent:
	''' Pressing the R key
	''' </summary>
	Public Sub ReloadScript()
		JadScriptOptions.Reinitialize = True
	End Sub

	''' <summary>
	''' Toggle on and off all the gizmo display in the engine.
	''' This is triggered by a KeyEvent:
	''' Pressing the G key
	''' </summary>
	Public Sub ToggleGizmos()
		Jad.Scene.ShowGizmos = Not Jad.Scene.ShowGizmos
	End Sub

	''' <summary>
	''' Toggle on and off all the help display.
	''' This is triggered by a KeyEvent:
	''' Pressing the F1 key
	''' </summary>
	Public Sub ToggleHelp()
		ShowHelp = Not ShowHelp
	End Sub

	#End Region
End Class
