#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using JadEngine;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Animation;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace Collada2Jad
{
    class Converter
    {
        #region Fields

        /// <summary>
        /// Contains data loaded from collada file
        /// </summary>
        private COLLADA collada;

        /// <summary>
        /// Images to be copied into the texture directory when saving
        /// </summary>
        private List<string> images;
        
        /// <summary>
        /// Path to the collada file
        /// </summary>
        private string filePath;

		private List<JAnimationChannel> channels;

        #endregion

        #region Constructors

        public Converter()
        {
            collada = null;
            images = new List<string>();
            channels = new List<JAnimationChannel>();
        }

        #endregion

        #region Properties

        #endregion
        
        #region Methods

        public void Reset()
        {
        	if(collada != null)
        	{
        		collada = null;
        	}
        	images.Clear();
        }
        
        /// <summary>
        /// Loads geometry from collada before exporting
        /// </summary>
        private void LoadGeometry()
        {
            library_geometries lib_geom = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_geometries)
                {
                    lib_geom = (library_geometries)collada.Items[i];
                    break;
                }
            }
            if(lib_geom != null)
            {
	            for (long geom_c = 0; geom_c < lib_geom.geometry.GetLongLength(0); geom_c++)
	            {
	                //get mesh
	                geometry geom = lib_geom.geometry[geom_c];
	                JMesh jm = new JMesh(geom.id);
	                mesh m = (mesh)geom.Item;
	
	                //switches for checking if such data is present in the collada file
	                bool vertex_is = false;
	                bool normal_is = false;
	                bool tangent_is = false;
	                bool color_is = false;
	                bool texture_is = false;
	
	                //offsets and sets
	                ulong vertex_offset = 0;
	                ulong normal_offset = 0;
	                ulong tangent_offset = 0;
	                ulong color_offset = 0;
	                ulong texture_offset = 0;
	
	                float[] vertex_array = null;
	                float[] normal_array = null;
	                float[] tangent_array = null;
	                float[] color_array = null;
	                float[] texture_array = null;
	
	                for(long item = 0; item < m.Items.GetLongLength(0); item ++)
	                {
	                    //if there are triangle items
	                    if(m.Items[item] is triangles)
	                    {
	                        triangles tr = (triangles)m.Items[item];
	                        for (long input_c = 0; input_c < tr.input.GetLongLength(0); input_c++)
	                        {
	                            InputLocalOffset ilo = tr.input[input_c];
	                            switch (ColladaUtils.GetSemantic(ilo.semantic))
	                            {
	                                case 21: //vertex
	                                    {
	                                        vertex_is = true;
	                                        vertex_offset = ilo.offset;
	                                       
	                                        if (m.vertices.id.CompareTo(ilo.source.Remove(0, 1)) == 0)
	                                        {
	                                            for (long k = 0; k < m.vertices.input.GetLongLength(0); k++)
	                                            {
	                                                switch (ColladaUtils.GetSemantic(m.vertices.input[k].semantic))
	                                                {
	                                                    case 15: //position
	                                                        {
	                                                            string src = m.vertices.input[k].source.Remove(0, 1);
	                                                            for (long j = 0; j < m.source.GetLongLength(0); j++)
	                                                            {
	                                                                if (m.source[j].id.CompareTo(src) == 0)
	                                                                {
	                                                                    float_array array = (float_array)m.source[j].Item;
	                                                                    vertex_array = ColladaUtils.StringToFloat(array.Text[0]);
	                                                                    break;
	                                                                }
	                                                            }
	                                                            break;
	                                                        }
                                                        
                                                        //NOTE: code for the other vertex data repeats here because it can be in both places (collada stuff) there is no need for separate methods yet
                                                        case 12: //normal
                                                            {
                                                                normal_is = true;
                                                                normal_offset = ilo.offset;
                                                                string src = m.vertices.input[k].source.Remove(0, 1);
                                                                for (long j = 0; j < m.source.GetLongLength(0); j++)
                                                                {
                                                                    if (m.source[j].id.CompareTo(src) == 0)
                                                                    {
                                                                        float_array array = (float_array)m.source[j].Item;
                                                                        normal_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                        break;
                                                                    }
                                                                }
                                                                break;
                                                            }
                                                        case 16: //tangent
                                                            {
                                                                tangent_is = true;
                                                                tangent_offset = ilo.offset;
                                                                string src = m.vertices.input[k].source.Remove(0, 1);
                                                                for (long j = 0; j < m.source.GetLongLength(0); j++)
                                                                {
                                                                    if (m.source[j].id.CompareTo(src) == 0)
                                                                    {
                                                                        float_array array = (float_array)m.source[j].Item;
                                                                        tangent_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                        break;
                                                                    }
                                                                }
                                                                break;
                                                            }
                                                        case 1: //color (diffuse)
                                                            {
                                                                color_is = true;
                                                                color_offset = ilo.offset;
                                                                string src = m.vertices.input[k].source.Remove(0, 1);
                                                                for (long j = 0; j < m.source.GetLongLength(0); j++)
                                                                {
                                                                    if (m.source[j].id.CompareTo(src) == 0)
                                                                    {
                                                                        float_array array = (float_array)m.source[j].Item;
                                                                        color_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                        break;
                                                                    }
                                                                }
                                                                break;
                                                            }
                                                        case 18: //tex coord
                                                            {
                                                                texture_is = true;
                                                                texture_offset = ilo.offset;
                                                                string src = m.vertices.input[k].source.Remove(0, 1);
                                                                for (long j = 0; j < m.source.GetLongLength(0); j++)
                                                                {
                                                                    if (m.source[j].id.CompareTo(src) == 0)
                                                                    {
                                                                        float_array array = (float_array)m.source[j].Item;
                                                                        texture_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                        break;
                                                                    }
                                                                }
                                                                break;
                                                            }
	                                                }
	                                            }
	                                            break;
	                                        }
	                                        break;
	                                    }
	                                case 12: //normal
	                                    {
	                                        normal_is = true;
	                                        normal_offset = ilo.offset;
	                                        string src = ilo.source.Remove(0, 1);
	                                        for (long j = 0; j < m.source.GetLongLength(0); j++)
	                                        {
	                                            if (m.source[j].id.CompareTo(src) == 0)
	                                            {
	                                                float_array array = (float_array)m.source[j].Item;
	                                                normal_array = ColladaUtils.StringToFloat(array.Text[0]);
	                                                break;
	                                            }
	                                        }
	                                        break;
	                                    }
	                                case 16: //tangent
	                                    {
	                                        tangent_is = true;
	                                        tangent_offset = ilo.offset;
	                                        string src = ilo.source.Remove(0, 1);
	                                        for (long j = 0; j < m.source.GetLongLength(0); j++)
	                                        {
	                                            if (m.source[j].id.CompareTo(src) == 0)
	                                            {
	                                                float_array array = (float_array)m.source[j].Item;
	                                                tangent_array = ColladaUtils.StringToFloat(array.Text[0]);
	                                                break;
	                                            }
	                                        }
	                                        break;
	                                    }
	                                case 1: //color (diffuse)
	                                    {
	                                        color_is = true;
	                                        color_offset = ilo.offset;
	                                        string src = ilo.source.Remove(0, 1);
	                                        for (long j = 0; j < m.source.GetLongLength(0); j++)
	                                        {
	                                            if (m.source[j].id.CompareTo(src) == 0)
	                                            {
	                                                float_array array = (float_array)m.source[j].Item;
	                                                color_array = ColladaUtils.StringToFloat(array.Text[0]);
	                                                break;
	                                            }
	                                        }
	                                        break;
	                                    }
	                                case 18: //tex coord
	                                    {
	                            			texture_is = true;
	                                        texture_offset = ilo.offset;
	                                        string src = ilo.source.Remove(0, 1);
	                                        for (long j = 0; j < m.source.GetLongLength(0); j++)
	                                        {
	                                            if (m.source[j].id.CompareTo(src) == 0)
	                                            {
                                                    float_array array = (float_array)m.source[j].Item;
                                                    texture_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                    break;
	                                            }
	                                        }
	                                        break;
	                                    }
	
	                            }
	                        }//end for
	                        //allocating memory
	                        int[] ind = ColladaUtils.StringToInt(tr.p);
	
	                        jm.VertexBuffer = new JVertexBuffer(true);
	                        jm.IndexBuffer = new JIndexBuffer();
	
	                        //indices
	
	                        jm.IndexBuffer.Indices = new int[tr.count * 3];
	                        
	                        //preprocessing
	                        
	                        List<long> index = new List<long>();
	                        List<Vector3> position = new List<Vector3>();
	                        List<Vector3> normal = new List<Vector3>();
				            List<Vector3> tangent = new List<Vector3>();
	                        List<Vector3> color = new List<Vector3>();
	                        List<Vector2> texture = new List<Vector2>();
	                        
	                        try
	                        {
		                        for(ulong i = 0; i < tr.count * 3; i ++)
		                        {
		                        	index.Add(-1);
		                        }
	                        }
	                        catch(Exception e)
	                        {
	                        	Console.WriteLine(e.ToString());
	                        }
	                        //computing step
	                        
	                        ulong step = 0; //used for parsing collada index, will be computed using the greatest offset
	                        { //logical block
	                            if (vertex_offset > step) step = vertex_offset;
	                            if (normal_offset > step) step = normal_offset;
	                            if (tangent_offset > step) step = tangent_offset;
	                            if (color_offset > step) step = color_offset;
                                if (texture_offset > step) step = texture_offset;
	                            step++; //because offset is zero based
	                        }
	                        
	                        //process the index
	                        long count = -1;
	                        List<int> offsets = new List<int>();
	                        for(ulong i = 0; i < tr.count * 3; i++)
	                        {
								//if unprocessed
								if(index[Convert.ToInt32(i)] == -1)
	                        	{
	                        		offsets.Clear();
	                        		//copy element indices
	                        		for(ulong j = 0; j < step; j ++)
	                        		{
	                        			offsets.Add(ind[i * step  + j]);
	                        		}
	                        		
	                        		//copy data into the right place
	                        		
	                        		//position
	                        		if(vertex_is)
	                        		{
	                        			Vector3 pos = new Vector3(
	                        				vertex_array[offsets[(int)vertex_offset] * 3],
	                        				vertex_array[offsets[(int)vertex_offset] * 3 + 1],
	                        				vertex_array[offsets[(int)vertex_offset] * 3 + 2]);
	                        			position.Add(pos);
	                        		}
	                        		
	                        		//normal
	                        		if(normal_is)
	                        		{
	                        			Vector3 norm = new Vector3(
	                        				normal_array[offsets[(int)normal_offset] * 3],
	                        				normal_array[offsets[(int)normal_offset] * 3 + 1],
	                        				normal_array[offsets[(int)normal_offset] * 3 + 2]);
	                        			normal.Add(norm);
	                        		}
	                        		
	                        		//tangent
	                        		if(tangent_is)
	                        		{
	                        			Vector3 tang = new Vector3(
	                        				tangent_array[offsets[(int)tangent_offset] * 3],
	                        				tangent_array[offsets[(int)tangent_offset] * 3 + 1],
	                        				tangent_array[offsets[(int)tangent_offset] * 3 + 2]);
	                        			tangent.Add(tang);
	                        		}
	                        		
	                        		//color
	                        		if(color_is)
	                        		{
	                        			Vector3 col = new Vector3(
	                        				color_array[offsets[(int)color_offset] * 3],
	                        				color_array[offsets[(int)color_offset] * 3 + 1],
	                        				color_array[offsets[(int)color_offset] * 3 + 2]);
	                        			color.Add(col);
	                        		}
	                        		
	                        		//texture
	                        		if(texture_is)
	                        		{
	                        			Vector2 tex = new Vector2(
	                        				texture_array[offsets[(int)texture_offset] * 2],
	                        				1.0f - texture_array[offsets[(int)texture_offset] * 2 + 1]);
	                        			texture.Add(tex);
	                        		}
	                        		
	                        		//fill index
	                        		count ++;
	                        		index[(int)i] = count;
	                        		
	                        		//check for other elements that have the same indices (they are identical with this one)
	                        		for(ulong j = i + 1; j < tr.count * 3; j++)
	                        		{
	                        			bool match = true;
	                        			for(ulong k = 0; k < step; k ++)
	                        			{
	                        				if(offsets[(int)k] != ind[j * step + k])
	                        				{
	                        					match = false;
	                        					break;
	                        				}
	                        			}
	                        			
	                        			//if found, the same index is attached
	                        			if(match && index[(int)j] == -1)
	                        			{
	                        				index[(int)j] = count;
	                        			}
	                        		}
	                        	}
	                        }
	                        
	                        //increment count to reflect the actual number of elements we have now
	                        count++;
	                        
	                        //vertices are included in the conditions.txt, they must be there
	                        if (vertex_is)
	                        {
	                        	jm.VertexBuffer.Position = new Vector3[count];
	                        }
	
	                        if (normal_is)
	                        {
	                            jm.VertexBuffer.Normal = new Vector3[count];
	                        }
	     
	                        if (tangent_is)
	                        {
	                            jm.VertexBuffer.Tangent = new Vector3[count];
	                        }
	                        if (color_is)
	                        {
	                            jm.VertexBuffer.Diffuse = new int[count];
	                        }
	                        if (texture_is)
	                        {
	                        	jm.VertexBuffer.TexCoord = new Vector2[1,count];
	                        }
	
	                        //filling data
	                        
	                        //vertex data
	                        for(ulong i = 0; i < (ulong)count; i ++)
	                        {
	                        	if(vertex_is) jm.VertexBuffer.Position[i] = position[(int)i];
	                        	if(normal_is) jm.VertexBuffer.Normal[i] = normal[(int)i];
	                        	if(tangent_is)jm.VertexBuffer.Tangent[i] = tangent[(int)i];
	                        	if(color_is) jm.VertexBuffer.Diffuse[i] = new JColor(color[(int)i].X, color[(int)i].Y, color[(int)i].Z).ToArgb();
	                        	if(texture_is) jm.VertexBuffer.TexCoord[0, i] = texture[(int)i];
	                        }
	                        
	                        //index data
	                        for(ulong i = 0; i < tr.count * 3; i++)
	                        {
	                        	jm.IndexBuffer.Indices[i] = (int)index[(int)i];
	                        }
	
	                        jm.NormalsComputed = true;
	                        jm.VertexBuffer.NumberOfVertices = (int)count;
	                        jm.IndexBuffer.NumberOfIndices = (int)(tr.count * 3);
	                        
	                        Jad.Video.Meshes.Add(jm);
	                        
	                        JMeshObject jmo = new JMeshObject(jm.Name, false, Jad.Scene.MeshObjects);
	                        jmo.Mesh = Jad.Video.Meshes[jm.Name];
	                        Jad.Scene.MeshObjects.Add(jmo);
	                    }
	                }
	            }
            }
        }


        /// <summary>
        /// Loads lights from collada before exporting
        /// </summary>
        private void LoadLights()
        {
            library_lights lib_light = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_lights)
                {
                    lib_light = (library_lights)collada.Items[i];
                    break;
                }
            }
            if(lib_light != null)
            {
	            for (long i = 0; i < lib_light.light.GetLongLength(0); i++)
	            {
	                //get light
	                light l = lib_light.light[i];
	
	                //ambient light
	                if(l.technique_common.Item is lightTechnique_commonAmbient)
	                {
	                    //TODO - add code when Jad will support ambient light type separately
	                }
	
	                //direct light
	                else if(l.technique_common.Item is lightTechnique_commonDirectional)
	                {
	                	JDirectLight jl = (JDirectLight) Jad.Scene.Lights.Create(l.id, JLightType.Directional);
	                    lightTechnique_commonDirectional directional = (lightTechnique_commonDirectional)l.technique_common.Item;
	                    
	                    //color
	                    float[] color_array = ColladaUtils.StringToFloat(directional.color.Text[0]);
	                    JColor c = new JColor(color_array[0], color_array[1], color_array[2], 1.0f);
	                    jl.Color = c;
	
	                    jl.Attenuation.Type = JLightAttenuationType.None;
	                }
	
	                //spot light
	                else if(l.technique_common.Item is lightTechnique_commonSpot)
	                {
	                	JSpotLight jl = (JSpotLight) Jad.Scene.Lights.Create(l.id, JLightType.Spot);
	                    lightTechnique_commonSpot spot = (lightTechnique_commonSpot)l.technique_common.Item;
	                    
	                    //color
	                    float[] color_array = ColladaUtils.StringToFloat(spot.color.Text[0]);
	                    JColor c = new JColor(color_array[0], color_array[1], color_array[2], 1.0f);
	                    jl.Color = c;
	
	                    //attenuation
	                    jl.Attenuation.Constant = Convert.ToSingle(spot.constant_attenuation.Value);
	                    jl.Attenuation.Linear = Convert.ToSingle(spot.linear_attenuation.Value);
	                    jl.Attenuation.Quadratic = Convert.ToSingle(spot.quadratic_attenuation.Value);
	
	                    jl.Attenuation.Type = JLightAttenuationType.Standard;
	
	                    //falloff
	                    float falloff_angle = Convert.ToSingle(spot.falloff_angle.Value);
	                    float falloff_exponent = Convert.ToSingle(spot.falloff_exponent.Value);
	
	                    jl.FallOffExponent = falloff_exponent;
	                    jl.FallOff = falloff_angle;
	
	                }
	
	                //omni light
	                else if(l.technique_common.Item is lightTechnique_commonPoint)
	                {
	                	JOmniLight jl = (JOmniLight) Jad.Scene.Lights.Create(l.id, JLightType.Omni);
	                    lightTechnique_commonPoint point = (lightTechnique_commonPoint)l.technique_common.Item;
	
	                    //color
	                    float[] color_array = ColladaUtils.StringToFloat(point.color.Text[0]);
	                    JColor c = new JColor(color_array[0], color_array[1], color_array[2], 1.0f);
	                    jl.Color = c;
	
	                    //attenuation
	                    jl.Attenuation.Constant = Convert.ToSingle(point.constant_attenuation.Value);
	                    jl.Attenuation.Linear = Convert.ToSingle(point.linear_attenuation.Value);
	                    jl.Attenuation.Quadratic = Convert.ToSingle(point.quadratic_attenuation.Value);
	
	                    jl.Attenuation.Type = JLightAttenuationType.Standard;
	                }
	            }
            }
        }

        /// <summary>
        /// Load all cameras in the scene
        /// </summary>
        private void LoadCameras()
        {
            library_cameras lib_cam = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_cameras)
                {
                    lib_cam = (library_cameras)collada.Items[i];
                    break;
                }
            }
            if(lib_cam != null)
            {
	            for (int i = 0; i < lib_cam.camera.Length; i++)
	            {
	                camera cam = lib_cam.camera[i];
	                
	                float znear = 0.0f;
	                float zfar = 0.0f;
	                float aspect_ratio = 0.0f;
	                float xmag_fov = 0.0f;
	                float ymag_fov = 0.0f;
	                bool use_aspect_ratio = false;
	                bool use_x_mag_fov = false;
	                bool use_y_mag_fov = false;
	
	                if (cam.optics.technique_common.Item is cameraOpticsTechnique_commonPerspective)
	                {
	                    //perspective
	                    cameraOpticsTechnique_commonPerspective perspective = (cameraOpticsTechnique_commonPerspective)cam.optics.technique_common.Item;
	                    for (int j = 0; j < perspective.ItemsElementName.Length; j++)
	                    {
	                        switch (perspective.ItemsElementName[j])
	                        {
	                            case ItemsChoiceType1.aspect_ratio:
	                                {
	                                    use_aspect_ratio = true;
	                                    aspect_ratio = Convert.ToSingle(perspective.Items[j].Value);
	                                    break;
	                                }
	                            case ItemsChoiceType1.xfov:
	                                {
	                                    use_x_mag_fov = true;
	                                    xmag_fov = Convert.ToSingle(perspective.Items[j].Value);
	                                    break;
	                                }
	                            case ItemsChoiceType1.yfov:
	                                {
	                                    use_y_mag_fov = true;
	                                    ymag_fov = Convert.ToSingle(perspective.Items[j].Value);
	                                    break;
	                                }
	                        }
	                    }
	
                        znear = Convert.ToSingle(perspective.znear.Value);
                        zfar = Convert.ToSingle(perspective.zfar.Value);

                        xmag_fov = (xmag_fov * Convert.ToSingle(Math.PI)) / 180.0f;
                        ymag_fov = (ymag_fov * Convert.ToSingle(Math.PI)) / 180.0f;

                        JCamera c = new JCamera(cam.id, xmag_fov, ymag_fov, znear, zfar, Jad.Scene.Cameras);
                        Jad.Scene.Cameras.Add(c);
                        
                        if (use_x_mag_fov && use_y_mag_fov)
                        {
                            c.AspectRatio = ymag_fov / xmag_fov;
                        }
                        else if (use_x_mag_fov && use_aspect_ratio && !use_y_mag_fov)
                        {
                            c.YFov = aspect_ratio * xmag_fov;
                            c.AspectRatio = aspect_ratio;
                        }
                        else if (use_y_mag_fov && use_aspect_ratio && !use_x_mag_fov)
                        {
                            c.XFov = ymag_fov / aspect_ratio;
                            c.AspectRatio = aspect_ratio;
                        }

	                }
	                else if (cam.optics.technique_common.Item is cameraOpticsTechnique_commonOrthographic)
	                {
	                    //orthographic
	                    //TODO - use this data to create an orthographic projection camera when Jad will support it
	                }
	            }
            }
        }

        /// <summary>
        /// Loads textures from collada
        /// </summary>
        private void LoadTextures()
        {
            library_images lib_img = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_images)
                {
                    lib_img = (library_images)collada.Items[i];
                    break;
                }
            }
            if(lib_img != null)
            {
	            for (int i = 0; i < lib_img.image.Length; i++)
	            {
	                image img = lib_img.image[i];
	                if (img.Item is string)
	                {
	                	string file = (string)img.Item;
	                	int pos = file.LastIndexOf('\\');
	                	if(pos != -1)
	                	{
	                		file = file.Substring(pos + 1, file.Length - pos - 1);
	                	}
	                	
	                	string source = filePath + '\\' + file;
	                	string target = filePath + '\\' + img.id + Path.GetExtension(source);
	                	
	                	try
	                	{
	                		System.IO.File.Move(source, target);
	                	}
	                	catch(Exception e)
	                	{
	                		Console.WriteLine(e.ToString());
	                	}
	                	
	                	images.Add(img.id + Path.GetExtension(source));
	                	
	                    Jad.Video.Textures.Create2D(img.id, true, filePath);
	                }
	            }
            }
        }

        /// <summary>
        /// Load materials from collada
        /// </summary>
        private void LoadMaterials()
        {
            library_materials lib_mat = null;
            library_effects lib_eff = null;
            
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_materials)
                {
                    lib_mat = (library_materials)collada.Items[i];
                    break;
                }
            }

            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_effects)
                {
                    lib_eff = (library_effects)collada.Items[i];
                    break;
                }
            }
            
            if(lib_mat != null && lib_eff != null)
            {
	            for (int i = 0; i < lib_mat.material.Length; i++)
	            {
	                material mat = lib_mat.material[i];
	                JMaterial jm = Jad.Scene.Materials.Create(mat.id);
	                JMaterialLayer layer = new JMaterialLayer();
	                string url = mat.instance_effect.url.Remove(0, 1);
	                for (int j = 0; j < lib_eff.effect.Length; j++)
	                {
	                    //find the effect in effects library
	                    if (lib_eff.effect[j].id.CompareTo(url) == 0)
	                    {
	                        effect eff = lib_eff.effect[j];
	                        switch (ColladaUtils.GetCommonEffectType(eff))
	                        {
	                            case 0:
	                                LoadConstantMaterial(eff, layer);
	                                break;
	                            case 1:
	                                LoadLambertMaterial(eff, layer);
	                                break;
	                            case 2:
	                                LoadBlinnMaterial(eff, layer);
	                                break;
	                            case 3:
	                                LoadPhongMaterial(eff, layer);
	                                break;
	                        }
	                        jm.AddLayer(layer);
	                    }
	                }
	            }
            }
        }

        /// <summary>
        /// Loads a diffuse map from collada to jad
        /// </summary>
        /// <param name="map"></param>
        /// <param name="tex"></param>
        private JDiffuseMap LoadDiffuseMap(common_color_or_texture_typeTexture tex, effectFx_profile_abstractProfile_COMMON profile)
        {
        	JDiffuseMap map = new JDiffuseMap();
            fx_sampler2D_common smp = ColladaUtils.GetSampler2D(tex, profile);
            map.Sampler.States.MagFilter = ColladaUtils.GetTextureFilter(smp.magfilter);
            map.Sampler.States.MinFilter = ColladaUtils.GetTextureFilter(smp.minfilter);
            map.Sampler.States.MipFilter = ColladaUtils.GetTextureFilter(smp.mipfilter);
            map.Sampler.States.MaxMipLevel = Convert.ToInt32(smp.mipmap_maxlevel);
            map.Sampler.States.MipMapLevelOfDetailBias = smp.mipmap_bias;
            
            map.Texture = (JTexture)Jad.Video.Textures[ColladaUtils.GetTextureBySampler(smp, profile, false)];
            map.TextureEnabled = true;
            map.Color = new JColor(1f, 1f, 1f);
            map.UTile = 1f;
            map.VTile = 1f;
            return map;
        }

        /// <summary>
        /// Loads an emissive map from collada to jad
        /// </summary>
        /// <param name="map"></param>
        /// <param name="tex"></param>
        /// <param name="profile"></param>
        private JEmissiveMap LoadEmissiveMap(common_color_or_texture_typeTexture tex, effectFx_profile_abstractProfile_COMMON profile)
        {
        	JEmissiveMap map = new JEmissiveMap();
            fx_sampler2D_common smp = ColladaUtils.GetSampler2D(tex, profile);
            map.Sampler.States.MagFilter = ColladaUtils.GetTextureFilter(smp.magfilter);
            map.Sampler.States.MinFilter = ColladaUtils.GetTextureFilter(smp.minfilter);
            map.Sampler.States.MipFilter = ColladaUtils.GetTextureFilter(smp.mipfilter);
            map.Sampler.States.MaxMipLevel = Convert.ToInt32(smp.mipmap_maxlevel);
            map.Sampler.States.MipMapLevelOfDetailBias = smp.mipmap_bias;

            map.Texture = (JTexture)Jad.Video.Textures[ColladaUtils.GetTextureBySampler(smp, profile, false)];
            map.Enabled = true;
            map.Color = new JColor(1f, 1f, 1f);
            return map;
        }

        /// <summary>
        /// Loads a reflection map from collada to jad
        /// </summary>
        /// <param name="map"></param>
        /// <param name="tex"></param>
        /// <param name="profile"></param>
        private JReflectionMap LoadReflectionMap(common_color_or_texture_typeTexture tex, effectFx_profile_abstractProfile_COMMON profile)
        {
        	JReflectionMap map = new JReflectionMap();
            fx_sampler2D_common smp = ColladaUtils.GetSampler2D(tex, profile);
            map.Sampler.States.MagFilter = ColladaUtils.GetTextureFilter(smp.magfilter);
            map.Sampler.States.MinFilter = ColladaUtils.GetTextureFilter(smp.minfilter);
            map.Sampler.States.MipFilter = ColladaUtils.GetTextureFilter(smp.mipfilter);
            map.Sampler.States.MaxMipLevel = Convert.ToInt32(smp.mipmap_maxlevel);
            map.Sampler.States.MipMapLevelOfDetailBias = smp.mipmap_bias;

            map.Enabled = true;
            map.Texture = (JTextureCube)Jad.Video.Textures[ColladaUtils.GetTextureBySampler(smp, profile, true)];
            return map;
        }

        /// <summary>
        /// Loads a specular map from collada to jad
        /// </summary>
        /// <param name="map"></param>
        /// <param name="tex"></param>
        /// <param name="profile"></param>
        private JSpecularMap LoadSpecularMap(common_color_or_texture_typeTexture tex, effectFx_profile_abstractProfile_COMMON profile)
        {
        	JSpecularMap map = new JSpecularMap();
            fx_sampler2D_common smp = ColladaUtils.GetSampler2D(tex, profile);
            map.Sampler.States.MagFilter = ColladaUtils.GetTextureFilter(smp.magfilter);
            map.Sampler.States.MinFilter = ColladaUtils.GetTextureFilter(smp.minfilter);
            map.Sampler.States.MipFilter = ColladaUtils.GetTextureFilter(smp.mipfilter);
            map.Sampler.States.MaxMipLevel = Convert.ToInt32(smp.mipmap_maxlevel);
            map.Sampler.States.MipMapLevelOfDetailBias = smp.mipmap_bias;

            map.Texture = (JTexture)Jad.Video.Textures[ColladaUtils.GetTextureBySampler(smp, profile, false)];
            map.Enabled = true;
            map.Color = new JColor(1f, 1f, 1f);
            return map;
        }

        /// <summary>
        /// Loads a constant material
        /// </summary>
        /// <param name="e"></param>
        /// <param name="l"></param>
        private void LoadConstantMaterial(effect e, JMaterialLayer l)
        {
            effectFx_profile_abstractProfile_COMMONTechniqueConstant shader = (effectFx_profile_abstractProfile_COMMONTechniqueConstant)e.Items[0].technique.Item;
            effectFx_profile_abstractProfile_COMMON profile = e.Items[0];
            //emissive map
            if (shader.emission.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.emission.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.EmissiveMap.Mode = JEmissiveMode.Color;
                l.EmissiveMap.Color.Red = color[0];
                l.EmissiveMap.Color.Green = color[1];
                l.EmissiveMap.Color.Blue = color[2];
                l.EmissiveMap.Color.Alpha = color[3];
            }
            else if (shader.emission.Item is common_color_or_texture_typeTexture)
            {
                l.EmissiveMap = LoadEmissiveMap((common_color_or_texture_typeTexture)shader.emission.Item, profile);
            }//end load emissive

            //reflection map & reflectivity
            if (shader.reflective.Item is common_color_or_texture_typeColor)
            {
                //TODO - implement when Jad will support color for the reflection map
            }
            else if (shader.reflective.Item is common_color_or_texture_typeTexture)
            {
                l.ReflectionMap = LoadReflectionMap((common_color_or_texture_typeTexture)shader.reflective.Item, profile);
            }//end load reflection map
            l.ReflectionMap.Reflectivity = Convert.ToSingle(((common_float_or_param_typeFloat)shader.reflectivity.Item).Value);

            //transparent map & transparency
            //TODO - not supported by the engine

            //index of refraction
            //NOTE - not exported correctly from 3d software, will be set in JadEd, along with the transparency map
        }

        /// <summary>
        /// Loads a lambert shader based material
        /// </summary>
        /// <param name="e"></param>
        /// <param name="l"></param>
        private void LoadLambertMaterial(effect e, JMaterialLayer l)
        {
            effectFx_profile_abstractProfile_COMMONTechniqueLambert shader = (effectFx_profile_abstractProfile_COMMONTechniqueLambert)e.Items[0].technique.Item;
            effectFx_profile_abstractProfile_COMMON profile = e.Items[0];
            //emissive map
            if (shader.emission.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.emission.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.EmissiveMap.Mode = JEmissiveMode.Color;
                l.EmissiveMap.Color.Red = color[0];
                l.EmissiveMap.Color.Green = color[1];
                l.EmissiveMap.Color.Blue = color[2];
                l.EmissiveMap.Color.Alpha = color[3];
            }
            else if (shader.emission.Item is common_color_or_texture_typeTexture)
            {
                l.EmissiveMap = LoadEmissiveMap((common_color_or_texture_typeTexture)shader.emission.Item, profile);
            }//end load emissive

            //reflection map & reflectivity
            if (shader.reflective.Item is common_color_or_texture_typeColor)
            {
                //TODO - implement when Jad will support color for the reflection map
            }
            else if (shader.reflective.Item is common_color_or_texture_typeTexture)
            {
                l.ReflectionMap = LoadReflectionMap((common_color_or_texture_typeTexture)shader.reflective.Item, profile);
            }//end load reflection map
            l.ReflectionMap.Reflectivity = Convert.ToSingle(((common_float_or_param_typeFloat)shader.reflectivity.Item).Value);

            //transparent map & transparency
            //TODO - not supported by the engine

            //index of refraction
            //NOTE - not exported correctly from 3d software, will be set in JadEd, along with the transparency map

            //ambient map or color
            //TODO - not supported by the engine

            //diffuse map or color
            if (shader.diffuse.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.diffuse.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.DiffuseMap.Color.Red = color[0];
                l.DiffuseMap.Color.Green = color[1];
                l.DiffuseMap.Color.Blue = color[2];
                l.DiffuseMap.Color.Alpha = color[3];
            }
            else if (shader.emission.Item is common_color_or_texture_typeTexture)
            {
                l.DiffuseMap = LoadDiffuseMap((common_color_or_texture_typeTexture)shader.diffuse.Item, profile);
            }//end load diffuse
        }

        /// <summary>
        /// Load Phong shader based material
        /// </summary>
        /// <param name="e"></param>
        /// <param name="l"></param>
        private void LoadPhongMaterial(effect e, JMaterialLayer l)
        {
            effectFx_profile_abstractProfile_COMMONTechniquePhong shader = (effectFx_profile_abstractProfile_COMMONTechniquePhong)e.Items[0].technique.Item;
            effectFx_profile_abstractProfile_COMMON profile = e.Items[0];
            //emissive map
            if (shader.emission.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.emission.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.EmissiveMap.Mode = JEmissiveMode.Color;
                l.EmissiveMap.Color.Red = color[0];
                l.EmissiveMap.Color.Green = color[1];
                l.EmissiveMap.Color.Blue = color[2];
                l.EmissiveMap.Color.Alpha = color[3];
            }
            else if (shader.emission.Item is common_color_or_texture_typeTexture)
            {
                l.EmissiveMap = LoadEmissiveMap((common_color_or_texture_typeTexture)shader.emission.Item, profile);
            }//end load emissive

            //reflection map & reflectivity
            if (shader.reflective.Item is common_color_or_texture_typeColor)
            {
                //TODO - implement when Jad will support color for the reflection map
            }
            else if (shader.reflective.Item is common_color_or_texture_typeTexture)
            {
                l.ReflectionMap = LoadReflectionMap((common_color_or_texture_typeTexture)shader.reflective.Item, profile);
            }//end load reflection map
            l.ReflectionMap.Reflectivity = Convert.ToSingle(((common_float_or_param_typeFloat)shader.reflectivity.Item).Value);

            //transparent map & transparency
            //TODO - not supported by the engine

            //index of refraction
            //NOTE - not exported correctly from 3d software, will be set in JadEd, along with the transparency map

            //ambient map or color
            //TODO - not supported by the engine

            //diffuse map or color
            if (shader.diffuse.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.diffuse.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.DiffuseMap.Color.Red = color[0];
                l.DiffuseMap.Color.Green = color[1];
                l.DiffuseMap.Color.Blue = color[2];
                l.DiffuseMap.Color.Alpha = color[3];
            }
            else if (shader.diffuse.Item is common_color_or_texture_typeTexture)
            {
                l.DiffuseMap = LoadDiffuseMap((common_color_or_texture_typeTexture)shader.diffuse.Item, profile);
            }//end load diffuse

            //specular map or color
            if (shader.specular.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.specular.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.SpecularMap.Color.Red = color[0];
                l.SpecularMap.Color.Green = color[1];
                l.SpecularMap.Color.Blue = color[2];
                l.SpecularMap.Color.Alpha = color[3];
            }
            else if (shader.specular.Item is common_color_or_texture_typeTexture)
            {
                l.SpecularMap = LoadSpecularMap((common_color_or_texture_typeTexture)shader.specular.Item, profile);
            }//end load specular

            //shininess
            common_float_or_param_typeFloat shin = (common_float_or_param_typeFloat)shader.shininess.Item;
            l.SpecularMap.Power = Convert.ToSingle(shin.Value);
        }

        /// <summary>
        /// Loads a Blinn shader based material
        /// </summary>
        /// <param name="e"></param>
        /// <param name="l"></param>
        private void LoadBlinnMaterial(effect e, JMaterialLayer l)
        {
            effectFx_profile_abstractProfile_COMMONTechniqueBlinn shader = (effectFx_profile_abstractProfile_COMMONTechniqueBlinn)e.Items[0].technique.Item;
            effectFx_profile_abstractProfile_COMMON profile = e.Items[0];
            //emissive map
            if (shader.emission.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.emission.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.EmissiveMap.Mode = JEmissiveMode.Color;
                l.EmissiveMap.Color.Red = color[0];
                l.EmissiveMap.Color.Green = color[1];
                l.EmissiveMap.Color.Blue = color[2];
                l.EmissiveMap.Color.Alpha = color[3];
            }
            else if (shader.emission.Item is common_color_or_texture_typeTexture)
            {
                l.EmissiveMap = LoadEmissiveMap((common_color_or_texture_typeTexture)shader.emission.Item, profile);
            }//end load emissive

            //reflection map & reflectivity
            if (shader.reflective.Item is common_color_or_texture_typeColor)
            {
                //TODO - implement when Jad will support color for the reflection map
            }
            else if (shader.reflective.Item is common_color_or_texture_typeTexture)
            {
                l.ReflectionMap = LoadReflectionMap((common_color_or_texture_typeTexture)shader.reflective.Item, profile);
            }//end load reflection map
            l.ReflectionMap.Reflectivity = Convert.ToSingle(((common_float_or_param_typeFloat)shader.reflectivity.Item).Value);

            //transparent map & transparency
            //TODO - not supported by the engine

            //index of refraction
            //NOTE - not exported correctly from 3d software, will be set in JadEd, along with the transparency map

            //ambient map or color
            //TODO - not supported by the engine

            //diffuse map or color
            if (shader.diffuse.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.diffuse.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.DiffuseMap.Color.Red = color[0];
                l.DiffuseMap.Color.Green = color[1];
                l.DiffuseMap.Color.Blue = color[2];
                l.DiffuseMap.Color.Alpha = color[3];
            }
            else if (shader.diffuse.Item is common_color_or_texture_typeTexture)
            {
                l.DiffuseMap = LoadDiffuseMap((common_color_or_texture_typeTexture)shader.diffuse.Item, profile);
            }//end load diffuse

            //specular map or color
            if (shader.specular.Item is common_color_or_texture_typeColor)
            {
                common_color_or_texture_typeColor item = (common_color_or_texture_typeColor)shader.specular.Item;
                float[] color = ColladaUtils.StringToFloat(item.Text[0]);
                l.SpecularMap.Color.Red = color[0];
                l.SpecularMap.Color.Green = color[1];
                l.SpecularMap.Color.Blue = color[2];
                l.SpecularMap.Color.Alpha = color[3];
            }
            else if (shader.specular.Item is common_color_or_texture_typeTexture)
            {
                l.SpecularMap = LoadSpecularMap((common_color_or_texture_typeTexture)shader.specular.Item, profile);
            }//end load specular

            //shininess
            common_float_or_param_typeFloat shin = (common_float_or_param_typeFloat)shader.shininess.Item;
            l.SpecularMap.Power = Convert.ToSingle(shin.Value);
        }

        /// <summary>
        /// Load animation controllers from collada
        /// </summary>
        private void LoadControllers()
        {
            library_controllers lib_con = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_controllers)
                {
                    lib_con = (library_controllers)collada.Items[i];
                    break;
                }
            }

            library_animations lib_anim = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_animations)
                {
                    lib_anim = (library_animations)collada.Items[i];
                    break;
                }
            }

            if (lib_con != null && lib_anim != null)
            {
                for (int i = 0; i < lib_con.controller.Length; i++)
                {
                    controller c = lib_con.controller[i];
                    if (c.Item is morph)
                    {
                        //Load morphing controller
                        string name = c.id;
                        morph m = (morph)c.Item;
                        
                        //source mesh
                        string source = m.source1.Remove(0, 1);

                        //obtaining source arrays for targets and weights
                        string target_source = "";
                        string weight_source = "";
                        for (int j = 0; j < m.targets.input.Length; j++)
                        {
                            InputLocal input = m.targets.input[j];
                            switch (ColladaUtils.GetSemantic(input.semantic))
                            {
                                case 10: //morph target
                                    {
                                        target_source = input.source.Remove(0, 1);
                                        break;
                                    }
                                case 11: //morph weight
                                    {
                                        weight_source = input.source.Remove(0, 1);
                                        break;
                                    }
                            }
                        }

                        //get targets
                        string[] targets = null;
                        for (int j = 0; j < m.source.Length; j++)
                        {
                            if (m.source[j].id.CompareTo(target_source) == 0)
                            {
                                IDREF_array array = (IDREF_array)m.source[j].Item;
                                targets = ColladaUtils.StringToString(array.Value);
                                break;
                            }
                        }

						JMorphingAnimation a = new JMorphingAnimation();

						//instantiate the collection if case
						if(Jad.Scene.Animations == null)
						{
							Jad.Scene.Animations = new List<JadEngine.Animation.IJAnimation>();
						}
						
						Jad.Scene.Animations.Add(a);
						
                        //get controller from animation
						JMorphingController jc = (JMorphingController)a.Controller;

                        //set up morphing targets

                        //setup first morphing targets which is the original state of the mesh
                        JMeshObject mesh = Jad.Scene.MeshObjects[source];
                        JAnimationMorphTargetKey k = new JAnimationMorphTargetKey(mesh);
                        jc.KeyFrames.Add(k);
                        mesh.Hide = true;

                        //setup additional morphing targets
                        for (int j = 0; j < targets.Length; j++)
                        {
                            mesh = Jad.Scene.MeshObjects[targets[j]];
							k = new JAnimationMorphTargetKey(mesh);
							k.Name = targets[j];
                            jc.KeyFrames.Add(k);
                            mesh.Hide = true;
                        }

                        
                        //for each morphing target, load up the weights animation
                        for (int j = 0; j < targets.Length; j++)
                        {
                            JAnimationChannel ch = new JAnimationChannel();
                            jc.Channels.Add(ch);
							ch.ParentController = jc;
							ch.TargetKeyframe = jc.KeyFrames[j];

							//load channel associated to this morph target
							LoadWeightAnimation(lib_anim, ch, j, weight_source);
						
							//add it to channels list for animation clip processing
							channels.Add(ch);
						}

                        //replace mesh source with animation source
						mesh = Jad.Scene.MeshObjects[source];
						mesh.Mesh = jc.Source;
                    }
                    else if (c.Item is skin)
                    {
                        //Load skin controller
                    }
                }
            }
        }

        /// <summary>
        /// Loads up weights animation for morphing
        /// </summary>
        /// <param name="lib_anim">Animation library</param>
        /// <param name="key">Key to be filled with data</param>
        /// <param name="index">Index of the key</param>
        /// <param name="anim_target">Animation target</param>
        private void LoadWeightAnimation(library_animations lib_anim, JAnimationChannel ch, int index,
            string anim_target)
        {
            string sampler = "";
            animation anim = null;

            //compute target string
            string target = anim_target + "(" + index.ToString() + ")";

            //search for the animation
            for (int i = 0; i < lib_anim.animation.Length; i++)
            {
                animation a = ColladaUtils.GetDeepestAnimation(lib_anim.animation[i]);
                for (int j = 0; j < a.Items.Length; j++)
                {
                    if (a.Items[j] is channel)
                    {
                        channel c = (channel)a.Items[j];
                        if (c.target.CompareTo(target) == 0)
                        {
                            sampler = c.source.Remove(0, 1);
                            anim = a;
                            ch.Name = anim.id;
                            break;
                        }
                    }
                }
            }

            string sinput = "";
            string soutput = "";
            string sinterp = "";

            //load the animation
            if (sampler.CompareTo("") != 0)
            {
                //parse the sampler for interpolation
                for (int i = 0; i < anim.Items.Length; i++)
                {
                    if (anim.Items[i] is sampler)
                    {
                        sampler s = (sampler)anim.Items[i];
                        if (s.id.CompareTo(sampler) == 0)
                        {
                            for (int j = 0; j < s.input.Length; j++)
                            {
                                switch (ColladaUtils.GetSemantic(s.input[j].semantic))
                                {
                                    case 4: //input
                                        sinput = s.input[j].source.Remove(0, 1);
                                        break;
                                    case 13: //output
                                        soutput = s.input[j].source.Remove(0, 1);
                                        break;
                                    case 6: //interpolation
                                        sinterp = s.input[j].source.Remove(0, 1);
                                        break;
                                }
                            }
                            break;
                        }
                    }
                }
                

                //parse the sources
                for (int i = 0; i < anim.Items.Length; i++)
                {
                    if (anim.Items[i] is source)
                    {
                        source s = (source)anim.Items[i];

                        //input
                        if (s.id.CompareTo(sinput) == 0)
                        {
                            if (s.Item is float_array)
                            {
                                float_array f = (float_array)s.Item;
                                float[] array = ColladaUtils.StringToFloat(f.Text[0]);
                                ch.Time = array;
                            }
                        }

                        //output
                        else if (s.id.CompareTo(soutput) == 0)
                        {
                            if (s.Item is float_array)
                            {
                                float_array f = (float_array)s.Item;
                                float[] array = ColladaUtils.StringToFloat(f.Text[0]);
                                ch.Weights = array;
                            }
                        }

                        //interpolations
                        else if (s.id.CompareTo(sinterp) == 0)
                        {
                            if (s.Item is Name_array)
                            {
                                Name_array n = (Name_array)s.Item;
                                if (n.Text[0].CompareTo("LINEAR") == 0)
                                {
                                    ch.InterpolationType = JadEngine.Mathematics.JInterpolationType.Linear;
                                }
                                else if (n.Text[0].CompareTo("BEZIER") == 0)
                                {
                                    ch.InterpolationType = JadEngine.Mathematics.JInterpolationType.Bezier;
                                }
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Loads animations clips from collada
        /// </summary>
        private void LoadAnimationClips()
        {
			library_animation_clips lib_clip = null;
			for (int i = 0; i < collada.Items.Length; i++)
			{
				if (collada.Items[i] is library_animation_clips)
				{
					lib_clip = (library_animation_clips)collada.Items[i];
					break;
				}
			}

			//load up clips if possible
			if (lib_clip != null)
			{
				for (int i = 0; i < lib_clip.animation_clip.Length; i++)
				{
					animation_clip c = lib_clip.animation_clip[i];
					JAnimationClip clip = new JAnimationClip(c.id);
					clip.StartTime = (float)c.start;
					clip.StopTime = (float)c.end;

					bool added = false;//flag for marking the insertion of the clip into an animation

					//get animation urls
					for (int j = 0; j < c.instance_animation.Length; j++)
					{
						string url = c.instance_animation[j].url.Remove(0, 1);
						//search for the animation channel with that id
						for (int k = 0; k < channels.Count; k++)
						{
							if (channels[k].Name.CompareTo(url) == 0)
							{
								//add the channel to the clip
								clip.Channels.Add(channels[k]);

								//add clip to the animation that contains the controller which is the 
								//parent of the first channel of the clips
								if (channels[k].ParentController.ParentAnimation is JMorphingAnimation && !added)
								{
									added = true;
									JMorphingAnimation m = (JMorphingAnimation)channels[k].ParentController.ParentAnimation;
									m.Clips.Add(clip);
									clip.ParentAnimation = m;
								}
							}
						}
					}
				}
			}
        }

		/// <summary>
		/// Loads animations from collada
		/// </summary>
		private void LoadAnimations()
		{
		}

        /// <summary>
        /// Converts collada scene to Jad's scene graph
        /// </summary>
        private void LoadSceneGraph()
        {
            library_visual_scenes lib_scene = null;
            for (int i = 0; i < collada.Items.Length; i++)
            {
                if (collada.Items[i] is library_visual_scenes)
                {
                    lib_scene = (library_visual_scenes)collada.Items[i];
                    break;
                }
            }

            if(lib_scene != null)
            {
	            //find the visual scene in the scene library
	            string scene_ref = collada.scene.instance_visual_scene.url.Remove(0, 1);
	            for (int i = 0; i < lib_scene.visual_scene.Length; i++)
	            {
	                if (lib_scene.visual_scene[i].id.CompareTo(scene_ref) == 0)
	                {
	                    //load scene graph
	                    visual_scene scene = lib_scene.visual_scene[i];
	                    for (int j = 0; j < scene.node.Length; j++)
	                    {
	                        LoadNode(Jad.Scene.Root, scene.node[j]);
	                    }
	
	                }
	            }
            }
        }

        /// <summary>
        /// Loads a scene graph hierarcy from collada to jad
        /// </summary>
        /// <param name="jn">Jad root node</param>
        /// <param name="cn">Collada child node</param>
        private void LoadNode(JNode jn, node cn)
        {
            JNode n = null;
            switch (ColladaUtils.GetNodeType(cn))
            {
                case 0:
                    n = (JNode)Jad.Scene.MeshObjects[cn.instance_geometry[0].url.Remove(0, 1)];
                    //link to material
                    ((JMeshObject)n).Material = new JMaterial[]{Jad.Scene.Materials[cn.instance_geometry[0].bind_material.technique_common[0].target.Remove(0, 1)]};
                    break;
                case 1:
                    n = (JNode)Jad.Scene.Lights[cn.instance_light[0].url.Remove(0, 1)];
                    break;
                case 2:
                    n = (JNode)Jad.Scene.Cameras[cn.instance_camera[0].url.Remove(0, 1)];
                    break;
                case 3:
                    //TODO - add for animation
                    break;
                case 4:
                    n = new JDummy(cn.id);
                    Jad.Scene.Dummies.Add((JDummy)n);
                    break;
            }
            //add as child
            jn.SceneGraph.AddChild(n);
            
            Vector3 t = new Vector3();
            Vector3 r = new Vector3();
            Vector3 s = new Vector3();

            //load transformation stuff
            for (int i = 0; i < cn.Items.Length; i++)
            {
            	if (cn.Items[i] is TargetableFloat3)
                {
            		TargetableFloat3 tf = (TargetableFloat3)cn.Items[i];
            		float[] trans = ColladaUtils.StringToFloat(tf.Text[0]);
            		if(tf.sid.CompareTo("translate") == 0)
            		{
            			t.X = trans[0]; t.Y = trans[1]; t.Z = trans[2];
            		}
            		else if(tf.sid.CompareTo("scale") == 0)
            		{
            			s.X = trans[0]; s.Y = trans[1]; s.Z = trans[2];
            		}
                }
            	else if(cn.Items[i] is rotate)
            	{
            		rotate rot = (rotate)cn.Items[i];
            		if(rot.sid.CompareTo("rotateX") == 0)
            		{
            			float[] x = ColladaUtils.StringToFloat(rot.Text[0]);
            			r.X = x[3];
            		}
            		else if(rot.sid.CompareTo("rotateY") == 0)
            		{
            			float[] y = ColladaUtils.StringToFloat(rot.Text[0]);
            			r.Y = y[3];
            		}
            		else if(rot.sid.CompareTo("rotateZ") == 0)
            		{
            			float[] z = ColladaUtils.StringToFloat(rot.Text[0]);
            			r.Z = z[3];
            		}
            	}
            }
            
            Matrix translate = Matrix.Translation(t);
            Matrix rotate = Matrix.RotationX(r.X);
            rotate = Matrix.Multiply(rotate, Matrix.RotationY(r.Y));
            rotate = Matrix.Multiply(rotate, Matrix.RotationZ(r.Z));
            
            n.Transform.LocalTM = Matrix.Multiply(translate, rotate);
            
            //recurse through all the children
            if(cn.node1 != null)
            {
	            for (int i = 0; i < cn.node1.Length; i++)
	            {
	                LoadNode(n, cn.node1[i]);
	            }
            }
        }

        /// <summary>
        /// Creates Jad scene from collada
        /// </summary>
        private void CreateScene()
        {
            LoadGeometry();
            LoadLights();
            LoadCameras();
            LoadTextures();
            LoadMaterials();
            LoadControllers(); 
			LoadAnimations();
			LoadAnimationClips();
            LoadSceneGraph();
        }

        /// <summary>
        /// Loads data from colalda file
        /// </summary>
        /// <param name="source">Source file</param>
        public void LoadData(string source)
        {
            try
            {
                XmlSerializer sr = new XmlSerializer(typeof(COLLADA));
                TextReader tr = new StreamReader(source);
                collada = (COLLADA)(sr.Deserialize(tr));
                tr.Close();
                
                filePath = Path.GetDirectoryName(source);

                CreateScene();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                Console.ReadLine();
            }
        }
        
        /// <summary>
        /// Saves data to jad binary format
        /// </summary>
        /// <param name="destination">Destiation file</param>
        public void SaveData(string destination)
        {
        	JBinaryExport.SaveFullScene(destination);
        	string path = destination + "\\textures";
        	foreach (string file in images)
        	{
        		string src = filePath + '\\' + file;
        		string dest = path + '\\' + file;
        		System.IO.File.Copy(src, dest);
        	}
        }

        /// <summary>
        /// Returns Jad's texture by it's name
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public JTextureBase GetTextureByName(string name)
        {
            for (int i = 0; i < Jad.Video.Textures.Count; i++)
            {
                if (Jad.Video.Textures.Items[i].Name.CompareTo(name) == 0)
                {
                    return Jad.Video.Textures.Items[i];
                }
            }
            return null;
        }

        #endregion
    }
}
