#region Using directives

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Microsoft.DirectX;

using Microsoft.DirectX.Direct3D;


using JadEngine;
using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Scene;
using JadEngine.Scene.MD5;
using JadEngine.Physics;
using JadEngine.Particles;
using JadEngine.Sound;
using JadEngine.Tools;

#endregion

namespace Collada2Jad
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            // Initialice the engine

            Jad.Video.Adapter.Initialization.ReadFromConfig();

            //GetNewSize();

            Jad.Form = this;

            Jad.Handle = this.Handle;

            Jad.Video.Adapter.CreateDevice(renderControl);

            Jad.IDontNeedToRender = true;

            Jad.AfterCreateDevice();

            Jad.LoadCreateInHouse();

            Jad.Begin();

           // Application.Idle += new EventHandler(OnApplicationIdle);
        }

        /// <summary>
        /// Gets if there are messages waiting to be processed or not
        /// </summary>
        private bool AppStillIdle
        {
            get
            {
                Message msg;
                return !PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
            }
        }

        // We won't use this maliciously
        [System.Security.SuppressUnmanagedCodeSecurity]
        [DllImport("User32.dll", CharSet = CharSet.Auto)]
        public static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="sender">TODO</param>
        /// <param name="e">TODO</param>
        private void OnApplicationIdle(object sender, EventArgs e)
        {
            // Render a frame during idle time (no messages are waiting)
            while (AppStillIdle)
            {
                // Begin application update
                //Jad.BeginUpdate();
                // End application update
                //Jad.EndUpdate();

            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Jad.End();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit();
        }

        private void toolStripButton3_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            if (openDlg.ShowDialog() == DialogResult.OK)
            {
            	converter.Reset();
                converter.LoadData(openDlg.FileName);
                Jad.Scene.Root.SceneGraph.Debug();
                //update the tree view
                tree.BeginUpdate();
                tree.Nodes.Clear();
                
                int cam_ind = tree.Nodes.Add(new TreeNode("Cameras"));
                int lt_ind = tree.Nodes.Add(new TreeNode("Lights"));
                int geo_ind = tree.Nodes.Add(new TreeNode("Geometry"));
                
                for(int i = 0; i < Jad.Scene.Cameras.Count; i++)
                {
                	tree.Nodes[cam_ind].Nodes.Add(new TreeNode(Jad.Scene.Cameras[i].Name));
                }
                if(Jad.Scene.Cameras.Count > 0)
                {
                	tree.Nodes[cam_ind].Expand();
                }

                for(int i = 0; i < Jad.Scene.Lights.Count; i++)
                {
                	tree.Nodes[lt_ind].Nodes.Add(new TreeNode(Jad.Scene.Lights[i].Name));
                }
                if(Jad.Scene.Lights.Count > 0)
                {
                	tree.Nodes[lt_ind].Expand();
                }
                
                for(int i = 0; i < Jad.Scene.MeshObjects.Count; i++)
                {
                	tree.Nodes[geo_ind].Nodes.Add(new TreeNode(Jad.Scene.MeshObjects[i].Name));
                }
                if(Jad.Scene.MeshObjects.Count > 0)
                {
                	tree.Nodes[geo_ind].Expand();
                }


                tree.EndUpdate();
            }
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            if (saveDlg.ShowDialog() == DialogResult.OK)
            {
            	converter.SaveData(saveDlg.FileName);
            }
        }

        
        void TreeAfterCheck(object sender, System.Windows.Forms.TreeViewEventArgs e)
        {
        	if(e.Node.Text.CompareTo("Cameras") == 0 || 
        	  e.Node.Text.CompareTo("Lights") == 0 ||
        	  e.Node.Text.CompareTo("Geometry") == 0)
        	{
        		if(e.Action != TreeViewAction.Unknown)
        		{
	        		bool state = e.Node.Checked;
	        		for(int i = 0; i < e.Node.Nodes.Count; i++)
	        		{
	        			e.Node.Nodes[i].Checked = state;
	        		}
        		}
        	}
        	else
        	{
        		if(e.Node.Checked && !e.Node.Parent.Checked)
        		{
        			e.Node.Parent.Checked = true;
        		}
        		else if(!e.Node.Checked && e.Node.Parent.Checked)
        		{
        			bool check = false;
        			for(int i = 0; i < e.Node.Parent.Nodes.Count; i++)
        			{
        				if(e.Node.Parent.Nodes[i].Checked)
        				{
        					check = true;
        				}
        			}
        			if(!check)
        			{
        				e.Node.Parent.Checked = false;
        			}
        		}
        	}
        }
    }
}
