#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace JadePackager
{
	/// <summary>
	/// Dialog used to configure a filter.
	/// </summary>
	public partial class ConfigurationDialog : Form
	{
		#region Constructors

		/// <summary>
		/// Default constructor.
		/// </summary>
		public ConfigurationDialog()
		{
			InitializeComponent();
		}

		#endregion

		#region Methods

		/// <summary>
		/// Adds a control to the dialog.
		/// </summary>
		/// <param name="control">Control to add.</param>
		public void AddUserControl(UserControl control)
		{
			if (control != null)
			{
				control.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
				Controls.Add(control);
				
				Height = (control.Height + 80);
				if (control.Width < 310)
				{
					control.Width = 310;
					Width = 310;
				}

				else
					Width = control.Width;

				control.Focus();
			}
		}

		#endregion

		#region Event Handlers

		/// <summary>
		/// Event raised when the user clicks on the OK button.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void btnOK_Click(object sender, EventArgs e)
		{
			Close();
		}

		/// <summary>
		/// Event raised when the user clicks on the Cancel button.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void btnCancel_Click(object sender, EventArgs e)
		{
			Close();
		}

		/// <summary>
		/// Event raised when the user presses a key.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void ConfigurationDialog_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				if (btnCancel.Focused)
					DialogResult = DialogResult.Cancel;

				else
					DialogResult = DialogResult.OK;

				Close();
			}
		}

		#endregion
	}
}
