#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace JadePackager
{
	/// <summary>
	/// This class is used to ask for input strings and show messages.
	/// </summary>
	public partial class InputDialog : Form
	{
		#region Fields

		/// <summary>
		/// If the input must be shown or not.
		/// </summary>
		private bool _showInput;

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the input text.
		/// </summary>
		public String Input
		{
			get { return this.txtInput.Text; }
			set
			{
				this.txtInput.Text = value;
				this.txtInput.SelectAll();
			}
		}

		/// <summary>
		/// Gets or sets the message.
		/// </summary>
		public String Message
		{
			get { return this.lblMessage.Text; }
			set
			{
				this.lblMessage.Text = value;

				if (lblMessage.Width + 24 > this.Width)
					this.Width = lblMessage.Width + 24;
			}
		}

		/// <summary>
		/// Gets or sets if the input must be shown or not.
		/// </summary>
		public bool ShowInput
		{
			get { return _showInput; }
			set
			{
				_showInput = value;

				if (_showInput == false)
				{
					txtInput.Visible = false;
					this.Size = new Size(this.Width, this.Height - 28);
				}
			}
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor.
		/// </summary>
		public InputDialog()
		{
			InitializeComponent();
		}

		/// <summary>
		/// Parametrized constructor.
		/// </summary>
		/// <param name="text">Text of the dialog.</param>
		/// <param name="message">Message of the dialog.</param>
		/// <param name="showInput">If the dialog must show the input or not.</param>
		public InputDialog(String text, String message, bool showInput)
		{
			InitializeComponent();

			this.Text = text;
			this.Message = message;
			this.ShowInput = showInput;
		}

		#endregion

		#region Event Handlers

		/// <summary>
		/// Handles the key down event to capture the enter key and close the dialog.
		/// </summary>
		/// <param name="sender">Sender object of the event.</param>
		/// <param name="e">Event parameters.</param>
		private void txtInput_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				DialogResult = DialogResult.OK;
				this.Close();
			}
		}

		#endregion
	}
}