#region Using directives

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;
using Microsoft.DirectX.Direct3D;

using JadEngine;
using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Scene;
using JadEngine.Scene.MD5;
using JadEngine.Physics;
using JadEngine.Particles;
using JadEngine.Sound;
using JadEngine.Tools;
using JadEngine.Input;

#endregion

namespace Tutorial
{
    public partial class SimpleFormControl : Form
    {
        public SimpleFormControl()
        {
            InitializeComponent();
        }

        private void SimpleFormControl_Load(object sender, EventArgs e)
        {
            // Initialice the engine


			Jad.Video.Adapter.Initialization.ReadFromConfig();
            
            Jad.Form = this;

            Jad.Handle = this.Handle;

            Jad.Video.Adapter.CreateDevice(renderControl);

			Jad.AfterCreateDevice();

			Jad.LoadCreateInHouse();

            // Let's add a cube 

            JMesh cubeMesh = JMesh.CreateCube(3, 3, 3);

            JMeshObject cube = Jad.Scene.MeshObjects.Create("room", false);

            cube.Mesh = cubeMesh;

            cube.SetDefaultMaterial();

            // Create an omni light
            JOmniLight light = (JOmniLight)Jad.Scene.Lights.Create("light", JLightType.Omni);

            light.Transform.Position = new Vector3(0, 0, -2f);

            light.Attenuation.Start = 1f;

            light.Attenuation.End = 1.3f;

            light.Color = JColor.Yellow;

            Jad.Scene.Lights.Ambient = new JColor(0.4f);

			Jad.Begin();
			
			// Create our input, but don't attach it (until we click in the view)
			Jad.CreateInput(Jad.CurrentView, false, false, true, true);
            
			Application.Idle += new EventHandler(OnApplicationIdle);
        }

		/// <summary>
		/// Gets if there are messages waiting to be processed or not
		/// </summary>
		private bool AppStillIdle
		{
			get
			{
				Message msg;
				return !PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
			}
		}

		// We won't use this maliciously
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("User32.dll", CharSet = CharSet.Auto)]
		public static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="sender">TODO</param>
        /// <param name="e">TODO</param>
        private void OnApplicationIdle(object sender, EventArgs e)
        {
            // Render a frame during idle time (no messages are waiting)
            while (AppStillIdle)
            {
                // Begin application update
                Jad.BeginUpdate();

                // Update the application
                Action();

                // End application update
                Jad.EndUpdate();

                // Render scene
                Render();
            }
        }

        private void SimpleFormControl_FormClosing(object sender, FormClosingEventArgs e)
        {
            Jad.End();
        }

        public bool Action()
        {
			if (Jad.Input.Keyboard[Key.Escape].Pressed)
				Jad.InputDevices.Detach();

            return true;
        }

        public void Render()
        {
            if (Jad.Video.Device == null) 
				return;

            JView backbuffer = Jad.Video.Views[0];
            backbuffer.Render();

            JFont font = Jad.Video.Fonts[0];
            Jad.Video.Fonts.Begin();
            font.RenderLine(Jad.Video.Views.CurrentView.Stats.Fps, 0, JColor.White);
            Jad.Video.Fonts.End();

            backbuffer.Present();
        }

        private void renderControl_MouseClick(object sender, MouseEventArgs e)
        {
            renderControl.Focus();

			Jad.InputDevices.Attach(Jad.CurrentView,true,true,true,true);
        }

        void GetNewSize()
        {
            Size s = renderControl.Size;

            // Modify the parameters to indicate the size change
            JView view = Jad.Video.Views[0];

            view.SetResetValues(s.Width, s.Height);
        }

        private void SimpleFormControl_Resize(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Minimized)
            {
                Jad.Active = false;
                return;
            }
            else if (Jad.Active == false)
                Jad.Active = true;

            GetNewSize();
            Jad.Video.Adapter.Reset();
        }

        private void SimpleFormControl_Activated(object sender, EventArgs e)
        {
            Jad.Active = true;
        }

        private void SimpleFormControl_Deactivate(object sender, EventArgs e)
        {
            Jad.Active = false;
        }
    }
}