#region Using directives

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;
using Microsoft.DirectX.Direct3D;

using JadEngine;
using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Scene;
using JadEngine.Scene.MD5;
using JadEngine.Physics;
using JadEngine.Particles;
using JadEngine.Sound;
using JadEngine.Tools;
using JadEngine.Input;

#endregion

namespace Tutorial
{
    public partial class SwapChainsFormControls : Form
	{
		#region Fields
		bool _displayTextInControl1 = true;
		bool _displayTextInControl2 = true;
        JSprite sprite;
		#endregion

		#region Internal Game Loop

		/// <summary>
        /// Gets if there are messages waiting to be processed or not
        /// </summary>
        private bool AppStillIdle
        {
            get
            {
                Message msg;
                return !PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
            }
        }

        // We won't use this maliciously
        [System.Security.SuppressUnmanagedCodeSecurity]
        [DllImport("User32.dll", CharSet = CharSet.Auto)]
        public static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="sender">TODO</param>
        /// <param name="e">TODO</param>
        private void OnApplicationIdle(object sender, EventArgs e)
        {
            // Render a frame during idle time (no messages are waiting)
            while (AppStillIdle)
            {
                // Begin application update
                Jad.BeginUpdate();

                // Update the application
                UpdateEngine();

                // End application update
                Jad.EndUpdate();

				// Render both controls.  This must be called on each JRenderControl
				renderControl.Render();
				renderControl2.Render();

            }
        }
        #endregion

		#region Constructors
		public SwapChainsFormControls()
        {
            InitializeComponent();
		}
		#endregion

		#region Methods

		private void CreateScene()
        {
			JScene scene = Jad.Scene;

			// Let's add a cube 
			JMesh cubeMesh = JMesh.CreateCube(3, 3, 3);
			JMeshObject cube = scene.MeshObjects.Create("room", false);
            cube.Mesh = cubeMesh;
            cube.SetDefaultMaterial();

            // Create an omni light
			JOmniLight light = (JOmniLight) scene.Lights.Create("light", JLightType.Omni);
            light.Transform.Position = new Vector3(0, 0, -2f);
            light.Attenuation.Start = 1f;
            light.Attenuation.End = 1.3f;
            light.Color = JColor.Yellow;

			// Set our ambient light
			scene.Lights.Ambient = new JColor(0.4f);

			// Explicitly set our camera's input to use Views[0]
			scene.Camera.InputView = Jad.Video.Views[0];
        }

        /// <summary>
        /// Creates a sphere that casts shadows
        /// </summary>
        void CreateSphere(JScene scene)
        {
            float radius = 0.25f;

            JMesh sphereMesh = JMesh.CreateSphere(radius, 20, 20);

            JMeshObject sphere = scene.MeshObjects.Create("sphere", false);

            sphere.Mesh = sphereMesh;

            JMaterial[] material = new JMaterial[1];

            material[0] = scene.Materials.Create();

            JMaterialLayer layer = new JMaterialLayer();

            material[0].AddLayer(layer);

            layer.DiffuseMap.Texture = Jad.Video.Textures.Create2D("mancha", true);

            sphere.Material = material;

            sphere.Transform.Position = new Vector3(0, 0, -2f);

            // This object casts shadows
            sphere.CastShadows = true;

            // Set the extrusion contact of the shadowvolume
            sphere.Mesh.ShadowVolume.ExtrusionConstant = 5f;

            // Optimize the shadowvolume in a 75% of the original mesh vertices
            sphere.Mesh.ShadowVolume.Optimize(0.75f);
        }

        private void CreateScene2()
        {
			// Create a second scene
			JScene scene = Jad.Scenes.Create("Second");

            CreateSphere(scene);
            sprite = new JSprite();
            scene.Shadows.ShadowMode = JShadowMode.StencilVolume;

            // Indicamos que queremos utilizar el modo depthpass, en lugar de depthfail
            scene.Shadows.Stencil.DepthPass = true;
            scene.Shadows.Stencil.Debug = true;

            // Let's add a cube 
            JMesh cubeMesh = JMesh.CreateCube(3, 3, 3);
            JMeshObject cube = scene.MeshObjects.Create("room", false);
            cube.Mesh = cubeMesh;
            cube.SetDefaultMaterial();

            // Create an omni light
            JOmniLight light = (JOmniLight)scene.Lights.Create("light", JLightType.Omni);
            light.Transform.Position = new Vector3(0, 0, -3f);
            light.Attenuation.Start = 1f;
            light.Attenuation.End = 1.8f;
            light.Color = JColor.White;
            light.CastShadows = true;

			// Set the ambient light color
            scene.Lights.Ambient = new JColor(1.0f,0.5f,0.5f);
            
			// Turn on gizmo's
            scene.ShowGizmos = true;

			// Create post production effects (required for gizmos)
            scene.CreatePostEffects();

			// Create a new trackball camera 
			JCameraTrackball camera2 = (JCameraTrackball) scene.Cameras.CreateTrackball("trackballCamera");
			// Set our camera's input to come from Views[1]
			camera2.InputView = Jad.Video.Views[1];

			// Remove the implicit camera - 
			//   This will set scene2.Camera to our new camera automatically
			scene.Camera.Remove();
        }

		public bool UpdateEngine()
		{
			// Update logic goes here

			// NOTE: We're using renderControl.View, but we could just as easily
			// use Jad.Video.Views[0] or Jad.Video.Views["Implicit"]
			if (renderControl.View.Input.Keyboard[Key.F].Pressed)
				_displayTextInControl1 = !_displayTextInControl1;

			if (renderControl2.View.Input.Keyboard[Key.F].Pressed)
				_displayTextInControl2 = !_displayTextInControl2;

			return true;
		}

		#endregion

		#region Event Handlers

		private void SwapChainsFormControls_Load(object sender, EventArgs e)
		{
			// Initialize the engine
			Jad.Video.Adapter.Initialization.ReadFromConfig();
			Jad.Form = this;
			Jad.Handle = this.Handle;
			Jad.Video.Adapter.CreateDevice(renderControl);
			Jad.AfterCreateDevice();
			Jad.LoadCreateInHouse();

			// Create and additional swap chain
			PresentParameters pp = new PresentParameters(Jad.Video.Adapter.Initialization.PresentParams);
			pp.BackBufferWidth = renderControl2.Width;
			pp.BackBufferHeight = renderControl2.Height;
			pp.DeviceWindow = renderControl2;
			pp.DeviceWindowHandle = renderControl2.Handle;
			Jad.Video.Views.CreateSwapChain("Swap chain", pp);

			// Create our scenes
			CreateScene();
			CreateScene2();

			// Create our DirectX input, but do not assign it to a View yet
			Jad.CreateInput(null, false, false, false, false);
			Jad.Begin();

			// Set up our game loop
			Application.Idle += new EventHandler(OnApplicationIdle);
		}
		
		private void SwapChainsFormControls_FormClosing(object sender, FormClosingEventArgs e)
        {
			// Tell the engine to shut down on close
            Jad.End();
        }

        private void SwapChainsFormControls_Resize(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Minimized)
            {
                Jad.Active = false;
                return;
            }
            else if (Jad.Active == false)
                Jad.Active = true;

			// NOTE: JRenderControls automatically handle their own resize events, so we don't have
			// to set anything specific to them.
        }

        private void SwapChainsFormControls_Activated(object sender, EventArgs e)
        {
            Jad.Active = true;
        }

        private void SwapChainsFormControls_Deactivate(object sender, EventArgs e)
        {
            Jad.Active = false;
        }

		
		// Handling the renderControl's OnRender event gives us a way to add
		// things like text, in this case, FPS, to the control during the rendering
		private void renderControl_OnRender(object sender, RenderEventArgs e)
		{
			if (_displayTextInControl1)
			{
				JFont font = Jad.Video.Fonts[0];
				Jad.Video.Fonts.Begin();
				font.RenderLine("FPS: " + e.View.Stats.Fps, 0, JColor.White);
				Jad.Video.Fonts.End();
			}
		}

		private void renderControl2_OnRender(object sender, RenderEventArgs e)
		{
            sprite.SetPositionSize(0, 0, 100, 100);
            sprite.Effect = Jad.Video.Effects.InHouse.PosColorTex1;
            sprite.Texture = Jad.Video.Views[1].RenderTargets.ShadowMask.Texture;
            sprite.Render();
            if (_displayTextInControl2)
			{
				JFont font = Jad.Video.Fonts[0];
				Jad.Video.Fonts.Begin();
				font.RenderLine("Total Meshes in Scene: " + e.Scene.Stats.NumberMeshTotal.ToString(), 0, JColor.White);
				font.RenderLine("FPS: " + e.View.Stats.Fps, 0, JColor.RedColor);
				Jad.Video.Fonts.End();
			}
		}

		#endregion
	}
}
