#region Using directives

using System;
using System.IO;

#endregion

namespace Tutorial
{
	/// <summary>
	/// Class that holds the main method. Its the application entry point
	/// </summary>
	public class JMain
	{
		/// <summary>
		/// Choose the tutorial to run by changing the following constant
		/// </summary>
		private const Tutorials _tutorialToRun = Tutorials.SimpleScene;

		#region Enums

		/// <summary>
		/// Enumeration of all available tutorials
		/// </summary>
		public enum Tutorials
		{
			#region Available Tutorials
			// Disable XML Documentation Warnings
#pragma warning disable 1591

			SimpleFormControl,
			SwapChainsFormControls,
			AdvancedParticleEffects,
			AdvancedStencilShadows,
			BillboardingAndHeatHaze,
			BuoyancyAndDestroying,
			DepthOfField,
			ElasticPhysics,
			EmptyProject,
			GlowAndFog,
			JointsAndScenegraph,
			LightEffects,
			LightingFragments,
			LightShafts,
			LoadingAndBuoyancy,
			LoadingHadddFiles,
			MotionBlur,
			MultilayerMaterials,
			NormalMaps,
			OtherMaps,
			ParallaxOcclusionMapping,
			ParticlesAndRecording,
			PostproductionAndCamera,
			ResetingTheEngine,
			ShadowMap,
			SimplePhysics,
			SimpleScene,
			SimpleSteeringBehaviors,
			SkyboxAndEffects,
			SmoothingGroups,
			SoundAndMusic,
			StandardAttenuation,
			StencilShadows,
			ThirdPersonCamTutorial,
			TrackballCameraTutorial,
			Water,
			WorldCollisions

			// Re-enable XML Doc warnings
#pragma warning restore 1591
			#endregion
		}

		#endregion

		#region Methods

		/// <summary>
		/// Application entry point
		/// </summary>
		[STAThreadAttribute]
		static void Main()
		{
			// Create a new file where well write console output
			StreamWriter sw = new StreamWriter("out.txt");

			// Redirect console output to this file
			Console.SetOut(sw);

			// Uncomment the first line to run the Form tutorial

			JadEngine.JApplication app;
			switch (_tutorialToRun)
			{
				#region Choose Correct Tutorial

				// Since we're using a const, we get an unreachable code detected warning
				// here... this removes that
#pragma warning disable 0162

				// If its a form tutorial, handle it explicitly
				case Tutorials.SimpleFormControl:
					System.Windows.Forms.Application.Run(new SimpleFormControl());
					sw.Close();
					return;
				case Tutorials.SwapChainsFormControls:
					System.Windows.Forms.Application.Run(new SwapChainsFormControls());
					sw.Close();
					return;
				// Otherwise, create the application object
				case Tutorials.AdvancedParticleEffects:
					app = new AdvancedParticleEffects();
					break;
				case Tutorials.AdvancedStencilShadows:
					app = new AdvancedStencilShadows();
					break;
				case Tutorials.BillboardingAndHeatHaze:
					app = new BillboardingAndHeatHaze();
					break;
				case Tutorials.BuoyancyAndDestroying:
					app = new BuoyancyAndDestroying();
					break;
				case Tutorials.DepthOfField:
					app = new DepthOfField();
					break;
				case Tutorials.ElasticPhysics:
					app = new ElasticPhysics();
					break;
				case Tutorials.EmptyProject:
					app = new EmptyProject();
					break;
				case Tutorials.GlowAndFog:
					app = new GlowAndFog();
					break;
				case Tutorials.JointsAndScenegraph:
					app = new JointsAndScenegraph();
					break;
				case Tutorials.LightEffects:
					app = new LightEffects();
					break;
				case Tutorials.LightingFragments:
					app = new LightingFragments();
					break;
				case Tutorials.LightShafts:
					app = new LightShafts();
					break;
				case Tutorials.LoadingAndBuoyancy:
					app = new LoadingAndBuoyancy();
					break;
				case Tutorials.LoadingHadddFiles:
					app = new LoadingHadddFiles();
					break;
				case Tutorials.MotionBlur:
					app = new MotionBlur();
					break;
				case Tutorials.MultilayerMaterials:
					app = new MultilayerMaterials();
					break;
				case Tutorials.NormalMaps:
					app = new NormalMaps();
					break;
				case Tutorials.OtherMaps:
					app = new OtherMaps();
					break;
				case Tutorials.ParallaxOcclusionMapping:
					app = new ParallaxOcclusionMapping();
					break;
				case Tutorials.ParticlesAndRecording:
					app = new ParticlesAndRecording();
					break;
				case Tutorials.PostproductionAndCamera:
					app = new PostproductionAndCamera();
					break;
				case Tutorials.ResetingTheEngine:
					app = new ResetEngine();
					break;
				case Tutorials.ShadowMap:
					app = new ShadowMap();
					break;
				case Tutorials.SimplePhysics:
					app = new SimplePhysics();
					break;
				case Tutorials.SimpleScene:
					app = new SimpleScene();
					break;
				case Tutorials.SimpleSteeringBehaviors:
					app = new SimpleSteeringBehaviors();
					break;
				case Tutorials.SkyboxAndEffects:
					app = new SkyboxAndEffects();
					break;
				case Tutorials.SmoothingGroups:
					app = new SmoothingGroups();
					break;
				case Tutorials.SoundAndMusic:
					app = new SoundAndMusic();
					break;
				case Tutorials.StandardAttenuation:
					app = new StandardAttenuation();
					break;
				case Tutorials.StencilShadows:
					app = new StencilShadows();
					break;
				case Tutorials.ThirdPersonCamTutorial:
					app = new ThirdPersonCamTutorial();
					break;
				case Tutorials.TrackballCameraTutorial:
					app = new TrackballCameraTutorial();
					break;
				case Tutorials.Water:
					app = new Water();
					break;
				case Tutorials.WorldCollisions:
					app = new WorldCollisions();
					break;

				default:
					app = new SimpleScene();
					break;

				// Restore our warning
#pragma warning restore 0162
				#endregion
			}

			// Start the application
			app.Start();

			// Close console output file
			sw.Close();
		}

		#endregion
	}
}
