#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using JadEngine.Mathematics;
using JadEngine.Collections;


#endregion

/*
 * 24/03/08
 *	-Changed to derive from JEngineElement
 */

namespace JadEngine.Animation
{
    public class JAnimationChannel : JObject
    {
        #region Fields

        /// <summary>
        /// True if the channel is active
        /// </summary>
        protected bool _active;		

        /// <summary>
        /// Name for the animation channel
        /// </summary>
        protected string _name;

        /// <summary>
        /// Keyframe's times
        /// </summary>
        protected float[] _time;

        /// <summary>
        /// Weights corresponding to each time key
        /// </summary>
        protected float[] _weights;

        /// <summary>
        /// Interpolation type for this keyframe
        /// </summary>
        private JInterpolationType _intType;

		/// <summary>
		/// Reference to parent controller
		/// </summary>
		protected JAnimationControllerBase _parent;

		/// <summary>
		/// Target keyframe
		/// </summary>
		protected JAnimationKey _targetKeyframe;

		/// <summary>
		/// Percent computed between two keyframes
		/// Used internally
		/// </summary>
		private float _percent;

		/// <summary>
		/// Weight computed based on current percent
		/// </summary>
		protected float _weight;
        
        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        public JAnimationChannel()
        {
            _name = "";
            _active = true;
            _intType = JInterpolationType.Linear;
			_parent = null;
			_targetKeyframe = null;
			_percent = 0f;
			_weight = 0f;
        }

        #endregion

        #region Properties

        /// <summary>
        /// TODO
        /// </summary>
        public bool Active
        {
            get { return _active; }
            set { _active = value; }
        }		

        /// <summary>
        /// TODO
        /// </summary>
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        /// <summary>
        /// TODO
        /// </summary>
        public JInterpolationType InterpolationType
        {
            get { return _intType; }
            set { _intType = value; }
        }

        /// <summary>
        /// TODO
        /// </summary>
        public float[] Time
        {
            get { return _time; }
            set { _time = value; }
        }

        /// <summary>
        /// TODO
        /// </summary>
        public float[] Weights
        {
            get { return _weights; }
            set { _weights = value; }
        }

		/// <summary>
		/// TODO
		/// </summary>
		public JAnimationControllerBase ParentController
		{
			get { return _parent; }
			set { _parent = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public JAnimationKey TargetKeyframe
		{
			get { return _targetKeyframe; }
			set { _targetKeyframe = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public float CurrentWeight
		{
			get { return _weight; }
			set { _weight = value; }
		}

        #endregion

        #region Methods

        /// <summary>
        /// Computes a keyframe using interpolation
        /// </summary>
        /// <param name="t">Animation time as an input parameter</param>
        /// <returns>Interpolated weight for the current channel</returns>
        public float ComputeWeight(float t)
        {
            float weight = 0f;
            if (t < _time[0])
            {
                weight = _weights[0];
            }
            else if (t > _time[_time.Length - 1])
            {
                weight = _weights[_time.Length - 1];
            }
            else
            {
				for (int i = 0; i < _time.Length - 1; i++)
				{
					if (t > _time[i] && t < _time[i + 1])
					{
						_percent = (t - _time[i]) / (_time[i + 1] - _time[i]);
						weight = JInterpolator.Float.Interpolate(_weights[i], _weights[i + 1], _percent, _intType);
						break;
					}
				}
            }

			_weight = weight;
            return weight;
        }

		/// <summary>
		/// Computes current keyframe for the target key(must be used after ComputeWeight
		/// </summary>
		/// <param name="t"></param>
		public void ComputeKeyframe(float t)
		{
			JAnimationKey key = this.ParentController.KeyFrames[0];
			//_percent is computed when computing the weight
			_targetKeyframe.ComputeKeyframe(key, _percent, _intType);
		}

        #endregion
    }
}
