#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using JadEngine.Collections;

#endregion

namespace JadEngine.Animation
{
    public class JMorphingAnimation : IJAnimation
    {
        #region Fields

        /// <summary>
        /// Animation clips
        /// </summary>
        private List<JAnimationClip> _clips;

        /// <summary>
        /// Animation controllers, one for each clip
        /// </summary>
		private JAnimationControllerBase _controller;

        #endregion

        #region Constructors
                
        /// <summary>
        /// Default constructor
        /// </summary>
        public JMorphingAnimation()
        {
			_clips = new List<JAnimationClip>();
            _controller = new JMorphingController();
			_controller.ParentAnimation = this;
        }

        #endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public List<JAnimationClip> Clips
		{
			get { return _clips; }
			set { _clips = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public JAnimationControllerBase Controller
		{
			get { return _controller; }
			set { _controller = value; }
		}

		#endregion

		#region Methods

		/// <summary>
        /// Plays an animation clip with a given speed
        /// </summary>
        /// <param name="name">Clip's name to play</param>
        /// <param name="speed">Animation speed, float multiplier</param>
        /// <param name="type">Playback type</param>
        public void StartAnimationClip(string name, float speed, JAnimationClip.PlaybackType type)
        {
			//activate channels in clip
            for(int i = 0; i < _clips.Count; i++)
            {
                if (_clips[i].Name.CompareTo(name) == 0)
                {
					_clips[i].Speed = speed;
					_clips[i].Playback = type;
					_clips[i].CurrentTime = _clips[i].StartTime;
					_clips[i].Active = true;
					break;
                }
            }
        }

		/// <summary>
        /// Stops an animation clip currently playing
        /// </summary>
        /// <param name="name"></param>
		public void StopAnimationClip(string name)
		{
			//deactivate channels in clip
			for (int i = 0; i < _clips.Count; i++)
			{
				if (_clips[i].Name.CompareTo(name) == 0)
				{
					_clips[i].Active = false;
					break;
				}
			}
		}

        /// <summary>
        /// Updates the animation using time parameter
        /// </summary>
        /// <param name="time">Time passed since last update</param>
        public void UpdateAnimation(float time)
        {
			for (int i = 0; i < _clips.Count; i++)
			{
				if (_clips[i].Active)
				{
					_clips[i].Play(time, _controller);
				}
			}
        }

		#endregion
	}
}
