#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using JadEngine.Scene;
using JadEngine.Collections;
using JadEngine.Video;

#endregion


namespace JadEngine.Animation
{
    /// <summary>
    /// Animation controller for morphing animations
    /// </summary>
    public class JMorphingController : JAnimationControllerBase
    {
        #region Fields

        /// <summary>
        /// Reference to the current mesh
        /// </summary>
        private JAnimationMorphTargetKey _morphSource;

        #endregion

        #region Constructors

        /// <summary>
        /// Public constructor
        /// </summary>
        public JMorphingController()
        {
            this.controllerType = AnimationControllerType.Morph;
            _morphSource = new JAnimationMorphTargetKey();
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="m"></param>
        public JMorphingController(JMeshObject m)
        {
            this.controllerType = AnimationControllerType.Morph;
            _morphSource = new JAnimationMorphTargetKey(m);
        }

        #endregion

        #region Properties

        /// <summary>
        /// Mesh to be morphed
        /// </summary>
        public JMesh Source
        {
            get { return _morphSource.MorphTarget; ; }
            set { _morphSource.MorphTarget = value; }
        }

        #endregion

        #region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="channels"></param>
		public override void Update(List<JAnimationChannel> channels)
		{
			base.Update(channels);

			float weightSum = 0f;
			
			//reset vertices
			for (int i = 0; i < _morphSource.MorphTarget.VertexBuffer.NumberOfVertices; i++)
			{
				_morphSource.MorphTarget.VertexBuffer.Position[i].X = 0f;
				_morphSource.MorphTarget.VertexBuffer.Position[i].Y = 0f;
				_morphSource.MorphTarget.VertexBuffer.Position[i].Z = 0f;
			}

			//add all keyframes into this one
			foreach (JAnimationChannel c in channels)
			{
				weightSum += c.CurrentWeight;
				for (int i = 0; i < _morphSource.MorphTarget.VertexBuffer.NumberOfVertices; i++)
				{
					_morphSource.MorphTarget.VertexBuffer.Position[i] =
						_morphSource.MorphTarget.VertexBuffer.Position[i] +
						(((JAnimationMorphTargetKey)c.TargetKeyframe).CurrentKey.VertexBuffer.Position[i] *
						c.CurrentWeight);
				}
			}

			//divide buy weighted sum
			for (int i = 0; i < _morphSource.MorphTarget.VertexBuffer.NumberOfVertices; i++)
			{
				_morphSource.MorphTarget.VertexBuffer.Position[i].X /= weightSum;
				_morphSource.MorphTarget.VertexBuffer.Position[i].Y /= weightSum;
				_morphSource.MorphTarget.VertexBuffer.Position[i].Z /= weightSum;
			}
		}

		#endregion
	}
}
