
#region Using directives

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;

using Microsoft.DirectX.Direct3D;

using JadEngine.Tools;
using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Scene;
using JadEngine.VFS;

#endregion

/*
 * 06/04/08
 *	-Made effect file saving dependent on whether it's the DEBUG build to speed up the release build
 * 14/04/08
 *	-Destroy() renamed to Cleanup()
 * 21.05.08
 *	-CHecks added for material.MapsEnabled
 */

namespace JadEngine.Core
{
	/// <summary>
	/// Values to identify the shader
	/// </summary>
	public enum JUniqueEffectIDGlobal
	{
		/// <summary>
		/// TODO
		/// </summary>
		FogPass = 1 << 0,

		/// <summary>
		/// TODO
		/// </summary>
		AffectDiffuse = 1 << 1,

		/// <summary>
		/// TODO
		/// </summary>
		AffectSpecular = 1 << 2,

		/// <summary>
		/// TODO
		/// </summary>
		ProjectorEnabled = 1 << 3,

		/// <summary>
		/// TODO
		/// </summary>
		ReflectionMapEnabled = 1 << 4,

		/// <summary>
		/// TODO
		/// </summary>
		RefractionMapEnabled = 1 << 5,

		/// <summary>
		/// TODO
		/// </summary>
		WorldSpaceNormals = 1 << 6,

		/// <summary>
		/// TODO
		/// </summary>
		ParallaxMapping = 1 << 7,

		/// <summary>
		/// TODO
		/// </summary>
		EmissiveEnabled = 1 << 8,

		/// <summary>
		/// TODO
		/// </summary>
		DetailMapEnabled = 1 << 10,

		/// <summary>
		/// TODO
		/// </summary>
		DepthOfField = 1 << 11,

		/// <summary>
		/// TODO
		/// </summary>
		SpecularMapEnabled = 1 << 12,

		/// <summary>
		/// TODO
		/// </summary>
		ShadowsOnRender = 1 << 13,

		/// <summary>
		/// TODO
		/// </summary>
		GeneratingShadowMap = 1 << 14,

		/// <summary>
		/// TODO
		/// </summary>
		GeneratingShadowMaskForShadowMap = 1 << 15,

		/// <summary>
		/// TODO
		/// </summary>
		GeneratingShadowMaskForShadowMapPCF = 1 << 16,

		/// <summary>
		/// TODO
		/// </summary>
		UseTexCoordForDetailMap = 1 << 18,

		/// <summary>
		/// TODO
		/// </summary>
		DiffuseMapEnabled = 1 << 19,

		/// <summary>
		/// TODO
		/// </summary>
		VolumetricLight = 1 << 20,

		/// <summary>
		/// TODO
		/// </summary>
		ZPass_WithoutAlphaTest = 1 << 21,

		/// <summary>
		/// TODO
		/// </summary>
		ZPass_WithAlphaTest = 1 << 22,

		/// <summary>
		/// TODO
		/// </summary>
		Ghost = 1 << 23,

		/// <summary>
		/// TODO
		/// </summary>
		PostPass = 1 << 24,

		/// <summary>
		/// TODO
		/// </summary>
		ApplyAmbient = 1 << 25,

		/// <summary>
		/// TODO
		/// </summary>
		AmbientOnly = 1 << 26,

		/// <summary>
		/// TODO
		/// </summary>
		MotionBlurPass = 1 << 27,

		/// <summary>
		/// TODO
		/// </summary>
		ParallaxPass = 1 << 28,

		/// <summary>
		/// TODO
		/// </summary>
		SphericalMapping = 1 << 29,

		/// <summary>
		/// TODO
		/// </summary>
		ParallaxOcclusion = 1 << 30,

		/// <summary>
		/// TODO
		/// </summary>
		ParallaxShadowOnlyPass = 1 << 31
	}

	/// <summary>
	/// structure used to encapsultate the fields necessary to identify the effect
	/// </summary>
	public struct JUniqueEffectID
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public int Global;

		/// <summary>
		/// TODO
		/// </summary>
		public byte NormalType;

		/// <summary>
		/// TODO
		/// </summary>
		public byte LightType;

		/// <summary>
		/// TODO
		/// </summary>
		public byte EmissiveModel;

		/// <summary>
		/// TODO
		/// </summary>
		public byte Attenuation;

		/// <summary>
		/// TODO
		/// </summary>
		public byte Fog;

		/// <summary>
		/// TODO
		/// </summary>
		public Int16 LightingID;

		/// <summary>
		/// TODO
		/// </summary>
		/// <remarks>
		/// Sadly the HashTable DOESNT support arrays, so all the posibilites
		/// must be built as a string
		/// </remarks>
		public string GeometryModifiersID;

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		public void Reset()
		{
			Global = 0;
			NormalType = (byte) JNormalType.None;    // Valor 0
			LightType = (byte) JLightType.Unknown;	// valor 3
			EmissiveModel = 0;
			Attenuation = (byte) JLightAttenuationType.None;
			LightingID = 0;
			GeometryModifiersID = null;
			Fog = 0;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public override string ToString()
		{
			string t = "/*\n\nScene status:";

			if (Jad.ActiveScene.Status == JSceneStatus.ZPass) t += "ZPass";
			if (Jad.ActiveScene.Status == JSceneStatus.ParallaxPass) t += "ParallaxPass";
			if (Jad.ActiveScene.Status == JSceneStatus.PostPass) t += "PostPass";
			if (Jad.ActiveScene.Status == JSceneStatus.FogPass) t += "FogPass";
			if (Jad.ActiveScene.Status == JSceneStatus.GlowDOFPass) t += "GlowDOFPass";
			if (Jad.ActiveScene.Status == JSceneStatus.MotionBlurPass) t += "MotionBlurPass";
			if (Jad.ActiveScene.Status == JSceneStatus.ParallaxShadowPass) t += "ParallaxShadowPass";

			if (Jad.ActiveScene.Status == JSceneStatus.Rendering)
			{
				if (Jad.ActiveScene.RenderWithShadows)
					t += "RenderWithShadow";
				else
					t += "Rendering";
			}

			if (Jad.ActiveScene.Status == JSceneStatus.RenderShadow) t += "RenderShadow";
			if (Jad.ActiveScene.Status == JSceneStatus.ShadowMapGeneration) t += "ShadowMapGeneration";

			t += "\n\n";

			if ((Global & (int) JUniqueEffectIDGlobal.FogPass) != 0) t = t + "FogPass\n";
			if ((Global & (int) JUniqueEffectIDGlobal.AffectDiffuse) != 0) t = t + "AffectDiffuse\n";
			if ((Global & (int) JUniqueEffectIDGlobal.AffectSpecular) != 0) t = t + "AffectSpecular\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ReflectionMapEnabled) != 0) t = t + "ReflectionMapEnabled\n";
			if ((Global & (int) JUniqueEffectIDGlobal.RefractionMapEnabled) != 0) t = t + "RefractionMapEnabled\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ParallaxMapping) != 0) t = t + "ParallaxMapping\n";
			if ((Global & (int) JUniqueEffectIDGlobal.EmissiveEnabled) != 0) t = t + "EmissiveEnabled\n";
			if ((Global & (int) JUniqueEffectIDGlobal.DepthOfField) != 0) t = t + "DepthOfField\n";
			if ((Global & (int) JUniqueEffectIDGlobal.SpecularMapEnabled) != 0) t = t + "SpecularMapEnabled\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ShadowsOnRender) != 0) t = t + "ShadowsOnRender\n";
			if ((Global & (int) JUniqueEffectIDGlobal.GeneratingShadowMap) != 0) t = t + "GeneratingShadowMap\n";
			if ((Global & (int) JUniqueEffectIDGlobal.GeneratingShadowMaskForShadowMap) != 0) t = t + "GeneratingShadowMaskForShadowMap\n";
			if ((Global & (int) JUniqueEffectIDGlobal.GeneratingShadowMaskForShadowMapPCF) != 0) t = t + "GeneratingShadowMaskForShadowMapPCF\n";
			if ((Global & (int) JUniqueEffectIDGlobal.DiffuseMapEnabled) != 0) t = t + "DiffuseMapEnabled\n";
			if ((Global & (int) JUniqueEffectIDGlobal.VolumetricLight) != 0) t = t + "VolumetricLight\n";
			if ((Global & (int) JUniqueEffectIDGlobal.DetailMapEnabled) != 0) t = t + "DetailMapEnabled\n";
			if ((Global & (int) JUniqueEffectIDGlobal.UseTexCoordForDetailMap) != 0) t = t + "UseTexCoordForDetailMap\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ZPass_WithoutAlphaTest) != 0) t = t + "ZPass_WithoutAlphaTest\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ZPass_WithAlphaTest) != 0) t = t + "ZPass_WithAlphaTest\n";
			if ((Global & (int) JUniqueEffectIDGlobal.Ghost) != 0) t = t + "Ghost\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ApplyAmbient) != 0) t = t + "ApplyAmbient\n";
			if ((Global & (int) JUniqueEffectIDGlobal.AmbientOnly) != 0) t = t + "AmbientOnly\n";
			if ((Global & (int) JUniqueEffectIDGlobal.WorldSpaceNormals) != 0) t = t + "WorldSpaceNormals\n";
			if ((Global & (int) JUniqueEffectIDGlobal.MotionBlurPass) != 0) t = t + "MotionBlurPass\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ParallaxOcclusion) != 0) t = t + "ParallaxOcclusion\n";
			if ((Global & (int) JUniqueEffectIDGlobal.ParallaxPass) != 0) t = t + "ParallaxPass\n";

			string normal = String.Empty, light = String.Empty;

			if (NormalType == (byte) JNormalType.None) normal = "None";
			if (NormalType == (byte) JNormalType.Mapping) normal = "Mapping";
			if (NormalType == (byte) JNormalType.Vertex) normal = "Vertex";

			t += "\n\nNormal Type:" + normal;

			t += "\n\nFog:" + Fog.ToString();

			if (LightType == (byte) JLightType.Unknown) light = "Unknown";
			if (LightType == (byte) JLightType.Directional) light = "Directional";
			if (LightType == (byte) JLightType.Omni) light = "Omni";
			if (LightType == (byte) JLightType.Spot) light = "Spot";

			t += "\nLight Type:" + light;
			t += "\nlightingID:" + LightingID.ToString();

			t += "\nAttenuation Type:" + ((JLightAttenuationType) Attenuation).ToString();
			t += "\ngeometryModifiersID:" + GeometryModifiersID;

			string emissive = String.Empty;

			if (EmissiveModel == (byte) JEmissiveMode.Texture) emissive = "Texture";
			if (EmissiveModel == (byte) JEmissiveMode.Color) emissive = "Color";
			if (EmissiveModel == (byte) JEmissiveMode.Diffuse) emissive = "Diffuse";

			t += "\n\nEmissive Model:" + emissive;

			t += "\n\n************************************************************/\n\n";

			return t;
		}

		#endregion
	}

	/// <summary>
	/// This class has the function to construct the final HLSL shader 
	/// </summary>
	public class JEffectBuilder
	{
		#region Struct JBuilderParams

		/// <summary>
		/// TODO
		/// </summary>
		struct JBuilderParams
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			internal string FogPass;

			/// <summary>
			/// TODO
			/// </summary>
			internal string GlowDOFPass;

			/// <summary>
			/// TODO
			/// </summary>
			internal string lightType;

			/// <summary>
			/// TODO
			/// </summary>
			internal string affectDiffuse;

			/// <summary>
			/// TODO
			/// </summary>
			internal string affectSpecular;

			/// <summary>
			/// TODO
			/// </summary>
			internal string projectorEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string reflectionMapEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string refractionMapEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string specularMapEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string diffuseMapEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string additiveEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string emissiveEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string emissiveMapEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string emissiveModel;

			/// <summary>
			/// TODO
			/// </summary>
			internal string normalType;

			/// <summary>
			/// TODO
			/// </summary>
			internal string volumetricLight;

			/// <summary>
			/// TODO
			/// </summary>
			internal string parallaxMapping;

			/// <summary>
			/// TODO
			/// </summary>
			internal string DOF;

			/// <summary>
			/// TODO
			/// </summary>
			internal string shadowsOnRender;

			/// <summary>
			/// TODO
			/// </summary>
			internal string generatingShadowMap;

			/// <summary>
			/// TODO
			/// </summary>
			internal string generatingShadowMaskForShadowMap;

			/// <summary>
			/// TODO
			/// </summary>
			internal string fog;

			/// <summary>
			/// TODO
			/// </summary>
			internal string detailMapEnabled;

			/// <summary>
			/// TODO
			/// </summary>
			internal string useTexCoordForDetailMap;

			/// <summary>
			/// TODO
			/// </summary>
			internal string zPass;

			/// <summary>
			/// TODO
			/// </summary>
			internal string ghost;

			/// <summary>
			/// TODO
			/// </summary>
			internal string applyAmbient;

			/// <summary>
			/// TODO
			/// </summary>
			internal string ambientOnly;

			/// <summary>
			/// TODO
			/// </summary>
			internal string sphericalMapping;

			/// <summary>
			/// TODO
			/// </summary>
			internal string parallaxOcclusion;

			/// <summary>
			/// TODO
			/// </summary>
			internal string parallaxPass;

			/// <summary>
			/// TODO
			/// </summary>
			internal string parallaxShadowOnlyPass;

			#endregion

			#region Methods

			/// <summary>
			/// TODO
			/// </summary>
			internal void Initialize()
			{
				// Here we define the uniforms
				FogPass = "false";
				GlowDOFPass = "false";
				lightType = "LIGHT_UNKNOWN";
				affectDiffuse = "false";
				affectSpecular = "false";
				projectorEnabled = "false";
				reflectionMapEnabled = "false";
				refractionMapEnabled = "false";
				specularMapEnabled = "false";
				diffuseMapEnabled = "false";
				additiveEnabled = "false";
				emissiveEnabled = "false";
				emissiveMapEnabled = "false";
				emissiveModel = "EMISSIVEMODEL_DIFFUSE";
				normalType = "NORMAL_NONE";
				volumetricLight = "false";
				parallaxMapping = "false";
				DOF = "false";
				shadowsOnRender = "false";
				generatingShadowMap = "false";
				generatingShadowMaskForShadowMap = "0";
				fog = "0";
				detailMapEnabled = "false";
				useTexCoordForDetailMap = "false";
				zPass = "ZPASS_NONE";
				ghost = "false";
				applyAmbient = "false";
				ambientOnly = "false";
				sphericalMapping = "false";
				parallaxOcclusion = "false";
				parallaxPass = "false";
				parallaxShadowOnlyPass = "false";
			}

			/// <summary>
			/// TODO
			/// </summary>
			/// <returns>TODO</returns>
			public string Process()
			{
				//TODO: Use a StringBuilder here to concatenate the strings
				string Uniform = "\n";

				Uniform += "#define Uniform_PixelShaderVersion " + PixelShaderVersionInteger + " \n";

				//  This is deprecated, and must de discarded and the effects modified
				if (Jad.Video.RenderTargets == null)
					Uniform += "#define Uniform_NORMALRTFLOATINGPOINT false \n";

				else
					Uniform += "#define Uniform_NORMALRTFLOATINGPOINT false \n";

				Uniform += "#define Uniform_FogPass " + FogPass + "\n";
				Uniform += "#define Uniform_GlowDOFPass " + GlowDOFPass + "\n";
				Uniform += "#define Uniform_lightType " + lightType + "\n";
				Uniform += "#define Uniform_affectDiffuse " + affectDiffuse + "\n";
				Uniform += "#define Uniform_affectSpecular " + affectSpecular + "\n";
				Uniform += "#define Uniform_projectorEnabled " + projectorEnabled + "\n";
				Uniform += "#define Uniform_reflectionMapEnabled " + reflectionMapEnabled + "\n";
				Uniform += "#define Uniform_refractionMapEnabled " + refractionMapEnabled + "\n";
				Uniform += "#define Uniform_diffuseMapEnabled " + diffuseMapEnabled + "\n";
				Uniform += "#define Uniform_specularMapEnabled " + specularMapEnabled + "\n";
				Uniform += "#define Uniform_additiveEnabled " + additiveEnabled + "\n";
				Uniform += "#define Uniform_emissiveEnabled " + emissiveEnabled + "\n";
				Uniform += "#define Uniform_emissiveMapEnabled " + emissiveMapEnabled + "\n";
				Uniform += "#define Uniform_emissiveModel " + emissiveModel + "\n";
				Uniform += "#define Uniform_normalType " + normalType + "\n";
				Uniform += "#define Uniform_volumetricLight " + volumetricLight + "\n";
				Uniform += "#define Uniform_parallaxMapping " + parallaxMapping + "\n";
				Uniform += "#define Uniform_shadowsOnRender " + shadowsOnRender + "\n";
				Uniform += "#define Uniform_DOF " + DOF + "\n";
				Uniform += "#define Uniform_GeneratingShadowMap " + generatingShadowMap + "\n";
				Uniform += "#define Uniform_GeneratingShadowMaskForShadowMap " + generatingShadowMaskForShadowMap + "\n";
				Uniform += "#define Uniform_Fog " + fog + "\n";
				Uniform += "#define Uniform_DetailMapEnabled " + detailMapEnabled + "\n";
				Uniform += "#define Uniform_useTexCoordForDetailMap " + useTexCoordForDetailMap + "\n";
				Uniform += "#define Uniform_ZPass " + zPass + "\n";
				Uniform += "#define Uniform_Ghost " + ghost + "\n";
				Uniform += "#define Uniform_ApplyAmbient " + applyAmbient + "\n";
				Uniform += "#define Uniform_AmbientOnly " + ambientOnly + "\n";
				Uniform += "#define Uniform_WorldSpaceNormals " + WorldSpaceNormals + "\n";
				Uniform += "#define Uniform_SphericalMapping " + sphericalMapping + "\n";
				Uniform += "#define Uniform_ParallaxOcclusion " + parallaxOcclusion + "\n";
				Uniform += "#define Uniform_ParallaxPass " + parallaxPass + "\n";
				Uniform += "#define Uniform_ParallaxShadowOnlyPass " + parallaxShadowOnlyPass + "\n";

				Uniform += "\n\n";

				return Uniform;
			}

			#endregion
		}

		#endregion

		#region Static Fields

		/// <summary>
		/// Vertex shader version that will be created the effect
		/// </summary>
		public static string VertexShaderVersion = "2_0";

		/// <summary>
		/// Pixel shader version that will be created the effect
		/// </summary>
		public static string PixelShaderVersion = "2_0";

		/// <summary>
		/// Pixel shader version used by the effect builder
		/// </summary>
		public static int PixelShaderVersionInteger = 20;

		/// <summary>
		/// TODO
		/// </summary>
		public static string WorldSpaceNormals = "false";

		/// <summary>
		/// TODO
		/// </summary>
		public static string Test = "false";

		/// <summary>
		/// TODO
		/// </summary>
		public static int NumberOfEffectsBuilt = 0;

		/// <summary>
		/// TODO
		/// </summary>
		static int numPreShader = 0;

		static int GlobalID = 0;

		#endregion

		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		JBuilderParams parameters;

		/// <summary>
		/// TODO
		/// </summary>
		StreamReader stream = null;

		/// <summary>
		/// TODO
		/// </summary>
		string allInOneFX;

		/// <summary>
		/// Used for the engine to build the effects
		/// </summary>
		internal string allInOneFXForEffect;

		/// <summary>
		/// TODO
		/// </summary>
		string lightPassFx;

		/// <summary>
		/// TODO
		/// </summary>
		string zPassFx;

		/// <summary>
		/// TODO
		/// </summary>
		string fogPassFx;

		/// <summary>
		/// TODO
		/// </summary>
		string glowDOFPassFx;

		/// <summary>
		/// TODO
		/// </summary>
		string postPassFx;

		/// <summary>
		/// TODO
		/// </summary>
		string motionBlurPassFx;

		/// <summary>
		/// TODO
		/// </summary>
		string source;

		/// <summary>
		/// TODO
		/// </summary>
		JUniqueEffectID UniqueEffectID;

		/// <summary>
		/// TODO
		/// </summary>
		Dictionary<String, JEffect> effects = new Dictionary<String, JEffect>();

		/// <summary>
		/// TODO
		/// </summary>
		public bool SavePreShaders = false;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JEffectBuilder()
		{
			JFilesSource runtime;
			Collection<String> files;

			runtime = Jad.VFS.Source("Runtime");

			// Read all the effect and save it in the fx var
			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "AllInOne.fx", true));
			allInOneFX = stream.ReadToEnd() + "\n";
			stream.Close();

			// The samplers just afte the AllInOne
			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "Samplers.fx", true));
			allInOneFX += stream.ReadToEnd() + "\n";
			stream.Close();

			// Delete the Enum region from the AllInOne
			int psEnd = allInOneFX.IndexOf("#end Enum");
			JEffectFragmentManager.ParseEnum(Jad.EffectFragments.EnumValues, allInOneFX.Substring(0, psEnd));
			allInOneFX = allInOneFX.Substring(psEnd + 9);

			// Search the *.h files in the shaders folder to include them in the AllInOne
			files = runtime.GetFilesFromDefinedPath("Shaders", false, "*.h");
			foreach (String qualifiedFileName in files)
			{
				// Read all the effect and save it in the fx var
				stream = new StreamReader(runtime.GetFile(qualifiedFileName));
				allInOneFX += stream.ReadToEnd() + "\n";
				stream.Close();
			}

			// Add the enums at the start
			string enums = JEffectFragmentManager.EnumToDefines(Jad.EffectFragments.EnumValues);
			foreach (JEffectFragment f in Jad.EffectFragments.Items)
				if (f.EnumValues != null)
					enums += JEffectFragmentManager.EnumToDefines(f.EnumValues);

			allInOneFX = enums + allInOneFX;

			parameters.Initialize();

			// Create the AllInOne that will be used in the InHouse effects

			string uniformDefines = parameters.Process();
			allInOneFXForEffect = uniformDefines + allInOneFX;

			// Take out the samplers, as its more logical that the effect defines them

			string temp;

			temp = allInOneFXForEffect.Substring(0, allInOneFXForEffect.IndexOf("//SamplersStart"));
			temp += allInOneFXForEffect.Substring(allInOneFXForEffect.IndexOf("//SamplersEnd"));
			allInOneFXForEffect = temp;

			// Read the vertex and pixel shaders and save them

			// ------------------------------------------------------------------------
			// ZPass
			// ------------------------------------------------------------------------

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "ZPass_VertexShader.fx", true));
			zPassFx = stream.ReadToEnd() + "\n";
			stream.Close();

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "ZPass_PixelShader.fx", true));
			zPassFx += stream.ReadToEnd() + "\n";
			stream.Close();

			zPassFx = allInOneFX + zPassFx;

			// ------------------------------------------------------------------------
			// FogPass
			// ------------------------------------------------------------------------

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "FogPass_VertexShader.fx", true));
			fogPassFx = stream.ReadToEnd() + "\n";
			stream.Close();

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "FogPass_PixelShader.fx", true));
			fogPassFx += stream.ReadToEnd() + "\n";
			stream.Close();

			fogPassFx = allInOneFX + fogPassFx;

			// ------------------------------------------------------------------------
			// GlowDOFPass
			// ------------------------------------------------------------------------

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "GlowDOFPass_VertexShader.fx", true));
			glowDOFPassFx = stream.ReadToEnd() + "\n";
			stream.Close();

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "GlowDOFPass_PixelShader.fx", true));
			glowDOFPassFx += stream.ReadToEnd() + "\n";
			stream.Close();

			glowDOFPassFx = allInOneFX + glowDOFPassFx;

			// ------------------------------------------------------------------------
			// MotionBlurPass
			// ------------------------------------------------------------------------

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "MotionBlurPass_VertexShader.fx", true));
			motionBlurPassFx = stream.ReadToEnd() + "\n";
			stream.Close();

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "MotionBlurPass_PixelShader.fx", true));
			motionBlurPassFx += stream.ReadToEnd() + "\n";
			stream.Close();

			motionBlurPassFx = allInOneFX + motionBlurPassFx;

			// ------------------------------------------------------------------------
			// LightPass
			// ------------------------------------------------------------------------

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "LightPass_VertexShader.fx", true));
			lightPassFx = stream.ReadToEnd() + "\n";
			stream.Close();

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "LightPass_PixelShader.fx", true));
			lightPassFx += stream.ReadToEnd() + "\n";
			stream.Close();

			lightPassFx = allInOneFX + lightPassFx;

			// ------------------------------------------------------------------------
			// PostPass
			// ------------------------------------------------------------------------

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "PostPass_VertexShader.fx", true));
			postPassFx = stream.ReadToEnd() + "\n";
			stream.Close();

			stream = new StreamReader(runtime.GetFileFromDefinedPath("Internal", "PostPass_PixelShader.fx", true));
			postPassFx += stream.ReadToEnd() + "\n";
			stream.Close();

			postPassFx = allInOneFX + postPassFx;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Clear the effects
		/// </summary>
		public void Cleanup()
		{
			effects.Clear();
		}

		/// <summary>
		/// Set the geometry shader fragments
		/// </summary>
		/// <param name="material">The material layer</param>
		/// <param name="pass">The shader used for this pass</param>
		void BuildGeometry(JMaterialLayer material, string pass)
		{
			// Initializa the local vars

			// In the end, "source" has the shader
			source = String.Empty;

			// Initialize the parameters
			parameters.Initialize();

			string geometry = String.Empty, ps = String.Empty;

			source = pass;

			// Assign the geometry fragments

			if (material.GeometryFragments.Count > 0)
			{
				foreach (JEffectFragment f in material.GeometryFragments)
				{
					geometry = geometry + f.VS;

					if (f.PS != null)
						ps += f.PS;

					if (f.Vars != null)
						source = f.Vars + source;
				}
			}

			if (material.Parent.GeometryFragments.Count > 0)
			{
				foreach (JEffectFragment f in material.Parent.GeometryFragments)
				{
					geometry = geometry + f.VS;

					if (f.PS != null)
						ps += f.PS;

					if (f.Vars != null)
						source = f.Vars + source;
				}
			}

			source = source.Replace("// ALLINONE:Geometry", geometry);
			source = source.Replace("// ALLINONE:PS_GEOMETRY", ps);
		}

		/// <summary>
		/// Creates the <see cref="JEffect"/> and save it to the disk for debugging purposes
		/// </summary>
		/// <returns>A valid JEffect</returns>
		JEffect Process()
		{
			source = parameters.Process() + source;

			// If we are usign Software Vertex, we need a compatible FVF. The FVF does not support
			// TANGENT, so we should use TEXCOORD(1) instead of TANGENT
			// So we must change the vertex description on the shader too
			if (Jad.Video.Adapter.Information.UseVertexSoftwareProcessing)
			{
				source=source.Replace(": TANGENT0;", ": TEXCOORD1;");
			}

			source += "technique RenderScene\n" +
					  "{\n" +
					  "\tpass p0\n" +
					  "\t{\n" +
					  "\t\tVertexShader = compile vs_" + JEffectBuilder.VertexShaderVersion + " VShader();\n" +
					  "\t\tPixelShader = compile ps_" + JEffectBuilder.PixelShaderVersion + " PShader();\n" +
					  "\t}\n" +
					  "}";

			JEffect j;

			// Create a unique name
			++GlobalID;

			string name = "Builder #" + GlobalID;

			// If we have the VideoRecorder activated, do not save the Effects to disk
			if (Jad.Video.Views.ActiveView.VideoRecorder.Enabled)
			{
				Effect effect = Effect.FromString(Jad.Video.Device, source, null, null, ShaderFlags.None, null);

				j = new JEffect(String.Empty, Jad.Video.Effects);
				j.DXResource = effect;
				j.GetHandles();
			}

			else
			{
#if DEBUG
				Stream stream;
				StreamWriter writer;

				stream = Jad.VFS.WritableSource("Build").CreateWritableFileOnDefinedPath("Effects", "internal.fx");

				writer = new StreamWriter(stream);
				writer.Write(source);
				writer.Close();

				Save(source);							
#endif
				j = Jad.Video.Effects.Create(source);
			}

			return j;
		}

		/// <summary>
		/// Save the effect for debugging purposes
		/// </summary>
		/// <param name="sourceText">The effect text</param>
		void Save(string sourceText)
		{
			Stream stream;
			StreamWriter writer;

			stream = Jad.VFS.WritableSource("Build").CreateWritableFileOnDefinedPath("Effects", numPreShader.ToString("D4") + ".fx");

			numPreShader++;

			writer = new StreamWriter(stream);
			writer.Write(UniqueEffectID.ToString());
			writer.Write(sourceText);
			writer.Close();
		}

		#endregion

		#region ZPass

		/// <summary>
		/// Creates the effect for the ZPass (and the shadowmap pass)
		/// </summary>
		/// <param name="material">TODO</param>
		/// <param name="shadowMap">TODO</param>
		/// <returns>The correct JEffect according to the parameters</returns>
		public JEffect SetZPass(JMaterialLayer material, bool shadowMap)
		{
			String hash;

			// Create the unique index that identifies this shader
			UniqueEffectID.Reset();

			material.BuildHashCodeZPass(ref UniqueEffectID);

			if (Jad.ActiveScene.Status == JSceneStatus.ParallaxPass)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.ParallaxPass;

			if (Jad.ActiveScene.Status == JSceneStatus.ParallaxShadowPass)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.ParallaxShadowOnlyPass;

			if (shadowMap)
			{
				if (Jad.ActiveScene.Status == JSceneStatus.ShadowMapGeneration)
					UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.GeneratingShadowMap;

				else
				{
					if (Jad.ActiveScene.Shadows.ShadowMap.UsePCF)
						UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.GeneratingShadowMaskForShadowMapPCF;

					else
						UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.GeneratingShadowMaskForShadowMap;
				}
			}

			hash = UniqueEffectID.GeometryModifiersID + UniqueEffectID.Global;
			//if (!effects.TryGetValue(hash, out fx))
			//{
			//    fx = BuildZPass(material);

			//    effects.Add(hash, fx);
			//    NumberOfEffectsBuilt++;
			//    Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			//}

			//return fx;

			if (!effects.ContainsKey(hash))
			{
				JEffect fx = BuildZPass(material);

				effects.Add(hash, fx);
				NumberOfEffectsBuilt++;
				Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			}

			return effects[hash];
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>TODO</returns>
		JEffect BuildZPass(JMaterialLayer material)
		{
			BuildGeometry(material, zPassFx);

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ZPass_WithAlphaTest) != 0)
				parameters.zPass = "ZPASS_WITHALPHATEST";

			else
				parameters.zPass = "ZPASS_WITHOUTALPHATEST";

			parameters.diffuseMapEnabled = ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.DiffuseMapEnabled) != 0 ? "true" : "false");
			parameters.parallaxMapping = ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ParallaxMapping) != 0 ? "true" : "false");

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ParallaxOcclusion) != 0)
				parameters.parallaxOcclusion = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ParallaxPass) != 0)
				parameters.parallaxPass = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ParallaxShadowOnlyPass) != 0)
				parameters.parallaxShadowOnlyPass = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.DetailMapEnabled) != 0)
				parameters.detailMapEnabled = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.UseTexCoordForDetailMap) != 0)
				parameters.useTexCoordForDetailMap = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.GeneratingShadowMap) != 0)
				parameters.generatingShadowMap = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.GeneratingShadowMaskForShadowMapPCF) != 0)
				parameters.generatingShadowMaskForShadowMap = "2";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.GeneratingShadowMaskForShadowMap) != 0)
				parameters.generatingShadowMaskForShadowMap = "1";

			return Process();
		}

		#endregion

		#region FogPass

		/// <summary>
		/// Creates the effect according to the parameters
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>The correct JEffect according to the parameters</returns>
		public JEffect SetFogPass(JMaterialLayer material)
		{
			String hash;

			// Create the unique index that identifies this shader
			UniqueEffectID.Reset();

			material.BuildHashCodeGeometry(ref UniqueEffectID);

			UniqueEffectID.Global = (int) JUniqueEffectIDGlobal.FogPass;

			UniqueEffectID.Fog = (byte) Jad.ActiveScene.Fog.Type;

			hash = UniqueEffectID.GeometryModifiersID + UniqueEffectID.Global;
			if (!effects.ContainsKey(hash))
			{
				JEffect fx = BuildFogPass(material);
				effects.Add(hash, fx);
				NumberOfEffectsBuilt++;
				Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			}

			return effects[hash];
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>TODO</returns>
		JEffect BuildFogPass(JMaterialLayer material)
		{
			BuildGeometry(material, fogPassFx);

			parameters.FogPass = "true";
			parameters.fog = ((int) Jad.ActiveScene.Fog.Type).ToString();

			return Process();
		}

		#endregion

		#region GlowDOFPass

		/// <summary>
		/// Creates the effect according to the parameters
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>The correct JEffect according to the parameters</returns>
		public JEffect SetGlowDOFPass(JMaterialLayer material)
		{
			String hash;

			// Create the unique index that identifies this shader
			UniqueEffectID.Reset();

			material.BuildHashCodeGlowDOFPass(ref UniqueEffectID);

			hash = UniqueEffectID.GeometryModifiersID + UniqueEffectID.Global;
			if (!effects.ContainsKey(hash))
			{
				JEffect fx = BuildGlowDOFPass(material);

				effects.Add(hash, fx);
				NumberOfEffectsBuilt++;
				Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			}

			return effects[hash];
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>TODO</returns>
		JEffect BuildGlowDOFPass(JMaterialLayer material)
		{
			BuildGeometry(material, glowDOFPassFx);

			if (Jad.ActiveScene.GlowEffect.Enabled)
			{
				parameters.emissiveEnabled = "true";
				parameters.emissiveModel = UniqueEffectID.EmissiveModel.ToString();
			}

			if (Jad.ActiveScene.DepthOfField.Enabled)
				parameters.DOF = "true";

			return Process();
		}

		#endregion

		#region MotionBlurPass

		/// <summary>
		/// Creates the effect according to the parameters
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>The correct JEffect according to the parameters</returns>
		public JEffect SetMotionBlurPass(JMaterialLayer material)
		{
			String hash;

			// Create the unique index that identifies this shader
			UniqueEffectID.Reset();

			material.BuildHashCodeGeometry(ref UniqueEffectID);

			UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.MotionBlurPass;

			hash = UniqueEffectID.GeometryModifiersID + UniqueEffectID.Global;
			if (!effects.ContainsKey(hash))
			{
				JEffect fx = BuildMotionBlurPass(material);

				effects.Add(hash, fx);
				NumberOfEffectsBuilt++;
				Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			}

			return effects[hash];
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="material">TODO</param>
		/// <returns>TODO</returns>
		JEffect BuildMotionBlurPass(JMaterialLayer material)
		{
			BuildGeometry(material, motionBlurPassFx);

			return Process();
		}

		#endregion

		#region LightPass

		/// <summary>
		/// Creates the effect according to the parameters
		/// </summary>
		/// <param name="light">The active light or null for ambient pass</param>
		/// <param name="material">Current material layer</param>
		/// <param name="applyAmbient">True to apply ambient. This way the first light pass can apply light+ambient with only one pass</param>
		/// <returns>The correct JEffect according to the parameters</returns>
		public JEffect SetLightPass(JBaseLight light, JMaterialLayer material, bool applyAmbient)
		{
			String hash;

			// Create the unique index that identifies this shader
			UniqueEffectID.Reset();

			material.BuildHashCodeLightPass(ref UniqueEffectID);

			bool diffuse = true;
			bool specular = true; //material.SpecularMap.Enabled;

			if (light != null)
			{
				diffuse = diffuse && light.AffectDiffuse;
				specular = specular && light.AffectSpecular;

				light.BuildHashCode(ref UniqueEffectID);
			}

			else
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.AmbientOnly;

			// Test if its needed to apply the ambient light and if the color is not 0
			// (if its 0 then no need to apply it)
			if (applyAmbient && !Jad.ActiveScene.Lights.Ambient.IsBlack())
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.ApplyAmbient;

			// If the normal map is disabled, change to per vertex normals
			if (!material.MapsEnabled || !material.NormalMap.Enabled && material.NormalType == JNormalType.Mapping)
				UniqueEffectID.NormalType = (byte) JNormalType.Vertex;

			else
				UniqueEffectID.NormalType = (byte) material.NormalType;

			// Test if the normals are in WorldSpace or it these maps are enabled...
			if (material.MapsEnabled && (material.Parent.WorldSpaceNormals || material.ReflectionMap.Enabled || material.RefractionMap.Enabled))
				JEffectBuilder.WorldSpaceNormals = "true";

			else
				JEffectBuilder.WorldSpaceNormals = "false";

			// Test if true was set
			if (JEffectBuilder.WorldSpaceNormals[0] == 't')
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.WorldSpaceNormals;

			// If rendering with shadows...
			if (Jad.ActiveScene.RenderWithShadows)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.ShadowsOnRender;

			if (diffuse)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.AffectDiffuse;

			if (specular)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.AffectSpecular;

			hash = UniqueEffectID.GeometryModifiersID + UniqueEffectID.Global;
			if (!effects.ContainsKey(hash))
			{
				JEffect fx = BuildLightPass(light, material);

				effects.Add(hash, fx);
				NumberOfEffectsBuilt++;
				Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			}

			return effects[hash];
		}

		/// <summary>
		/// Build the light pass shader
		/// </summary>
		/// <param name="light">Active light (or null for ambient pass)</param>
		/// <param name="material">Current Material Layer</param>
		/// <returns>A valid JEffect</returns>
		JEffect BuildLightPass(JBaseLight light, JMaterialLayer material)
		{
			BuildGeometry(material, lightPassFx);

			if (light != null)
				source = source.Replace("// ALLINONE:LIGHT SHADER", light.ShaderForAttenuation());
			else
				source = source.Replace("// ALLINONE:LIGHT SHADER", "float GetAttenuation(float3 wLight, float3 nLight) { return 1.0f;}\n");

			// Assign the lighting fragments
			source = source.Replace("// ALLINONE:LIGHTING_PS", material.LightingFragment.PS);
			source = source.Replace("// ALLINONE:LIGHTING", material.LightingFragment.VS);

			// If the lightingfragment has vars, add them
			if (material.LightingFragment.Vars != null)
				source = material.LightingFragment.Vars + source;

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.AffectDiffuse) != 0) parameters.affectDiffuse = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.AffectSpecular) != 0) parameters.affectSpecular = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.DiffuseMapEnabled) != 0) parameters.diffuseMapEnabled = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.SpecularMapEnabled) != 0) parameters.specularMapEnabled = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.DetailMapEnabled) != 0) parameters.detailMapEnabled = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ParallaxMapping) != 0) parameters.parallaxMapping = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ParallaxOcclusion) != 0) parameters.parallaxOcclusion = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ReflectionMapEnabled) != 0) parameters.reflectionMapEnabled = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.RefractionMapEnabled) != 0) parameters.refractionMapEnabled = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.UseTexCoordForDetailMap) != 0) parameters.useTexCoordForDetailMap = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.ApplyAmbient) != 0) parameters.applyAmbient = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.AmbientOnly) != 0) parameters.ambientOnly = "true";
			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.SphericalMapping) != 0) parameters.sphericalMapping = "true";

			parameters.normalType = ((int) UniqueEffectID.NormalType).ToString();

			// If light == null then its the ambient pass
			if (light != null)
			{
				parameters.lightType = light.NameInShaders;

				if (light.Projector.Enabled)
					parameters.projectorEnabled = "true";

				parameters.volumetricLight = (light.Volumetric.Enabled ? "true" : "false");
				parameters.shadowsOnRender = (Jad.ActiveScene.RenderWithShadows ? "true" : "false");
			}

			return Process();
		}

		#endregion

		#region PostPass

		/// <summary>
		/// Creates the effect according to the parameters
		/// </summary>
		/// <param name="material">The material layer</param>
		/// <returns>The correct JEffect according to the parameters</returns>
		public JEffect SetPostPass(JMaterialLayer material)
		{
			String hash;

			// Create the unique index that identifies this shader
			UniqueEffectID.Reset();

			material.BuildHashCodeGeometry(ref UniqueEffectID);

			UniqueEffectID.Global = (int) JUniqueEffectIDGlobal.PostPass;

			if (material.Parent.SphericalMapping)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.SphericalMapping;

			// In the wireframe mode, use the material color
			if (material.MapsEnabled)
			{
				if (material.DiffuseMap.Enabled && material.DiffuseMap.Texture != null)
					UniqueEffectID.Global |= (int)JUniqueEffectIDGlobal.DiffuseMapEnabled;
			}

			if (material.Ghost)
				UniqueEffectID.Global |= (int) JUniqueEffectIDGlobal.Ghost;

			hash = UniqueEffectID.GeometryModifiersID + UniqueEffectID.Global;
			if (!effects.ContainsKey(hash))
			{
				JEffect fx = BuildPostPass(material);

				effects.Add(hash, fx);
				NumberOfEffectsBuilt++;
				Jad.LogWriter.WriteLine("Effect Built #:" + NumberOfEffectsBuilt.ToString());
			}

			return effects[hash];
		}

		/// <summary>
		/// Builds the post pass
		/// </summary>
		/// <param name="material">Current material layer</param>
		/// <returns>A valid JEffect</returns>
		JEffect BuildPostPass(JMaterialLayer material)
		{
			BuildGeometry(material, postPassFx);

			parameters.diffuseMapEnabled = ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.DiffuseMapEnabled) != 0 ? "true" : "false");

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.SphericalMapping) != 0)
				parameters.sphericalMapping = "true";

			if ((UniqueEffectID.Global & (int) JUniqueEffectIDGlobal.Ghost) != 0)
				parameters.ghost = "true";

			return Process();
		}

		#endregion
	}
}
