#region Using directives

using System;
using System.Collections.ObjectModel;
using System.IO;

using JadEngine.Core;
using JadEngine.Collections;

#endregion

/*
 * 24/03/08
 *	-Changed to derive from JEngineElementManager
 * 01/04/08
 *	-Renamed from JEffectFragments to JEffectFragmentManager
 */
namespace JadEngine.Core
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JEffectFragmentManager : JObjectManager<JEffectFragment>
	{
		#region Fields

		/// <summary>
		/// Maintains the vars list that are enumerated in AllInOne
		/// </summary>
		public JListString EnumValues = new JListString();

		/// <summary>
		/// TODO
		/// </summary>
		static int sourcepos = 0;		

		#endregion		

		#region Methods

		public JEffectFragment FindFirst(string name)
		{
			for (int i = 0; i < _items.Count; i++)
			{
				if (_items[i].Name == name)
					return _items[i];
			}

			return null;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="source">TODO</param>
		/// <returns>TODO</returns>
		static string ParseEnumGetLine(string source)
		{
			string r = string.Empty;

			for (; sourcepos < source.Length; sourcepos++)
			{
				if (source[sourcepos] == '\n')
				{
					sourcepos++;
					return r;
				}

				r += source[sourcepos];
			}

			return null;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="list">TODO</param>
		/// <param name="source">TODO</param>
		public static void ParseEnum(JListString list, string source)
		{
			// Read the lines
			sourcepos = 0;

			string line;

			while ((line = ParseEnumGetLine(source)) != null)
			{
				int bracket1 = line.IndexOf('{');
				if (bracket1 == -1) continue;

				int bracket2 = line.IndexOf('}');
				if (bracket2 == -1) continue;

				string name = line.Substring(0, bracket1 - 1).Trim();

				string cleanline = line.Substring(bracket1 + 1, bracket2 - bracket1 - 1);

				int numElements;

				string[] elements = cleanline.Split(',');

				numElements = elements.Length;

				JFragmentEnum frag = new JFragmentEnum(name, numElements);

				for (int c = 0; c < numElements; c++)
				{
					string key = elements[c].Trim();
					frag.Values[c] = key;
				}

				list.Add(name, frag);
			}
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="list">TODO</param>
		/// <returns>TODO</returns>
		public static string EnumToDefines(JListString list)
		{
			string r = String.Empty;

			foreach (JFragmentEnum f in list.List)
			{
				r += "// Enum values " + f.Name + "\n";

				for (int c = 0; c < f.Values.Length; c++)
				{
					// BeRSeRKeR: Ive changed f.Values[c] also to uppercase
					r += "#define " + f.Name.ToUpper() + "_" + f.Values[c].ToUpper() + " " + c.ToString() + "\n";
				}

				r += "\n";
			}

			return r;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="enumType">TODO</param>
		/// <param name="enumElement">TODO</param>
		/// <returns>TODO</returns>
		public static int EnumElementToInt(string enumType, string enumElement)
		{
			// Search the AllInOne.fx
			string define = EnumToDefines(Jad.EffectFragments.EnumValues);

			if (define.Contains(enumType))
			{
				string[] defineLines = define.Split('\n');

				foreach (string line in defineLines)
				{
					if (line.Contains(enumElement))
					{
						string[] defineTokens = line.Split(' ');
						return (int.Parse(defineTokens[defineTokens.Length - 1]));
					}
				}
			}

			else
			{
				// If its not in the AllInOne.fx, search in the fragment list of the shader
				foreach (JEffectFragment f in Jad.EffectFragments.Items)
				{
					if (f.EnumValues != null)
					{
						define = EnumToDefines(f.EnumValues);

						if (define.Contains(enumType))
						{
							string[] defineLines = define.Split('\n');

							foreach (string line in defineLines)
							{
								if (line.Contains(enumElement))
								{
									string[] defineTokens = line.Split(' ');
									return (int.Parse(defineTokens[defineTokens.Length - 1]));
								}
							}
						}
					}
				}
			}

			// Couldnt resolve the value of the enum element
			return -1;
		}

		/// <summary>
		/// Loads all the shaders located at the directories InHouse and Base
		/// </summary>
		internal void LoadAllShaders()
		{
			Collection<string> files;

			files = Jad.VFS.GetFilesFromDefinedPath("Lighting", true, "*.fx");
			foreach (string file in files)
				Create(file, Jad.VFS.GetFile(file));

			files = Jad.VFS.GetFilesFromDefinedPath("Geometry", true, "*.fx");
			foreach (string file in files)
				Create(file, Jad.VFS.GetFile(file));
		}

		/// <summary>
		/// Inserts a fragment in the fragments list using a resource
		/// </summary>
		/// <param name="name">Name of the object</param>
		/// <param name="stream">TODO</param>
		/// <returns>TODO</returns>
		public JEffectFragment Create(string name, Stream stream)
		{
			JEffectFragment fragment = new JEffectFragment(name, stream);

			_items.Add(fragment);

			return fragment;
		}

		#endregion

		public override JEffectFragment Create()
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public override void Add(JEffectFragment element)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public override void Delete(JEffectFragment element)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public override void Cleanup()
		{
			throw new Exception("The method or operation is not implemented.");
		}
	}
}
