using System;
namespace JadEngine
{
	public struct JId : IComparable, IEquatable<JId>
	{
		public static JId Null = new JId(-1);

		private int _id;

		public int Value
		{
			get { return _id; }
		}

		internal JId(int value)
		{
			_id = value;
		}

		#region IComparable Member

		public int CompareTo(object obj)
		{
			if (obj is JId)
			{
				JId jId = (JId)obj;
				return CompareTo(jId);
			}
			throw new ArgumentException("object is not a JId");
		}

		public int CopareTo(JId other)
		{
			return _id.CompareTo(other._id);
		}

		#endregion

		#region IEquatable<JId> Member

		public bool Equals(JId other)
		{
			return _id.Equals(other.Value);
		}

		#endregion

		public static bool operator ==(JId a, JId b)
		{
			// If both are null, or both are same instance, return true.
			if (System.Object.ReferenceEquals(a, b))
			{
				return true;
			}

			// If one is null, but not both, return false.
			if (((object)a == null) || ((object)b == null))
			{
				return false;
			}

			// Return true if the fields match:
			return a.Value == b.Value ;
		}

		public static bool operator !=(JId a, JId b)
		{
			return !(a == b);
		}

		public override string ToString()
		{
			return "ID_" + _id.ToString();
		}

	}
}
