#region Using directives

using System;
using System.Collections.Generic;
using System.IO;

using JadEngine.VFS;

#endregion

/*
 * 29/01/08
 *	-Initial cleanup
 */

namespace JadEngine.Core
{
	/// <summary>
	/// This class holds information on the paths for loading textures, scenes...
	/// </summary>
	/// <remarks>
	/// This class is obsolete. Use the new Virtual File System (VFS) instead.
	/// </remarks>
	public sealed class JPath
	{
		#region Structs

		/// <summary>
        /// Struct that contains the default directories.
		/// </summary>
		/// <remarks>
		/// Remember that the engine adds the base directory.
		/// </remarks>
		[Obsolete("Use the new VFS class to get access to files.")]
		public struct JDirectoriesPath
		{
			/// <summary>
			/// Directory for the models.
			/// </summary>
			public string Models;
			
			/// <summary>
			/// Directory for the textures.
			/// </summary>
			public string Textures;
			
			/// <summary>
			/// Directory for the shaders.
			/// </summary>
			public string Shaders;
			
			/// <summary>
			/// Directory for the materials.
			/// </summary>
			public string Materials;
			
			/// <summary>
			/// Directory for the scenes.
			/// </summary>
			public string Scenes;
		}
	
		#endregion

		#region Fields

		/// <summary>
		/// This is the base directory that will be added to all the archives.
		/// </summary>
		private static string baseDirectory = @"base\";

		/// <summary>
		/// The directory where the engines executes in.
		/// </summary>
		public static string GameDirectory;

		/// <summary>
		/// Resource directories (models, textures...).
		/// </summary>
		[Obsolete("Use the new VFS class to get access to files.")]
		public static JDirectoriesPath Directories;

		/// <summary>
		/// InHouse resource directory.
		/// </summary>
		[Obsolete("Use the new VFS class to get access to files.")]
		public static string InHouseDirectory = @"InHouse\";

		#endregion

        #region Constructors

		/// <summary>
		/// Initializes the <see cref="JPath"/> class.
		/// </summary>
        static JPath()
        {
			//Directories.Materials = "materials";
			//Directories.Models = "models";
			//Directories.Shaders = "shaders";
			//Directories.Textures = "textures";
			//Directories.Scenes = "scenes";
		}

        #endregion

		#region Propierties

		/// <summary>
		/// Gets or sets the base directory that will be added to all the archives.
		/// </summary>
		[Obsolete("Use the new VFS class to get access to files.")]
		public static string BaseDirectory
		{
			get { return baseDirectory; }
			set
			{
				baseDirectory = value;

				// The path must finish with \ or : to be able to add the folders
				if (baseDirectory != "")
				{
					char c = baseDirectory[baseDirectory.Length - 1];

					if (c != '\\' && c != ':')
						baseDirectory += '\\';
				}
			}
		}

		#endregion
		
		#region Methods

		/// <summary>
        /// Checks all the paths and extensions to get the full archive path.
		/// </summary>
        /// <param name="filename">Filename (without extensions and path).</param>
        /// <param name="pathList">Possible paths list.</param>
        /// <param name="extensions">Possible extensions array.</param>
        /// <param name="fullPath">The full path if it's found or null if it's not.</param>
		/// <returns>True if the archive is found, false otherwise.</returns>
		[Obsolete("Use the new VFS class to get access to files")]
        public static bool FindFullPath(string filename, List<string> pathList, string[] extensions, out string fullPath)
		{
			//Loop the elements of the path
            foreach (string path in pathList)
			{
                if (extensions == null)
				{
                    if (File.Exists(path + filename))
					{
                        fullPath = path + filename;
						return true;
					}
				}
				
				else
				{
					//Test with all the extensions
                    for (int c = 0; c < extensions.Length; c++)
					{
                        if (File.Exists(path + filename + extensions[c]))
						{
                            fullPath = path + filename + extensions[c];
							return true;
						}
					}
				}
			}

            fullPath = null;
			return false;
		}

		/// <summary>
        /// Checks all the paths and extensions to get the full archive path. First checks 
        /// BaseDirectory and then InHouseDirectory.
		/// </summary>
        /// <param name="filename">Filename (without extensions and path).</param>
		/// <param name="subPath">Path.</param>
        /// <param name="extensions">Possible extensions array.</param>
        /// <param name="fullPath">The full path if it's found or null if it's not.</param>
        /// <returns>True if the archive is found, false otherwise.</returns>
		[Obsolete("Use the new VFS class to get access to files")]
        public static bool FindFullPath(string filename, string subPath, string[] extensions, out string fullPath)
		{
			if (!FindFullPath(filename, BaseDirectory, subPath, extensions, out fullPath))
				if (!FindFullPath(filename, InHouseDirectory, subPath, extensions, out fullPath))
					if (!FindFullPath(filename, null, subPath, extensions, out fullPath)) //Test if the method was given the full path
						return false;

			return true;
		}

		/// <summary>
        /// Checks the path and all the extensions to get the full archive path.
		/// </summary>
        /// <param name="filename">Filename (without extensions and path).</param>
        /// <param name="baseDir">Base directory.</param>
        /// <param name="subPath">Path.</param>
        /// <param name="extensions">Possible extensions array.</param>
		/// <param name="fullPath">The full path if it's found or null if it's not.</param>
		/// <returns>True if the archive is found, false otherwise.</returns>
		[Obsolete("Use the new VFS class to get access to files")]
		public static bool FindFullPath(string filename, string baseDir, string subPath, string[] extensions, out string fullPath)
		{
			// If theres no baseDir, test only the extensions
			if (baseDir == null)
			{
                if (TestExtensions(filename, extensions, out fullPath))
					return true;

				return false;
			}

			string destDir = baseDir;

			if (subPath == null)
			{
				// If the path doesnt end with a \, add it
				if (destDir[destDir.Length - 1] != '\\')
					destDir += @"\";
			}
			else
			{
				// If the path doesnt end with a \, add it
				if (subPath[subPath.Length - 1] != '\\')
					subPath += @"\";

				destDir += subPath;
			}

			if (!Directory.Exists(destDir))
			{
                fullPath = null;
				return false;
			}

			// Search in the files of this directory
			string[] files = Directory.GetFiles(destDir);
			for (int i = 0; i < files.Length; i++)
				if (TestExtensions(destDir + filename, extensions, out fullPath))
					return true;

			// Not found yet, try in all the possible subdirectories
			string[] dirs = Directory.GetDirectories(destDir);
			for (int i = 0; i < dirs.Length; i++)
				if (FindFullPath(filename, "", dirs[i], extensions, out fullPath))
					return true;

			fullPath = null;
			return false;
		}
        
		/// <summary>
        /// Checks all extensions with the filename to get if the archive exists.
        /// </summary>
        /// <param name="filename">Filename (without extensions and path).</param>
        /// <param name="extensions">Array of entensions.</param>
		/// <param name="fullPath">The fullpath if it's found or null if it's not.</param>
		/// <returns>True if the archive is found, false if otherwise.</returns>
		[Obsolete("Use the new VFS class to get access to files")]
		public static bool TestExtensions(string filename, string[] extensions, out string fullPath)
		{
			if (extensions == null)
			{
				if (File.Exists(filename))
				{
					fullPath = filename;
					return true;
				}
			}
			
			else
			{
				// Test all the extensions
				for (int c = 0; c < extensions.Length; c++)
				{
					if (File.Exists(filename + extensions[c]))
					{
						fullPath = filename + extensions[c];
						return true;
					}
				}
			}

			fullPath = null;
			return false;
		}

		/// <summary>
        /// Change the current dir until the configuration.xml file is found.
        /// This is done to ensure that the current directory is always at the same
        /// level as the file configuration.xml
		/// You can set the specific directory where configuration.xml is located
		/// creating a file named 'ResourcesPath.txt' at the bin directory,
		/// with the full path. The method will look first into this file if it
		/// exists.
		/// </summary>
		/// <remarks>This gets the configuration filename (normally configuration.xml) from JConfiguration.</remarks>
		[Obsolete("Use the new VFS class to get access to files")]
		public static void ChangeDirectory()
		{
			throw new NotSupportedException("JPath is not supported anymore!");

			//string currentDir = System.Windows.Forms.Application.StartupPath;

			//FileInfo fInfo = new FileInfo(Tools.JConfiguration.ConfigurationFilename);

			//Directory.SetCurrentDirectory(fInfo.DirectoryName);

			//// If the file doesnt exist we are on the wrong folder ("debug" or "release" probably)
			//// Make sure we have an InHouse folder there, too...
			//// TODO: This should change with the new VFS stuff

			//try
			//{
			//    while
			//        (
			//            !File.Exists(Tools.JConfiguration.ConfigurationFilename) ||
			//            !Directory.Exists(Directory.GetCurrentDirectory() + "\\InHouse")
			//        )
			//    {
			//        // Look if there is a file named 'resourcesPath.txt'
			//        if (File.Exists("ResourcesPath.txt"))
			//        {
			//            StreamReader file = new StreamReader("ResourcesPath.txt");
			//            currentDir = file.ReadLine();
			//            file.Close();
			//            Directory.SetCurrentDirectory(currentDir);
			//            JVFS.RootDirectory = currentDir;
			//            return;
			//        }

			//        // Backtrack
			//        currentDir += @"\..";
			//        Directory.SetCurrentDirectory(currentDir);
			//    }

			//    JVFS.RootDirectory = currentDir;
			//}

			//catch (Exception)
			//{
			//    throw new ApplicationException("The engine couldn't find the " + JadEngine.Tools.JConfiguration.ConfigurationFilename + " file. Check your paths.");
			//}
		}
		
		#endregion
	}
}
