#region Using Directives

using System;
using JadEngine.Collections;

#endregion

/*
 * 28/01/08
 *	-Initial cleanup
 * 24/03/08
 *	-Changed to derive from JEngineElement
 */

namespace JadEngine.Core
{
	/// <summary>
	/// State object to use with a <see cref="JadEngine.Core.JStateMachine"/>.
	/// </summary>
	public class JState : JObject
	{
		#region Fields

		/// <summary>
		/// Name of the state.
		/// </summary>
		/// <remarks>
		/// Used for state transitions
		/// </remarks>
		private string _name;

		/// <summary>
		/// Reference to the state machine for easy changing states from within
		/// other states.
		/// </summary>
		private JStateMachine _machine;

		#endregion

		#region Constructors
		
		/// <summary>
		/// Initializes a new instance of the <see cref="JState"/> class.
		/// </summary>
		public JState()
		{			
		}

		/// <summary>
		/// Creates a new state with a name.
		/// </summary>
		/// <param name="name">Name of the state.</param>
		public JState(string name)
		{
			_name = name;			
		}

		#endregion

		#region Properties		

		/// <summary>
		/// Gets or sets the name of the state.
		/// </summary>
		public string Name
		{
			get { return _name; }
			set { _name = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// Sets a reference to the parent state machine.
		/// </summary>
		/// <param name="machine">The parent state machine.</param>
		public void SetStateMachine(JStateMachine machine)
		{
			_machine = machine;
		}

		/// <summary>
		/// Called when entering a new state.
		/// </summary>
		/// <returns>Auxiliar return value</returns>
		public virtual bool OnEnter()
		{
			return true;
		}

		/// <summary>
		/// Called when running the state.
		/// </summary>
		/// <returns>Auxiliar return value</returns>
		public virtual bool OnRun()
		{
			return true;
		}

		/// <summary>
		/// Called when exiting the state.
		/// </summary>
		/// <returns>Auxiliar return value</returns>
		public virtual bool OnExit()
		{
			return true;
		}

		#endregion
	}
}
