#region Using Directives

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using JadEngine.Core;

#endregion

/*
 * 28/01/08
 *	-Initial cleanup
 *	-Win32 interop moved to Win32Interop.cs
 */

namespace JadEngine
{
	/// <summary>
	/// Helper class to ease creation of state machine based Jad applications.
	/// </summary>
	public class JStateApplication
	{
		#region Fields

		/// <summary>
		/// Indicates if the application must quit
		/// </summary>
		private static bool _quit = false;

		/// <summary>
		/// Application's state machine
		/// </summary>
		private JStateMachine _machine;

		#endregion		

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JStateApplication()
		{
			_machine = new JStateMachine();
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets if there are messages waiting to be processed or not
		/// </summary>
		private bool ApplicationIdle
		{
			get
			{
				Win32Interop.Message msg;
				return !Win32Interop.PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
			}
		}

		/// <summary>
		/// Set the quit value to true for quiting the application.
		/// </summary>
		public static bool Quit
		{
			set { _quit = value; }
		}

		/// <summary>
		/// Gets the application's state machine.
		/// </summary>
		public JStateMachine States
		{
			get { return _machine; }
		}

		#endregion

		#region Methods       
		/// <summary>
		/// Gets called when the application idle event occurs.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		protected void OnApplicationIdle(object sender, EventArgs e)
		{
			// Render a frame during idle time (no messages are waiting)
			while (ApplicationIdle)
			{
				// Run the current state 
				_machine.OnRun();

				if (_quit)
				{
					//Runs the ending code of the current state
					_machine.OnEnd();

					// Free resources and shutdown the engine
					Jad.End();

					// Close the main form
					Jad.Form.Close();
				}
			}
		}

		/// <summary>
		/// Adds the Idle event handler when needed and runs the application
		/// </summary>
		/// <returns></returns>
		public void AddOnIdle()
		{
			Application.Idle += new EventHandler(OnApplicationIdle);
			Application.Run(Jad.Form);
		}

		#endregion
	}
}
