#region Using Directives

using System;
using JadEngine.Collections;
using System.Collections.Generic;

#endregion

/*
 * 28/01/08
 *	-Initial cleanup
 */

namespace JadEngine.Core
{
	/// <summary>
	/// A simple state machine to control application states.
	/// </summary>
	public class JStateMachine
	{
		#region Fields

		/// <summary>
		/// The current state (current running code).
		/// </summary>
		private JState _currentState;

		/// <summary>
		/// List of all states.
		/// </summary>
		private List<JState> _statesList;

		private Dictionary<string, JState> _states;

		#endregion

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="JStateMachine"/> class.
		/// </summary>
		public JStateMachine()
		{			
			_states = new Dictionary<string, JState>();
		}

		#endregion

		#region Methods

		/// <summary>
		/// Registers a new state.
		/// </summary>
		/// <param name="state">State to register.</param>
		public void RegisterState(JState state)
		{			
			_states.Add(state.Name, state);

			//We set a reference to this machine to the state so it will be able
			//to change the current state from within any state.
			state.SetStateMachine(this);
		}

		/// <summary>
		/// Sets the current state to a registered state.
		/// </summary>
		/// <param name="state">Name of the state to set.</param>
		public bool SetCurrentState(string state)
		{
			//Run exiting code for the current state
			if (_currentState != null)
				_currentState.OnExit();

			//Set the new current state
			_currentState = _states[state];

			//Run the entering code for the new current state
			if (_currentState != null)
				return _currentState.OnEnter();

			return false;
		}		

		/// <summary>
		/// Runs the current state's OnRun method.
		/// </summary>
		public void OnRun()
		{
			if (_currentState != null)
				_currentState.OnRun();
		}

		/// <summary>
		/// Runs the current state's OnExit method.
		/// </summary>
		public void OnEnd()
		{
			if (_currentState != null)
				_currentState.OnExit();
		}

		#endregion
	}
}
