#region Using Directives

using System;

#endregion

/*
 * 29/01/08
 *	-Initial cleanup
 */

namespace JadEngine.Core
{
	/// <summary>
	/// This exception is raised when a field receives an invalid value (invalid according to the 
	/// program logic). For example: a property setter that only allows positive integers and receives a
	/// negative number will throw this exception.
	/// </summary>
	public sealed class JValidationException : Exception
	{
		#region Fields

		/// <summary>
		/// The field that has recieved the invalid value.
		/// </summary>
		private String _fieldName;

		/// <summary>
		/// The class where the field is located.
		/// </summary>
		private String _className;

		/// <summary>
		/// The validation error message.
		/// </summary>
		private String _errorMessage;

		#endregion

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="JValidationException"/> class.
		/// </summary>
		/// <param name="fieldName">The  name of the field that has received the invalid value.</param>
		/// <param name="className">The class where the field is located.</param>
		/// <param name="errorMessage">The validation error message.</param>
		public JValidationException(String fieldName, String className, String errorMessage)
		{
			this._fieldName = fieldName;
			this._className = className;
			this._errorMessage = errorMessage;
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets the validation error message.
		/// </summary>
		public override string Message
		{
			get
			{
				return "Invalid value assigned to \"" + _className + "." + _fieldName + "\". The validation error was: " + _errorMessage;
			}
		}

		#endregion
	}
}
