#region Using Directives

using System;
using System.Runtime.InteropServices;

#endregion

/*
 * 28/01/08
 *	-Created
 * 11/02/08
 *	-Added BitBlt import declaration
 */

namespace JadEngine
{
	/// <summary>
	/// Class that encapsulates Win32 interop calls.
	/// </summary>
	public static class Win32Interop
	{
		#region Structs

		/// <summary>
		/// Implements the Windows MSG structure.
		/// The MSG structure contains message information from a thread's message queue. 
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
		public struct Message
		{
			#region Fields

			/// <summary>
			/// Handle to the window whose window procedure receives the message. hWnd is null when the message is a thread message.
			/// </summary>
			public IntPtr hWnd;

			/// <summary>
			/// Specifies the message identifier. Applications can only use the low word; the high word is reserved by the system. 
			/// </summary>
			public uint msg;

			/// <summary>
			/// Specifies additional information about the message. The exact meaning depends on the value of the message member. 
			/// </summary>
			public IntPtr wParam;

			/// <summary>
			/// Specifies additional information about the message. The exact meaning depends on the value of the message member. 
			/// </summary>
			public IntPtr lParam;

			/// <summary>
			/// Specifies the time at which the message was posted. 
			/// </summary>
			public uint time;

			/// <summary>
			/// Specifies the cursor position, in screen coordinates, when the message was posted. 
			/// </summary>
			public System.Drawing.Point p;

			#endregion
		}

		#endregion

		#region Imported Functions

		/// <summary>
		/// PeekMessage pInvoke: We will not use this maliciously
		/// </summary>
		/// <param name="msg">Pointer to a MSG structure that receives message information.</param>
		/// <param name="hWnd">Handle to the window whose messages are to be examined. The window must belong to the current thread.</param>
		/// <param name="messageFilterMin">Specifies the value of the first message in the range of messages to be examined. Use WM_KEYFIRST to specify the first keyboard message or WM_MOUSEFIRST to specify the first mouse message.  If wMsgFilterMin and wMsgFilterMax are both zero, PeekMessage returns all available messages (that is, no range filtering is performed).</param>
		/// <param name="messageFilterMax">Specifies the value of the last message in the range of messages to be examined. Use WM_KEYLAST to specify the last keyboard message or WM_MOUSELAST to specify the last mouse message.  If wMsgFilterMin and wMsgFilterMax are both zero, PeekMessage returns all available messages (that is, no range filtering is performed).</param>
		/// <param name="flags">Specifies how messages are handled. This parameter can be one of the following values: PM_NOREMOVE or PM_REMOVE</param>
		/// <returns>If a message is available, the return value is nonzero.</returns>
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("User32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);

		/// <summary>
		/// Retrieves the frequency of the high-resolution performance counter, if one exists. The frequency cannot change while the system is running.
		/// </summary>
		/// <param name="performanceFrequency">Pointer to a variable that receives the current performance-counter frequency, in counts per second. If the installed hardware does not support a high-resolution performance counter, this parameter can be zero. </param>
		/// <returns>If the installed hardware supports a high-resolution performance counter, the return value is nonzero.</returns>
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("Kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern bool QueryPerformanceFrequency(ref long performanceFrequency);

		/// <summary>
		/// Retrieves the current value of the high-resolution performance counter.
		/// </summary>
		/// <param name="performanceCount">Pointer to a variable that receives the current performance-counter value, in counts.</param>
		/// <returns>If the function succeeds, the return value is nonzero.</returns>
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("Kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern bool QueryPerformanceCounter(ref long performanceCount);

		/// <summary>
		/// Retrieves the system time, in milliseconds. The system time is the time elapsed since Windows was started.
		/// </summary>
		/// <returns>Returns the system time, in milliseconds.</returns>
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("Winmm.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern int timeGetTime();

		/// <summary>
		/// The BitBlt function performs a bit-block transfer of the color data corresponding to a rectangle of pixels from the specified source device context into a destination device context. 
		/// </summary>
		/// <param name="hdcDest">Handle to destination DC.</param>
		/// <param name="nXDest">x-coord of destination upper-left corner.</param>
		/// <param name="nYDest">y-coord of destination upper-left corner.</param>
		/// <param name="nWidth">Width of destination rectangle.</param>
		/// <param name="nHeight">Height of destination rectangle.</param>
		/// <param name="hdcSrc">Handle to source DC.</param>
		/// <param name="nXSrc">x-coordinate of source upper-left corner.</param>
		/// <param name="nYSrc">y-coordinate of source upper-left corner.</param>
		/// <param name="dwRop">Raster operation code.</param>
		/// <returns>If the function succeeds, the return value is nonzero. If the function fails, the return value is zero.</returns>
		[DllImport("Gdi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern int BitBlt(IntPtr hdcDest, int nXDest, int nYDest, int nWidth, int nHeight, IntPtr hdcSrc, int nXSrc, int nYSrc, int dwRop);


		#endregion
	}
}
