#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using JadEngine;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Animation;
using JadEngine.Import.Collada;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Import
{
	/// <summary>
	/// Main class to import collada files
	/// </summary>
    internal class JCollada
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JImport _import;

        string _path;

        COLLADA _collada;

        JScene _scene;

        JColladaImages _images;

        JColladaMaterials _materials;

        JColladaSceneGraph _sceneGraph;

        JColladaMeshes _meshes;

        JColladaLights _lights;

        JColladaCameras _cameras;

        internal Vector3 scale;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the parent object
        /// </summary>
        internal JImport Import
        {
            get { return _import; }
        }

        internal JScene Scene
        {
            get { return _scene; }
        }

        public JColladaMeshes Meshes
        {
            get { return _meshes; }
        }

        public JColladaLights Lights
        {
            get { return _lights; }
        }

        public JColladaCameras Cameras
        {
            get { return _cameras; }
        }

        /// <summary>
        /// Gets the COLLADA object
        /// </summary>
        internal COLLADA Collada
        {
            get { return _collada; }
        }

        /// <summary>
        /// Gets the file path
        /// </summary>
        internal string Path
        {
            get { return _path; }
        }
        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="import">The parent object</param>
        public JCollada(JImport import)
        {
            _import = import;
        }

        /// <summary>
        /// Loads a collada file and creates all the objects into Jad
        /// </summary>
        /// <param name="scene">The scene where you want to load the file</param>
        /// <param name="fileName">Full path file name</param>
        /// <param name="scale">Scale</param>
        /// <returns>True if everything is ok</returns>
        public bool Load(JScene scene,string fileName,Vector3 scale)
        {
            _collada = null;

            _scene = scene;

            this.scale = scale;

            XmlSerializer sr = new XmlSerializer(typeof(COLLADA));
            TextReader tr = new StreamReader(fileName);
            _collada = (COLLADA)(sr.Deserialize(tr));
            tr.Close();

            _path = System.IO.Path.GetDirectoryName(fileName);

            CreateScene();

            return true;
        }

        void CreateScene()
        {
            _images = new JColladaImages(this);
            _materials = new JColladaMaterials(this);
            _meshes = new JColladaMeshes(this);
            _lights = new JColladaLights(this);
            _cameras = new JColladaCameras(this);

            _sceneGraph = new JColladaSceneGraph(this);
        }
    }
}
