using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using JadEngine.Core;
using JadEngine.Video;

namespace JadEngine.Import.Collada
{
    /// <summary>
    /// Used to Load the images that comes from the collada file
    /// </summary>
    internal class JColladaImages
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JCollada _collada;

        /// <summary>
        /// The images library
        /// </summary>
        library_images _library;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the COLLADA instance
        /// </summary>
        internal COLLADA Collada
        {
            get { return _collada.Collada; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="collada">The parent object</param>
        internal JColladaImages(JCollada collada)
        {
            _collada = collada;

            for (int i = 0; i < Collada.Items.Length; i++)
            {
                if (Collada.Items[i] is library_images)
                {
                    _library = (library_images)Collada.Items[i];

                    Load();
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Loads the image tag and creates the 2D texture
        /// The texture name will be the img.id tag
        /// </summary>
        void Load()
        {
            foreach (image img in _library.image)
            {
                if (img.Item is string)
                {
                    string file = (string)img.Item;

                    // Let's see if the file exists

                    //file = Path.GetFileNameWithoutExtension(file);

                    Jad.Video.Textures.Create2D(img.id, file, true);                    
                }
            }
        }

        #endregion
    }
}
