#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using JadEngine;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Animation;
//using JadEngine.Import.Collada;
using JadEngine.Import.DeleD;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Import
{
	/// <summary>
	/// Main class to import collada files
	/// </summary>
	internal class JDeleD
	{
		#region Fields

		/// <summary>
		/// The parent object
		/// </summary>
		JImport _import;

		string _path;
		
		scene _delEd;

		JScene _scene;

        bool _importAsJWorld;

        /// <summary>
        /// The World instance if the scene has to be loaded as a World
        /// </summary>
        JWorldLightMap _world;

        /// <summary>
        /// The materials library
        /// </summary>
		JDelEdMaterials _materials;

        JDeleDPrimitives _primitives;

        /// <summary>
        /// The scale value to apply to vertices
        /// </summary>
		Vector3 _scale;

		#endregion

		#region Properties

        /// <summary>
        /// The World instance if the scene has to be loaded as a World
        /// </summary>
        internal JWorldLightMap World
        {
            get { return _world; }
        }

        /// <summary>
        /// Gets/Sets to import the scene as a JWorld class
        /// </summary>
        public  bool ImportAsJWorld
        {
            get { return _importAsJWorld; }
            set { _importAsJWorld=value; }
        }

        /// <summary>
        /// The materials library
        /// </summary>
        internal JDelEdMaterials Materials
        {
            get { return _materials; }
        }

        /// <summary>
        /// Gets the scale value to apply to vertices
        /// </summary>
        internal Vector3 Scale
        {
            get { return _scale; }
        }
		/// <summary>
		/// Gets the parent object
		/// </summary>
		internal JImport Import
		{
			get { return _import; }
		}

		internal JScene Scene
		{
			get { return _scene; }
		}

		/// <summary>
		/// Gets the DelED object
		/// </summary>
		internal scene DelEd
		{
			get { return _delEd; }
		}

		/// <summary>
		/// Gets the file path
		/// </summary>
		internal string Path
		{
			get { return _path; }
		}
		#endregion

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="import">The parent object</param>
		public JDeleD(JImport import)
		{
			_import = import;
            _importAsJWorld = true;
		}

		/// <summary>
		/// Loads a DeleD file and creates all the objects into Jad
		/// </summary>
		/// <param name="scene">The scene where you want to load the file</param>
		/// <param name="fileName">Full path file name</param>
		/// <param name="scale">Scale</param>
		/// <returns>True if everything is ok</returns>
		public bool Load(JScene scene, string fileName, Vector3 scale)
		{
			_scene = scene;

			_scale = scale;

			XmlSerializer sr = new XmlSerializer(typeof(scene));
			TextReader tr = new StreamReader(fileName);
			_delEd = (scene)(sr.Deserialize(tr));
			tr.Close();

			_path = System.IO.Path.GetDirectoryName(fileName);

			CreateScene();

			return true;
		}

		void CreateScene()
		{
            if (_importAsJWorld)
            {
                _scene.World = _world=new JWorldLightMap();
            }

			_materials = new JDelEdMaterials(this);

            _primitives = new JDeleDPrimitives(this);
		}
	}
}
