#region using

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Scene;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Import.DeleD
{
    /// <summary>
    /// Used to Load the materials that comes from the DeleD file
    /// </summary>
    internal class JDelEdMaterials
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JDeleD _deleD;

        Dictionary<string, object> _materials;

        #endregion

        #region Properties

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="collada">The parent object</param>
        internal JDelEdMaterials(JDeleD delEd)
        {
            _deleD = delEd;

            _materials = new Dictionary<string, object>();

            // Due to DXS files has all the texture information, even
            // the texutre is not used, we first build the material list
            // and when the material is needed we will Load it.

            foreach (sceneMaterials sceneMat in _deleD.DelEd.materials)
            {
                foreach (sceneMaterialsCategory category in sceneMat.category)
                {
                    foreach (sceneMaterialsCategoryMaterial material in category.material)
                    {
                        _materials.Add(material.id, material);
                        //Load(material);
                    }
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the material given an id
        /// </summary>
        /// <param name="id">The material ID</param>
        /// <returns>The JMaterial found. If it's not found, this method will raise an exception</returns>
        internal JMaterial GetMaterial(string id)
        {
            object mat=_materials[id];

            if (mat is JMaterial) return (JMaterial)mat;
            
            sceneMaterialsCategoryMaterial material=(sceneMaterialsCategoryMaterial)mat;

            JMaterial jadMat=Load(material);

            _materials[id] = jadMat;

            return jadMat;

        }

        /// <summary>
        /// Loads the image tag and creates the 2D texture
        /// The texture name will be the img.id tag
        /// </summary>
        JMaterial Load(sceneMaterialsCategoryMaterial material)
        {
            JMaterial mat = _deleD.Scene.Materials.Create();

            int numLayer = 0;

            JMaterialLayer jLayer = null;

            foreach (sceneMaterialsCategoryMaterialLayer layer in material.layer)
            {
                bool newLayer = true;

                if (numLayer == 1 && layer.type == "lightmap")
                {
                    // If we have a lightmap, we do not need to create
                    // another layer
                    newLayer = false;
                }

                if (newLayer)
                {
                    jLayer = new JMaterialLayer();
                }

                string file = layer.texture[0].file;

                JTexture texture = Jad.Video.Textures.Create2D(file, true);                

                if (layer.type == "texture")
                {
                    jLayer.DiffuseMap.Texture = texture;

                    jLayer.DiffuseMap.Enabled = true;
                }

                if (layer.type == "lightmap")
                {
                    jLayer.LightMap.Texture = texture;

                    jLayer.LightMap.Enabled = true;
                }

                if (newLayer) mat.AddLayer(jLayer);

                numLayer++;
            }

            return mat;
        }

        #endregion
    }
}
