#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using JadEngine;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Animation;
//using JadEngine.Import.Collada;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace JadEngine.Import.DeleD
{
    /// <summary>
    /// Used to Load the images that comes from the collada file
    /// </summary>
    internal class JDeleDPrimitives
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JDeleD _deleD;

        #endregion

        #region Properties

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="delEd">The parent object</param>
        internal JDeleDPrimitives(JDeleD delEd)
        {
            _deleD = delEd;

            foreach (scenePrimitives scenePrimitive in _deleD.DelEd.primitives)
            {
                foreach (scenePrimitivesPrimitive primitive in scenePrimitive.primitive)
                {
                    LoadPrimitive(primitive);
                }
            }
        }

        #endregion

        #region Methods
        
        Vector3[] vertices;

        List<int> _materialList;

        /// <summary>
        /// Fill the vertices array
        /// </summary>
        /// <param name="primitive">The primitive source</param>
        void FillVertices(scenePrimitivesPrimitive primitive)
        {
            vertices = new Vector3[primitive.vertices.Length];

            int i = 0;

            foreach (vertex ivertex in primitive.vertices)
            {
                float x = float.Parse(ivertex.x, Jad.DecimalFormatInfo);
                float y = float.Parse(ivertex.y, Jad.DecimalFormatInfo);
                float z = float.Parse(ivertex.z, Jad.DecimalFormatInfo);

                Vector3 position = new Vector3(x, y, z);

                position.X *= _deleD.Scale.X;
                position.Y *= _deleD.Scale.Y;
                position.Z *= _deleD.Scale.Z;

                vertices[i++] = position;

            }

        }

        /// <summary>
        /// Get the number of materials needed
        /// </summary>
        /// <param name="primitive">The source primitive</param>
        /// <returns>The number of materials</returns>
        int GetNumMatFromPrimitive(scenePrimitivesPrimitive primitive)
        {
            _materialList = new List<int>();

            foreach (scenePrimitivesPrimitivePolygonsPoly poly in primitive.polygons)
            {
                int matID = int.Parse(poly.mid);

                bool found = false;

                foreach (int i in _materialList)
                {
                    if (matID == i) { found = true; break; }
                }
                if (!found) _materialList.Add(matID);
            }

            int numMat = _materialList.Count;

            return numMat;
        }

        /// <summary>
        /// Gets the index material(starting at 0) given the material id
        /// </summary>
        /// <param name="matID">The DeleD material id for the poly</param>
        /// <returns>The layer index.-1 if not found</returns>
        int GetIndexMaterial(int matID)
        {
            int index=0;

            foreach (int i in _materialList)
            {
                if (matID == i) return index;
                index++;
            }

            return -1;
        }

        /// <summary>
        /// Loads the primitive and build the JMeshObject
        /// </summary>
        void LoadPrimitive(scenePrimitivesPrimitive primitive)
        {
            
            JMeshObject meshObject;

            if (_deleD.ImportAsJWorld)
            {
                meshObject = _deleD.World.MeshObjects.Create();
            }
            else
            {
                meshObject = _deleD.Scene.MeshObjects.Create();
            }
            
            int numMatUsed = GetNumMatFromPrimitive(primitive);

            //meshObject.Materials = new JMaterial[numMatUsed];

            JVertexCollector collector = new JVertexCollector(Jad.Video);

            FillVertices(primitive);

            collector.Initialize(false);

            foreach (scenePrimitivesPrimitivePolygonsPoly poly in primitive.polygons)
            {
                int matID = int.Parse(poly.mid);

                meshObject.Materials[GetIndexMaterial(matID)] = _deleD.Materials.GetMaterial(poly.mid);

                foreach (vertex ivertex in poly.vertex)
                {
                    collector.Begin(matID);

                    collector.AddPosition(vertices[int.Parse(ivertex.vid)]);

                    float u = float.Parse(ivertex.u0, Jad.DecimalFormatInfo);
                    float v = float.Parse(ivertex.v0, Jad.DecimalFormatInfo);

                    Vector2 texCoord = new Vector2(u, v);

                    collector.AddTexCoord(0,texCoord);

                    if (ivertex.u1 != "" && ivertex.u1!=null)
                    {
                        u = float.Parse(ivertex.u1, Jad.DecimalFormatInfo);
                        v = float.Parse(ivertex.v1, Jad.DecimalFormatInfo);

                        texCoord = new Vector2(u, v);

                        collector.AddTexCoord(1, texCoord);
                    }

                    collector.End();
                }
            }

            collector.FinishAndFill(meshObject.Mesh);

            meshObject.Mesh.VertexBuffer.InvertNormals();
            meshObject.Mesh.IndexBuffer.InvertFaces();

            if (_deleD.ImportAsJWorld)
            {
                meshObject.Mesh.CenteredAtOrigin = true;
            }
            else
            {
                meshObject.Mesh.CenterAtOrigin();

                meshObject.Transform.Position = meshObject.Mesh.TranslationToLocal;
            }
        }

        #endregion
    }
}
