#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

#endregion

namespace JadEngine.Input
{
	/// <summary>
	/// Wrapper to handle a single button press in the keyboard or joysticks.
	/// This manages the difference between down and pressed for us, 
	/// so we can differentiate between them.
	/// </summary>
	public class JButtonValue
	{
		/// <summary>
		/// A blank (0) value
		/// </summary>
		public readonly static JButtonValue Untouched = new JButtonValue();

		#region Fields
		private bool _down;
		private bool _pressed;
		#endregion

		#region Properties

		/// <summary>
		/// True when the button is being held down
		/// </summary>
		public bool Down
		{
			get { return _down; }
			set
			{
				if (value)
					_down = _pressed = true;
				else
					_down = false;
				//if ( (_down) && (!value) && (!_pressed) )
				//	_pressed = true;
				//_down = value;
				// When we release the button, turn on pressed
			}
		}

		/// <summary>
		/// Set to true when the button was pushed and released
		/// </summary>
		public bool Pressed
		{
			get
			{
				if (!_down && _pressed)
				{
					_pressed = false;
					return true;
				}
				return false;
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Reinitialize us
		/// </summary>
		public void Clear()
		{
			_down = _pressed = false;
		}

		#endregion
	}

}
