#region Credits
// Portions of this code are based on algorithms and code from softSurfer.  Terms of use are included here:

// Copyright 2000, softSurfer (www.softsurfer.com)
// This code may be freely used and modified for any purpose
// providing that this copyright notice is included with it.
// SoftSurfer makes no warranty for this code, and cannot be held
// liable for any real or imagined damage resulting from its use.
// Users of this code must verify correctness for their application.
#endregion

#region Using Directives

using System;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics;
using JadEngine.Mathematics.Matrices;

#endregion

namespace JadEngine.Mathematics.Geometry
{
	/// <summary>
	/// Represents a triangle in 2D
	/// </summary>
	public class JTriangle2
	{
		#region Fields

		/// <summary>Triangle points</summary>
		public Vector2 P0, P1, P2;

        #endregion

        #region Properties

        /// <summary>
        /// Computes the matrix determinant.
        /// Can be used to know if the triangle is clockwise or anticlockwise, or to determine the triangle's area
        /// </summary>
        public float Determinant
        {
            get
            {
                // Inlined this since it's a LOT (100x) faster. This lets us not precompute it.
                return ((P1.X - P0.X) * (P2.Y - P0.Y) - (P2.X - P0.X) * (P1.Y - P0.Y));

                //JMatrix m = new JMatrix(3, 3);
                //m[0, 0] = m[1, 0] = m[2, 0] = 1f;
                //m[0, 1] = P0.X; m[1, 1] = P1.X; m[2, 1] = P2.X;
                //m[0, 2] = P0.Y; m[1, 2] = P1.Y; m[2, 2] = P2.Y;
                //return (float)m.Determinant;
            }
        }

		/// <summary>Return the order of the triangle giving the points</summary>
		public JVerticesOrder Order
		{
            get
            {
                if (Determinant < 0) return JVerticesOrder.ClockWise;
                else return JVerticesOrder.AntiClockWise;
            }
        }

        /// <summary>Return the area of the triangle</summary>
        public float Area
        {
            get
            {
                return (float)(0.5 * Math.Abs(Determinant));
            }
        }

        #endregion

        #region Constructors

        /// <summary>
		/// Build a 2D triangle, giving the 3 points
		/// </summary>
		/// <param name="p0">First point</param>
		/// <param name="p1">Second point</param>
		/// <param name="p2">Third point</param>
		public JTriangle2(Vector2 p0, Vector2 p1, Vector2 p2)
		{
			P0 = p0;
			P1 = p1;
			P2 = p2;
		}

		#endregion

		#region Methods

        /// <summary>Return as text</summary>
        public override string ToString()
        {
            return String.Format("P0 ({0}, {1}), P1 ({2}, {3}), P2 ({4}, {5})", P0.X, P0.Y, P1.X, P1.Y, P2.X, P2.Y);
        }

		#endregion
	}
}
