#region Using directives

using System.Collections.Generic;

using Microsoft.DirectX;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JConvexHull
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		private List<Plane> planes;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JConvexHull()
		{
			planes = new List<Plane>();
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="planes">TODO</param>
		public JConvexHull(List<Plane> planes)
		{
			this.planes = planes;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="planes">TODO</param>
		public JConvexHull(Plane[] planes)
		{
			this.planes = new List<Plane>(planes.Length);

			for(int i = 0; i < planes.Length; i++)
				this.planes[i] = planes[i];
		}

		#endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public List<Plane> Planes
		{
			get { return planes; }
			set { planes = value; }
		}
		
		#endregion
		
		#region Methods

		/// <summary>
		/// Tests if a point is inside or outside the convex hull
		/// </summary>
		/// <param name="point">Point to test</param>
		/// <returns>True if the point is inside the convex hull, false if its not</returns>
		public bool PointInside(Vector3 point)
		{
			foreach (Plane plane in planes)
				if (plane.Dot(point) < 0.0f)
					return false;

			return true;
		}

		/// <summary>
		/// Test if a bounding box is inside or outside the convex hull
		/// </summary>
		/// <param name="min">Minimum point of the bounding box</param>
		/// <param name="max">Maximum point of the bounding box</param>
		/// <returns>True if the bounding box is inside the convex hull, false if its not</returns>
		/// <remarks>Probably this method doesnt work with convex hull with many angles</remarks>
		public bool BBoxInside(Vector3 min, Vector3 max)
		{
			Vector3 nearPoint;

			foreach (Plane plane in planes)
			{
				if (plane.A > 0.0f)
				{
					if (plane.B > 0.0f)
					{
						if (plane.C > 0.0f)
						{
							nearPoint.X = max.X;
							nearPoint.Y = max.Y;
							nearPoint.Z = max.Z;
						}
						
						else
						{
							nearPoint.X = max.X;
							nearPoint.Y = max.Y;
							nearPoint.Z = min.Z;
						}
					}
					
					else
					{
						if (plane.C > 0.0f)
						{
							nearPoint.X = max.X;
							nearPoint.Y = min.Y;
							nearPoint.Z = max.Z;
						}
						
						else
						{
							nearPoint.X = max.X;
							nearPoint.Y = min.Y;
							nearPoint.Z = min.Z;
						}
					}
				}
				
				else
				{
					if (plane.B > 0.0f)
					{
						if (plane.C > 0.0f)
						{
							nearPoint.X = min.X;
							nearPoint.Y = max.Y;
							nearPoint.Z = max.Z;
						}
						
						else
						{
							nearPoint.X = min.X;
							nearPoint.Y = max.Y;
							nearPoint.Z = min.Z;
						}
					}
					
					else
					{
						if (plane.C > 0.0f)
						{
							nearPoint.X = min.X;
							nearPoint.Y = min.Y;
							nearPoint.Z = max.Z;
						}
						
						else
						{
							nearPoint.X = min.X;
							nearPoint.Y = min.Y;
							nearPoint.Z = min.Z;
						}
					}
				}

				if (plane.Dot(nearPoint) < 0.0f)
					return false;
			}

			return true;
		}

		/// <summary>
		/// Test if at least one point of the bounding box is inside de convex hull
		/// </summary>
		/// <param name="min">Minimum point of the bounding box</param>
		/// <param name="max">Maximum point of the bounding box</param>
		/// <param name="numPointsInside">This parameter will contain the number of points inside the convex hull</param>
		/// <returns>True if at least one point of the bounding box is inside the convex hull, false if otherwise</returns>
		public bool BBoxOnePointInside(Vector3 min, Vector3 max, out int numPointsInside)
		{
			Vector3[] points = new Vector3[8];

			points[0] = new Vector3(min.X, min.Y, min.Z);
			points[1] = new Vector3(min.X, min.Y, max.Z);
			points[2] = new Vector3(min.X, max.Y, min.Z);
			points[3] = new Vector3(min.X, max.Y, max.Z);
			points[4] = new Vector3(max.X, min.Y, min.Z);
			points[5] = new Vector3(max.X, min.Y, max.Z);
			points[6] = new Vector3(max.X, max.Y, min.Z);
			points[7] = new Vector3(max.X, max.Y, max.Z);

			numPointsInside = 0;

			for (int i = 0; i < 8; i++)
				if (PointInside(points[i]))
					numPointsInside++;

			return (numPointsInside > 0);
		}

		/// <summary>
		/// Tests if an sphere is inside or outside the convex hull
		/// </summary>
		/// <param name="origin">Origin of the sphere</param>
		/// <param name="radius">Radius of the sphere</param>
		/// <returns>True if the sphere is inside the convex hull, false its not</returns>
		public bool BSphereInside(Vector3 origin, float radius)
		{
			foreach (Plane plane in planes)
				if (plane.Dot(origin) < -radius)
					return false;

			return true;
		}

		#endregion
	}
}