#region Using Directives

using System;

using System.Collections.Generic;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics;

#endregion

namespace JadEngine.Mathematics.Geometry
{
    /// <summary>
    /// Class for representing a Polygon in 3D.  
    /// The polygon is defined by the three or more points
	/// in order
    /// </summary>
	public class JPolygon3
	{
		#region Fields
		private List<Vector3> _points;
		#endregion

		#region Constructors
		/// <summary>
		/// Construct a polygon from any enumeration of points (list/array/etc)
		/// </summary>
		/// <param name="points"></param>
		public JPolygon3(IEnumerable<Vector3> points)
		{
			_points = new List<Vector3>(points);

			// Don't duplicate the starting point
			if (_points[0] == _points[_points.Count - 1])
				_points.RemoveAt(_points.Count - 1);

			if (_points.Count < 3)
				throw new ArgumentException("You must supply at least 3 points for a polygon!");
		}
		#endregion

		#region Properties
		/// <summary>
		/// Get a point around the polygon
		/// </summary>
		public Vector3 this[int index]
		{
			get
			{
				return _points[index];
			}
			set
			{
				_points[index] = value;
			}
		}
		/// <summary>
		/// Return the number of points in the polygon ring
		/// </summary>
		public int Count
		{
			get { return _points.Count; }
		}
		#endregion

		#region Methods
		// TODO: Triangulate this?		
		#endregion
	}
}
