#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

#endregion

namespace JadEngine.Mathematics.Interpolation
{
    /// <summary>
    /// Interpolation calculator for double values
    /// </summary>
    public struct doubleCalculator : IJInterpolationCalculator<double>
    {
        #region Methods

        /// <summary>
        /// Add two numbers
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The sum</returns>
        public double Add(double a, double b)
        {
            return a + b;
        }

        /// <summary>
        /// Subtract two numbers
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The difference (a-b)</returns>
        public double Subtract(double a, double b)
        {
            return a - b;
        }

        /// <summary>
        /// Multiply two numbers
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The product</returns>
        public double Scale(double a, double b)
        {
            return a * b;
        }

        /// <summary>
        /// Calculate interval between two numbers (The difference)
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The difference</returns>
        public double IntervalSize(double a, double b)
        {
            return Math.Abs(b - a);
        }

        /// <summary>
        /// Check to see if the numbers are equal
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>True if a==b</returns>
        public bool Equals(double a, double b)
        {
            return (a == b);
        }

        #endregion
    }

    /// <summary>
    /// Interpolation calculator for float values
    /// </summary>
    public struct floatCalculator : IJInterpolationCalculator<float>
    {
        #region Methods

        /// <summary>
        /// Add two numbers
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The sum</returns>
        public float Add(float a, float b)
        {
            return a + b;
        }

        /// <summary>
        /// Subtract two numbers
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The difference (a-b)</returns>
        public float Subtract(float a, float b)
        {
            return a - b;
        }

        /// <summary>
        /// Multiply two numbers
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The product</returns>
        public float Scale(float a, double b)
        {
            return a * (float)b;
        }

        /// <summary>
        /// Calculate interval between two numbers (The difference)
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>The difference</returns>
        public double IntervalSize(float a, float b)
        {
            return Math.Abs(b - a);
        }

        /// <summary>
        /// Check to see if the numbers are equal
        /// </summary>
        /// <param name="a">First number</param>
        /// <param name="b">Second number</param>
        /// <returns>True if a==b</returns>
        public bool Equals(float a, float b)
        {
            return (a == b);
        }

        #endregion
    }
}
