#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;

#endregion

namespace JadEngine.Mathematics.Interpolation
{

    /// <summary>
    /// Interpolation calculator for Vector2 values
    /// </summary>
    public struct Vector2Calculator : IJInterpolationCalculator<Vector2>
    {
        #region Methods
        /// <summary>
        /// Add two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The sum of the vectors</returns>
        public Vector2 Add(Vector2 a, Vector2 b)
        {
            return Vector2.Add(a, b);
        }

        /// <summary>
        /// Subtract two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The difference (a - b)</returns>
        public Vector2 Subtract(Vector2 a, Vector2 b)
        {
            return Vector2.Subtract(a, b);
        }

        /// <summary>
        /// Scale a vector by a double
        /// </summary>
        /// <param name="a">The Vector</param>
        /// <param name="b">Scale factor</param>
        /// <returns>The scaled vector</returns>
        public Vector2 Scale(Vector2 a, double b)
        {
            return Vector2.Multiply(a, (float)b);
        }

        /// <summary>
        /// Return the distance between two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The distance</returns>
        public double IntervalSize(Vector2 a, Vector2 b)
        {
            return (double)Vector2.Length(b - a);
        }

        /// <summary>
        /// Check to see if the vectors are equal
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>True if a==b</returns>
        public bool Equals(Vector2 a, Vector2 b)
        {
            return (a == b);
        }

        #endregion
    }

    /// <summary>
    /// Interpolation calculator for Vector3 values
    /// </summary>
    public struct Vector3Calculator : IJInterpolationCalculator<Vector3>
    {
        #region Methods
        /// <summary>
        /// Add two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The sum of the vectors</returns>
        public Vector3 Add(Vector3 a, Vector3 b)
        {
            return Vector3.Add(a, b);
        }

        /// <summary>
        /// Subtract two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The difference (a - b)</returns>
        public Vector3 Subtract(Vector3 a, Vector3 b)
        {
            return Vector3.Subtract(a, b);
        }

        /// <summary>
        /// Scale a vector by a double
        /// </summary>
        /// <param name="a">The Vector</param>
        /// <param name="b">Scale factor</param>
        /// <returns>The scaled vector</returns>
        public Vector3 Scale(Vector3 a, double b)
        {
            return Vector3.Multiply(a, (float)b);
        }
        
        /// <summary>
        /// Return the distance between two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The distance</returns>
        public double IntervalSize(Vector3 a, Vector3 b)
        {
            return (double)Vector3.Length(b - a);
        }

        /// <summary>
        /// Check to see if the vectors are equal
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>True if a==b</returns>
        public bool Equals(Vector3 a, Vector3 b)
        {
            return (a == b);
        }

        #endregion
    }

    /// <summary>
    /// Interpolation calculator for Vector4 values
    /// </summary>
    public struct Vector4Calculator : IJInterpolationCalculator<Vector4>
    {
        #region Methods
        /// <summary>
        /// Add two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The sum of the vectors</returns>
        public Vector4 Add(Vector4 a, Vector4 b)
        {
            return Vector4.Add(a, b);
        }

        /// <summary>
        /// Subtract two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The difference (a - b)</returns>
        public Vector4 Subtract(Vector4 a, Vector4 b)
        {
            return Vector4.Subtract(a, b);
        }

        /// <summary>
        /// Scale a vector by a double
        /// </summary>
        /// <param name="a">The Vector</param>
        /// <param name="b">Scale factor</param>
        /// <returns>The scaled vector</returns>
        public Vector4 Scale(Vector4 a, double b)
        {
            return Vector4.Multiply(a, (float)b);
        }

        /// <summary>
        /// Return the distance between two vectors
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>The distance</returns>
        public double IntervalSize(Vector4 a, Vector4 b)
        {
            return (double)Vector4.Length(b - a);
        }

        /// <summary>
        /// Check to see if the vectors are equal
        /// </summary>
        /// <param name="a">First vector</param>
        /// <param name="b">Second vector</param>
        /// <returns>True if a==b</returns>
        public bool Equals(Vector4 a, Vector4 b)
        {
            return (a == b);
        }

        #endregion
    }

}
