#region Using directives

using System;

using Microsoft.DirectX;

using JadEngine.Core;
using JadEngine.Scene;

#endregion

namespace JadEngine.Particles
{
	/// <summary>
	/// TODO
	/// </summary>
	public enum JParticleOrientation
	{
		/// <summary>
		/// TODO
		/// </summary>
		View,
		
		/// <summary>
		/// TODO
		/// </summary>
		Aimed
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JParticle //: IComparable
	{
		#region Fields

		/// <summary>
		/// Life time of the particle
		/// </summary>
		private float lifeTime;

		/// <summary>
		/// Actual age of the particle
		/// </summary>
		private float age;

		/// <summary>
		/// Width of the particle
		/// </summary>
		private float width;
		
		/// <summary>
		/// Height of the particle
		/// </summary>
		private float height;
		
		/// <summary>
		/// Increment of the particle size in each frame
		/// </summary>
		private float sizeDelta;

		/// <summary>
		/// Aspect ratio of the dimensions of the particle in each frame
		/// </summary>
		private float aspect;
		
		/// <summary>
		/// Increment of the aspect ratio of the dimesions of the particle in each frame
		/// </summary>
		private float aspectDelta;

		/// <summary>
		/// Actual color of the particle
		/// </summary>
        private JColor color;
        
		/// <summary>
		/// Increment of the color of the particle in each frame
		/// </summary>
        private JColor colorDelta;

        /// <summary>
        /// Actual position of the particle
        /// </summary>
		private Vector3 position;
		
		/// <summary>
		/// Previous position of the particle
		/// </summary>
		private Vector3 oldPosition;
		
		/// <summary>
		/// Direction of movement of the particle
		/// </summary>
		private Vector3 direction;
		
		/// <summary>
		/// Velocity of the particle
		/// </summary>
		private Vector3 velocity;

		/// <summary>
		/// Actual speed of the particle
		/// </summary>
		private float speed;
		
		/// <summary>
		/// Increment of the speed of the particle in each frame
		/// </summary>
		private float speedDelta;

		/// <summary>
		/// Actual rotation of the particle
		/// </summary>
		private float angle;
		
		/// <summary>
		/// Increment of the rotation of the particle in each frame</summary>
		private float angleDelta;

		/// <summary>
		/// Reference to the particle system that contains this particle
		/// </summary>
		private JParticleSystem parent;

		/// <summary>
		/// TODO
		/// </summary>
		private JParticle[] trailSamples;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public JParticle()
		{
			lifeTime	= 0.0f;
			age			= -1.0f;

			width		= 0.0f;
			height		= 0.0f;
			sizeDelta	= 0.0f;

			aspect		= 0.0f;
			aspectDelta = 1.0f;

            color		= new JColor();
            colorDelta	= new JColor();

            position	= new Vector3();
			oldPosition	= new Vector3();
			direction	= new Vector3();
			velocity	= new Vector3();

			speed		= 0.0f;
			speedDelta	= 0.0f;

			angle		= 0.0f;
			angleDelta	= 0.0f;

			parent		= null;
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the life time of the particle
		/// </summary>
		public float LifeTime
		{
			get { return lifeTime; }
			set { lifeTime = value; }
		}

		/// <summary>
		/// Gets or sets the actual age of the particle
		/// </summary>
		public float Age
		{
			get { return age; }
			set { age = value; }
		}

		/// <summary>
		/// Gets or sets the actual width of the particle
		/// </summary>
		public float Width
		{
			get { return width; }
			set { width = value; }
		}

		/// <summary>
		/// Gets or sets the actual height of the particle
		/// </summary>
		public float Height
		{
			get  { return height; }
			set { height = value; }
		}

		/// <summary>
		/// Gets or sets the size increment of the particle in each frame
		/// </summary>
		public float SizeDelta
		{
			get { return sizeDelta; }
			set { sizeDelta = value; }
		}

		/// <summary>
		/// Gets or sets the aspect ratio of the dimensions of the particle
		/// </summary>
		public float Aspect
		{
			get { return aspect; }
			set { aspect = value; }
		}

		/// <summary>
		/// Gets or sets the increment of the aspect ratio of the dimensions of the particle in each frame
		/// </summary>
		public float AspectDelta
		{
			get { return aspectDelta; }
			set { aspectDelta = value; }
		}

		/// <summary>
		/// Gets or sets the actual color of the particle
		/// </summary>
		public JColor Color
		{
			get { return color; }
			set { color = value; }
		}

		/// <summary>
		/// Gets or sets the color increment of the particle in each frame
		/// </summary>
		public JColor ColorDelta
		{
			get { return colorDelta; }
			set { colorDelta = value; }
		}

		/// <summary>
		/// Gets or sets the actual position of the particle
		/// </summary>
		public Vector3 Position
		{
			get { return position; }
			set { position = value; }
		}

		/// <summary>
		/// Gets or sets the previous position of the particle
		/// </summary>
		public Vector3 PreviousPosition
		{
			get { return oldPosition; }
			set { oldPosition = value; }
		}

		/// <summary>
		/// Gets or sets the actual direction in which the particle moves
		/// </summary>
		public Vector3 Direction
		{
			get { return direction; }
			set { direction = value; }
		}

		/// <summary>
		/// Gets or sets the velocity of the particle</summary>
		public Vector3 Velocity
		{
			get { return velocity; }
			set { velocity = value; }
		}

		/// <summary>
		/// Gets or sets the actual speed of the particle
		/// </summary>
		public float Speed
		{
			get { return speed; }
			set { speed = value; }
		}

		/// <summary>
		/// Gets or sets the increment of speed of the particle in each frame
		/// </summary>
		public float SpeedDelta
		{
			get { return speedDelta; }
			set { speedDelta = value; }
		}

		/// <summary>
		/// Gets or sets the rotation angle of the particle
		/// Retorna/Asigna el ngulo de rotacin en cada frame
		/// </summary>
		public float Angle
		{
			get { return angle; }
			set { angle = value; }
		}

		/// <summary>
		/// Gets or sets the rotation angle increment of the particle in each frame
		/// </summary>
		public float AngleDelta
		{
			get { return angleDelta; }
			set { angleDelta = value; }
		}

		/// <summary>
		/// Gets or sets the particle system that contains this particle
		/// </summary>
		public JParticleSystem Parent
		{
			get { return parent; }
			set { parent = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public JParticle[] TrailSamples
		{
			get { return trailSamples; }
			set { trailSamples = value; }
		}

		#endregion
		
		#region Methods

		/// <summary>
		/// Updates the properties of the particle
		/// </summary>
		/// <param name="deltaTime">Time since last update</param>
		/// <returns>Returns true if the particle continues alive, false if its life time has run out</returns>
		public bool Update(float deltaTime)
		{
			// Test if the particle is going to die in this update
			if (age + deltaTime >= lifeTime)
			{
				// Sadly the particle is going to die, so dont update it :(
				age = -1.0f;
				
				// R.I.P.
				return false;
			}

			// Update the particle age
			age += deltaTime;

			// Update the particle size
			width  += sizeDelta * deltaTime;
			height = width * aspect;

			// Update the aspect ratio
			aspect += aspectDelta * deltaTime;

			// Update the color of the particle
			color += colorDelta * deltaTime;			

			// Update the velocity
			velocity += direction * (deltaTime * speedDelta);

			// Add gravity
			velocity += parent.Gravity * deltaTime;

			// Save the position before modifying it
			oldPosition = position;

			// Update the position of the particle
			position += velocity * deltaTime;

			// Update the rotation angle of the particle
			angle += angleDelta * deltaTime;

			// Update the samples that form the particle trail
			if (parent.OrientationType == JParticleOrientation.Aimed)
			{
				trailSamples[0].Position = position;
				
				float dt = 0.0f;
				for (uint i = 1; i < parent.NumberOfTrailSamples; i++)
				{
					dt += parent.TrailTimeDelta;

					// Calculate the position of the actual sample
					Vector3 v = velocity + parent.Gravity * dt;
					trailSamples[i].Position = position + v * dt;
				}
			}

			return true;
		}

		/// <summary>
		/// Compares this particle with another particle
		/// </summary>
		/// <param name="rhs">The particle to compare to</param>
		/// <returns>TODO</returns>
		//public int CompareTo(Object rhs)
		//{
		//    JParticle p = (JParticle)rhs;

		//    Vector3 p1 = Vector3.TransformCoordinate(position, Jad.Scene.Camera.View);
		//    Vector3 p2 = Vector3.TransformCoordinate(p.Position, Jad.Scene.Camera.View);

		//    float res = p1.Z - p2.Z;

		//    return ((res > 0.0f) ? -1 : (res < 0.0f) ? 1 : 0 );
		//}

		#endregion Mtodos
	}
}
