#region Using directives

using JadEngine.Scene;
using Microsoft.DirectX;
using System;
using JadEngine.Video;
using JadEngine.Core;
using Microsoft.DirectX.Direct3D;

#endregion

/*
 * 30/03/08
 *	-Renamed from JParticleEffects to JParticleEffectManager
 */

namespace JadEngine.Particles
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JParticleEffectManager : JObjectManager<JParticleEffect>
	{
		#region Custom Gizmo Class
		/// <summary>
		/// TODO
		/// </summary>
		private class JParticleEffectGizmo : JGizmo
		{
			#region Constructor

			/// <summary>
			/// Default constructor
			/// </summary>
			public JParticleEffectGizmo()
				: base("particleEffects.tga")
			{
				// Gizmo lines color
				Color = new JColor(0.0f, 0.0f, 1.0f, 1.0f);

				// Allocate memory for the vertices
				NumVertices = 8;
				Vertices = new CustomVertex.PositionOnly[NumVertices];

				// Allocate memory for the indices
				NumIndices = 24;
				Indices = new short[NumIndices];

				// Define the indices
				Indices[0] = 0;	// Top
				Indices[1] = 1;
				Indices[2] = 1;
				Indices[3] = 2;
				Indices[4] = 2;
				Indices[5] = 3;
				Indices[6] = 3;
				Indices[7] = 0;
				Indices[8] = 4;	// Bottom
				Indices[9] = 5;
				Indices[10] = 5;
				Indices[11] = 6;
				Indices[12] = 6;
				Indices[13] = 7;
				Indices[14] = 7;
				Indices[15] = 4;
				Indices[16] = 0;	// Right-Far
				Indices[17] = 6;
				Indices[18] = 1;	// Right-Near
				Indices[19] = 7;
				Indices[20] = 3;	// Left-Far
				Indices[21] = 5;
				Indices[22] = 2;	// Left-Near
				Indices[23] = 4;
			}

			#endregion

			#region Methods

			/// <summary>
			/// Renders the gizmo
			/// </summary>
			/// <param name="node">Node associated with this gizmo</param>
			public override void Render(JNode node, JCamera camera)
			{
				Vector3 min;
				Vector3 max;

				JParticleEffect fx = node as JParticleEffect;

				min = fx.Boundings.LocalBoundingBox.Min;
				max = fx.Boundings.LocalBoundingBox.Max;

				// Define the vertices
				Vertices[0] = new CustomVertex.PositionOnly(max.X, max.Y, max.Z);
				Vertices[1] = new CustomVertex.PositionOnly(max.X, max.Y, min.Z);
				Vertices[2] = new CustomVertex.PositionOnly(min.X, max.Y, min.Z);
				Vertices[3] = new CustomVertex.PositionOnly(min.X, max.Y, max.Z);

				Vertices[4] = new CustomVertex.PositionOnly(min.X, min.Y, min.Z);
				Vertices[5] = new CustomVertex.PositionOnly(min.X, min.Y, max.Z);
				Vertices[6] = new CustomVertex.PositionOnly(max.X, min.Y, max.Z);
				Vertices[7] = new CustomVertex.PositionOnly(max.X, min.Y, min.Z);

				// Render
				base.Render(node, camera);
			}

			#endregion
		}

		/// <summary>
		/// Our gizmo
		/// </summary>
		private static JGizmo _particleGizmo = new JParticleEffectGizmo();

		#endregion

		#region Methods

		/// <summary>
		/// Creates a new <see cref="JParticleEffect"/> instance and adds it to the internal list.
		/// </summary>
		/// <returns>The new instance.</returns>
		public override JParticleEffect Create()
		{
			return Create(null);
		}

		/// <summary>
		/// Creates a new particle effect
		/// </summary>
		/// <param name="name">Name of the effect</param>
		/// <param name="fileName">File to load the effect if it doesnt exist</param>
		/// <returns>A ParticleEffect object</returns>
		public JParticleEffect Create(string fileName)
		{
			JParticleEffect fx = null;

			//fx = Exists(name);

			if (fx == null)
			{
				if (fileName != null && fileName.Length > 0)
					fx = JParticleEffect.LoadFromFile(fileName); // Didnt find it, so load it

				//else
				//    fx = new JParticleEffect(name);

				// Set the effect name
				//fx.Name = name;

				// Add the particle effect to the list of effects
				Add(fx);

				// Set the gizmo for this type of node
				fx.Gizmo = JParticleEffectManager._particleGizmo;

				// The particle effect has the root node as parent
				Jad.ActiveScene.Root.AddChild(fx);
			}

			return fx;
		}

		/// <summary>
		/// Adds the specified effect to the list.
		/// </summary>
		/// <param name="effect">The effect to add.</param>
		public override void Add(JParticleEffect effect)
		{
			_items.Add(effect);
		}

		public override void Delete(JParticleEffect effect)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public override void Cleanup()
		{
			_items.Clear();
		}

		/// <summary>
		/// TODO
		/// </summary>
		public void Update(JCamera camera)
		{
			for (int i = 0; i < Count; i++)
			{
				JParticleEffect fx = this[i];

				if (!fx.Update(camera))
					Delete(fx);
			}
		}

		/// <summary>
		/// TODO
		/// </summary>
		public void Render()
		{
			for (int i = 0; i < Count; i++)
			{
				JParticleEffect fx = this[i];

				if (!fx.Hide)
					fx.Render();
			}
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="start">TODO</param>
		/// <param name="end">TODO</param>
		/// <returns>TODO</returns>
		public JParticleEffect Intersect(Vector3 start, Vector3 end)
		{
			JParticleEffect colision = null;
			float distance = float.MaxValue;

			// Create an object to collide
			JMesh mesh = Jad.Video.Meshes.Primitives.CreateSphere(0.25f, 4, 4);

			for (int j = 0; j < Count; j++)
			{
				JParticleEffect m = this[j];

				// Transform the vectors to Object space
				Vector3 startOS = start, endOS = end;

				Matrix toObjectSpace = Matrix.Invert(m.Transform.WorldTM);

				startOS.TransformCoordinate(toObjectSpace);
				endOS.TransformCoordinate(toObjectSpace);

				float dist;

				Vector3 rayDir = Vector3.Normalize(endOS - startOS);

				if (mesh.Intersect(startOS, rayDir, false, out dist))
				{
					// Only the nearest one is needed
					if (dist < distance)
					{
						colision = m;
						distance = dist;
					}
				}
			}

			Jad.Video.Meshes.Delete(mesh);

			return colision;
		}

		#endregion
	}
}