#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;
using JadEngine.Scene;
using JadEngine.Video;

#endregion

namespace JadEngine.Particles
{
	#region Emitter types

	/// <summary>
	/// TODO
	/// </summary>
	public abstract class JEmitter
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JMersenneTwister rand;

		#endregion

		#region Constructors
		
		/// <summary>
		/// Default constructor
		/// </summary>
		public JEmitter()
		{
			rand = new JMersenneTwister((uint)DateTime.Now.Millisecond);
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public abstract Vector3 EmitPosition();

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="direction">TODO</param>
		/// <param name="halfAngle">TODO</param>
		/// <returns>TODO</returns>
		public Vector3 EmitDirection(Vector3 direction, float halfAngle)
		{
			float pitch	= rand.NextFloat(-halfAngle, halfAngle);
			float yaw	= rand.NextFloat(-halfAngle, halfAngle);
			float roll	= rand.NextFloat(-halfAngle, halfAngle);

			Matrix rotation = Matrix.RotationYawPitchRoll(Geometry.DegreeToRadian(yaw), Geometry.DegreeToRadian(pitch), Geometry.DegreeToRadian(roll));

			return Vector3.TransformNormal(direction, rotation);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JEmitterPoint : JEmitter
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 point;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="point">TODO</param>
		public JEmitterPoint(Vector3 point) : base()
		{
			this.point = point;
		}

		#endregion

		#region Properties
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <value>TODO</value>
		public Vector3 Point
		{
			get { return point; }
			set { point = value; }
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public override Vector3 EmitPosition()
		{
			return point;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JEmitterLine : JEmitter
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 point1;
		
		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 point2;

		#endregion

		#region Constructors
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="point1">TODO</param>
		/// <param name="point2">TODO</param>
		public JEmitterLine(Vector3 point1, Vector3 point2) : base()
		{
			this.point1 = point1;
			this.point2 = point2;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 StartPoint
		{
			get { return point1; }
			set { point1 = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 EndPoint
		{
			get { return point2; }
			set { point2 = value; }
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public override Vector3 EmitPosition()
		{
			return (point1 + (point2 - point1) * (float)rand.NextDouble());
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JEmitterSphere : JEmitter
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		private bool onSphere;
		
		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 origin;
		
		/// <summary>
		/// TODO
		/// </summary>
		private float radius;

		#endregion

		#region Consctructors
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="origin">TODO</param>
		/// <param name="radius">TODO</param>
		/// <param name="onSphere">TODO</param>
		public JEmitterSphere(Vector3 origin, float radius, bool onSphere) : base()
		{
			this.origin		= origin;
			this.radius		= radius;
			this.onSphere	= onSphere;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public bool OnSphere
		{
			get { return onSphere; }
			set { onSphere = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Origin
		{
			get { return origin; }
			set { origin = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public float Radius
		{
			get { return radius; }
			set { radius = value; }
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public override Vector3 EmitPosition()
		{
			Vector3 res;

			if (onSphere)
				res = EmitDirection(new Vector3(1.0f, 1.0f, 1.0f), 180.0f)*radius + origin;
			
			else
			{
				float x = rand.NextFloat(-radius, radius) + origin.X;
				float y = rand.NextFloat(-radius, radius) + origin.Y;
				float z = rand.NextFloat(-radius, radius) + origin.Z;

				res = new Vector3(x, y, z);
			}

			return res;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JEmitterBox : JEmitter
	{
		#region Fields
		
		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 min;
		
		/// <summary>
		/// TODO
		/// </summary>
		private Vector3 max;

		#endregion

		#region Constructors
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="min">TODO</param>
		/// <param name="max">TODO</param>
		public JEmitterBox(Vector3 min, Vector3 max) : base()
		{
			this.min = min;
			this.max = max;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 MinPoint
		{
			get { return min; }
			set { min = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 MaxPoint
		{
			get { return max; }
			set { max = value; }
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public override Vector3 EmitPosition()
		{
			float x = rand.NextFloat(min.X, max.X);
			float y = rand.NextFloat(min.Y, max.Y);
			float z = rand.NextFloat(min.Z, max.Z);

			return new Vector3(x, y, z);
		}

		#endregion
	}

	#endregion

	/// <summary>
	/// Struct that contains the properties of a particle system
	/// </summary>
	public struct JParticleSystemProfile
	{
		#region Fields
		
		/// <summary>
		/// Life time of the particle system
		/// </summary>
		public float lifeTime;

		/// <summary>
		/// Number of particles that will be emited in the life time of the particle system
		/// </summary>
		public uint numParticles;

		/// <summary>
		/// Number of particles to emit in a second
		/// </summary>
		public float birthRate;
		/// <summary>
		/// Tells if the emission will be constant or will be specified by <see cref="birthRate"/>
		/// </summary>
		public bool constBirthRate;

		/// <summary>
		/// Initial size of the system particles
		/// </summary>
		public float sizeStart;
		/// <summary>
		/// Final size of the system particles
		/// </summary>
		public float sizeEnd;

		/// <summary>
		/// Starting aspect ratio for the size of the system particles
		/// </summary>
		/// <remarks>
		/// Thanks to this field we can create non-squared particles. The width ofthe particle
		/// is sizeStart/sizeEnd but the height will depend on this field (height = width * aspect)
		/// </remarks>
		public float aspectStart;
		/// <summary>
		/// Final aspect ratio for the size of the system particles
		/// </summary>
		/// <see cref="aspectStart"/>
		public float aspectEnd;

		/// <summary>
		/// Initial speed of the system particles
		/// </summary>
		public float speedStart;
		/// <summary>
		/// Final speed of the particles of the system</summary>
		public float speedEnd;

		/// <summary>
		/// Starting rotation angle of the system particles
		/// </summary>
		public float angleStart;
		/// <summary>
		/// Final rotation angle of the system particles
		/// </summary>
		public float angleEnd;

		/// <summary>
		/// Half angle of the imaginary cone that emits the particles
		/// </summary>
		public float theta;

		/// <summary>
		/// Starting color of the system particles
		/// </summary>
        public JColor colorStart;
        /// <summary>
        /// Final color of the system particles
        /// </summary>
        public JColor colorEnd;

        /// <summary>
        /// Position of the particle system
        /// </summary>
		public Vector3 position;
		/// <summary>
		/// Direction of the emission of the particles
		/// </summary>
		public Vector3 direction;
		/// <summary>
		/// Gravity force that affects the particle system
		/// </summary>
		public Vector3 gravity;

		/// <summary>
		/// Indicates the method that will be used to orientate the particle</summary>
		public JParticleOrientation orientationType;

		/// <summary>
		/// Number of segments that will form the trail in "Aimed" type particles</summary>
		public uint trailSegments;
		/// <summary>
		/// Time between the starting and the end position of the trail in "Aimed" type particles
		/// </summary>
		public float trailTime;

		/// <summary>
		/// Indicates if the particle system will loop forever or only once
		/// </summary>
		public bool loop;

		/// <summary>
		/// Indicates if the particles must be sorted by distance
		/// </summary>
		public bool sort;

		/// <summary>
		/// TODO
		/// </summary>
		public JEmitter emitter;

		/// <summary>
		/// Material that will be used by the system particles
		/// </summary>
		public JMaterial material;

		#endregion

		#region Methods
		
		/// <summary>
		/// Creates a default struct
		/// </summary>
		/// <returns>Returns a JParticleSystemProfile struct with the default values</returns>
		public static JParticleSystemProfile CreateDefaultProfile()
		{
			JParticleSystemProfile defProfile = new JParticleSystemProfile();

			defProfile.loop = true;
			defProfile.lifeTime = 2.0f;
			defProfile.numParticles = 30;
			defProfile.birthRate = 0.0f;
			defProfile.constBirthRate = true;
			defProfile.sizeStart = 0.1f;
			defProfile.sizeEnd = 0.3f;
			defProfile.aspectStart = 1.0f;
			defProfile.aspectEnd = 1.0f;
			defProfile.speedStart = 0.2f;
			defProfile.speedEnd = 0.2f;
			defProfile.angleStart = 0.0f;
			defProfile.angleEnd = 0.0f;
			defProfile.theta = 10.0f;
			defProfile.colorStart = new JColor(1.0f, 1.0f, 1.0f, 0.5f);
			defProfile.colorEnd = new JColor(0.0f, 0.0f, 0.0f, 0.0f);
			defProfile.position = Vector3.Empty;
			defProfile.direction = new Vector3(0.0f, 0.0f, 1.0f);
			defProfile.gravity = Vector3.Empty;
			defProfile.orientationType = JParticleOrientation.View;
			defProfile.trailSegments = 3;
			defProfile.trailTime = 0.2f;
			defProfile.sort = false;
			defProfile.emitter = new JEmitterPoint(defProfile.position);
			//defProfile.material = Jad.Scene.Materials.Load("default_particle");

			return defProfile;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JParticleSystem
	{
		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		struct JParticlesInstances
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			internal Vector3 Offset;
			
			/// <summary>
			/// TODO
			/// </summary>
			internal JMeshObject MeshObject;

			#endregion
		}

		#endregion

		#region Fields

		/// <summary>
		/// Fields of the particle system
		/// </summary>
		private JParticleSystemProfile profile;

		/// <summary>
		/// Number of alive particles in a given moment
		/// </summary>
		private uint numParticlesAlive;

		/// <summary>
		/// Number of particles that will be emited in a second
		/// </summary>
		private float numParticlesPerSecond;
		/// <summary>
		/// Number of particles that couldnt be emitted in the previous frame
		/// </summary>
		private float emissionResidue;

		/// <summary>
		/// List of particles that the particle system holds
		/// </summary>
		private List<JParticle> particles;

		/// <summary>
		/// Time increment between the subdivisions in "Aimed" type particles
		/// </summary>
		private float trailTimeDelta;
		/// <summary>
		/// Number of positions that will form the trail in "Aimed" type particles</summary>
		private uint numTrailSamples;

		/// <summary>
		/// BoundingBox that contains the particle system
		/// </summary>
        private JBoundingBox3 boundingBox;

        /// <summary>
        /// TODO
        /// </summary>
		private int totalParticles;

		/// <summary>
		/// ParticleEffect that contains this particle system
		/// </summary>
		private JParticleEffect parent;

		/// <summary>
		/// Pseudo random number generator
		/// </summary>
		private JMersenneTwister rand;

		/// <summary>
		/// Effect used for the render in the particle system
		/// </summary>
		private JEffect effect;

		/// <summary>
		/// TODO
		/// </summary>
		private int numIndices;

		/// <summary>
		/// TODO
		/// </summary>
		private int numVertices;

		/// <summary>
		/// List of instances
		/// </summary>
		private List<JParticlesInstances> instances;

		/// <summary>
		/// TODO
		/// </summary>
		internal JMeshObject meshObject;

		/// <summary>
		/// TODO
		/// </summary>
		JVertexBuffer vertexBuffer;

		/// <summary>
		/// TODO
		/// </summary>
		JIndexBuffer indexBuffer;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="parent">TODO</param>
		/// <param name="config">TODO</param>
		public JParticleSystem(JParticleEffect parent, JParticleSystemProfile config)
		{
			profile = config;

			if (profile.constBirthRate)
				numParticlesPerSecond = profile.numParticles / profile.lifeTime;
			
			else
				numParticlesPerSecond = profile.birthRate;

			numParticlesAlive	= 0;
			emissionResidue		= 0.0f;

			numTrailSamples = profile.trailSegments + 2;
			trailTimeDelta = profile.trailTime / (profile.trailSegments + 1);

			particles = new List<JParticle>();

			for (int i = 0; i < profile.numParticles; i++)
			{
				JParticle particle = new JParticle();

				particle.TrailSamples = new JParticle[numTrailSamples];

				for (int sample = 0; sample < numTrailSamples; sample++)
					particle.TrailSamples[sample] = new JParticle();

				particles.Add(particle);
			}

			rand = new JMersenneTwister((uint)DateTime.Now.Millisecond);

			int numVertices = 0, numIndices = 0;

			switch (profile.orientationType)
			{
				case JParticleOrientation.View:
					numIndices = (int)(profile.numParticles * 6);
					numVertices = (int)(profile.numParticles * 4);
					break;
					
				case JParticleOrientation.Aimed:
					numIndices = (int)(profile.numParticles * (numTrailSamples - 1) * 6);
					numVertices = (int)(profile.numParticles * numTrailSamples * 2);
					break;
			}

            boundingBox = new JBoundingBox3();

            this.parent = parent;

			vertexBuffer = new JVertexBuffer(numVertices);

			vertexBuffer.Position = new Vector3[numVertices];
            vertexBuffer.Normal = new Vector3[numVertices];
            vertexBuffer.Tangent = new Vector3[numVertices];
            vertexBuffer.Diffuse = new int[numVertices];
            vertexBuffer.TexCoord = new Vector2[1, numVertices];

            indexBuffer = new JIndexBuffer(false, numIndices);

			//Type t = HMeshObjectParticleSystem.Type;
            //meshObject = (HMeshObjectParticleSystem)Jad.Scene.MeshObjects.Create("particleSystem",t);
			meshObject = Jad.ActiveScene.MeshObjects.Create(true);
			//meshObject.Material = new JMaterial[1];
			meshObject.Materials.Add(Material);
            Material.Priority = JMaterialPriority.PostProcess;
            Material.Layers[0].NormalType = JNormalType.Vertex;

            JMesh mesh = meshObject.Mesh;

			mesh.VertexBuffer = vertexBuffer;

			mesh.IndexBuffer = indexBuffer;

            mesh.TangentComputed = true;
            mesh.CenteredAtOrigin = true;

			instances = new List<JParticlesInstances>();
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the material that the system particles use
		/// </summary>
		[Browsable(false)]
		[Description("The material that the system particles use")]
		public JMaterial Material
		{
			get { return profile.material; }
			set { profile.material = value; }
		}

		/// <summary>
		/// Gets or sets the life time of the particle system</summary>
		[Browsable(true)]
		[Description("The life time of the particle system")]
		public float LifeTime
		{
			get { return profile.lifeTime; }
			set { profile.lifeTime = value; }
		}

		/// <summary>
		/// Gets or sets the number of particles to emit in one second
		/// </summary>
		[Browsable(true)]
		[Description("Number of particles to emit in one second")]
		public float BirthRate
		{
			get { return profile.birthRate; }
			set { profile.birthRate = value; }
		}

		/// <summary>
		/// Gets or sets if the particle emission will be constant or specified by <see cref="JParticleSystemProfile.birthRate"/>
		/// </summary>
		[Browsable(true)]
		[Description("If the particle emission will be constant or specified by the birthRate")]
		public bool ConstantBirthRate
		{
			get { return profile.constBirthRate; }
			set { profile.constBirthRate = value; }
		}

		/// <summary>
		/// Gets or sets the number of particles that will be emited in the life time of the particle system
		/// </summary>
		[Browsable(true)]
		[Description("Number of particles that will be emited in the life time of the particle system")]
		public uint NumberOfParticles
		{
			get { return profile.numParticles; }
			set{ profile.numParticles = value; }
		}

		/// <summary>
		/// Gets the number of alive particles in a given moment
		/// </summary>
		[Browsable(false)]
		public uint NumberOfParticlesAlive
		{
			get { return numParticlesAlive; }
		}

		/// <summary>
		/// Gets the number of particles that will be emitted in a second
		/// </summary>
		[Browsable(false)]
		public float NumberOfParticlesPerSecond
		{
			get { return numParticlesPerSecond; }
		}

		/// <summary>
		/// Gets or sets the initial size of the system particles</summary>
		[Browsable(true)]
		[Description("The initial size of the system particles")]
		public float SizeStart
		{
			get { return profile.sizeStart; }
			set { profile.sizeStart = value; }
		}

		/// <summary>
		/// Gets or sets the final size of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The final size of the system particles")]
		public float SizeEnd
		{
			get { return profile.sizeEnd; }
			set { profile.sizeEnd = value; }
		}

		/// <summary>
		/// Gets or sets the initial aspect ratio of the system particles</summary>
		[Browsable(true)]
		[Description("The initial aspect ratio of the system particles (height = width * aspect)")]
		public float AspectStart
		{
			get { return profile.aspectStart; }
			set { profile.aspectStart = value; }
		}

		/// <summary>
		/// Gets or sets the final aspect ratio of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The final aspect ratio of the system particles (height = width * aspect)")]
		public float AspectEnd
		{
			get { return profile.aspectEnd; }
			set { profile.aspectEnd = value; }
		}

		/// <summary>
		/// Gets or sets the initial speed of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The initial speed of the system particles")]
		public float SpeedStart
		{
			get { return profile.speedStart; }
			set { profile.speedStart = value; }
		}

		/// <summary>
		/// Gets or sets the final speed of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The final speed of the system particles")]
		public float SpeedEnd
		{
			get { return profile.speedEnd; }
			set { profile.speedEnd = value; }
		}

		/// <summary>
		/// Gets or sets the initial rotation angle of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The initial rotation angle of the system particles")]
		public float AngleStart
		{
			get { return profile.angleStart; }
			set { profile.angleStart = value; }
		}

		/// <summary>
		/// Gets or sets the final rotation angle of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The final rotation angle of the system particles")]
		public float AngleEnd
		{
			get { return profile.angleEnd; }
			set { profile.angleEnd = value; }
		}

		/// <summary>
		/// Gets or sets the half angle of the imaginary cone that emits the particles
		/// </summary>
		[Browsable(true)]
		[Description("The half angle of the imaginary cone that emits the particles")]
		public float Theta
		{
			get { return profile.theta; }
			set { profile.theta = value; }
		}

		/// <summary>
		/// Gets or sets the initial color of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The initial color of the system particles")]
		//[TypeConverter(typeof(ColorTypeConverter))]
		//[EditorAttribute(typeof(ColorTypeEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public JColor ColorStart
		{
			get { return profile.colorStart; }
			set { profile.colorStart = value; }
		}

		/// <summary>
		/// Gets or sets the end color of the system particles
		/// </summary>
		[Browsable(true)]
		[Description("The end color of the system particles")]
		//[TypeConverter(typeof(ColorTypeConverter))]
		//[EditorAttribute(typeof(ColorTypeEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public JColor ColorEnd
		{
			get { return profile.colorEnd; }
			set { profile.colorEnd = value; }
		}

		/// <summary>
		/// Gets or sets the position of the particle system
		/// </summary>
		[Browsable(true)]
		[Description("The position of the particle system")]
		//[TypeConverter(typeof(Vector3TypeConverter))]
		//[EditorAttribute(typeof(Vector3TypeEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public Vector3 Position
		{
			get { return profile.position; }
			set { profile.position = value; }
		}

		/// <summary>
		/// Gets or sets the direction of the particle emission
		/// </summary>
		[Browsable(true)]
		[Description("The direction of the particle emission")]
		//[TypeConverter(typeof(Vector3TypeConverter))]
		//[EditorAttribute(typeof(Vector3TypeEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public Vector3 Direction
		{
			get { return profile.direction; }
			set { profile.direction = value; }
		}

		/// <summary>
		/// Gets or sets the gravity force that affects the particle system
		/// </summary>
		[Browsable(true)]
		[Description("The gravity force that affects the particle system")]
		//[TypeConverter(typeof(Vector3TypeConverter))]
		//[EditorAttribute(typeof(Vector3TypeEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public Vector3 Gravity
		{
			get { return profile.gravity; }
			set { profile.gravity = value; }
		}

		/// <summary>
		/// Gets or sets the method that will be used to orientate the particle</summary>
		[Browsable(true)]
		[Description("The method that will be used to orientate the particle")]
		public JParticleOrientation OrientationType
		{
			get { return profile.orientationType; }
			set { profile.orientationType = value; }
		}

		/// <summary>
		/// Gets or sets the number of segments that will form the train in "Aimed" type particles
		/// </summary>
		[Browsable(true)]
		[Description("The number of segments that will form the train in \"Aimed\" type particles")]
		public uint TrailSegments
		{
			get { return profile.trailSegments; }
			set { profile.trailSegments = value; }
		}

		/// <summary>
		/// Gets or sets the time between the start and end positions of the trail in "Aimed" type particles
		/// </summary>
		[Browsable(true)]
		[Description("The time between the start and end positions of the trail in \"Aimed\" type particles")]
		public float TrailTime
		{
			get { return profile.trailTime; }
			set { profile.trailTime = value; }
		}

		/// <summary>
		/// Gets or sets if the particle system will loop forever or not</summary>
		[Browsable(true)]
		[Description("If the particle system will loop forever or not")]
		public bool Loop
		{
			get { return profile.loop; }
			set { profile.loop = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		[Browsable(false)]
		public JEmitter Emitter
		{
			get { return profile.emitter; }
			set { profile.emitter = value; }
		}

		/// <summary>
		/// Gets or sets the time increment between the subdivisons in "Aimed" type particles
		/// </summary>
		[Browsable(false)]
		public float TrailTimeDelta
		{
			get { return trailTimeDelta; }
			set { trailTimeDelta = value; }
		}
	
		/// <summary>
		/// Gets or sets the number of positions that will form the trail in "Aimed" type particles</summary>
		[Browsable(false)]
		public uint NumberOfTrailSamples
		{
			get { return numTrailSamples; }
			set { numTrailSamples = value; }
		}

		/// <summary>
		/// Gets or sets the bounding box that holds the particle system
		/// </summary>
		[Browsable(false)]
		public JBoundingBox3 BoundingBox
		{
			get { return boundingBox; }
			set { boundingBox = value; }
		}

		/// <summary>
		/// Gets or sets if the particles will be sorted using the distance
		/// </summary>
		[Browsable(true)]
		[Description("If the particles will be sorted using the distance")]
		public bool Sort
		{
			get { return profile.sort; }
			set { profile.sort = value; }
		}

		/// <summary>
		/// Gets or sets the JParticleEffect that contains the particle system</summary>
		[Browsable(false)]
		public JParticleEffect Parent
		{
			get { return parent; }
			set { parent = value; }
		}

		/// <summary>
		/// Gets or sets the JParticleSystemProfile struct that holds the properties of the particle system
		/// </summary>
		[Browsable(false)]
		public JParticleSystemProfile Profile
		{
			get { return profile; }
			set { profile = value; }
		}

		/// <summary>
		/// Gets the HMeshobject linked with this system
		/// </summary>
		[Browsable(false)]
		public JMeshObject MeshObject
		{
			get { return meshObject; }
		}

		#endregion 
		
		#region Methods

		/// <summary>
		/// Deletes the meshobject and instances
		/// </summary>
		internal void Destroy()
		{
			Jad.ActiveScene.MeshObjects.Delete(meshObject);

			// Now the instances
			foreach (JParticlesInstances instance in instances)
				Jad.ActiveScene.MeshObjects.Delete(instance.MeshObject);
		}

		/// <summary>
		/// Adds a new instance
		/// </summary>
		/// <param name="offset">The offset position</param>
		internal void AddInstance(Vector3 offset)
		{
			// Must add a JMeshObject for each effect
			
			JParticlesInstances pi = new JParticlesInstances();
			pi.Offset = offset;

			pi.MeshObject = meshObject.Clone(false);

			instances.Add(pi);
		}

		/// <summary>
		/// Updates the properties of the particle system
		/// </summary>
		/// <returns>Returns true if at least one particle is alive, false if otherwise</returns>
		public bool Update(JCamera camera)
		{
			// Get the time since the last frame
			float deltaTime = Jad.Timer.DeltaTime;

			// Reset the active particles counter
			numParticlesAlive = 0;

			// Update the active particles
			foreach (JParticle particle in particles)
			{
				// If the particle is active...
				if (particle.Age >= 0.0f)
				{
					// Update the particle
					// If this method returns true the particle continues alive
					if (particle.Update(deltaTime))
						numParticlesAlive++; // Increment the number of alive particles
				}
			}

			// If there arent alive particles, exit
			//if (numParticlesAlive == 0 && totalParticles > 0)
			//{
			//    totalParticles = 0;
			//    return false;
			//}

			// If the particle system loops forever or it doesnt loop forever and it hasnt produced the maximum
			// number of particles, create the new needed particles
			if (profile.loop || (!profile.loop && totalParticles < profile.numParticles))
			{
				// Calculate the number of particles needed
				float numParticlesNeeded = numParticlesPerSecond * deltaTime + emissionResidue;

				// Round that number
				uint numParticlesCreated = Convert.ToUInt32(numParticlesNeeded);

				// Save the number of particles that werent created (from the rounding)
				emissionResidue = numParticlesNeeded - numParticlesCreated;

				// If some particle must be created...
				if (numParticlesCreated > 0)
				{
					// For each particle search the inactives to use them
					//foreach (JParticle particle in particles)
					for (int p = 0; p < particles.Count; p++)
					{
						JParticle particle = particles[p];

						// Test that we have to create particles and that the actual particle is not alive
						if (numParticlesCreated <= 0 || particle.Age >= 0.0f)
							continue;

						// Life time of the particle
						particle.LifeTime = rand.NextFloat(profile.lifeTime * 0.7f, profile.lifeTime);
						//particle.LifeTime = profile.lifeTime;

						// Initial age of the particle
						particle.Age = rand.NextFloat(0.0f, deltaTime);

						// Size of the particle
						particle.Width = profile.sizeStart + rand.NextFloat(0.0f, particle.Width * 0.1f);
						particle.Height = particle.Width * profile.aspectStart;

						// Aspect ratio of the particle
						particle.Aspect = profile.aspectStart;
						particle.AspectDelta = (profile.aspectEnd - particle.Aspect) / particle.LifeTime;

						// Size increment of the particle each frame
						particle.SizeDelta = (profile.sizeEnd - particle.Width) / particle.LifeTime;
						// Aspect ratio increment of the particle dimensions each frame
						particle.AspectDelta = (profile.aspectEnd - profile.aspectStart) / particle.LifeTime;

						// Particle color
						particle.Color = profile.colorStart;
						particle.Color.Normalize();

						// Color increment of the particle each frame
						particle.ColorDelta = (profile.colorEnd - particle.Color) / particle.LifeTime;

						// Particle position
						particle.Position = profile.position + profile.emitter.EmitPosition();
						if (parent != null)
							particle.Position += parent.Transform.Position;

						particle.PreviousPosition = particle.Position;

						// Start the samples
						if (profile.orientationType == JParticleOrientation.Aimed)
							for (uint i = 0; i < numTrailSamples; i++)
								particle.TrailSamples[i].Position = particle.Position;


						// Particle speed
						particle.Speed = profile.speedStart + rand.NextFloat(0.0f, profile.speedEnd);
						// Speed increment of the particle each frame
						particle.SpeedDelta = (profile.speedEnd - particle.Speed) / particle.LifeTime;

						// Rotation angle of the particle
						particle.Angle = profile.angleStart;
						particle.AngleDelta = (profile.angleEnd - particle.Angle) / particle.LifeTime;

						// Velocity vector of the particle
						particle.Direction = Vector3.TransformNormal(profile.emitter.EmitDirection(profile.direction, profile.theta), parent.Transform.WorldTM);
						//particle.Direction = profile.emitter.EmitDirection(parent.Look, profile.theta);
						//System.Console.WriteLine(parent.Look.ToString());
						particle.Velocity = particle.Direction * particle.Speed;

						// Tell the particle its particle system
						particle.Parent = this;

						// Decrement the number of particles created 
						numParticlesCreated--;

						// TEST
						totalParticles++;
					}
				}
			}

			// Create the vertices and indices
			//JCamera camera = Jad.Scene.Camera;			
			Vector3 cameraUp = camera.Up;
			Vector3 cameraRight = camera.Right;
			Vector3 cameraLook = camera.Look;

			if (profile.sort)
				particles.Sort();

			// Reset the bounding box that contains the system
			boundingBox.Reset();

			numIndices = 0;
			numVertices = 0;

			foreach (JParticle particle in particles)
			{
				if (particle.Age < 0.0f)
					continue;

				float width = particle.Width * 0.5f;
				float height = particle.Height * 0.5f;
				switch (profile.orientationType)
				{
					case JParticleOrientation.View:
					{
						Vector3 pos		= particle.Position;
						Vector3 up = cameraUp * width;
						Vector3 right = cameraRight * height;
						
						int color = particle.Color.ToArgb();

						indexBuffer.Indices[0 + numIndices] = (0 + numVertices);
                        indexBuffer.Indices[1 + numIndices] = (1 + numVertices);
                        indexBuffer.Indices[2 + numIndices] = (2 + numVertices);
                        indexBuffer.Indices[3 + numIndices] = (0 + numVertices);
                        indexBuffer.Indices[4 + numIndices] = (2 + numVertices);
                        indexBuffer.Indices[5 + numIndices] = (3 + numVertices);

						vertexBuffer.Position[numVertices + 0] = -right + up;
						vertexBuffer.Position[numVertices + 1] = -right - up;
						vertexBuffer.Position[numVertices + 2] = right - up;
						vertexBuffer.Position[numVertices + 3] = right + up;

						Matrix rotZ = Matrix.RotationAxis(cameraLook, Geometry.DegreeToRadian(particle.Angle));
						Vector3 tangent = Vector3.TransformNormal(cameraRight,rotZ);
						for (int i = 0; i < 4; i++)
						{
							vertexBuffer.Position[i + numVertices] = pos + Vector3.TransformCoordinate(vertexBuffer.Position[i + numVertices], rotZ);
                            vertexBuffer.Normal[i + numVertices] = -cameraLook;
							vertexBuffer.Tangent[i + numVertices] = tangent;
                            vertexBuffer.Diffuse[i + numVertices] = color;

							boundingBox.Check(vertexBuffer.Position[i + numVertices]);
						}

						vertexBuffer.TexCoord[0, numVertices + 0].X = 0;
						vertexBuffer.TexCoord[0, numVertices + 0].Y = 0;
						vertexBuffer.TexCoord[0, numVertices + 1].X = 0;
						vertexBuffer.TexCoord[0, numVertices + 1].Y = 1;
						vertexBuffer.TexCoord[0, numVertices + 2].X = 1;
						vertexBuffer.TexCoord[0, numVertices + 2].Y = 1;
						vertexBuffer.TexCoord[0, numVertices + 3].X = 1;
						vertexBuffer.TexCoord[0, numVertices + 3].Y = 0;

						numIndices += 6;
						numVertices += 4;

						break;
					}
						
					case JParticleOrientation.Aimed:
					{
						float vInc = 1.0f / (numTrailSamples - 1);

						Vector3 right;
						Vector3 up;
						Vector3 pos1;
						Vector3 pos2;

						for (int i = 0; i < numTrailSamples; i++)
						{
							pos1 = particle.TrailSamples[i].Position;

							// Redim the bounding box to hold the trail
							boundingBox += pos1;

							if (i == numTrailSamples - 1)
							{
								pos2 = particle.TrailSamples[i - 1].Position;
								up = pos1 - pos2;
							}
							
							else
							{
								pos2 = particle.TrailSamples[i + 1].Position;
								up = pos2 - pos1;
							}

							right = Vector3.Normalize(Vector3.Cross(cameraLook, up)) * particle.Width;

							Vector3 v0 = pos1 - right;
							Vector3 v1 = pos1 + right;

							int color = particle.Color.ToArgb();

							// Normal and tangent are lacking
							int index=i * 2 + numVertices;
							vertexBuffer.Position[index] = v0;
							vertexBuffer.Diffuse[index] = color;
							vertexBuffer.TexCoord[0, index].X = 0;
							vertexBuffer.TexCoord[0, index].Y = 1.0f - (i * vInc);

							index++;
							vertexBuffer.Position[index] = v1;
							vertexBuffer.Diffuse[index] = color;
							vertexBuffer.TexCoord[0, index].X = 1.0f;
							vertexBuffer.TexCoord[0, index].Y = 1.0f - (i * vInc);

							boundingBox.Check(v0);
							boundingBox.Check(v1);

							if (i < numTrailSamples - 1)
							{
								indexBuffer.Indices[i * 6 + 0 + numIndices] = ((i + 1) * 2 + 0 + numVertices);
								indexBuffer.Indices[i * 6 + 1 + numIndices] = (i * 2 + 0 + numVertices);
								indexBuffer.Indices[i * 6 + 2 + numIndices] = (i * 2 + 1 + numVertices);

								indexBuffer.Indices[i * 6 + 3 + numIndices] = ((i + 1) * 2 + 0 + numVertices);
								indexBuffer.Indices[i * 6 + 4 + numIndices] = (i * 2 + 1 + numVertices);
								indexBuffer.Indices[i * 6 + 5 + numIndices] = ((i + 1) * 2 + 1 + numVertices);
							}
						}

						numIndices += (int)((numTrailSamples - 1) * 6);
						numVertices += (int)(numTrailSamples * 2);

						break;
					}
				}
			}

            // Copy the values of the bounding box
			vertexBuffer._boundingBox = boundingBox;
			vertexBuffer.BoundingNeedsUpdate = false;

			vertexBuffer._boundingSphere.FromBoundingBox(boundingBox);
			
            vertexBuffer.NumberOfVertices = numVertices;
			vertexBuffer.NotifyChange();
			vertexBuffer.Update();

            indexBuffer.NumberOfIndices = numIndices;
            indexBuffer.Update();

			// Now the instances
			foreach (JParticlesInstances instance in instances)
				instance.MeshObject.Transform.Position = instance.Offset;

			return true;
		}

		#endregion
	}
}
