#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Mathematics.Geometry;
using JadEngine.Input;
using System.Collections.ObjectModel;

#endregion

/* 
 * 17/03/08
 *	-Initial cleanup
 *	-List now returns a ReadOnlyCollection object
 * 24/03/08
 *	-Changed to derive from JEngineElementManager
 * 30/03/08
 *	-Renamed from JCameras to JCameraManager
 * 11/04/08
 *	-CreateDefault() moved to JScene and renamed to CreateDefaultCamera()
 * 27/05/08	
 *	-Conditional auto adding of new cameras to the scene graph
 */

namespace JadEngine.Scene
{
	#region Cameras

	/// <summary>
	/// TODO
	/// </summary>
	public class JCameraManager : JObjectManager<JCamera>
	{
		#region Fields

        /// <summary>The parent scene</summary>
        JScene _scene; 
		#endregion		

        #region Properties

        /// <summary>Return the scene linked to this object</summary>
        public JScene Scene
        {
            get
            {
                return _scene;
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default Constructor
        /// </summary>
        /// <param name="scene">A valid object</param>
        public JCameraManager(JScene scene)
		{
            this._scene = scene;			
		}

		#endregion

		#region Methods			 
		
		/// <summary>
		/// Create a standard camera
		/// </summary>
		/// <param name="name">Camera name</param>
		/// <returns>A valid object</returns>
		public override JCamera Create()
		{			

			JCamera camera = new JCamera(this);

			// Add the camera to the list
			Add(camera);

			return camera;
		}

		/// <summary>
		/// Creates a self driven camera
		/// </summary>
		/// <param name="name">TODO</param>
		/// <param name="loop">TODO</param>
		/// <returns>TODO</returns>
		public JCameraSelfDriven CreateSelfDriven(string fileName, bool loop)
		{
			JCameraSelfDriven camera = new JCameraSelfDriven(fileName, loop, this);

			// Add the camera to the list
			Add(camera);

			return camera;
		}

		/// <summary>
		/// Creates a first persona camera
		/// </summary>
		/// <param name="name">TODO</param>
		/// <returns>TODO</returns>
		public JCameraFirstPerson CreateFirstPerson()
		{
			JCameraFirstPerson camera = new JCameraFirstPerson(this);

			// Add the camera to the list
			Add(camera);

			return camera;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="name">TODO</param>
		/// <param name="target">TODO</param>
		/// <returns>TODO</returns>
		public JCamera CreateSimple3rdPerson()
		{
			JCameraSimple3rdPerson camera = new JCameraSimple3rdPerson(this);

			// Add the camera to the list
			Add(camera);

			return camera;
		}

		/// <summary>
		/// Create a <see cref="JCameraTrackball"/>
		/// </summary>
		/// <param name="name">The name for the camera</param>
		/// <returns>The new <see cref="JCameraTrackball"/></returns>
		public JCamera CreateTrackball()
		{
			JCameraTrackball camera = new JCameraTrackball(this);

			// Add the camera to the list
			Add(camera);

			return camera;
		}

		/// <summary>
		/// Adds a camera to the list and the scene graph
		/// </summary>
		/// <param name="camera">A valid JCamera</param>
		public override void Add(JCamera camera)
		{
			// Add the camera to the list
			_items.Add(camera);

			// Add the camera to the scene graph
			if (_scene != null && _scene.AutoAdd)
				_scene.Root.AddChild(camera);
		}

		///// <summary>
		///// Creates the default self driven camera
		///// </summary>
		//public JCameraFirstPerson CreateDefault()
		//{
		//    // Create the user directed camera
		//    JCameraFirstPerson user_camera = CreateFirstPerson();

		//    user_camera.XFov = Geometry.DegreeToRadian(60.0f);
		//    user_camera.FarPlane = 20.0f;  // 20 m
		//    user_camera.NearPlane = 0.10f;   // 10 cm
		//    user_camera.Sensitivity = 20.0f;
		//    user_camera.Speed = 5.0f;
		//    user_camera.Transform.Position = new Vector3(0.0f, 0.0f, -5.0f);

		//    return user_camera;
		//}

		#endregion		

		/// <summary>
		/// Removes the specified camera.
		/// </summary>
		/// <param name="camera">The camera to remove.</param>
		public override void Delete(JCamera camera)
		{
			_items.Remove(camera);
		}

		/// <summary>
		/// Does a cleanup of the manager's internal data structures.
		/// </summary>
		public override void Cleanup()
		{
			_items.Clear();
		}
	}

	#endregion
}
