using System;

namespace JadEngine.Scene
{
    /// <summary>
    /// An entity is what normally describes an object's look, behaviour, parameters 
    /// and attributes. It can be literally anything, a mesh object
    /// or a sound for example.
    /// A single entity can be assigned to several <see cref="IJSceneNode"/> objects to have copies of the
    /// same object at different places in 3D space for example.
    /// </summary>
    public interface IJEntity
    {
        #region Properties        
        /// <summary>
        /// Gets the unique id that identifies the entity clearly.
        /// </summary>
        int ID
        {
            get;
        }
        /// <summary>
        /// Gets the local bounding information of the entity.
        /// </summary>
        JBounds LocalBounds
        {
            get;
        }
        #endregion
        #region Methods
        /// <summary>
        /// Normally, but not necessarily called once per frame by the <see cref="IJSceneNode"/> container and the 
        /// place where the entity can do calculations and update its data, state and/or the nodes parameters
        /// if applicable.
        /// </summary>
        /// <param name="node">
        /// One of the nodes the entity is attatched to.
        /// </param>
        void Update(IJSceneNode node);
        /// <summary>
        /// Normally, but not necessarily called once per frame by the <see cref="IJSceneNode"/> container after Update()
        /// was called and is the place where the entity can draw itself.
        /// </summary>
        /// <param name="node">
        /// One of the nodes the entity is attatched to. The entity can use the JNode's
        /// data like position, orientation and size to determine how and where it should draw
        /// itself.
        /// </param>
        void Render(IJSceneNode node);
        #endregion


    }
}