// Name:			JAnimationKey.cs
// Author:			Jonathan Garca aka BeRSeRKeR
// Date:			27/11/04 2:37
// Description:	Classes that implement different types of animation keys

#region Using directives

using System;

using Microsoft.DirectX;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// TODO
	/// </summary>
	public abstract class JAnimationKey
	{
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JPositionAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected Vector3 keyValue;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JPositionAnimationKey() : base()
		{
			keyValue = Vector3.Empty;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="position">TODO</param>
		public JPositionAnimationKey(Vector3 position) : base()
		{
			keyValue = position;
		}

		#endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Value
		{
			get { return keyValue; }
			set { keyValue = value; }
		}

		#endregion
		
		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="k1">TODO</param>
		/// <param name="k2">TODO</param>
		/// <param name="w">TODO</param>
		/// <returns>TODO</returns>
		public static JPositionAnimationKey Lerp(JPositionAnimationKey k1, JPositionAnimationKey k2, float w)
		{
			return new JPositionAnimationKey(Vector3.Lerp(k1.Value, k2.Value, w));
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="k1">TODO</param>
		/// <param name="k2">TODO</param>
		/// <param name="k3">TODO</param>
		/// <param name="k4">TODO</param>
		/// <param name="w">TODO</param>
		/// <returns>TODO</returns>
		public static JPositionAnimationKey CatmullRom(JPositionAnimationKey k1, JPositionAnimationKey k2,
													  JPositionAnimationKey k3, JPositionAnimationKey k4, float w)
		{
			return new JPositionAnimationKey(Vector3.CatmullRom(k1.Value, k2.Value, k3.Value, k4.Value, w));
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JRotationAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected Quaternion keyValue;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JRotationAnimationKey()
			: base()
		{
			keyValue = Quaternion.Identity;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="rotation">TODO</param>
		public JRotationAnimationKey(Quaternion rotation)
			: base()
		{
			keyValue = rotation;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Quaternion Value
		{
			get { return keyValue; }
			set { keyValue = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="k1">TODO</param>
		/// <param name="k2">TODO</param>
		/// <param name="w">TODO</param>
		/// <returns>TODO</returns>
		public static JRotationAnimationKey Slerp(JRotationAnimationKey k1, JRotationAnimationKey k2, float w)
		{
			return new JRotationAnimationKey(Quaternion.Slerp(k1.Value, k2.Value, w));
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JScaleAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected Vector3 keyValue;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JScaleAnimationKey() : base()
		{
			keyValue = new Vector3(1.0f, 1.0f, 1.0f);
		}

		/// <summary>
		/// TODO 
		/// </summary>
		/// <param name="scale">TODO</param>
		public JScaleAnimationKey(Vector3 scale) : base()
		{
			keyValue = scale;
		}

		#endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Value
		{
			get { return keyValue; }
			set { keyValue = value; }
		}

		#endregion
		
		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="k1">TODO</param>
		/// <param name="k2">TODO</param>
		/// <param name="w">TODO</param>
		/// <returns>TODO</returns>
		public static JScaleAnimationKey Lerp(JScaleAnimationKey k1, JScaleAnimationKey k2, float w)
		{
			return new JScaleAnimationKey(Vector3.Lerp(k1.Value, k2.Value, w));
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JFOVAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected float keyValue;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JFOVAnimationKey()
			: base()
		{
			keyValue = (float)(Math.PI / 2.0);
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fov">TODO</param>
		public JFOVAnimationKey(float fov)
			: base()
		{
			keyValue = fov;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public float Value
		{
			get { return keyValue; }
			set { keyValue = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="k1">TODO</param>
		/// <param name="k2">TODO</param>
		/// <param name="w">TODO</param>
		/// <returns>TODO</returns>
		public static JFOVAnimationKey Lerp(JFOVAnimationKey k1, JFOVAnimationKey k2, float w)
		{
			return new JFOVAnimationKey(k1.Value + (k2.Value - k1.Value) * w);
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JPRAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected JPositionAnimationKey position;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JRotationAnimationKey rotation;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JPRAnimationKey()
			: base()
		{
			position = new JPositionAnimationKey();
			rotation = new JRotationAnimationKey();
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="position">TODO</param>
		/// <param name="rotation">TODO</param>
		public JPRAnimationKey(Vector3 position, Quaternion rotation)
			: base()
		{
			this.position = new JPositionAnimationKey(position);
			this.rotation = new JRotationAnimationKey(rotation);
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Position
		{
			get { return position.Value; }
			set { position.Value = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Quaternion Rotation
		{
			get { return rotation.Value; }
			set { rotation.Value = value; }
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JPRSAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected JPositionAnimationKey position;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JRotationAnimationKey rotation;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JScaleAnimationKey scale;

		#endregion

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JPRSAnimationKey()
			: base()
		{
			position = new JPositionAnimationKey();
			rotation = new JRotationAnimationKey();
			scale = new JScaleAnimationKey();
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="position">TODO</param>
		/// <param name="rotation">TODO</param>
		/// <param name="scale">TODO</param>
		public JPRSAnimationKey(Vector3 position, Quaternion rotation, Vector3 scale)
			: base()
		{
			this.position = new JPositionAnimationKey(position);
			this.rotation = new JRotationAnimationKey(rotation);
			this.scale = new JScaleAnimationKey(scale);
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Position
		{
			get { return position.Value; }
			set { position.Value = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Quaternion Rotation
		{
			get { return rotation.Value; }
			set { rotation.Value = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Scale
		{
			get { return scale.Value; }
			set { scale.Value = value; }
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JPRFAnimationKey : JAnimationKey
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected JPositionAnimationKey position;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JRotationAnimationKey rotation;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected JFOVAnimationKey fov;

		#endregion Campos

		#region Constructors

		/// <summary>
		/// TODO
		/// </summary>
		public JPRFAnimationKey()
			: base()
		{
			position = new JPositionAnimationKey();
			rotation = new JRotationAnimationKey();
			fov = new JFOVAnimationKey();
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="position">TODO</param>
		/// <param name="rotation">TODO</param>
		/// <param name="fov">TODO</param>
		public JPRFAnimationKey(Vector3 position, Quaternion rotation, float fov)
			: base()
		{
			this.position = new JPositionAnimationKey(position);
			this.rotation = new JRotationAnimationKey(rotation);
			this.fov = new JFOVAnimationKey(fov);
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public Vector3 Position
		{
			get { return position.Value; }
			set { position.Value = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public Quaternion Rotation
		{
			get { return rotation.Value; }
			set { rotation.Value = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public float FOV
		{
			get { return fov.Value; }
			set { fov.Value = value; }
		}

		#endregion
	}
}