// Name:			PathAnimator.cs
// Author:			Jonathan Garca aka BeRSeRKeR
// Date:			27/11/04 2:20
// Description:	Classes that implement different animation paths

#region Using directives

using System;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Scene.MD5;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JPathAnimatorException : Exception
	{
		#region Constructors
		
		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="message">Error message</param>
		public JPathAnimatorException(string message)
			: base(message)
		{ }

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public abstract class JPathAnimator
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected float startTime;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected bool loop;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="loop">TODO</param>
		public JPathAnimator(bool loop)
		{
			this.loop = loop;
			Reset();
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public bool Loop
		{
			get { return loop; }
			set { loop = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		public virtual void Reset()
		{
			startTime = Jad.Timer.Time;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="time">TODO</param>
		/// <returns>TODO</returns>
		public abstract JAnimationKey AtTime(float time);

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JStraightPathAnimator : JPathAnimator
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected Vector3 start;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected Vector3 end;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected Vector3 direction;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected float length;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected float timeToEnd;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected float timeScale;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected bool follow;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected bool invert;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="start">TODO</param>
		/// <param name="end">TODO</param>
		/// <param name="timeToEnd">TODO</param>
		/// <param name="loop">TODO</param>
		/// <param name="follow">TODO</param>
		/// <param name="invert">TODO</param>
		public JStraightPathAnimator(Vector3 start, Vector3 end, float timeToEnd, bool loop, bool follow, bool invert)
			: base(loop)
		{
			this.start = start;
			this.end = end;
			this.timeToEnd = timeToEnd;
			this.follow = follow;
			this.invert = invert;

			direction = end - start;
			length = direction.Length();
			direction.Normalize();

			timeScale = length / timeToEnd;
		}

		#endregion
		
		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public bool Follow
		{
			get { return follow; }
			set { follow = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public bool Invert
		{
			get { return invert; }
			set { invert = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="time">TODO</param>
		/// <returns>TODO</returns>
		public override JAnimationKey AtTime(float time)
		{
			JAnimationKey finalKey;

			float t = time - startTime;

			// Calculate the position
			Vector3 position = start;

			if (!loop && t >= timeToEnd)
				position = end;
			
			else
				position += direction * (t % timeToEnd) * timeScale;

			if (follow)
			{
				Vector3 dir = direction;

				if (invert)
					dir = -dir;

				// Calculate the rotation
				Matrix lookAt = Matrix.LookAtLH(position, position + dir*length, new Vector3(0.0f, 1.0f, 0.0f));
				lookAt.Invert();
				Quaternion rotation = Quaternion.RotationMatrix(lookAt);

				finalKey = new JPRAnimationKey(position, rotation);
			}

			else
				finalKey = new JPositionAnimationKey(position);

			return finalKey;
		}

		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JCirclePathAnimator : JPathAnimator
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		protected Vector3 center;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected float radius;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected float timeToEnd;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected float timeScale;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected bool follow;
		
		/// <summary>
		/// TODO
		/// </summary>
		protected bool invert;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="center">TODO</param>
		/// <param name="radius">TODO</param>
		/// <param name="timeToEnd">TODO</param>
		/// <param name="loop">TODO</param>
		/// <param name="follow">TODO</param>
		/// <param name="invert">TODO</param>
		public JCirclePathAnimator(Vector3 center, float radius, float timeToEnd, bool loop, bool follow, bool invert)
			: base(loop)
		{
			this.center = center;
			this.radius = radius;
			this.timeToEnd = timeToEnd;
			this.follow = follow;
			this.invert = invert;

			timeScale = (float)(Math.PI * 2.0) / timeToEnd;
		}

		#endregion
		
		#region Properties
		
		/// <summary>
		/// TODO
		/// </summary>
		public bool Follow
		{
			get { return follow; }
			set { follow = value; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public bool Invert
		{
			get { return invert; }
			set { invert = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="time">TODO</param>
		/// <returns>TODO</returns>
		public override JAnimationKey AtTime(float time)
		{
			JAnimationKey finalKey;

			double t	= (time - startTime) / timeScale;
			double pi2	= Math.PI * 2.0;

			if (invert)
			{
				t	= -t;
				pi2	= -pi2;
			}

			float sin, cos;

			if (!loop && t >= pi2)
			{
				sin = (float)Math.Sin(pi2);
				cos = (float)Math.Cos(pi2);
			}
			
			else
			{
				sin = (float)Math.Sin(t);
				cos = (float)Math.Cos(t);
			}

			// Calculate the position
			Vector3 position = new Vector3(radius * sin, 0.0f, radius * cos);

			// Calculate the rotation
			if (follow)
			{
				Vector3 right = new Vector3(sin, 0.0f, cos);
				Vector3 up = new Vector3(0.0f, 1.0f, 0.0f);
				Vector3 look = Vector3.Cross(right, up);

				Matrix R = Matrix.Identity;

				R.M11 = right.X;
				R.M12 = right.Y;
				R.M13 = right.Z;

				R.M21 = up.X;
				R.M22 = up.Y;
				R.M23 = up.Z;

				R.M31 = look.X;
				R.M32 = look.Y;
				R.M33 = look.Z;

				Quaternion rotation = Quaternion.RotationMatrix(R);

				finalKey = new JPRAnimationKey(position + center, rotation);
			}

			else
				finalKey = new JPositionAnimationKey(position + center);

			return finalKey;
		}
		
		#endregion
	}

	/// <summary>
	/// TODO
	/// </summary>
	public class JCameraPathAnimator : JPathAnimator
	{
		#region Fields

		/// <summary>
		/// The file where the path was loaded from
		/// </summary>
		private string fileName;
		
		/// <summary>
		/// Number of animation keys of the path
		/// </summary>
		private int nKeys;
		
		/// <summary>
		/// Number of cut-scenes
		/// </summary>
		private int nCuts;
		
		/// <summary>
		/// Frame rate of the animation
		/// </summary>
		private int frameRate;
		
		/// <summary>
		/// Array of cut-scenes
		/// </summary>
		private int[] cuts;
		
		/// <summary>
		/// Array of animation keys
		/// </summary>
		private JPRFAnimationKey[] keys;
		
		/// <summary>
		/// Actual cut-scene
		/// </summary>
		private int currentCut;
		
		/// <summary>
		/// Indicates if the transition between one cut-scene and another has started
		/// </summary>
		private bool cutStart = false;
		
		/// <summary>
		/// TODO
		/// </summary>
		private int currentKey;
		
		/// <summary>
		/// Indicates if the animation is paused or not
		/// </summary>
		private bool pause;

		/// <summary>
		/// TODO
		/// </summary>
		private float curTime = 0;
		
		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="fileName">File with the animation path</param>
		/// <param name="loop">TODO</param>
		/// <exception cref="JPathAnimatorException">TODO</exception>
		public JCameraPathAnimator(string fileName, bool loop)
			: base(loop)
		{
			JMD5Camera camera = new JMD5Camera();

			if (!camera.Load(fileName))
				throw new Exception(string.Format("Unable to load file: \"{0}\"", fileName));

			nKeys = camera.NumberOfFrames;
			nCuts = camera.NumberOfCuts;
			frameRate = camera.FrameRate;

			cuts = new int[nCuts];
			for (int nCut = 0; nCut < nCuts; nCut++)
				cuts[nCut] = camera.Cuts[nCut];

			int nKey = 0;
			keys = new JPRFAnimationKey[nKeys];
			foreach (JMD5Camera.JKey key in camera.Keys)
			{
				keys[nKey] = new JPRFAnimationKey(new Vector3(key.Position[0], key.Position[1], key.Position[2]),
												 new Quaternion(key.Rotation[0], key.Rotation[1], key.Rotation[2], key.Rotation[3]),
												 Geometry.DegreeToRadian(key.Fov));
				nKey++;
			}

			currentCut = 0;
			cutStart = false;

			CurrentKey = 0;

			startTime = 0.0f;

			this.fileName = fileName;
		}
		
		#endregion
		
		#region Properties

		/// <summary>
		/// Gets the file where the path was loaded from
		/// </summary>
		public string FileName
		{
			get { return fileName; }
		}

		/// <summary>
		/// Gets or sets the frame rate of the animation
		/// </summary>
		public int FrameRate
		{
			get { return frameRate; }
			set { frameRate = value; }
		}

		/// <summary>
		/// Gets the number of animation keys of the path
		/// </summary>
		public int NumberOfKeys
		{
			get { return nKeys; }
		}

		/// <summary>
		/// Gets the number of cut-scenes
		/// </summary>
		public int NumberOfCuts
		{
			get { return nCuts; }
		}

		/// <summary>
		/// Get the array of animation keys
		/// </summary>
		public JPRFAnimationKey[] Keys
		{
			get  { return keys; }
		}

		/// <summary>
		/// Gets the array of cut-scenes
		/// </summary>
		public int[] Cuts
		{
			get { return cuts; }
		}

		/// <summary>
		/// Gets or sets the actual cut-scene
		/// </summary>
		public int CurrentKey
		{
			get { return currentKey; }
			set { currentKey = value; }
		}

		/// <summary>
		/// Gets or sets if the animation is paused or not
		/// </summary>
		public bool Pause
		{
			get { return pause; }
			set { pause = value; }
		}

		/// <summary>
		/// Gets if the animation is paused
		/// </summary>
		public bool Paused
		{
			get { return pause; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
  		public override void Reset()
		{
			currentKey = 0;

			curTime = 0;

			base.Reset();
		}

		/// <summary>
		/// Calculates the animation key at a given time
		/// </summary>
		/// <param name="time">The time frame where we want to know the animation key</param>
		/// <returns>The animation key for the specified time</returns>
		/// <remarks>TODO: THIS MUST WORK WITH "deltaTime"</remarks>
		public override JAnimationKey AtTime(float time)
		{
			JPRFAnimationKey finalKey = new JPRFAnimationKey();

			float t = time - startTime;

			//if (!pause)
			//	curTime += Jad.Timer.DeltaTime;

			//float t = curTime;

			if (loop || currentKey < NumberOfKeys - 1)
			{

				// Get the current animation key
				float fKey = (t * FrameRate) % NumberOfKeys;

				// Redondeamos el ndice a la clave de animacin actual
				currentKey = (int)fKey;

				// Get the normalized time [0, 1] that will be the interpolation var
				float weight = fKey % 1;

				JPRFAnimationKey key1, key2;

				// Do we have to interpolate?
				int cut = NumberOfCuts > 0 ? cuts[currentCut] - 1 : -1;
				if (currentKey < (NumberOfKeys - 1) && currentKey != cut)
				{
					key1 = keys[currentKey];
					key2 = keys[currentKey + 1];

					// Interpolate //

					// For the position Catmull-Rom will be used, but for now, we use Linear Interpolation
					finalKey.Position = Vector3.Lerp(key1.Position, key2.Position, weight);
					
					// For the rotation use Spherical Linear Interpolation
					finalKey.Rotation = Quaternion.Slerp(key1.Rotation, key2.Rotation, weight);
					
					// For the FOV use Linear Interpolation
					finalKey.FOV = key1.FOV + (key2.FOV - key1.FOV) * weight;

					// If we get here we arent changing from one cut-scene to another
					if (cutStart)
					{
						// Move to the next cut-scene
						currentCut = ++currentCut < nCuts ? currentCut : 0;
						// Indicate that we arent between cut-scene and cut-scene
						cutStart = false;
					}
				}
				else
				{
					// Do not interpolate
					finalKey = keys[currentKey];
					// A new cut-scene is going to start
					cutStart = true;
				}
			}
			
			else
			{
				finalKey = keys[NumberOfKeys - 1];

				if (loop)
					currentKey = 0;
				else
					currentKey = NumberOfKeys - 1;
			}

			return finalKey;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <returns>TODO</returns>
		public override string ToString()
		{
			return System.IO.Path.GetFileNameWithoutExtension(fileName);
		}

		#endregion
	}
}