#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;
using JadEngine.Tools;
using JadEngine.Collections;

#endregion

/*
 * 24/03/08
 *	-Changed to derive from JEngineElementManager
 * 30/03/08
 *	-Renamed from JScenes to JSceneManager
 */

namespace JadEngine.Scene
{

    #region Scenes

    /// <summary>
    /// This class mantains all the scenes loaded on the engine
    /// </summary>
    public class JSceneManager : JObjectManager<JScene>, IDisposable
    {		

        #region Properties

        #endregion		

        #region IDisposable

        /// <summary>
        /// Disposes the scene and all the scenes
        /// </summary>
        public void Dispose()
        {
            foreach (JScene scene in _items)
			{
                scene.Dispose();
			}

            _items.Clear();
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default Constructor
        /// </summary>
		public JSceneManager()
		{			
        }

		#endregion
    	
		#region Methods

        /// <summary>
        /// Create a new scene and add it to the list
        /// If the scene already exists, returns the scene and
        /// do not creates a new one
        /// </summary>
        /// <param name="name">Name of the scene</param>
        /// <returns>The scene</returns>
		public override JScene Create()
		{
			JScene scene = null; //Items.Exists(name);

            if (scene != null) return scene;

            scene = new JScene();

            Add(scene);

            return scene;
		}

		/// <summary>
		/// Add a scene to the list
		/// </summary>
		/// <param name="scene">A valid object</param>
		public override void Add(JScene scene)
		{
            _items.Add(scene);
		}

		/// <summary>
		/// Delete the scenes, calling to Destroy for every one
		/// </summary>
		public override void Cleanup()
		{
            foreach (JScene scene in _items)
			{
                scene.Cleanup();
			}

            _items.Clear();

        }

        #endregion		

		public override void Delete(JScene element)
		{
			throw new Exception("The method or operation is not implemented.");
		}
	}

	#endregion

}