#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Video;
using JadEngine.Core;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;
using JadEngine.Tools;
using JadEngine.Collections;
using System.Collections.ObjectModel;

#endregion

/*
 * TODO:
 * I think the Projector & shafts textures should be samplers.
 *  
 * 17/03/08
 *	-Initial cleanup
 *	-List now returns a ReadOnlyCollection object
 * 24/03/08
 *	-Changed to derive from JEngineElementManager
 * 30/03/08
 *	-Renamed from JLights to JLightManager
 * 18/05/08
 *	-Changed Intersect() method to work with JBaseLight.Intersect()
 * 27/05/08
 *	-Conditional auto adding of new lights to the scene graph
 */

namespace JadEngine.Scene
{
	/// <summary>
    /// Class that sorts lights by ID
    /// </summary>
    public class JSortLightsByID : IComparer<JBaseLight>
    {
		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="a">TODO</param>
		/// <param name="b">TODO</param>
		/// <returns>TODO</returns>
        public int Compare(JBaseLight a, JBaseLight b)
        {
            if (a.ID.CompareTo(b.ID) < 0)
                return -1;
			
            if (a.ID == b.ID)
                return 0;

            return 1;
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="x">TODO</param>
		/// <param name="y">TODO</param>
		/// <returns>TODO</returns>
        public bool Equals(JBaseLight x, JBaseLight y)
        {
            throw new NotImplementedException();
        }

    	/// <summary>
    	/// TODO
    	/// </summary>
		/// <param name="obj">TODO</param>
		/// <returns>TODO</returns>
        public int GetHashCode(JBaseLight obj)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// This class mantains all the lights on the scene
    /// </summary>
    public class JLightManager : JObjectManager<JBaseLight>
    {
        #region Structs

    	/// <summary>
    	/// TODO
    	/// </summary>
        public struct JShafts
		{
			#region Fields
    		
			/// <summary>
            /// Process a blur after the shafts
            /// </summary>
            public bool ApplyBlur;

            /// <summary>
            /// Marks the stencil so only 50% pixels are drawn
            /// </summary>
            public bool UseStencil;

            /// <summary>
            /// Postproduction filter for soft shadows
            /// </summary>
            public JPostProduction PostProduction;

    		/// <summary>
    		/// TODO
    		/// </summary>
            public JPostFilterGaussianBlur BlurH;

    		/// <summary>
    		/// TODO
    		/// </summary>
    		public JPostFilterGaussianBlur BlurV;

			#endregion

			#region Methods
    		
			/// <summary>
    		/// Sets the default options
    		/// </summary>
            public void Initialize()
            {
                // If its inizialized, exit
                if (PostProduction != null) return;

                // Create the filters to make the blur in the soft shadows
                PostProduction = new JPostProduction();

                // Create the gaussian blur filter
                JGaussianProperties[] props = new JGaussianProperties[1];
				
                // This values are very dependant on the number of samples
                // If the amp is very low, its possible that the total weight is < 1
                // and thenn all the scene will be darker

                int diameter;
				
                // Values for 4
                diameter = 4;
                props[0].RadiusScale = 0.11f;
                props[0].Amp = 0.27f;

                // Values for 16
                //diameter = 16;
                //props[0].radius_scale = 0.11f;
                //props[0].amp = 0.1f;

                JPostProductionEffect post = new JPostProductionEffect("LightShafts");

                // -----------------------------------------------------------------
                // We want to "protect" the active render target, because it will be
                // used at the end of the effect
                // -----------------------------------------------------------------

                post.Add(new JPostFilterProtectPrimary(null, false));

                JGaussianBlur gaussianBlur = new JGaussianBlur(diameter, props, false);
                gaussianBlur.GenerateKernel();

                JPostProductionRT postRT = new JPostProductionRT(0, 1f);

                postRT.FixedRenderTarget = Jad.Video.Views.ActiveView.RenderTargets.ShadowMask2;

                // Set an horizontal blur filter
                // Set a different RT that will act as texture

                BlurH = new JPostFilterGaussianBlur(postRT, gaussianBlur, JBlurType.Horizontal);

                post.Add(BlurH);

				// Set a vertical blur filter
                postRT = new JPostProductionRT(1, 1f);

                // The result will be saved in this RT
                postRT.FixedRenderTarget = Jad.Video.Views.ActiveView.RenderTargets.ShadowMask;

                BlurV = new JPostFilterGaussianBlur(postRT, gaussianBlur, JBlurType.Vertical);

                post.Add(BlurV);

                // -----------------------------------------------------------------
                // Now, we want to blend the original RT, (locked by the JPostFilterProtectPrimary)
                // with the blurred
                // -----------------------------------------------------------------

                postRT = new JPostProductionRT(2, 1f);
                post.Add(new JPostFilterBlend(postRT));

                PostProduction.Add(post);

                PostProduction.MakeARenderTargetEnd = false;

                // Set the value to 1 so there is no attenuation
                BlurH.GlowAttenuation = BlurV.GlowAttenuation = 1f;
			}

			#endregion
		}

		/// <summary>
		/// TODO
		/// </summary>
    	struct CoronaVertex
		{
			#region Fields
			
			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 vtxCoord;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector2 texCoord0;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector2 axes;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector4 color;

			#endregion
		}
    	
        #endregion

		#region Constants

    	/// <summary>
    	/// TODO
    	/// </summary>
		const float coronaProbeSize = 0.3f;

		/// <summary>
		/// Glow max size
		/// </summary>
		const float maxSize = 1.0f;
    	
    	/// <summary>
    	/// Glow min size
    	/// </summary>
		const float minSize = 0.25f;

		/// <summary>
		/// TODO
		/// </summary>
		const float minDistSq = 0.7f * 0.7f;
		
    	/// <summary>
    	/// TODO
    	/// </summary>
    	const float maxDistSq = 4.0f * 4.0f;
    	
		#endregion
    	
		#region Fields

        /// <summary>The parent scene</summary>
        JScene scene;
		
		/// <summary>
		/// Encapsulates the properties of the light shafts
		/// </summary>
		public JShafts Shafts;

		/// <summary>
		/// Intensity that will be added to all the lights
		/// </summary>
		public float Intensity;

		/// <summary>
		/// Ambient light used for the ambient pass
		/// </summary>
		public JColor Ambient;

		/// <summary>
		/// Enable / Disable the lights
		/// </summary>
		public bool Enabled = true;

		/// <summary>
		/// Default values for the shafts constructor
		/// </summary>
		internal Vector3[] FrustumVerts;

    	/// <summary>
    	/// TODO
    	/// </summary>
		short[] coronaIndices;

		/// <summary>
		/// TODO
		/// </summary>
		CoronaVertex[] coronaVertices;

		/// <summary>
		/// TODO
		/// </summary>
		VertexDeclaration coronaVertexDecl;

		/// <summary>
		/// TODO
		/// </summary>
		JEffect coronaEffect;

		/// <summary>
		/// TODO
		/// </summary>
		CoronaVertex[] coronaProbeVertices;

		private static JColor _black = JColors.Black;
    	
		#endregion

        #region Properties

        /// <summary>Return the scene linked to this object</summary>
        public JScene Scene
        {
            get
            {
                return scene;
            }
        }

        #endregion        

		#region Constructors

        /// <summary>
        /// Default Constructor
        /// </summary>
        /// <param name="scene">A valid object</param>
		public JLightManager(JScene scene)
		{
            this.scene = scene;			

			// Set the ambient light
			Ambient = new JColor(0.22f, 0.22f, 0.22f, 1f);

			// Set this values to use them in the shafts
			FrustumVerts = new Vector3[8];

			FrustumVerts[0] = new Vector3(1.0f, 1.0f, 1.0f);
			FrustumVerts[1] = new Vector3(1.0f, 1.0f, -1.0f);
			FrustumVerts[2] = new Vector3(-1.0f, 1.0f, -1.0f);
			FrustumVerts[3] = new Vector3(-1.0f, 1.0f, 1.0f);

			FrustumVerts[4] = new Vector3(-1.0f, -1.0f, -1.0f);
			FrustumVerts[5] = new Vector3(-1.0f, -1.0f, 1.0f);
			FrustumVerts[6] = new Vector3(1.0f, -1.0f, 1.0f);
			FrustumVerts[7] = new Vector3(1.0f, -1.0f, -1.0f);

			Shafts.ApplyBlur = true;
        
            if(!Jad.IDontNeedToRender) InitCoronas();

        }

		#endregion
    	
		#region Methods

		public override JBaseLight Create()
		{
			return Create(JLightType.Omni);
		}

        /// <summary>
        /// Create a new light and add it to the scene
        /// </summary>
        /// <param name="name">Name of the light</param>
        /// <param name="type">Type of light</param>
        /// <returns>The light created</returns>
		public JBaseLight Create(JLightType type)
		{
			JBaseLight light = null;

			if (light != null) return light;

			switch (type)
			{
				case JLightType.Omni:
				{
					// Create a omni light
					light = new JOmniLight(this);
					break;
				}

				case JLightType.Spot:
				{
					// Create a spot light
					light = new JSpotLight(this);
					break;
				}

				case JLightType.Directional:
				{
					// Create a directional light
					light = new JDirectLight(this);
					break;
				}

				default:
					light = null;
					break;
			}

			Add(light);

			return light;
		}

		/// <summary>
		/// Add a light to the list and to the scene graph
		/// </summary>
		/// <param name="light"></param>
		public override void Add(JBaseLight light)
		{
			_items.Add(light);

			// Add the light to the scene graph
			if (scene != null && scene.AutoAdd)
				scene.Root.AddChild(light);
		}

		/// <summary>
		/// Does a cleanup of the manager's internal data structures.
		/// </summary>
		public override void Cleanup()
		{
			_items.Clear();
		}

        /// <summary>
        /// Render the shafts effect if enabled
        /// </summary>
		internal void RenderShafts()
		{
			bool processAny = false;

			foreach (JBaseLight light in _items)
			{
				if (light.Visible && !light.Hide && light.Type == JLightType.Spot)
				{
					JSpotLight spot = (JSpotLight)light;

					if (spot.Shafts.Enabled)
					{
						processAny = true;
						break;
					}
				}
			}

			if (processAny)
			{
				Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);
				Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, true);
				Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.SourceAlpha);
				Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.One);

				JRenderTarget lastRT = Jad.Video.RenderTargets.Active;
				JRenderTarget activeRT = null;

				Shafts.Initialize();

				if (Shafts.ApplyBlur)
				{
					// Search a free RT

					if (lastRT == Jad.Video.Views.ActiveView.RenderTargets.Primary)
						activeRT = Jad.Video.Views.ActiveView.RenderTargets.Secondary;

					else
						activeRT = Jad.Video.Views.ActiveView.RenderTargets.Primary;

					activeRT.Begin();

					Jad.Video.Views.ActiveView.Clear(ClearFlags.Target, ref _black, 1, 0);
				}

				foreach (JBaseLight light in _items)
				{
					if (light.Visible && !light.Hide && light.Type == JLightType.Spot)
					{
						JSpotLight spot = (JSpotLight)light;
						spot.RenderShafts();
					}
				}
                
                Jad.Video.StateManager.SetRenderState(RenderStates.StencilEnable, false);
				Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.One);
				Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.Zero);
				Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
				Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, true);

				if (Shafts.ApplyBlur)
				{
					Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);

					activeRT.End();

					// Activate again the RT with the scene render

					// Process the RT

					Shafts.PostProduction.Process(activeRT, null);

					Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
				}
			}
		}

		/// <summary>
		/// Searches the closest light that intersects a line.
		/// NOTE: This can get slow because it tests ALL lights for intersection. If
		/// you want to test only visible lights in a scene, you should use Scene.Intersect() instead.
		/// </summary>
		/// <param name="start">The start ray point</param>
		/// <param name="end">The end ray point</param>		
		/// <param name="infinite">If set to true, the starting distance will be set to <see cref="float.MaxValue"/>,
		/// otherwise the distance between the start and end point will be used.</param>
		/// <returns>The nearest light or null</returns>
		public JBaseLight Intersect(Vector3 start, Vector3 end, bool infinite)
		{
			JBaseLight intersection = null;
			float smallestDistance;

			if (infinite)
			{
				smallestDistance = float.MaxValue;
			}
			else
			{
				smallestDistance = (end - start).Length();
			}

			for (int j = 0; j < Count; j++)
			{
				JBaseLight light = this[j];

				float distance;

				if (light.Intersect(start, end, out distance))
				{
					// Get the closest one
					if (distance < smallestDistance)
					{
						intersection = light;
						smallestDistance = distance;
					}
				}
			}

			return intersection;
			//JBaseLight collision = null;
			//float distance = float.MaxValue;

			//// Create an object to test collisions
			//JMesh mesh = Jad.Video.Meshes.Primitives.CreateSphere(0.25f, 4, 4);

			//for (int j = 0; j < Count; j++)
			//{
			//    JBaseLight m = this[j];

			//    // Transform the vectors to object space
			//    Vector3 startOS = start, endOS = end;

			//    Matrix toObjectSpace = Matrix.Invert(m.Transform.WorldTM);

			//    startOS.TransformCoordinate(toObjectSpace);
			//    endOS.TransformCoordinate(toObjectSpace);

			//    float dist;

			//    Vector3 rayDir = Vector3.Normalize(endOS - startOS);

			//    if (mesh.Intersect(startOS, rayDir, false, out dist))
			//    {
			//        // Get the closest one
			//        if (dist < distance)
			//        {
			//            collision = m;
			//            distance = dist;
			//        }
			//    }
			//}

			//Jad.Video.Meshes.Delete(mesh);

			//return collision;
		}
		
		/// <summary>
		/// Called when the render begins. Used for assign matrix accorcing
		/// the render target selected
		/// </summary>
		internal void BeforeRender()
		{
			foreach (JBaseLight light in _items)
			{
				light.BeforeRender();
			}
		}

        #endregion

		#region Coronas

    	/// <summary>
    	/// Initialize the halo corona common variables
    	/// </summary>
		public void InitCoronas()
		{
            if (coronaIndices != null) return;

			VertexElement[] vElements = new VertexElement[5];
			vElements[0] = new VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0);
			vElements[1] = new VertexElement(0, 12, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0);
			vElements[2] = new VertexElement(0, 20, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 1);
			vElements[3] = new VertexElement(0, 28, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.Color, 0);
			vElements[4] = VertexElement.VertexDeclarationEnd;

			coronaVertexDecl = new VertexDeclaration(Jad.Video.Device, vElements);

			coronaProbeVertices = new CoronaVertex[4];
			coronaProbeVertices[0] = new CoronaVertex();
			coronaProbeVertices[0].vtxCoord = Vector3.Empty;
			coronaProbeVertices[0].texCoord0 = new Vector2(0.0f, 0.0f);
			coronaProbeVertices[0].axes = new Vector2(-1.0f, 1.0f) * coronaProbeSize;
			coronaProbeVertices[0].color = Vector4.Empty;

			coronaProbeVertices[1] = new CoronaVertex();
			coronaProbeVertices[1].vtxCoord = Vector3.Empty;
			coronaProbeVertices[1].texCoord0 = new Vector2(1.0f, 0.0f);
			coronaProbeVertices[1].axes = new Vector2(1.0f, 1.0f) * coronaProbeSize;
			coronaProbeVertices[1].color = Vector4.Empty;

			coronaProbeVertices[2] = new CoronaVertex();
			coronaProbeVertices[2].vtxCoord = Vector3.Empty;
			coronaProbeVertices[2].texCoord0 = new Vector2(0.0f, 1.0f);
			coronaProbeVertices[2].axes = new Vector2(-1.0f, -1.0f) * coronaProbeSize;
			coronaProbeVertices[2].color = Vector4.Empty;

			coronaProbeVertices[3] = new CoronaVertex();
			coronaProbeVertices[3].vtxCoord = Vector3.Empty;
			coronaProbeVertices[3].texCoord0 = new Vector2(1.0f, 1.0f);
			coronaProbeVertices[3].axes = new Vector2(1.0f, -1.0f) * coronaProbeSize;
			coronaProbeVertices[3].color = Vector4.Empty;

			coronaIndices = new short[6];
			coronaVertices = new CoronaVertex[4];

			coronaVertices[0] = new CoronaVertex();
			coronaVertices[0].vtxCoord = Vector3.Empty;
			coronaVertices[0].texCoord0 = new Vector2(0.0f, 0.0f);
			coronaVertices[0].axes = new Vector2(-1.0f, 1.0f);
			coronaVertices[0].color = Vector4.Empty;

			coronaVertices[1] = new CoronaVertex();
			coronaVertices[1].vtxCoord = Vector3.Empty;
			coronaVertices[1].texCoord0 = new Vector2(1.0f, 0.0f);
			coronaVertices[1].axes = new Vector2(1.0f, 1.0f);
			coronaVertices[1].color = Vector4.Empty;

			coronaVertices[2] = new CoronaVertex();
			coronaVertices[2].vtxCoord = Vector3.Empty;
			coronaVertices[2].texCoord0 = new Vector2(1.0f, 1.0f);
			coronaVertices[2].axes = new Vector2(1.0f, -1.0f);
			coronaVertices[2].color = Vector4.Empty;

			coronaVertices[3] = new CoronaVertex();
			coronaVertices[3].vtxCoord = Vector3.Empty;
			coronaVertices[3].texCoord0 = new Vector2(0.0f, 1.0f);
			coronaVertices[3].axes = new Vector2(-1.0f, -1.0f);
			coronaVertices[3].color = Vector4.Empty;

			coronaIndices[0] = 0;
			coronaIndices[1] = 1;
			coronaIndices[2] = 3;
			coronaIndices[3] = 1;
			coronaIndices[4] = 2;
			coronaIndices[5] = 3;

			coronaEffect = Jad.Video.Effects.CreateInHouse("corona.fx");
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		public void DestroyCoronas()
		{
			coronaVertexDecl.Dispose();

    		for (int i = 0; i < Count; i++)
				_items[i].Corona.Destroy();
		}

    	/// <summary>
    	/// TODO
    	/// </summary>
		public void RenderCoronas()
		{
			//Jad.Video.Device.SetRenderState(RenderStates.SourceBlend, (int)Blend.SourceAlpha);
			Jad.Video.StateManager.SetRenderState(RenderStates.SourceBlend, (int)Blend.One);
			Jad.Video.StateManager.SetRenderState(RenderStates.DestinationBlend, (int)Blend.One);
			Jad.Video.StateManager.SetRenderState(RenderStates.CullMode, (int)Cull.None);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);

			// Configurate the effect
			coronaEffect.DXResource.Technique = coronaEffect.Handlers.RenderScene;
			coronaEffect.Begin();

			Matrix viewProj = scene.ActiveCamera.ViewProjection;
			Vector4 right = new Vector4(scene.ActiveCamera.Right.X, scene.ActiveCamera.Right.Y, scene.ActiveCamera.Right.Z, 0.0f);
			Vector4 up = new Vector4(scene.ActiveCamera.Up.X, scene.ActiveCamera.Up.Y, scene.ActiveCamera.Up.Z, 0.0f);

			coronaEffect.DXResource.SetValue("cam_right", right);
			coronaEffect.DXResource.SetValue("cam_up", up);
			coronaEffect.DXResource.SetValue(coronaEffect.Handlers.WorldViewProjection, viewProj);

			Jad.Video.Device.VertexDeclaration = coronaVertexDecl;

			coronaEffect.DXResource.BeginPass(0);

			for (int i = 0; i < Count; i++)
			{
				JBaseLight light = _items[i];

				if (!light.Corona.Enabled) continue;
                
				// If the coronas are not initiated, init it
				if (light.Corona.Texture == null) light.Corona.Init();

				Vector3 pos = light.Transform.Position;

				Vector4 lightPos = new Vector4(pos.X, pos.Y, pos.Z, 0.0f);
				coronaEffect.DXResource.SetValue("lightPos", lightPos);

				coronaEffect.DXResource.CommitChanges();

				Vector3 cPos = scene.ActiveCamera.Transform.Position;

				float distSq = (pos - cPos).LengthSq();

				float weight = (minDistSq - distSq) / (minDistSq - maxDistSq);
				float coronaSize = (1.0f - weight) * minSize + weight * maxSize;
				coronaSize = Math.Min(maxSize, Math.Max(coronaSize, minSize));

                light.Corona.CurPixels = light.Corona.MaxPixels = 0;

				// Get the number of pixels from the last frame
                while ((light.Corona.MaxPixels = (int)light.Corona.MaxPixelsQuery.GetData(typeof(int), true)) == 0)
				{
				}
				
                while ((light.Corona.CurPixels = (int)light.Corona.CurPixelsQuery.GetData(typeof(int), true)) == 0)
				{
				}

				// Real Pixels

                Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
                Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
                Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.LessEqual);
                Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, 0);

                light.Corona.CurPixelsQuery.Issue(IssueFlags.Begin);
				Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, coronaProbeVertices);
                light.Corona.CurPixelsQuery.Issue(IssueFlags.End);

				// Pixels maximum

                Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.Always);

                light.Corona.MaxPixelsQuery.Issue(IssueFlags.Begin);
				Jad.Video.Device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, coronaProbeVertices);
                light.Corona.MaxPixelsQuery.Issue(IssueFlags.End);

				// Render

                float alpha = (light.Corona.MaxPixels > 0 ? ((float)light.Corona.CurPixels / (float)light.Corona.MaxPixels) : 0.0f) * light.Multiplier * 2.0f;

				if (alpha > 0.0f)
				{
					Vector4 color = light.Color.ToVector4();

					color.W = alpha < 0.0f ? 0.0f : (alpha > 1.0f ? 1.0f : alpha);

					coronaEffect.DXResource.SetValue("color", color);
					coronaEffect.DXResource.SetValue("size", coronaSize);
                    coronaEffect.DXResource.SetValue(coronaEffect.Handlers.TextureDiffuse, light.Corona.Texture.DXResource);

					coronaEffect.DXResource.CommitChanges();

					Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.Always);
					Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, true);
					Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, (int)(ColorWriteEnable.RedGreenBlueAlpha));

					Jad.Video.Device.DrawIndexedUserPrimitives(PrimitiveType.TriangleList, 0, 4, 2, coronaIndices, true, coronaVertices);
				}
			}

			coronaEffect.DXResource.EndPass();
			coronaEffect.End();

			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferFunction, (int)Compare.LessEqual);
			Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);
			Jad.Video.StateManager.SetRenderState(RenderStates.CullMode, (int)Cull.CounterClockwise);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, true);
			Jad.Video.StateManager.SetRenderState(RenderStates.ColorWriteEnable, (int)(ColorWriteEnable.RedGreenBlueAlpha));
		}

        /// <summary>
        /// Called OnReset device
        /// </summary>
        public void Reset()
        {
			Jad.LogWriter.WriteLine("Lights.Reset");

            for (int c = 0; c < Count; c++)
            {
                JBaseLight light = this[c];

            	if (light.Corona.MaxPixelsQuery != null)
            		light.Corona.Reset();
            }
        }

        /// <summary>
        /// Called OnLostDevice
        /// </summary>
        internal void OnLost()
        {
			Jad.LogWriter.WriteLine("Lights.OnLost");

            for (int c = 0; c < Count; c++)
            {
                JBaseLight light = this[c];

            	if (light.Corona.MaxPixelsQuery != null)
            		light.Corona.OnLost();
            }
        }

        #endregion


		public override void Delete(JBaseLight element)
		{
			throw new Exception("The method or operation is not implemented.");
		}
	}
}