using System;

namespace JadEngine.Scene
{
    /// <summary>
    /// Parameters for a light's attenuation.
    /// </summary>
    public class JLightAttenuation
    {
        #region Fields
        private JLightAttenuationType _type;        
        private float _start;        
        private float _end;
        private float _constant;       
        private float _linear;       
        private float _quadratic;
        #endregion

        #region Properties
        /// <summary>
        /// The type of attenuation.
        /// </summary>
        public JLightAttenuationType Type
        {
            get { return _type; }
            set { _type = value; }
        }

        /// <summary>
        /// The attenuation start point, relative to the light's center.
        /// Not used for the standard attenuation type.
        /// </summary>
        public float Start
        {
            get { return _start; }
            set { _start = value; }
        }

        /// <summary>
        /// The attenuation end point, relative to the light's center.
        /// Used for discarding bounding objects affected from light, too.
        /// It's the same value as range.
        /// </summary>
        public float End
        {
            get { return _end; }
            set { _end = value; }
        }

        /// <summary>The constant value used only for the standard type.</summary>
        public float Constant
        {
            get { return _constant; }
            set { _constant = value; }
        }

        /// <summary>The linear value used only for the standard type.</summary>
        public float Linear
        {
            get { return _linear; }
            set { _linear = value; }
        }

        /// <summary>The quadratic value used only for the standard type.</summary>
        public float Quadratic
        {
            get { return _quadratic; }
            set { _quadratic = value; }
        }
        #endregion

        #region Constructors
        /// <summary>
        /// Default constructor.
        /// </summary>
        public JLightAttenuation()
        {
            this._type = JLightAttenuationType.None;
        }

        /// <summary>
        /// Extended constructor.
        /// </summary>
        /// <param name="type">The type of attenuation.</param>
        /// <param name="start">The attenuation start point, relative to the light's center.</param>
        /// <param name="end">The attenuation end point, relative to the light's center.</param>
        public JLightAttenuation(JLightAttenuationType type, float start, float end)
            : this(type, start, end, 0, 0, 0)
        {}

        /// <summary>
        /// Extended constructor.
        /// </summary>
        /// <param name="type">The type of attenuation.</param>
        /// <param name="start">The attenuation start point, relative to the light's center.</param>
        /// <param name="end">The attenuation end point, relative to the light's center.</param>
        /// <param name="constant">The constant value used only for standard type.</param>
        /// <param name="linear">The linear value used only for standard type.</param>
        /// <param name="quadratic">The quadratic value used only for standard type.</param>
        public JLightAttenuation(JLightAttenuationType type, float start, float end,
            float constant, float linear, float quadratic)
        {
            this._type = type;
            this._start = start;
            this._end = end;
            this._constant = constant;
            this._linear = linear;
            this._quadratic = quadratic;
        }
        #endregion

    }
}