using System;
using JadEngine.Core;
using Microsoft.DirectX.Direct3D;
using JadEngine.Video;

namespace JadEngine.Scene
{
    /// <summary>
    /// A corona is a texture that simulates an halo around the light.
    /// It uses queries to check the number of pixels and make it
    /// more or less bright.
    /// </summary>
    public class JLightCorona
    {
        #region Fields        
        private bool _enabled;
        private Query _maxPixelsQuery;        
        private Query _curPixelsQuery;
        private int _maxPixels;        
        private int _curPixels;        
        private JTexture _texture;        
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets if the light should have it's corona component enabled.
        /// </summary>
        public bool Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        } 
        /// <summary>
        /// The texture that represents the corona.
        /// </summary>
        public JTexture Texture
        {
            get { return _texture; }
            set { _texture = value; }
        }

        
        internal Query CurPixelsQuery
        {
            get { return _curPixelsQuery; }
            set { _curPixelsQuery = value; }
        }
        internal Query MaxPixelsQuery
        {
            get { return _maxPixelsQuery; }
            set { _maxPixelsQuery = value; }
        }
        internal int MaxPixels
        {
            get { return _maxPixels; }
            set { _maxPixels = value; }
        }
        internal int CurPixels
        {
            get { return _curPixels; }
            set { _curPixels = value; }
        }
        #endregion

        #region Constructors
        /// <summary>
        /// Default constructor.
        /// </summary>
        public JLightCorona()
        {}
        #endregion


        #region Methods

        /// <summary>
        /// Creates the texture and the device dependant objects
        /// </summary>
        public void Init()
        {
            this._texture = Jad.Video.Textures.Create2D("corona2.jpg", true);
            Reset();
        }

        /// <summary>
        /// TODO
        /// </summary>
        public void Destroy()
        {
            if (this._maxPixelsQuery != null)
            {
                this._maxPixelsQuery.Dispose();
                this._curPixelsQuery.Dispose();
            }
        }

        /// <summary>
        /// Creates the device dependent objects
        /// </summary>
        internal void Reset()
        {
            this._maxPixelsQuery = new Query(Jad.Video.Device, QueryType.Occlusion);
            this._curPixelsQuery = new Query(Jad.Video.Device, QueryType.Occlusion);
        }

        
        internal void OnLost()
        {
            Destroy();
        }

		//internal bool Load(JBinaryImport import)
		//{
		//    this._enabled = import.LoadBoolean();
		//    this._texture = (JTexture)import.LoadTexture();
		//    return true;
		//}

		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(this._enabled);
		//    export.Save(this._texture);
		//    return true;
		//}

        #endregion        
    }

}