using System;
using JadEngine.Video;
using JadEngine.Core;

namespace JadEngine.Scene
{
    /// <summary>
    /// This class represents the parameters for the light shaft effect.
    /// The number of planes determined by NumPlanes are 
    /// distributed between the lights position and its Attenuation.End 
    /// to create the visible effect.
    /// </summary>
    /// <remarks>Be careful with setting its values as you will get performance 
    /// slowdowns easily by setting too high values.
    /// </remarks>
    public class JLightShaft
    {
        #region Fields
        private bool _enabled;
        private int _numPlanes;
        private float _power;
        private float _intensity;
        private JTexture _cookieTexture;
        private bool _useCookie;
        private bool _projectShadow;
        private bool _softwareClipping;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets if the light shafts effect should be rendered.
        /// </summary>
        public bool Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        }
        /// <summary>
        /// Gets or sets the number of planes used to render the shafts effect.
        /// </summary>
        /// <remarks>
        /// Set this carefully as setting this too high will give a performance
        /// breakdown very fast.
        /// </remarks>
        public int NumPlanes
        {
            get { return _numPlanes; }
            set { _numPlanes = value; }
        }
        /// <summary>
        /// TODO
        /// </summary>
        public float Power
        {
            get { return _power; }
            set { _power = value; }
        }
        /// <summary>
        /// TODO
        /// </summary>
        public float Intensity
        {
            get { return _intensity; }
            set { _intensity = value; }
        }
        /// <summary>
        /// Gets or sets the texture used for the shafts effect.
        /// </summary>
        public JTexture CookieTexture
        {
            get { return _cookieTexture; }
            set
            {
                this._cookieTexture = value;
                this._useCookie = (value != null);
            }
        }
        /// <summary>
        /// Gets or sets if the cookie texture should be used to render the shaft effect.
        /// </summary>
        public bool UseCookie
        {
            get { return _useCookie; }
            set { _useCookie = value; }
        }
        /// <summary>
        /// Gets or sets if the shaft should be cut if it's in the shadow of an object the light points at.
        /// </summary>
        public bool ProjectShadow
        {
            get { return _projectShadow; }
            set { _projectShadow = value; }
        }        
        /// <summary>
        /// Gets or sets if software or hardware clipping should be used.
        /// </summary>
        public bool SoftwareClipping
        {
            get { return _softwareClipping; }
            set { _softwareClipping = value; }
        }
        #endregion

        #region Constrcutors
        /// <summary>
        /// Default constructor.
        /// </summary>
        public JLightShaft()
        {}
        #endregion

        #region Serialize

		//internal bool Load(JBinaryImport import)
		//{
		//    this._enabled = import.LoadBoolean();
		//    this._numPlanes = import.LoadInt32();
		//    this._power = import.LoadSingle();
		//    this._intensity = import.LoadSingle();
		//    this._useCookie = import.LoadBoolean();
		//    this._projectShadow = import.LoadBoolean();
		//    this._softwareClipping = import.LoadBoolean();

		//    bool textured = import.LoadBoolean();

		//    if (textured)
		//    {
		//        string textureName = import.LoadString();

		//        this._cookieTexture = (JTexture)Jad.Video.Textures[textureName];
		//    }

		//    return true;
		//}

		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(this._enabled);
		//    export.Save(this._numPlanes);
		//    export.Save(this._power);
		//    export.Save(this._intensity);
		//    export.Save(this._useCookie);
		//    export.Save(this._projectShadow);
		//    export.Save(this._softwareClipping);
		//    if (this._cookieTexture != null)
		//    {
		//        export.Save(true);
		//        export.Save(this._cookieTexture.Name);
		//    }
		//    else
		//    {
		//        export.Save(false);
		//    }

		//    return true;
		//}

        #endregion
    }

}