#region Using directives

using System;

using JadEngine.Tools.Parser;

#endregion

namespace JadEngine.Scene.MD5
{
	/// <summary>
	/// TODO
	/// </summary>
	public class JMD5Camera
	{
		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		public struct JKey
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public float Fov;

			/// <summary>
			/// TODO
			/// </summary>
			public float[] Position;

			/// <summary>
			/// TODO
			/// </summary>
			public float[] Rotation;

			#endregion
		}

		#endregion

		#region Fields

		/// <summary>
		/// Valid version of the MD5
		/// </summary>
		private const int MD5_VERSION = 10;

		/// <summary>
		/// TODO
		/// </summary>
		private int numFrames;
		
		/// <summary>
		/// TODO
		/// </summary>
		private int frameRate;
		
		/// <summary>
		/// TODO
		/// </summary>
		private int numCuts;
		
		/// <summary>
		/// TODO
		/// </summary>
		private int[] cuts;
		
		/// <summary>
		/// TODO
		/// </summary>
		private JKey[] keys;

		#endregion

		#region Properties

		/// <summary>
		/// TODO
		/// </summary>
		public int NumberOfFrames
		{
			get { return numFrames; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public int FrameRate
		{
			get { return frameRate; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public int NumberOfCuts
		{
			get { return numCuts; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public int[] Cuts
		{
			get { return cuts; }
		}

		/// <summary>
		/// TODO
		/// </summary>
		public JKey[] Keys
		{
			get { return keys; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="fileName">TODO</param>
		/// <returns>TODO</returns>
		public bool Load(string fileName)
		{
			int version, i;

			JParser parser = new JParser();

			if (!parser.LoadFromPath("Cameras", fileName))
			{
				parser.Error("Couldn't load file: {0}", fileName);
				return false;
			}

			// Version
			parser.ExpectTokenString("MD5Version");
			version = parser.ParseInt();
			if (version != MD5_VERSION)
			{
				parser.Error("Invalid version {0}. Should be version {1}", version, MD5_VERSION);
				return false;
			}

			// Command line
			parser.ExpectTokenString("commandline");
			parser.ParseString();

			// Number of frames
			parser.ExpectTokenString("numFrames");
			numFrames = parser.ParseInt();
			if (numFrames <= 0)
			{
				parser.Error("Invalid number of frames: {0}", numFrames);
				return false;
			}

			// Allocate memory for key frames
			keys = new JKey[numFrames];

			// Frame rate
			parser.ExpectTokenString("frameRate");
			frameRate = parser.ParseInt();
			if (frameRate <= 0)
			{
				parser.Error("Invalid frame rate: {0}", frameRate);
				return false;
			}

			// Number of cut-scenes
			parser.ExpectTokenString("numCuts");
			numCuts = parser.ParseInt();
			if (numCuts < 0)
			{
				parser.Error("Invalid number of cut-scenes: {0}", numCuts);
				return false;
			}

			// Allocate memory for cut-scenes
			cuts = new int[numCuts];

			// Parse cut-scenes if any
			if (numCuts > 0)
			{
				parser.ExpectTokenString("cuts");
				parser.ExpectTokenString("{");

				for (i = 0; i < numCuts; i++)
					cuts[i] = parser.ParseInt();

				parser.ExpectTokenString("}");
			}

			// Start camera data
			parser.ExpectTokenString("camera");
			parser.ExpectTokenString("{");

			// Parse camera keys
			for (i = 0; i < numFrames; i++)
			{
				// Position
				keys[i].Position = new float[3];
				parser.Parse1DMatrix(3, keys[i].Position);

				// Rotation
				keys[i].Rotation = new float[4];
				parser.Parse1DMatrix(3, keys[i].Rotation);

				// Calculate quaternion's W
				float term = 1.0f - keys[i].Rotation[0] * keys[i].Rotation[0] -
									keys[i].Rotation[1] * keys[i].Rotation[1] -
									keys[i].Rotation[2] * keys[i].Rotation[2];
				if (term < 0.0f)
					keys[i].Rotation[3] = 0.0f;
				
				else
					keys[i].Rotation[3] = -(float)Math.Sqrt(term);

				// FOV
				keys[i].Fov = parser.ParseFloat();
			}

			// End of file
			parser.ExpectTokenString("}");

			return true;
		}

		#endregion
	}
}