
using JadEngine.Video;
using System;

/*
 * 27/05/08
 *	-Added ICloneable interface
 */

namespace JadEngine.Scene
{
	/// <summary>
	/// Base class for the engine's material maps. 
	/// </summary>
	public abstract class JBaseMap : ICloneable
	{
		#region Fields
		protected JSampler _sampler;
		protected bool _enabled;
		#endregion

		#region Properties
		/// <summary>
		/// Gets or sets the sampler used for this map. 
		/// </summary>
		public JSampler Sampler
		{
			get { return _sampler; }
			set { Sampler = value; }
		}

		/// <summary>
		/// The texture used for this map. 
		/// </summary>
		public JTexture Texture
		{
			get
			{
				return (JTexture)_sampler.Texture;
			}
			set
			{
				_sampler.Texture = value;
			}
		}

		/// <summary>
		/// Gets or sets a value indicating whether this map is enabled and will be rendered.
		/// </summary>		
		public bool Enabled
		{
			get { return _enabled; }
			set { _enabled = value; }
		}

		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="JBaseMap"/> class.
		/// </summary>
		protected JBaseMap()
		{
			_sampler = new JSampler();
		}


		#endregion

		#region Methods

		/// <summary>
		/// Applies this map to an effect.
		/// </summary>
		/// <param name="effect">The effect to apply this map to.</param>
		public abstract void ApplyToEffect(JEffect effect);
		

		#endregion

		#region ICloneable Member

		public abstract object Clone();

		#endregion
	}
}