#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Tools.Parser;
using JadEngine.Physics;

#endregion

/*
 * 27/05/08
 *	-Added Clone()
 */

namespace JadEngine.Scene
{

	/// <summary>
	/// The Diffuse map
	/// </summary>
	public class JDiffuseMap : JBaseMap
	{
		#region Fields	
		
		private float _uTile;
		private float _vTile;		
		private JColor _color;		

		/// <summary>
		/// Indicates if the texture is used as difuse map or its the color
		/// </summary>
		//public bool TextureEnabled;

		#endregion

		#region Properties

		/// <summary>
		/// Tiling. This values will be used for all the maps!!
		/// </summary>
		public float UTile
		{
			get { return _uTile; }
			set { _uTile = value; }
		}

		/// <summary>
		/// Tiling. This values will be used for all the maps!!
		/// </summary>
		public float VTile
		{
			get { return _vTile; }
			set { _vTile = value; }
		}

		/// <summary>
		/// Color. It will be modulated with the texture
		/// </summary>
		public JColor Color
		{
			get { return _color; }
			set { _color = value; }
		}	

		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="JDiffuseMap"/> class.
		/// </summary>
		public JDiffuseMap()
		{
			_enabled = true;
			_color = JColors.White;
			_uTile = _vTile = 1f;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Initialize the map with the default values
		/// </summary>
		/// <param name="effect">The effect to apply this map to.</param>
		//public void Initialize()
		//{
		//    TextureEnabled = true;
		//    Color = JColors.White;
		//    UTile = VTile = 1f;
		//    Sampler.Initialize();
		//}


		/// <summary>
		/// Applies this map to an effect.
		/// </summary>
		/// <param name="effect">The effect to apply this map to.</param>
		public override void ApplyToEffect(JEffect effect)
		{
			if (_enabled)
			{
				if (effect.Handlers.TextureDiffuse != null && _sampler.Texture != null)
					effect.DXResource.SetValue(effect.Handlers.TextureDiffuse, _sampler.Texture.DXResource);

				_sampler.States.Process((int)JSamplerStage.Diffuse);
			}
		}

		public override object Clone()
		{
			JDiffuseMap clone = new JDiffuseMap();
			
			clone.Enabled = _enabled;
			clone.Texture = Texture;
			clone.Color = _color;
			clone.UTile = _uTile;
			clone.VTile = _vTile;

			return clone;
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    TextureEnabled = import.LoadBoolean();
		//    UTile = import.LoadSingle();
		//    VTile = import.LoadSingle();
		//    Color = import.LoadColor();

		//    Sampler.Load(import);

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save(TextureEnabled);
		//    export.Save(UTile);
		//    export.Save(VTile);
		//    export.Save(Color);

		//    Sampler.Save(export);

		//    return true;
		//}

		#endregion		
	}

}