#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Tools.Parser;
using JadEngine.Physics;

#endregion

/*
 * 27/05/08
 *	-Added Clone()
 */

namespace JadEngine.Scene
{
	/// <summary>
	/// Used for the parallax effect.
	/// Actually the heightmap its in the alpha of the normal map
	/// </summary>
	public class JHeightMap : JBaseMap
	{
		#region Fields
		
		private bool _occlusionMapping;		
		private float _parallaxAmount;
		private float _perspectiveBias;
		private float _shadowOpacity;		

		#endregion

		#region Properties

		/// <summary>
		/// Enable / Disable the parallax occlusion mapping advanced effect
		/// </summary>
		public bool OcclusionMapping
		{
			get { return _occlusionMapping; }
			set { _occlusionMapping = value; }
		}

		/// <summary>
		/// Parallax effect amount
		/// </summary>
		public float ParallaxAmount
		{
			get { return _parallaxAmount; }
			set { _parallaxAmount = value; }
		}

		/// <summary>
		/// Perspective bias amount, used only for Occlusion mapping
		/// </summary>
		public float PerspectiveBias
		{
			get { return _perspectiveBias; }
			set { _perspectiveBias = value; }
		}

		/// <summary>
		/// Shadow opacity
		/// </summary>
		public float ShadowOpacity
		{
			get { return _shadowOpacity; }
			set { _shadowOpacity = value; }
		}

		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="JHeightMap"/> class.
		/// </summary>
		public JHeightMap()
		{			
			_parallaxAmount = 0.1f;
			_perspectiveBias = 0.5f;
			_shadowOpacity = 0.5f;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Initialize the map with the default values
		/// </summary>
		//public void Initialize()
		//{
		//    Enabled = false;
		//    OcclusionMapping = false;
		//    ParallaxAmount = 0.1f;
		//    PerspectiveBias = 0.5f;
		//    ShadowOpacity = 0.5f;
		//    Sampler.Initialize();
		//}

		/// <summary>
		/// Applies the map to the effect
		/// Actually the  heightmap its in the alpha of the normal
		/// </summary>
		/// <param name="effect">A valid instance</param>
		public override void ApplyToEffect(JEffect effect)
		{
			// Actually the  heightmap its in the alpha of the normal
		}

		public override object Clone()
		{
			JHeightMap clone = new JHeightMap();

			clone.Enabled = _enabled;
			clone.Texture = Texture;
			clone.OcclusionMapping = _occlusionMapping;
			clone.ParallaxAmount = _parallaxAmount;
			clone.PerspectiveBias = _perspectiveBias;
			clone.ShadowOpacity = _shadowOpacity;

			return clone;
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    Enabled = import.LoadBoolean();
		//    OcclusionMapping = import.LoadBoolean();
		//    ParallaxAmount = import.LoadSingle();
		//    PerspectiveBias = import.LoadSingle();
		//    ShadowOpacity = import.LoadSingle();

		//    Sampler.Load(import);

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save(Enabled);
		//    export.Save(OcclusionMapping);
		//    export.Save(ParallaxAmount);
		//    export.Save(PerspectiveBias);
		//    export.Save(ShadowOpacity);

		//    Sampler.Save(export);

		//    return true;
		//}

		#endregion
	}


}