#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Tools.Parser;
using JadEngine.Physics;

#endregion

/*
 * 27/05/08
 *	-Added Clone()
 */

namespace JadEngine.Scene
{
	/// <summary>
	/// The lightmap is a map that is modulated with the diffuse to
	/// give an aproximation of global ilumination
	/// </summary>
	public class JLightMap : JBaseMap
	{
		#region Fields
		
		private JColor _color;		

		#endregion

		#region Properties

		/// <summary>
		/// The color will be modulated with the texture
		/// </summary>
		public JColor Color
		{
			get { return _color; }
			set { _color = value; }
		}	

		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="JLightMap"/> class.
		/// </summary>
		public JLightMap()
		{
			_color = JColors.White;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Initialize the map with the default values
		/// </summary>
		//public void Initialize()
		//{
		//    Enabled = false;
		//    Color = JColors.White;
		//    Sampler.Initialize();
		//}

		/// <summary>
		/// Not implemented.
		/// </summary>
		/// <param name="effect">Not implemented.</param>
		public override void ApplyToEffect(JEffect effect)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public override object Clone()
		{
			JLightMap clone = new JLightMap();

			clone.Enabled = _enabled;
			clone.Texture = Texture;
			clone.Color = _color;

			return clone;
		}

		/// <summary>
		/// Return the color if the map is active, JColors.White if not
		/// </summary>
		/// <returns>The color of the map if active or White if not</returns>
		//public JColor ActiveColor()
		//{
		//    if (_enabled)
		//        return Color;
		//    else
		//        return JColors.White;
		//}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    Enabled = import.LoadBoolean();
		//    Color = import.LoadColor();

		//    Sampler.Load(import);

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save(Enabled);
		//    export.Save(Color);

		//    Sampler.Save(export);

		//    return true;
		//}

		#endregion		
	}

}