#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Tools.Parser;
using JadEngine.Physics;

#endregion

/*
 * 27/05/08
 *	-Added Clone()
 */

namespace JadEngine.Scene
{
	/// <summary>
	/// The normal map has normal information and its used to give a bumpy effect
	/// </summary>
	public class JNormalMap : JBaseMap
	{
		#region Fields	
		
		private float _bumpiness;
		
		#endregion

		#region Properties

		/// <summary>
		/// The Bumpy strength
		/// </summary>
		public float Bumpiness
		{
			get { return _bumpiness; }
			set { _bumpiness = value; }
		}	

		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="JNormalMap"/> class.
		/// </summary>
		public JNormalMap()
		{
			_bumpiness = 1.0f;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Initialize the map with the default values
		/// </summary>
		//public void Initialize()
		//{
		//    Enabled = false;
		//    Bumpiness = 1f;
		//    Sampler.Initialize();
		//}

		/// <summary>
		/// Applies this map to an effect.
		/// </summary>
		/// <param name="effect">The effect to apply this map to.</param>
		public override void ApplyToEffect(JEffect effect)
		{
			if (_enabled)
			{
				if (effect.Handlers.TextureNormal != null && _sampler.Texture != null)
					effect.DXResource.SetValue(effect.Handlers.TextureNormal, _sampler.Texture.DXResource);

				_sampler.States.Process((int)JSamplerStage.Normal);
			}
		}

		public override object Clone()
		{
			JNormalMap clone = new JNormalMap();

			clone.Enabled = _enabled;
			clone.Texture = Texture;
			clone.Bumpiness = _bumpiness;

			return clone;
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    Enabled = import.LoadBoolean();
		//    Bumpiness = import.LoadSingle();

		//    Sampler.Load(import);

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save(Enabled);
		//    export.Save(Bumpiness);

		//    Sampler.Save(export);

		//    return true;
		//}

		#endregion
	}

}