#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Tools.Parser;
using JadEngine.Physics;

#endregion

/*
 * 27/05/08
 *	-Added Clone()
 */

namespace JadEngine.Scene
{
	/// <summary>
	/// The cube reflection map
	/// </summary>
	public class JReflectionMap : JBaseMap
	{
		#region Fields
		
		private float _reflectivity;
		
		#endregion

		#region Properties

		/// <summary>
		/// The cube texture used for this map.
		/// </summary>
		public new JCubeTexture Texture
		{
			get
			{
				return (JCubeTexture)_sampler.Texture;
			}
			set
			{
				_sampler.Texture = value;
			}
		}

		/// <summary>
		/// Reflectivity
		/// </summary>
		public float Reflectivity
		{
			get { return _reflectivity; }
			set { _reflectivity = value; }
		}		

		#endregion

		#region Constructor
		/// <summary>
		/// Initializes a new instance of the <see cref="JReflectionMap"/> class.
		/// </summary>
		public JReflectionMap()
		{
			_reflectivity = 0.5f;
		}
		#endregion

		#region Methods

		/// <summary>
		/// Initialize the map with the default values
		/// </summary>
		//public void Initialize()
		//{
		//    Enabled = false;
		//    Reflectivity = 0.5f;
		//    Sampler.Initialize();
		//}

		/// <summary>
		/// Applies this map to an effect.
		/// </summary>
		/// <param name="effect">The effect to apply this map to.</param>
		public override void ApplyToEffect(JEffect effect)
		{
			if (_enabled)
			{
				if (effect.Handlers.TextureReflection != null && _sampler.Texture != null)
					effect.DXResource.SetValue(effect.Handlers.TextureReflection, _sampler.Texture.DXResource);

				_sampler.States.Process((int)JSamplerStage.Reflection);
			}
		}

		public override object Clone()
		{
			JReflectionMap clone = new JReflectionMap();

			clone.Enabled = _enabled;
			clone.Texture = Texture;
			clone.Reflectivity = _reflectivity;

			return clone;
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    Enabled = import.LoadBoolean();
		//    Reflectivity = import.LoadSingle();

		//    Sampler.Load(import);

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save(Enabled);
		//    export.Save(Reflectivity);

		//    Sampler.Save(export);

		//    return true;
		//}

		#endregion
	}

}