#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Tools.Parser;
using JadEngine.Physics;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The cube refraction map
	/// </summary>
	public class JRefractionMap : JBaseMap
	{
		#region Fields

		private float _transmittance;			
		private float _indexOfRefraction;		
				
		#endregion

		#region Properties

		/// <summary>
		/// The cube texture used for this map.
		/// </summary>
		public new JCubeTexture Texture
		{
			get
			{
				return (JCubeTexture)_sampler.Texture;
			}
			set
			{
				_sampler.Texture = value;
			}
		}

		/// <summary>
		/// Index Of Refraction
		/// </summary>
		public float IndexOfRefraction
		{
			get { return _indexOfRefraction; }
			set { _indexOfRefraction = value; }
		}

		/// <summary>
		/// Transmittance
		/// </summary>
		public float Transmittance
		{
			get { return _transmittance; }
			set { _transmittance = value; }
		}	

		#endregion

		#region Constructor
		public JRefractionMap()
		{
			_transmittance = 0.5f;
			_indexOfRefraction = 1.0f;
		}
		#endregion

		#region Methods

		/// <summary>
		/// Initialize the map with the default values
		/// </summary>
		//public void Initialize()
		//{
		//    Enabled = false;
		//    Transmittance = 0.5f;
		//    IndexOfRefraction = 1.0f;
		//    Sampler.Initialize();
		//}

		/// <summary>
		/// Applies this map to an effect.
		/// </summary>
		/// <param name="effect">The effect to apply this map to.</param>
		public override void ApplyToEffect(JEffect effect)
		{
			if (_enabled)
			{
				if (effect.Handlers.TextureRefraction != null && _sampler.Texture != null)
					effect.DXResource.SetValue(effect.Handlers.TextureRefraction, _sampler.Texture.DXResource);

				_sampler.States.Process((int)JSamplerStage.Refraction);
			}
		}

		public override object Clone()
		{
			JRefractionMap clone = new JRefractionMap();

			clone.Enabled = _enabled;
			clone.Texture = Texture;
			clone.IndexOfRefraction = _indexOfRefraction;
			clone.Transmittance = _transmittance;

			return clone;
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    Enabled = import.LoadBoolean();
		//    Transmittance = import.LoadSingle();
		//    IndexOfRefraction = import.LoadSingle();

		//    Sampler.Load(import);

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save(Enabled);
		//    export.Save(Transmittance);
		//    export.Save(IndexOfRefraction);

		//    Sampler.Save(export);

		//    return true;
		//}

		#endregion
	}

}