using System;
using JadEngine.Mathematics.Geometry;

namespace JadEngine.Scene
{
    /// <summary>
    /// The bounding information of a 3D object.
    /// </summary>
    public class JBounds
    {
        #region Fields
        private JBoundingBox3 _localBoundingBox;
        private JBoundingSphere _localBoundingSphere;
        private JBoundingBox3 _boundingBox;
        private JBoundingSphere _boundingSphere;
        private JBoundingBox3 _hierarchicalBoundingBox;
        private JBoundingSphere _hierarchicalBoundingSphere;
        private bool _useToComputeBoundings;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets if this node is taken into account when computing boundings.
        /// </summary>
        /// <remarks>Normally only set to true for mesh objects.</remarks>
        public bool UseToComputeBoundings
        {
            get { return _useToComputeBoundings; }
            set { _useToComputeBoundings = value; }
        }

        /// <summary>
        /// Gets the hierarchical bounding box.
        /// </summary>
        public JBoundingBox3 HierarchicalBoundingBox
        {
            get { return _hierarchicalBoundingBox; }
            internal set { this._hierarchicalBoundingBox = value; }
        }

        /// <summary>
        /// Gets the world bounding box.
        /// </summary>
        public JBoundingBox3 BoundingBox
        {
            get { return _boundingBox; }
            internal set { this._boundingBox = value; }
        }

        /// <summary>
        /// Gets the world bounding sphere.
        /// </summary>
        public JBoundingSphere BoundingSphere
        {
            get { return _boundingSphere; }
            internal set { this._boundingSphere = value; }
        }

        /// <summary>
        /// Gets the hierarchical bounding sphere.
        /// </summary>
        public JBoundingSphere HierarchicalBoundingSphere
        {
            get { return _hierarchicalBoundingSphere; }
            internal set { this._hierarchicalBoundingSphere = value; }
        }

        /// <summary>
        /// Gets or sets the object space bounding box.
        /// </summary>
        public JBoundingBox3 LocalBoundingBox
        {
            get { return _localBoundingBox; }
            set { _localBoundingBox = value; }
        }

        /// <summary>
        /// Gets or sets the object space bounding sphere.
        /// </summary>
        public JBoundingSphere LocalBoundingSphere
        {
            get { return _localBoundingSphere; }
            set { _localBoundingSphere = value; }
        }        
        #endregion
        
        #region Constructors
        /// <summary>
        /// Default constructor.
        /// </summary>
        public JBounds()
        {
            this._localBoundingBox = new JBoundingBox3();
            this._localBoundingSphere = new JBoundingSphere();

            this._boundingBox = new JBoundingBox3();
            this._boundingSphere = new JBoundingSphere();

            this._hierarchicalBoundingBox = new JBoundingBox3();
            this._hierarchicalBoundingSphere = new JBoundingSphere();
        }
        #endregion
    }
}