#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{
		static JColor _black = JColors.Black;

		#region FogPass

		/// <summary>
		/// Renders the fog pass
		/// </summary>
		void RenderFogPass()
		{
			// To make the fog pass, proccess the objects again and
			// mix the final color with the fog

			// Copy the backbuffer to another texture

            CopyBackBufferToBackBufferTexture();

			Jad.Video.StateManager.SetRenderState(RenderStates.ZEnable, true);
			Jad.Video.StateManager.SetRenderState(RenderStates.ZBufferWriteEnable, false);
			Jad.Video.StateManager.SetRenderState(RenderStates.AlphaBlendEnable, false);

			// Clear the render because if not the whole screen  is rendered some ugly artifacts
			// will appear

			view.Clear(ClearFlags.Target, ref _black, 1, 0);

            if (scene.World != null) scene.World.RenderFog(scene);

			for (int i = 0; i < materials.Count; i++)
			{
				JMaterial m = materials[i];

				// If there arent objects to render, continue
				if (m.meshObjectPerMaterial.Count == 0) continue;

				JMaterialLayer layer = m.Layers[0];

				m.Begin();

				layer.Begin();

				// Create the effect for this pass

				JEffect effect = Jad.EffectBuilder.SetFogPass(layer);

				int passes = effect.Begin();

				// Set the material values

				layer.ApplyEffectFogPass(effect);

				for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
				{
					JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
					JMeshObject mesh = mp.meshObject;

					mesh.RenderBase(mp.numMat, passes, effect);
				}

				effect.End();
				layer.End();
			}

			scene.BackBuffer = null;

		}

		#endregion

	}
}