#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Video;
using JadEngine.Particles;
using JadEngine.Mathematics;
using JadEngine.Mathematics.Geometry;

#endregion

namespace JadEngine.Scene
{
	/// <summary>
	/// The Render scene class for shaders 2.0
	/// </summary>
	public partial class JSceneRenderer2_0 : JSceneRenderer
	{
		static JColor _clearColor0_5 = new JColor(0, 0, 0, 0.5f);

		#region GlowDOFPass

		/// <summary>
		/// Renders the Glow and DOF Pass
		/// </summary>
		void RenderGlowDOFPass()
		{
			// If the Glow effect is activated, generate the Glow map
			// Give a second pass to calculate the Glow and the DOF

			if (scene.GlowEffect.Enabled || scene.DepthOfField.Enabled)
			{
				status = JSceneStatus.GlowDOFPass;

				// Now we need to copy the backbuffer to a RenderTarget
				// Because we need to use the backbuffer as a source for the
				// Glow Effect

				view.CopyToRenderTarget(view.RenderTargets.Primary);

				scene.BackBuffer = view.RenderTargets.Primary;

				view.RenderTargets.Glow.Begin();

				// The DOF value is saved in the Z, 0.5 means focused
				view.Clear(ClearFlags.Target, ref _clearColor0_5, 0, 0);

				for (int i = 0; i < materials.Count; i++)
				{
					JMaterial m = materials[i];

					// If there arent objects to render continue
					if (m.meshObjectPerMaterial.Count == 0) continue;

					JMaterialLayer layer = m.Layers[0];

					// Generate this pass if DOF or EmissiveMap are enabled
					if (layer.EmissiveMap.Enabled == false && scene.DepthOfField.Enabled == false) continue;

					m.Begin();

					layer.Begin();

					// Create the effect for this pass

					JEffect effect = Jad.EffectBuilder.SetGlowDOFPass(layer);

					int npasos = effect.Begin();

					// Set the material values

					layer.ApplyEffectGlowDOFPass(effect);

					// Do not use alpha test, because we return the DOF value in the alpha
					Jad.Video.StateManager.SetRenderState(RenderStates.AlphaTestEnable, false);

					for (int p = 0; p < m.meshObjectPerMaterial.Count; p++)
					{
						JMeshObjectPerMaterial mp = m.meshObjectPerMaterial[p];
						JMeshObject mesh = mp.meshObject;

						mesh.RenderBase(mp.numMat, npasos, effect);
					}

					effect.End();
					layer.End();
				}

				view.RenderTargets.Glow.End();
				scene.BackBuffer = null;
			}
		}

		#endregion

	}
}