using System;
using System.Collections.Generic;
using System.Text;

namespace JadEngine.Sound
{
    public class JSoundSourceBase : IJSoundSource
    {
        private string _name;
        private JSoundState _state;
        private bool _looping;
        private float _frequency, _pitch, _gain;
        private int _length;

        /// <summary>
        /// Creates a new instance of the <see cref="JSoundSourceBase"/> class
        /// </summary>
        /// <param name="name">Name of the <see cref="IJSoundSource"/></param>
        protected internal JSoundSourceBase(string name)
        {
            this._name = name;
            this._state = JSoundState.Stopped;
            this._looping = false;
            this._frequency = 0.0f;
            this.Length = 0;
            this._pitch = 0;
            this._gain = 0;
        }

        #region IJSoundSource Members

        public virtual string Name
        {
            get { return this._name; }
            protected set { this._name = value; }
        }

        public int Length
        {
            get { return this._length; }
            protected set { this._length = value; }
        }

        public virtual JSoundState State
        {
            get { return this._state; }
            protected set { this._state = value; }
        }

        public virtual bool IsLooping
        {
            get
            {
                return this._looping;
            }
            set
            {
                this._looping = value;
            }
        }

        public virtual float Pitch
        {
            get { return this._pitch; }
            set { this._pitch = value; }
        }

        public virtual float Gain
        {
            get { return this._gain; }
            set { this._gain = value; }
        }

        public virtual float Frequency
        {
            get { return this._frequency; }
            protected set { this._frequency = value; }
        }

        public virtual void Play()
        {
        }

        public virtual void Pause()
        {
        }

        public virtual void Resume()
        {
        }

        public virtual void Stop()
        {
        }

        public virtual void Rewind()
        {
        }

        public virtual void SetPosition(float x, float y, float z)
        {
        }

        public virtual void SetOrientation(float[] orientation)
        {
        }

        public virtual void SetVelocity(float x, float y, float z)
        {
        }

        public virtual void MakeGlobal()
        {
        }

        #endregion

        #region IDisposable Members

        public virtual void Dispose()
        {
        }

        #endregion
    }
}
