/*      Author Name:    Kristijan Simic
 *      E-Mail:         simic (at) cip(dot)ifi(dot)lmu(dot)de
 *      
 *      The Terrain rendering classes and methods were created for the JadEngine using C# and managed directx
 *      during my university project thesis. The source code was mainly influenced and led by the following
 *      ressources:
 * 
 *      http://www.flipcode.com/articles/article_geomipmaps.pdf by Willem H. de Boer, whdeboer@iname.com
 *      
 *      "Focus on 3D Terrain Programming" by Trent Pollack - Premier Press (C) 2003
 *      http://portal.acm.org/citation.cfm?id=862706&dl=acm&coll=&CFID=15151515&CFTOKEN=6184618
 * 
 *      Thanks go out to:
 * 
 *      ReedC for great tips around the engine, its structure and simply beeing the nice guy that he is.
 *      
 *      Feel free to contact me.
 * */

#region Using directives

using System;
using System.Collections.Generic;
using System.Text;
using JadEngine.Mathematics.Geometry;
#endregion

namespace JadEngine.Terrain
{
    /// <summary>
    /// This is the terrain patch structure. Every terrain patch has its LOD level, a indice list and a index count
    /// specifying the number of elements to be used from the indice array.
    /// </summary>
    struct JTerrainPatch
    {
        #region Fields

        /// <summary>
        /// The current LOD level of the terrain patch
        /// </summary>
        private int   s_LOD;

        /// <summary>
        /// The index list for the terrain patch, containing
        /// the vertex render order.
        /// </summary>
        private int[] s_Indices;

        /// <summary>
        /// Holds the number of indices from the s_Indices array that are required to render the patch
        /// </summary>
        private int   s_IndiceCount;

        /// <summary>
        /// Holds the BoundingBox of each terrain patch
        /// </summary>
        private JBoundingBox3 s_bbox;

        /// <summary>
        /// Holds the state, wheater the boundingbox should be rendered or not
        /// </summary>
        private bool s_renderBBox;

        #endregion Fields

        #region Properties

        /// <summary>
        /// Propertie to get / set the current LOD level
        /// </summary>
        public int lodLevel
        {
            set { s_LOD = value; }
            get { return s_LOD; }
        }

        /// <summary>
        /// Propertie to get / set the current indice list
        /// </summary>
        public int[] Indices
        {
            set { s_Indices = value; }
            get { return s_Indices; }
        }

        /// <summary>
        /// Propertie to get / set the current inice count
        /// </summary>
        public int indiceCount
        {
            set { s_IndiceCount = value; }
            get { return s_IndiceCount; }
        }

        /// <summary>
        /// Propertie to get / set the bounding box
        /// </summary>
        public JBoundingBox3 boundingbox
        {
            set { s_bbox = value; }
            get { return s_bbox; }
        }

        /// <summary>
        /// Propertie to get / set renderBBox value
        /// </summary>
        public bool renderBBox
        {
            set { s_renderBBox = value; }
            get { return s_renderBBox; }
        }

        #endregion
    }
}
