#region Using Directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;

#endregion

namespace JadEngine.Tools
{
	/// <summary>
	/// This class produces various shapes that can be used for debugging graphically
	/// </summary>
	/// <remarks>
	/// This class has a similar idea to the <see cref="JadEngine.Scene.JGizmo"/> classes, but
	/// instead of building a mesh and drawing it in wireframe mode it creates points to
	/// later call the method Jad.Video.Effects.InHouse.posColor.RenderLines.
	/// I have no idea which way is better to be truth, but this one is very easy to implement
	/// </remarks>
	public static class JDebugShapes
	{
		#region Methods

		/// <summary>
		/// Creates a small cross in the XZ plane
		/// </summary>
		/// <param name="points">List of points where the new ones will be added</param>
		/// <param name="center">Center of the cross</param>
		/// <param name="halfLength">Half lenght of one side of the cross</param>
		public static void CrossInXZ(List<Vector3> points, Vector3 center, float halfLength)
		{
			center.X -= halfLength;
			points.Add(center);

			center.X += 2 * halfLength;
			points.Add(center);

			center.X -= halfLength;
			center.Z -= halfLength;
			points.Add(center);

			center.Z += 2 * halfLength;
			points.Add(center);
		}

		/// <summary>
		/// Creates a circle in the XZ plane
		/// </summary>
		/// <param name="points">List of points where the new ones will be added</param>
		/// <param name="center">Center of the circle</param>
		/// <param name="radius">Radius of the circle</param>
		/// <param name="numberPoints">Number of points to form the circle</param>
		public static void CircleInXZ(List<Vector3> points, Vector3 center, float radius, int numberPoints)
		{
			float delta = (float) (2.0 * Math.PI) / numberPoints;

			for (int i = 0; i < numberPoints; i++)
			{
				float radians = delta * i;
				float cos = (float) Math.Cos(radians) * radius;
				float sin = (float) Math.Sin(radians) * radius;
				
				Vector3 point = new Vector3(cos + center.X, center.Y, sin + center.Z);
				points.Add(point);
				
				if (i != 0)
					points.Add(point);
			}

			//Close the circle
			points.Add(points[points.Count - 2 * numberPoints + 1]);
		}

		/// <summary>
		/// Creates a dotted circle in the XZ plane
		/// </summary>
		/// <param name="points">List of points where the new ones will be added</param>
		/// <param name="center">Center of the circle</param>
		/// <param name="radius">Radius of the circle</param>
		/// <param name="numberPoints"></param>
		public static void DottedCircleInXZ(List<Vector3> points, Vector3 center, float radius, int numberPoints)
		{
			float delta = (float) (2.0 * Math.PI) / numberPoints;

			for (int i = 0; i < numberPoints; i++)
			{
				float radians = delta * i;
				float cos = (float) Math.Cos(radians) * radius;
				float sin = (float) Math.Sin(radians) * radius;

				Vector3 point = new Vector3(cos + center.X, center.Y, sin + center.Z);
				points.Add(point);
			}
		}

		#endregion
	}
}
