#region Using directives

using System.Drawing;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Video.Adapter;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Implements the DX9 device
    /// </summary>
    public class JDeviceDX9
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JVideo _video;

        /// <summary>
        /// The DX9 Device
        /// </summary>
        internal Device _device;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the Device
        /// </summary>
        public Device Device
        {
            get { return _device; }
        }

        /// <summary>
        /// Gets/Sets the VertexShader
        /// </summary>
        public VertexShader VertexShader
        {
            get { return _device.VertexShader; }
            set { _device.VertexShader = value; }
        }

        /// <summary>
        /// Gets/Sets the PixelShader
        /// </summary>
        public PixelShader PixelShader
        {
            get { return _device.PixelShader; }
            set { _device.PixelShader = value; }
        }

        /// <summary>
        /// Gets/Sets the IndexBuffer
        /// </summary>
        public IndexBuffer Indices
        {
            get { return _device.Indices; }
            set { _device.Indices = value; }
        }

        /// <summary>
        /// Gets/Sets the current depth stencil surface
        /// </summary>
        public Surface DepthStencilSurface
        {
            get { return _device.DepthStencilSurface; }
            set { _device.DepthStencilSurface = value; }
        }
        /// <summary>
        /// Gets The Clip Planes
        /// </summary>
        public ClipPlanes ClipPlanes
        {
            get { return _device.ClipPlanes; }
        }

        /// <summary>
        /// Gets/Sets The Viewport
        /// </summary>
        public Viewport Viewport
        {
            get { return _device.Viewport; }
            set { _device.Viewport = value; }
        }

        /// <summary>
        /// Gets/Sets The VertexFormat
        /// </summary>
        public VertexFormats VertexFormat
        {
            get { return _device.VertexFormat; }
            set { _device.VertexFormat = value; }
        }

        /// <summary>
        /// Retrieves or sets a vertex shader declaration
        /// </summary>
        public VertexDeclaration VertexDeclaration
        {
            get { return _device.VertexDeclaration; }
            set { _device.VertexDeclaration = value; }
        }

        #endregion

        #region Constructors

        internal JDeviceDX9(JVideo video)
        {
            _video = video;
        }

        #endregion

        #region GetRenderState

        /// <summary>
        /// Retrieves the integer value of a given render state.
        /// </summary>
        /// <param name="renderState">A member from the RenderStates enumeration that represents the render state value to retreive.</param>
        /// <returns>The render state value retrieved.</returns>
        public int GetRenderStateInt32(RenderStates renderState)
        {
            return _device.GetRenderStateInt32(renderState);
        }

        /// <summary>
        /// Retrieves the boolean value of a given render state.
        /// </summary>
        /// <param name="renderState">A member from the RenderStates enumeration that represents the render state value to retreive.</param>
        /// <returns>The render state value retrieved.</returns>
        public bool GetRenderStateBoolean(RenderStates renderState)
        {
            return _device.GetRenderStateBoolean(renderState);
        }

        /// <summary>
        /// Retrieves the single value of a given render state.
        /// </summary>
        /// <param name="renderState">A member from the RenderStates enumeration that represents the render state value to retreive.</param>
        /// <returns>The render state value retrieved.</returns>
        public float GetRenderStateSingle(RenderStates renderState)
        {
            return _device.GetRenderStateSingle(renderState);
        }

        #endregion

        #region GetTextureStageState

        /// <summary>
        /// Retrieves the floating-point value of a given texture stage state.
        /// </summary>
        /// <param name="stage">Index value of the texture stage to retrieve</param>
        /// <param name="state">A member from the TextureStageStates enumeration that represents the texture stage state value to retreive.</param>
        /// <returns>The texture stage state value retrieved.</returns>
        public float GetTextureStageStateSingle(int stage, TextureStageStates state)
        {
            return _device.GetTextureStageStateSingle(stage, state);
        }

        /// <summary>
        /// Retrieves the integer value of a given texture stage state.
        /// </summary>
        /// <param name="stage">Index value of the texture stage to retrieve</param>
        /// <param name="state">A member from the TextureStageStates enumeration that represents the texture stage state value to retreive.</param>
        /// <returns>The texture stage state value retrieved.</returns>
        public int GetTextureStageStateInt32(int stage, TextureStageStates state)
        {
            return _device.GetTextureStageStateInt32(stage, state);
        }

        /// <summary>
        /// Retrieves the boolean value of a given texture stage state.
        /// </summary>
        /// <param name="stage">Index value of the texture stage to retrieve</param>
        /// <param name="state">A member from the TextureStageStates enumeration that represents the texture stage state value to retreive.</param>
        /// <returns>The texture stage state value retrieved.</returns>
        public bool GetTextureStageStateBoolean(int stage, TextureStageStates state)
        {
            return _device.GetTextureStageStateBoolean(stage, state);
        }

        #endregion

        #region GetSamplerStageState

        /// <summary>
        /// Retrieves the floating-point value of a given sampler stage state.
        /// </summary>
        /// <param name="stage">Index value of the texture stage to retrieve</param>
        /// <param name="state">A member from the SamplerStageStates enumeration that represents the texture stage state value to retreive.</param>
        /// <returns>The sampler stage state value retrieved.</returns>
        public float GetSamplerStageStateSingle(int stage, SamplerStageStates state)
        {
            return _device.GetSamplerStageStateSingle(stage, state);
        }

        /// <summary>
        /// Retrieves the integer value of a given sampler stage state.
        /// </summary>
        /// <param name="stage">Index value of the texture stage to retrieve</param>
        /// <param name="state">A member from the SamplerStageStates enumeration that represents the texture stage state value to retreive.</param>
        /// <returns>The sampler stage state value retrieved.</returns>
        public int GetSamplerStageStateInt32(int stage, SamplerStageStates state)
        {
            return _device.GetSamplerStageStateInt32(stage, state);
        }

        /// <summary>
        /// Retrieves the boolean value of a given sampler stage state.
        /// </summary>
        /// <param name="stage">Index value of the texture stage to retrieve</param>
        /// <param name="state">A member from the SamplerStageStates enumeration that represents the texture stage state value to retreive.</param>
        /// <returns>The tesamplerxture stage state value retrieved.</returns>
        public bool GetSamplerStageStateBoolean(int stage, SamplerStageStates state)
        {
            return _device.GetSamplerStageStateBoolean(stage, state);
        }

        #endregion

        #region SetSamplerStageState

        /// <summary>
        /// Sets a sampler stage state value
        /// </summary>
        /// <param name="stage">Index value of the sampler stage to set.</param>
        /// <param name="state">A member from the SamplerStageStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A Boolean value to set for the given sampler stage state.</param>
        public void SetSamplerState(int stage, SamplerStageStates state, bool value)
        {
            _device.SetSamplerState(stage, state, value);
        }

        /// <summary>
        /// Sets a sampler stage state value
        /// </summary>
        /// <param name="stage">Index value of the sampler stage to set.</param>
        /// <param name="state">A member from the SamplerStageStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A integer value to set for the given sampler stage state.</param>
        public void SetSamplerState(int stage, SamplerStageStates state, int value)
        {
            _device.SetSamplerState(stage, state, value);
        }

        /// <summary>
        /// Sets a sampler stage state value
        /// </summary>
        /// <param name="stage">Index value of the sampler stage to set.</param>
        /// <param name="state">A member from the SamplerStageStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A float value to set for the given sampler stage state.</param>
        public void SetSamplerState(int stage, SamplerStageStates state, float value)
        {
            _device.SetSamplerState(stage, state, value);
        }

        #endregion

        #region SetTextureStageState

        /// <summary>
        /// Sets a texture stage state value
        /// </summary>
        /// <param name="stage">Index value of the texture stage to set.</param>
        /// <param name="state">A member from the TextureStageStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A Boolean value to set for the given sampler stage state.</param>
        public void SetTextureStageState(int stage, TextureStageStates state, bool value)
        {
            _device.SetTextureStageState(stage, state, value);
        }

        /// <summary>
        /// Sets a texture stage state value
        /// </summary>
        /// <param name="stage">Index value of the texture stage to set.</param>
        /// <param name="state">A member from the TextureStageStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A integer value to set for the given sampler stage state.</param>
        public void SetTextureStageState(int stage, TextureStageStates state, int value)
        {
            _device.SetTextureStageState(stage, state, value);
        }

        /// <summary>
        /// Sets a sampler stage state value
        /// </summary>
        /// <param name="stage">Index value of the texture stage to set.</param>
        /// <param name="state">A member from the TextureStageStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A float value to set for the given sampler stage state.</param>
        public void SetTextureStageState(int stage, TextureStageStates state, float value)
        {
            _device.SetTextureStageState(stage, state, value);
        }

        #endregion

        #region SetRenderState

        /// <summary>
        /// Sets a render state value
        /// </summary>
        /// <param name="stage">Index value of the render stage to set.</param>
        /// <param name="state">A member from the RenderStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A Boolean value to set for the given sampler stage state.</param>
        public void SetRenderState(RenderStates state, bool value)
        {
            _device.SetRenderState(state, value);
        }

        /// <summary>
        /// Sets a render state value
        /// </summary>
        /// <param name="stage">Index value of the sampler stage to set.</param>
        /// <param name="state">A member from the RenderStates enumeration that represents the sampler stage state value to set.</param>
        public void SetRenderState(RenderStates state, int value)
        {
            _device.SetRenderState(state, value);
        }

        /// <summary>
        /// Sets a render state value
        /// </summary>
        /// <param name="state">A member from the RenderStates enumeration that represents the sampler stage state value to set.</param>
        /// <param name="value">A float value to set for the given sampler stage state.</param>
        public void SetRenderState(RenderStates state, float value)
        {
            _device.SetRenderState(state, value);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Called inmediately the device is created
        /// </summary>
        internal void AfterCreate(Device device)
        {
            _device = device;

            // Dont use lighting, so disable it
            _device.RenderState.Lighting = false;

            // Activate the z-buffer
            _device.RenderState.ZBufferEnable = true;
        }

        /// <summary>
        /// Reports the current cooperative-level status of the Microsoft Direct3D device for a windowed or full-screen application.
        /// </summary>
        /// <param name="result">[in, out] Current cooperative-level status of the device for a windowed or full-screen application, reported using a ResultCode value. A Success result indicates that the device is operational and that the calling application can continue. A DeviceLost result indicates that the device is lost but cannot be reset at this time; therefore, rendering is not possible. A DeviceNotReset result indicates that the device is lost but can be reset at this time</param>
        /// <returns>Current cooperative-level status of the device for a windowed or full-screen application. A return value of true indicates that the device is operational and that the calling application can continue; a value of false indicates that the device is lost or needs to be reset</returns>
        public bool CheckCooperativeLevel(out int result)
        {
            return _device.CheckCooperativeLevel(out result);
        }

        /// <summary>
        /// Retrieves a render target surface
        /// </summary>
        /// <param name="renderTargetIndex">Index of the render target</param>
        /// <returns>A Surface that represents the returned render target surface for the current device</returns>
        public Surface GetRenderTarget(int renderTargetIndex)
        {
            return _device.GetRenderTarget(renderTargetIndex);
        }

        /// <summary>
        /// Retrieves a back buffer from a device's swap chain
        /// </summary>
        /// <param name="swapChain">Unsigned integer that specifies the swap chain.</param>
        /// <param name="backBuffer">Index of the back buffer object to return</param>
        /// <param name="backBufferType">Stereo view is not supported in Microsoft DirectX 9.0, so the only valid value for this parameter is Mono.</param>
        /// <returns>A Surface that represents the returned back buffer surface.</returns>
        public Surface GetBackBuffer(int swapChain, int backBuffer, BackBufferType backBufferType)
        {
            return _device.GetBackBuffer(swapChain, backBuffer, backBufferType);
        }

        /// <summary>
        /// Begins a scene 
        /// </summary>
        public void BeginScene()
        {
            _device.BeginScene();
        }

        /// <summary>
        /// Ends a scene 
        /// </summary>
        public void EndScene()
        {
            _device.EndScene();
        }

        /// <summary>
        /// Creates a new VertexDeclaration
        /// </summary>
        /// <param name="elements">A valid array of VertexElements</param>
        /// <returns>A new VertexDeclaration instance</returns>
        public VertexDeclaration CreateVertexDeclaration(VertexElement[] elements)
        {
            return new VertexDeclaration(_device, elements);
        }

        /// <summary>
        /// Creates a new Sprite instance
        /// </summary>
        /// <returns></returns>
        public Sprite CreateSprite()
        {
            return new Sprite(_device);
        }

        /// <summary>
        /// Binds a vertex buffer to a device data stream
        /// </summary>
        /// <param name="streamNumber">Data stream in the range of 0 to the maximum number of streams -1</param>
        /// <param name="streamData">Pointer to a VertexBuffer class that represents the vertex buffer to bind to the specified data stream</param>
        /// <param name="offsetInBytes">Offset from the beginning of the stream to the beginning of the vertex data, in bytes. To determine whether the device supports stream offsets, see DeviceCaps.SupportsStreamOffset.</param>
        /// <param name="stride">Stride of the component, in bytes</param>
        public void SetStreamSource(int streamNumber, VertexBuffer streamData, int offsetInBytes, int stride)
        {
            _device.SetStreamSource(streamNumber, streamData, offsetInBytes,stride);
        }

        /// <summary>
        /// Clears the viewport or a set of rectangles in the viewport to a specified RGBA color, clears the depth buffer, and erases the stencil buffer.
        /// </summary>
        /// <param name="flags">Flags that indicate which surfaces to clear. This parameter can be any combination of the following flags, but at least one flag must be used.</param>
        /// <param name="color">A 32-bit ARGB color value to which the render target surface is cleared</param>
        /// <param name="zdepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance</param>
        /// <param name="stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
        /// <param name="regions">Array of Rectangle  structures that describe the rectangles to clear. To clear the entire surface, set a rectangle to the dimensions of the rendering target. Each rectangle uses screen coordinates that correspond to points on the render target surface. Coordinates are clipped to the bounds of the viewport rectangle</param>
        public void Clear(ClearFlags flags, int color, float zdepth, int stencil, Rectangle[] regions)
        {
            _device.Clear(flags, color, zdepth, stencil, regions);
        }

        /// <summary>
        /// Clears the viewport or a set of rectangles in the viewport to a specified RGBA color, clears the depth buffer, and erases the stencil buffer.
        /// </summary>
        /// <param name="flags">Flags that indicate which surfaces to clear. This parameter can be any combination of the following flags, but at least one flag must be used.</param>
        /// <param name="color">A 32-bit ARGB color value to which the render target surface is cleared</param>
        /// <param name="zdepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance</param>
        /// <param name="stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
        public void Clear(ClearFlags flags, int color, float zdepth, int stencil)
        {
            _device.Clear(flags, color, zdepth, stencil);
        }

        /// <summary>
        /// Sets a new color buffer for a device
        /// </summary>
        /// <param name="number">Render target index, starting at 0</param>
        /// <param name="surface">A valid surface or null to disable it</param>
        public void SetRenderTarget(int number, Surface surface)
        {
            _device.SetRenderTarget(number, surface);
        }

        /// <summary>
        /// Assigns a texture to a device stage.
        /// </summary>
        /// <param name="stage">Index value for the device stage</param>
        /// <param name="texture">A BaseTexture object that represents the texture being set.</param>
        public void SetTexture(int stage, BaseTexture texture)
        {
            _device.SetTexture(stage, texture);
        }

        /// <summary>
        /// Renders data specified by a user memory pointer as a sequence of geometric primitives of the specified type.
        /// </summary>
        /// <param name="startVertex">Index of the first vertex to load. Beginning at param_Int32_startVertex, the correct number of vertices is read out of the vertex buffer</param>
        /// <param name="primitiveCount">Number of primitives to render. To determine the maximum number of primitives allowed, check the Caps.MaxPrimitiveCount member</param>
        public void DrawPrimitives(PrimitiveType primitiveType, int startVertex, int primitiveCount)
        {
            _device.DrawPrimitives(primitiveType, startVertex, primitiveCount);
        }

        /// <summary>
        /// Renders data specified by a user memory pointer as a sequence of geometric primitives of the specified type.
        /// </summary>
        /// <param name="primitiveType">Member of the PrimitiveType enumerated type that describes the type of primitive to render</param>
        /// <param name="primitiveCount">Number of primitives to render. To determine the maximum number of primitives allowed, check the Caps.MaxPrimitiveCount member</param>
        /// <param name="vertexStreamZeroData">User memory vertex data to use for vertex stream 0</param>
        public void DrawUserPrimitives(PrimitiveType primitiveType, int primitiveCount, object vertexStreamZeroData)
        {
            _device.DrawUserPrimitives(primitiveType, primitiveCount, vertexStreamZeroData);
        }

        /// <summary>
        /// Renders the specified geometric primitive, based on indexing into an array of vertices.
        /// </summary>
        /// <param name="primitiveType">Member of the PrimitiveType enumerated type that describes the type of primitive to render. The PointList constant is not supported with this method</param>
        /// <param name="baseVertex">Offset from the start of the index buffer to the first vertex index</param>
        /// <param name="minVertexIndex">Minimum vertex index for vertices used during the call</param>
        /// <param name="numVertices">Number of vertices used during the call, starting from param_Int32_baseVertex + param_Int32_minVertexIndex</param>
        /// <param name="startIndex">Location in the index array at which to start reading vertices</param>
        /// <param name="primitiveCount">Number of primitives to render. The number of vertices used is a function of the param_Int32_primCount and param_PrimitiveType_primitiveType. To determine the maximum number of primitives allowed, check the MaxPrimitiveCount member of the Caps structure</param>
        public void DrawIndexedPrimitives(PrimitiveType primitiveType, int baseVertex, int minVertexIndex, int numVertices, int startIndex, int primitiveCount)
        {
            _device.DrawIndexedPrimitives(primitiveType, baseVertex, minVertexIndex, numVertices, startIndex, primitiveCount);
        }

        /// <summary>
        /// Renders the specified geometric primitive with data specified by a user memory pointer.
        /// </summary>
        /// <param name="primitiveType">Member of the PrimitiveType enumerated type that describes the type of primitive to render. The PointList constant is not supported with this method</param>
        /// <param name="minVertexIndex">Minimum vertex index, relative to 0 (the start of param_Object_vertexStreamZeroData), for vertices used during the call.</param>
        /// <param name="numVertexIndices">Number of vertices used during the call, starting from param_Int32_minVertexIndex.</param>
        /// <param name="primitiveCount">Number of primitives to render. The number of vertices used is a function of the param_Int32_primCount and param_PrimitiveType_primitiveType. To determine the maximum number of primitives allowed, check the MaxPrimitiveCount member of the Caps structure</param>
        /// <param name="indexData">User memory pointer to the index data.</param>
        /// <param name="sixteenBitIndices">Set to true to indicate 16-bit indices. Set to false if to indicate 32-bit indices</param>
        /// <param name="vertexStreamZeroData">User memory vertex data to use for vertex stream 0</param>
        public void DrawIndexedUserPrimitives(PrimitiveType primitiveType, int minVertexIndex, int numVertexIndices, int primitiveCount, object indexData, bool sixteenBitIndices, object vertexStreamZeroData)
        {
            _device.DrawIndexedUserPrimitives(primitiveType, minVertexIndex, numVertexIndices, primitiveCount, indexData, sixteenBitIndices, vertexStreamZeroData);
        }

        #endregion

    }
}
