#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Here we have then <see cref="EffectHandle"/> references used
    /// for the camera on the shaders.
    /// </summary>
    public class JEffectCameraHandler
	{

		#region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectHandles _parent;

        /// <summary>
		/// The effect Vector4 Position
		/// </summary>
		EffectHandle _position;

		#endregion

        #region Properties

        /// <summary>
        /// This is the effect Vector4 FogValues
        /// </summary>
        public EffectHandle Position
        {
            get { return _position; }
        }

        /// <summary>
        /// Gets the effect DX resource
        /// </summary>
        Effect DXResource
        {
            get { return _parent.DXResource; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid JEffectHandles object</param>
        internal JEffectCameraHandler(JEffectHandles parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the effect handlers
        /// </summary>
        internal void GetHandles()
        {
            _position = DXResource.GetParameter("camera", "position");

        }

		/// <summary>
		/// Apply the values to the effect
		/// </summary>
		/// <param name="position">Camera position</param>
		internal void ApplyEffect(Vector3 position)
		{
			if (_position != null)
			{
				_parent._parent.SetVector4(_position, position);
			}

		}

		#endregion
    }
}
