#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Here we have then <see cref="EffectHandle"/> references used
    /// for the Depth Of Field on the shaders.
    /// </summary>
    public class JEffectDOFHandler
	{
		#region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectHandles _parent;

        /// <summary>
		/// The Planes Vector4 values for the effect
		/// </summary>
		EffectHandle _planes;

		/// <summary>
		/// The MaxBlur value for the effect
		/// </summary>
		EffectHandle _maxBlur;

		#endregion

        #region Properties

        /// <summary>
        /// The Planes Vector4 values for the effect
        /// </summary>
        public EffectHandle Planes
        {
            get { return _planes; }
        }

        /// <summary>
        /// The MaxBlur value for the effect
        /// </summary>
        public EffectHandle MaxBlur
        {
            get { return _maxBlur; }
        }

        /// <summary>
        /// Gets the effect DX resource
        /// </summary>
        Effect DXResource
        {
            get { return _parent.DXResource; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid JEffectHandles object</param>
        internal JEffectDOFHandler(JEffectHandles parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the effect handlers
        /// </summary>
        internal void GetHandles()
        {
            _planes = DXResource.GetParameter(null, "DOF_planes");
            _maxBlur = DXResource.GetParameter(null, "DOF_maxBlur");
        }

		/// <summary>
		/// Apply the values to the effect
		/// </summary>
		/// <param name="dof_planes">The Depth of Field Planes</param>
		/// <param name="maxBlur">The MaxBlur value</param>
		internal void ApplyEffect(Vector4 dof_planes,float maxBlur)
		{
			if (_planes != null)
			{
				DXResource.SetValue(_planes, dof_planes);
				DXResource.SetValue(_maxBlur, maxBlur);
			}

		}

        #endregion
    }
}
