#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
	public class JEffectEnvironmentHandler
	{
		#region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectHandles _parent;

        /// <summary>
        /// The effect Vector4 Ambient_TimeElapsed
		/// </summary>
		EffectHandle _ambient_TimeElapsed;

		/// <summary>
        /// The effect Vector4 Screen_Shadows
		/// </summary>
		EffectHandle _screen_Shadows;

		#endregion

        #region Properties

        /// <summary>
        /// The effect Vector4 Ambient_TimeElapsed
        /// </summary>
        public EffectHandle Ambient_TimeElapsed
        {
            get { return _ambient_TimeElapsed; }
        }

        /// <summary>
        /// The effect Vector4 Screen_Shadows
        /// </summary>
        public EffectHandle Screen_Shadows
        {
            get { return _screen_Shadows; }
        }

        /// <summary>
        /// Gets the effect DX resource
        /// </summary>
        Effect DXResource
        {
            get { return _parent.DXResource; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid JEffectHandles object</param>
        internal JEffectEnvironmentHandler(JEffectHandles parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the effect handlers
        /// </summary>
        internal void GetHandles()
        {
            _ambient_TimeElapsed = DXResource.GetParameter(null, "Env_Ambient_TimeElapsed");
            _screen_Shadows = DXResource.GetParameter(null, "Env_Screen_Shadows");

        }

		/// <summary>
		/// Apply the smbient, time elapsed and screen shadows to the effect
		/// </summary>
		/// <param name="color">The ambient color</param>
		/// <param name="timeElapsed">The time elapsed</param>
		internal void ApplyEffect(JColor color, float timeElapsed)
		{
			if (_ambient_TimeElapsed != null)
			{
				_parent._parent.SetVector4(_ambient_TimeElapsed,color.Red, color.Green, color.Blue, timeElapsed);
			}

			if (_screen_Shadows != null)
			{
				if (Jad.Video.RenderTargets.Active != null)
				{
					if (Jad.Video.Views.ActiveView.RenderTargets.ShadowMap != null)
						_parent._parent.SetVector4(_screen_Shadows, Jad.Video.RenderTargets.Active.Width, Jad.Video.RenderTargets.Active.Height, Jad.Video.Views.ActiveView.RenderTargets.ShadowMap.Width, Jad.Video.Views.ActiveView.RenderTargets.ShadowMap.Height);

					else
						_parent._parent.SetVector4(_screen_Shadows, Jad.Video.RenderTargets.Active.Width, Jad.Video.RenderTargets.Active.Height);
				}
			}
		}

		#endregion
    }
}
