#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Here we have then <see cref="EffectHandle"/> references used
    /// for the Lights on the shaders.
    /// </summary>
	public class JEffectLightHandler
	{
		#region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectHandles _parent;

		/// <summary>
        /// This is the effect Vector4 FarAttenStart_End_hotSpot_FallOff
		/// </summary>
		EffectHandle _farAttenStart_End_hotSpot_FallOff;

		/// <summary>
        /// This is the effect Vector4 AttenOpenGL
		/// </summary>
		EffectHandle _attenOpenGL;

		/// <summary>
        /// This is the effect light Position
		/// </summary>
		EffectHandle _position;

		/// <summary>
		/// The effect light Color
		/// </summary>
		EffectHandle _color;

		/// <summary>
        /// The effect light Direction
		/// </summary>
		EffectHandle _direction;

		/// <summary>
        /// The effect light ProjectorTransformation
		/// </summary>
		EffectHandle _projectorTransformation;

		/// <summary>
        /// The effect light ShadowMapTransformation
		/// </summary>
		EffectHandle _shadowMapTransformation;

		#endregion

        #region Properties

        /// <summary>
        /// This is the effect Vector4 FarAttenStart_End_hotSpot_FallOff
        /// </summary>
        public EffectHandle FarAttenStart_End_hotSpot_FallOff
        {
            get { return _farAttenStart_End_hotSpot_FallOff; }
        }

        /// <summary>
        /// This is the effect Vector4 AttenOpenGL
        /// </summary>
        public EffectHandle AttenOpenGL
        {
            get { return _attenOpenGL; }
        }

        /// <summary>
        /// This is the effect light Position
        /// </summary>
        public EffectHandle Position
        {
            get { return _position; }
        }

        /// <summary>
        /// The effect light Color
        /// </summary>
        public EffectHandle Color
        {
            get { return _color; }
        }

        /// <summary>
        /// The effect light Direction
        /// </summary>
        public EffectHandle Direction
        {
            get { return _direction; }
        }

        /// <summary>
        /// The effect light ProjectorTransformation
        /// </summary>
        public EffectHandle ProjectorTransformation
        {
            get { return _projectorTransformation; }
        }

        /// <summary>
        /// The effect light ShadowMapTransformation
        /// </summary>
        public EffectHandle ShadowMapTransformation
        {
            get { return _shadowMapTransformation; }
        }

        /// <summary>
        /// Gets the effect DX resource
        /// </summary>
        Effect DXResource
        {
            get { return _parent.DXResource; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid JEffectHandles object</param>
        internal JEffectLightHandler(JEffectHandles parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the effect handlers
        /// </summary>
        internal void GetHandles()
        {
            _farAttenStart_End_hotSpot_FallOff = DXResource.GetParameter(null, "Light_FarAttenStart_End_hotSpot_FallOff");
            _attenOpenGL = DXResource.GetParameter(null, "Light_AttenOpenGL");
            _position = DXResource.GetParameter(null, "Light_Position");
            _color = DXResource.GetParameter(null, "Light_Color");
            _direction = DXResource.GetParameter(null, "Light_Direction");
            _projectorTransformation = DXResource.GetParameter(null, "Light_ProjectorTransformation");
            _shadowMapTransformation = DXResource.GetParameter(null, "Light_ShadowMapTransformation");

        }

        #endregion

    }
}
