#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Here we have then <see cref="EffectHandle"/> references used
    /// for the light shafts on the shaders.
    /// </summary>
	public class JEffectLightShaftHandler
	{
		#region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectHandles _parent;

        /// <summary>
		/// The effect float power
		/// </summary>
		EffectHandle _power;

        /// <summary>
		/// The effect float Intensity
		/// </summary>
		EffectHandle _intensity;

		#endregion

        #region Properties

        /// <summary>
        /// This is the effect float power
        /// </summary>
        public EffectHandle Power
        {
            get { return _power; }
        }

        /// <summary>
        /// This is the effect float Intensity
        /// </summary>
        public EffectHandle Intensity
        {
            get { return _intensity; }
        }

        /// <summary>
        /// Gets the effect DX resource
        /// </summary>
        Effect DXResource
        {
            get { return _parent.DXResource; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid JEffectHandles object</param>
        internal JEffectLightShaftHandler(JEffectHandles parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the effect handlers
        /// </summary>
        internal void GetHandles()
        {
            _power = DXResource.GetParameter(null, "power");
            _intensity = DXResource.GetParameter(null, "intensity");
        }

		/// <summary>
		/// Apply the values to the effect
		/// </summary>
		/// <param name="shaft">A valid object</param>
		internal void ApplyEffect(JLightShaft shaft)
		{
			if (_power != null)
			{
				DXResource.SetValue(_power, shaft.Power);
				DXResource.SetValue(_intensity, shaft.Intensity);
			}
		}

		#endregion
    }
}
