#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// Here we have then <see cref="EffectHandle"/> references used
    /// for the material on the shaders.
    /// </summary>
    public class JEffectMaterialHandler
	{
		#region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectHandles _parent;

        /// <summary>
		/// The effect Vector4 Specular
		/// </summary>
		EffectHandle _specular;

		/// <summary>
        /// The effect Vector4 SpecularPower_Bumpiness_Reflectivity_Transmittance
		/// </summary>
		EffectHandle _specularPower_Bumpiness_Reflectivity_Transmittance;

		/// <summary>
        /// The effect Vector4 Diffuse
		/// </summary>
		EffectHandle _diffuse;

		/// <summary>
        /// The effect Vector4 Emissive
		/// </summary>
		EffectHandle _emissive;

		/// <summary>
        /// The effect Vector4 IndexOfRefraction
		/// </summary>
		EffectHandle _indexOfRefraction;

		/// <summary>
        /// The effect Vector4 ParallaxAmount
		/// </summary>
		EffectHandle _parallaxAmount;

		#endregion

        #region Properties

        /// <summary>
        /// The effect Vector4 Specular
        /// </summary>
        public EffectHandle Specular
        {
            get { return _specular; }
        }

        /// <summary>
        /// The effect Vector4 SpecularPower_Bumpiness_Reflectivity_Transmittance
        /// </summary>
        public EffectHandle SpecularPower_Bumpiness_Reflectivity_Transmittance
        {
            get { return _specularPower_Bumpiness_Reflectivity_Transmittance; }
        }

        /// <summary>
        /// The effect Vector4 Diffuse
        /// </summary>
        public EffectHandle Diffuse
        {
            get { return _diffuse; }
        }

        /// <summary>
        /// The effect Vector4 Emissive
        /// </summary>
        public EffectHandle Emissive
        {
            get { return _emissive; }
        }

        /// <summary>
        /// The effect Vector4 IndexOfRefraction
        /// </summary>
        public EffectHandle IndexOfRefraction
        {
            get { return _indexOfRefraction; }
        }

        /// <summary>
        /// The effect Vector4 ParallaxAmount
        /// </summary>
        public EffectHandle ParallaxAmount
        {
            get { return _parallaxAmount; }
        }

        /// <summary>
        /// Gets the effect DX resource
        /// </summary>
        Effect DXResource
        {
            get { return _parent.DXResource; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid JEffectHandles object</param>
        internal JEffectMaterialHandler(JEffectHandles parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Get the effect handlers
        /// </summary>
        internal void GetHandles()
        {
            _specular = DXResource.GetParameter(null, "Material_Specular");
            _specularPower_Bumpiness_Reflectivity_Transmittance = DXResource.GetParameter(null, "Material_SpecularPower_Bumpiness_Reflectivity_Transmittance");
            _diffuse = DXResource.GetParameter(null, "Material_Diffuse");
            _emissive = DXResource.GetParameter(null, "Material_Emissive");
            _indexOfRefraction = DXResource.GetParameter(null, "Material_IOR");
            _parallaxAmount = DXResource.GetParameter(null, "Material_ParallaxAmount");

        }

        #endregion
    }
}
