using System;
using System.Collections.Generic;
using System.Text;

namespace JadEngine.Video
{
    /// <summary>
    /// In house effects loaded when the engine starts related to PostProcess
    /// </summary>
    public class JEffectInHousePostProcess
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JEffectsInHouse _parent;

        /// <summary>
        /// The Radial Blur effect
        /// </summary>
        JEffect _radialBlur;

        /// <summary>
        /// The Depth of Field effect
        /// </summary>
        JEffect _depthOfField;

        /// <summary>
        /// The Gaussian Blur effect
        /// </summary>
        JEffect _gaussianBlur;

        /// <summary>
        /// Edge detection
        /// </summary>
        JEffect _sobel;

        /// <summary>
        /// HDR filter
        /// </summary>
        JEffect _hDR;

        /// <summary>
        /// Used in HDR
        /// </summary>
        JEffect _toneMapping;

        /// <summary>
        /// The basic Blur effect
        /// </summary>
        JEffect _blur;

        /// <summary>
        /// A filter effect used to get only high color values
        /// </summary>
        JEffect _highPass;

        /// <summary>
        /// The Motion Blur effect
        /// </summary>
        JEffect _motionBlur;

        #endregion

        #region Properties

        /// <summary>
        /// The Radial Blur effect
        /// </summary>
        public JEffect RadialBlur
        {
            get { return _radialBlur; }
        }

        /// <summary>
        /// The Depth of Field effect
        /// </summary>
        public JEffect DepthOfField
        {
            get { return _depthOfField; }
        }

        /// <summary>
        /// The Gaussian Blur effect
        /// </summary>
        public JEffect GaussianBlur
        {
            get { return _gaussianBlur; }
        }


        /// <summary>
        /// Edge detection
        /// </summary>
        public JEffect Sobel
        {
            get { return _sobel; }
        }

        /// <summary>
        /// HDR filter
        /// </summary>
        public JEffect HDR
        {
            get { return _hDR; }
        }

        /// <summary>
        /// Used in HDR
        /// </summary>
        public JEffect ToneMapping
        {
            get { return _toneMapping; }
        }

        /// <summary>
        /// The basic Blur effect
        /// </summary>
        public JEffect Blur
        {
            get { return _blur; }
        }

        /// <summary>
        /// A filter effect used to get only high color values
        /// </summary>
        public JEffect HighPass
        {
            get { return _highPass; }
        }

        /// <summary>
        /// The Motion Blur effect
        /// </summary>
        public JEffect MotionBlur
        {
            get { return _motionBlur; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="parent">A valid object</param>
        internal JEffectInHousePostProcess(JEffectsInHouse parent)
        {
            _parent = parent;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Loads the InHouse effects
        /// </summary>
        internal void Load()
        {
            _radialBlur = _parent.CreateInHouse("RadialBlur.fx");
			_depthOfField = _parent.CreateInHouse("DepthOfField.fx");
			_gaussianBlur = _parent.CreateInHouse("GaussianBlur.fx");
			_sobel = _parent.CreateInHouse("Sobel.fx");
			_hDR = _parent.CreateInHouse("HDR.fx");
			_toneMapping = _parent.CreateInHouse("ToneMapping.fx");
			_blur = _parent.CreateInHouse("Blur.fx");
			_highPass = _parent.CreateInHouse("HighPass.fx");
			_motionBlur = _parent.CreateInHouse("MotionBlur.fx");
        }

        #endregion
    }
}
