#region Using directives

using System;
using System.Drawing;
using System.Windows.Forms;

#endregion

/*
 * 04/02/08
 *	-Initial cleanup
 */

namespace JadEngine.Video
{
	/// <summary>
	/// The standard Jad rendering Form. It's a standard Windows Form, where the rendering fills the whole client area.
	/// </summary>
	public class JBaseForm : Form
	{
		#region Constructors		
		
		/// <summary>
		/// Initializes a new instance of the <see cref="JBaseForm"/> class.
		/// </summary>
		public JBaseForm()
		{
			// Set the size of the client area to the backbuffer size.
			ClientSize = new Size(
				Jad.Video.Adapter.Settings.PresentParams.BackBufferWidth,
				Jad.Video.Adapter.Settings.PresentParams.BackBufferHeight);

			Text = "Jad 3D Engine";

			StartPosition = FormStartPosition.CenterScreen;
		}

		#endregion

		#region Methods


		/// <summary>
		/// Overrides Form.OnResize() to resize the view accordingly and reset the adapter. 
		/// </summary>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			// No resize in full screen mode
			if (!Jad.Video.Adapter.Settings.PresentParams.Windowed)
				return;

			if (!Jad.Active) return;

			// Modify the parameters to indicate the size change
			JView view = Jad.Video.Views[0];
			view.SetNewDimensions(Size.Width, Size.Height);

			// Reset the device. This will call the Lost and Reset events of the device.
			Jad.Video.Adapter.Reset();
		}

		#endregion
	}
}
