#region Using Directives

using System;

#endregion

/*
 * 01/02/08
 *	-Created
 *	-struct JResolutionProperties removed, fields moved up.
 *	-Renamed to JVideoInitialization (was JVideoProperties)
 * 07/04/08
 *	-Extended to also hold the multisample setting
 */

namespace JadEngine.Video
{
	/// <summary>
	/// Encapsulates the video initialization properties.
	/// </summary>
	public struct JVideoInitialization
	{
		#region Fields
		
		private int _width;		
		private int _height;		
		private bool _windowed;		
		private bool _hdr;
		private int _multisample;

		#endregion

		#region Properties

		/// <summary>
		/// Gets the initial width of the client area or the horizontal screen resolution in fullscreen mode.
		/// </summary>
		/// <value>The width in pixels.</value>
		public int Width 
		{ 
			get { return _width; }			
		}

		/// <summary>
		/// Gets the initial height of the client area or the vertical screen resolution in fullscreen mode.
		/// </summary>
		/// <value>The height in pixels.</value>
		public int Height 
		{ 
			get { return _height;}			
		}

		/// <summary>
		/// Gets a value indicating whether the application is supposed to run in a window or in full screen mode.
		/// </summary>
		/// <value><c>true</c> if windowed; otherwise, <c>false</c>.</value>
		public bool Windowed 
		{ 
			get { return _windowed; }			
		}

		/// <summary>
		/// Gets a value indicating whether HDR rendering should be used.
		/// </summary>
		/// <value><c>true</c> if HDR; otherwise, <c>false</c>.</value>
		public bool HDR 
		{ 
			get { return _hdr; }			
		}

		public int MultiSample
		{
			get { return _multisample; }
		}

		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="JVideoInitialization"/> struct.
		/// </summary>
		/// <param name="width">The initial width of the client area or the horizontal screen resolution in fullscreen mode.</param>
		/// <param name="height">The initial height of the client area or the vertical screen resolution in fullscreen mode.</param>
		/// <param name="windowed">If set to <c>true</c>, the application is supposed to run in a windowed mode.</param>
		/// <param name="hdr">If set to <c>true</c>, HDR rendering should be used.</param>
		public JVideoInitialization(int width, int height, bool windowed, bool hdr, int multisample)
		{
			_width = width;
			_height = height;
			_windowed = windowed;
			_hdr = hdr;
			_multisample = multisample;
		}

		#endregion
	}
}
