#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics.Geometry;
using JadEngine.Physics;
using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;
using JadEngine.Collections;

#endregion

namespace JadEngine.Video
{
	public static partial class JFaceDelegates
	{
		/// <summary>
		/// Builds the tangent space
		/// </summary>
		/// <param name="face">Face information</param>
		/// <returns>Always returns true</returns>
		internal static bool ComputeTangentCB(JForEachFaceParams face)
		{
			Vector3 v0, v1, v2, normal = Vector3.Empty;
			Vector2 uv0, uv1, uv2;
			float area, f, sign;

			bool[] used = (bool[])face.User;

			// Indice of the three vertex that make the face

			int i0 = face.Vertex0Index;
			int i1 = face.Vertex1Index;
			int i2 = face.Vertex2Index;

			// Vertices of the face
			v0 = face.v0;
			v1 = face.v1;
			v2 = face.v2;

			// Coordenadas de textura de la cara

			JVertexBuffer _vertexBuffer = face.Mesh.VertexBuffer;

			uv0 = _vertexBuffer.TexCoord[0, i0];
			uv1 = _vertexBuffer.TexCoord[0, i1];
			uv2 = _vertexBuffer.TexCoord[0, i2];

			// XYZ & UV derivatives

			float[] d0 = new float[5];
			float[] d1 = new float[5];

			d0[0] = v0.X - v1.X;
			d0[1] = v0.Y - v1.Y;
			d0[2] = v0.Z - v1.Z;
			d0[3] = uv0.X - uv1.X;
			d0[4] = uv0.Y - uv1.Y;

			d1[0] = v0.X - v2.X;
			d1[1] = v0.Y - v2.Y;
			d1[2] = v0.Z - v2.Z;
			d1[3] = uv0.X - uv2.X;
			d1[4] = uv0.Y - uv2.Y;

			// Normal
			bool _normalsComputed = face.Mesh.NormalsComputed;

			if (!_normalsComputed)
			{

				normal.X = d0[1] * d1[2] - d0[2] * d1[1];
				normal.Y = d0[2] * d1[0] - d0[0] * d1[2];
				normal.Z = d0[0] * d1[1] - d0[1] * d1[0];

				f = (float)(1.0 / Math.Sqrt(normal.X * normal.X + normal.Y * normal.Y + normal.Z * normal.Z));

				normal.X *= f;
				normal.Y *= f;
				normal.Z *= f;
			}

			// Area sign

			area = d0[3] * d1[4] - d0[4] * d1[3];
			sign = Math.Sign(area);

			// First tangent

			Vector3 tangent;

			tangent.X = d0[0] * d1[4] - d0[4] * d1[0];
			tangent.Y = d0[1] * d1[4] - d0[4] * d1[1];
			tangent.Z = d0[2] * d1[4] - d0[4] * d1[2];

			f = (float)(1.0 / Math.Sqrt(tangent.X * tangent.X + tangent.Y * tangent.Y + tangent.Z * tangent.Z));
			f *= sign;

			tangent.X *= f;
			tangent.Y *= f;
			tangent.Z *= f;

			if (used[i0])
			{
				if (!_normalsComputed) _vertexBuffer.Normal[i0] += normal;

				_vertexBuffer.Tangent[i0] += tangent;
			}

			else
			{
				if (!_normalsComputed) _vertexBuffer.Normal[i0] = normal;

				_vertexBuffer.Tangent[i0] = tangent;

				used[i0] = true;
			}

			if (used[i1])
			{
				if (!_normalsComputed) _vertexBuffer.Normal[i1] += normal;

				_vertexBuffer.Tangent[i1] += tangent;
			}

			else
			{
				if (!_normalsComputed) _vertexBuffer.Normal[i1] = normal;

				_vertexBuffer.Tangent[i1] = tangent;

				used[i1] = true;
			}

			if (used[i2])
			{
				if (!_normalsComputed) _vertexBuffer.Normal[i2] += normal;

				_vertexBuffer.Tangent[i2] += tangent;
			}

			else
			{
				if (!_normalsComputed) _vertexBuffer.Normal[i2] = normal;

				_vertexBuffer.Tangent[i2] = tangent;

				used[i2] = true;
			}

			return true;
		}

	}
}
