#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics.Geometry;
using JadEngine.Physics;
using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;
using JadEngine.Collections;

#endregion

namespace JadEngine.Video
{
	public static partial class JFaceDelegates
	{
		/// <summary>
		/// Generates a tree collision for Newton
		/// </summary>
		/// <param name="face">Face information</param>
		/// <returns>Always true</returns>
		public static bool GenerateTreeCollisionCB(JForEachFaceParams face)
		{
			//Add this face to the collision tree

			float[] v = new float[9];

			v[0] = face.v0.X;
			v[1] = face.v0.Y;
			v[2] = face.v0.Z;
			v[3] = face.v1.X;
			v[4] = face.v1.Y;
			v[5] = face.v1.Z;
			v[6] = face.v2.X;
			v[7] = face.v2.Y;
			v[8] = face.v2.Z;

			JCollision collision = (JCollision)face.User;

			// Get the subset n of the face

			int numAttID = 0;

			if (face.Mesh.AttributeRange != null)
			{
				for (int i = 0; i < face.Mesh.AttributeRange.Length; i++)
				{
					if (face.NumFace >= face.Mesh.AttributeRange[i].FaceStart)
					{
						numAttID = i;
						break;
					}
				}
			}

			int materialID = face.MeshObject.Materials[numAttID].PhysicsPhysicsMaterial.Material;

			collision.TreeCollisionAddFace(3, v, 12, materialID);

			return true;
		}

	}
}
