#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics.Geometry;
using JadEngine.Physics;
using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;
using JadEngine.Collections;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// Builds mesh primitives
	/// </summary>
	public class JPrimitives
	{
		/// <summary>
		/// TODO
		/// </summary>
		struct JVertexFormat
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public float X;

			/// <summary>
			/// TODO
			/// </summary>
			public float Y;

			/// <summary>
			/// TODO
			/// </summary>
			public float Z;

			/// <summary>
			/// TODO
			/// </summary>
			public float Tu;

			/// <summary>
			/// TODO
			/// </summary>
			public float Tv;

			#endregion
		}

		#region Static Fields

		/// <summary>
		/// Vertices for the method CreateCube
		/// </summary>
		static float[] cubeVertices = new float[] {
			-0.5f, 0, -0.5f,1,1,	// 0. Bottom
			0.5f,0,0.5f,0,0,		// 1
			-0.5f,0,0.5f,1,0,		// 2
			0.5f,0,-0.5f,0,1,		// 3
			-0.5f,1,-0.5f,0,1,		// 4. Top
			0.5f,1,0.5f,1,0,		// 5
			0.5f,1,-0.5f,1,1,		// 6
			-0.5f,1,0.5f,0,0,		// 7
			-0.5f,0,-0.5f,0,1,		// 8. Front
			0.5f,1,-0.5f,1,0,		// 9
			0.5f,0,-0.5f,1,1,		// 10
			-0.5f,1,-0.5f,0,0,		// 11
			0.5f,0,-0.5f,0,1,		// 12. Right
			0.5f,1,0.5f,1,0,		// 13
			0.5f,0,0.5f,1,1,		// 14
			0.5f,1,-0.5f,0,0,		// 15
			0.5f,0,0.5f,0,1,		// 16. Back
			-0.5f,1,0.5f,1,0,		// 17
			-0.5f,0,0.5f,1,1,		// 18
			0.5f,1,0.5f,0,0,		// 19
			-0.5f,0,0.5f,0,1,		// 20. Left
			-0.5f,1,-0.5f,1,0,		// 21
			-0.5f,0,-0.5f,1,1,		// 22
			-0.5f,1,0.5f,0,0 };		// 23

		/// <summary>
		/// Indices definition for the method CreateCube24Vertices
		/// </summary>
		static int[] cubeFaces = new int[] {
			0,1,2,
			1,0,3,
			4,5,6,
			5,4,7,
			8,9,10,
			9,8,11,
			12,13,14,
			13,12,15,
			16,17,18,
			17,16,19,
			20,21,22,
			21,20,23 };

		#endregion

		#region Fields

		/// <summary>
		/// The parent object
		/// </summary>
		JMeshManager _meshes;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="meshes">The parent instance</param>
		internal JPrimitives(JMeshManager meshes)
		{
			_meshes = meshes;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Creates a textured Rectangle
		/// </summary>
		/// <param name="width">Width</param>
		/// <param name="height">Height</param>
		/// <param name="numVertexWidth">Number of vertices per width</param>
		/// <param name="numVertexHeight">Number of vertices per height</param>
		/// <param name="creationType">Creation Plane</param>
		/// <param name="invert_faces">True to invert the faces</param>
		/// <param name="scale_u">Scale for the u texcoord</param>
		/// <param name="scale_v">Scale for the u texcoord</param>
		/// <returns>a valid JMesh object</returns>
		public JMesh CreateRectangle(float width, float height, int numVertexWidth, int numVertexHeight, JRectangleOrientation creationType, bool invert_faces, float scale_u, float scale_v)
		{
			int n, a;
			int x, y;
			float fx, fy, w, h, tu, tv;

			if (numVertexWidth < 2) numVertexWidth = 2;
			if (numVertexHeight < 2) numVertexHeight = 2;

			n = numVertexWidth * numVertexHeight;

			JMesh mesh = _meshes.Create();

			mesh.VertexBuffer = new JVertexBuffer(n);

			mesh.VertexBuffer.Position = new Vector3[n];

			mesh.VertexBuffer.TexCoord = new Vector2[1, n];

			w = (float)numVertexWidth - 1;
			h = (float)numVertexHeight - 1;

			for (y = a = 0; y < numVertexHeight; y++)
			{
				tv = ((float)y / h);
				fy = tv * height;
				for (x = 0; x < numVertexWidth; x++)
				{
					tu = ((float)x / w);
					fx = tu * width;
					if (creationType == JRectangleOrientation.XZ)
					{
						mesh.VertexBuffer.Position[a].X = fx;
						mesh.VertexBuffer.Position[a].Y = 0;
						mesh.VertexBuffer.Position[a].Z = fy;
						// Centramos el rectngulo
						mesh.VertexBuffer.Position[a].X -= width / 2;
						mesh.VertexBuffer.Position[a].Z -= height / 2;
					}
					else if (creationType == JRectangleOrientation.XY)
					{
						mesh.VertexBuffer.Position[a].X = fx;
						mesh.VertexBuffer.Position[a].Y = fy;
						mesh.VertexBuffer.Position[a].Z = 0;

						// Center the rectangle
						mesh.VertexBuffer.Position[a].X -= width / 2;
						mesh.VertexBuffer.Position[a].Y -= height / 2;

					}
					else
					{
						mesh.VertexBuffer.Position[a].X = 0;
						mesh.VertexBuffer.Position[a].Y = fy;
						mesh.VertexBuffer.Position[a].Z = fx;

						// Center the rectangle
						mesh.VertexBuffer.Position[a].Z -= width / 2;
						mesh.VertexBuffer.Position[a].Y -= height / 2;
					}

					mesh.VertexBuffer.TexCoord[0, a].X = tu * scale_u;
					mesh.VertexBuffer.TexCoord[0, a].Y = (1 - tv) * scale_v;

					a++;
				}
			}

			// Now the indices
			int[] indices, i;
			int numberOfIndices = 0, indexToCurrentVertex;

			// Suppose the worst scenario: 6 indices per vertex
			indices = i = new int[n * 6];

			int yv = 0, yJump = numVertexWidth;

			for (y = 0; y < numVertexHeight - 1; y++)
			{
				indexToCurrentVertex = yv;

				yv += numVertexWidth;

				for (x = 0; x < numVertexWidth - 1; x++)
				{

					if (invert_faces)
					{
						i[numberOfIndices + 2] = (indexToCurrentVertex);
						i[numberOfIndices + 1] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 0] = (indexToCurrentVertex + 1);

						i[numberOfIndices + 5] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 4] = (indexToCurrentVertex + yJump + 1);
						i[numberOfIndices + 3] = (indexToCurrentVertex + 1);
					}

					else
					{
						i[numberOfIndices + 0] = indexToCurrentVertex;
						i[numberOfIndices + 1] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 2] = (indexToCurrentVertex + 1);

						i[numberOfIndices + 3] = (indexToCurrentVertex + yJump);
						i[numberOfIndices + 4] = (indexToCurrentVertex + yJump + 1);
						i[numberOfIndices + 5] = (indexToCurrentVertex + 1);
					}

					numberOfIndices += 6;

					indexToCurrentVertex++;
				}
			}

			mesh.IndexBuffer = new JIndexBuffer(false, numberOfIndices);

			Array.Copy(indices, mesh.IndexBuffer.Indices, numberOfIndices);

			return mesh;
		}

		/// <summary>
		/// Builds a Text 3DMesh(without text coords) 
		/// </summary>
		/// <param name="fuente">System font</param>
		/// <param name="text">text to build</param>
		/// <param name="extrude">Z extrusion</param>
		/// <returns>A valid JMesh object</returns>
		public JMesh CreateText3D(System.Drawing.Font fuente, string text, float extrude)
		{
			Mesh temp;
			temp = Mesh.TextFromFont(Jad.Video.Device, fuente, text, 0.001f, extrude);

			// Must be sure that there are texture coordiantes
			// Must do this because the engine uses by default the PosTex1 appeal, and
			// when using shaders < 2.0 if you use a channel that its not defined,
			// it generates an error

			JMesh mesh = _meshes.Create();

			JMeshHelper.BuffersFromMesh(temp, mesh);

			mesh.VertexBuffer.TexCoord = new Vector2[1, mesh.VertexBuffer.NumberOfVertices];

			temp.Dispose();

			return mesh;

		}

		/// <summary>
		/// Creates a textured cylinder
		/// </summary>
		/// <param name="radio1">Inner radio</param>
		/// <param name="radio2">Outer radio</param>
		/// <param name="length">Cylinder's lenght</param>
		/// <param name="numHorizontalSlices">Number of horizontal slices</param>
		/// <param name="numVerticalSlices">Number of vertical slices</param>
		/// <returns>A valid JMesh object</returns>
		public JMesh CreateCylinder(float radio1, float radio2, float length, int numHorizontalSlices, int numVerticalSlices)
		{
			Mesh tmp = Mesh.Cylinder(Jad.Video.Device, radio1, radio2, length, numHorizontalSlices, numVerticalSlices);

			return CalculateTextureCoords(tmp);
		}

		/// <summary>
		/// Creates a sphere with spherical texture coords
		/// </summary>
		/// <param name="radius">Radius</param>
		/// <param name="numHorizontalSlices">Number of horizontal slices</param>
		/// <param name="numVerticalSlices">Number of vertical slices</param>
		/// <returns>A valid JMesh object</returns>
		public JMesh CreateSphere(float radius, int numHorizontalSlices, int numVerticalSlices)
		{
			Mesh tmp = Mesh.Sphere(Jad.Video.Device, radius, numHorizontalSlices, numVerticalSlices);

			return CalculateTextureCoords(tmp);
		}

		/// <summary>
		/// Creates a textured cube with 24 vertices. The faces are distributed this way:
		/// 0,1: Bottom
		/// 2,3: Top
		/// 4,5: Front
		/// 6,7: Right
		/// 8,9: Back
		/// 10,11: Left
		/// </summary>
		/// <param name="size">Width, height, depth</param>
		/// <returns>A valid JMesh object</returns>
		public JMesh CreateCube(Vector3 size)
		{
			return CreateCube(size.X, size.Y, size.Z);
		}

		/// <summary>
		/// Creates a textured cube with 24 vertices
		/// </summary>
		/// <returns>A valid JMesh object</returns>
		public JMesh CreateCube(float width, float height, float depth)
		{
			JMesh mesh = _meshes.Create();

			mesh.VertexBuffer = new JVertexBuffer(24);
			mesh.VertexBuffer.Position = new Vector3[24];
			mesh.VertexBuffer.TexCoord = new Vector2[1, 24];

			for (int i = 0, n = 0; i < cubeVertices.Length; i += 5, n++)
			{
				mesh.VertexBuffer.Position[n].X = cubeVertices[i] * width;
				mesh.VertexBuffer.Position[n].Y = cubeVertices[i + 1] * height;
				mesh.VertexBuffer.Position[n].Z = cubeVertices[i + 2] * depth;
				mesh.VertexBuffer.TexCoord[0, n].X = cubeVertices[i + 3];
				mesh.VertexBuffer.TexCoord[0, n].Y = cubeVertices[i + 4];
			}

			mesh.IndexBuffer = new JIndexBuffer(false, cubeFaces.Length);

			cubeFaces.CopyTo(mesh.IndexBuffer.Indices, 0);

			return mesh;
		}

		/// <summary>
		/// Compute the texture coord for the cylinder and sphere objects
		/// </summary>
		/// <param name="tmp">A valid Mesh object</param>
		private JMesh CalculateTextureCoords(Mesh tmp)
		{
			Mesh pTempMesh;

			pTempMesh = tmp.Clone(tmp.Options.Value, VertexFormats.Position | VertexFormats.Normal | VertexFormats.Texture1, Jad.Video.Device);

			pTempMesh.ComputeNormals();

			VertexBuffer v = pTempMesh.VertexBuffer;
			int n = pTempMesh.NumberVertices;

			CustomVertex.PositionNormalTextured[] verts = (CustomVertex.PositionNormalTextured[])v.Lock(0, typeof(CustomVertex.PositionNormalTextured), LockFlags.None, n);

			// For Positional Spherical Mapping
			// Calculate center
			Vector3 vCent = new Vector3();

			using (VertexBuffer vb = tmp.VertexBuffer)
			{
				GraphicsStream vertexData = vb.Lock(0, 0, LockFlags.None);
				Geometry.ComputeBoundingSphere(vertexData,
					tmp.NumberVertices,
					tmp.VertexFormat,
					out vCent);
				vb.Unlock();
			}

			for (int i = 0; i < n; i++)
			{

				// Both Spherical Mapping techniques from
				// http://www.mvps.org/directx/articles/spheremap.htm

				/*
					// Spherical Mapping with Normals - start
					verts[i].Tu = (float)(verts[i].Nx / Math.PI + 0.5f);
					verts[i].Tv = (float)(verts[i].Ny / Math.PI + 0.5f);
					// Spherical Mapping with Normals - end
				*/

				//*
				// Positional Spherical Mapping
				// Calculate normalized offset from center
				Vector3 vert;
				vert = verts[i].Position - vCent;
				vert = Vector3.Normalize(vert);

				// calculate texture coordinates
				verts[i].Tu = (float)(Math.Asin(vert.X) / Math.PI + 0.5f);
				verts[i].Tv = (float)(Math.Asin(vert.Y) / Math.PI + 0.5f);
			}

			v.Unlock();
			v.Dispose();

			tmp.Dispose();

			JMesh mesh = _meshes.Create();

			// Transform the DX mesh to Vertex & Index

			JMeshHelper.BuffersFromMesh(pTempMesh, mesh);

			pTempMesh.Dispose();

			return mesh;
		}

		/// <summary>
		/// Creates a polygon with any number of sides
		/// </summary>
		/// <param name="verts">Vertices that form the polygon. Must form a convex hull</param>
		/// <param name="reverse">TODO</param>
		/// <param name="u_tile">TODO</param>
		/// <param name="v_tile">TODO</param>
		public JMesh CreatePolygon(List<Vector3> verts, bool reverse, float u_tile, float v_tile)
		{
			short numIndices = (short)((verts.Count - 2) * 3);
			short[] indices = new short[numIndices];

			JVertexFormat[] vertexFormat = new JVertexFormat[verts.Count];

			// Fill the vertex coordinates
			for (int v = 0; v < verts.Count; v++)
			{
				int index = reverse ? verts.Count - v - 1 : v;

				vertexFormat[v].X = verts[index].X;
				vertexFormat[v].Y = verts[index].Y;
				vertexFormat[v].Z = verts[index].Z;
			}

			// Calculate the texture coordinates

			// Calculate the maximum and minimim of the polygon
			Vector3 min = new Vector3(float.MaxValue, float.MaxValue, float.MaxValue);
			Vector3 max = new Vector3(float.MinValue, float.MinValue, float.MinValue);
			foreach (Vector3 v in verts)
			{
				// Min
				if (v.X < min.X)
					min.X = v.X;
				if (v.Y < min.Y)
					min.Y = v.Y;
				if (v.Z < min.Z)
					min.Z = v.Z;

				// Max
				if (v.X > max.X)
					max.X = v.X;
				if (v.Y > max.Y)
					max.Y = v.Y;
				if (v.Z > max.Z)
					max.Z = v.Z;
			}

			// Calculate the plane where the poligon is
			Plane plane = Plane.FromPoints(verts[0], verts[1], verts[2]);

			// Depending on the plane, calculate the texture coordinates
			// This is the same process as the one used to calculate the texture coordinates on a lightmap
			for (int v = 0; v < vertexFormat.Length; v++)
			{
				// ZY plane
				if (Math.Abs(plane.A) > Math.Abs(plane.B) && Math.Abs(plane.A) > Math.Abs(plane.C))
				{
					float width = Math.Abs(max.Z - min.Z);
					float height = Math.Abs(max.Y - min.Y);

					vertexFormat[v].Tu = (vertexFormat[v].Z - min.Z) / width;
					vertexFormat[v].Tv = 1.0f - ((vertexFormat[v].Y - min.Y) / height);
				}

				// XZ plane
				else if (Math.Abs(plane.B) > Math.Abs(plane.A) && Math.Abs(plane.B) > Math.Abs(plane.C))
				{
					float width = Math.Abs(max.X - min.X);
					float height = Math.Abs(max.Z - min.Z);

					vertexFormat[v].Tu = (vertexFormat[v].X - min.X) / width;
					vertexFormat[v].Tv = 1.0f - ((vertexFormat[v].Z - min.Z) / height);
				}

				// XY plane
				else if (Math.Abs(plane.C) > Math.Abs(plane.A) && Math.Abs(plane.C) > Math.Abs(plane.B))
				{
					float width = Math.Abs(max.X - min.X);
					float height = Math.Abs(max.Y - min.Y);

					vertexFormat[v].Tu = (vertexFormat[v].X - min.X) / width;
					vertexFormat[v].Tv = 1.0f - ((vertexFormat[v].Y - min.Y) / height);
				}

				vertexFormat[v].Tu *= u_tile;
				vertexFormat[v].Tv *= v_tile;
			}

			// Calculate the indices
			short v0 = 0, v1 = 1, v2 = 2;
			for (short i = 0; i < numIndices / 3; i++)
			{
				if (i > 0)
					v1 = v2++;

				indices[i * 3 + 0] = v0;
				indices[i * 3 + 1] = v1;
				indices[i * 3 + 2] = v2;
			}

			// Create the mesh and fill it
			JMesh mesh = _meshes.Create();

			Mesh temp = new Mesh(numIndices / 3, verts.Count, MeshFlags.Managed, CustomVertex.PositionTextured.Format, Jad.Video.Device);
			temp.SetVertexBufferData(vertexFormat, LockFlags.None);
			temp.SetIndexBufferData(indices, LockFlags.None);

			JMeshHelper.BuffersFromMesh(temp, mesh);

			temp.Dispose();

			return mesh;
		}

		#endregion
	}
}
