#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Mathematics.Geometry;
using JadEngine.Scene;
using JadEngine.Core;
using JadEngine.Tools;

#endregion

/*
 * 30/03/08
 *	-Renamed from JShadowVolumes to JShadowVolumeManager
 */

namespace JadEngine.Video
{
	/// <summary>
	/// Mantains the cache of shadow volumes generated
	/// </summary>
	public class JShadowVolumeManager
	{
		#region Constants

		/// <summary>
		/// TODO
		/// </summary>
		public const int MaxVertices = 65535 * 2;

		#endregion

		#region Structs

		/// <summary>
		/// TODO
		/// </summary>
		struct JPositionSV
		{
			#region Fields

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 ObjectPosition;

			/// <summary>
			/// TODO
			/// </summary>
			public Vector3 LightPosition;

			#endregion
		}

		#endregion

		#region Fields

		/// <summary>
		/// Dictionary to find if the mesh position and light position exists on the cache
		/// </summary>
		Dictionary<JPositionSV, Vector3[]> _positions = new Dictionary<JPositionSV, Vector3[]>();

		/// <summary>
		/// Vertices that contain the final information to render. This array is always used for all Build Volumes
		/// </summary>
		public Vector3[] FinalVertex = new Vector3[MaxVertices];

		/// <summary>
		/// Tells the engine if we must use the shadow volume cache
		/// </summary>
		public bool UseCache = true;

		/// <summary>
		/// Tells the engine if we must add the shadow volume to the cache if needed
		/// </summary>
		public bool AddToCache = false;

		/// <summary>
		/// Number of times that a shadow volume was found on the list in this frame
		/// </summary>
		public int Matched;

		/// <summary>
		/// Bytes used on shadow volume cache
		/// </summary>
		public int TotalMemoryUsage;

		#endregion

		#region Properties

		/// <summary>
		/// Gets the number of shadow volumes in the list
		/// </summary>
		public int NumElements
		{
			get { return _positions.Count; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// Destroy all the list elements
		/// </summary>
		public void Cleanup()
		{
			_positions.Clear();
		}

		/// <summary>
		/// Adds the vertices to the list, if they arent there yet
		/// </summary>
		/// <param name="objectPos">TODO</param>
		/// <param name="lightPos">TODO</param>
		/// <param name="finalNumVertex">TODO</param>
		public void Add(Vector3 objectPos, Vector3 lightPos, int finalNumVertex)
		{
			JPositionSV p;

			p.ObjectPosition = VectorRounded(objectPos);

			p.LightPosition = VectorRounded(lightPos);

			if (!_positions.ContainsKey(p))
			{
				// Copy the vertices
				Vector3[] v = new Vector3[finalNumVertex];
				Array.Copy(FinalVertex, v, finalNumVertex);

				_positions.Add(p, v);

				TotalMemoryUsage += finalNumVertex * 12;
			}
		}

		/// <summary>
		/// Rounds the vector with a small error value
		/// </summary>
		/// <param name="v">Vector to round</param>
		Vector3 VectorRounded(Vector3 v)
		{
			return new Vector3((float)Math.Round(v.X, 1), (float)Math.Round(v.Y, 1), (float)Math.Round(v.Z, 1));
		}

		/// <summary>
		/// Returns the best volume match or null if no one could be found
		/// </summary>
		/// <param name="objectPos">TODO</param>
		/// <param name="lightPos">TODO</param>
		public Vector3[] BestMatch(Vector3 objectPos, Vector3 lightPos)
		{
			JPositionSV p;

			p.ObjectPosition = VectorRounded(objectPos);

			p.LightPosition = VectorRounded(lightPos);

			if (_positions.ContainsKey(p))
				return _positions[p];

			return null;
		}

		#endregion
	}
}
