#region Using directives

using System;
using System.IO;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Core;
using JadEngine.Collections;
using JadEngine.Scene;
using JadEngine.Tools;
using System.Collections.ObjectModel;

#endregion

/* 
 * 17/03/08
 *	-Initial cleanup
 *	-List now returns a ReadOnlyCollection object
 * 24/03/08
 *	-Changed to derive from JEngineElementManager
 * 30/03/08
 *	-Renamed from JSprites to JSpriteManager
 */

namespace JadEngine.Video
{
    /// <summary>
    /// Mantains the list of <see cref="JSprite"/>
    /// </summary>
    public class JSpriteManager : JObjectManager<JSprite>, IDisposable
    {

        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JVideo _video;        

        #endregion        

        #region Properties

        /// <summary>
        /// Gets the parent object
        /// </summary>
        public JVideo Video
        {
            get { return _video; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Default Constructor
        /// </summary>
        /// <param name="video">The parent <see cref="JVideo"/> instance</param>
        internal JSpriteManager(JVideo video)
        {
            _video = video;          

        }

        #endregion

        #region Dispose

        /// <summary>
        /// Implement IDisposable
        /// </summary>
        public void Dispose()
        {
            // Dispose managed resources.
            for (int c = 0; c < _items.Count; c++)
                _items[c].Dispose();

            _items.Clear();
        }


        #endregion

        #region Methods

        /// <summary>
        /// Inserts an sprite. If already exists, adds a numeric id to the name
        /// </summary>
        /// <param name="name">Sprite name</param>
        /// <returns>valid JSprite reference</returns>
        public override JSprite Create()
        {
            JSprite sprite= new JSprite(this);

            Add(sprite);

            return sprite;
        }

        /// <summary>
        /// Add the sprite to the list
        /// </summary>
        /// <param name="sprite">A valid object</param>
        public override void Add(JSprite sprite)
        {
            _items.Add(sprite);
        }

        /// <summary>
        /// Destroys all the elements
        /// </summary>
        public override void Cleanup()
        {
            for (int c = 0; c < _items.Count; c++)
            {
                JSprite sprite = _items[c];

                Delete(sprite);
            }
        }

        /// <summary>
        /// Deletes an sprite and remove it from the list
        /// </summary>
        /// <param name="sprite">A valid object</param>
        public override void Delete(JSprite sprite)
        {
            sprite.Dispose();

            _items.Remove(sprite);
        }

        #endregion		
	}
}
