#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;

#endregion

namespace JadEngine.Video
{
    /// <summary>
    /// The effect state manager class. This class hooks the DX effect, and all the
    /// states changes through this class methods
    /// </summary>
    internal class JEffectStateManager : EffectStateManager
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        JVideo _video;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="video">The parent object</param>
        internal JEffectStateManager(JVideo video)
        {
            _video = video;
        }

        #endregion

        #region Override Methods

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="state">TODO</param>
        /// <param name="value">TODO</param>
        public override void SetRenderState(RenderStates state, int value)
        {
            _video.StateManager.SetRenderState(state, value);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="stage">TODO</param>
        /// <param name="state">TODO</param>
        /// <param name="value">TODO</param>
        public override void SetTextureStageState(int stage, TextureStageStates state, int value)
        {
            _video.StateManager.SetTextureStageState(stage, state, value);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="stage">TODO</param>
        /// <param name="state">TODO</param>
        /// <param name="value">TODO</param>
        public override void SetSamplerState(int stage, SamplerStageStates state, int value)
        {
            _video.StateManager.SetSamplerState(stage, state, value);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="t">TODO</param>
        /// <param name="m">TODO</param>
        /// <remarks>Not used by the engine</remarks>
        public override void SetTransform(TransformType t, ref Matrix m)
        {
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="m">TODO</param>
        /// <remarks>Not used by the engine</remarks>
        public override void SetMaterial(ref Material m)
        {
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="l">TODO</param>
        /// <remarks>Not used by the engine</remarks>
        public override void LightEnable(int n, bool l)
        {
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="f">TODO</param>
        /// <remarks>Not used by the engine</remarks>
        public override void SetNPatchMode(float f)
        {
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetVertexFormat(VertexFormats v)
        {
            _video.RenderDevice.VertexFormat = v;
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="l">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetLight(int n, Light l)
        {
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="canal">TODO</param>
        /// <param name="textura">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetTexture(int canal, BaseTexture textura)
        {
            _video.RenderDevice.SetTexture(canal, textura);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="shader">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetPixelShader(PixelShader shader)
        {
            if (shader != JStateManager._lastPS)
            {
                _video.RenderDevice.PixelShader = shader;
                JStateManager._lastPS = shader;
            }
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="shader">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetVertexShader(VertexShader shader)
        {
            if (shader != JStateManager._lastVS)
            {
                _video.RenderDevice.VertexShader = shader;
                JStateManager._lastVS = shader;
            }
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="s">TODO</param>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetVertexShaderConstantSingle(int n, GraphicsStream s, int v)
        {
            _video.Device.SetVertexShaderConstantSingle(n, s, v);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="s">TODO</param>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetVertexShaderConstantInt32(int n, GraphicsStream s, int v)
        {
            _video.Device.SetVertexShaderConstantInt32(n, s, v);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="s">TODO</param>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetVertexShaderConstantBoolean(int n, GraphicsStream s, int v)
        {
            _video.Device.SetVertexShaderConstantBoolean(n, s, v);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="s">TODO</param>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetPixelShaderConstantSingle(int n, GraphicsStream s, int v)
        {
            _video.Device.SetPixelShaderConstantSingle(n, s, v);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="s">TODO</param>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetPixelShaderConstantInt32(int n, GraphicsStream s, int v)
        {
            _video.Device.SetPixelShaderConstantInt32(n, s, v);
        }

        /// <summary>
        /// TODO
        /// </summary>
        /// <param name="n">TODO</param>
        /// <param name="s">TODO</param>
        /// <param name="v">TODO</param>
        /// <remarks>Without any changes, as its not used in the effects</remarks>
        public override void SetPixelShaderConstantBoolean(int n, GraphicsStream s, int v)
        {
            _video.Device.SetPixelShaderConstantBoolean(n, s, v);
        }

        #endregion
    }
}
