#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// The <see cref="JMaterial"/> video states
	/// </summary>
	public struct JMaterialStates
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		public FillMode FillMode;

		/// <summary>
		/// TODO
		/// </summary>
		public ShadeMode ShadeMode;

		/// <summary>
		/// TODO
		/// </summary>
		public Cull CullMode;

		/// <summary>
		/// TODO
		/// </summary>
		public float DepthBias;

		/// <summary>
		/// Only set it to true for a postprocess material
		/// </summary>
		public bool ZBufferWriteEnable;

		#endregion

		#region Methods

		/// <summary>
		/// Sets the default values
		/// </summary>
		public void Initialize()
		{
			FillMode = FillMode.Solid;
			ShadeMode = ShadeMode.Gouraud;
			CullMode = Cull.CounterClockwise;
			ZBufferWriteEnable = false;
			DepthBias = 0;
		}

		/// <summary>
		/// Sends the states to the hardware using the <see cref="JStateManager"/>
		/// </summary>
		public void Process()
		{
			JStateManager sm = Jad.Video.StateManager;

			sm.SetRenderState(RenderStates.FillMode, (int)FillMode);
			sm.SetRenderState(RenderStates.ShadeMode, (int)ShadeMode);
			sm.SetRenderState(RenderStates.CullMode, (int)CullMode);
			sm.SetRenderState(RenderStates.DepthBias, DepthBias);
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    FillMode = (FillMode)import.LoadInt32();
		//    ShadeMode = (ShadeMode)import.LoadInt32();
		//    CullMode = (Cull)import.LoadInt32();
		//    DepthBias = import.LoadSingle();
		//    ZBufferWriteEnable = import.LoadBoolean();

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);

		//    export.Save((int)FillMode);
		//    export.Save((int)ShadeMode);
		//    export.Save((int)CullMode);
		//    export.Save(DepthBias);
		//    export.Save(ZBufferWriteEnable);

		//    return true;
		//}

		#endregion
	}

}
