#region Using directives

using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using JadEngine.Scene;
using JadEngine.Tools;
using JadEngine.Core;

#endregion

namespace JadEngine.Video
{
	/// <summary>
	/// The sampler states for one stage. This samplers will be used in <see cref="JMaterialLayer"/>
	/// </summary>
	public class JSamplerVideoStates
	{
		#region Fields

		/// <summary>
		/// TODO
		/// </summary>
		private TextureAddress _addressU;

		/// <summary>
		/// TODO
		/// </summary>
		private TextureAddress _addressV;

		/// <summary>
		/// TODO
		/// </summary>
		private TextureAddress _addressW;

		/// <summary>
		/// TODO
		/// </summary>
		private System.Drawing.Color _borderColor;

		/// <summary>
		/// TODO
		/// </summary>
		private TextureFilter _magFilter;

		/// <summary>
		/// TODO
		/// </summary>
		private TextureFilter _minFilter;

		/// <summary>
		/// TODO
		/// </summary>
		private TextureFilter _mipFilter;

		/// <summary>
		/// TODO
		/// </summary>
		private float _mipMapLevelOfDetailBias;

		/// <summary>
		/// TODO
		/// </summary>
		private int _maxMipLevel;

		/// <summary>
		/// TODO
		/// </summary>
		private int _maxAnisotropy;

		/// <summary>
		/// TODO
		/// </summary>
		private bool _srgbTexture;

		/// <summary>
		/// TODO
		/// </summary>
		private int _elementIndex;

		/// <summary>
		/// TODO
		/// </summary>
		private int _dMapOffset;


		#endregion

		#region Properties

		public TextureAddress AddressU 
		{ 
			get { return _addressU; } 
			set { _addressU = value; } 
		}

		public TextureAddress AddressV 
		{ 
			get { return _addressV; } 
			set { _addressV = value; } 
		}

		public TextureAddress AddressW 
		{ 
			get { return _addressW; } 
			set { _addressW = value; } 
		}

		public System.Drawing.Color BorderColor 
		{ 
			get { return _borderColor; } 
			set { _borderColor = value; } 
		}

		public TextureFilter MagFilter 
		{ 
			get { return _magFilter; } 
			set { _magFilter = value; } 
		}

		public TextureFilter MinFilter 
		{ 
			get { return _minFilter; } 
			set { _minFilter = value; } 
		}

		public TextureFilter MipFilter 
		{ 
			get { return _mipFilter; } 
			set { _mipFilter = value; } 
		}

		public float MipMapLevelOfDetailBias 
		{ 
			get { return _mipMapLevelOfDetailBias; } 
			set { _mipMapLevelOfDetailBias = value; } 
		}

		public int MaxMipLevel 
		{ 
			get { return _maxMipLevel; } 
			set { _maxMipLevel = value; } 
		}

		public int MaxAnisotropy 
		{ 
			get { return _maxAnisotropy; } 
			set { _maxAnisotropy = value; } 
		}

		public bool SrgbTexture 
		{ 
			get { return _srgbTexture; } 
			set { _srgbTexture = value; } 
		}

		public int ElementIndex 
		{ 
			get { return _elementIndex; } 
			set { _elementIndex = value; } 
		}

		public int DMapOffset 
		{ 
			get { return _dMapOffset; } 
			set { _dMapOffset = value; } 
		}


		#endregion

		#region Constructor

		public JSamplerVideoStates()
		{
			AddressU = TextureAddress.Wrap;
			AddressV = TextureAddress.Wrap;
			AddressW = TextureAddress.Wrap;
			BorderColor = System.Drawing.Color.Black;
			MagFilter = TextureFilter.Linear;
			MinFilter = TextureFilter.Linear;
			MipFilter = TextureFilter.Linear;
			MipMapLevelOfDetailBias = 0;
			MaxMipLevel = 0;
			MaxAnisotropy = 1;
			SrgbTexture = false;
			ElementIndex = 0;
			DMapOffset = 0;
		}


		#endregion

		#region Methods

		/// <summary>
		/// Sets the default values
		/// </summary>
		//public void Initialize()
		//{
		//    AddressU = TextureAddress.Wrap;
		//    AddressV = TextureAddress.Wrap;
		//    AddressW = TextureAddress.Wrap;
		//    BorderColor = System.Drawing.Color.Black;
		//    MagFilter = TextureFilter.Linear;
		//    MinFilter = TextureFilter.Linear;
		//    MipFilter = TextureFilter.Linear;
		//    MipMapLevelOfDetailBias = 0;
		//    MaxMipLevel = 0;
		//    MaxAnisotropy = 1;
		//    SrgbTexture = false;
		//    ElementIndex = 0;
		//    DMapOffset = 0;
		//}

		/// <summary>
		/// Sends the states to the hardware using the <see cref="JStateManager"/>
		/// </summary>
		public void Process(int stage)
		{
			JStateManager sm = Jad.Video.StateManager;

			sm.SetSamplerState(stage, SamplerStageStates.AddressU, (int)_addressU);
			sm.SetSamplerState(stage, SamplerStageStates.AddressV, (int)_addressV);
			sm.SetSamplerState(stage, SamplerStageStates.AddressW, (int)_addressW);
			sm.SetSamplerState(stage, SamplerStageStates.BorderColor, _borderColor.ToArgb());
			sm.SetSamplerState(stage, SamplerStageStates.MagFilter, (int)_magFilter);
			sm.SetSamplerState(stage, SamplerStageStates.MinFilter, (int)_minFilter);
			sm.SetSamplerState(stage, SamplerStageStates.MipFilter, (int)_mipFilter);
			sm.SetSamplerState(stage, SamplerStageStates.MipMapLevelOfDetailBias, _mipMapLevelOfDetailBias);
			sm.SetSamplerState(stage, SamplerStageStates.MaxMipLevel, _maxMipLevel);
			sm.SetSamplerState(stage, SamplerStageStates.MaxAnisotropy, _maxAnisotropy);
			sm.SetSamplerState(stage, SamplerStageStates.SrgbTexture, _srgbTexture);
			sm.SetSamplerState(stage, SamplerStageStates.ElementIndex, _elementIndex);
			sm.SetSamplerState(stage, SamplerStageStates.DMapOffset, _dMapOffset);
		}

		#endregion

		#region Serialize

		/// <summary>Current FileVersion</summary>
		const int fileVersion = 1;

		/// <summary>
		/// Loads the values from the binary file
		/// </summary>
		/// <param name="import">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Load(JBinaryImport import)
		//{
		//    int fileVersionLoaded = import.LoadInt32();

		//    AddressU = (TextureAddress)import.LoadInt32();
		//    AddressV = (TextureAddress)import.LoadInt32();
		//    AddressW = (TextureAddress)import.LoadInt32();
		//    BorderColor = System.Drawing.Color.FromArgb(import.LoadInt32());
		//    MagFilter = (TextureFilter)import.LoadInt32();
		//    MinFilter = (TextureFilter)import.LoadInt32();
		//    MipFilter = (TextureFilter)import.LoadInt32();
		//    MipMapLevelOfDetailBias = import.LoadSingle();
		//    MaxMipLevel = import.LoadInt32();
		//    MaxAnisotropy = import.LoadInt32();
		//    SrgbTexture = import.LoadBoolean();
		//    ElementIndex = import.LoadInt32();
		//    DMapOffset = import.LoadInt32();

		//    return true;
		//}

		/// <summary>
		/// Save the values to the binary file
		/// </summary>
		/// <param name="export">A valid instance</param>
		/// <returns>True if everything is ok</returns>
		//internal bool Save(JBinaryExport export)
		//{
		//    export.Save(fileVersion);
		//    export.Save((int)AddressU);
		//    export.Save((int)AddressV);
		//    export.Save((int)AddressW);
		//    export.Save(BorderColor.ToArgb());
		//    export.Save((int)MagFilter);
		//    export.Save((int)MinFilter);
		//    export.Save((int)MipFilter);
		//    export.Save(MipMapLevelOfDetailBias);
		//    export.Save(MaxMipLevel);
		//    export.Save(MaxAnisotropy);
		//    export.Save(SrgbTexture);
		//    export.Save(ElementIndex);
		//    export.Save(DMapOffset);
		//    return true;
		//}

		#endregion
	}
}
