#region Using Statements

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace JadScript
{
	/// <summary>
	/// Form used to display exception information
	/// </summary>
    public partial class ExceptionDisplayForm : System.Windows.Forms.Form
	{
		#region Fields
		private const int _shortHeight = 155;
        private const int _longHeight = 500;
		#endregion

		#region Constructors
		/// <summary>
        /// Construct our form from a System.Exception
        /// </summary>
        /// <param name="ex">The Exception</param>
        public ExceptionDisplayForm(Exception ex)
        {
			if (ex == null)
				ex = new ArgumentException("Exception display form called without exception", "ex");
            InitializeComponent();
            TreeNode mainNode = this.treeExceptions.Nodes[0];
            System.Type type = ex.GetType();
            mainNode.Text = type.ToString();
            if (ex.Source == null)
                labelType.Text = "A " + type.ToString() + " occurred in the application.";
            else
                labelType.Text = "A " + type.ToString() + " occurred in " + ex.Source + ".";
            textBoxMessage2.Text = ex.Message;
            mainNode.Tag = ex;
            Exception e = ex;
            TreeNode parent = mainNode;
            while (e.InnerException != null)
            {
                e = e.InnerException;
                TreeNode newNode = new System.Windows.Forms.TreeNode(e.GetType().ToString());
                newNode.Tag = e;
                parent.Nodes.Add(newNode);
                parent = newNode;
            }
            treeExceptions.Select();
            treeExceptions.SelectedNode = mainNode;
            treeExceptions.ExpandAll();

            Height = _shortHeight;
		}

		#endregion
	
		#region Methods

		#region Private Implementation
		private void treeExceptions_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeNode node = e.Node;
            if (!(node.Tag is Exception))
                return;
            Exception ex = node.Tag as Exception;
            textBoxExceptionType.Text = ex.GetType().ToString();
            textBoxMessage.Text = ex.Message;
            textBoxSource.Text = ex.Source;
            textBoxStack.Text = ex.StackTrace;
            if (ex.TargetSite != null)
                textBoxTargetSite.Text = ex.TargetSite.ToString();
            else
                textBoxTargetSite.Text = "";
        }

        private void buttonDetails_Click(object sender, EventArgs e)
        {
            if (this.Size.Height < 200)
            {
                buttonDetails.Text = "Hide Details";
				Height = _longHeight;
            }
            else
            {
                buttonDetails.Text = "Show Details";
                Height = _shortHeight;
            }
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            Close();
		}

		private void buttonExit_Click(object sender, EventArgs e)
		{
			Application.Exit();
		}

		#endregion

		#endregion
	}
}