#region Using Directives

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;

using Pluto.Core;
using Pluto.Scripting;

#endregion

namespace JadScript
{
	static class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string [] args)
		{
			// If we are passed a configuration file, use it
			if (args.Length == 0)
			{
					OpenFileDialog openFileDialog1 = new OpenFileDialog();
					DirectoryInfo dInfo = new DirectoryInfo(Application.StartupPath);
					while (dInfo.GetDirectories("Scripts").Length == 0)
					{
						if (dInfo.FullName.Length < 4)
							break;
						dInfo = dInfo.Parent;
					}

					if (dInfo.GetDirectories("Scripts").Length == 0)
						openFileDialog1.InitialDirectory = dInfo.FullName;
					else
						openFileDialog1.InitialDirectory = dInfo.FullName + "\\Scripts";

					openFileDialog1.Filter = "XML files (*.XML)|*.XML";
					openFileDialog1.RestoreDirectory = true;
					openFileDialog1.Title = "Choose Configuration XML File:";
					if(openFileDialog1.ShowDialog() != DialogResult.OK)
						return;
					JadEngine.Jad.ConfigurationFileName = openFileDialog1.FileName;
			}
			else
				JadEngine.Jad.ConfigurationFileName = Path.Combine(Directory.GetCurrentDirectory(), args[0]);

			Directory.SetCurrentDirectory(Path.GetDirectoryName(JadEngine.Jad.ConfigurationFileName));
			

			// Create a new file where well write console output
			StreamWriter sw = new StreamWriter("out.txt");

			// Redirect console output to this file
			Console.SetOut(sw);
			
			// Load Jad, and initialize it
			JadEngine.Jad.Initialize();

			// Get the service provider
			IPlutoRootServiceProvider serviceProvider = JadEngine.Jad.ServiceProvider;
			IScriptEngineFactory engineFactory = serviceProvider.GetService<IScriptEngineFactory>();
			if (engineFactory == null)
				throw new ApplicationException("This application requires a valid Pluto.Scripting plugin to be loaded");

			// Set this in the static JadScriptOptions class
			JadScriptOptions._factory = engineFactory;

			// Add in Boo as a language
			serviceProvider.CreateContainer().AddService<IScriptLanguage>(new LangBoo(), "Boo");

			// Create the application object
			JadScriptApp app = new JadScriptApp(JadEngine.Jad.ServiceProvider);

			// Start the application
			try
			{
				app.Start();
			}
			catch (Exception e)
			{
				// Put in a custom exception display handler for our application
				ExceptionDisplayForm form = new ExceptionDisplayForm(e);
				form.ShowDialog();

				// On an exception, we need to tell the engine to shutdown gracefully,
				// or it causes another exception to throw
				JadEngine.Jad.End();
			}
			

			// Close console output file
			sw.Close();
		}
	}
}