//----------------------------------------------------------------------------------
//*********** Las siguientes variables DEBEN tener EXACTAMENTE este nombre ***********

// Definimos el tipo MATRIX como una matriz de 4x4 floats

#define MATRIX matrix<float, 4, 4>

MATRIX WorldViewProjection;		// World * View * Projection

struct Camara {
	float4	posicion;
};

struct Material {
	float4	specular;
	float	specularPower;
	float4	diffuse;
	float3	emissive;
	float	bumpiness;
	float	reflectivity;
};

struct Entorno {
	float4	ambiente;
	float4	ancho_alto;	// los 2 primeros informan del ancho/alto del RT. Los 2 siguientes del ancho/alto de las sombras
	float	time;		// tiempo transcurrido
};

//------------------------------------------------------------------------------------
// Mapas de textura. Por desgracia no se puede aadir la textura dentro de la estructura
//------------------------------------------------------------------------------------

struct map {
	float2	tile;		// u,v del tiling de la textura
};


// definimos las variables que utilizaremos en los shaders asociadas a los mapas
// Siempre utilizaremos diffuse, as que se ha aadido en este include

map mapDiffuse;
texture textureDiffuse;

Camara camara;
Entorno entorno;
Material material;

float4 cam_right = float4(1.0f, 0.0f, 0.0f, 0.0f);
float4 cam_up = float4(0.0f, 1.0f, 0.0f, 0.0f);

float size = 1.0f;
float3 lightPos;
float4 color = float4(1.0f, 1.0f, 1.0f, 1.0f);

struct VS_IN
{
	float4 position		: POSITION;
	float2 texCoord0	: TEXCOORD0;
	float2 axes			: TEXCOORD1;
	float4 color		: COLOR0;
};

struct VS_OUT
{
	float4 position		: POSITION;
	float2 texCoord0	: TEXCOORD0;
	float4 color		: COLOR0;
};

VS_OUT VShader( VS_IN data_in )
{
	VS_OUT data_out = (VS_OUT)0;

	float4 pos = data_in.position;
	
	pos.xyz = cam_right.xyz * data_in.axes.x * size + lightPos;
	pos.xyz += cam_up.xyz * data_in.axes.y * size;
	pos.w = 1.0f;

	// Pasamos la posicin a clip-space
	data_out.position = mul(pos, WorldViewProjection);

	// Coordenada de textura
	data_out.texCoord0 = data_in.texCoord0;
	
	// Color
	//data_out.color = data_in.color;
	data_out.color = color;

	return data_out;
}

sampler base_map =
sampler_state
{
	Texture		= <textureDiffuse>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	AddressU	= Clamp;
	AddressV	= Clamp;
};

struct PS_IN
{
	float4 texCoord0	: TEXCOORD0;
	float4 color		: COLOR0;
};

float4 PShader( PS_IN data_in ) : COLOR0
{
	float4 fragmentColor = float4(0.0f, 0.0f, 0.0f, 1.0f);

	// Base texture look up
	float4 base = tex2D(base_map, data_in.texCoord0);

	fragmentColor.rgb = base.rgb * data_in.color.rgb * data_in.color.a;

	return fragmentColor;
}

technique RenderScene
{
	pass P0
	{
		VertexShader = compile vs_1_1 VShader();
		PixelShader = compile ps_1_1 PShader();
	}
}