AllInOne

struct VS_INPUT
{
   float4 position:     POSITION;
   float2 texCoord0:    TEXCOORD0;
};

struct VS_OUTPUT
{
   float4 position:     POSITION;
   float2 texCoord0:    TEXCOORD0;
   float4 texCoord1:    TEXCOORD1;
};

VS_OUTPUT VShader( VS_INPUT vin )
{
	VS_OUTPUT data_out;
   
	data_out.position = mul(vin.position, WorldViewProjection);
	data_out.texCoord0 = vin.texCoord0;
	data_out.texCoord1 = ToScreenSpace(data_out.position);

	return data_out;
}

struct PS_INPUT
{
   float2 texCoord0:     TEXCOORD0;
   float4 texCoord1:     TEXCOORD1;
};

texture _currentRender;
texture _normalMap;
texture _mask;

sampler currentRender : register(s0) = 
sampler_state
{
    Texture = <_currentRender>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler normalMap : register(s1) = 
sampler_state
{
    Texture = <_normalMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler mask : register(s2) = 
sampler_state
{
    Texture = <_mask>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

bool doScroll = true;
float2 scroll = float2(0.5f, -0.5f);
float2 distortion = float2(0.025f, 0.025f);
float globalMultiplier = 0.5f;

float4 PShader(PS_INPUT data_in):COLOR0
{
   float2 tex0 = data_in.texCoord0.xy;
   float2 tex1 = data_in.texCoord1.xy / data_in.texCoord1.w;

   float2 normalTex = float2( tex0.x + ENV_Time * scroll.x * doScroll,
                              tex0.y + ENV_Time * scroll.y * doScroll);
   float3 normal = tex2D(normalMap, normalTex) * 2.0f - 1.0f;

   float mask = tex2D(mask, tex0);

   normal *= mask * globalMultiplier;

   tex1.x += normal * distortion.x;
   tex1.y += normal * distortion.y;

   return tex2D(currentRender, tex1);
}

technique RenderScene
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader();
    }
}

