//----------------------------------------------------------------------------------
//*********** Las siguientes variables DEBEN tener EXACTAMENTE este nombre ***********

// Definimos el tipo MATRIX como una matriz de 4x4 floats

#define MATRIX matrix<float, 4, 4>

MATRIX World;					// World
MATRIX WorldView;				// World * View
MATRIX WorldViewProjection;		// World * View * Projection

struct Camara {
	float4	posicion;
};

struct Material {
	float4	specular;
	float	specularPower;
	float4	diffuse;
	float3	emissive;
	float	bumpiness;
	float	reflectivity;
};

struct Entorno {
	float4	ambiente;
	float4	ancho_alto;	// los 2 primeros informan del ancho/alto del RT. Los 2 siguientes del ancho/alto de las sombras
	float	time;		// tiempo transcurrido
};

//------------------------------------------------------------------------------------
// Mapas de textura. Por desgracia no se puede aadir la textura dentro de la estructura
//------------------------------------------------------------------------------------

struct map {
	float2	tile;		// u,v del tiling de la textura
};


// definimos las variables que utilizaremos en los shaders asociadas a los mapas
// Siempre utilizaremos diffuse, as que se ha aadido en este include

map mapDiffuse;
texture texturaDiffuse;
texture texturaNormal;
texture texturaEnvironment;

Camara camara;
Entorno entorno;
Material material;

struct VS_IN
{
	float4 position		: POSITION;
	float3 normal		: NORMAL;
	float3 tangent		: TANGENT;
	float2 texCoord0	: TEXCOORD0;
};

struct VS_OUT
{
	float4 position		: POSITION;
	float4 texCoord0	: TEXCOORD0;
	float3 normal		: TEXCOORD1;
	float3 viewVector	: TEXCOORD2;
};

VS_OUT VShader( VS_IN data_in )
{
	VS_OUT data_out = (VS_OUT)0;

	float4 finalPosition	= data_in.position;
	float3 finalNormal		= data_in.normal;
	float3 finalTangent		= data_in.tangent;

	// Pasamos la posicin a clip-space
	data_out.position = mul(finalPosition, WorldViewProjection);

	// Coordenada de textura
	data_out.texCoord0.xy = data_in.texCoord0 * mapDiffuse.tile;
	
	// Normal en world-space
	float3 wNormal = mul(finalNormal, (float3x3)World);

	data_out.normal = wNormal;
	
	// Calculamos la posicin del vrtice en world-space
	float3 wPosition = mul(finalPosition, World);
	
	// tangent-space
	float3x3 toTangentSpace;
	toTangentSpace[0] = mul(finalTangent, (float3x3)World);
	toTangentSpace[2] = wNormal;
	toTangentSpace[1] = cross(toTangentSpace[0], toTangentSpace[2]);
	
	// Pasamos el vector view a tangent-space
	data_out.viewVector	= camara.posicion - wPosition;
	data_out.viewVector	= mul(toTangentSpace, data_out.viewVector);
	
	return data_out;
}

sampler base_map =
sampler_state
{
	Texture		= <texturaDiffuse>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	AddressU	= Wrap;
	AddressV	= Wrap;
};

sampler normal_map =
sampler_state
{
	Texture		= <texturaNormal>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	AddressU	= Wrap;
	AddressV	= Wrap;
};

sampler environment_map =
sampler_state
{
	Texture		= <texturaEnvironment>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
    AddressU	= Clamp;
    AddressV	= Clamp;
};

struct PS_IN
{
	float4 texCoord0	: TEXCOORD0;
	float3 normal		: TEXCOORD1;
	float3 viewVector	: TEXCOORD2;
};

// Funcin que calcula la contribucin de un environment map
float4 environment(sampler envMap, float3 texCoords)
{
	return texCUBE(envMap, texCoords);
}

// Funcin que pasa un vector del rango [0,1] al [-1,1]
float3 expand(float3 v)
{
	return (v * 2.0f - 1.0f);
}

float4 PShader( PS_IN data_in ) : COLOR0
{
	float4 fragmentColor = float4(0.0f, 0.0f, 0.0f, 1.0f);

	// View vector
	float3 V = normalize(data_in.viewVector);
	
	// Normal vector
	float3 N = normalize(data_in.normal);

	// Base texture look up
	float4 base = tex2D(base_map, data_in.texCoord0);

	// Environment Mapping
	R = reflect(-V, N);
	float4 envColor = environment(environment_map, R);
		
	// Modulamos el color del fragtmento con el environment map
	fragmentColor.rgb = lerp(fragmentColor.rgb, envColor.rgb, material.reflectivity);

	fragmentColor.a = base.a;

	return fragmentColor;
}