//----------------------------------------------------------------------------------
texture textureDiffuse;		// Mapa de textura diffuse (contiene la imagen downsampled con 3x3 gaussian blur)
texture textureCurRender;	// Contiene la escena original

float2 pixelSizeHigh;
float2 pixelSizeLow;

float2 vMaxCoC = float2(5.0f, 10.0f);

float radiusScale = 0.4f;
//----------------------------------------------------------------------------------

struct VS_INPUT
{
   float4 position:     POSITION;
   float2 texCoord0:    TEXCOORD0;
};

struct VS_OUTPUT
{
   float4 position:     POSITION;
   float2 texCoord0:    TEXCOORD0;
};

VS_OUTPUT VShader( VS_INPUT data_in )
{
   VS_OUTPUT data_out;
   
   data_out.position = data_in.position;
   data_out.texCoord0 = data_in.texCoord0;

   return data_out;
}

sampler map_blurred = 
sampler_state
{
    Texture = <textureDiffuse>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler map_curRender = 
sampler_state
{
    Texture = <textureCurRender>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//float2 vPoisson[10] = {
//
//	float2(0.0,0.0),
//	float2(0.527837, -0.085868),
//	float2(-0.040088, 0.536087),
//	float2(-0.670445, -0.179949),
//	float2(-0.419418, -0.616039),
//	float2(0.440453, -0.639399),
//	float2(-0.757088, 0.349334),
//	float2(0.574619, 0.685879),
//	float2(0.03851, -0.939059),
//	float2(-0.341977, 0.939091)
//};

float2 vPoisson[4] = {
	float2(-0.615914f, 0.5137f),
	float2(0.47234f, 0.4983f),
	float2(-0.49642f, -0.458f),
	float2(0.539f, -0.5771f)
};

struct PS_INPUT
{
   float2 texCoord0: TEXCOORD0;
};

float4 PShader(PS_INPUT data_in) : COLOR0
{
	float4 fragmentColor;
	float4 tmp;
	float discRadiusHigh, discRadiusLow, centerDepth;

	fragmentColor = tex2D(map_curRender, data_in.texCoord0);

	centerDepth = fragmentColor.a;
	
	discRadiusHigh = abs(centerDepth * vMaxCoC.y - vMaxCoC.x);
	discRadiusLow = discRadiusHigh * radiusScale;
	
	fragmentColor = 0;
	
	for(int t = 0; t < 4; t++)
	{
		float4 texCoord;

		float2 texCoordLow = data_in.texCoord0 + (pixelSizeLow * vPoisson[t] * discRadiusLow);
		float2 texCoordHigh = data_in.texCoord0 + (pixelSizeHigh * vPoisson[t] * discRadiusHigh);
		
		float4 tapLow = tex2D(map_blurred, texCoordLow);
		float4 tapHigh = tex2D(map_curRender, texCoordHigh);

		tapLow.a = tapHigh.a;
		tapHigh.a = tapHigh.a;
		
		float tapBlur = abs(tapHigh.a * 2.0f - 1.0f);
		float4 tap = lerp(tapHigh, tapLow, tapBlur);
		
		tap.a = (tap.a >= centerDepth) ? 1.0f : abs(tap.a * 2.0f - 1.0f);
		
		fragmentColor.rgb += tap.rgb * tap.a;
		fragmentColor.a += tap.a;
	}

	return (fragmentColor/fragmentColor.a);
}

technique RenderScene
{
    pass p0
    {
        VertexShader = compile vs_1_1 VShader();
        PixelShader = compile ps_2_0 PShader();
    }
}

