//--------------------------------------------------------------------------------------------------
// ALLINONE:PixelShader
//--------------------------------------------------------------------------------------------------
struct PS_IN
{
	float4 texCoord0	: TEXCOORD0;
	float4 t1	: TEXCOORD1;
	float4 t2	: TEXCOORD2;
	float4 t3	: TEXCOORD3;
	float4 t4	: TEXCOORD4;
	float4 t5	: TEXCOORD5;
	float4 t6	: TEXCOORD6;
	float4 t7		: TEXCOORD7;
	float4 diffuse		: COLOR0;
	float4 specular		: COLOR1;
};

float4 PShader(PS_IN data_in) : COLOR0
{
	// Recuperamos los valores del vertex shader
	
    float3 tViewVector;					// Tangent space View Vector
    float3 wNormal,wTangent,wBinormal;	// Normal to World Space
    float4 screenCoord,projTexCoord;
    float3 wLightVector,wViewVector,wPosition;
    float3 nWorldLightVector;
	
    tViewVector=normalize(data_in.t1.xyz);
    wNormal.x=data_in.t1.w;
    wNormal.yz=data_in.t2.xy;
    wTangent.xy=data_in.t2.zw;
    wTangent.z=data_in.t3.x;
    wBinormal=data_in.t3.yzw;
    screenCoord.xyw=data_in.t4.xyz;
    screenCoord.z=0;	// La z no se utiliza en el shader y la necesito para pasar parmetros
    wPosition.z=data_in.t4.w;    
    projTexCoord=data_in.t5;
    wLightVector=data_in.t6.xyz;
    wViewVector.x=data_in.t6.w;
    wViewVector.yz=data_in.t7.xy;
    wPosition.xy=data_in.t7.zw;
    
	float3 fragmentColor = 0.0f;
				
	float4 base = 0;			// Base color
	float4 specularColor = 0;	// Specular Color

	// View vector
	float3 V;
	
	// Light vector
	float3 L;
	
	nWorldLightVector=normalize(wLightVector);
	
	// If we want the normals in world space
	
	if(Uniform_WorldSpaceNormals)
	{
		V = normalize(wViewVector);
	
		L = nWorldLightVector;
	}
	else	// The normals are in tangent space
	{
		V=tViewVector;				// the tangent space view vector

		L = normalize(wBinormal);	// the binormal here represents the tangent space light vector
	}
	
	// Normal vector
	float3 N;

	// Used for the envmap 
	float NdotL = 0.0f;

	float2 displaced_texcoord= data_in.texCoord0.xy;
	float parallaxOcclusionShadowing=1;
	
	if(Uniform_parallaxMapping)
	{
		float3 parallaxValues=tex2Dproj( ParallaxValues_map, screenCoord ).xyz;
		
		parallaxOcclusionShadowing=parallaxValues.z;
		
		displaced_texcoord=parallaxValues.xy;
		/*
		float height = tex2D(normal_map, displaced_texcoord).a;
		displaced_texcoord = CalculateParallaxOcclusion(normal_map, displaced_texcoord, data_in.texCoord0.zw);
		*/
	}

	if(Uniform_diffuseMapEnabled)
		base= tex2D(base_map, displaced_texcoord) * MATERIAL_Diffuse;
	else
		base = MATERIAL_Diffuse;

	if(Uniform_affectSpecular)
	{
		if(Uniform_specularMapEnabled)
			specularColor = tex2D(specular_map, displaced_texcoord) * MATERIAL_Specular;
		else
			specularColor = MATERIAL_Specular;
	}

	// Si est activado el detail mapping, debemos recuperar el Alpha del
	// mapa de mascaras del detail.
	
	if(Uniform_DetailMapEnabled)
	{
		float2 displaced_texcoord_detail=displaced_texcoord;
		
		// si las coordenadas no son las mismas que las del diffuse,
		// las recogemos y hacemos el parallax mapping si es necesario
		
	    if(Uniform_useTexCoordForDetailMap) 
	    {
	        displaced_texcoord_detail= data_in.texCoord0.zw;

	        if(Uniform_parallaxMapping)
	        {
		        float height = tex2D(normal_map, displaced_texcoord_detail).a;
		        displaced_texcoord_detail = CalculateParallax(height, displaced_texcoord_detail, tViewVector, MATERIAL_PARALLAXAMOUNT);
	        }
	    }
	
	    base.a = tex2D(detailMask_map, displaced_texcoord_detail).x + DETAIL_Amount;
	}

	// La normal la necesitamos siempre que no estemos renderizando slo ambiente
	// o en el caso que s lo estemos renderizando, que haya refraccin o reflexin.
	
	if(!Uniform_AmbientOnly || Uniform_reflectionMapEnabled || Uniform_refractionMapEnabled)
	{
		if(Uniform_SphericalMapping)
		{
				float3 normal = tex2D(normal_map, displaced_texcoord);
				// Normal vector (parece que no hace falta re-normalizar)
				N = expand(normal);
				N.x = -N.x;
				N = mul((float3x3)View,N);		// N* Inversa de View(traspuesta)
				
				//base.a*=base.a;	// forzamos el alpha
				
		}
		else
		{
			// If we are using normal mapping, get the value from the normal map
			if(Uniform_normalType == NORMAL_MAPPING)
			{
				float3 normal = tex2D(normal_map, displaced_texcoord);
				
				// Interpolamos entre una normal plana y el valor recogido en el bump
				// para acentuar o reducir la potencia del bump.
				float3 smooth = float3(0.5f, 0.5f, 1.0f);
				
				normal = lerp(smooth, normal, MATERIAL_Bumpiness);
			
				// Normal vector (parece que no hace falta re-normalizar)
				N = expand(normal);

				// Si queremos la normal en World space, debemos convertirla
				if(Uniform_WorldSpaceNormals)
				{
					// Pasamos la normal a world-space. No es necesario normalizar
					float3x3 tangentToWorld = float3x3(wTangent, wBinormal, wNormal);
					N = normalize(mul(N, tangentToWorld));	// Esto equivale a normalize(mul(transpose(tangentToWorld), N))
				}
			}
			else if(Uniform_normalType == NORMAL_VERTEX)
			{
				N = normalize(wNormal);
			}
		}
		
	}
	
	// Si estamos renderizando el paso de ambiente, no necesitamos
	// calcular la iluminacin, puesto que no hay vector de luz
	
	if(Uniform_AmbientOnly)
	{
		fragmentColor=base.xyz*ENV_Ambient;
	}
	else
	{

		
		// Pasamos a la atenuacin el vector en World Space y ya normalizado, para ahorrar clculos
		
		float attenuation = GetAttenuation(wLightVector,nWorldLightVector);

		//--------------------------------------------------------------------------------------------------
		// ALLINONE:LIGHTING_PS
		//--------------------------------------------------------------------------------------------------
		
		if(Uniform_projectorEnabled)
		{
			// It's possible to have a light that do not affects diffuse & specular, only projects
			// a bitmap
					
			if(!Uniform_affectDiffuse && !Uniform_affectSpecular)
			{
				fragmentColor.rgb = projector(projector_map, projTexCoord) * LIGHT_Color * diffuse(N, L);
				
				// si utilizamos la luz como proyector, no queremos que le afecte la atenuacin
				attenuation=1.0f;
			}
			else
			{
				fragmentColor.rgb *= projector(projector_map, projTexCoord);
			}
		}

		fragmentColor *= attenuation;

	}	// Fin del if(Uniform_OnlyAmbient)
	
	// Variables que almacenarn la contribucin de la reflexin y la refraccin
	float3 reflColor = 0.0f;
	float3 refrColor = 0.0f;
	float fresnel = 1.0f;

	if(Uniform_reflectionMapEnabled || Uniform_refractionMapEnabled)
	{
		fresnel = 1.0f - ComputeFresnelApprox(N, V);
	}

	// Calculamos la reflexin
	if(Uniform_reflectionMapEnabled)
	{
		float3 R = reflect(-V, N);
		
		// Environment Mapping
		reflColor = fresnel * environment(reflection_map, R);
		
		// Modulamos el color del fragmento con el environment map
		//reflColor = reflColor * MATERIAL_Reflectivity;
		
		fragmentColor=lerp(fragmentColor,reflColor,MATERIAL_Reflectivity);
	}
	
	// Calculamos la refraccin
	if(Uniform_refractionMapEnabled)
	{
		// Calculamos el vector refractado
		float3 T = TransmissionDirection (1.0f, MATERIAL_IOR, -V, N);
		
		// sumamos la desviacin de la posicin
		//T +=wPosition;
	
		// Environment Mapping
		refrColor = fresnel * environment(refraction_map, T);

		// Modulamos el color del fragmento con el environment map
		//refrColor = refrColor * MATERIAL_Transmittance;
		
		fragmentColor=lerp(fragmentColor,refrColor,MATERIAL_Transmittance);
	}
	
	if(Uniform_shadowsOnRender)
	{
		// Grab the shadow term
		float fShadowTerm = tex2Dproj( shadow_map, screenCoord );
		
		fragmentColor *= fShadowTerm;
				
	}

	fragmentColor *=parallaxOcclusionShadowing;

	if(Uniform_ApplyAmbient) fragmentColor+=base.xyz*ENV_Ambient;
	
	if(Uniform_volumetricLight)
	{
		// Calculamos la distancia de la luz a la lnea que va del vrtice en WS a la cmara
		float4 intersection=DistancePointLine(LIGHT_Position,camera.position,wPosition);
		
		float distance=intersection.w;
		
		// A mayor distancia menos influencia
		
		if(LIGHT_FarAttenEnd-distance>=0)
		{
			distance=max(0,1-(distance*LIGHT_PowVolumetric));
			
			intersection.w=1;
				    
			fragmentColor+=(LIGHT_Color*distance);
			
		}
	}
	
	// Alpha channel es siempre el canal alpha del diffuse
	
	return float4(fragmentColor * data_in.diffuse.rgb, base.a* data_in.diffuse.a);
}


