//--------------------------------------------------------------------------------------------------
// ALLINONE:PixelShader
//--------------------------------------------------------------------------------------------------
struct PS_IN
{
	float4 texCoord0	: TEXCOORD0;
	float3 wNormal		: TEXCOORD1;
	float3 wBinormal	: TEXCOORD2;
	float3 lightVector	: TEXCOORD3;
	float3 viewVector	: TEXCOORD4;
	float3 eyeVectorTangentSpace	: TEXCOORD5;
	float4 screenCoord	: TEXCOORD6;
	float4 wTangent		: TEXCOORD7;
	float4 diffuse		: COLOR0;
	float4 specular		: COLOR1;
};

// input is x = (Distance^2/Attenuation^2)
float ghost(float x)
{
	float fx = 1.0f - x;
	float fi = min(1.0f, max(0.0f, min(fx * fx + 0.1f, 1 - fx * fx * fx)));

	return fi;
}

float4 PShader(PS_IN data_in) : COLOR0
{
	float4 finalColor;

	float2 displaced_texcoord= data_in.texCoord0.xy;
	
	if(Uniform_diffuseMapEnabled)
		finalColor= tex2D(base_map, displaced_texcoord)*MATERIAL_Diffuse;
	else
		finalColor = MATERIAL_Diffuse;
	
	if(Uniform_SphericalMapping)
	{
	//****************** TEST ******************
	float3 L = -normalize(float3(2.0f, -2.0f, -2.0f));
	float3 N = tex2D(normal_map, displaced_texcoord).rgb;
	
	N = N * 2.0f - 1.0f;
	N.x = -N.x;
	N = mul((float3x3)View,N);		// N* Inversa de View(traspuesta)
	float NDotL = max(0.1f, abs(dot(N, L) * 1.5f));
	
	float3 col = data_in.diffuse.rgb * NDotL;
	finalColor *= float4(col.r, col.g, col.b, data_in.diffuse.a * finalColor.a);
	//finalColor.rgb = NDotL;
	//******************************************
	}
	else
	{
		finalColor *= data_in.diffuse;
	}
	
    if(Uniform_Ghost) 
    {
	    // View vector
	    float3 V = normalize(data_in.viewVector);
	    float3 N=normalize(data_in.wNormal);
	
        float g=ghost(dot(N,V))*MATERIAL_GHOST_POWER;
        finalColor*=g+MATERIAL_GHOST_ADD;
		
    }

	//--------------------------------------------------------------------------------------------------
	// ALLINONE:PS_GEOMETRY
	//--------------------------------------------------------------------------------------------------
	
	return finalColor;
}


