//--------------------------------------------------------------------------------------------------
// ALLINONE:PixelShader
//--------------------------------------------------------------------------------------------------
struct PS_IN
{
	float4 texCoord0				: TEXCOORD0;
	float4 texCoord1				: TEXCOORD1;	// Para ShadowMap
	float3 eyeVectorTangentSpace	: TEXCOORD2;
	float4 projTexCoord				: TEXCOORD3;
	float4 screenCoord				: TEXCOORD4;
	float3 tLightVectorForParallax	: TEXCOORD5;
	float4 diffuse					: COLOR0;
	float4 specular					: COLOR1;
};

float4 PShader(PS_IN data_in) : COLOR0
{
	float4 colorFirstPass;

	float2 displaced_texcoord= data_in.texCoord0.xy;
	
	// Para shaders 2.0 necesitamos dar 2 pasadas. Una para calcular el valor del parallax
	// y la otra para calcular la sombra
	if(Uniform_ParallaxShadowOnlyPass)
	{
		float2 parallaxValues = tex2Dproj(ParallaxValues_map, data_in.screenCoord).xy;
		float shadow = ComputeParallaxOcclusionShadowing(normal_map, parallaxValues, data_in.tLightVectorForParallax.xy, float2(1, 0));
		
		return float4(parallaxValues, shadow, 1);
	}
		
	if(Uniform_ParallaxPass)
	{
		float3 parallaxValues;
		
		if(!Uniform_ParallaxOcclusion)
		{
			float height = tex2D(normal_map, displaced_texcoord).a;
			parallaxValues = float3(CalculateParallax(height, displaced_texcoord, data_in.eyeVectorTangentSpace,MATERIAL_PARALLAXAMOUNT),1);
		}
		else
			parallaxValues = CalculateParallaxOcclusion(normal_map, displaced_texcoord, data_in.texCoord0.zw, data_in.tLightVectorForParallax.xy);
			
		return float4(parallaxValues,1);
	}
	
    // Si el paso de la Z no tiene Alpha test, podemos volver con 0, porque no necesitamos
    // analizar las texturas a ver si pasan o no el depth test
	
	// El valor 1,1,1 es el que se utilizar para el WireFrame
    if(Uniform_ZPass==ZPASS_WITHOUTALPHATEST) colorFirstPass=float4(1,1,1,0);
	else
    {
	    // si hay alpha test, debemos buscar el valor del alpha del diffuse
	    // porque eso significa que puede o no pasar la funcin indicada en el material
	    // para el alpha test
	    	
		if(Uniform_diffuseMapEnabled)
		{
			colorFirstPass= tex2D(base_map, displaced_texcoord);
		}
		else
			colorFirstPass = MATERIAL_Diffuse;
	}
	
   // Generamos la posicin Z desde el punto de vista de la luz			
	if(Uniform_GeneratingShadowMap)
	{
		//
		// Depth is z / w
		//
		float4 f = 1-(data_in.texCoord1.z / data_in.texCoord1.w);
		
		// Indicamos el alpha por si tiene activado el alpha test
		f.a=colorFirstPass.a;
		return f;
	}
	
	// Ahora comprobamos si este pixel est en sombra o no segn el
	// valor de la Z generada en el paso de shadowmap
	
	if(Uniform_GeneratingShadowMaskForShadowMap!=0)
	{
		// Ahora tengo el valor de la distancia a la luz
		
		float distancia = 1-(data_in.projTexCoord.z / data_in.projTexCoord.w);
		float distanciaShadowMap;
		
		float x, y, sum, lightAmount;
	
		if(Uniform_GeneratingShadowMaskForShadowMap==2)	// PCF
		{
			int i = 0;		
			float w = ENV_ShadowWidth;
			float h = ENV_ShadowHeight;
			
		    // Recupero el valor XY proyectado
		    float2 shadowXY = data_in.projTexCoord.xy / data_in.projTexCoord.w;
		
			sum = 0;
			
			for(y = -1; y <= 1; y += 1)
			{
				for(x = -1; x <= 0; x += 1)
				{
					distanciaShadowMap=tex2D(projector_map, shadowXY + float2(x/w, y/h));
					sum += (distanciaShadowMap < distancia) ? 1.0f : 0.0f;  
					i++;
				}
			}
		
			lightAmount = sum/i;
		}
		else
		{
			distanciaShadowMap=tex2Dproj(projector_map, data_in.projTexCoord );
			
			// Recordar que la formula es 1-z
			sum = ( distanciaShadowMap> distancia) ? 0.0f : 1.0f;
				
			lightAmount = sum;

		}	
	
		// 1:NO est en sombra ; 0:S est en sombra
				
		// Indicamos el alpha por si tiene activado el alpha test
		return float4(lightAmount,lightAmount,lightAmount,colorFirstPass.a);
	}
	
	return colorFirstPass;

}


