// Phong LUT //////////////////////////////////////////////////

texture phong128_power64;

sampler phong128_power64_LUT =
sampler_state
{
	Texture		= <phong128_power64>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	AddressU	= Clamp;
	AddressV	= Clamp;
};

float4 FillPhong_Power64_LUT( float2 vTexCoord : POSITION,
															float2 vTexelSize : PSIZE) : COLOR
{
	float SpecularPower = 64;

	float NdotL = vTexCoord.x;
	float RdotV = vTexCoord.y;

	return float4(NdotL, NdotL, NdotL, pow(RdotV, SpecularPower));
}

///////////////////////////////////////////////////////////////

// Oren-Nayar LUT //////////////////////////////////////////////////

texture oren_nayar128;

sampler oren_nayar128_LUT =
sampler_state
{
	Texture		= <oren_nayar128>;
	MipFilter	= LINEAR;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	AddressU	= Clamp;
	AddressV	= Clamp;
};

float FillOrenNayar_LUT( float2 dotProducts : POSITION,
												 float2 vTexelSize : PSIZE ) : COLOR
{
   float VdotN = 2.0f * dotProducts.x - 1.0f;
   float LdotN = 2.0f * dotProducts.y - 1.0f;
   
   float angleViewNormal = acos(VdotN);
   float angleLightNormal = acos(LdotN);
   
   float alpha = max(angleViewNormal, angleLightNormal);
   float beta = min(angleViewNormal, angleLightNormal);

   return (sin(alpha) * tan(beta));
}

///////////////////////////////////////////////////////////////