﻿#region Using Directives

using System;
using JadEngine;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Video;
using JadEngine.Input;
using JadScript;

#endregion

/// <summary>
/// Class to handle our standard key events from nearly every application.
/// </summary>
public class StandardKeyEventHandler
{
	#region Properties
	/// <summary>
	/// True to display help on the screen
	/// </summary>
	public bool ShowHelp
	{
		get 
		{
			return JadScriptOptions.GetObject<bool>("ShowHelp");
		}
		set 
		{ 
			JadScriptOptions.SetObject<bool>("ShowHelp",value);
		}
	}
	#endregion

	#region Constructor
	public StandardKeyEventHandler()
	{
		ShowHelp = true;
	}
	#endregion
	
	#region Methods
	
	/// <summary>
	/// Shutdown the application.  
	/// This is triggered by a KeyEvent:
	/// Pressing the Escape key
	/// </summary>
	public void Shutdown()
	{
		JadScriptOptions.Shutdown = true;
	}
	
	/// <summary>
	/// Reload all the scripts for this application.
	/// This is triggered by a KeyEvent:
	/// Pressing the R key
	/// </summary>
	public void ReloadScript()
	{
		JadScriptOptions.Reinitialize = true;
	}

	/// <summary>
	/// Toggle on and off all the gizmo display in the engine.
	/// This is triggered by a KeyEvent:
	/// Pressing the G key
	/// </summary>
	public void ToggleGizmos()
	{
		Jad.Scene.ShowGizmos = !Jad.Scene.ShowGizmos;
	}
	
	/// <summary>
	/// Toggle on and off all the help display.
	/// This is triggered by a KeyEvent:
	/// Pressing the F1 key
	/// </summary>
	public void ToggleHelp()
	{
		ShowHelp = !ShowHelp;
	}
	
	#endregion
}
