#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using JadEngine;
using JadEngine.Core;
using JadEngine.Scene;
using JadEngine.Video;
using JadEngine.Tools;
using JadEngine.Animation;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

#endregion

namespace Collada2Jad
{
    /// <summary>
    /// Converts a mesh collada object into a JMesh object
    /// </summary>
    public class MeshConverter
    {
        #region Fields

        /// <summary>
        /// The geomtery collada object
        /// </summary>
        geometry _geometry;

        /// <summary>
        /// The collada mesh
        /// </summary>
        mesh _meshCollada;

        /// <summary>
        /// The Jad mesh
        /// </summary>
        JMesh _meshJad;


        JMeshObject _meshObject;

        Converter _converter;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="mesh">A valid mesh object</param>
        public MeshConverter(Converter converter,geometry geom)
        {
            _converter = converter;

            _geometry = geom;

            _meshCollada = (mesh)_geometry.Item;            

            _meshObject = Jad.ActiveScene.MeshObjects.Create(true);

            _meshObject.Mesh = _meshJad;

            GetMesh(_meshJad);
        }

        #endregion

        #region Methods

        Vector2[] GetVector2FromArray(float_array fa)
        {
            Vector2[] values = new Vector2[fa.count / 2];

            string text = fa.Text[0];
            string[] textValues = text.Split(' ');
            int a = 0;
            for (uint i = 0; i < fa.count; i += 2, a++)
            {
                values[a].X = float.Parse(textValues[i + 0], Jad.DecimalFormatInfo);
                values[a].Y = float.Parse(textValues[i + 1], Jad.DecimalFormatInfo);
            }

            return values;
        }

        Vector3[] GetVector3FromArray(float_array fa)
        {
            Vector3[] values = new Vector3[fa.count/3];

            string text = fa.Text[0];
            string[] textValues=text.Split(' ');
            int a=0;
            for (uint i = 0; i < fa.count; i+=3,a++)
            {
                values[a].X = float.Parse(textValues[i + 0],Jad.DecimalFormatInfo);
                values[a].Y = float.Parse(textValues[i + 1], Jad.DecimalFormatInfo);
                values[a].Z = float.Parse(textValues[i + 2], Jad.DecimalFormatInfo);
            }

            return values;
        }

        object GetSource(string sourceID)
        {
            // Discards the #
            string newSourceID = sourceID.Substring(1);

            foreach (source src in _meshCollada.source)
            {
                if (src.id == newSourceID)
                {
                    if (src.Item is float_array)
                    {
                        float_array fa = (float_array)src.Item;
                        // Get the number of subelements ( 1,2, 3, 4) If it was a float3, it's a 3
                        ulong stride = src.technique_common.accessor.stride;

                        switch(stride)
                        {
                            case 2:
                                return GetVector2FromArray(fa);

                            case 3:
                                return GetVector3FromArray(fa);

                        }
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// The mesh is builded using a polylist
        /// </summary>
        void GetPolyMesh(polylist polys)
        {
            foreach (InputLocalOffset input in polys.input)
            {
                switch(input.semantic)
                {
                    case "VERTEX":
                        string source=_meshCollada.vertices.input[0].source;
                        Vector3[] position = (Vector3[])GetSource(source);
                        _meshJad.VertexBuffer = new JVertexBuffer(position.Length);
                        _meshJad.VertexBuffer.Position = position;
                        break;
                    case "NORMAL":
                        Vector3[] normal = (Vector3[])GetSource(input.source);
                        _meshJad.VertexBuffer.Normal = normal;
                        break;
                    case "TEXCOORD":
                        Vector2[] texCoord = (Vector2[])GetSource(input.source);
                        Vector2[,] textCoordFinal = new Vector2[1, texCoord.Length];
                        for (int i = 0; i < texCoord.Length; i++)
                        {
                            textCoordFinal[0, i] = texCoord[i];
                        }
                        _meshJad.VertexBuffer.TexCoord = textCoordFinal;

                        break;                
                }
            }

            GetMaterial(polys);
        }

        void GetMesh(JMesh jm)
        {

            //switches for checking if such data is present in the collada file
            bool vertex_is = false;
            bool normal_is = false;
            bool tangent_is = false;
            bool color_is = false;
            bool texture_is = false;

            //offsets and sets
            ulong vertex_offset = 0;
            ulong normal_offset = 0;
            ulong tangent_offset = 0;
            ulong color_offset = 0;
            ulong texture_offset = 0;

            float[] vertex_array = null;
            float[] normal_array = null;
            float[] tangent_array = null;
            float[] color_array = null;
            float[] texture_array = null;

            for (long item = 0; item < _meshCollada.Items.GetLongLength(0); item++)
            {
                if (_meshCollada.Items[item] is polylist)
                {
                    GetPolyMesh((polylist)_meshCollada.Items[item]);
                }
                //if there are triangle items
                if (_meshCollada.Items[item] is triangles)
                {
                    triangles tr = (triangles)_meshCollada.Items[item];
                    for (long input_c = 0; input_c < tr.input.GetLongLength(0); input_c++)
                    {
                        InputLocalOffset ilo = tr.input[input_c];
                        switch (ColladaUtils.GetSemantic(ilo.semantic))
                        {
                            case 21: //vertex
                                {
                                    vertex_is = true;
                                    vertex_offset = ilo.offset;

                                    if (_meshCollada.vertices.id.CompareTo(ilo.source.Remove(0, 1)) == 0)
                                    {
                                        for (long k = 0; k < _meshCollada.vertices.input.GetLongLength(0); k++)
                                        {
                                            switch (ColladaUtils.GetSemantic(_meshCollada.vertices.input[k].semantic))
                                            {
                                                case 15: //position
                                                    {
                                                        string src = _meshCollada.vertices.input[k].source.Remove(0, 1);
                                                        for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                                        {
                                                            if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                                            {
                                                                float_array array = (float_array)_meshCollada.source[j].Item;
                                                                vertex_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                break;
                                                            }
                                                        }
                                                        break;
                                                    }

                                                //NOTE: code for the other vertex data repeats here because it can be in both places (collada stuff) there is no need for separate methods yet
                                                case 12: //normal
                                                    {
                                                        normal_is = true;
                                                        normal_offset = ilo.offset;
                                                        string src = _meshCollada.vertices.input[k].source.Remove(0, 1);
                                                        for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                                        {
                                                            if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                                            {
                                                                float_array array = (float_array)_meshCollada.source[j].Item;
                                                                normal_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                break;
                                                            }
                                                        }
                                                        break;
                                                    }
                                                case 16: //tangent
                                                    {
                                                        tangent_is = true;
                                                        tangent_offset = ilo.offset;
                                                        string src = _meshCollada.vertices.input[k].source.Remove(0, 1);
                                                        for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                                        {
                                                            if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                                            {
                                                                float_array array = (float_array)_meshCollada.source[j].Item;
                                                                tangent_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                break;
                                                            }
                                                        }
                                                        break;
                                                    }
                                                case 1: //color (diffuse)
                                                    {
                                                        color_is = true;
                                                        color_offset = ilo.offset;
                                                        string src = _meshCollada.vertices.input[k].source.Remove(0, 1);
                                                        for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                                        {
                                                            if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                                            {
                                                                float_array array = (float_array)_meshCollada.source[j].Item;
                                                                color_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                break;
                                                            }
                                                        }
                                                        break;
                                                    }
                                                case 18: //tex coord
                                                    {
                                                        texture_is = true;
                                                        texture_offset = ilo.offset;
                                                        string src = _meshCollada.vertices.input[k].source.Remove(0, 1);
                                                        for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                                        {
                                                            if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                                            {
                                                                float_array array = (float_array)_meshCollada.source[j].Item;
                                                                texture_array = ColladaUtils.StringToFloat(array.Text[0]);
                                                                break;
                                                            }
                                                        }
                                                        break;
                                                    }
                                            }
                                        }
                                        break;
                                    }
                                    break;
                                }
                            case 12: //normal
                                {
                                    normal_is = true;
                                    normal_offset = ilo.offset;
                                    string src = ilo.source.Remove(0, 1);
                                    for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                    {
                                        if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                        {
                                            float_array array = (float_array)_meshCollada.source[j].Item;
                                            normal_array = ColladaUtils.StringToFloat(array.Text[0]);
                                            break;
                                        }
                                    }
                                    break;
                                }
                            case 16: //tangent
                                {
                                    tangent_is = true;
                                    tangent_offset = ilo.offset;
                                    string src = ilo.source.Remove(0, 1);
                                    for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                    {
                                        if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                        {
                                            float_array array = (float_array)_meshCollada.source[j].Item;
                                            tangent_array = ColladaUtils.StringToFloat(array.Text[0]);
                                            break;
                                        }
                                    }
                                    break;
                                }
                            case 1: //color (diffuse)
                                {
                                    color_is = true;
                                    color_offset = ilo.offset;
                                    string src = ilo.source.Remove(0, 1);
                                    for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                    {
                                        if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                        {
                                            float_array array = (float_array)_meshCollada.source[j].Item;
                                            color_array = ColladaUtils.StringToFloat(array.Text[0]);
                                            break;
                                        }
                                    }
                                    break;
                                }
                            case 18: //tex coord
                                {
                                    texture_is = true;
                                    texture_offset = ilo.offset;
                                    string src = ilo.source.Remove(0, 1);
                                    for (long j = 0; j < _meshCollada.source.GetLongLength(0); j++)
                                    {
                                        if (_meshCollada.source[j].id.CompareTo(src) == 0)
                                        {
                                            float_array array = (float_array)_meshCollada.source[j].Item;
                                            texture_array = ColladaUtils.StringToFloat(array.Text[0]);
                                            break;
                                        }
                                    }
                                    break;
                                }

                        }
                    }//end for
                    //allocating memory
                    int[] ind = ColladaUtils.StringToInt(tr.p);

                    jm.VertexBuffer = new JVertexBuffer(true);
                    jm.IndexBuffer = new JIndexBuffer();

                    //indices

                    jm.IndexBuffer.Indices = new int[tr.count * 3];

                    //preprocessing

                    List<long> index = new List<long>();
                    List<Vector3> position = new List<Vector3>();
                    List<Vector3> normal = new List<Vector3>();
                    List<Vector3> tangent = new List<Vector3>();
                    List<Vector3> color = new List<Vector3>();
                    List<Vector2> texture = new List<Vector2>();

                    try
                    {
                        for (ulong i = 0; i < tr.count * 3; i++)
                        {
                            index.Add(-1);
                        }
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                    }
                    //computing step

                    ulong step = 0; //used for parsing collada index, will be computed using the greatest offset
                    { //logical block
                        if (vertex_offset > step) step = vertex_offset;
                        if (normal_offset > step) step = normal_offset;
                        if (tangent_offset > step) step = tangent_offset;
                        if (color_offset > step) step = color_offset;
                        if (texture_offset > step) step = texture_offset;
                        step++; //because offset is zero based
                    }

                    //process the index
                    long count = -1;
                    List<int> offsets = new List<int>();
                    for (ulong i = 0; i < tr.count * 3; i++)
                    {
                        //if unprocessed
                        if (index[Convert.ToInt32(i)] == -1)
                        {
                            offsets.Clear();
                            //copy element indices
                            for (ulong j = 0; j < step; j++)
                            {
                                offsets.Add(ind[i * step + j]);
                            }

                            //copy data into the right place

                            //position
                            if (vertex_is)
                            {
                                Vector3 pos = new Vector3(
                                    vertex_array[offsets[(int)vertex_offset] * 3],
                                    vertex_array[offsets[(int)vertex_offset] * 3 + 1],
                                    vertex_array[offsets[(int)vertex_offset] * 3 + 2]);
                                position.Add(pos);
                            }

                            //normal
                            if (normal_is)
                            {
                                Vector3 norm = new Vector3(
                                    normal_array[offsets[(int)normal_offset] * 3],
                                    normal_array[offsets[(int)normal_offset] * 3 + 1],
                                    normal_array[offsets[(int)normal_offset] * 3 + 2]);
                                normal.Add(norm);
                            }

                            //tangent
                            if (tangent_is)
                            {
                                Vector3 tang = new Vector3(
                                    tangent_array[offsets[(int)tangent_offset] * 3],
                                    tangent_array[offsets[(int)tangent_offset] * 3 + 1],
                                    tangent_array[offsets[(int)tangent_offset] * 3 + 2]);
                                tangent.Add(tang);
                            }

                            //color
                            if (color_is)
                            {
                                Vector3 col = new Vector3(
                                    color_array[offsets[(int)color_offset] * 3],
                                    color_array[offsets[(int)color_offset] * 3 + 1],
                                    color_array[offsets[(int)color_offset] * 3 + 2]);
                                color.Add(col);
                            }

                            //texture
                            if (texture_is)
                            {
                                Vector2 tex = new Vector2(
                                    texture_array[offsets[(int)texture_offset] * 2],
                                    1.0f - texture_array[offsets[(int)texture_offset] * 2 + 1]);
                                texture.Add(tex);
                            }

                            //fill index
                            count++;
                            index[(int)i] = count;

                            //check for other elements that have the same indices (they are identical with this one)
                            for (ulong j = i + 1; j < tr.count * 3; j++)
                            {
                                bool match = true;
                                for (ulong k = 0; k < step; k++)
                                {
                                    if (offsets[(int)k] != ind[j * step + k])
                                    {
                                        match = false;
                                        break;
                                    }
                                }

                                //if found, the same index is attached
                                if (match && index[(int)j] == -1)
                                {
                                    index[(int)j] = count;
                                }
                            }
                        }
                    }

                    //increment count to reflect the actual number of elements we have now
                    count++;

                    //vertices are included in the conditions.txt, they must be there
                    if (vertex_is)
                    {
                        jm.VertexBuffer.Position = new Vector3[count];
                    }

                    if (normal_is)
                    {
                        jm.VertexBuffer.Normal = new Vector3[count];
                    }

                    if (tangent_is)
                    {
                        jm.VertexBuffer.Tangent = new Vector3[count];
                    }
                    if (color_is)
                    {
                        jm.VertexBuffer.Diffuse = new int[count];
                    }
                    if (texture_is)
                    {
                        jm.VertexBuffer.TexCoord = new Vector2[1, count];
                    }

                    //filling data

                    //vertex data
                    for (ulong i = 0; i < (ulong)count; i++)
                    {
                        if (vertex_is) jm.VertexBuffer.Position[i] = position[(int)i];
                        if (normal_is) jm.VertexBuffer.Normal[i] = normal[(int)i];
                        if (tangent_is) jm.VertexBuffer.Tangent[i] = tangent[(int)i];
                        if (color_is) jm.VertexBuffer.Diffuse[i] = new JColor(color[(int)i].X, color[(int)i].Y, color[(int)i].Z).ToArgb();
                        if (texture_is) jm.VertexBuffer.TexCoord[0, i] = texture[(int)i];
                    }

                    //index data
                    for (ulong i = 0; i < tr.count * 3; i++)
                    {
                        jm.IndexBuffer.Indices[i] = (int)index[(int)i];
                    }

                    jm.NormalsComputed = true;
                    jm.VertexBuffer.NumberOfVertices = (int)count;
                    jm.IndexBuffer.NumberOfIndices = (int)(tr.count * 3);

                    Jad.Video.Meshes.Add(jm);

                    JMeshObject jmo = new JMeshObject(false, Jad.ActiveScene.MeshObjects);
					jmo.Mesh = jm;
                    Jad.ActiveScene.MeshObjects.Add(jmo);
                }
            }

        }

        void GetMaterial(polylist polys)
        {
            string materialName = polys.material;

            JMaterial materialJad = Jad.ActiveScene.Materials[materialName];

            _meshObject.Materials.Add(materialJad);

            library_materials materials = _converter.lib_mat;

            foreach(material mat in materials.material)
            {
            }
        }

        #endregion

    }
}
