#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using JadEngine.VFS.Storage.Filters;

#endregion

namespace JadePackager
{
	/// <summary>
	/// This dialog displays a list of configured filters and allows
	/// to select which ones will be applied to a file.
	/// </summary>
	public partial class FilterSelectionDialog : Form
	{
		#region Properties

		/// <summary>
		/// Gets the list of selected filters.
		/// </summary>
		public List<String> SelectedFilters
		{
			get
			{
				List<String> filters;

				filters = new List<string>();

				foreach (object filterObject in lbSelectedFilters.Items)
					filters.Add(filterObject.ToString());

				return filters;
			}
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor.
		/// </summary>
		public FilterSelectionDialog()
		{
			InitializeComponent();
		}

		#endregion

		#region Methods

		/// <summary>
		/// Adds a list of configured filters.
		/// </summary>
		/// <param name="filters">The filters to add.</param>
		internal void SetConfiguredFilters(List<IJFilter> filters)
		{
			foreach (IJFilter filter in filters)
				lbFilters.Items.Add(filter.LongName);
		}

		/// <summary>
		/// Adds a list of filters from a file.
		/// </summary>
		/// <param name="filters">The filters to add.</param>
		internal void SetFileFilters(List<IJFilter> filters)
		{
			foreach (IJFilter filter in filters)
			    lbSelectedFilters.Items.Add(filter.LongName);
		}

		#endregion

		#region Event Handlers

		/// <summary>
		/// Event raised when the user double clicks on the configured filters listview.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void lbFilters_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			String selectedFilter;
			bool found;

			if (lbFilters.SelectedItem != null)
			{
				selectedFilter = lbFilters.SelectedItem.ToString();

				found = false;
				foreach (object item in lbSelectedFilters.Items)
					if (item.ToString().Equals(selectedFilter))
					{
						found = true;
						break;
					}

				if (!found)
					lbSelectedFilters.Items.Add(selectedFilter);
			}
		}

		/// <summary>
		/// Event raised when the user double clicks on the selected filters listview.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void lbSelectedFilters_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			if (lbSelectedFilters.SelectedItem != null)
				lbSelectedFilters.Items.Remove(lbSelectedFilters.SelectedItem);
		}

		/// <summary>
		/// Event raised when the user clicks on the Up button.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void btnUp_Click(object sender, EventArgs e)
		{
			int index;
			object item;

			index = lbSelectedFilters.SelectedIndex;
			if (index != -1 && index != 0)
			{
				item = lbSelectedFilters.SelectedItem;

				lbSelectedFilters.Items.Remove(item);
				lbSelectedFilters.Items.Insert(index - 1, item);
			}
		}

		/// <summary>
		/// Event raised when the user clicks on the Down button.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void btnDown_Click(object sender, EventArgs e)
		{
			int index;
			object item;

			index = lbSelectedFilters.SelectedIndex;
			if (index != -1 && index != lbSelectedFilters.Items.Count - 1)
			{
				item = lbSelectedFilters.SelectedItem;

				lbSelectedFilters.Items.Remove(item);
				lbSelectedFilters.Items.Insert(index + 1, item);
			}
		}

		/// <summary>
		/// Event raised when the user clicks on the OK button.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void btnOK_Click(object sender, EventArgs e)
		{
			Close();
		}

		/// <summary>
		/// Event raised when the user clicks on the Cancel button.
		/// </summary>
		/// <param name="sender">Sender object.</param>
		/// <param name="e">Arguments of the event.</param>
		private void btnCancel_Click(object sender, EventArgs e)
		{
			Close();
		}

		#endregion
	}
}
