#region Using Statements

using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

using Jade.Ai.Graphs.Implementations;

#endregion

namespace AITestApplication.Graphs
{
	public static class AdjacencyListBuilder
	{
		#region Methods

		public static AdjacencyList<Vertex, Edge> CreateBFSorDFSExample(string filename)
		{
			AdjacencyList<Vertex, Edge> graph = new AdjacencyList<Vertex,Edge>(true);

			using (StreamReader reader = new StreamReader(filename))
			{
				string[] lines = reader.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

				Dictionary<string, Vertex> vertexLabels = new Dictionary<string, Vertex>();
				Dictionary<Vertex, List<string>> vertexEdges = new Dictionary<Vertex, List<string>>();

				foreach (string line in lines)
				{

					List<string> elements = new List<string>(line.Split(new string[] { " " }, StringSplitOptions.RemoveEmptyEntries));

					if (elements.Count > 0)
					{
						Vertex v = new Vertex(elements[0]);
						graph.AddVertex(v);

						vertexLabels.Add(elements[0], v);

						elements.RemoveAt(0);
						vertexEdges.Add(v, elements);
					}
				}

				foreach (Vertex v in vertexEdges.Keys)
				{
					List<string> edgesTargets = vertexEdges[v];

					foreach (string edgeTarget in edgesTargets)
						graph.AddEdge(new Edge(v, vertexLabels[edgeTarget]));
				}
			}

			return graph;
		}

		public static AdjacencyList<HeuristicVertex, WeightedEdge> CreateDijkstraExample(string filename)
		{
			AdjacencyList<HeuristicVertex, WeightedEdge> graph = new AdjacencyList<HeuristicVertex, WeightedEdge>(true);

			using (StreamReader reader = new StreamReader(filename))
			{
				string[] lines = reader.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

				Dictionary<string, HeuristicVertex> vertexLabels = new Dictionary<string, HeuristicVertex>();
				Dictionary<HeuristicVertex, List<string>> vertexEdges = new Dictionary<HeuristicVertex, List<string>>();

				foreach (string line in lines)
				{

					List<string> elements = new List<string>(line.Split(new string[] { " " }, StringSplitOptions.RemoveEmptyEntries));

					if (elements.Count > 0)
					{
						HeuristicVertex v = new HeuristicVertex(elements[0], new Point());
						graph.AddVertex(v);

						vertexLabels.Add(elements[0], v);

						elements.RemoveAt(0);
						vertexEdges.Add(v, elements);
					}
				}

				foreach (HeuristicVertex v in vertexEdges.Keys)
				{
					List<string> edgesTargets = vertexEdges[v];

					foreach (string edgeTarget in edgesTargets)
					{
						string targetVertexLabel = edgeTarget.Substring(0, edgeTarget.IndexOf("("));
						string edgeWeight = edgeTarget.Substring(edgeTarget.IndexOf("(") + 1, edgeTarget.IndexOf(")") - edgeTarget.IndexOf("(") - 1);

						graph.AddEdge(new WeightedEdge(v, vertexLabels[targetVertexLabel], double.Parse(edgeWeight)));
					}
				}
			}

			return graph;
		}

		#endregion
	}
}
