using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;
using Pluto.Math;

using Jade.Entities;
using Jade.Core;
using Jade.SceneGraph;

namespace EngineTestApplication
{
	/// <summary>
	/// A TextEntity that moves around randomly
	/// </summary>
	class RandomMovingTextEntity : OverlayTextEntity
	{
		static System.Random _random = new Random();

		Vector2 _direction = new Vector2(1f,0f);
		float _lastUpdated = 0f;
		float _updateInterval = 0.1f;
		ITimer _timer;
		

		public RandomMovingTextEntity(ITimer timer, string text, Vector2 position, Color color)
			:base(text,position,color)
		{
			_timer = timer;
		}

		/// <summary>
		/// Called during update.
		/// </summary>
		/// <param name="args">The <see cref="NodeWalkerArgs"/>.  This provides access to the current <see cref="Node"/> being updated</param>
		public override void OnUpdate(NodeWalkerArgs args)
		{
			MoveText();

			if (_timer.Time - _lastUpdated > _updateInterval)
				ChangeDirection();
			
			base.OnUpdate(args);
		}

		private void ChangeDirection()
		{
			_updateInterval = (float) (_random.NextDouble() / 2.0);
			_lastUpdated = _timer.Time;
			_direction.X = (float) ((_random.NextDouble() - 0.5) / 8.0);
			_direction.Y = (float) ((_random.NextDouble() - 0.5) / 8.0);
		}	

		private void MoveText()
		{
			Position += _direction;
			
			if (Position.X < 0)
				_direction.X = Math.Abs(_direction.X);
			else if (Position.X > 200)
				_direction.X = -1f * Math.Abs(_direction.X);

			if (Position.Y < 0)
				_direction.Y = Math.Abs(_direction.Y);
			else if (Position.Y > 200)
				_direction.Y = -1f * Math.Abs(_direction.Y);
		}
	}
}
