//#define SPRITE_SCREEN_SPACE

#region Using Statements

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.IO;
using SD = System.Drawing;
using ST = System.Threading;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Pluto.Core;
using Pluto.Math;
using Pluto.VFS;
using Pluto.Configuration;

using Jade.Core;
using Jade.Entities;
using Jade.SceneGraph;
using Jade.SceneGraph.Transforms;
using Jade.Rendering;
using Jade.Rendering.Utility;
using Jade.Resources;
using Jade.Input;
using Jade.Rendering.States;
using Jade.Rendering.Enums;
using Jade.Import;
#endregion

namespace EngineTestApplication
{
	public partial class TestForm : Form, IPathDiscoveryService
	{
		#region Declarations

		public TestForm()
		{
			InitializeComponent();
		}

		// Our main game object
		Game _game;

		Window _window, _window2;
		RenderState _renderState;

		Node _sceneRoot;
		Node _timingsNode;

		// Sprite info
		Node _spriteRoot;
		Node _spriteRoot2;
		Node _spriteNode;
		Sprite _sprite;
		RotationPosition2D _spriteTransform;
		Node _spriteOmniNode;
		OmniLight _spriteOmni;

		//OrientationPosition _meshTransform;
		Camera _camera;
		Camera _camera2;
		OverlayTextEntity _cameraInfo;
		OverlayTextEntity _camera2Info;

		FirstPersonTransform _cameraTransform;
		OrbitalTransform _camera2Transform;
		Node _light1Node;
		Node _light2Node;
		OrientationPosition _lightTransform;
		AmbientLight _lightAmbient;
		SpotLight _light1;
		SubMesh _cube;
		bool _animateCube = false;
		VertexAction _explode, _sin;
		TimingsEntity _timings;
		Node _statsNode;
		Node _import;

		OverlayTextEntity _mouseStatusEntity;

		EventListener _listener;

		int counter;

		#endregion

		private void TestForm_Load(object sender, EventArgs e)
		{
			_game = new Game("./configuration.xml");
			_game.InitializeGame += new EventHandler(_game_InitializeGame);
			_game.RenderLoop.FrameUpdating += new EventHandler(RenderLoop_FrameUpdating);
			//_game.RenderLoop.RunMultiThreaded = RenderLoopThreadOptions.SingleThread;			

			foreach (string item in _game.Resources.LightingEffects.AvailableResources)
			{
				if (item != "Default")
					this.comboBoxLightFragment.Items.Add(item);
			}

			_listener = new EventListener();
			_listener.KeyPressed += new EventHandler<KeyPressedEventArgs>(_listener_KeyPressed);
			_listener.KeyDown += new EventHandler<Jade.Input.KeyEventArgs>(_listener_KeyDown);
			_listener.KeyUp += new EventHandler<Jade.Input.KeyEventArgs>(_listener_KeyUp);

			_listener.MouseButtonDown += new EventHandler<MouseButtonEventArgs>(_listener_MouseButtonDown);
			_listener.MouseButtonUp += new EventHandler<MouseButtonEventArgs>(_listener_MouseButtonUp);
			//_listener.MouseButtonPressed += new MouseButtonInputEventHandler(_listener_MouseButtonPressed);
			_listener.MouseWheel += new EventHandler<MouseWheelEventArgs>(_listener_MouseWheel);

			// Game handles any threading issues with creating the windows
			// This will guarantee that they are created with the correct threading
			// context for later if we're using a threaded render system
			//string alternativeRenderSystem = "DX9 Fixed Function Renderer";
			string renderSystemName = "DX9 Renderer";

			// If we have more than one adapter, use it for the second window.  
			// NOTE: This is entirely optional, but (at least on Vista) will 
			// dramatically improve performance if multi threaded rendering is
			// enabled
			int adapter2 = 0;
			RenderingSystemFactory renderSystem = _game.RootServiceProvider.GetService<RenderingSystemFactory>(renderSystemName);
			if (renderSystem != null)
			{
				if (renderSystem.RenderingSystem.Adapters.Count > 1)
					adapter2 = 1;
			}

			// Use the simple form here
			_window = _game.CreateWindow(panelRender, _listener);

			//_window2 = _game.CreateWindow(panelRender2, _listener);
			// Define this using the complex form, since we want
			// to allow multiple adapters
			_window2 = _game.CreateWindow(renderSystemName, adapter2, panelRender2.Width, panelRender2.Height, panelRender2.Handle);
			panelRender2.Context = _game;
			panelRender2.Window = _window2;
			_window2.InputState.AddListener(_listener, true);

			_window.ClearColor = Colors.DarkGrey;
			_window2.ClearColor = Colors.DarkGrey;

			//_window2.FillMode = FillMode.WireFrame;

			if ((_window == null) || (_window2 == null))
			{
				MessageBox.Show("The window can not be created.");
				Close();
				return;
			}


			_game.Start();
		}

		void RenderLoop_FrameUpdating(object sender, EventArgs e)
		{
			this._cameraInfo.Text = "Camera look: " + _camera.Transform.Transform.Look.ToString() + "\n";
			this._camera2Info.Text = "Camera2 look: " + _camera2.Transform.Transform.Look.ToString() + "\n";			
		}

		void _listener_MouseButtonPressed(object sender, MouseButtonEventArgs args)
		{
			_mouseStatusEntity.Text = "Button Pressed: " + args.MouseButton.ToString();
		}

		void _listener_MouseWheel(object sender, MouseWheelEventArgs args)
		{
			_mouseStatusEntity.Text = "Wheel Moved: " + args.Delta.ToString();
		}

		void _listener_MouseButtonUp(object sender, MouseButtonEventArgs args)
		{
			_mouseStatusEntity.Text = "";
		}

		void _listener_MouseButtonDown(object sender, MouseButtonEventArgs args)
		{
			_mouseStatusEntity.Text = "Button Down: " + args.MouseButton.ToString();
		}

		void _listener_KeyUp(object sender, Jade.Input.KeyEventArgs args)
		{
			if (args.Key == Key.M)
			{
				if (checkBoxAnimate.InvokeRequired)
				{
					checkBoxAnimate.BeginInvoke(new MethodInvoker(delegate()
					{
						checkBoxAnimate.Checked = false;
					}));
				}
				else
					checkBoxAnimate.Checked = false;
				args.Handled = true;

			}
		}

		void _listener_KeyDown(object sender, Jade.Input.KeyEventArgs args)
		{
			if (args.Key == Key.M)
			{
				if (checkBoxAnimate.InvokeRequired)
				{
					checkBoxAnimate.BeginInvoke(new MethodInvoker(delegate()
					{
						checkBoxAnimate.Checked = true;
					}));
				}
				else
					checkBoxAnimate.Checked = true;

				args.Handled = true;
			}

			OrientationPosition op = this._import.Transform as OrientationPosition;

			switch (args.Key)
			{
				case Key.NumPad6:
					op.Position = new Vector3d(op.Position.X + 1, op.Position.Y, op.Position.Z);
					break;
				case Key.NumPad4:
					op.Position = new Vector3d(op.Position.X - 1, op.Position.Y, op.Position.Z);
					break;
				case Key.NumPad8:
					op.Position = new Vector3d(op.Position.X, op.Position.Y, op.Position.Z + 1);
					break;
				case Key.NumPad2:
					op.Position = new Vector3d(op.Position.X, op.Position.Y, op.Position.Z - 1);
					break;
				case Key.NumPad9:
					op.Position = new Vector3d(op.Position.X, op.Position.Y + 1, op.Position.Z);
					break;
				case Key.NumPad3:
					op.Position = new Vector3d(op.Position.X, op.Position.Y - 1, op.Position.Z);
					break;
			}
		}

		void _listener_KeyPressed(object sender, KeyPressedEventArgs args)
		{
			switch (args.Key)
			{
				case Key.F1:
					_statsNode.Active = !_statsNode.Active;
					// Prevent other listeners from getting this event....
					args.Handled = true;
					break;
				case Key.LeftArrow:
					_mouseStatusEntity.Text = "Left Arrow";
					break;
			}
			_mouseStatusEntity.Text = args.Key.ToString();
			if (args.ShiftDown)
				_mouseStatusEntity.Text += " (Shift)";
			if (args.ControlDown)
				_mouseStatusEntity.Text += " (Control)";
			if (args.AltDown)
				_mouseStatusEntity.Text += " (Alt)";

			_mouseStatusEntity.Text += "  [" + args.Character + "]";
		}

		void _game_InitializeGame(object sender, EventArgs e)
		{
			// Create the scenes' root nodes
			_sceneRoot = new Node();

			Material material = CreateCubes();

			/*for (int i = -3; i < 3; i+=2)
			{
				OrientationPosition trans = new OrientationPosition();
				trans.Position = new Vector3d(3 * i, -3 * i, 30 + 3*i);
				_sceneRoot.AttachChild(new Node(renderable,trans));
			}*/

			// Create the sprite's main info
			_spriteRoot = new Node();
			_spriteRoot2 = new Node();

			//CreateLightSprite();
			//CreateDonutSprite();

			//OBJImporter obj = new OBJImporter(_game);
			//obj.Load(_sceneRoot, new Vector3(0.2f, 0.2f, 0.2f), @"d:\JadeSG\Data\Models\starport.obj");
			
			ObjImporter objImp = new ObjImporter(_game.Resources.VFS, Encoding.UTF7);
			// Importing from Blender (OpenGL)
			objImp.FlipVCoord = true;
            //objImp.FlipTriangleVertexOrder = false;
			//objImp.FlipZAxis = true;			
			//objImp.FlipTriangleVertexOrder = true;
			objImp.LineColor = Colors.Grey;

			this._import = new Node();
			this._import.Transform = new OrientationPosition();

			List<Node> objs = objImp.Import(new Vector3(5,5,5), "Models", "test_cube.obj");
			//Mesh wagner = objs[0].Entity as Mesh;
			//wagner.Meshes[0].RenderState.Materials[0].LightingEffect = _game.Resources.LightingEffects["Anisotropic"];
			//wagner.Meshes[0].RenderState.Materials[0].NormalMap.Enabled = true;
			//wagner.Meshes[0].RenderState.Materials[0].NormalMap.Texture = new Texture(_game.Resources.VFS, "wagner_n.png", "Textures");
			//wagner.Meshes[0].RenderState.Materials[0].SpecularMap.Enabled = false;
			this._import.AttachChildren(objs);
			objs[0].Update += new NodeUpdateDelegate(node_UpdateRenderable);

			objs = objImp.Import(new Vector3(1, 1, 1), "Models", "axes.obj");
			OrientationPosition op = objs[0].Transform as OrientationPosition;
			op.Position = new Vector3d(0, 30, 0);
			this._import.AttachChildren(objs);

			this._sceneRoot.AttachChild(_import);
			DeleDImporter deleD = new DeleDImporter(_game);

			deleD.Load(_sceneRoot, new Vector3(0.2f, 0.2f, 0.2f), _game.Resources.VFS.GetFile("Chapel.dxs", true));
			//deleD.Load(_sceneRoot, new Vector3(0.05f, 0.05f, 0.05f), _game.Resources.VFS.GetFile("testCubeLightmapped.dxs", true));

			//ObjImporter obj = new ObjImporter(_game.Resources.VFS, Encoding.UTF7);
			//Node objNode = obj.Import(new Vector3(10.0f, 10.0f, 10.0f), "models", "test_plane.obj");

			//this._sceneRoot.AttachChild(objNode);

			//MilkshapeImporter milkShape = new MilkshapeImporter(_game);
			//milkShape.Load(_sceneRoot, new Vector3(3f, 3f, 3f), _game.Resources.VFS.GetFile("zombie02.ms3d", true),MilkShapeModelType.Binary);

			CreateOverlays();			

			_camera = new Camera();
			_cameraTransform = new FirstPersonTransform(_window);

			Matrixd matrix = Matrixd.CreateFromYawPitchRoll(0, 0, 0);
			_cameraTransform.Orientation = Quaterniond.CreateFromRotationMatrix(ref matrix);
			_cameraTransform.Position = new Vector3d(0, 0, 80);

			_cameraTransform.Transformers.Add(new FirstPersonKeysInputTransformer());
			_cameraTransform.Transformers.Add(new MouseDragRotateInputTransformer());
			_cameraTransform.ActivateTransformers();

			_camera.Transform = _cameraTransform;
			//_camera.Transform = new OrientationPosition();

			// We need to add this to the scene so that it gets updates
			_sceneRoot.AttachChild(new Node(_cameraTransform));

			_camera2 = new Camera();
			_camera2Transform = new OrbitalTransform(_window2);
			_camera2Transform.Center = new Vector3d(0, 0, 0);
			_camera2Transform.Orientation = Quaterniond.CreateFromRotationMatrix(ref matrix);
			_camera2Transform.Radius = 80;

			_camera2Transform.Transformers.Add(new MouseWheelZoomInputTransformer());
			_camera2Transform.Transformers.Add(new MouseDragRotateInputTransformer());
			_camera2Transform.ActivateTransformers();

			_camera2.Transform = _camera2Transform;
			_sceneRoot.AttachChild(new Node(_camera2Transform));


			// Add 3 lights to the scene
			_lightAmbient = new AmbientLight(new Color(1.0f));
			_sceneRoot.AttachChild(new Node(_lightAmbient));

			// Add the ambient light to the sprite's scene graph as well
			_spriteRoot.AttachChild(new Node(_lightAmbient));

			_lightTransform = new OrientationPosition();
			//_lightTransform.Position = new Vector3d(5, -5, -15);
			//_lightTransform.Position = new Vector3d(0, 0, 0);
			_light1 = new SpotLight(Colors.White);
			//OmniLight _light1 = new OmniLight(Colors.White);
			_light1.Attenuation.Type = AttenuationType.Inverse;
			_light1.Attenuation.Start = 10;
			_light1.Attenuation.End = 80;
			_light1.Radius = 80;
			_light1.InnerConeAngle = 23;
			_light1.OuterConeAngle = 25;
			_light1.Projector.Texture = _game.Resources.Textures["Logo"];
			_light1.Projector.Enabled = true;
			Vector3d look = new Vector3d(0, 1, 0);
			look.Normalize();
			//_lightTransform.Orientation = Quaterniond.CreateFromRotationMatrix(ref matrix);
			_lightTransform.Position = new Vector3d(-10, 1, 0);
			_lightTransform.Orientation = Quaterniond.CreateFromAxisAngle(ref look, System.Math.PI/2);
			_light1Node = new Node(_light1, _lightTransform);

			_sceneRoot.AttachChild(_light1Node);

			objs = objImp.Import(Vector3.One, "Models", "arrow.obj");
			objs[0].Transform = _lightTransform;
			_sceneRoot.AttachChildren(objs);

			_lightTransform = new OrientationPosition();
			_lightTransform.Position = new Vector3d(20, 0, -20);
			OmniLight _light2 = new OmniLight(Colors.Red);
			_light2.Radius = 100;
			_light2Node = new Node(_light2, _lightTransform);
			_light2Node.Active = true;
			_sceneRoot.AttachChild(_light2Node);

			// Fix the UI to match up with our settings
			UpdateUI(material);

			_game.RenderScene("Window1", _sceneRoot, _spriteRoot, _camera, _window);
			_game.RenderScene("Window2", _sceneRoot, _spriteRoot2, _camera2, _window2);
		}

		private void CreateOverlays()
		{
			OverlayTextFont smallFont = new OverlayTextFont(12, "Arial");
			// Add in our info to the overlay scene
			_timings = new TimingsEntity(_game.RenderLoop, new Vector2(5f, 5f), Colors.Yellow, smallFont);
			_timings.Active = false;
			_timingsNode = new Node(_timings);
			_spriteRoot.AttachChild(_timingsNode);

			// Add a FPS node to the overlay scene
			_statsNode = new Node(new StatisticsEntity(_game.RenderLoop));
			_spriteRoot.AttachChild(_statsNode);

			// Add camera info overlay
			_cameraInfo = new OverlayTextEntity("", new Vector2(5f, 36f), Colors.Red, smallFont);
			_spriteRoot.AttachChild(new Node(_cameraInfo));

			// Add camera2 info overlay
			_camera2Info = new OverlayTextEntity("", new Vector2(5f, 36f), Colors.Red, smallFont);
			_spriteRoot2.AttachChild(new Node(_camera2Info));

			_mouseStatusEntity = new OverlayTextEntity("", new Vector2(5f, 18f), Colors.Red, smallFont);
			_spriteRoot.AttachChild(new Node(_mouseStatusEntity));
			_spriteRoot2.AttachChild(new Node(_mouseStatusEntity));
		}

		void CreateSphereScene()
		{
			DeleDImporter deleD = new DeleDImporter(_game);
			deleD.Load(_sceneRoot, 0.015f, _game.Resources.VFS.GetFile("sphere.dxs", true));

			Mesh sphere = deleD.Meshes[0];

			Material material = sphere.Meshes[0].RenderState.Materials[0];

			material.NormalMap.Texture = new Texture(_game.Resources.VFS, "brick_n.tga", @"Textures");
			material.NormalMap.Enabled = true;

		}

		private Material CreateCubes()
		{
			// Create a rectangle and position it at Z=40 
			_cube = Primitive.CreateCube(10, 10, 10);

			//Primitive.CreateRectangle(5, 5, 2, 2, RectanglePlane.XY, false, 1f, 1f);
			Material material = new Material();
			// The ["Default"] lighting effect goes in by default.  You can specify
			// it explicitly, or use another by name
			//material.LightingEffect = _game.Resources.LightingEffects["Default"];
			//material.LightingEffect = _game.Resources.LightingEffects["Blinn-Phong"];
			// Use the global "Brick_Diffuse" texture as specified in configuration
			material.DiffuseMap.Texture = _game.Resources.Textures["Brick_Diffuse"];
			//material.DiffuseMap.Texture = Jad.Engine.Textures["White"];
			material.NormalMap.Texture = _game.Resources.Textures["Brick_Normal"];
			material.SpecularMap.Enabled = true;
			material.NormalMap.Enabled = true;

			// We should want to add an explode modificador
			_explode = _game.Resources.VertexActions["Explode"];
			//	new VertexAction("position.xyz+=normal*explodeFactor;"); //env.TimeElapsed
			//explode.Variables.Add(new ShaderVariable("explodeFactor", ShaderVariable.ShaderVariableType.Float, 1.0f));
			_sin = _game.Resources.VertexActions["SinY"];

			// Create our render state with our shader + materials
			_renderState = new RenderState(material);
			//_renderState.Actions.Add(explode);

			// Create a renderable object (Mesh)
			Mesh renderable = new Mesh(_cube, _renderState);

			Vector3d axis = new Vector3d(0, 0, 1);
			// Create a node to hold the tree of renderable objects in the scene graph
			Node bricks = new Node();
			_sceneRoot.AttachChild(bricks);

			const int numRenderables = 0;
			// Create a series of cubes
			double delta = (2.0 * Math.PI) / numRenderables;
			for (int i = 0; i < numRenderables; ++i)
			{
				double radians = delta * i;
				double cos = Math.Cos(radians) * 30.0;
				double sin = Math.Sin(radians) * 30.0;

				OrientationPosition transform = new OrientationPosition();
				transform.Position = new Vector3d(cos, sin, 0);
				transform.Orientation = Quaterniond.CreateFromAxisAngle(ref axis, (double) i * (System.Math.PI / (double) numRenderables));

				Node node = new Node(renderable, transform);
				node.Update += new NodeUpdateDelegate(node_UpdateRenderable);
				bricks.AttachChild(node);
			}
			return material;
		}

		void CreateLightSprite()
		{

			// Create a second material so changes to the main cube don't effect
			// the sprite
			Material spriteMaterial = new Material();
			spriteMaterial.LightingEffect = _game.Resources.LightingEffects["Blinn-Phong"];
			// Use the global "Brick_Diffuse" texture as specified in configuration
			spriteMaterial.DiffuseMap.Texture = _game.Resources.Textures["Donuts"];

			// We want the black color be transparent
			spriteMaterial.DiffuseMap.Texture.TransparentKeyColor = Colors.Black;

			//material.DiffuseMap.Texture = Jad.Engine.Textures["White"];
			spriteMaterial.NormalMap.Texture = _game.Resources.Textures["Brick_Normal"];
			spriteMaterial.SpecularMap.Enabled = true;
			spriteMaterial.NormalMap.Enabled = false;

#if SPRITE_SCREEN_SPACE

            // as the sprite has screen space coordinates, we define the size in ScreenSpace ( 100x100)
            SubMesh spriteMesh = Primitive.CreateRectangle(100, 100, 2, 2, RectanglePlane.XY, true, 1.0f, 1.0f);

            _sprite = new Sprite(spriteMesh, new RenderState(spriteMaterial));

            _sprite.CoordinateSystem = CoordinateSystem.Screen;

            _spriteTransform = new RotationPosition2D();
            _spriteTransform.Position2D = new Vector2(panelRender2.Width / 2, panelRender2.Height / 2);
            _spriteTransform.RotationDegrees = 45;

            OrientationPosition _spriteLightTransform = new OrientationPosition();
            _spriteLightTransform.Position = new Vector3d(panelRender2.Width / 2, panelRender2.Height / 2, 20);

#else

			SubMesh spriteMesh = Primitive.CreateRectangle(0.5f, 0.5f, 2, 2, RectanglePlane.XY, true, 1.0f, 1.0f);

			_sprite = new Sprite(spriteMesh, new RenderState(spriteMaterial));

			_sprite.CoordinateSystem = CoordinateSystem.Clip;

			_spriteTransform = new RotationPosition2D();
			_spriteTransform.Position2D = new Vector2(.25f, 0);
			_spriteTransform.PivotPoint = new Vector2(-.25f, 0f);
			_spriteTransform.RotationDegrees = 0;

			OrientationPosition _spriteLightTransform = new OrientationPosition();
			_spriteLightTransform.Position = new Vector3d(1, 0, -10);

#endif
			Material mat = _sprite.RenderState.Materials[0];
			mat.DiffuseMap.Color.Alpha = 0.6f;
			mat.BlendState.BlendEnable[0] = true;
			mat.BlendState.SrcBlend = BlendOption.SrcAlpha;
			mat.BlendState.DestBlend = BlendOption.InvSrcAlpha;
			mat.BlendState.BlendOp = BlendOperation.Add;

			// Add in the node
			_spriteNode = new Node(_sprite, _spriteTransform);
			_spriteNode.Update += new NodeUpdateDelegate(_spriteNode_Update);
			_spriteRoot.AttachChild(_spriteNode);

			//return;

			// Add in a omni light
			_spriteOmni = new OmniLight();
			_spriteOmni.Color = Colors.Blue;
			_spriteOmni.Attenuation.Type = AttenuationType.None;
			_spriteOmniNode = new Node(_spriteOmni, _spriteLightTransform);
			_spriteRoot.AttachChild(_spriteOmniNode);
			_spriteOmni.Radius = 2000;

			// Add a second light
			_spriteOmni = new OmniLight();
			_spriteOmni.Color = Colors.White;
			OrientationPosition _spriteLightTransform2 = new OrientationPosition();
			_spriteLightTransform2.Position = _spriteLightTransform.Position - new Vector3d(5, 0, 0);
			_spriteOmniNode = new Node(_spriteOmni, _spriteLightTransform2);
			_spriteRoot.AttachChild(_spriteOmniNode);
		}

		void CreateDonutSprite()
		{

			// Create the sprite's main info
			//_spriteRoot = new Node();

			// Create a second material so changes to the main cube don't effect
			// the sprite
			Material spriteMaterial = new Material();
			spriteMaterial.LightingEffect = _game.Resources.LightingEffects["Blinn-Phong"];

			spriteMaterial.DiffuseMap.Texture = _game.Resources.Textures["Donuts"];

			// This texture has a lot of frames inside. 
			// We inform the material about each frame, and inform it to get the needed one

			ImageMap clips = new ImageMap();

			// We have 4 rows of 8 sprites
			for (int y = 0; y < 4; y++)
			{
				for (int x = 0; x < 8; x++)
				{
					Rectangle rect = new Rectangle();
					rect.X = 32 * x;
					rect.Y = 32 * y;
					rect.Width = 32;
					rect.Height = 32;
					clips.Add(rect);
				}
			}

			// Link the clips with the bitmap
			spriteMaterial.DiffuseMap.Texture.ClipBitmap = clips;

			// as the sprite has screen space coordinates, we define the size in ScreenSpace ( 100x100)
			SubMesh spriteMesh = Primitive.CreateRectangle(100, 100, 2, 2, RectanglePlane.XY, true, 1.0f, 1.0f);

			_sprite = new Sprite(spriteMesh, new RenderState(spriteMaterial));

			_sprite.CoordinateSystem = CoordinateSystem.Screen;

			_sprite.AffectedByLights = false;

			_spriteTransform = new RotationPosition2D();
			_spriteTransform.Position2D = new Vector2(panelRender2.Width / 4, 100);
			_spriteTransform.RotationDegrees = 0;

			Material mat = _sprite.RenderState.Materials[0];
			mat.DiffuseMap.Color.Alpha = 1f;
			mat.BlendState.BlendEnable[0] = true;
			mat.BlendState.SrcBlend = BlendOption.SrcAlpha;
			mat.BlendState.DestBlend = BlendOption.InvSrcAlpha;
			mat.BlendState.BlendOp = BlendOperation.Add;

			// Add in the node
			_spriteNode = new Node(_sprite, _spriteTransform);
			_spriteNode.Update += new NodeUpdateDelegate(_spriteNode_Update);
			_spriteRoot.AttachChild(_spriteNode);

		}

		delegate void UpdDelegate(Material mat);

		private void UpdateUI(Material material)
		{
			if (this.InvokeRequired)
			{
				this.BeginInvoke(new UpdDelegate(UpdateUI), material);
			}
			else
			{
				checkBoxLight2Enable.Checked = _light2Node.Active;
				checkBoxAnimate.Checked = _animateCube;
				checkBoxTimingsActive.Checked = _timings.Active;
				checkBoxProjectorEnable.Checked = _light1.Projector.Enabled;
				checkBoxNormalEnable.Checked = material.NormalMap.Enabled;
				checkBoxSpecularEnable.Checked = material.SpecularMap.Enabled;
				numericUpDownSpecularPower.Value = (decimal) material.SpecularMap.Power;
				numericUpDownNormalBump.Value = (decimal) material.NormalMap.Bumpiness;
				labelSpecularColorView.BackColor = material.SpecularMap.Color.ToSystemColor();
				labelAmbientColorView.BackColor = _lightAmbient.Color.ToSystemColor();
				labelLight1ColorView.BackColor = _light1.Color.ToSystemColor();
				numericUpDownAttStart.Value = (decimal) _light1.Attenuation.Start;
				numericUpDownAttEnd.Value = (decimal) _light1.Attenuation.End;
				numericUpProjectorNearPlane.Value = (decimal) _light1.Projector.ApertureFactor;

				comboBoxLightFragment.SelectedIndex = 0;
			}
		}


		/// <summary>
		/// Spin the cube
		/// </summary>
		void node_UpdateRenderable(object sender, NodeUpdateEventArgs args)
		{
			/*const double speed = 20;
			double z = speed * _resources.Timer.Time;
			z = z % 80;
			if (z > 40)
				z = 80 - z;*/
			double z = -20;

			Vector3d axis = new Vector3d(0, 1, 0);
			axis.Normalize();
			_cube.BoundingInvalidated = true;
			// Spin the cube

			if (_animateCube)
			{
				++counter;
				if (counter > 500)
					counter = -500;
				OrientationPosition transform = args.Node.Transform as OrientationPosition;
				if (transform != null)
				{
					//transform.Position = new Vector3d(counter / 100.0, 0, 30);
					transform.Orientation = Quaterniond.CreateFromAxisAngle(ref axis, Jad.Timer.Time / 2);
					//_meshTransform.Position = new Vector3d(counter / 100.0, 0, 30);
					//_meshTransform.Orientation = Quaterniond.CreateFromAxisAngle(ref axis, Jad.Timer.Time / 2);
				}
				//_spriteTransform.Position = new Vector3d(0, 0, z);
			}
		}

		int donutFrame = 0;
		float lastTime = 0;

		/// <summary>
		/// Handles the Update event of the _spriteNode node
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="args">The <see cref="Jade.SceneGraph.NodeUpdateEventArgs"/> instance containing the event data.</param>
		void _spriteNode_Update(object sender, NodeUpdateEventArgs args)
		{
			if (_animateCube)
				_spriteTransform.Rotation = Jad.Timer.Time;

			if (checkBoxAnimDonuts.Checked)
			{
				// Assign the clip number 0 to the sprite0
				_sprite.AssignCoordstoFrame(donutFrame);

				if (lastTime > 0)
				{
					if (Jad.Timer.Time - lastTime > 1f / 30f)
					{
						donutFrame++;
						if (donutFrame > 29) donutFrame = 0;
						lastTime = Jad.Timer.Time;
					}
				}
				else lastTime = Jad.Timer.Time;


			}
		}


		// Turn off our Timings node
		private void checkBoxTimingsActive_CheckedChanged(object sender, EventArgs e)
		{
			_timings.Active = checkBoxTimingsActive.Checked;
		}



		private void TestForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			// Shutdown our game
			_game.Dispose();
		}

		private void buttonClosePanel2_Click(object sender, EventArgs e)
		{
			_game.RemoveScene("Window2");
			if (_window2 != null)
			{
				_window2.Dispose();
				_window2 = null;
			}
		}

		private void buttonClosePanel1_Click(object sender, EventArgs e)
		{
			_game.RemoveScene("Window1");
			if (_window != null)
			{
				_window.Dispose();
				_window = null;
			}
		}
		private void TestForm_Deactivate(object sender, EventArgs e)
		{
			_game.UpdateSpeed = UpdateSpeed.Slow;
		}

		private void TestForm_Activated(object sender, EventArgs e)
		{
			_game.UpdateSpeed = UpdateSpeed.Aggressive;
		}

		private void TestForm_Resize(object sender, EventArgs e)
		{
			if (this.WindowState == FormWindowState.Minimized)
			{
				_game.Pause();
				if (_window != null)
					_window.Deactivate();
				if (_window2 != null)
					_window2.Deactivate();
			}
			else
			{
				_game.Start();
				if (_window != null)
					_window.Activate();
				if (_window2 != null)
					_window2.Activate();
			}
		}

		private void checkBoxSpecular_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate
			{
				_renderState.Materials[0].SpecularMap.Enabled = checkBoxSpecularEnable.Checked;
				_renderState.Invalidate();
			});
		}


		#region IPathDiscoveryService Members

		public string Path
		{
			get { return "..\\.."; }
		}

		#endregion

		private void checkBoxNormal_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_renderState.Materials[0].NormalMap.Enabled = checkBoxNormalEnable.Checked;
				_renderState.Invalidate();
			});


		}

		private void numericUpDownSpecularPower_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_renderState.Materials[0].SpecularMap.Power = (float) numericUpDownSpecularPower.Value;
				_renderState.Invalidate();
			});
		}

		private void numericUpDown1_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_renderState.Materials[0].NormalMap.Bumpiness = (float) numericUpDownNormalBump.Value;
				_renderState.Invalidate();
			});
		}

		private void labelLightColor1_Click(object sender, EventArgs e)
		{
			ColorDialog color = new ColorDialog();
			Light _light1 = _light1Node.Entity as Light;
			color.Color = _light1.Color.ToSystemColor();
			if (color.ShowDialog() == DialogResult.OK)
			{
				labelLight1ColorView.BackColor = color.Color;
				Color newColor = new Color(color.Color);
				_game.BeginInvoke(delegate()
				{

					_light1.Color = newColor;
					_light1.Invalidate();
				});
			}
		}

		private void checkBoxLight1Enabled_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_light1Node.Active = checkBoxLight1Enable.Checked;
			});
		}

		private void numericUpDownAttStart_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				Light _light1 = _light1Node.Entity as Light;
				_light1.Attenuation.Start = (float) numericUpDownAttStart.Value;
				_light1.Invalidate();
			});
		}

		private void numericUpDownAttEnd_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				Light _light1 = _light1Node.Entity as Light;
				_light1.Attenuation.End = (float) numericUpDownAttEnd.Value;
				_light1.Invalidate();
			});
		}

		private void labelAmbientColor_Click(object sender, EventArgs e)
		{
			ColorDialog color = new ColorDialog();
			color.Color = _lightAmbient.Color.ToSystemColor();
			if (color.ShowDialog() == DialogResult.OK)
			{
				labelAmbientColorView.BackColor = color.Color;

				_game.BeginInvoke(delegate()
				{
					_lightAmbient.Color = new Color(color.Color);
					_lightAmbient.Invalidate();
				});
			}
		}

		private void checkBoxLight2Enabled_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_light2Node.Active = checkBoxLight2Enable.Checked;
			});
		}

		private void label11_Click(object sender, EventArgs e)
		{
			ColorDialog color = new ColorDialog();
			color.Color = _renderState.Materials[0].SpecularMap.Color.ToSystemColor();
			if (color.ShowDialog() == DialogResult.OK)
			{
				_game.BeginInvoke(delegate()
				{
					labelSpecularColorView.BackColor = color.Color;

					_renderState.Materials[0].SpecularMap.Color = new Color(color.Color);
					_renderState.Invalidate();
				});
			}
		}

		private void checkBoxTiling_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				if (checkBoxNormalTiling.Checked)
				{
					_renderState.Materials[0].NormalMap.Channel.UseDiffuseChannel = false;
					_renderState.Materials[0].NormalMap.Channel.UTile = 2f;
					_renderState.Materials[0].NormalMap.Channel.VTile = 2f;
					_renderState.Invalidate();
				}
				else
				{
					_renderState.Materials[0].NormalMap.Channel.UseDiffuseChannel = true;
					_renderState.Invalidate();
				}
			});
		}

		private void checkBoxAnimate_CheckedChanged(object sender, EventArgs e)
		{
			_animateCube = checkBoxAnimate.Checked;
		}

		private void checkBoxProjector_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_light1.Projector.Enabled = checkBoxProjectorEnable.Checked;
				_light1.Invalidate();
			});
		}


		private void numericUpProjectorNearPlane_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_light1.Projector.ApertureFactor = (float) numericUpProjectorNearPlane.Value;
				_light1.Invalidate();
			});
		}

		private void texture_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				if (radioButtonBrick.Checked)
					_renderState.Materials[0].DiffuseMap.Texture = _game.Resources.Textures["Brick_Diffuse"];
				else
					_renderState.Materials[0].DiffuseMap.Texture = _game.Resources.Textures["White"];
				_renderState.Invalidate();
			});
		}

		private void checkBoxThread_CheckedChanged(object sender, EventArgs e)
		{
			if (checkBoxThread.Checked)
				_game.RenderLoop.RunMultiThreaded = RenderLoopThreadOptions.ThreadedCull | RenderLoopThreadOptions.ThreadedRender;
			else
				_game.RenderLoop.RunMultiThreaded = RenderLoopThreadOptions.SingleThread;
		}

		private void checkBoxExplode_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				if (checkBoxExplode.Checked)
				{
					_renderState.Actions.Add(_explode);
					numericUpDownExplode_ValueChanged(null, null);
					// We need to invalidate the render state, because we changed the actions list
					// This will cause the shaders to be rebuilt
					_renderState.Invalidate();
				}
				else
				{
					_renderState.Actions.Remove(_explode);
					_renderState.Invalidate();
				}
			});
		}

		private void numericUpDownExplode_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				// Update the shader variable here
				_cube.SetShaderVariable<float>("explodeFactor", (float) numericUpDownExplode.Value);
			});
		}

		private void checkBoxSin_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				if (checkBoxSin.Checked)
				{
					_renderState.Actions.Add(_sin);
					numericUpDownSinFreq_ValueChanged(null, null);
					// We need to invalidate the render state, because we changed the actions list
					// This will cause the shaders to be rebuilt
					_renderState.Invalidate();
				}
				else
				{
					_renderState.Actions.Remove(_sin);
					_renderState.Invalidate();
				}
			});
		}

		private void numericUpDownSinFreq_ValueChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				_cube.SetShaderVariable<float>("frequency", (float) numericUpDownSinFreq.Value);
			});
		}

		private void checkBoxWireframe_CheckedChanged(object sender, EventArgs e)
		{
			_game.BeginInvoke(delegate()
			{
				if (checkBoxWireframe.Checked)
					_window.FillMode = Jade.Rendering.Enums.FillMode.WireFrame;
				else
					_window.FillMode = Jade.Rendering.Enums.FillMode.Solid;
			});
		}

		private void comboBoxLightFragment_SelectedIndexChanged(object sender, EventArgs e)
		{
			string lightingModel = comboBoxLightFragment.Text;
			_game.BeginInvoke(delegate()
			{
				_renderState.Materials[0].LightingEffect = _game.Resources.LightingEffects[lightingModel];
				_renderState.Invalidate();
			});
		}

		private void numericUpDownPhongExponent_ValueChanged(object sender, EventArgs e)
		{
			float value = (float) numericUpDownPhongExponent.Value;
			_game.BeginInvoke(delegate
			{
				_cube.SetShaderVariable<float>("phong_exponent", value);

			});
		}

		private void numericUpDownVelocity_ValueChanged(object sender, EventArgs e)
		{
			float value = (float) numericUpDownVelocity.Value;
			_game.BeginInvoke(delegate
			{
				_cameraTransform.SetMaxTranslationVelocities(value);
				_camera2Transform.SetMaxTranslationVelocities(value);
			});
		}

		private void numericUpDownAccel_ValueChanged(object sender, EventArgs e)
		{

			//float value = (float) numericUpDownAccel.Value;
			//_game.BeginInvoke(delegate
			//{
			//    _cameraTransform.Acceleration = value;
			//    _camera2Transform.Acceleration = value;
			//});
		}

		private void numericUpDownDecel_ValueChanged(object sender, EventArgs e)
		{
			//float value = (float) numericUpDownDecel.Value;
			//_game.BeginInvoke(delegate
			//{
			//    _cameraTransform.Deceleration = value;
			//    _camera2Transform.Deceleration = value;
			//});

		}

		private void numericUpDownRotationalVelocity_ValueChanged(object sender, EventArgs e)
		{
			float value = (float) numericUpDownRotationalVelocity.Value;
			_game.BeginInvoke(delegate
			{
				_cameraTransform.SetMaxRotationVelocities(value);
				_camera2Transform.SetMaxRotationVelocities(value);
			});
		}



	}
}
