#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Statements

using System;

#endregion

namespace Jade.Ai.Graphs.Algorithms
{
	/// <summary>
	/// Delegate to call when an edge event fires.
	/// </summary>
	/// <remarks>
	/// Based on the Boost Graph Library and QuickGraph ideas and implementations.
	/// http://www.boost.org/libs/graph/doc/index.html
	/// http://www.codeplex.com/quickgraph
	/// </remarks>
	public delegate void EdgeEventHandler<TVertex, TEdge>(Object sender, EdgeEventArgs<TVertex, TEdge> e) 
		where TEdge : IEdge<TVertex>;

	/// <summary>
	/// Edge args for an edge event.
	/// </summary>
	/// <typeparam name="TVertex">The type of the vertex.</typeparam>
	/// <typeparam name="TEdge">The type of the edge.</typeparam>
	public class EdgeEventArgs<TVertex, TEdge> : EventArgs
		where TEdge : IEdge<TVertex>
	{
		#region Fields and Properties

		private TEdge _edge;
		/// <summary>
		/// Gets the edge.
		/// </summary>
		/// <value>The edge.</value>
		public TEdge Edge { get { return _edge; } }

		#endregion

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="EdgeEventArgs&lt;TVertex, TEdge&gt;"/> class.
		/// </summary>
		/// <param name="edge">The edge.</param>
		public EdgeEventArgs(TEdge edge)
		{
			_edge = edge;
		}

		#endregion
	}
}
