#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

namespace Jade.Ai.Graphs.Algorithms
{
	/// <summary>
	/// Algorithm that has a root (start) node.
	/// </summary>
	/// <typeparam name="TGraph">The type of the graph.</typeparam>
	/// <typeparam name="TVertex">The type of the vertex.</typeparam>
	/// <typeparam name="TEdge">The type of the edge.</typeparam>
	/// <remarks>
	/// Based on the Boost Graph Library and QuickGraph ideas and implementations.
	/// http://www.boost.org/libs/graph/doc/index.html
	/// http://www.codeplex.com/quickgraph
	/// </remarks>
	public abstract class RootAlgorithm<TGraph, TVertex, TEdge> : BaseAlgorithm<TGraph, TVertex, TEdge>
		where TGraph : IGraph<TVertex, TEdge>
		where TEdge : IEdge<TVertex>
	{
		#region Fields and Properties

		private TVertex _root;
		/// <summary>
		/// Gets or sets the root of the algorithm.
		/// </summary>
		/// <value>The root.</value>
		public TVertex Root
		{
			get { return _root; }
			protected set { _root = value; }
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="RootAlgorithm&lt;TGraph, TVertex, TEdge&gt;"/> class.
		/// </summary>
		/// <param name="graph">The graph.</param>
		public RootAlgorithm(TGraph graph)
			: base(graph) { }

		#endregion

		#region Methods

		/// <summary>
		/// Initializes the algorithm.
		/// </summary>
		public override void Initialize()
		{
			Initialize(Root);
		}

		/// <summary>
		/// Initializes the algorithm.
		/// </summary>
		/// <param name="root">The root.</param>
		public abstract void Initialize(TVertex root);

		#endregion
	}
}
