#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Statements

using System.Collections.Generic;

#endregion

namespace Jade.Ai.Graphs
{
	/// <summary>
	/// Represents a graph where the total list of edges is accessible.
	/// </summary>
	/// <typeparam name="TVertex">The type of the vertex.</typeparam>
	/// <typeparam name="TEdge">The type of the edge.</typeparam>
	/// <remarks>
	/// Based on the Boost Graph Library and QuickGraph ideas and implementations.
	/// http://www.boost.org/libs/graph/doc/index.html
	/// http://www.codeplex.com/quickgraph
	/// </remarks>
	public interface IEdgeListGraph<TVertex, TEdge> : IGraph<TVertex, TEdge>
		where TEdge : IEdge<TVertex>
	{
		#region Fields and Properties

		/// <summary>
		/// Gets the list of all the edges of the graph.
		/// </summary>
		/// <value>The edges.</value>
		IEnumerable<TEdge> Edges { get; }

		/// <summary>
		/// Number of edges in the graph.
		/// </summary>
		/// <value>The number of edges.</value>
		int EdgesCount { get; }

		#endregion
	}
}
