#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Statements

using System;

#endregion

namespace Jade.Ai.Graphs
{
	/// <summary>
	/// Represents a <see cref="IEdgeListGraph"/> where add and remove operations related to the
	/// edges can be performed.
	/// </summary>
	/// <typeparam name="TVertex">The type of the vertex.</typeparam>
	/// <typeparam name="TEdge">The type of the edge.</typeparam>
	/// <remarks>
	/// Based on the Boost Graph Library and QuickGraph ideas and implementations.
	/// http://www.boost.org/libs/graph/doc/index.html
	/// http://www.codeplex.com/quickgraph
	/// </remarks>
	public interface IMutableEdgeListGraph<TVertex, TEdge> : IEdgeListGraph<TVertex, TEdge>
		where TEdge : IEdge<TVertex>
	{
		#region Methods

		/// <summary>
		/// Adds the edge.
		/// </summary>
		/// <param name="edge">The edge.</param>
		void AddEdge(TEdge edge);

		/// <summary>
		/// Tries to add the edge.
		/// </summary>
		/// <param name="edge">The edge.</param>
		/// <returns><c>true</c> if the edge was added; otherwise, <c>false</c>.</returns>
		bool TryAddEdge(TEdge edge);

		/// <summary>
		/// Removes the edge.
		/// </summary>
		/// <param name="edge">The edge.</param>
		/// <returns><c>true</c> if the edge was removed; otherwise, <c>false</c>.</returns>
		bool RemoveEdge(TEdge edge);

		/// <summary>
		/// Removes the edge.
		/// </summary>
		/// <param name="source">The source.</param>
		/// <param name="target">The target.</param>
		/// <returns><c>true</c> if the edge was removed; otherwise, <c>false</c>.</returns>
		/// <remarks>If the graph allows parallel edges only the first one found is removed.</remarks>
		bool RemoveEdge(TVertex source, TVertex target);

		/// <summary>
		/// Removes all the edges that match a predicate.
		/// </summary>
		/// <param name="match">The predicate.</param>
		void RemoveAllEdges(Predicate<TEdge> match);

		#endregion
	}
}
