#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Statements

using System;
using System.Collections.Generic;

using Jade.Ai.Core;

using Pluto.Math;

#endregion

namespace Jade.Ai.SteeringBehaviors
{
	public class Flee : SteeringBehavior
	{
		#region Fields and Properties

		private List<IMovableEntity> _targets = new List<IMovableEntity>();
		public ICollection<IMovableEntity> Targets
		{
			get { return _targets; }
		}

		#endregion

		#region Methods

		public override Vector3 Calculate()
		{
			Vector3 result = Vector3.Zero;

			foreach (IMovableEntity target in _targets)
			{
				Vector3 targetPos = target.Position;
				Vector3 ownPos = Owner.Position;
				Vector3 direction = Vector3.Subtract(ref ownPos, ref targetPos);

				Vector3 desiredSpeed = Vector3.Multiply(ref direction, Owner.MaximumSpeed);

				Vector3 ownVel = Owner.Velocity;
				Vector3 difference = Vector3.Subtract(ref desiredSpeed, ref ownVel);

				result = Vector3.Add(ref result, ref difference);
			}

			return result;
		}

		#endregion
	}
}
