#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Statements

using System;
using System.Collections.Generic;

using Jade.Ai.Core;

using Pluto.Math;

#endregion

namespace Jade.Ai.SteeringBehaviors
{
	public abstract class SteeringManager<TBehaviorData>
	{
		#region Internal Classes

		public class BehaviorDataPair<TData>
		{
			#region Fields and Properties

			public SteeringBehavior SteeringBehavior;
			public TData Data;

			#endregion

			#region Constructors

			public BehaviorDataPair(SteeringBehavior behavior, TData data)
			{
				SteeringBehavior = behavior;
				Data = data;
			}

			#endregion
		}

		#endregion

		#region Fields and Properties

		private IMovableEntity _owner;
		protected IMovableEntity Owner { get { return _owner; } }

		private IDictionary<Type, BehaviorDataPair<TBehaviorData>> _behaviorDataPairs = new Dictionary<Type, BehaviorDataPair<TBehaviorData>>();
		protected IDictionary<Type, BehaviorDataPair<TBehaviorData>> BehaviorDataPairs { get { return _behaviorDataPairs; } }

		#endregion

		#region Constructors

		public SteeringManager(IMovableEntity owner)
		{
			_owner = owner;
		}

		#endregion

		#region Methods

		public virtual SteeringManager<TBehaviorData> ActivateBehavior<T>() where T : SteeringBehavior
		{
			BehaviorDataPair<TBehaviorData> pair;

			_behaviorDataPairs.TryGetValue(typeof(T), out pair);
			if (pair != null)
				pair.SteeringBehavior.Active = true;

			return this;
		}

		public virtual SteeringManager<TBehaviorData> DeactivateBehavior<T>() where T : SteeringBehavior
		{
			BehaviorDataPair<TBehaviorData> pair;

			_behaviorDataPairs.TryGetValue(typeof(T), out pair);
			if (pair != null)
				pair.SteeringBehavior.Active = false;

			return this;
		}

		public virtual T GetBehavior<T>() where T : SteeringBehavior
		{
			return (T) _behaviorDataPairs[typeof(T)].SteeringBehavior;
		}

		public virtual SteeringManager<TBehaviorData> SetBehavior<T>(T behavior, TBehaviorData data) where T : SteeringBehavior
		{
			behavior.SetOwner(_owner);
			_behaviorDataPairs[behavior.GetType()] = new BehaviorDataPair<TBehaviorData>(behavior, data);
			return this;
		}

		public abstract Vector3 Calculate();

		#endregion
	}
}
