#pragma once

#include "DXAdapter.h"

namespace JadeRenderingDx9 {

	[CompletionStatus(DevelopmentState::Complete)]
	ref class DXDepthStencil :	public DepthStencilBase
	{
	private:
		DXAdapter^ _dxAdapter;

		void OnReset(Object^ sender, EventArgs^ args)
		{
			Create();
		}
		void OnLost(Object^ sender, EventArgs^ args)
		{
			CleanupSurface();
		}
		void OnDisposing(Object^ sender, EventArgs^ args)
		{
			delete this;
		}

		void Create()
		{
			CleanupSurface();

			IDirect3DSurface9* _surface;
			DXCHECK(_dxAdapter->Device->CreateDepthStencilSurface( this->Width, this->Height, 
				(D3DFORMAT) this->Format, (D3DMULTISAMPLE_TYPE) this->MultiSample, this->MultiSampleQuality, 
				false, &_surface, NULL ));
			Surface = _surface;
		}

		void CleanupSurface()
		{
			FREEDX(Surface);
		}
	internal:
		property IDirect3DSurface9* Surface;


	public:
		DXDepthStencil(DXAdapter^ adapter, int width, int height, DepthFormat format, int multiSample, int multiSampleQuality)
			:DepthStencilBase(width,height,format,multiSample,multiSampleQuality)
		{
			_dxAdapter = adapter;
			_dxAdapter->DeviceReset += gcnew EventHandler(this, &DXDepthStencil::OnReset);
			_dxAdapter->DeviceLost += gcnew EventHandler(this, &DXDepthStencil::OnLost);
			_dxAdapter->Disposing += gcnew EventHandler(this,&DXDepthStencil::OnDisposing);

			Create();
		}

		virtual ~DXDepthStencil()
		{
			_dxAdapter->DeviceReset -= gcnew EventHandler(this, &DXDepthStencil::OnReset);
			_dxAdapter->DeviceLost -= gcnew EventHandler(this, &DXDepthStencil::OnLost);
			_dxAdapter->Disposing -= gcnew EventHandler(this,&DXDepthStencil::OnDisposing);
			this->CleanupSurface();
		}
	};
}