#pragma once

#include "D3D.h"

using namespace System;
using namespace System::Collections::Generic;
using namespace System::Text;

using namespace Jade::Core;
using namespace Jade::Entities;
using namespace Jade::Rendering;
using namespace Jade::Resources;

namespace JadeRenderingDx9 {

	ref class DXAdapter;
	ref class DXEffect;

	ref class DXEffectBuilder
	{
	private:

        /// <summary>
        /// The adapter for this effect
        /// </summary>
		DXAdapter^ _adapter;

		// The shader generator
		ShaderGenerator^ _generator;

		bool NormalHasIsOwnTextureMapping(Material^ material)
		{
			return ((!material->NormalMap.Channel.UseDiffuseChannel) && (material->NormalMap.Enabled) && (material->NormalMap.Texture != nullptr));
		}

		int _numberOfLights;
		ID3DXEffect* Process(String^ shader);
	public:
		DXEffectBuilder(DXAdapter^ adapter);
		virtual ~DXEffectBuilder(void);

		// Gets the best supported pixel shader by this device.
		property String^ PixelShaderSupported
		{
			String^ get();
		}

		// Gets the best supported vertex shader by this device.
		property String^ VertexShaderSupported
		{
			String^ get();
		}

		DXEffect^ GenerateZPass(Material^ material, RenderState^ renderState);
		DXEffect^ GenerateAmbientPass(Material^ material, RenderState^ renderState);
		DXEffect^ GenerateOverlayPass(Sprite^ sprite, IList<Light^>^ lights, Material^ material, RenderState^ renderState);
		DXEffect^ GenerateLightPass(Light^ light, Material^ material, bool renderAmbient, RenderState^ renderState);

		void SetLightShaderConstants(DXEffect^ dxeffect, Light^ light, int numLight, SubMesh^ mesh, D3DXMATRIX& projectorMatrix, D3DXVECTOR4 position, D3DXVECTOR4 direction);
		void SetMaterialShaderConstants(DXEffect^ dxeffect, SubMesh^ mesh, Material^ material, Color ambientColor);

	};

}