#pragma once

#include "D3D.h"
#include "Exceptions.h"
#include "DXAdapter.h"

using namespace System;
using namespace System::Collections::Generic;

using namespace Jade::Core;
using namespace Jade::Resources;
using namespace Jade::Rendering;

using namespace Pluto::Core;
using namespace Pluto::Configuration;

namespace JadeRenderingDx9 {

	[CompletionStatus(DevelopmentState::Complete)]
	ref class DXRenderTarget :	public RenderTargetBase
	{
	private:
		DXAdapter^ _dxAdapter;
		D3DData* _D3D;

		void OnReset(Object^ sender, EventArgs^ args)
		{
			if (!IsSwapChain)
				GetDeviceSurface();
			else
			{
				_D3D->PresentParameters.BackBufferWidth = this->Width;
				_D3D->PresentParameters.BackBufferHeight = this->Height;
				CreateSwapChainSurface();
			}

		}
		void OnLost(Object^ sender, EventArgs^ args)
		{
			Cleanup();
		}
		void OnDisposing(Object^ sender, EventArgs^ args)
		{
			delete this;
		}

		void Cleanup()
		{
			FREEDX(Surface);
			FREEDX(SwapChain);
		}
	public:
		DXRenderTarget(DXAdapter ^adapter, D3DPRESENT_PARAMETERS& d3dPresentParameters, int width, int height)
			: RenderTargetBase(width, height)
		{
			_D3D = new D3DData();
			_D3D->PresentParameters = d3dPresentParameters;
						
			_dxAdapter = adapter;
			_dxAdapter->DeviceReset += gcnew EventHandler(this, &DXRenderTarget::OnReset);
			_dxAdapter->DeviceLost += gcnew EventHandler(this, &DXRenderTarget::OnLost);
			_dxAdapter->Disposing += gcnew EventHandler(this,&DXRenderTarget::OnDisposing);

		}

		virtual ~DXRenderTarget()
		{
			delete _D3D;
			_dxAdapter->DeviceReset -= gcnew EventHandler(this, &DXRenderTarget::OnReset);
			_dxAdapter->DeviceLost -= gcnew EventHandler(this, &DXRenderTarget::OnLost);
			_dxAdapter->Disposing -= gcnew EventHandler(this,&DXRenderTarget::OnDisposing);
			Cleanup();
		}

	internal:
		property bool IsSwapChain;
		property IDirect3DSurface9* Surface;
		property IDirect3DSwapChain9* SwapChain;

		/// <summary>
		/// Gets the device surface.
		/// ATTENTION WE MUST DO A DISPOSE TO RESET THE DEVICE
		/// </summary>
		void GetDeviceSurface()
		{
			IsSwapChain = false;

			IDirect3DSurface9* _surface;
			DXCHECK(_dxAdapter->Device->GetBackBuffer( 0, 0, D3DBACKBUFFER_TYPE_MONO, &_surface ));
			Surface = _surface;

			//D3DSURFACE_DESC desc;
			//_surface->GetDesc(&desc);
			//Log::WriteLines(desc, "GetDeviceSurface", 1);
			//Log::WriteLine("GetDeviceSurface","DXRenderTarget",1);
		}

		void CreateSwapChainSurface()
		{
			// Needed for multisampling
			
			_D3D->PresentParameters.SwapEffect = D3DSWAPEFFECT_DISCARD;
			IDirect3DSwapChain9* _swapChain;
			
			DXCHECK(_dxAdapter->Device->CreateAdditionalSwapChain( &_D3D->PresentParameters, &_swapChain ));
			IsSwapChain = true;
			SwapChain = _swapChain;

			IDirect3DSurface9* _surface;
			DXCHECK(_swapChain->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &_surface));
			Surface = _surface;

			//Log::WriteLine("CreateSwapChainSurface","DXRenderTarget",1);
		}

	protected:
	};

}