#pragma once

#include "DXRenderer.h"
#include "DXFixedFunctionRenderer.h"

using namespace System;

using namespace Jade::Rendering;

using namespace Pluto::Core;
using namespace Pluto::Configuration;

namespace JadeRenderingDx9 {

	// Rendering system factor object for the plugin
	// This is what actually creates our renderer
	ref class DX9RenderingSystemFactory : RenderingSystemFactory
	{
	public:
		/// <summary>
		/// Initializes a new instance of the <see cref="DX9RenderingSystemFactory"/> class.
		/// </summary>
		/// <param name="provider">The provider.</param>
		DX9RenderingSystemFactory(IPlutoRootServiceProvider ^ provider)
			: RenderingSystemFactory(provider)
		{
		}

	protected:
		/// <summary>
		/// Creates the render system.
		/// </summary>
		/// <returns></returns>
		virtual IRenderingSystem ^ CreateRenderSystem() override
		{
			return gcnew DXRenderer(this->Provider,this->Provider->GetService<Pluto::Configuration::IConfigurationService^>());
		}
	};

	// Rendering system factor object for the plugin
	// This is what actually creates our renderer
	ref class DX9FixedFunctionRenderingSystemFactory : RenderingSystemFactory
	{
	public:
		/// <summary>
		/// Initializes a new instance of the <see cref="DX9RenderingSystemFactory"/> class.
		/// </summary>
		/// <param name="provider">The provider.</param>
		DX9FixedFunctionRenderingSystemFactory(IPlutoRootServiceProvider ^ provider)
			: RenderingSystemFactory(provider)
		{
		}

	protected:
		/// <summary>
		/// Creates the render system.
		/// </summary>
		/// <returns></returns>
		virtual IRenderingSystem ^ CreateRenderSystem() override
		{
			return gcnew DXFixedFunctionRenderer(this->Provider,this->Provider->GetService<Pluto::Configuration::IConfigurationService^>());
		}
	};


	public ref class PlutoPlugin : ServicePluginBase
	{
	public:
		PlutoPlugin(void)
		{
		}

	protected:

		virtual bool RegisterServices(IPlutoServiceContainer ^serviceContainer) override;

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		~PlutoPlugin()
		{
		
		}

	};
}
