#include "StdAfx.h"

#include "DXAdapter.h"
#include "DXUtility.h"
#include "DXEffectBuilder.h"


namespace JadeRenderingDx9 {

	void DXAdapter::RenderAmbientPass(RenderList^ renderList, Color ambientLight, Window^ window, D3DXMATRIX& dxProjection, D3DXMATRIX& dxView, D3DXMATRIX& dxViewProjection)
	{
		for each (TransformedMesh^ renderable in renderList->AmbientOnlyRenderables)
		{
			// if ambient is black, we do not need to render this mesh
			if ((renderable->RenderState->Materials[0]->Ambient + ambientLight).IsBlack()) 
				continue;

			D3DXMATRIX dxWorldViewProjection = DXUtility::CreateMatrix(renderable->Transform) * dxViewProjection;

			// Get (or create) the effect we need
			RenderStateHandler^ handler = this->GetRenderStateHandler(renderable->RenderState);
			
			DXEffect^ dxeffect = handler->GetAmbientPassEffect();
			ID3DXEffect* effect = dxeffect->Effect;
			UINT numPasses;
			DXCHECK(effect->Begin(&numPasses, D3DXFX_DONOTSAVESTATE));
			for (UINT i = 0; i < numPasses; i++)
			{
				effect->BeginPass(i);

				SetEffectMatrices(effect, dxWorldViewProjection);

				SetEffectCommonConstants(effect, renderable);

				_effectBuilder->SetMaterialShaderConstants(dxeffect, renderable->Mesh, renderable->RenderState->Materials[0], ambientLight);

				DXCHECK(effect->CommitChanges());
				//this->DrawIndexedPrimitives(renderable->Mesh->VertexBuffer, renderable->Mesh->IndexBuffer, PrimitiveType::TriangleList, 0, 0, renderable->Mesh->VertexBuffer->VertexCount, 0, renderable->Mesh->IndexBuffer->FaceCount);
        this->DrawTransformedMesh(renderable);
				DXCHECK(effect->EndPass());
			}
			DXCHECK(effect->End());
		}
	}
}