#include "StdAfx.h"

#include "DXAdapter.h"
#include "DXUtility.h"
#include "DXEffectBuilder.h"

namespace JadeRenderingDx9 {

	void DXAdapter::RenderOverlay(SpriteRenderList^ sprite, Color ambientLight, Camera^ camera, Window^ window, D3DXMATRIX& dxProjection, D3DXMATRIX& dxView, D3DXMATRIX& dxViewProjection)
	{
		Matrixd spriteTransform = sprite->Transform;
		D3DXMATRIX projectorMatrix;
		D3DXMatrixIdentity(&projectorMatrix);

		RenderStateHandler^ handler = this->GetRenderStateHandler(sprite->Sprite->RenderState);
		Material^ material = sprite->Sprite->RenderState->Materials[0];

		SetBlendState(material->BlendState);

		// Make sure we're current before we use them
		sprite->Sprite->RenderState->Validate();
		for each (TransformedLight^ i in sprite->Lights)
			i->Light->Validate();


		/*
		SpotLight^ spotLight = nullptr;
		if (sprite->Light->Type == LightType::Spot)
		{
			spotLight = (SpotLight)sprite->Light;

			if (spotLight->Projector->Enabled && spotLight->Projector->Texture != nullptr)
			{
				D3DXMATRIX dxLightWorld = DXUtility::CreateMatrix(sprite->Transform);
				projectorMatrix = BuildProjectorMatrix(dxLightWorld, spotLight);
			}
		}
		*/
		
		D3DXMATRIX dxWorld = DXUtility::CreateMatrix(sprite->Transform);

		DXEffect^ dxeffect = handler->GetOverlayEffect(sprite);
		ID3DXEffect* effect = dxeffect->Effect;
		UINT numPasses;
		DXCHECK(effect->Begin(&numPasses, D3DXFX_DONOTSAVESTATE));
		for (UINT i = 0; i < numPasses; i++)
		{
			effect->BeginPass(i);

			SetEffectMatrices(effect, dxWorld, dxWorld, dxView);
			SetEffectCommonConstants(effect, sprite->Sprite->RenderState, sprite->Sprite->SubMesh);

			int lightNumber = 0;
			for each (TransformedLight^ light in sprite->Lights)
			{
				Matrixd lightTransform = light->Transform;

				D3DXVECTOR4 lightPosition = DXUtility::Vector3dToVector4(light->Transform.Translation);
				D3DXVECTOR4 lightDirection = DXUtility::Vector3dToVector4(light->Transform.Look);

				_effectBuilder->SetLightShaderConstants(dxeffect, light->Light, lightNumber, sprite->Sprite->SubMesh, projectorMatrix, lightPosition, lightDirection);

				lightNumber++;
			}
			_effectBuilder->SetMaterialShaderConstants(dxeffect, sprite->Sprite->SubMesh, material, ambientLight);
		
			DXCHECK(effect->CommitChanges());
			VertexBuffer^ vb = sprite->Sprite->SubMesh->VertexBuffer;
			this->DrawIndexedPrimitives(vb, sprite->Sprite->SubMesh->IndexBuffer, sprite->Sprite->SubMesh->PrimitiveType, 0, 0, sprite->Sprite->SubMesh->VertexBuffer->VertexCount, 0, sprite->Sprite->SubMesh->IndexBuffer->PrimitiveCount);
			DXCHECK(effect->EndPass());
		}
		DXCHECK(effect->End());
	}

}