#include "StdAfx.h"

#include "DXAdapter.h"
#include "RenderStateHandler.h"
#include "DXEffectBuilder.h"

namespace JadeRenderingDx9 {

	RenderStateHandler::RenderStateHandler(DXAdapter^ adapter, DXEffectBuilder^ effectBuilder, RenderState^ renderState)
		: ResourceHandlerBase(adapter)
	{
		_adapter = adapter;
		_lightPasses = gcnew GraphicsResourceCollection<Light^, LightShaderHandler^>();
		_spritePasses = gcnew GraphicsResourceCollection<Sprite^, SpriteShaderHandler^>();

		_effectBuilder = effectBuilder;
		_renderState = renderState;

		this->AttachResource(_renderState);
	}

	LightShaderHandler^ RenderStateHandler::GetLightShaderHandler(Light^ light)
	{
		LightShaderHandler^ result;
		if (!_lightPasses->TryGetValue(light, result))
		{
			result = gcnew LightShaderHandler(_adapter, _effectBuilder, light, _renderState);
			_lightPasses->Add(light, result);
		}
		return result;
	}

	SpriteShaderHandler^ RenderStateHandler::GetSpriteShaderHandler(SpriteRenderList^ spriteList)
	{
		SpriteShaderHandler^ result;
		if (!_spritePasses->TryGetValue(spriteList->Sprite, result))
		{
			result = gcnew SpriteShaderHandler(_adapter, _effectBuilder, spriteList, _renderState);
			_spritePasses->Add(spriteList->Sprite, result);
		}

		// Add the light check in here now to make sure we're up to date with our light list
		result->Check(spriteList->Lights);
		return result;
	}

	DXEffect^ RenderStateHandler::GetZPassEffect()
	{
		this->ValidateResources();

		if (_zPassEffect == nullptr)
			_zPassEffect = _effectBuilder->GenerateZPass(_renderState->Materials[0], _renderState);
		return _zPassEffect;
	}

	DXEffect^ RenderStateHandler::GetAmbientPassEffect()
	{
		this->ValidateResources();

		if (_ambientPassEffect == nullptr)
			_ambientPassEffect = _effectBuilder->GenerateAmbientPass(_renderState->Materials[0], _renderState);
		return _ambientPassEffect;
	}
	DXEffect^ RenderStateHandler::GetLightPassEffect(Light^ light, bool renderAmbient)
	{
		this->ValidateResources();

		LightShaderHandler^ handler = GetLightShaderHandler(light);
		return handler->GetLightEffect(renderAmbient);
	}
	DXEffect^ RenderStateHandler::GetOverlayEffect(SpriteRenderList^ spriteList)
	{
		this->ValidateResources();

		SpriteShaderHandler^ handler = GetSpriteShaderHandler(spriteList);
		return handler->GetSpriteEffect();
	}
}