#pragma once

#include "LightShaderHandler.h"
#include "SpriteShaderHandler.h"
#include "DXEffect.h"

using namespace System;
using namespace System::Collections::Generic;

using namespace Jade::Resources;

namespace JadeRenderingDx9 {

	ref class DXAdapter;
	ref class DXEffectBuilder;

	ref class RenderStateHandler : ResourceHandlerBase
	{
	private:
		RenderState^ _renderState;
		DXEffectBuilder^ _effectBuilder;
		DXAdapter^ _adapter;

		// Our effects
		DXEffect^ _zPassEffect;
		DXEffect^ _ambientPassEffect;

		// The light passes
		GraphicsResourceCollection<Light^, LightShaderHandler^>^ _lightPasses; 
		// The sprite passes
		GraphicsResourceCollection<Sprite^, SpriteShaderHandler^>^ _spritePasses;

		LightShaderHandler^ GetLightShaderHandler(Light^ light);
		SpriteShaderHandler^ GetSpriteShaderHandler(SpriteRenderList^ spriteList);

	protected:
		// We have to override this to free our DX resources
		virtual void FreeResources() override
		{
			delete _zPassEffect; _zPassEffect = nullptr;
			delete _ambientPassEffect; _ambientPassEffect = nullptr;
			// Dispose of our light pass shaders
			_lightPasses->Clear();
			_spritePasses->Clear();
		}

		// And override this to update us correctly
		virtual void UpdateResources() override
		{
			FreeResources();
		}
	
	public:
		RenderStateHandler(DXAdapter^ adapter, DXEffectBuilder^ effectBuilder, RenderState^ renderState);

		DXEffect^ GetZPassEffect();
		DXEffect^ GetAmbientPassEffect();
		DXEffect^ GetLightPassEffect(Light^ light, bool renderAmbient);
		DXEffect^ GetOverlayEffect(SpriteRenderList^ spriteList);
	};

}