#include "StdAfx.h"

#include "DXAdapter.h"
#include "DXUtility.h"
#include "DXEffectBuilder.h"

namespace JadeRenderingDx9 {

	void DXAdapter::RenderZPass(RenderList^ renderList, Window^ window, D3DXMATRIX& dxProjection, D3DXMATRIX& dxView, D3DXMATRIX& dxViewProjection)
	{
		// We do not want to use the color stage, only the Z buffer
		BlendState^ blend = gcnew BlendState();
		blend->DisableColor();
		this->SetBlendState(blend);

		for each (TransformedMesh^ renderable in renderList->Renderables)
		{
			D3DXMATRIX dxWorldViewProjection = DXUtility::CreateMatrix(renderable->Transform) * dxViewProjection;

			// Get (or create) the effect we need
			RenderStateHandler^ handler = this->GetRenderStateHandler(renderable->RenderState);
			ID3DXEffect* effect = handler->GetZPassEffect()->Effect;


			UINT numPasses;
			DXCHECK(effect->Begin(&numPasses, D3DXFX_DONOTSAVESTATE));
			for (UINT i = 0; i < numPasses; i++)
			{
				effect->BeginPass(i);
				
				D3DXHANDLE effectHandle = effect->GetParameterByName(NULL, "WorldViewProjection");
				effect->SetMatrix(effectHandle, &dxWorldViewProjection);

				SetEffectCommonConstants(effect, renderable);

				DXCHECK(effect->CommitChanges());
				//this->DrawIndexedPrimitives(renderable->Mesh->VertexBuffer, renderable->Mesh->IndexBuffer, PrimitiveType::TriangleList, 0, 0, renderable->Mesh->VertexBuffer->VertexCount, 0, renderable->Mesh->IndexBuffer->FaceCount);
        this->DrawTransformedMesh(renderable);
				DXCHECK(effect->EndPass());
			}
			DXCHECK(effect->End());
		}

	}

}