#include "StdAfx.h"

#include "DXAdapter.h"
#include "DXUtility.h"
#include "DXRenderTarget.h"
#include "DXDepthStencil.h"

namespace JadeRenderingDx9 {

	void DXAdapter::SetEffectCommonConstants(ID3DXEffect* effect,RenderState^ renderState,SubMesh^ mesh)
	{
		// Handle all of the actions
		for each (VertexAction^ action in renderState->Actions)
		{
			// Now set each variable in the action
			for each (ShaderVariable^ variable in action->Variables)
				DXUtility::ShaderVariableToDXEffect(variable, effect, mesh);
		}

		// Get our environment handle
		D3DXHANDLE handle = effect->GetParameterByName(NULL, "environment");
		D3DVIEWPORT9 vp;
		DXCHECK(Device->GetViewport(&vp));
		D3DXVECTOR4 vec((float)vp.Width, (float)vp.Height,Timer->Time,0.0);
		effect->SetVector(handle, &vec);
		/*D3DXHANDLE handleTop = effect->GetParameterByName(NULL, "env");
		if (handleTop != NULL)
		{
			D3DXHANDLE handle = effect->GetParameterByName(handleTop,"TimeElapsed");
			if (handle != NULL)
				effect->SetFloat(handle, Timer->Time);
			handle = effect->GetParameterByName(handleTop, "RenderTargetSize");
			if (handle != NULL)
			{
				D3DVIEWPORT9 vp;
				DXCHECK(Device->GetViewport(&vp));

				D3DXVECTOR4 vec((float)vp.Width, (float)vp.Height,0.0,0.0);
				effect->SetVector(handle, &vec);
			}
		}*/
	}

	void DXAdapter::SetEffectMatrices(ID3DXEffect* effect, D3DXMATRIX& worldViewProject, D3DXMATRIX& world, D3DXMATRIX& view)
	{
		SetEffectMatrices(effect,worldViewProject);

		D3DXHANDLE effectHandle = effect->GetParameterByName(NULL, "World");
		if (effectHandle != NULL) 
			effect->SetMatrix(effectHandle, &world);
		effectHandle = effect->GetParameterByName(NULL, "View");
		if (effectHandle != NULL) 
			effect->SetMatrix(effectHandle, &view);
	}

	void DXAdapter::SetEffectMatrices(ID3DXEffect* effect, D3DXMATRIX& worldViewProject)
	{
		D3DXHANDLE effectHandle = effect->GetParameterByName(NULL, "WorldViewProjection");
		if (effectHandle != NULL) 
			effect->SetMatrix(effectHandle, &worldViewProject);
	}


	ID3DXFont* DXAdapter::GetFont(OverlayTextFont^ font)
	{

		if (!Object::ReferenceEquals(font, _activeOverlayTextFont))
		{
			_activeOverlayTextFont = font;
			// At this point, get our font
			if (!_overlayTextFonts->TryGetValue(_activeOverlayTextFont, _activeDXOverlayTextFont))
			{
				// Create our font
				_activeDXOverlayTextFont = gcnew OverlayTextFontHandler(this, _activeOverlayTextFont);
				_overlayTextFonts->Add(_activeOverlayTextFont, _activeDXOverlayTextFont);
			}
		}

		if (_activeDXOverlayTextFont->Resource == NULL)
			throw gcnew InvalidOperationException("The font object doesnt exist");
		return _activeDXOverlayTextFont->Resource;
	}

	IDirect3DBaseTexture9* DXAdapter::GetTexture(Texture^ texture)
	{
		if ((!texture->Loaded) || (!Object::ReferenceEquals(texture,_activeTexture)))
		{
			texture->Load();
			_activeTexture = texture;
			if (!_textures->TryGetValue(_activeTexture, _activeTextureHandler))
			{
				// Create our texture
				_activeTextureHandler = gcnew TextureHandler(this, _activeTexture);
				_textures->Add(_activeTexture, _activeTextureHandler);
			}
		}

		return _activeTextureHandler->DxTexture;
	}

	RenderStateHandler^ DXAdapter::GetRenderStateHandler(RenderState^ renderState)
	{
		RenderStateHandler^ result;
		if (!_renderStates->TryGetValue(renderState, result))
		{
			// Create our render state
			result = gcnew RenderStateHandler(this, _effectBuilder, renderState);
			_renderStates->Add(renderState, result);
		}

		return result;
	}



	void DXAdapter::SetIndexBuffer(IndexBuffer^ indices)
	{
		if (indices == nullptr)
			throw gcnew ArgumentNullException("indices");

		// Do nothing in this case
		if (Object::ReferenceEquals(indices, _activeIndexBuffer))
			return;

		// Get our DX9IndexBuffer, if it exists
		IndexBufferHandler^ dx9Indices;
		if (!_indexBuffers->TryGetValue(indices, dx9Indices))
		{
			// Create our indices
			dx9Indices = gcnew IndexBufferHandler(this, indices);
			_indexBuffers->Add(indices, dx9Indices);
		}

		// Save this
		_activeIndexBuffer = indices;

		// Set this to the device
		DXCHECK(Device->SetIndices( dx9Indices->Resource ));
	}

	void DXAdapter::SetVertexBuffer(VertexBuffer^ vertices, int stream)
	{
		if (vertices == nullptr)
			throw gcnew ArgumentException("DrawIndexedPrimitives called with invalid VertexBuffer");

		if (Object::ReferenceEquals(vertices, _activeVertexBuffer))
			return;

		VertexBufferHandler^ dx9Vertices;
		if (!_vertexBuffers->TryGetValue(vertices, dx9Vertices))
		{
			dx9Vertices = gcnew VertexBufferHandler(this, vertices);
			_vertexBuffers->Add(vertices, dx9Vertices);
		}
		// save this
		_activeVertexBuffer = vertices;


		DXCHECK(Device->SetVertexDeclaration(dx9Vertices->Declaration));
		//_device.VertexFormat = (D3D.VertexFormats) ((int) D3D.VertexFormats.Position + (int) D3D.VertexFormats.Texture1 + (int) D3D.VertexFormats.Normal);
		DXCHECK(Device->SetStreamSource(stream,dx9Vertices->Resource,0,dx9Vertices->Size));
	}

	void DXAdapter::SetViewport(Viewport viewport)
	{

		D3DVIEWPORT9 vPort = {
			viewport.X,
			viewport.Y,
			viewport.Width,
			viewport.Height,
			viewport.ZMin,
			viewport.ZMax
		};

		DXCHECK(Device->SetViewport( &vPort ));
	}

	void DXAdapter::SetBlendState(BlendState^ state)
	{
		this->Device->SetRenderState(D3DRS_BLENDFACTOR, state->BlendFactor.ToRGB());
		this->Device->SetRenderState(D3DRS_ALPHABLENDENABLE, state->BlendEnable[0]);
		this->Device->SetRenderState(D3DRS_SRCBLEND, (DWORD) state->SrcBlend);
		this->Device->SetRenderState(D3DRS_DESTBLEND, (DWORD) state->DestBlend);
		this->Device->SetRenderState(D3DRS_SRCBLENDALPHA, (DWORD) state->SrcBlendAlpha);
		this->Device->SetRenderState(D3DRS_DESTBLENDALPHA, (DWORD) state->DestBlendAlpha);
		this->Device->SetRenderState(D3DRS_BLENDOP, (DWORD) state->BlendOp);
		this->Device->SetRenderState(D3DRS_BLENDOPALPHA, (DWORD) state->BlendOpAlpha);
		this->Device->SetRenderState(D3DRS_COLORWRITEENABLE, (DWORD) state->RenderTargetWriteMask[0]);
		if (this->_numRenderTargets > 1)
			this->Device->SetRenderState(D3DRS_COLORWRITEENABLE1, (DWORD) state->RenderTargetWriteMask[1]);
		if (this->_numRenderTargets > 2)
			this->Device->SetRenderState(D3DRS_COLORWRITEENABLE2, (DWORD) state->RenderTargetWriteMask[2]);
		if (this->_numRenderTargets > 3)
			this->Device->SetRenderState(D3DRS_COLORWRITEENABLE3, (DWORD) state->RenderTargetWriteMask[3]);
	}

	void DXAdapter::SetDepthStencilState(DepthStencilState^ state)
	{
		if (state->DepthEnable)
			this->Device->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE );
		else
			this->Device->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE );
		this->Device->SetRenderState(D3DRS_ZWRITEENABLE , state->DepthWriteEnable);
		this->Device->SetRenderState(D3DRS_ZFUNC, (DWORD) state->DepthFunc);
		this->Device->SetRenderState(D3DRS_STENCILENABLE, state->StencilEnable);
		this->Device->SetRenderState(D3DRS_STENCILMASK , state->StencilReadMask);
		this->Device->SetRenderState(D3DRS_STENCILWRITEMASK, state->StencilWriteMask);
		this->Device->SetRenderState(D3DRS_STENCILFAIL, (DWORD)state->FrontFace.Fail);
		this->Device->SetRenderState(D3DRS_STENCILZFAIL, (DWORD)state->FrontFace.DepthFail);
		this->Device->SetRenderState(D3DRS_STENCILFUNC, (DWORD)state->FrontFace.Comparisson);
		this->Device->SetRenderState(D3DRS_STENCILPASS, (DWORD)state->FrontFace.Pass);
		this->Device->SetRenderState(D3DRS_TWOSIDEDSTENCILMODE, (DWORD)state->TwoSidedDepthStencil);
		this->Device->SetRenderState(D3DRS_CCW_STENCILFAIL, (DWORD)state->BackFace.Fail);
		this->Device->SetRenderState(D3DRS_CCW_STENCILZFAIL, (DWORD)state->BackFace.DepthFail);
		this->Device->SetRenderState(D3DRS_CCW_STENCILFUNC ,(DWORD)state->BackFace.Comparisson);
		this->Device->SetRenderState(D3DRS_CCW_STENCILPASS, (DWORD)state->BackFace.Pass);
	}

	void DXAdapter::SetRasterizerState(RasterizerState^ state)
	{
		this->Device->SetRenderState(D3DRS_CULLMODE , (DWORD)state->CullMode);
		this->Device->SetRenderState(D3DRS_FILLMODE, (DWORD)state->FillMode);
		this->Device->SetRenderState(D3DRS_DEPTHBIAS, state->DepthBias);
		this->Device->SetRenderState(D3DRS_SCISSORTESTENABLE, state->ScissorEnabled);
	}

	void DXAdapter::SetDepthStencil(DepthStencilBase^ depthstencil)
	{
		DXDepthStencil^ surface = (DXDepthStencil^) depthstencil;
		// If it's already set, return
		if (_activeDepthStencil == surface) 
			return;

		_activeDepthStencil = surface;

		IDirect3DSurface9* dxsurface = surface->Surface;
		DXCHECK(Device->SetDepthStencilSurface( dxsurface ));
	}
	void DXAdapter::SetRenderTarget(int number, RenderTargetBase^ renderTarget)
	{
		DXRenderTarget^ surface = (DXRenderTarget^) renderTarget;
		// If it's already set, return
		if (_activeRenderTarget == surface) 
			return;

		_activeRenderTarget = surface;

		IDirect3DSurface9* dxsurface = surface->Surface;
		HRESULT hr = Device->SetRenderTarget( number, dxsurface );
		GraphicsException::CheckHResult( hr );
	}

}