#include "StdAfx.h"

#include "DXAdapter.h"
#include "SpriteShaderHandler.h"
#include "DXEffectBuilder.h"
#include "DXEffect.h"

namespace JadeRenderingDx9 {

	SpriteShaderHandler::SpriteShaderHandler(DXAdapter^ adapter, DXEffectBuilder^ effectBuilder, SpriteRenderList^ spriteList, RenderState^ renderState)
		: ResourceHandlerBase(adapter)
	{
		_effectBuilder = effectBuilder;
		_sprite = spriteList->Sprite;
		_renderState = renderState;

		_lights = gcnew List<Light^>();

		UpdateLightList(spriteList->Lights);

		this->AttachResource(_renderState);
		this->AttachResource(_sprite);
	}

	SpriteShaderHandler::~SpriteShaderHandler(void)
	{
		// We have to do this so we unsubscribe from our light events, too
		UpdateLightList(nullptr);
	}

	void SpriteShaderHandler::UpdateLightList(IList<TransformedLight^>^ lights)
	{
		IsValid = false;

		// Remove all of our current lights
		while (_lights->Count > 0)
		{
			Light^ light = _lights[_lights->Count - 1];
			this->DetachResource(light);
			light->Disposing -= gcnew EventHandler(this, &SpriteShaderHandler::OnLight_Disposing);
			_lights->Remove(light);
		}

		// If we don't have any lights now, just return
		if (lights == nullptr)
			return;

		if (lights->Count == 0)
			return;

		// Add in the lights to our new light
		for each (TransformedLight^ lightEntity in lights)
		{
			Light^ light = lightEntity->Light;
			this->AttachResource(light);
			light->Disposing += gcnew EventHandler(this, &SpriteShaderHandler::OnLight_Disposing);
			_lights->Add(light);
		}
	}

	void SpriteShaderHandler::Check(IList<TransformedLight^>^ lights)
	{
		// If we're invalidated, just update us
		if (!IsValid)
		{
			UpdateLightList(lights);
			return;
		}

		bool passed = true;
		// First, check for the counts to match
		if (lights->Count == _lights->Count)
		{
			// Okay - now sort our lights and compare them against our old ones
			//List<Light> newLights = new List<Light>(lights);
			//newLights.Sort(lightComp);
			for (int i = 0; i < lights->Count; ++i)
			{
				if (_lights[i]->ID != lights[i]->Light->ID)
				{
					passed = false;
					break;
				}
			}
		}
		else
			passed = false;

		if (!passed)
		{
			IsValid = false;
			UpdateLightList(lights);
		}
	}

	DXEffect^ SpriteShaderHandler::GetSpriteEffect()
	{
		this->ValidateResources();
		if (_spriteEffect == nullptr)
			_spriteEffect = _effectBuilder->GenerateOverlayPass(_sprite, _lights, _renderState->Materials[0], _renderState);
		return _spriteEffect;
	}

}