#pragma once

using namespace System;
using namespace System::Collections::Generic;

using namespace Jade::Resources;

using namespace Pluto::Core;

namespace JadeRenderingDx9 {

	[CompletionStatus(DevelopmentState::Complete)]
	ref class VertexBufferHandler : ResourceHandlerBase
	{
	private:
		DXAdapter^ _adapter;
		VertexBuffer^ _vertexBuffer;

		// Displacement to the position on the vertexbuffer
		int _positionOffset;
		// Displacement to the normal on the vertexbuffer
		int _normalOffset;
		// Displacement to the diffuse on the vertexbuffer
		int _diffuseOffset;
		// Displacement to the tangent on the vertexbuffer
		int _tangentOffset;
		// Displacement to the texCoord 0 on the vertexbuffer
		int _texCoordOffset;

		IDirect3DVertexDeclaration9* _declaration;
		IDirect3DVertexBuffer9* _dxResource;

		void BuildDeclaration();
		void UpdateDeclaration(D3DVERTEXELEMENT9* elements, int numElements);

	protected:
		// We have to override this to free our DX resources
		virtual void FreeResources() override
		{
			FREEDX(_dxResource);
			FREEDX(_declaration);
		}

		// And override this to update us correctly
		virtual void UpdateResources() override;
	public:
		// Size of the resource in bytes. Used when calling to SetStreamSource
		property int Size;

		property IDirect3DVertexDeclaration9* Declaration
		{
			IDirect3DVertexDeclaration9* get();
		}
		property IDirect3DVertexBuffer9* Resource
		{
			IDirect3DVertexBuffer9* get();
		}

		VertexBufferHandler(DXAdapter^ adapter, VertexBuffer^ vertexBuffer);
	};

}