using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Math;

using Jade.SceneGraph;
using Jade.SceneGraph.Transforms;
using Jade.Rendering;


namespace Jade.Core
{
	/// <summary>
	/// Basic camera class - this contains just the information required
	/// for doing a simple rendering
	/// </summary>
	public class Camera
	{
		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="Camera"/> class.
		/// </summary>
		public Camera()
		{
			_transform = new OrientationPosition();
		}
		#endregion

		ITransform _transform;
		/// <summary>
		/// Gets or sets the transform associated with this camera.
		/// </summary>
		/// <value>The transform.</value>
		public ITransform Transform
		{
			get { return _transform; }
			set { _transform = value; }
		}

		private float _fieldOfView = 45f;
		/// <summary>
		/// Gets or sets the field of view.
		/// </summary>
		/// <value>The field of view.</value>
		public float FieldOfView
		{
			get { return _fieldOfView; }
			set { _fieldOfView = value; }
		}

		private float _aspectRatio = 1f;
		/// <summary>
		/// Gets or sets the aspect ratio for this camera.
		/// </summary>
		/// <value>The aspect ratio.</value>
		public float AspectRatio
		{
			get { return _aspectRatio; }
			set { _aspectRatio = value; }
		}

		private float _nearPlane = 0.1f;
		/// <summary>
		/// Gets or sets the near plane.
		/// </summary>
		/// <value>The near plane.</value>
		public float NearPlane
		{
			get { return _nearPlane; }
			set { _nearPlane = value; }
		}

		private float _farPlane = 1000f;
		/// <summary>
		/// Gets or sets the far plane.
		/// </summary>
		/// <value>The far plane.</value>
		public float FarPlane
		{
			get { return _farPlane; }
			set { _farPlane = value; }
		}

		#region Methods

		/// <summary>
		/// Creates a render list for this camera
		/// </summary>
		public RenderList CreateRenderList(Node root, Node overlay)
		{
			RenderList renderList = new RenderList();
			if (root != null)
			{
				Vector3d pos = this.Transform.Position * -1.0; // Make a matrix to undo our translation
				root.Walk(new CullNodeWalker(renderList), Matrixd.CreateTranslation(ref pos));
			}

			if (overlay != null)
				overlay.Walk(new CullOverlayNodeWalker(renderList), Matrixd.Identity);

			return renderList;
		}

		#endregion
	}
}
