﻿using System;
using System.Collections.Generic;

namespace Jade.Core
{
	/// <summary>
	/// Interface to wrap an array and manage as a list.
	/// This is similar to <see cref="List{T}" />, but specialized for
	/// our use
	/// </summary>
	/// <typeparam name="T"></typeparam>
	public interface IArrayList<T>
	 where T : IEquatable<T>
	{
		/// <summary>
		/// Gets or sets the value at the specified index.
		/// </summary>
		/// <value></value>
		T this[int index] { get; set; }

		/// <summary>
		/// Retrieves the data managed as an ArraySegment
		/// </summary>
		/// <param name="readOnly">Set to false if you plan to edit the data.</param>
		ArraySegment<T> Data(bool readOnly);
		
		/// <summary>
		/// Retrieves the data managed as an ArraySegment
		/// </summary>
		/// <param name="readOnly">Set to false if you plan to edit the data.</param>
		/// <param name="offset">The zero-based index of the first element in the range.</param>
		/// <param name="count">The number of elements in the range.</param>
		/// <returns></returns>
		ArraySegment<T> Data(bool readOnly, int offset, int count);
		
		/// <summary>
		/// Gets the length.
		/// </summary>
		/// <value>The length.</value>
		int Length { get; }

		/// <summary>
		/// Copies from an existing array, setting us to a copy of the original array
		/// </summary>
		/// <param name="source">The source.</param>
		void CopyFrom(T[] source);

		/// <summary>
		/// Sets the array to an existing array
		/// </summary>
		/// <param name="array">The array.</param>
		void SetArray(T[] array);
	}
}
