using System;
using System.Threading;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Pluto.Core;

namespace Jade.Core
{
	/// <summary>
	/// Render loop which runs in a separate thread
	/// </summary>
	internal class RenderLoopThreaded : RenderLoopBase
	{
		Thread _ourThread;
		bool _running;

		/// <summary>
		/// Initializes a new instance of the <see cref="RenderLoopThreaded"/> class.
		/// </summary>
		/// <param name="serviceProvider">The service provider.</param>
		/// <param name="start">if set to <c>true</c> [start].</param>
		/// <param name="updateSpeed">The update speed.</param>
		public RenderLoopThreaded(IPlutoRootServiceProvider serviceProvider, bool start, UpdateSpeed updateSpeed)
			:base(serviceProvider,start,updateSpeed)
		{
			_running = false;
		}

		/// <summary>
		/// Hooks the on application idle message to our render loop's idle handler.
		/// </summary>
		/// <returns></returns>
		protected override bool OnStart()
		{
			if (_running == false)
			{
				_ourThread = new Thread(new ThreadStart(Run));
				_ourThread.IsBackground = true;
				this.ThreadContext = _ourThread;
				_running = true;
				_ourThread.Start();
			}
			return true;
		}

		/// <summary>
		/// Runs this instance.
		/// </summary>
		private void Run()
		{
			while (_running)
			{
				this.Frame();
				Sleep();
			}
			InvokeMethods();
		}

		/// <summary>
		/// Unhooks the on application idle message to our render loop's idle handler.
		/// </summary>
		protected override void OnStop()
		{
			if (_running)
			{
				_running = false;
				// If we're stopping from another thread, block until we know
				// that we've stopped
				//if (this._ourThread != Thread.CurrentThread)
					//_ourThread.Join();
				_ourThread = null;
			}
		}
	}
}
