using System;
using System.Threading;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using Pluto.Core;

namespace Jade.Core
{
	/// <summary>
	/// Render loop for Windows Forms Applications
	/// </summary>
	internal class RenderLoopWindowsForms : RenderLoopBase
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="RenderLoopWindowsForms"/> class.
		/// </summary>
		/// <param name="serviceProvider">The service provider.</param>
		/// <param name="start">if set to <c>true</c> [start].</param>
		/// <param name="updateSpeed">The update speed.</param>
		public RenderLoopWindowsForms(IPlutoRootServiceProvider serviceProvider, bool start, UpdateSpeed updateSpeed)
			:base(serviceProvider,start,updateSpeed)
		{
		}

		/// <summary>
		/// Hooks the on application idle message to our render loop's idle handler.
		/// </summary>
		/// <returns></returns>
		protected override bool OnStart()
		{
			Application.Idle += OnApplicationIdle;
			return true;
		}
		/// <summary>
		/// Unhooks the on application idle message to our render loop's idle handler.
		/// </summary>
		protected override void OnStop()
		{
			Application.Idle -= OnApplicationIdle;
		}


		/// <summary>
		/// Handles the Idle event of the Application control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void OnApplicationIdle(object sender, EventArgs e)
		{
			while (AppStillIdle)
				this.Frame();
		}

		/// <summary>
		/// Gets if there are messages waiting to be processed or not
		/// </summary>
		bool AppStillIdle
		{
			get
			{
				Sleep();
				Message msg;
				return !PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
			}
		}

		/// <summary>
		/// Peeks the message pump, checking to see if we're idle.
		/// </summary>
		/// <param name="msg">The MSG.</param>
		/// <param name="hWnd">The h WND.</param>
		/// <param name="messageFilterMin">The message filter min.</param>
		/// <param name="messageFilterMax">The message filter max.</param>
		/// <param name="flags">The flags.</param>
		/// <returns></returns>
		[System.Security.SuppressUnmanagedCodeSecurity]
		[DllImport("User32.dll", CharSet = CharSet.Auto)]
		private static extern bool PeekMessage(out Message msg, IntPtr hWnd, uint messageFilterMin, uint messageFilterMax, uint flags);
	}
}
