using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Math;

using Jade.Core;
using Jade.SceneGraph;
using Jade.Resources;

namespace Jade.Entities
{
	/// <summary>
	/// Class for creating an ambient light source
	/// </summary>
	public class AmbientLight : Light
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Light"/> class.
		/// </summary>
		public AmbientLight()
			: base(LightType.Ambient, Colors.White)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="Light"/> class.
		/// </summary>
		/// <param name="color">The color.</param>
		public AmbientLight(Color color)
			: base(LightType.Ambient, color)
		{
		}

		/// <summary>
		/// Returns true if this light potentially affects objects inside
		/// the specified BoundingBox or BoundingSphere
		/// </summary>
		/// <param name="transform">The transform being applied to this light.</param>
		/// <param name="box">The box.</param>
		/// <param name="sphere">The sphere.</param>
		/// <returns></returns>
		public override bool Intersects(Matrixd transform, ref Pluto.Math.BoundingBox box, ref Pluto.Math.BoundingSphere sphere)
		{
			return true;
		}
	}
}
