using System;
using System.Collections.Generic;
using System.Text;

using Jade.Core;
using Jade.SceneGraph;
using Jade.Resources;

namespace Jade.Entities
{
	/// <summary>
	/// Class for creating a single omni light source
	/// </summary>
	public class SpotLight : Light
	{
		/// <summary>
		/// Class to hold the properties of a spot light projector
		/// </summary>
        public class ProjectorProperties
        {
            bool _enabled;
            /// <summary>
            /// Get/Set the projector enabled status
            /// </summary>
            public bool Enabled
            {
                get { return _enabled; }
                set { _enabled = value; }
            }

            Texture _texture;
            /// <summary>
            /// The Texture used for projector
            /// </summary>
            public Texture Texture
            {
                get { return _texture; }
                set { _texture = value; }
            }

            float _apertureFactor=1f;
            /// <summary>
            /// Get/Sets the aperture factor. Defaults 1f, a major value makes the texture appear greater.
            /// A minor value makes it appear smaller.
            /// </summary>
            public float ApertureFactor
            {
                get { return _apertureFactor; }
                set { _apertureFactor = value; }
            }
        }

        ProjectorProperties _projector = new ProjectorProperties();

        /// <summary>
        /// The projector properties
        /// </summary>
        public ProjectorProperties Projector
        {
            get { return _projector; }
            set { _projector = value; }
        }

        private float _outerConeAngle;
        private float _innerConeAngle;
        /// <summary>
		/// This is the outer cone angle ( in degrees). Values outside this cone are black
        /// </summary>
        public float OuterConeAngle
        {
            get { return _outerConeAngle; }
            set { _outerConeAngle = value; }
        }

        /// <summary>
        /// This is the inner cone angle ( in degrees). This is when the attenuation start
        /// </summary>
        public float InnerConeAngle
        {
            get { return _innerConeAngle; }
            set { _innerConeAngle = value; }
        }

       /// <summary>
		/// Initializes a new instance of the <see cref="Light"/> class.
		/// </summary>
		public SpotLight()
			: base(LightType.Spot, Colors.White)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="Light"/> class.
		/// </summary>
		/// <param name="color">The color.</param>
        public SpotLight(Color color)
            : base(LightType.Spot, color)
		{
            _outerConeAngle = 45f;
            _innerConeAngle = 30f;
        }
	}
}
