using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Math;

using Jade.Core;
using Jade.Resources;


namespace Jade.Entities
{
	/// <summary>
	/// <see cref="IEntity"/> for handling 2D Text in pixel space
	/// </summary>
	public class OverlayTextEntity: IEntity
	{
		#region Fields and Properties

		private Vector2 _position = new Vector2(0f,0f);
		/// <summary>
		/// Gets or sets the position in pixel space.
		/// </summary>
		/// <value>The position.</value>
		public Vector2 Position
		{
			get { return _position; }
			set { _position = value; }
		}

		private string _text = "";
		/// <summary>
		/// Gets or sets the text.
		/// </summary>
		/// <value>The text.</value>
		public string Text
		{
			get { return _text; }
			set { _text = value; }
		}

		private OverlayTextFont _font;
		/// <summary>
		/// Gets or sets the font.
		/// </summary>
		/// <value>The font.</value>
		public OverlayTextFont Font
		{
			get { return _font; }
			set { _font = value; }
		}

		private Color _color = Colors.White;
		/// <summary>
		/// Gets or sets the color.
		/// </summary>
		/// <value>The color.</value>
		public Color Color
		{
			get { return _color; }
			set { _color = value; }
		}

		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="OverlayTextEntity"/> class.
		/// </summary>
		/// <param name="text">The text.</param>
		/// <param name="position">The position.</param>
		/// <param name="color">The color.</param>
		/// <param name="font">The font.</param>
		public OverlayTextEntity(string text, Vector2 position, Color color, OverlayTextFont font)
		{
			_text = text;
			_position = position;
			_color = color;
			_font = font;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="OverlayTextEntity"/> class.
		/// </summary>
		/// <param name="text">The text.</param>
		/// <param name="position">The position.</param>
		/// <param name="color">The color.</param>
		public OverlayTextEntity(string text, Vector2 position, Color color)
			: this(text,position,color,null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="OverlayTextEntity"/> class.
		/// </summary>
		public OverlayTextEntity()
		{
		}
		#endregion

		#region IEntity Members

		/// <summary>
		/// Called during update.
		/// </summary>
		/// <param name="args">The <see cref="Jade.SceneGraph.NodeWalkerArgs"/>.  This provides access to the current <see cref="Jade.SceneGraph.Node"/> being updated</param>
		public virtual void OnUpdate(Jade.SceneGraph.NodeWalkerArgs args)
		{
		}

		#endregion
	}
}
