#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using Jade.Core;
using Jade.Entities;
using Jade.SceneGraph;
using Jade.SceneGraph.Transforms;
using Jade.Rendering;
using Jade.Rendering.Utility;
using Jade.Resources;
using Jade.Input;
using Jade.Rendering.States;
using Jade.Rendering.Enums;

using Pluto.Core;
using Pluto.Math;
using Pluto.VFS;
using Pluto.Configuration;

using Jade.Import.DeleD;

#endregion

namespace Jade.Import
{
	/// <summary>
	/// Main class to import DeleD files v1.6. Currently only supported triangulated primitives.
    /// Get DeleD at http://www.delgine.com/
	/// </summary>
	public class DeleDImporter
	{
		#region Fields

		scene _delEd;

        Node _sceneRoot;

        /// <summary>
        /// The materials library
        /// </summary>
		DelEdMaterials _materials;

        DeleDPrimitives _primitives;

        /// <summary>
        /// The scale value to apply to vertices
        /// </summary>
		Vector3 _scale;

        Game _game;

        /// <summary>
        /// List of meshes imported
        /// </summary>
        internal List<Mesh> _meshes;

		#endregion

		#region Properties

        /// <summary>
        /// Gets the list of meshes imported
        /// </summary>
        public List<Mesh> Meshes
        {
            get { return _meshes; }
        }

        internal Game Game
        {
            get { return _game; }
        }

        /// <summary>
        /// The materials library
        /// </summary>
        internal DelEdMaterials Materials
        {
            get { return _materials; }
        }

        /// <summary>
        /// Gets the scale value to apply to vertices
        /// </summary>
        internal Vector3 Scale
        {
            get { return _scale; }
        }

		internal Node SceneRoot
		{
			get { return _sceneRoot; }
		}

		/// <summary>
		/// Gets the DelED object
		/// </summary>
		internal scene DelEd
		{
			get { return _delEd; }
		}


		#endregion

        #region Constructor

        /// <summary>
		/// Default constructor
		/// </summary>
        public DeleDImporter(Game game)
		{
            _game = game;
		}

        #endregion

        #region Methods

        /// <summary>
        /// Loads a DeleD file and creates all the objects into Jad
        /// </summary>
        /// <param name="sceneRoot">The scene root.</param>
        /// <param name="scale">Scale</param>
        /// <param name="stream">The stream.</param>
        /// <returns>True if everything is ok</returns>
        public bool Load(Node sceneRoot, float scale, Stream stream)
        {
            return Load(sceneRoot, new Vector3(scale, scale, scale), stream);
        }

        /// <summary>
		/// Loads a DeleD file and creates all the objects into Jad
		/// </summary>
		/// <param name="sceneRoot">The scene root.</param>
		/// <param name="scale">Scale</param>
		/// <param name="stream">The stream.</param>
		/// <returns>True if everything is ok</returns>
		public bool Load(Node sceneRoot, Vector3 scale, Stream stream)
		{
            _sceneRoot = sceneRoot;

			_scale = scale;

            _meshes = new List<Mesh>();

			Log.WriteLine("Importing DeleD scene");

			XmlSerializer sr = new XmlSerializer(typeof(scene));
			_delEd = (scene)(sr.Deserialize(stream));

			CreateScene();

			Log.WriteLine("Finishing importing DeleD scene");

			return true;
		}

		void CreateScene()
		{
			_materials = new DelEdMaterials(this);

            _primitives = new DeleDPrimitives(this);
		}

        #endregion
    }
}
