#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using Jade.Core;
using Jade.Entities;
using Jade.SceneGraph;
using Jade.SceneGraph.Transforms;
using Jade.Rendering;
using Jade.Rendering.Utility;
using Jade.Resources;
using Jade.Input;
using Jade.Rendering.States;
using Jade.Rendering.Enums;

using Pluto.Core;
using Pluto.Math;
using Pluto.VFS;
using Pluto.Configuration;

using Jade.Import.DeleD;

#endregion

namespace Jade.Import.DeleD
{
    /// <summary>
    /// Used to Load the materials that comes from the DeleD file
    /// </summary>
    internal class DelEdMaterials
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        DeleDImporter _deleD;

        Dictionary<string, object> _materials;

        #endregion

        #region Properties

        #endregion

        #region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		/// <param name="delEd">The del ed.</param>
        internal DelEdMaterials(DeleDImporter delEd)
        {
            _deleD = delEd;

            _materials = new Dictionary<string, object>();

            // Due to DXS files has all the texture information, even
            // the texutre is not used, we first build the material list
            // and when the material is needed we will Load it.

            foreach (sceneMaterials sceneMat in _deleD.DelEd.materials)
            {
                foreach (sceneMaterialsCategory category in sceneMat.category)
                {
                    foreach (sceneMaterialsCategoryMaterial material in category.material)
                    {
                        _materials.Add(material.id, material);
                        //Load(material);
                    }
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the material given an id
        /// </summary>
        /// <param name="id">The material ID</param>
        /// <returns>The Material found. If it's not found, this method will raise an exception</returns>
        internal Material GetMaterial(string id)
        {
            object mat=_materials[id];

            if (mat is Material) return (Material)mat;
            
            sceneMaterialsCategoryMaterial material=(sceneMaterialsCategoryMaterial)mat;

            Material jadMat=Load(material);

            _materials[id] = jadMat;

            return jadMat;

        }

        /// <summary>
        /// Loads the image tag and creates the 2D texture
        /// The texture name will be the img.id tag
        /// </summary>
        Material Load(sceneMaterialsCategoryMaterial material)
        {
            Material mat = new Material();  // material.name

            int numLayer = 0;

            //JMaterialLayer jLayer = null;

            foreach (sceneMaterialsCategoryMaterialLayer layer in material.layer)
            {
                bool newLayer = true;

                if (numLayer == 1 && layer.type == "lightmap")
                {
                    // If we have a lightmap, we do not need to create
                    // another layer
                    newLayer = false;
                }

                if (newLayer)
                {
                    //jLayer = new JMaterialLayer();
                }
                
                //string file = Path.GetFileNameWithoutExtension(layer.texture[0].file);
                string file = Path.GetFileName(layer.texture[0].file);

                if (file != "system.bmp")
                {
                    Log.WriteLine("Texture:" + file);

                    Texture texture = new Texture(_deleD.Game.Resources.VFS, file, @"Textures");

                    if (layer.type == "texture")
                    {
                        //jLayer.DiffuseMap.Texture = texture;

                        //jLayer.DiffuseMap.TextureEnabled = true;

                        mat.DiffuseMap.Texture = texture;

                        mat.DiffuseMap.TextureEnabled = true;
                    }

                    if (layer.type == "lightmap")
                    {
                        mat.LightMap.Texture = texture;

                        mat.LightMap.Enabled = true;
                    }
                }
                //if (newLayer) mat.AddLayer(jLayer);

                numLayer++;
            }

            return mat;
        }

        #endregion
    }
}
