#region Using Directives

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

using System.IO;
using System.Xml.Serialization;

using Jade.Core;
using Jade.Entities;
using Jade.SceneGraph;
using Jade.SceneGraph.Transforms;
using Jade.Rendering;
using Jade.Rendering.Utility;
using Jade.Resources;
using Jade.Input;
using Jade.Rendering.States;
using Jade.Rendering.Enums;

using Pluto.Core;
using Pluto.Math;
using Pluto.VFS;
using Pluto.Configuration;

using Jade.Import;

#endregion

namespace Jade.Import.DeleD
{
    /// <summary>
    /// Used to Load the images that comes from the collada file
    /// </summary>
    internal class DeleDPrimitives
    {
        #region Fields

        /// <summary>
        /// The parent object
        /// </summary>
        DeleDImporter _deleD;

        NumberFormatInfo _decimalFormatInfo;

        #endregion

        #region Properties

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="delEd">The parent object</param>
        internal DeleDPrimitives(DeleDImporter delEd)
        {
            _deleD = delEd;

            _decimalFormatInfo = new NumberFormatInfo();
            _decimalFormatInfo.NumberDecimalSeparator = ".";

            foreach (scenePrimitives scenePrimitive in _deleD.DelEd.primitives)
            {
                foreach (scenePrimitivesPrimitive primitive in scenePrimitive.primitive)
                {
                    LoadPrimitive(primitive);
                }
            }
        }

        #endregion

        #region Methods
        
        Vector3[] vertices;

        List<MaterialInfo> _materialList;

        /// <summary>
        /// Fill the vertices array
        /// </summary>
        /// <param name="primitive">The primitive source</param>
        void FillVertices(scenePrimitivesPrimitive primitive)
        {
            vertices = new Vector3[primitive.vertices.Length];

            int i = 0;

            foreach (vertex ivertex in primitive.vertices)
            {
                float x = float.Parse(ivertex.x, _decimalFormatInfo);
                float y = float.Parse(ivertex.y, _decimalFormatInfo);
                float z = float.Parse(ivertex.z, _decimalFormatInfo);

                Vector3 position = new Vector3(x, y, z);

                position.X *= _deleD.Scale.X;
                position.Y *= _deleD.Scale.Y;
                position.Z *= _deleD.Scale.Z;

                vertices[i++] = position;

            }

        }
        internal struct MaterialInfo
        {
            internal int polyMid;
            internal Material Material;
        }
        /// <summary>
        /// Get the number of materials needed
        /// </summary>
        /// <param name="primitive">The source primitive</param>
        /// <returns>The number of materials</returns>
        int GetNumMatFromPrimitive(scenePrimitivesPrimitive primitive)
        {
            _materialList = new List<MaterialInfo>();

            foreach (scenePrimitivesPrimitivePolygonsPoly poly in primitive.polygons)
            {
                int matID = int.Parse(poly.mid);

                bool found = false;

                foreach (MaterialInfo i in _materialList)
                {
                    if (matID == i.polyMid) { found = true; break; }
                }
                if (!found)
                {
                    MaterialInfo mi = new MaterialInfo();
                    mi.polyMid = matID;
                    mi.Material = _deleD.Materials.GetMaterial(poly.mid);
                    _materialList.Add(mi);
                }
            }

            int numMat = _materialList.Count;

            return numMat;
        }

		/*/// <summary>
		/// Gets the index material(starting at 0) given the material id
		/// </summary>
		/// <param name="matID">The DeleD material id for the poly</param>
		/// <returns>The layer index.-1 if not found</returns>
		int GetIndexMaterial(int matID)
		{
			int index=0;

			foreach (int i in _materialList)
			{
				if (matID == i) return index;
				index++;
			}

			return -1;
		}
		*/
        /// <summary>
        /// Loads the primitive and build the JMeshObject
        /// </summary>
        void LoadPrimitive(scenePrimitivesPrimitive primitive)
        {
            
            SubMesh mesh;

            //meshObject = _deleD.Scene.MeshObjects.Create(primitive.name);

            int numMatUsed = GetNumMatFromPrimitive(primitive);

            //meshObject.Material = new JMaterial[numMatUsed];

            VertexCollector collector = new VertexCollector();

            FillVertices(primitive);

            collector.Initialize(false);

            foreach (scenePrimitivesPrimitivePolygonsPoly poly in primitive.polygons)
            {
                int matID = int.Parse(poly.mid);

                //materialList[GetIndexMaterial(matID)]= _deleD.Materials.GetMaterial(poly.mid);
                //meshObject.Material[GetIndexMaterial(matID)] = _deleD.Materials.GetMaterial(poly.mid);

                //material = _deleD.Materials.GetMaterial(poly.mid);

                foreach (vertex ivertex in poly.vertex)
                {
					if (poly.vertex.Length != 3)
					{
						throw new Exception("You must triangulate the scene to import it");
					}

                    collector.Begin(matID);

                    collector.AddPosition(vertices[int.Parse(ivertex.vid)]);

                    float u = float.Parse(ivertex.u0, _decimalFormatInfo);
                    float v = float.Parse(ivertex.v0, _decimalFormatInfo);

                    Vector2 texCoord = new Vector2(u, v);

                    collector.AddTexCoord(0,texCoord);

                    if (ivertex.u1 != "" && ivertex.u1!=null)
                    {
                        u = float.Parse(ivertex.u1, _decimalFormatInfo);
                        v = float.Parse(ivertex.v1, _decimalFormatInfo);

                        texCoord = new Vector2(u, v);

                        collector.AddTexCoord(1, texCoord);
                    }

                    collector.End();
                }
            }

            collector.Finish();

			Log.WriteLine("Primitive:" + primitive.name + ". VertexCount=" + collector.VertexBuffer.VertexCount + ". IndexCount=" + collector.IndexBuffer.Count + ". Num Materials:" + collector.NumberOfMaterials);

            // We do this now to compute the Normals & Tangent for all the Vertices

            collector.IndexBuffer.InvertFacesPersistent();

            mesh = new SubMesh();
            
            collector.FillMesh(mesh);

            //mesh.VertexBuffer.InvertNormals();

            mesh.GenerateVertexNormalsAndTangents();

            // Create a renderable object (Mesh)
            Mesh renderable = new Mesh();

            // Add it to the list
            _deleD._meshes.Add(renderable);

            if (collector.NumberOfMaterials == 1)
            {
                RenderState renderState = new RenderState(_materialList[0].Material);

                renderable.Add(mesh, renderState);
            }
            else
            {
                for (int i = 0; i < collector.NumberOfMaterials; i++)
                {
                    mesh = new SubMesh();

                    collector.FillMesh(mesh, i);

                    RenderState renderState = new RenderState(_materialList[i].Material);

                    renderable.Add(mesh, renderState);
                }
            }
            OrientationPosition transform = new OrientationPosition();
            
            Node node = new Node(renderable, transform);

            _deleD.SceneRoot.AttachChild(node);

        }

        #endregion
    }
}
