using Pluto.Math;
using System.Collections.Generic;
using System;
using System.IO;
using System.Globalization;
using Jade.Core;
namespace Jade.Import
{
  public class ObjModel
  {
    private static readonly NumberFormatInfo _numberFormatInfo = System.Globalization.NumberFormatInfo.InvariantInfo;

    public ObjModel()
    {
      //this._materials.Add(new ObjMaterial("default"));
    }

    //// Path to this model
    //private string _pathName;
    //// Name of the material library
    //private string _mtlLibName;

    // All of the vertices in the model
    private List<ObjVertex> _vertices = new List<ObjVertex>();
    // The triangles in the model
    private List<ObjTriangle> _triangles = new List<ObjTriangle>();
    // The lines in the model
    private List<ObjLine> _lines = new List<ObjLine>();
    // All of the materials in the model
    private List<ObjMaterial> _materials = new List<ObjMaterial>();
    // Groups in the model
    private List<ObjGroup> _groups = new List<ObjGroup>();
    // position of the model
    private Vector3 _position;
    private BoundingBox _bbox = new BoundingBox();
    private BoundingBox _originalBoundingBox;

    //public string PathName
    //{
    //  get { return this._pathName; }
    //  set { this._pathName = value; }
    //}

    //public string MaterialLibraryPathName
    //{
    //  get { return this._mtlLibName; }
    //  set { this._mtlLibName = value; }
    //}

    public List<ObjVertex> Vertices
    {
      get { return this._vertices; }
      set { this._vertices = value; }
    }

    public List<ObjTriangle> Triangles
    {
      get { return this._triangles; }
      set { this._triangles = value; }
    }

    public List<ObjLine> Lines
    {
      get { return this._lines; }
      set { this._lines = value; }
    }

    public List<ObjMaterial> Materials
    {
      get { return this._materials; }
      set { this._materials = value; }
    }

    public List<ObjGroup> Groups
    {
      get { return this._groups; }
      set { this._groups = value; }
    }

    public Vector3 Position
    {
      get { return this._position; }
      set { this._position = value; }
    }

    public BoundingBox BoundingBox
    {
      get { return this._bbox; }
      set { this._bbox = value; }
    }

    public BoundingBox OriginalBoundingBox
    {
      get { return this._originalBoundingBox; }
      internal set { this._originalBoundingBox = value; }
    }

    internal void UpdateBoundingBox(Vector3 position)
    {
      Vector3 min = new Vector3();
      Vector3 max = new Vector3();

      min.X = Math.Min(this._bbox.Min.X, position.X);
      min.Y = Math.Min(this._bbox.Min.Y, position.Y);
      min.Z = Math.Min(this._bbox.Min.Z, position.Z);

      max.X = Math.Max(this._bbox.Max.X, position.X);
      max.Y = Math.Max(this._bbox.Max.Y, position.Y);
      max.Z = Math.Max(this._bbox.Max.Z, position.Z);

      this._bbox.Min = min;
      this._bbox.Max = max;
      //this.BoundingBox.MinX = Math.Min(this.BoundingBox.MinX, position.X);
      //this.BoundingBox.MaxX = Math.Max(this.BoundingBox.MaxX, position.X);

      //this.BoundingBox.MinY = Math.Min(this.BoundingBox.MinY, position._y);
      //this.BoundingBox.MaxY = Math.Max(this.BoundingBox.MaxY, position._y);

      //this.BoundingBox.MinZ = Math.Min(this.BoundingBox.MinZ, position._z);
      //this.BoundingBox.MaxZ = Math.Max(this.BoundingBox.MaxZ, position._z);
    }

    //internal void ReadMaterials(Stream matStream)
    //{

    //  //string directory = Path.GetDirectoryName(this.PathName);
    //  //string libFile = Path.Combine(directory, fileName);
    //  ObjMaterial currentMaterial = null;

    //  //if (!File.Exists(libFile))
    //  //  throw new FileNotFoundException("Material library '" + libFile + "' not found");

    //  //StreamReader reader = File.OpenText(libFile);
    //  StreamReader reader = new StreamReader(matStream);
    //  while (reader.Peek() >= 0)
    //  {
    //    string line = reader.ReadLine();
    //    if (line.Length == 0)
    //      continue;

    //    string[] elements = line.Trim().Replace("  ", " ").Split(new char[] { ' ', '\n' });
    //    if (elements.Length == 0)
    //      continue;

    //    switch (elements[0])
    //    {
    //      case "#":
    //        break;
    //      case "newmtl":
    //        if (currentMaterial != null)
    //          this._materials.Add(currentMaterial);
    //        currentMaterial = new ObjMaterial(elements[1]);
    //        break;
    //      case "Ns":
    //        currentMaterial.Shininess = Convert.ToSingle(elements[1], _numberFormatInfo);
    //        // Convert to Jade shininess = specularity power ([0; 1000] -> [0; 128])
    //        currentMaterial.Shininess /= 1000;
    //        currentMaterial.Shininess *= 128;
    //        break;
    //      case "Kd":
    //        currentMaterial.Diffuse = new Color(Convert.ToSingle(elements[1], _numberFormatInfo),
    //            Convert.ToSingle(elements[2], _numberFormatInfo),
    //            Convert.ToSingle(elements[3], _numberFormatInfo),
    //            1.0f);
    //        break;
    //      case "Ks":
    //        currentMaterial.Specular = new Color(Convert.ToSingle(elements[1], _numberFormatInfo),
    //            Convert.ToSingle(elements[2], _numberFormatInfo),
    //            Convert.ToSingle(elements[3], _numberFormatInfo),
    //            1.0f);
    //        break;
    //      case "Ka":
    //        currentMaterial.Ambient = new Color(Convert.ToSingle(elements[1], _numberFormatInfo),
    //            Convert.ToSingle(elements[2], _numberFormatInfo),
    //            Convert.ToSingle(elements[3], _numberFormatInfo),
    //            1.0f);
    //        break;
    //    }
    //  }

    //  reader.Close();
    //}

    internal uint FindMaterial(string materialId)
    {
      for (uint i = 0; i < this._materials.Count; i++)
        if (this._materials[(int)i].Name == materialId)
          return i;

      return 0;
    }

    public void BuildLists(out float[] positions, out float[] normals, out float[] texCoords,
        ref Dictionary<uint, uint[]> indicesPerMaterial, out uint[] lineIndices)
    {
      // Build position, normal and texCoord arrays
      positions = new float[this._vertices.Count * 3];
      normals = new float[this._vertices.Count * 3];
      texCoords = new float[this._vertices.Count * 2];
      for (int i = 0; i < this._vertices.Count; i++)
      {
        ObjVertex currentVertex = this._vertices[i];

        positions[i * 3] = currentVertex.Position.X;
        positions[i * 3 + 1] = currentVertex.Position.Y;
        positions[i * 3 + 2] = currentVertex.Position.Z;

        normals[i * 3] = currentVertex.Normal.X;
        normals[i * 3 + 1] = currentVertex.Normal.Y;
        normals[i * 3 + 2] = currentVertex.Normal.Z;

        texCoords[i * 2] = currentVertex.TexCoord.X;
        texCoords[i * 2 + 1] = currentVertex.TexCoord.Y;
      }

      // Sort the triangle list by materials
      if (this._triangles.Count > 0)
      {

        Dictionary<uint, List<uint>> tempIndicesByMaterial = new Dictionary<uint, List<uint>>();
        List<ObjTriangle> sortedTriangles = new List<ObjTriangle>(this._triangles);
        sortedTriangles.Sort(new Comparison<ObjTriangle>(delegate(ObjTriangle tri1, ObjTriangle tri2)
        {
          return tri1.MaterialIndex.CompareTo(tri2.MaterialIndex);
        }));

        // Build the index list for each material
        uint currentMaterialIndex = uint.MaxValue;
        List<uint> currentIndexList = null;
        for (int i = 0; i < sortedTriangles.Count; i++)
        {
          if (currentMaterialIndex != sortedTriangles[i].MaterialIndex)
          {
            currentIndexList = new List<uint>();
            tempIndicesByMaterial.Add(sortedTriangles[i].MaterialIndex, currentIndexList);
            currentMaterialIndex = sortedTriangles[i].MaterialIndex;
          }

          currentIndexList.Add(sortedTriangles[i].VertexIndices[0]);
          currentIndexList.Add(sortedTriangles[i].VertexIndices[1]);
          currentIndexList.Add(sortedTriangles[i].VertexIndices[2]);
        }

        // Copy index table to OpenGL usable index table
        foreach (KeyValuePair<uint, List<uint>> pair in tempIndicesByMaterial)
          indicesPerMaterial.Add(pair.Key, pair.Value.ToArray());
      }

      lineIndices = null;
      if (this._lines.Count > 0)
      {
        // Create lines index array
        lineIndices = new uint[this._lines.Count * 2];
        for (int i = 0; i < this._lines.Count; i++)
        {
          lineIndices[i * 2 + 0] = this._lines[i].VertexIndices[0];
          lineIndices[i * 2 + 1] = this._lines[i].VertexIndices[1];
        }
      }
    }

    public void CenterOnOrigin()
    {
      Vector3 min = new Vector3(this._bbox.Min.X, this._bbox.Min.Y, this._bbox.Min.Z);
      Vector3 offset = new Vector3(Math.Abs(this._bbox.Max.X - this._bbox.Min.X) / 2,
          Math.Abs(this._bbox.Max.Y - this._bbox.Min.Y) / 2,
          Math.Abs(this._bbox.Max.Z - this._bbox.Min.Z) / 2);

      foreach (ObjVertex vertex in this._vertices)
        vertex.Position -= min + offset;

      this._bbox.Min.X -= min.X + offset.X;
      this._bbox.Max.X -= min.X + offset.X;

      this._bbox.Min.Y -= min.Y + offset.Y;
      this._bbox.Max.Y -= min.Y + offset.Y;

      this._bbox.Min.Z -= min.Z + offset.Z;
      this._bbox.Max.Z -= min.Z + offset.Z;
    }

  }
}