namespace Jade.Input
{
	/// <summary>
	/// Wrapper to handle a single button press in the keyboard or joysticks.
	/// This manages the difference between down and pressed for us, 
	/// so we can differentiate between them.
	/// </summary>
	public class ButtonValue
	{
		#region Static Fields

		/// <summary>
		/// A blank (0) value
		/// </summary>
		public readonly static ButtonValue Untouched = new ButtonValue();

		#endregion

		#region Fields and Properties

		private bool _down;
		/// <summary>
		/// True when the button is being held down
		/// </summary>
		public bool Down
		{
			get { return _down; }
			set
			{
				if (value)
					_down = _pressed = true;
				else
					_down = false;
			}
		}

		private bool _pressed;
		/// <summary>
		/// Set to true when the button was pushed and released
		/// </summary>
		public bool Pressed
		{
			get
			{
				if (!_down && _pressed)
				{
					_pressed = false;
					return true;
				}
				return false;
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Reinitialize us
		/// </summary>
		public void Clear()
		{
			_down = _pressed = false;
		}

		#endregion
	}
}
