#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;

using Pluto.Core;

using Jade.Rendering;

#endregion

namespace Jade.Rendering
{
	/// <summary>
	/// The base interface for all renderer classes to support.
	/// Each render system must implement this class, and register it via
	/// a plugin in Pluto.  The engine initialization will load and set
	/// the appropriate renderer at startup, based off the user's capabilities
	/// and preferences.
	/// 
	/// A user of the engine does not need to know what type of renderer
	/// the end-user will be using.  Everything is programmed against this
	/// interface and the classes created here.  Each render system can
	/// provide their own internal implementation of any class required
	/// by it, which provides full control and flexibility in how different
	/// rendering systems work internally.
	/// </summary>
	[CompletionStatus(DevelopmentState.NearCompletion, Notes = "Device Restore")]
	public interface IRenderingSystem : IDisposable
	{
		#region Fields and Properties
		
		/// <summary>
		/// Gets the name of this renderer.
		/// </summary>
		/// <value>The name.</value>
		string Name
		{
			get;
		}

		/// <summary>
		/// Gets the adapters managed by this instance.
		/// </summary>
		/// <value>The adapters.</value>
		IList<Adapter> Adapters
		{
			get;
		}
		#endregion
	}
}
