using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;

namespace Jade.Rendering
{
	/// <summary>
	/// Rendering system factory for the DX9 renderer
	/// </summary>
	public abstract class RenderingSystemFactory
	{
		// The rendering system singleton
		IRenderingSystem _renderer;
		// The root service provider
		private IPlutoRootServiceProvider _provider;

		/// <summary>
		/// Gets the root service provider.
		/// </summary>
		/// <value>The provider.</value>
		protected IPlutoRootServiceProvider Provider
		{
			get { return _provider; }
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="RenderingSystemFactory"/> class.
		/// </summary>
		/// <param name="provider">The provider.</param>
		public RenderingSystemFactory(IPlutoRootServiceProvider provider)
		{
			_provider = provider;
		}

		/// <summary>
		/// Gets the rendering system.
		/// </summary>
		/// <value>The rendering system.</value>
		public IRenderingSystem RenderingSystem
		{
			get
			{
				if (_renderer == null)
					_renderer = CreateRenderSystem();
				return _renderer;
			}
		}

		/// <summary>
		/// Creates the render system.
		/// </summary>
		/// <returns></returns>
		protected abstract IRenderingSystem CreateRenderSystem();
	}
}
