using System;
using System.Collections.Generic;
using System.Text;

namespace Jade.Rendering.Enums
{
    /// <summary>
    /// Blend options. A blend option identifies the data source and an optional pre-blend operation
    /// </summary>
    public enum BlendOption
    {
        /// <summary>
        /// The data source is the color black (0, 0, 0, 0). No pre-blend operation. 
        /// </summary>
        Zero=1,

        /// <summary>
        /// The data source is the color white (1, 1, 1, 1). No pre-blend operation. 
        /// </summary>
        One=2,

        /// <summary>
        /// The data source is color data (RGB) from a pixel shader. No pre-blend operation. 
        /// </summary>
        SrcColor=3,

        /// <summary>
        /// The data source is color data (RGB) from a pixel shader. The pre-blend operation inverts the data, generating 1 - RGB. 
        /// </summary>
        InvSrcColor=4,

        /// <summary>
        /// The data source is alpha data (A) from a pixel shader. No pre-blend operation.
        /// </summary>
        SrcAlpha=5,

        /// <summary>
        /// The data source is alpha data (A) from a pixel shader. The pre-blend operation inverts the data, generating 1 - A. 
        /// </summary>
        InvSrcAlpha=6,

        /// <summary>
        /// The data source is alpha data from a rendertarget. No pre-blend operation. 
        /// </summary>
        DestAlpha=7,

        /// <summary>
        /// The data source is alpha data from a rendertarget. The pre-blend operation inverts the data, generating 1 - A. 
        /// </summary>
        InvDestAlpha=8,

        /// <summary>
        /// The data source is color data from a rendertarget. No pre-blend operation. 
        /// </summary>
        DestColor=9,

        /// <summary>
        /// The data source is color data from a rendertarget. The pre-blend operation inverts the data, generating 1 - RGB. 
        /// </summary>
        InvDestColor=10,

        /// <summary>
        /// The data source is alpha data from a pixel shader. The pre-blend operation clamps the data to 1 or less. 
        /// </summary>
        SrcAlphaSat=11,

        /// <summary>
        /// The data source is the blend factor. No pre-blend operation. 
        /// </summary>
        Factor=14,

        /// <summary>
        /// The data source is the blend factor. The pre-blend operation inverts the blend factor, generating 1 - blend_factor. 
        /// </summary>
        InvFactor=15,

        /// <summary>
        /// The data sources are both color data output by a pixel shader. There is no pre-blend operation. This options supports dual-source color blending. 
        /// </summary>
        Src1Color=20,

        /// <summary>
        /// The data sources are both color data output by a pixel shader. The pre-blend operation inverts the data, generating 1 - RGB. This options supports dual-source color blending. 
        /// </summary>
        InvSrc1Color=21,

        /// <summary>
        /// The data sources are alpha data output by a pixel shader. There is no pre-blend operation. This options supports dual-source color blending. 
        /// </summary>
        Src1Alpha=22,

        /// <summary>
        /// The data sources are alpha data output by a pixel shader. The pre-blend operation inverts the data, generating 1 - A. This options supports dual-source color blending. 
        /// </summary>
        InvSrc1Alpha=23

    }
}
