using System;
using System.Collections.Generic;
using System.Text;

namespace Jade.Rendering.Enums
{
	/// <summary>
	/// Defines the primitives supported by the engine
	/// </summary>
	public enum PrimitiveType
	{
		/// <summary>
		///  Renders the vertices as a triangle fan. 
		/// </summary>
		TriangleFan=6,

		/// <summary>
		/// Renders the vertices as a triangle strip. The back face culling flag is flipped automatically on even-numbered triangles. 
		/// </summary>
		TriangleStrip=5,

		/// <summary>
		/// Renders the specified vertices as a sequence of isolated triangles. Each group of three vertices defines a separate triangle. Back face culling is affected by the current winding-order render state. 
		/// </summary>
		TriangleList=4,

		/// <summary>
		/// Renders the vertices as a single polyline. Calls that use this primitive type fail if the count is less than two. 
		/// </summary>
		LineStrip=3,

		/// <summary>
		/// Renders the vertices as a list of isolated straight line segments. Calls that use this primitive type fail if the count is less than two or is odd. 
		/// </summary>
		LineList=2,

		/// <summary>
		/// Renders the vertices as a collection of isolated points. This value is unsupported for indexed primitives. 
		/// </summary>
		PointList=1
	}
}
