using System;
using System.Collections.Generic;
using System.Text;

namespace Jade.Rendering
{
    /// <summary>
    /// A gaussian blur post production effect
    /// </summary>
    public class BlurFilter:PostProductionEffect
    {

        public BlurFilter(RenderTargetBase destination, float sizeX, float sizeY)
            : base(destination,sizeX,sizeY)
        {
            _hLSL = @"

// This is the current texture
texture currentTexture;

struct VS_INPUT
{
   float4 position:     POSITION;
   float2 texcoord0:    TEXCOORD0;
};

struct VS_OUTPUT
{
   float4 position:     POSITION;
   float2 texcoord0:    TEXCOORD0;
};

struct PS_INPUT
{
   float2 texcoord0:     TEXCOORD0;
};

sampler map_diffuse = 
sampler_state
{
    Texture = <currentTexture>;
};

float4 BlurOffset;

VS_OUTPUT VShader( VS_INPUT vin )
{
   VS_OUTPUT vout;
   
   vout.position = vin.position;
   vout.texcoord0 = vin.texcoord0;

   return vout;
}

float PixelWeight[8];

float4 PShader(float2 texCoord: TEXCOORD0,uniform int numSamples) : COLOR 
{
	float4 sum = 0.0f;
	

	for (int i = 0; i < numSamples; i++) 
	{
		sum += tex2D(map_diffuse, texCoord + ( BlurOffset.xy * i ) ) * PixelWeight[i];
		sum += tex2D(map_diffuse, texCoord - ( BlurOffset.xy * i ) ) * PixelWeight[i];
		
	}
	
	return sum;

}


//	Permitimos 16 samples com mximo

int CurNumSamples = 3;
PixelShader psArray[8] = { 
	compile ps_2_0 PShader(1), 
	compile ps_2_0 PShader(2), 
	compile ps_2_0 PShader(3), 
	compile ps_2_0 PShader(4), 
	compile ps_2_0 PShader(5), 
	compile ps_2_0 PShader(6), 
	compile ps_2_0 PShader(7), 
	compile ps_2_0 PShader(8)
};

technique RenderScene
{
	pass p0
	{
		VertexShader = compile vs_2_0 VShader();
		PixelShader = (psArray[CurNumSamples]);
	}
}
";

        }

        public override void Process(PostProduction postProduction)
        {
            // Set the texture and fill the PixelWeight vars.
            // We should do this on C# instead of C++ because of easy code.
            // We could do some thing like :
            // Effect.Set("PixelWeight",new float[] { ... } );
            // Effect.Set("texture..")
            // If you build a C# effect file, we could remove the  DXEffectBuilder::SetMaterialShaderConstants
            // as the user should add is owns materials....

            // The base.Process only has to Render the Quad
            base.Process(postProduction);
        }
    }
}
