#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;

using Pluto.Core;
using Pluto.Math;

using Jade.Core;
using Jade.Rendering;
using Jade.Rendering.States;
using Jade.Entities;
using Jade.SceneGraph;
using Jade.Resources;
#endregion

namespace Jade.Rendering
{
	/// <summary>
	/// Contains the lists of objects to render for this frame and this light
	/// This is built during the cull pass of the engine
	/// </summary>
	public class LightRenderList : TransformedLight
	{
		List<TransformedMesh> _renderables;
		/// <summary>
		/// Gets the meshes to render.
		/// </summary>
		/// <value>The meshes to render.</value>
		public IList<TransformedMesh> Renderables
		{
			get { return _renderables; }
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LightRenderList"/> class.
		/// </summary>
		/// <param name="entity">The entity.</param>
		/// <param name="transform">The transform.</param>
		public LightRenderList(Light entity, Matrixd transform)
			:base(entity,transform)
		{
			_renderables = new List<TransformedMesh>();
		}
	}
}
