using System;
using System.Collections.Generic;
using System.Text;

namespace Jade.Rendering
{
	#region Enums

	/// <summary>
	/// The normal type used on the shaders
	/// </summary>
	public enum NormalType
	{
		/// <summary>
		/// We do not use normals
		/// </summary>
		None = 0,

		/// <summary>
		/// Vertex normals
		/// </summary>
		Vertex = 1,

		/// <summary>
		/// Bump mapping
		/// </summary>
		Mapping = 2
	}

	/// <summary>
	/// The order the material has to be rendered
	/// </summary>
	public enum MaterialPriority
	{
		/// <summary>
		/// Render in the transparent pass
		/// </summary>
		Transparency = 0,

		/// <summary>
		/// Normal priority
		/// </summary>
		Normal = 1,

		/// <summary>
		/// Render in the post-processing pass
		/// </summary>
		PostProcess = 2
	}

	/// <summary>
	/// The emmisive mode controls the Emissive pass render material
	/// </summary>
	public enum EmissiveMode
	{
		/// <summary>
		/// Use the emissive texture as source for emissive
		/// </summary>
		Texture = 1,

		/// <summary>
		/// Use the emissive color as source for the emissive
		/// </summary>
		Color = 2,

		/// <summary>
		/// Use the backbuffer as source for emissive
		/// </summary>
		Diffuse = 3
	}

	#endregion

}
