#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;
using Pluto.Math;

using Jade.Rendering;
using Jade.Rendering.Enums;
using Jade.Core;
using System.Collections;
using Jade.Rendering.States;
using Jade.Resources;

#endregion

namespace Jade.Rendering
{
    /// <summary>
	/// The material layer has the information to apply to the material
	/// </summary>
	public class Material
	{
		#region Fields

		uint _id;
		/// <summary>
		/// Get the unique id value for this object
		/// </summary>
		public uint ID
		{
			get { return _id; }
		}

		/// <summary>
		/// The material priority in rendering
		/// </summary>
		public MaterialPriority Priority;

		/// <summary>
		/// The Lighting Effect used in rendering
		/// </summary>
		public LightingEffect LightingEffect;

		/// <summary>
		/// DiffuseMap
		/// </summary>
		public DiffuseMap DiffuseMap;

		/// <summary>
		/// NormalMap
		/// </summary>
		public NormalMap NormalMap;

		/// <summary>
		/// SpecularMap
		/// </summary>
		public SpecularMap SpecularMap;

		/// <summary>
		/// EmissiveMap
		/// </summary>
		public EmissiveMap EmissiveMap;

		/// <summary>
		/// HeightMap
		/// </summary>
		public HeightMap HeightMap;

		/// <summary>
		/// ReflectionMap
		/// </summary>
		public ReflectionMap ReflectionMap;

		/// <summary>
		/// RefractionMap
		/// </summary>
		public RefractionMap RefractionMap;

		/// <summary>
		/// LightMap
		/// </summary>
		public LightMap LightMap;

		/// <summary>
		/// DetailMap
		/// </summary>
		public DetailMap DetailMap;

		/// <summary>
		/// Normal process used for this layer
		/// </summary>
		public NormalType NormalType;

        Color _ambient;
        /// <summary>
        /// The ambient color. This value will be added to Light.Ambient
        /// </summary>
        public Color Ambient
        {
            get { return _ambient; }
            set { _ambient = value; }
        }

        BlendState _blendState;

        /// <summary>
        /// The blend State used for this material. Not the this can oinly be used for Overlays and Post materials
        /// </summary>
        public BlendState BlendState
        {
            get { return _blendState; }
            set { _blendState = value; }
        }

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor
		/// </summary>
		public Material()
		{
			_id = Jad.NextUniqueID;
			Priority = MaterialPriority.Normal;
			LightingEffect = LightingEffect.Default;
			Initialize();
		}

		#endregion

		#region Methods

		/// <summary>
		/// Set the default values
		/// </summary>
		void Initialize()
		{
			// Default values
            _blendState = new BlendState();

			DiffuseMap.Initialize();

			SpecularMap.Initialize();

			EmissiveMap.Initialize();

			NormalType = NormalType.Vertex;
			NormalMap.Initialize();

			HeightMap.Initialize();

			ReflectionMap.Initialize();

			RefractionMap.Initialize();

			LightMap.Initialize();

			DetailMap.Initialize();

            _ambient.ToBlack();
		}

		#endregion
	}
}
