#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;

using Jade.Rendering;
using Jade.Rendering.Enums;
using Jade.Core;

#endregion

namespace Jade.Rendering
{
    /// <summary>
    /// A RenderTarget is an special texture used to render scenes.
    /// Usually, the main rendertarget is linked to the BackBuffer, so when
    /// the scene is rendered, the card can present the frame.
    /// But it has a lot of uses. You can create a RenderTarget to render
    /// a frame, then make some nice PostProcess on it, and Present the results.
    /// When the engine is created, a Primary RenderTarget is created, to
    /// identify the adapter BackBuffer.
    /// </summary>
	[CompletionStatus(DevelopmentState.Alpha, Notes = "Initial implementation in place")]
	public abstract class RenderTargetBase : IDisposable
	{
		#region Fields And Properties

		private int _width;
		/// <summary>
		/// The Width
		/// </summary>
		protected int Width
		{
			get { return _width; }
			set { _width = value; }
		}

		private int _height;
		/// <summary>
		/// The height
		/// </summary>
		protected int Height
		{
			get { return _height; }
			set { _height = value; }
		}

		#endregion

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="RenderTargetBase"/> class.
		/// </summary>
		/// <param name="width">Width</param>
		/// <param name="height">Height</param>
		public RenderTargetBase(int width, int height)
		{
			_width = width;
			_height = height;
		}

		#endregion

		#region Methods

		/// <summary>
		/// Tells the engine the new size due a Resize event
		/// </summary>
		/// <param name="width">The new window with</param>
		/// <param name="height">The new window height</param>
		public void Resize(int width, int height)
		{
			_width = width;
			_height = height;
		}

		#endregion

		#region IDisposable Members

		private bool _disposed;

		/// <summary>
		/// Releases unmanaged and - optionally - managed resources
		/// </summary>
		/// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
		protected virtual void Dispose(bool disposing)
		{
			if (!_disposed)
			{
				if (disposing)
				{
					// Dispose managed resources: 
				}

				// Dispose unmanaged resources
			}
			_disposed = true;
			// base.Dispose(disposing);
		}

		/// <summary>
		/// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion

	}
}

