using System;
using System.Collections.Generic;
using System.Text;
using Jade.Rendering.Enums;

namespace Jade.Rendering.States
{
    /// <summary>
    /// Stencil operations that can be performed based on the results of stencil test
    /// </summary>
    public struct DepthStencilOperation
    {
        /// <summary>
        /// The stencil operation to perform when stencil testing fails. 
        /// </summary>
        public StencilOperation Fail;

        /// <summary>
        /// The stencil operation to perform when stencil testing passes and depth testing fails.
        /// </summary>
        public StencilOperation DepthFail;

        /// <summary>
        /// The stencil operation to perform when stencil testing and depth testing both pass.  
        /// </summary>
        public StencilOperation Pass;

        /// <summary>
        /// A function that compares stencil data against existing stencil data
        /// </summary>
        public ComparisonFunc Comparisson;

        /// <summary>
        /// Initialize the states with the default values
        /// </summary>
        internal void Initialize()
        {
            Fail = StencilOperation.Zero;
            DepthFail = StencilOperation.Zero;
            Pass= StencilOperation.Zero;
            Comparisson = ComparisonFunc.Never;
        }
    }
}
