using System;
using System.Collections.Generic;
using System.Text;
using Jade.Rendering.Enums;

namespace Jade.Rendering.States
{

    /// <summary>
    /// Describes depth-stencil state.
    /// </summary>
    public class DepthStencilState
    {
		bool _depthEnable;
		/// <summary>
        /// Enable depth testing. 
        /// </summary>
        public bool DepthEnable
        {
            get { return _depthEnable; }
            set { _depthEnable = value; }
        }

		bool _depthWriteEnable;
		/// <summary>
        /// Enable depth write
        /// </summary>
        public bool DepthWriteEnable
        {
            get { return _depthWriteEnable; }
            set { _depthWriteEnable = value; }
        }

		ComparisonFunc _depthFunc;
		/// <summary>
        /// A function that compares depth data against existing depth data.
        /// </summary>
        public ComparisonFunc DepthFunc
        {
            get { return _depthFunc; }
            set { _depthFunc = value; }
        }

		bool _stencilEnable;
		/// <summary>
        /// Enable stencil testing. 
        /// </summary>
        public bool StencilEnable
        {
            get { return _stencilEnable; }
            set { _stencilEnable = value; }
        }

		uint _stencilReadMask;
		/// <summary>
        /// Identify a portion of the depth-stencil buffer for reading stencil data.
        /// </summary>
        public uint StencilReadMask
        {
            get { return _stencilReadMask; }
            set { _stencilReadMask = value; }
        }

		uint _stencilWriteMask;
		/// <summary>
        /// Identify a portion of the depth-stencil buffer for writing stencil data.
        /// </summary>
        public uint StencilWriteMask
        {
            get { return _stencilWriteMask; }
            set { _stencilWriteMask = value; }
        }

		bool _twoSidedDepthStencil;
		/// <summary>
        /// Use the two sided mode stencil
        /// </summary>
        public bool TwoSidedDepthStencil
        {
            get { return _twoSidedDepthStencil; }
            set { _twoSidedDepthStencil = value; }
        }


		DepthStencilOperation _frontFace;
		/// <summary>
        /// Identify how to use the results of the depth test and the stencil test for pixels whose surface normal is facing towards the camera. 
        /// </summary>
        public DepthStencilOperation FrontFace
        {
            get { return _frontFace; }
            set { _frontFace = value; }
        }

		DepthStencilOperation _backFace;
		/// <summary>
        /// Identify how to use the results of the depth test and the stencil test for pixels whose surface normal is facing away from the camera.
        /// </summary>
        public DepthStencilOperation BackFace
        {
            get { return _backFace; }
            set { _backFace = value; }
        }

        /// <summary>
        /// Initialize the states with the default values
        /// </summary>
        public DepthStencilState()
        {
            _depthEnable = true;
            _depthWriteEnable = true;
            _depthFunc = ComparisonFunc.LessEqual;
            _stencilEnable = false;
            _stencilReadMask = 0;
            _stencilWriteMask=0xffffffff;

            _frontFace.Initialize();
            _backFace.Initialize();
        }
     }
}
