using System;
using System.Collections.Generic;
using System.Text;
using Jade.Rendering.Enums;
using Jade.Core;

namespace Jade.Rendering.States
{
    /// <summary>
    /// Describes a sampler state
    /// </summary>
    public struct SamplerState
    {
        FilterSampling _filterMin;
		/// <summary>
		/// Filtering method to use when sampling a texture 
		/// </summary>
		public FilterSampling FilterMin
        {
            get { return _filterMin; }
            set { _filterMin = value; }
        }

        FilterSampling _filterMag;
		/// <summary>
		/// Filtering method to use when sampling a texture 
		/// </summary>
		public FilterSampling FilterMag
        {
            get { return _filterMag; }
            set { _filterMag = value; }
        }

        FilterSampling _filterMip;
		/// <summary>
		/// Filtering method to use when sampling a texture 
		/// </summary>
		public FilterSampling FilterMip
        {
            get { return _filterMip; }
            set { _filterMip = value; }
        }

        
        TextureAddress _addressU;
		/// <summary>
		/// Method to use for resolving a u texture coordinate that is outside the 0 to 1 range 
		/// </summary>
        public TextureAddress AddressU
        {
            get { return _addressU; }
            set { _addressU = value; }
        }

        TextureAddress _addressV;
		/// <summary>
		/// Method to use for resolving a v texture coordinate that is outside the 0 to 1 range 
		/// </summary>
		public TextureAddress AddressV
        {
            get { return _addressV; }
            set { _addressV = value; }
        }

        TextureAddress _addressW;
		/// <summary>
		/// Method to use for resolving a w texture coordinate that is outside the 0 to 1 range 
		/// </summary>
        public TextureAddress AddressW
        {
            get { return _addressW; }
            set { _addressW = value; }
        }

        
        float _mipLODBias;
		/// <summary>
        /// Offset from the calculated mipmap level. For example, if Direct3D calculates that a texture should be sampled at mipmap level 3 and MipLODBias is 2, then the texture will be sampled at mipmap level 5. 
        /// </summary>
		public float MipLODBias
        {
            get { return _mipLODBias; }
            set { _mipLODBias = value; }
        }

        uint _maxAnisotropy;
        /// <summary>
        /// Clamping value used if ANISOTROPIC or COMPARISON_ANISOTROPIC is specified in Filter. Valid values are between 1 and 16. 
        /// </summary>
        public uint MaxAnisotropy
        {
            get { return _maxAnisotropy; }
            set { _maxAnisotropy = value; }
        }

        ComparisonFunc _comparisonFunc;
		/// <summary>
		/// A function that compares sampled data against existing sampled data
		/// </summary>
		public ComparisonFunc ComparisonFunc
        {
            get { return _comparisonFunc; }
            set { _comparisonFunc = value; }
        }

        Color[] _borderColor;
		/// <summary>
		/// Border color to use if BORDER is specified for AddressU, AddressV, or AddressW. Range must be between 0.0 and 1.0 inclusive
		/// </summary>
		public Color[] BorderColor
        {
            get { return _borderColor; }
            set { _borderColor = value; }
        }

        float _minLOD;
		/// <summary>
		/// Lower end of the mipmap range to clamp access to, where 0 is the largest and most detailed mipmap level and any level higher than that is less detailed
		/// </summary>
		public float MinLOD
        {
            get { return _minLOD; }
            set { _minLOD = value; }
        }

        float _maxLOD;
		/// <summary>
		/// Upper end of the mipmap range to clamp access to, where 0 is the largest and most detailed mipmap level and any level higher than that is less detailed. This value must be greater than or equal to MinLOD. To have no upper limit on LOD set this to a large value such as D3D10_FLOAT32_MAX.
		/// </summary>
		public float MaxLOD
        {
            get { return _maxLOD; }
            set { _maxLOD = value; }
        }

		/// <summary>
		/// Set default values
		/// </summary>
		public void Initialize()
        {
            _filterMin = FilterSampling.Linear;
            _filterMag = FilterSampling.Linear;
            _filterMip = FilterSampling.Linear;

            _addressU = TextureAddress.Wrap;
            _addressV = TextureAddress.Wrap;
            _addressW = TextureAddress.Wrap;

            _borderColor = new Color[4];

            for (int i = 0; i < 4; i++)
                _borderColor[i] = Colors.White;

            _minLOD = float.MinValue;
            _maxLOD = float.MaxValue;

            _mipLODBias = 0;

            _maxAnisotropy = 1;

            _comparisonFunc = ComparisonFunc.Never;
        }
    }
}
