#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Jade.Rendering.Enums;

#endregion

namespace Jade.Rendering
{
	/// <summary>
	/// Class for loading and managing textures.
	/// </summary>
	/// <remarks>
	/// The texture manager serves as an abstract class for all 
	/// API specific texture managers.  When a class inherits from this 
	/// and is created, a instance of that class (i.e. GLTextureManager)
	///	is stored in the global singleton instance of the TextureManager.  
	///	Note: This will not take place until the IRenderer is initialized.
	/// </remarks>
	public abstract class TextureManager
	{
		#region Fields and Properties
		
		Dictionary<string, Texture> _textures = new Dictionary<string,Texture>();

		/// <summary>
		/// Gets the textures managed by this <see cref="TextureManager"/>.
		/// </summary>
		/// <value>The textures.</value>
		public IDictionary<string, Texture> Textures
		{
			get { return _textures; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// Creates a texture of the specified type.
		/// </summary>
		/// <param name="name">The name.</param>
		/// <param name="type">The type.</param>
		/// <param name="width">The width.</param>
		/// <param name="height">The height.</param>
		/// <param name="depth">The depth.</param>
		/// <param name="numMipMaps">The number of mip maps.</param>
		/// <returns>
		/// A <see cref="Texture"/> specific to this renderer.
		/// </returns>
		/// <remarks>This may, internally, be based off a D3D9 BaseTexture,
		/// or an OpenGL texture, etc</remarks>
		public abstract Texture Create(string name, TextureType type, int width, int height, int depth, int numMipMaps);

		/// <summary>
		/// Gets the texture named "name"
		/// </summary>
		/// <param name="name">The name.</param>
		/// <returns>A <see cref="Texture"/></returns>
		public virtual Texture GetTexture(string name)
		{
			Texture texture;
			if (_textures.TryGetValue(name, out texture))
				return texture;
			return null;
		}
		#endregion
	}
}
