#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using Pluto.Core;

#endregion

namespace Jade.Rendering
{
    /// <summary>
    /// A Viewport is the place into the RenderTarget that the frame is rendered.
    /// </summary>
	[CompletionStatus(DevelopmentState.Complete)]
	public struct Viewport
    {
		#region Fields and Properties

		/// <summary>
        /// The x start position
        /// </summary>
        public int X;

        /// <summary>
        /// The y start position
        /// </summary>
		public int Y;

        /// <summary>
        /// The width
        /// </summary>
		public int Width;

        /// <summary>
        /// The height
        /// </summary>
		public int Height;

        /// <summary>
        /// The zmin, in Clip space coords, used to render. Usually 0
        /// </summary>
		public float ZMin;

        /// <summary>
        /// The zmax, in Clip space coords, used to render. Usually 1
        /// </summary>
		public float ZMax;

		#endregion

		#region Methods

		/// <summary>
        /// Initialize the class with the default values
		/// The width and height value are set to -1
        /// </summary>
        public void Initialize()
        {
            X = 0;
            Y = 0;
            Width = -1;
            Height = -1;
            ZMin = 0;
            ZMax = 1f;
		}

		#endregion

	}
}
