using System;
using System.Collections.Generic;
using System.Text;
using Jade.Rendering.Enums;

namespace Jade.Resources
{
	/// <summary>
	/// This class maintains an IndexBuffer specifically created for holding triangle faces. 
	/// You can use the arrays accessors to set the different index values 
	/// </summary>
	public class TriangleIndexBuffer : IndexBuffer
	{
		#region Properties
		/// <summary>
		/// Get or set the number of Triangles
		/// </summary>
		public int TriangleCount
		{
			get { return _indicesList.Length / 3; }
			set
			{
				int indices = 3 * value;
				//Invalidate();
				/*if (Indices.Capacity < indices)
					Indices.Capacity = indices;
				while (Indices.Count < indices)
					Indices.Add(0);
				 */
				this.Count = indices;
			}
		}

		/// <summary>
		/// Used to access the indices array directly using a triangle + index
		/// </summary>
		public int this[int triangle, int index]
		{
			get { return _indicesList[3 * triangle + index]; }
			set
			{
				int loc = 3 * triangle + index;
				//Invalidate();
				// Make sure we allocate space
				//if (loc >= Indices.Count)
				//	Count = loc + 1;
				_indicesList[loc] = value;
			}
		}

		#endregion


		#region Constructors

		/// <summary>
		/// Creates a TriangleIndexBuffer indicating the format
		/// </summary>
		/// <param name="is32Bits">True if you want to use a 32 bits indices</param>
		public TriangleIndexBuffer(IndexBuffer.IndexSize precision)
			: base(precision, PrimitiveType.TriangleList)
		{
		}

		/// <summary>
		/// Creates a TriangleIndexBuffer
		/// </summary>
		public TriangleIndexBuffer()
			: base(PrimitiveType.TriangleList)
		{
		}
		
		#endregion

		#region Methods
		/// <summary>
		/// Adds a new triangle to the end of our index buffer
		/// </summary>
		/// <param name="index0">The first index.</param>
		/// <param name="index1">The second index.</param>
		/// <param name="index2">The third index.</param>
		public void AddTriangle(int index0, int index1, int index2)
		{
			int len = _indicesList.Length;
			_indicesList[len++] = index0;
			_indicesList[len++] = index1;
			_indicesList[len++] = index2;
			Invalidate();
		}
		
		#endregion
	}
}
