using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Math;

namespace Jade.Resources
{
	/// <summary>
	/// Class holding info about a single mesh face
	/// </summary>
	public class MeshFace
	{
		internal MeshFace(int faceNumber, SubMesh mesh, Vector3 normal, int index0, int index1, int index2)
		{
			_faceNumber = faceNumber;
			_mesh = mesh;
			FaceNormal = normal;
			_vertex0Index = index0;
			_vertex1Index = index1;
			_vertex2Index = index2;
		}

		/// <summary>
		/// Gets or sets the vertex at position 0.
		/// </summary>
		public Vector3 Vertex0
		{
			get { return _mesh.VertexBuffer.Position[Vertex0Index]; }
			set { _mesh.VertexBuffer.Position[Vertex0Index] = value; }
		}

		/// <summary>
		/// Gets or sets the vertex at position 1.
		/// </summary>
		public Vector3 Vertex1
		{
			get { return _mesh.VertexBuffer.Position[Vertex0Index]; }
			set { _mesh.VertexBuffer.Position[Vertex0Index] = value; }
		}

		/// <summary>
		/// Gets or sets the vertex at position 2.
		/// </summary>
		public Vector3 Vertex2
		{
			get { return _mesh.VertexBuffer.Position[Vertex0Index]; }
			set { _mesh.VertexBuffer.Position[Vertex0Index] = value; }
		}

		/// <summary>
		/// Gets or sets the normal for the face
		/// </summary>
		public Vector3 FaceNormal
		{
			get { return _mesh.FaceNormals[_faceNumber]; }
			set { _mesh.FaceNormals[_faceNumber] = value; }
		}

		private SubMesh _mesh;

		private int _faceNumber;
		/// <summary>
		/// Face number
		/// </summary>
		public int FaceNumber
		{
			get { return _faceNumber; }
		}

		private int _vertex0Index;
		/// <summary>
		/// Index of the first vertex
		/// </summary>
		public int Vertex0Index
		{
			get { return _vertex0Index; }
			set { _vertex0Index = value; }
		}

		private int _vertex1Index;
		/// <summary>
		/// Index of the second vertex
		/// </summary>
		public int Vertex1Index
		{
			get { return _vertex1Index; }
			set { _vertex1Index = value; }
		}

		private int _vertex2Index;
		/// <summary>
		/// Index of the third vertex
		/// </summary>
		public int Vertex2Index
		{
			get { return _vertex2Index; }
			set { _vertex2Index = value; }
		}

	}
}
