#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using Jade.Core;
using Jade.Rendering.Enums;

#endregion

namespace Jade.Resources
{
	/// <summary>
	///		Class representing a Text font resource.
	/// </summary>
	public class OverlayTextFont : Resource
	{
		#region Static Members
		private static OverlayTextFont _defaultFont;

		/// <summary>
		/// Gets the default font used by the engine.
		/// </summary>
		public static OverlayTextFont Default
		{
			get 
			{
				if (_defaultFont == null)
					_defaultFont = new OverlayTextFont(18,"Arial");
				return _defaultFont; 
			}
		}
		
		#endregion

		#region Fields and Properties

		private int _height;

		/// <summary>
		/// Gets or sets the height.
		/// </summary>
		/// <value>The height.</value>
		public int Height
		{
			get { return _height; }
			protected set 
			{
				if (_height != value)
				{
					_height = value;
					Invalidate();
				}
			}
		}

		private string _fontFace;

		/// <summary>
		/// Gets or sets the name of the font face.
		/// </summary>
		/// <value>The name of the font.</value>
		public string FontFace
		{
			get { return _fontFace; }
			protected set 
			{
				if (_fontFace != value)
				{
					_fontFace = value;
					Invalidate();
				}
			}
		}
		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="OverlayTextFont"/> class.
		/// May only be created by concrete implementations of the font class.
		/// </summary>
		/// <param name="height">The height.</param>
		/// <param name="fontFace">The font face.</param>
		/// <returns></returns>
		public OverlayTextFont(int height, string fontFace)
			:base()
		{
			_height = height;
			_fontFace = fontFace;
		}
		#endregion

	}
}
