#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;

#endregion

namespace Jade.Resources
{
	/// <summary>
	/// Base interface supported by all resource managers
	/// </summary>
	/// <typeparam name="T"></typeparam>
	public interface IResourceLibrary<T>
	{
		/// <summary>
		/// Adds the resource with the specified name if it doesn't already exist.
		/// If this resource exists, do nothing.
		/// </summary>
		/// <param name="name">The name.</param>
		/// <param name="resource">The resource.</param>
		void AddResource(string name, T resource);

		/// <summary>
		/// Determines whether the resource with the specified name exists in this manager.
		/// </summary>
		/// <param name="name">The name.</param>
		bool ContainsResource(string name);

		/// <summary>
		/// Gets the resource with the specified name.
		/// </summary>
		/// <value></value>
		T this[string name] { get; }

		/// <summary>
		/// Gets the names of the available resources in no particular order.
		/// </summary>
		/// <value>The available resources.</value>
		string[] AvailableResources { get;}

	}
}
