using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Configuration;
using Pluto.VFS;

namespace Jade.Resources
{
	/// <summary>
	/// A ResourceLibrary to manage lighting effects
	/// </summary>
	public class LightingEffectLibrary : ResourceLibrary<LightingEffect>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="LightingEffectLibrary"/> class.
		/// </summary>
		/// <param name="configurationService">The configuration service from which to load actions.</param>
		/// <param name="vfs">The VFS.</param>
		public LightingEffectLibrary(IConfigurationService configurationService, IVFSService vfs)
		{
			// Add our default effect in
			AddResource("Default", LightingEffect.Default);
			LoadFromConfigurationService(configurationService, vfs);
		}

		/// <summary>
		/// Loads all VertexActions from the configuration service.
		/// </summary>
		/// <param name="configurationService">The configuration service.</param>
		/// <param name="vfs">The VFS.</param>
		private void LoadFromConfigurationService(IConfigurationService configurationService, IVFSService vfs)
		{
			string[] paths = configurationService.GetPaths("Resources/LightingEffects/Effect", "Name");
			foreach (string path in paths)
			{
				string name = configurationService.ReadValue<string>(path, "Name", "");
				if (string.IsNullOrEmpty(name))
					continue;
				LightingEffect effect = LightingEffect.LoadFromConfiguration(configurationService, vfs, name);
				if (effect != null)
					AddResource(name, effect);
			}
		}
	}
}
