using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Configuration;

namespace Jade.Resources
{
	/// <summary>
	/// A ResourceLibrary to manage vertex actions
	/// </summary>
	public class VertexActionLibrary: ResourceLibrary<VertexAction>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="VertexActionLibrary"/> class.
		/// </summary>
		/// <param name="configurationService">The configuration service from which to load actions.</param>
		public VertexActionLibrary(IConfigurationService configurationService)
		{
			LoadFromConfigurationService(configurationService);
		}

		/// <summary>
		/// Loads all VertexActions from the configuration service.
		/// </summary>
		/// <param name="configurationService">The configuration service.</param>
		private void LoadFromConfigurationService(IConfigurationService configurationService)
		{
			string[] paths = configurationService.GetPaths("Resources/VertexActions/Action", "Name");
			foreach (string path in paths)
			{
				string name = configurationService.ReadValue<string>(path, "Name", "");
				if (string.IsNullOrEmpty(name))
					continue;
				VertexAction action = VertexAction.LoadFromConfiguration(configurationService, name);
				if (action != null)
					AddResource(name, action);
			}
		}
	}
}
