using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Configuration;

namespace Jade.Resources
{
	/// <summary>
	/// Definition of a variable used by a shader
	/// </summary>
	public class ShaderVariable
	{
		/// <summary>
		/// The type of a shader variable
		/// </summary>
		public enum ShaderVariableType
		{
			/// <summary>
			/// Single float value
			/// </summary>
			Float,
			/// <summary>
			/// Vector of length 2
			/// </summary>
			Vec2,
			/// <summary>
			/// Vector of length 3
			/// </summary>
			Vec3,
			/// <summary>
			/// Vector of length 4
			/// </summary>
			Vec4,
			/// <summary>
			/// 4x4 Matrix
			/// </summary>
			Matrix
		}

		/// <summary>
		/// The varible name
		/// </summary>
		public string Name;

		/// <summary>
		/// The variable type
		/// </summary>
		public ShaderVariableType Type;

		/// <summary>
		/// The default value used for this shader
		/// </summary>
		public object DefaultValue;

		/// <summary>
		/// Initializes a new instance of the <see cref="ShaderVariable"/> class.
		/// </summary>
		/// <param name="name">The name.</param>
		/// <param name="type">The type.</param>
		/// <param name="defaultValue">The default value.</param>
		public ShaderVariable(string name, ShaderVariableType type, object defaultValue)
		{
			Name = name;
			Type = type;
			DefaultValue = defaultValue;
		}
	}
}
