using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Configuration;

namespace Jade.Resources
{
	/// <summary>
	/// This is the code to include at shader level to modify the vertices
	/// </summary>
	public class VertexAction
	{
		private string _HLSLShader;
		/// <summary>
		/// Gets or sets the HLSL shader.
		/// </summary>
		/// <value>The HLSL shader.</value>
		public string HLSLShader
		{
			get { return _HLSLShader; }
			set { _HLSLShader = value; }
		}

		private List<ShaderVariable> _variables = new List<ShaderVariable>();
		/// <summary>
		/// Gets the variables list used by this shader.
		/// </summary>
		/// <value>The variables.</value>
		public IList<ShaderVariable> Variables
		{
			get { return _variables; }
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="VertexAction"/> class.
		/// </summary>
		/// <param name="HLSL">The HLSL.</param>
		public VertexAction(string HLSL)
		{
			_HLSLShader = HLSL;
		}

		/// <summary>
		/// Loads the vertex action from the configuration service.
		/// </summary>
		/// <param name="configurationService">The configuration service.</param>
		/// <param name="name">The name.</param>
		/// <returns>A new VertexAction</returns>
		static public VertexAction LoadFromConfiguration(IConfigurationService configurationService, string name)
		{
			if (configurationService == null)
				return null;

			string _name = configurationService.ReadValue<string>("Resources/VertexActions/Action[@Name='" + name + "']", "Name", "");
			if (name != _name)
				return null;

			string _hlsl = configurationService.ReadValue<string>("Resources/VertexActions/Action[@Name='" + name + "']", "HLSL", "");
			if (string.IsNullOrEmpty(_hlsl))
				return null;

			VertexAction result = new VertexAction(_hlsl);
			string[] paths = configurationService.GetPaths("Resources/VertexActions/Action[@Name='" + name + "']/Variable", "Name");
			foreach (string path in paths)
			{
				string varName = configurationService.ReadValue<string>(path, "Name", "");
				string varType = configurationService.ReadValue<string>(path, "Type", "");
				if (string.IsNullOrEmpty(varName) || string.IsNullOrEmpty(varType))
					continue;
				switch (varType)
				{
					case "Float":
						result.Variables.Add(new ShaderVariable(varName, ShaderVariable.ShaderVariableType.Float, configurationService.ReadValue<float>(path, "Default", 0.0f)));
						break;
					default:
						throw new NotImplementedException();
				}
			}
			return result;
		}

	}
}
