#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;

using Jade.Entities;

#endregion

namespace Jade.SceneGraph
{
    /// <summary>
    /// Node visitor used internally to create a node's paths through it's parents in
    /// the scene graph.
    /// </summary>
	[CompletionStatus(DevelopmentState.Complete, UnitTestedPercent=100)]
	internal class CollectParentPathWalker : NodeWalker
    {
        #region Fields and Properties
        Node _haltTraversalAtNode;

		IList<Node> _nodePath = new List<Node>();
        /// <summary>
        /// Gets the node paths to root or the halt node.
        /// </summary>
        /// <value>The node paths.</value>
		public IList<Node> NodePath
        {
            get
            {
                return _nodePath;
            }
        }
        #endregion

        #region Constructor
        public CollectParentPathWalker(Node haltTraversalAtNode)
            : base(TraversalType.Internal, TraversalMode.Parents)
        {
            _haltTraversalAtNode = haltTraversalAtNode;
        }
        #endregion

        #region Methods
		public new bool Execute(IEntity entity, NodeWalkerArgs args)
        {
			Node node = args.Node;
			_nodePath.Add(node);
			// If we're at the node where we're supposed to halt, stop or traversal
			if (node == _haltTraversalAtNode)
				this.Traverse = TraversalMode.None;
			return true;
        }
        #endregion
    }
}
