#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;
using Pluto.Math;

using Jade.Rendering;
using Jade.Resources;
using Jade.Entities;
using Jade.Core;

#endregion

namespace Jade.SceneGraph
{
	/// <summary>
	/// Construct a render list for culling purposes of the overlay scene
	/// </summary>
	internal class CullOverlayNodeWalker : NodeWalker
	{
		#region Fields and Properties
		RenderList _renderList;
		#endregion

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="RenderList"/> class.
		/// </summary>
		public CullOverlayNodeWalker(RenderList renderList)
			: base(TraversalType.Cull, TraversalMode.ActiveChildren) 
        {
			_renderList = renderList;
        }

        #endregion

        #region Methods
		/// <summary>
		/// Executes the specified entity.
		/// </summary>
		/// <param name="entity">The entity.</param>
		/// <param name="args">The args.</param>
		/// <returns></returns>
		public bool Execute(OverlayTextEntity entity, NodeWalkerArgs args)
		{
			_renderList.TextToRender.Add(new TransformedEntity(entity, args.Transform));
			return true;
		}

		/// <summary>
		/// Executes the specified entity.
		/// </summary>
		/// <param name="entity">The entity.</param>
		/// <param name="args">The args.</param>
		/// <returns></returns>
		public bool Execute(Sprite entity, NodeWalkerArgs args)
		{
			_renderList.Sprites.Add(new SpriteRenderList(entity,args.Transform));
			return true;
		}

		/// <summary>
		/// Executes the specified entity.
		/// </summary>
		/// <param name="entity">The entity.</param>
		/// <param name="args">The args.</param>
		/// <returns></returns>
		public bool Execute(Light entity, NodeWalkerArgs args)
		{
			_renderList.LightsOverlay.Add(new TransformedLight(entity, args.Transform));
			return true;
		}


		/// <summary>
		/// Executes the specified entity.
		/// </summary>
		/// <param name="entity">The entity.</param>
		/// <param name="args">The args.</param>
		/// <returns></returns>
		public bool Execute(AmbientLight entity, NodeWalkerArgs args)
		{
			_renderList.AmbientLightColorOverlay += entity.Color;
			return true;
		}

        #endregion
	}
}
