#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;

using Jade.Entities;

#endregion

namespace Jade.SceneGraph
{
	/// <summary>
	/// Main NodeWalker used by the Scene internally to trigger an Update
	/// </summary>
	[CompletionStatus(DevelopmentState.Complete, UnitTestedPercent = 50)]
	public class UpdateWalker : NodeWalker
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="UpdateWalker"/> class.
		/// </summary>
		public UpdateWalker()
			: base(TraversalType.Update, TraversalMode.Children)
		{
			this.UpdateAfterChildren = false;
		}

		/// <summary>
		/// Executes the node walker's operation on the specified node.
		/// </summary>
		public new bool Execute(IEntity entity, NodeWalkerArgs args)
		{
			if (entity != null)
				entity.OnUpdate(args);

			return true;
		}

	}
}
