#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;

using Jade.Entities;

#endregion

namespace Jade.SceneGraph
{
	/// <summary>
	/// Argument to Node Update events
	/// </summary>
	[CompletionStatus(DevelopmentState.NearCompletion,UnitTestedPercent=30)]
	public class NodeUpdateEventArgs : EventArgs
	{
		private Node _node;

		/// <summary>
		/// Gets the node.
		/// </summary>
		/// <value>The node.</value>
		public Node Node
		{
			get { return _node; }
		}
		private IEntity _entity;

		/// <summary>
		/// Gets the entity.
		/// </summary>
		/// <value>The entity.</value>
		public IEntity Entity
		{
			get { return _entity; }
		}
		private NodeWalker _walker;

		/// <summary>
		/// Gets the walker.
		/// </summary>
		/// <value>The walker.</value>
		public NodeWalker Walker
		{
			get { return _walker; }
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="NodeUpdateEventArgs"/> class.
		/// </summary>
		/// <param name="node">The node.</param>
		/// <param name="entity">The entity.</param>
		/// <param name="walker">The walker.</param>
		public NodeUpdateEventArgs(Node node, IEntity entity, NodeWalker walker)
		{
			_node = node;
			_entity = entity;
			_walker = walker;
		}
	}

	/// <summary>
	/// Delegate used for all Node Update events
	/// </summary>
	[CompletionStatus(DevelopmentState.NearCompletion, UnitTestedPercent = 100)]
	public delegate void NodeUpdateDelegate(object sender, NodeUpdateEventArgs args);
}
