#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;
using Pluto.Math;


#endregion

namespace Jade.SceneGraph
{
	/// <summary>
	/// Arguments used during the node traversal - Allows update function to
	/// determine current location in the scene (Node)
	/// </summary>
	[CompletionStatus(DevelopmentState.NearCompletion,UnitTestedPercent=50)]
	public class NodeWalkerArgs
	{
		/// <summary>
		/// The current state/direction we are walking
		/// </summary>
		[CompletionStatus(DevelopmentState.Complete,UnitTestedPercent=50,Notes="Tests needed for AfterChildUpdate")]
		public enum WalkerState
		{
			/// <summary>
			/// This update is happening before the children are 
			/// updated, in a normal traversal pattern.  This state is always 
			/// active during a standard update.
			/// </summary>
			BeforeChildUpdate,
			/// <summary>
			/// This update is happening (by request) after a node has updated
			/// it's children.  This happens on standard, top-down traversals
			/// after all of a node's children are updated.
			/// </summary>
			AfterChildUpdate
		}

		private Node _node;

		/// <summary>
		/// Gets or sets the node.
		/// </summary>
		/// <value>The node.</value>
		public Node Node
		{
			get { return _node; }
		}

		private NodeWalker _walker;

		/// <summary>
		/// Gets the node walker
		/// </summary>
		/// <value>The walker.</value>
		public NodeWalker Walker
		{
			get { return _walker; }
		}

		private WalkerState _state;

		/// <summary>
		/// Gets or sets the <see cref="WalkerState"/>.
		/// </summary>
		/// <value>The state.</value>
		private WalkerState State
		{
			get { return _state; }
			set { _state = value; }
		}

		private Matrixd _transform;
		/// <summary>
		/// Gets the accumulated transform to this node.
		/// </summary>
		/// <value>The transform.</value>
		public Matrixd Transform
		{
			get { return _transform; }
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="NodeWalkerArgs"/> class.
		/// </summary>
		/// <param name="node">The node.</param>
		/// <param name="state">The state.</param>
		/// <param name="walker">The walker.</param>
		/// <param name="transform">The transform.</param>
		public NodeWalkerArgs(Node node, WalkerState state, NodeWalker walker, Matrixd transform)
		{
			_node = node;
			_state = state;
			_walker = walker;
			_transform = transform;
		}
	}
}
