#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Math;

using Jade.SceneGraph;

#endregion


namespace Jade.SceneGraph.Transforms
{
	/// <summary>
	/// Define a transform based on an orientation and a position
	/// </summary>
	public class OrientationPosition : ITransform
	{
		private Quaterniond _orientation = new Quaterniond(0,0,0,1);
		/// <summary>
		/// Gets or sets the orientation.
		/// </summary>
		/// <value>The orientation.</value>
		public Quaterniond Orientation
		{
			get { return _orientation; }
			set { _orientation = value; }
		}

		private Vector3d _position = new Vector3d(0,0,0);
		/// <summary>
		/// Gets or sets the position.
		/// </summary>
		/// <value>The position.</value>
		public Vector3d Position
		{
			get { return _position; }
			set { _position = value; }
		}

		private Vector3d _scale = new Vector3d(1,1,1);
		/// <summary>
		/// Gets or sets the scale.
		/// </summary>
		/// <value>The scale.</value>
		public Vector3d Scale
		{
			get { return _scale; }
			//TODO set { _scale = value; }
		}

		#region ITransform Members


		/// <summary>
		/// Gets the transform.
		/// </summary>
		/// <value>The transform.</value>
		public Matrixd Transform
		{
			get 
			{
				Matrixd matrix = Matrixd.CreateFromQuaternion(ref _orientation);
				matrix.Translation = _position;
				// TODO: Handle scale
				return matrix;
			}
		}

		#endregion
	}
}
