#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Math;
using SM = System.Math;
using Jade.SceneGraph;

#endregion


namespace Jade.SceneGraph.Transforms
{
	/// <summary>
	/// Define a transform based on an orientation and a position
	/// </summary>
	public class RotationPosition2D : ITransform
	{
		private Vector2 _pivotPoint;
		/// <summary>
		/// Gets or sets the pivot point for rotations.
		/// </summary>
		/// <value>The pivot point.</value>
		public Vector2 PivotPoint
		{
			get { return _pivotPoint; }
			set { _pivotPoint = value; }
		}

		private double _rotation = 0;
		/// <summary>
		/// Gets or sets the rotation in radians.
		/// </summary>
		/// <value>The rotation.</value>
		public double Rotation
		{
			get { return _rotation; }
			set { _rotation = value; }
		}
		/// <summary>
		/// Gets or sets the rotation in degrees.
		/// </summary>
		/// <value>The rotation degrees.</value>
		public double RotationDegrees
		{
			get { return _rotation * 180.0 / SM.PI; }
			set { _rotation = value * SM.PI / 180.0; }
		}


		private Vector3d _position = new Vector3d(0, 0, 0);
		/// <summary>
		/// Gets or sets the position.
		/// </summary>
		/// <value>The position.</value>
		public Vector3d Position
		{
			get { return _position; }
			set 
			{
				_position = value;
				_position.Z = 0;
			}
		}

		/// <summary>
		/// Gets or sets the position in 2D.
		/// </summary>
		public Vector2 Position2D
		{
			get { return new Vector2((float) _position.X, (float) _position.Y); }
			set
			{
				_position.X = value.X;
				_position.Y = value.Y;
			}
		}

		#region ITransform Members


		/// <summary>
		/// Gets the transform.
		/// </summary>
		/// <value>The transform.</value>
		public Matrixd Transform
		{
			get
			{
				Vector3d axis = Vector3d.Forward;
				Quaterniond _orientation = Quaterniond.CreateFromAxisAngle(ref axis, _rotation);
				
				Matrixd pivot = Matrixd.CreateTranslation(-_pivotPoint.X, -_pivotPoint.Y, 0);
				Matrixd rotation = Matrixd.CreateFromQuaternion(ref _orientation);

				Matrixd result = Matrixd.Multiply(ref rotation, ref pivot);

				result.Translation += new Vector3d(_position.X + _pivotPoint.X,_position.Y + _pivotPoint.Y,0);

				// TODO: Handle scale
				return result;
			}
		}

		#endregion

		#region ITransform Members


		#endregion
	}
}
