#region LGPL License
/*
Jad Engine Library
Copyright (C) 2007 Jad Engine Project Team

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

using Pluto.Core;
using Pluto.Configuration;
using Pluto.VFS;

using Jade.Resources;
using Jade.Core;

#endregion

namespace Jade.Services
{
	/// <summary>
	/// Class to manage shared textures.  Uses the <see cref="IConfigurationService"/>
	/// to discover texture sources automatically.
	/// </summary>
	internal class DefaultTextureManager : ResourceManager<Texture>
	{
		#region Pluto Creation Callback
		
		/// <summary>
		/// Creates the service via a callback.
		/// </summary>
		public static IResourceManager<Texture> CreateServiceCallback(IPlutoRootServiceProvider provider)
		{
			return new DefaultTextureManager(provider.GetService<IVFSService>(), provider.GetService<IConfigurationService>());
		}

		#endregion

		/// <summary>
		/// Initializes a new instance of the <see cref="DefaultTextureManager"/> class.
		/// </summary>
		public DefaultTextureManager(IVFSService vfs, IConfigurationService configurationService)
		{
			LoadTexturesFromConfig(vfs, configurationService);
		}

		/// <summary>
		/// Loads the textures from the config service
		/// </summary>
		private void LoadTexturesFromConfig(IVFSService vfs, IConfigurationService config)
		{      
			string[] texturePaths = config.GetPaths("Resources/Textures/Texture", "Name");
			foreach (string texturePath in texturePaths)
			{
				string name = config.ReadValue<string>(texturePath, "Name", "");
				string file = config.ReadValue<string>(texturePath, "File", "");
				string vfsfile = config.ReadValue<string>(texturePath, "VFSFile", "");
				string vfspath = config.ReadValue<string>(texturePath, "VFSPath", "");

				if (string.IsNullOrEmpty(name))
					continue;

        Log.WriteLine("Loading Texture <Name: " + name + ", File: " + file + ", VFSFile: " + vfsfile + ", VFSPath: " + vfspath + ">", "TextureManager", 1);
				if (string.IsNullOrEmpty(file))
				{
					if (string.IsNullOrEmpty(vfsfile))
						continue;
					if (vfs == null)
						throw new ArgumentNullException("VFS Service is missing");
					Texture texture = new Texture(vfs,vfsfile,vfspath);
					AddResource(name, texture);
				}
				else
				{
					Texture texture = new Texture(file);
					AddResource(name, texture);
				}
			}      
		}
	}
}

