using System;
using System.Collections.Generic;
using System.Text;
using MbUnit.Framework;
using Jade.SceneGraph;

namespace Jade.UnitTests
{
	/// <summary>
	/// Suite of tests to check the plugin loading capabilities in Pluto
	/// </summary>
	[TestFixture("Node Hierarchy Tests")]
	public class NodeHierarchyTests
	{
		Node _rootNode;
		Node _leafDownTwo;

		[TestFixtureSetUp]
		public void Init()
		{
			_rootNode = new Node();
			_rootNode.Descriptions.Add("Root Node");
			
			Node child = new Node();
			child.Descriptions.Add("First Child Node");
			_rootNode.AttachChild(child);

			_leafDownTwo = new Node();
			_leafDownTwo.Descriptions.Add("Leaf Down Two Levels");
			child.AttachChild(_leafDownTwo);
		}

		[Test]
		public void GetPathListFromLeafToRoot()
		{

			IList<Node> path = _leafDownTwo.GetParentalNodePaths();
			Assert.IsTrue(path.Count == 3, "Incorrect number of nodes in path");
			Assert.AreEqual(path[0], _leafDownTwo,"Leaf is not first node in path");
			Assert.AreEqual(path[2], _rootNode, "Root is not last (third) node in path");
			
			StringAssert.FullMatch(path[0].Descriptions[0], "Leaf Down Two Levels");
			StringAssert.FullMatch(path[1].Descriptions[0], "First Child Node");
			StringAssert.FullMatch(path[2].Descriptions[0], "Root Node");
		}
	}
}
