using System;
using System.Collections.Generic;
using System.Text;
using MbUnit.Framework;
using Jade.SceneGraph;
using Pluto.Math;

namespace Jade.UnitTests
{
	[TestFixture("Node Update Event Tests")]
	public class NodeUpdateEventTests
	{
		Node _rootNode;
		Node _leafDownTwo;
		bool _leafUpdated = false;

		private class UpdateWalker : NodeWalker
		{
			public UpdateWalker()
				: base(TraversalType.Update, TraversalMode.Children)
			{
				this.UpdateAfterChildren = false;
			}
		}
		[TestFixtureSetUp]
		public void Init()
		{
			_rootNode = new Node();
			_rootNode.Descriptions.Add("Root Node");

			Node child = new Node();
			child.Descriptions.Add("First Child Node");
			_rootNode.AttachChild(child);

			_leafDownTwo = new Node();
			_leafDownTwo.Descriptions.Add("Leaf Down Two Levels");
			_leafDownTwo.Update += new NodeUpdateDelegate(_leafDownTwo_Update);
			child.AttachChild(_leafDownTwo);
		}

		void _leafDownTwo_Update(object sender, NodeUpdateEventArgs args)
		{
			_leafUpdated = true;
		}


		[Test]
		public void CheckUpdatedAsFalse()
		{
			Assert.IsTrue(_leafUpdated == false, "Leaf has already been updated!");
		}
		[Test]
		public void UpdateTreeUsingWalker()
		{
			Assert.IsTrue(_leafUpdated == false, "Leaf was not reset!");
			UpdateWalker walker = new UpdateWalker();
			_rootNode.Walk(walker,Matrixd.Identity);
			Assert.IsTrue(_leafUpdated == true, "Leaf was not updated!");
			_leafUpdated = false;
			Assert.IsTrue(_leafUpdated == false, "Leaf was not reset!");
		}
		[Test]
		public void UpdateTreeFromScene()
		{
			Assert.IsTrue(_leafUpdated == false, "Leaf was not reset!");
			_rootNode.Walk(new UpdateWalker(),Matrixd.Identity);
			Assert.IsTrue(_leafUpdated == true, "Leaf was not updated!");
			_leafUpdated = false;
			Assert.IsTrue(_leafUpdated == false, "Leaf was not reset!");
		}

	}
}
